// swad_text.c: text messages on screen, in several languages

/*
    SWAD (Shared Workspace At a Distance in Spanish),
    is a web platform developed at the University of Granada (Spain),
    and used to support university teaching.

    This file is part of SWAD core.
    Copyright (C) 1999-2015 Antonio Caas Vargas

    Catal translation:
	   Antonio Caas Vargas
	   Joan Llus Daz Rodrguez
    German translation:
	   Antonio Caas Vargas
	   Rafael Barranco-Droege
    English translation (finished):
	   Antonio Caas Vargas
    Spanish translation (finished):
	   Antonio Caas Vargas
    French translation:
	   Antonio Caas Vargas
    Guarani translation:
	   Antonio Caas Vargas
    Italian translation (finished):
	   Antonio Caas Vargas
	   Nicola Comunale Rizzo
	   Francisco Manuel Herrero Prez
	   Giuseppe Antonio Pagin
	   Antonella Grande
    Polish translation:
	   Antonio Caas Vargas
	   Wojtek Kieca
	   Tomasz Olechowski
	   Mateusz Stanko
    Portuguese translation:
	   Antonio Caas Vargas

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/*
 Sabies que pots utilitzar SWAD en catal? https://swad.ugr.es/ca
 Wussten Sie, dass Sie verwenden knnen, SWAD auf Deutsch? https://swad.ugr.es/de
 Did you know that you can use SWAD in English? https://swad.ugr.es/en
 Saviez-vous que vous pouvez utiliser SWAD en franais? https://swad.ugr.es/fr
 Lo sapevate che  possibile utilizzare SWAD in italiano? https://swad.ugr.es/it
 Czy wiesz, ze mozesz uzywac SWAD po polsku? https://swad.ugr.es/pl
 Sabia que voc pode usar SWAD em portugus? https://swad.ugr.es/pt
 */

/*****************************************************************************/
/********************************* Headers ***********************************/
/*****************************************************************************/

#include <linux/stddef.h>	// For NULL

#include "swad_assignment.h"
#include "swad_attendance.h"
#include "swad_country.h"
#include "swad_course.h"
#include "swad_degree.h"
#include "swad_file_browser.h"
#include "swad_forum.h"
#include "swad_holiday.h"
#include "swad_info.h"
#include "swad_menu.h"
#include "swad_notification.h"
#include "swad_photo.h"
#include "swad_QR.h"
#include "swad_record.h"
#include "swad_statistic.h"
#include "swad_survey.h"
#include "swad_syllabus.h"
#include "swad_tab.h"
#include "swad_test.h"
#include "swad_text.h"
#include "swad_timetable.h"

/*****************************************************************************/
/******************************** Constants **********************************/
/*****************************************************************************/

const char *Txt_NEW_LINE = "\r\n";	// End of line in a file. If we put only \n the file does not look good in some Windows text editors

// The HTML entity for "" is &szlig; It stands for "S-Z ligature", because this symbol comes from the ligature of a Gothic S and a Z.
// For polish HTML entities, see http://pl.wikipedia.org/wiki/Alfabet_polski
// Guarani:
// E with tilde: &#7868;
// e with tilde: &#7869;
// I with tilde: &#296;
// i with tilde: &#297;

/***** Languages *****/
const char *Txt_STR_LANG_ID[Txt_NUM_LANGUAGES] = // ISO 639-1 language codes
	{
	"ca",
	"de",
	"en",
	"es",
	"fr",
	"gn",
	"it",
	"pl",
	"pt",
	};

#if   L==0
const unsigned Txt_Current_CGI_SWAD_Language = Txt_LANGUAGE_CA;
#elif L==1
const unsigned Txt_Current_CGI_SWAD_Language = Txt_LANGUAGE_DE;
#elif L==2
const unsigned Txt_Current_CGI_SWAD_Language = Txt_LANGUAGE_EN;
#elif L==3
const unsigned Txt_Current_CGI_SWAD_Language = Txt_LANGUAGE_ES;
#elif L==4
const unsigned Txt_Current_CGI_SWAD_Language = Txt_LANGUAGE_FR;
#elif L==5
const unsigned Txt_Current_CGI_SWAD_Language = Txt_LANGUAGE_GN;
#elif L==6
const unsigned Txt_Current_CGI_SWAD_Language = Txt_LANGUAGE_IT;
#elif L==7
const unsigned Txt_Current_CGI_SWAD_Language = Txt_LANGUAGE_PL;
#elif L==8
const unsigned Txt_Current_CGI_SWAD_Language = Txt_LANGUAGE_PT;
#endif

const char *Txt_STR_LANG_NAME[Txt_NUM_LANGUAGES] =
	{
	"Catal&agrave;",
	"Deutsch",
	"English",
	"Espa&ntilde;ol",
	"Fran&ccedil;ais",
	"Ava&ntilde;e'&#7869;",
	"Italiano",
	"Polski",
	"Portugu&ecirc;s",
	};
const char *Txt_Do_you_want_to_change_the_language_to_LANGUAGE[Txt_NUM_LANGUAGES] =
	{
	"Voleu canviar l'idioma a catal&agrave;?",
	"Wollen Sie die Sprache auf Deutsch umstellen?",
	"Do you want to change the language to English?",
	"&iquest;Desea cambiar el idioma a espa&ntilde;ol?",
	"Voulez-vous changer la langue au fran&ccedil;ais?",
	"&iquest;Oipota moambue pe &ntilde;e'&#7869; pe ava&ntilde;e'&#7869;?",
	"Vuoi cambiare la lingua in italiano?",
	"Czy chcesz zmieni&cacute; j&eogon;zyk na polski?",
	"Voc&ecirc; quer mudar o idioma para portugu&ecirc;s?",
	};
const char *Txt_Do_you_want_to_change_your_language_to_LANGUAGE[Txt_NUM_LANGUAGES] =
	{
	"Voleu canviar el seu idioma a catal&agrave;?",
	"Wollen Sie Ihre Sprache auf Deutsch umstellen?",
	"Do you want to change your language to English?",
	"&iquest;Desea cambiar su idioma a espa&ntilde;ol?",
	"Voulez-vous changer votre langue au fran&ccedil;ais?",
	"&iquest;Oipota moambue pe &ntilde;e'&#7869; pe ava&ntilde;e'&#7869;?",
	"Vuoi cambiare la tua lingua in italiano?",
	"Czy chcesz zmieni&cacute; j&eogon;zyk na polski?",
	"Voc&ecirc; quer mudar suo idioma para portugu&ecirc;s?",
	};
const char *Txt_Switch_to_LANGUAGE[Txt_NUM_LANGUAGES] =
	{
	"Canviar a catal&agrave;",
	"Umschalten auf Deutsch",
	"Switch to English",
	"Cambiar a espa&ntilde;ol",
	"Passer au fran&ccedil;ais",
	"Moambue pe &ntilde;e'&#7869; ava&ntilde;e'&#7869;",
	"Passare a italiano",
	"Prze&lstrok;&aogon;cz na polski",
	"Mudar para portugu&ecirc;s",
	};
const char *Txt_Switching_to_LANGUAGE[Txt_NUM_LANGUAGES] =
	{
	"Canviant a catal&agrave;...",
	"Umschaltung auf Deutsch...",
	"Switching to English...",
	"Cambiando a espa&ntilde;ol...",
	"Passant au fran&ccedil;ais...",
	"Moambue pe &ntilde;e'&#7869; ava&ntilde;e'&#7869;...",
	"Passando a italiano...",
	"Prze&lstrok;&aogon;cz na polski...",
	"Mudando para portugu&ecirc;s...",
	};

/*****************************************************************************
		     #   #  ###       #   # ##### #   # #
		     ##  # #   #      #   #   #   ## ## #
		     # # # #   #      #####   #   # # # #
		     #  ## #   #      #   #   #   #   # #
		     #   #  ###       #   #   #   #   # #####
******************************************************************************/
/**** Don't use HTML entities like &oacute; when variable ends in NO_HTML ****/
/****              Charset must be windows-1252 in this file              ****/
/*****************************************************************************/

const char *Txt_Country_NO_HTML =
#if   L==0
	"Pas";
#elif L==1
	"Land";
#elif L==2
	"Country";
#elif L==3
	"Pas";
#elif L==4
	"Pays;";
#elif L==5
	"Teta";
#elif L==6
	"Paese";
#elif L==7
	"Kraj";
#elif L==8
	"Pa&iacute;s";
#endif

const char *Txt_Confirmation_of_your_email_NO_HTML =	// Don't use HTML entities like &egrave; here
#if   L==0
	"Confirmacin de su direccin de correo";	// Necessita traduccio
#elif L==1
	"Confirmation of your e-mail";			// Need bersetzung
#elif L==2
	"Confirmation of your e-mail";
#elif L==3
	"Confirmacin de su direccin de correo";
#elif L==4
	"Confirmation of your e-mail";			// Besoin de traduction
#elif L==5
	"Confirmacin de su direccin de correo";	// Okoteve traduccin
#elif L==6
	"Confirmation of your e-mail";			// Bisogno di traduzione
#elif L==7
	"Confirmation of your e-mail";			// Potrzebujesz tlumaczenie
#elif L==8
	"Confirmation of your e-mail";			// Necessita de traduo
#endif

// The following variables are compilated together in all languages because they are used in emails...
// ...and each email is sent in the language of the recipient

const char *Txt_Course_NO_HTML[Txt_NUM_LANGUAGES] =	// Don't use HTML entities like &egrave; here
	{
	"Assignatura",
	"Kurs",
	"Course",
	"Asignatura",
	"Matire",
	"Mbo'esyry",
	"Corso",
	"Kurs",
	"Disciplina",
	};

const char *Txt_Dear_NO_HTML[Usr_NUM_SEXS][Txt_NUM_LANGUAGES] =	// Don't use HTML entities like &oacute; here
{
	{
	// Usr_SEX_UNKNOWN
	"Estimat/da",
	"Sehr geehrte/r",
	"Dear",
	"Estimado/a",
	"Cher/Chre",
	"Estimado/a",	// Okoteve traduccin
	"Egregio/a",
	"Drogi",
	"Prezado/a",
	},
	{
	// Usr_SEX_FEMALE
	"Estimada",
	"Sehr geehrte",
	"Dear",
	"Estimada",
	"Chre",
	"Estimada",	// Okoteve traduccin
	"Egregia",
	"Drogi",
	"Prezada",
	},
	{
	// Usr_SEX_MALE
	"Estimat",
	"Sehr geehrter",
	"Dear",
	"Estimado",
	"Cher",
	"Estimado",	// Okoteve traduccin
	"Egregio",
	"Drogi",
	"Prezado",
	},
	{
	// Usr_SEX_ALL
	"Estimat/da",
	"Sehr geehrte/r",
	"Dear",
	"Estimado/a",
	"Cher/Chre",
	"Estimado/a",	// Okoteve traduccin
	"Egregio/a",
	"Drogi",
	"Prezado/a",
	}
};

const char *Txt_Forum_NO_HTML[Txt_NUM_LANGUAGES] =	// Don't use HTML entities like &oacute; here
	{
	"Frum",
	"Forum",
	"Forum",
	"Foro",
	"Forum",
	"Foro",		// Okoteve traduccin
	"Forum",
	"Forum",
	"Frum",
	};

const char *Txt_General_NO_HTML[Txt_NUM_LANGUAGES] =	// Don't use HTML entities like &eacute; here
	{
	"General",
	"Allgemein",
	"General",
	"General",
	"Gnral",
	"General",	// Okoteve traduccin
	"Generale",
	"Oglne",
	"Geral",
	};

const char *Txt_Go_to_NO_HTML[Txt_NUM_LANGUAGES] =	// Don't use HTML entities like &agrave; here
	{
	"Anar a",
	"Gehe zum",
	"Go to",
	"Ir a",
	"Aller ",
	"Ir a",		// Okoteve traduccin
	"Vai a",
	"Idz do",
	"Ir para",
	};

const char *Txt_If_you_just_request_from_X_the_confirmation_of_your_email_Y_NO_HTML =	// Don't use HTML entities like &egrave; here
#if   L==0
	"Si usted acaba de solicitar desde %s"
	" la confirmacin de su correo electrnico %s,"
	" pulse en el siguiente enlace"
	" para confirmar dicha direccin:"
	" %s/?ActCod=%ld&MailKey=%s\n\n"
	"Si no ha sido usted, no pulse en el enlace anterior!"
	" En este caso, le recomendamos que compruebe en %s"
	" si ha confirmado su direccin de correo.\n\n";	// Necessita traduccio
#elif L==1
	"If you just request from %s"
	" the confirmation of your e-mail %s,"
	" click on the following link"
	" to confirm that direction:"
	" %s/?ActCod=%ld&MailKey=%s\n\n"
	"If it was not you, do not click on the link above!"
	" In this case, we recommend that you check in %s"
	" if you have confirmed your e-mail address.\n\n";	// Need bersetzung
#elif L==2
	"If you just request from %s"
	" the confirmation of your e-mail %s,"
	" click on the following link"
	" to confirm that direction:"
	" %s/?ActCod=%ld&MailKey=%s\n\n"
	"If it was not you, do not click on the link above!"
	" In this case, we recommend that you check in %s"
	" if you have confirmed your e-mail address.\n\n";
#elif L==3
	"Si usted acaba de solicitar desde %s"
	" la confirmacin de su correo electrnico %s,"
	" pulse en el siguiente enlace"
	" para confirmar dicha direccin:"
	" %s/?ActCod=%ld&MailKey=%s\n\n"
	"Si no ha sido usted, no pulse en el enlace anterior!"
	" En este caso, le recomendamos que compruebe en %s"
	" si ha confirmado su direccin de correo.\n\n";
#elif L==4
	"If you just request from %s"
	" the confirmation of your e-mail %s,"
	" click on the following link"
	" to confirm that direction:"
	" %s/?ActCod=%ld&MailKey=%s\n\n"
	"If it was not you, do not click on the link above!"
	" In this case, we recommend that you check in %s"
	" if you have confirmed your e-mail address.\n\n";	// Besoin de traduction
#elif L==5
	"Si usted acaba de solicitar desde %s"
	" la confirmacin de su correo electrnico %s,"
	" pulse en el siguiente enlace"
	" para confirmar dicha direccin:"
	" %s/?ActCod=%ld&MailKey=%s\n\n"
	"Si no ha sido usted, no pulse en el enlace anterior!"
	" En este caso, le recomendamos que compruebe en %s"
	" si ha confirmado su direccin de correo.\n\n";	// Okoteve traduccin
#elif L==6
	"If you just request from %s"
	" the confirmation of your e-mail %s,"
	" click on the following link"
	" to confirm that direction:"
	" %s/?ActCod=%ld&MailKey=%s\n\n"
	"If it was not you, do not click on the link above!"
	" In this case, we recommend that you check in %s"
	" if you have confirmed your e-mail address.\n\n";	// Bisogno di traduzione
#elif L==7
	"If you just request from %s"
	" the confirmation of your e-mail %s,"
	" click on the following link"
	" to confirm that direction:"
	" %s/?ActCod=%ld&MailKey=%s\n\n"
	"If it was not you, do not click on the link above!"
	" In this case, we recommend that you check in %s"
	" if you have confirmed your e-mail address.\n\n";	// Potrzebujesz tlumaczenie
#elif L==8
	"If you just request from %s"
	" the confirmation of your e-mail %s,"
	" click on the following link"
	" to confirm that direction:"
	" %s/?ActCod=%ld&MailKey=%s\n\n"
	"If it was not you, do not click on the link above!"
	" In this case, we recommend that you check in %s"
	" if you have confirmed your e-mail address.\n\n";	// Necessita de traduo
#endif

const char *Txt_If_you_no_longer_wish_to_receive_email_notifications_NO_HTML[Txt_NUM_LANGUAGES] =	// Don't use HTML entities like &egrave; here
	{
	"Si no desitgeu rebre ms avisos per correu electrnicc,"
	   " desactiveu la opci corresponent en les preferncies del seu perfil.",
	"Wenn Sie nicht mehr erhalten mchten, E-Mail-Benachrichtigungen,"
	   " Deaktivieren Sie die Option in den Einstellungen Ihres Profils.",
	"If you no longer wish to receive e-mail notifications,"
	   " uncheck the option in the preferences of your profile.",
	"Si no desea recibir ms avisos por correo electrnico,"
	   " desactive la opcin correspondiente en las preferencias de su perfil.",
	"Si vous ne souhaitez plus recevoir de notifications par e-mail,"
	   " dcochez l'option dans les prfrences de votre profil.",
	"Si no desea recibir ms avisos por correo electrnico,"
	   " desactive la opcin correspondiente en las preferencias de su perfil.",	// Okoteve traduccin
	"Se non desideri pi ricevere notifiche via e-mail,"
	   " deselezionare l'opzione nelle preferenze del tuo profilo.",
	"Jesli nie chcesz otrzymywac powiadomien e-mail,"
	   " usu&nacute; zaznaczenie opcje w ustawieniach swojego profilu.",
	"Se j no desejar receber notificaes por e-mail,"
	   " desmarque a opo nas preferncias do seu perfil.",
	};

const char *Txt_Institutions_NO_HTML =
#if   L==0
	"Institucions";
#elif L==1
	"Hochschulen";
#elif L==2
	"Institutions";
#elif L==3
	"Instituciones";
#elif L==4
	"tablissements";
#elif L==5
	"Mbo'ehao";
#elif L==6
	"Istituzioni";
#elif L==7
	"Instytucje";
#elif L==8
	"Institues";
#endif

const char *Txt_MSG_From_NO_HTML[Txt_NUM_LANGUAGES] =	// Don't use HTML entities like &egrave; here
	{
	"De",
	"Von",
	"From",
	"De",
	"De",
	"De",	// Okoteve traduccin
	"Da",
	"Od",
	"De",
	};

const char *Txt_Notifications_NO_HTML[Txt_NUM_LANGUAGES] =	// Don't use HTML entities like &egrave; here
	{
	"Notificacions",
	"Anmeldungen",
	"Notifications",
	"Notificaciones",
	"Notifications",
	"Momarandu",
	"Notifiche",
	"Powiadomienia",
	"Notificaes",
	};

const char *Txt_New_password_NO_HTML[Txt_NUM_LANGUAGES] =	// Don't use HTML entities like &ntilde; here.
	{
	"Nova contrasenya",
	"Neues Passwort",
	"New password",
	"Nueva contrasea",
	"Nouveau mot de passe",
	"Nueva contrasea",	// Okoteve traduccin
	"Nuova password",
	"Nowe has&lstrok;o",
	"Nova senha",
	};

const char *Txt_NOTIFY_EVENTS_SINGULAR_NO_HTML[Ntf_NUM_NOTIFY_EVENTS][Txt_NUM_LANGUAGES] =	// Don't use HTML entities like &uuml; here.
{
	{
	// Ntf_EVENT_UNKNOWN
	"Esdeveniment desconegut",
	"Unbekannt Ereignis",
	"Unknown event",
	"Evento desconocido",
	"vnement inconnu",
	"Evento desconocido",			// Okoteve traduccin
	"Evento sconosciuto",
	"Nieznane zdarzenia",
	"Evento desconhecido",
	},
	{
	// Ntf_EVENT_DOCUMENT_FILE
	"Nou document",
	"Neue Dokumentdatei",
	"New document file",
	"Nuevo documento",
	"Nouveau fichier de document",
	"Nuevo documento",			// Okoteve traduccin
	"Nuovo documento",
	"Nowy plik dokumentu",
	"Novo arquivo de documento",
	},
	{
	// Ntf_EVENT_SHARED_FILE
	"Nou arxiu compartit",
	"Neue freigegebene Datei",
	"New shared file",
	"Nuevo archivo compartido",
	"Nouveau fichier partag",
	"Nuevo archivo compartido",		// Okoteve traduccin
	"Nuovo file condiviso",
	"Nowy Udostepniony plik",
	"Novo arquivo compartilhado",
	},
	{
	// Ntf_EVENT_ASSIGNMENT
	"Nova activitat",
	"Neue Aufgabe",
	"New assignment",
	"Nueva actividad",
	"Nouvelle activit",
	"Nueva actividad",			// Okoteve traduccin
	"Nuova attivit",
	"Nowe zadania",
	"Nova atividade",
	},
	{
	// Ntf_EVENT_EXAM_ANNOUNCEMENT
	"Nova convocatria d'examen",
	"Neue Aufrufe fr Prfung",
	"New announcement of exam",
	"Nueva convocatoria de examen",
	"Nouvelle convocation  un examen",
	"Nueva convocatoria de examen",		// Okoteve traduccin
	"Nuova sessione d'esame",
	"Nowe ogloszenie egzaminu",
	"Nova chamada para exame",
	},
	{
	// Ntf_EVENT_MARKS_FILE
	"Nou fitxer de calificacions",
	"Neue Datei mit Bewertung",
	"New file with marks",
	"Nuevo archivo de calificaciones",
	"Nouveau fichier avec des notes",
	"Nuevo archivo de calificaciones",	// Okoteve traduccin
	"Nuovo file con i risultati",
	"Nowy plik ze znakami",
	"Novo arquivo de notas",
	},
	{
	// Ntf_EVENT_ENROLLMENT
	"Nova inscripci",
	"New Einschreibung",
	"New enrollment",
	"Nueva inscripcin",
	"Nouvelle inscription",
	"Nueva inscripcin",			// Okoteve traduccin
	"Nuova iscrizione",
	"Nowe zapisy",
	"Nova inscrio",
	},
	{
	// Ntf_EVENT_ENROLLMENT_REQUEST
	"Nova petici d'inscripci",
	"New Registrierungsanforderung",
	"New enrollment request",
	"Nueva peticin de inscripcin",
	"Nouvelle demande d'inscription",
	"Nueva peticin de inscripcin",	// Okoteve traduccin
	"Nuova richiesta di iscrizione",
	"Nowe prosby o rejestracji",
	"Novo pedido de inscrio",
	},
	{
	// Ntf_EVENT_NOTICE
	"Nou avs",
	"Neuen Ankndigungen",
	"New notice",
	"Nuevo aviso",
	"Nouvel avis",
	"Nuevo aviso",				// Okoteve traduccin
	"Nuovo avviso",
	"Nowe obwieszczenie",
	"Novo anncio",
	},
	{
	// Ntf_EVENT_FORUM_POST_COURSE
	"Novo missatge en un frum",
	"Neue Nachricht in einem Forum",
	"New post in a forum",
	"Nuevo mensaje en un foro",
	"Nouveau message dans un forum",
	"Nuevo mensaje en un foro",		// Okoteve traduccin
	"Nuovo post in un forum",
	"Post nowe na forum",
	"Novo post em um frum",
	},
	{
	// Ntf_EVENT_FORUM_REPLY
	"Nova resposta en un frum",
	"Neue Antwort in einem Forum",
	"New response in a forum",
	"Nueva respuesta en un foro",
	"Nouvelle rponse dans un forum",
	"Nueva respuesta en un foro",		// Okoteve traduccin
	"Nuova risposta in un forum",
	"Odpowiedz nowe na forum",
	"Nova resposta em um frum",
	},
	{
	// Ntf_EVENT_MESSAGE
	"Nou missatge",
	"Neue Nachricht",
	"New message",
	"Nuevo mensaje",
	"Nouveau message",
	"Nuevo mensaje",			// Okoteve traduccin
	"Nuovo messaggio",
	"Nowa wiadomosc",
	"Nova mensagem",
	},
	{
	// Ntf_EVENT_SURVEY
	"Nova enquesta",
	"Neuer Umfrage",
	"New survey",
	"Nueva encuesta",
	"Nouveau sondage",
	"Nueva encuesta",			// Okoteve traduccin
	"Nuovo sondaggio",
	"Nowe badania",
	"Novo inqu&eacute;rito",
	},
};

const char *Txt_NOTIFY_EVENTS_There_is_a_new_event_NO_HTML[Txt_NUM_LANGUAGES] =	// Warning: it is very important to include %s in the following sentences
	 {			// Don't use HTML entities like &uuml; here.
	 "Hi ha un nou esdeveniment en %s",
	 "Es ist eine neue Ereignis in %s",
	 "There is a new event in %s",
	 "Hay un nuevo evento en %s",
	 "Il y un nouvel vnement dans %s",
	 "Hay un nuevo evento en %s",	// Okoteve traduccin
	 "C' un nuovo evento in %s",
	 "Jest nowe wydarzenie w %s",
	 "Existe um novo evento em %s",
	 };

const char *Txt_NOTIFY_EVENTS_There_are_X_new_events_NO_HTML[Txt_NUM_LANGUAGES] =	// Warning: it is very important to include %u and %s in the following sentences
	 {			// Don't use HTML entities like &uuml; here.
	 "Hi ha %u nous esdeveniments en %s",
	 "Es gibt %u neue Ereignisse in %s",
	 "There are %u new events in %s",
	 "Hay %u nuevos eventos en %s",
	 "Il y a %u nouveaux vnements dans %s",
	 "Hay %u nuevos eventos en %s",	// Okoteve traduccin
	 "Ci sono %u nuovi eventi in %s",
	 "Istnieje %u nowych wydarzen w %s",
	 "H %u novos eventos em %s",
	 };

const char *Txt_only_teachers_NO_HTML[Txt_NUM_LANGUAGES] =	// Don't use HTML entities like &oacute; here
	{
	", professors",
	", Lehrkrfte",
	", teachers",
	", profesores",
	", enseignants",
	", profesores",	// Okoteve traduccin
	", professori",
	", nauczyciel",
	", professores",
	};

const char *Txt_Please_do_not_reply_to_this_automatically_generated_email_NO_HTML[Txt_NUM_LANGUAGES] =	// Don't use HTML entities like &egrave; here
	{
	"Si us plau, no respongui a aquest correu electrnic generat automticament.",
	"Bitte, antworten Sie nicht auf diese automatisch generierte E-Mail.",
	"Please do not reply to this automatically generated e-mail.",
	"Por favor, no responda a este correo electrnico generado automticamente.",
	"S'il vous plat, ne pas rpondre  ce courrier lectronique gnr automatiquement.",
	"Por favor, no responda a este correo electrnico generado automticamente.",	// Okoteve traduccin
	"Per favore, non rispondere a questa e-mail generata automaticamente.",
	"Prosimy nie odpowiadac na to automatycznie wygenerowany e-mail.",
	"Por favor, no responda a este e-mail gerado automaticamente.",
	 };

const char *Txt_TAB_Messages_NO_HTML[Txt_NUM_LANGUAGES] =	// Don't use HTML entities like &egrave; here
	{
	"Missatges",
	"Nachrichten",
	"Messages",
	"Mensajes",
	"Messages",
	"Marandu",
	"Messaggi",
	"Wiadomosci",
	"Mensagens",
	};

const char *Txt_The_following_password_has_been_assigned_to_you_to_log_in_X_NO_HTML =	// Warning: it is very important to include %s and %u in the following sentences. Don't use HTML entities like &egrave; here
#if   L==0
	"Se li ha assignat la segent contrasenya per entrar a %s: %s\n"
	"Si desitja activar aquesta nova contrasenya,"
	" ha de entrar a %s amb el seu ID (DNI/cdula), sobrenom o e-mail"
	" i aquesta nova contrasenya abans de %u dies."
	" Un cop hagi entrat, pot canviar la contrasenya per una altra.\n"
	"Si no accedeix amb aquesta nova contrasenya,"
	" seguir tenint la contrasenya que tingus fins ara.\n"
	"Si ha rebut aquest missatge sense sollicitar-lo,"
	" es deu a que alg que coneix el seu ID (DNI/cdula), el seu sobrenom o el seu e-mail"
	" ha sollicitat l'enviament de una nova contrasenya per a vost.\n"
	"Aquesta nova contrasenya se ha enviat noms a la adrea de correu %s,"
	" que s la que apareix a la seva fitxa.\n\n";
#elif L==1
	"The following password has been assigned to you to log in %s: %s\n"
	"If you want to activate the new password,"
	" you must enter %s with your ID/nickname/e-mail"
	" and this new password before %u days."
	" Once you have logged in, you can change your password.\n"
	"If you do not log in with this new password,"
	" the old password will remain valid.\n"
	"If you have received this message without having requested it,"
	" is that someone who knows your ID, nickname or e-mail has requested a new password sent to you.\n"
	"This new password has been sent only to the email address %s, which is on your record card.\n\n";	// Need bersetzung
#elif L==2
	"The following password has been assigned to you to log in %s: %s\n"
	"If you want to activate the new password,"
	" you must enter %s with your ID/nickname/e-mail"
	" and this new password before %u days."
	" Once you have logged in, you can change your password.\n"
	"If you do not log in with this new password,"
	" the old password will remain valid.\n"
	"If you have received this message without having requested it,"
	" is that someone who knows your ID, nickname or e-mail has requested a new password sent to you.\n"
	"This new password has been sent only to the email address %s, which is on your record card.\n\n";
#elif L==3
	"Se le ha asignado la siguiente contrasea para entrar en %s: %s\n"
	"Si desea activar esta nueva contrasea,"
	" debe entrar en %s con su ID (DNI/cdula), apodo o direccin de correo electrnico"
	" y esta nueva contrasea antes de %u das."
	" Una vez que haya entrado, puede cambiar la contrasea por otra.\n"
	"Si no accede con esta nueva contrasea,"
	" seguir teniendo la contrasea que tuviera hasta ahora.\n"
	"Si ha recibido este mensaje sin solicitarlo,"
	" se debe a que alguien que conoce su ID (DNI/cdula), su apodo o su direccin de correo electrnico"
	" ha solicitado el envo de una nueva contrasea para usted.\n"
	"Esta nueva contrasea se ha enviado slo a la direccin de correo %s,"
	" que es la que aparece en su ficha.\n\n";
#elif L==4
	"The following password has been assigned to you to log in %s: %s\n"
	"If you want to activate the new password,"
	" you must enter %s with your ID/nickname/e-mail"
	" and this new password before %u days."
	" Once you have logged in, you can change your password.\n"
	"If you do not log in with this new password,"
	" the old password will remain valid.\n"
	"If you have received this message without having requested it,"
	" is that someone who knows your ID, nickname or e-mail has requested a new password sent to you.\n"
	"This new password has been sent only to the email address %s, which is on your record card.\n\n";	// Besoin de traduction
#elif L==5
	"Se le ha asignado la siguiente contrasea para entrar en %s: %s\n"
	"Si desea activar esta nueva contrasea,"
	" debe entrar en %s con su ID (DNI/c&eacute;dula), apodo o direccin de correo electrnico"
	" y esta nueva contrasea antes de %u das."
	" Una vez que haya entrado, puede cambiar la contrasea por otra.\n"
	"Si no accede con esta nueva contrasea,"
	" seguir teniendo la contrasea que tuviera hasta ahora.\n"
	"Si ha recibido este mensaje sin solicitarlo,"
	" se debe a que alguien que conoce su ID (DNI/cdula), su apodo o su direccin de correo electrnico"
	" ha solicitado el envo de una nueva contrasea para usted.\n"
	"Esta nueva contrasea se ha enviado slo a la direccin de correo %s,"
	" que es la que aparece en su ficha.\n\n";	// Okoteve traduccin
#elif L==6
	"La seguente password ti  stata assegnata per entrare in %s: %s\n"
	"Se vuoi attivare la nuova password,"
	" devi entrare in %s con il tuo ID / nome utente / e-mail"
	" e li c' la nuova password prima di %u giorni."
	" Una volta che ti sei loggato, puoi scegliere la tua password.\n"
	"Se tu non entri con questa nuova password,"
	" rimarr valida la vecchia password.\n"
	"Se hai ricevuto questo messaggio senza averlo richiesto,"
	"  perch qualcuno che sa il tuo ID, nome utente o e-mail ha richiesto una nuova password da inviarti.\n"
	"Questa nuova password  stata inviata solo all'indirizzo e-mail %s, che  presente sulla tua scheda.\n\n";
#elif L==7
	"The following password has been assigned to you to log in %s: %s\n"
	"If you want to activate the new password,"
	" you must enter %s with your ID/nickname/e-mail"
	" and this new password before %u days."
	" Once you have logged in, you can change your password.\n"
	"If you do not log in with this new password,"
	" the old password will remain valid.\n"
	"If you have received this message without having requested it,"
	" is that someone who knows your ID, nickname or e-mail has requested a new password sent to you.\n"
	"This new password has been sent only to the email address %s, which is on your record card.\n\n";	// Potrzebujesz tlumaczenie
#elif L==8
	"The following password has been assigned to you to log in %s: %s\n"
	"If you want to activate the new password,"
	" you must enter %s with your ID/nickname/e-mail"
	" and this new password before %u days."
	" Once you have logged in, you can change your password.\n"
	"If you do not log in with this new password,"
	" the old password will remain valid.\n"
	"If you have received this message without having requested it,"
	" is that someone who knows your ID, nickname or e-mail has requested a new password sent to you.\n"
	"This new password has been sent only to the email address %s, which is on your record card.\n\n";	// Necessita de traduo
#endif

const char *Txt_UPLOAD_FILE_could_not_create_file_NO_HTML =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"%s: no s'ha pogut crear el fitxer.";
#elif L==1
	"%s: Datei konnte nicht erstellt werden.";
#elif L==2
	"%s: could not create file.";
#elif L==3
	"%s: no se ha podido crear el archivo.";
#elif L==4
	"%s: impossible de crer le fichier.";
#elif L==5
	"%s: no se ha podido crear el archivo.";	// Okoteve traduccin
#elif L==6
	"%s: impossibile creare il file.";
#elif L==7
	"%s: nie mozna utworzyc plik.";
#elif L==8
	"%s: no foi possvel criar arquivo.";
#endif

const char *Txt_UPLOAD_FILE_Forbidden_NO_HTML =
#if   L==0
	"No puede crear archivos aqu.";	// Necessita traduccio
#elif L==1
	"You can not create files here.";	// Need bersetzung
#elif L==2
	"You can not create files here.";
#elif L==3
	"No puede crear archivos aqu.";
#elif L==4
	"You can not create files here.";	// Besoin de traduction
#elif L==5
	"No puede crear archivos aqu.";	// Okoteve traduccin
#elif L==6
	"Non puoi creare file qui.";
#elif L==7
	"You can not create files here.";	// Potrzebujesz tlumaczenie
#elif L==8
	"You can not create files here.";	// Necessita de traduo
#endif

const char *Txt_UPLOAD_FILE_You_must_specify_the_file_NO_HTML =
#if   L==0
	"Debe especificar el archivo.";	// Necessita traduccio
#elif L==1
	"Die Datei muss angegeben werden.";
#elif L==2
	"You must specify the file.";
#elif L==3
	"Debe especificar el archivo.";
#elif L==4
	"Vous devez spcifier le fichier.";
#elif L==5
	"Debe especificar el archivo.";	// Okoteve traduccin
#elif L==6
	" necessario specificare il file.";
#elif L==7
	"You must specify the file.";	// Potrzebujesz tlumaczenie
#elif L==8
	"Voc deve especificar o arquivo.";
#endif

const char *Txt_UPLOAD_FILE_X_extension_not_allowed_NO_HTML =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"%s: extensin no permitida.";		// Necessita traduccio
#elif L==1
	"%s: Endung nicht untersttzt.";
#elif L==2
	"%s: extension not allowed.";
#elif L==3
	"%s: extensin no permitida.";
#elif L==4
	"%s: extension not allowed.";		// Besoin de traduction
#elif L==5
	"%s: extensin no permitida.";		// Okoteve traduccin
#elif L==6
	"%s: estensione non permessa.";
#elif L==7
	"%s: extension not allowed.";		// Potrzebujesz tlumaczenie
#elif L==8
	"%s: extension not allowed.";		// Necessita de traduo
#endif

const char *Txt_UPLOAD_FILE_X_file_already_exists_NO_HTML =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"%s: el archivo ya existe.";	// Necessita traduccio
#elif L==1
	"%s: file already exists.";	// Need bersetzung
#elif L==2
	"%s: file already exists.";
#elif L==3
	"%s: el archivo ya existe.";
#elif L==4
	"%s: file already exists.";	// Besoin de traduction
#elif L==5
	"%s: el archivo ya existe.";	// Okoteve traduccin
#elif L==6
	"%s: file already exists.";	// Bisogno di traduzione
#elif L==7
	"%s: file already exists.";	// Potrzebujesz tlumaczenie
#elif L==8
	"%s: file already exists.";	// Necessita de traduo
#endif

const char *Txt_UPLOAD_FILE_File_too_large_maximum_X_MiB_NO_HTML =	// Warning: it is very important to include %lu in the following sentences
#if   L==0
	"Archivo demasiado grande (mximo %lu MiB) /"
	" un altre problema d'enviament.";	// Necessita traduccio
#elif L==1
	"Datei zu gro (Maximum %lu MiB) /"
	" ein weiteres Upload-Problem.";
#elif L==2
	"File too large (maximum %lu MiB) /"
	" another upload problem.";
#elif L==3
	"Archivo demasiado grande (mximo %lu MiB) /"
	" otro problema de envo.";
#elif L==4
	"Fichier est trop gros (maximum %lu MiB) /"
	" un autre problme de tlchargement.";
#elif L==5
	"Archivo demasiado grande (mximo %lu MiB) /"
	" otro problema de envo.";		// Okoteve traduccin
#elif L==6
	"File troppo grande (massimo %lu MiB) /"
	" un altro problema di caricamento.";
#elif L==7
	"Plik jest za duzy (maksimum %lu MiB) /"
	" kolejny problem wgrac.";
#elif L==8
	"Arquivo muito grande (mximo %lu MiB) /"
	" outro problema de upload.";
#endif

const char *Txt_UPLOAD_FILE_Invalid_name_NO_HTML =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"Nombre no vlido.";	// Necessita traduccio
#elif L==1
	"Invalid name.";	// Need bersetzung
#elif L==2
	"Invalid name.";
#elif L==3
	"Nombre no vlido.";
#elif L==4
	"Invalid name.";	// Besoin de traduction
#elif L==5
	"Nombre no vlido.";	// Okoteve traduccin
#elif L==6
	"Nome non valido.";
#elif L==7
	"Invalid name.";	// Potrzebujesz tlumaczenie
#elif L==8
	"Invalid name.";	// Necessita de traduo
#endif

const char *Txt_UPLOAD_FILE_X_invalid_name_NO_HTML =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"%s: nombre no vlido.";	// Necessita traduccio
#elif L==1
	"%s: invalid name.";		// Need bersetzung
#elif L==2
	"%s: invalid name.";
#elif L==3
	"%s: nombre no vlido.";
#elif L==4
	"%s: invalid name.";		// Besoin de traduction
#elif L==5
	"%s: nombre no vlido.";	// Okoteve traduccin
#elif L==6
	"%s: nome non valido.";
#elif L==7
	"%s: invalid name.";		// Potrzebujesz tlumaczenie
#elif L==8
	"%s: invalid name.";		// Necessita de traduo
#endif

const char *Txt_UPLOAD_FILE_X_MIME_type_Y_not_allowed_NO_HTML =	// Warning: it is very important to include two %s in the following sentences
#if   L==0
	"%s: tipo MIME %s no permitido.";	// Necessita traduccio
#elif L==1
	"%s: MIME-Typ %s nicht untersttzt.";
#elif L==2
	"%s: MIME type %s not allowed.";
#elif L==3
	"%s: tipo MIME %s no permitido.";
#elif L==4
	"%s: MIME type %s not allowed.";	// Besoin de traduction
#elif L==5
	"%s: tipo MIME %s no permitido.";	// Okoteve traduccin
#elif L==6
	"%s: tipo MIME %s non consentito.";
#elif L==7
	"%s: MIME type %s not allowed.";	// Potrzebujesz tlumaczenie
#elif L==8
	"%s: MIME type %s not allowed.";	// Necessita de traduo
#endif

const char *Txt_UPLOAD_FILE_X_not_HTML_NO_HTML =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"%s: no HTML";		// Necessita traduccio
#elif L==1
	"%s: not HTML";		// Need bersetzung
#elif L==2
	"%s: not HTML";
#elif L==3
	"%s: no HTML";		// Bisogno di traduzione
#elif L==4
	"%s: not HTML";		// Besoin de traduction
#elif L==5
	"%s: no HTML";		// Okoteve traduccin
#elif L==6
	"%s: not HTML";
#elif L==7
	"%s: not HTML";		// Potrzebujesz tlumaczenie
#elif L==8
	"%s: not HTML";		// Necessita de traduo
#endif

const char *Txt_UPLOAD_FILE_X_quota_exceeded_NO_HTML =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"%s: cuota excedida.";		// Necessita traduccio
#elif L==1
	"%s: Kontingent berschritten.";
#elif L==2
	"%s: quota exceeded.";
#elif L==3
	"%s: cuota excedida.";
#elif L==4
	"%s: quota dpass.";
#elif L==5
	"%s: cuota excedida.";		// Okoteve traduccin
#elif L==6
	"%s: quota superata.";
#elif L==7
	"%s: przekroczono przydzial.";
#elif L==8
	"5s: quota excedida.";
#endif

const char *Txt_UPLOAD_FILE_Upload_time_too_long_maximum_X_minutes_NO_HTML =	// Warning: it is very important to include %lu in the following sentences
#if   L==0
	"Tiempo de subida demasiado largo (mximo %lu minutos).";	// Necessita traduccio
#elif L==1
	"Upload ist zu lang (Maximum %lu Minute).";
#elif L==2
	"Upload time too long (maximum %lu minutes).";
#elif L==3
	"Tiempo de subida demasiado largo (mximo %lu minutos).";
#elif L==4
	"Temps de tlchargement trop longs (maximum %lu minutes).";
#elif L==5
	"Tiempo de subida demasiado largo (mximo %lu minutos).";	// Okoteve traduccin
#elif L==6
	"Durata del caricamento troppo lungo (massimo %lu minuti).";
#elif L==7
	"Upload time too long (maximum %lu minuty).";			// Potrzebujesz tlumaczenie
#elif L==8
	"Tempo de upload muito longo (mximo %lu minutos).";
#endif

const char *Txt_user_NO_HTML[Usr_NUM_SEXS][Txt_NUM_LANGUAGES] =	// Don't use HTML entities like &egrave; here
{
	{
	// Usr_SEX_UNKNOWN
	"usuari/a",
	"Benutzer",
	"user",
	"usuario/a",
	"utilisateur/se",
	"puruhra",
	"utente",
	"uzytkownika",
	"utilizador/a",
	},
	{
	// Usr_SEX_FEMALE
	"usuaria",
	"Benutzer",
	"user",
	"usuaria",
	"utilisateuse",
	"usuaria",	// Okoteve traduccin
	"utente",
	"uzytkownika",
	"utilizadora",
	},
	{
	// Usr_SEX_MALE
	"usuari",
	"Benutzer",
	"user",
	"usuario",
	"utilisateur",
	"puruhra",
	"utente",
	"uzytkownika",
	"utilizador",
	},
	{
	// Usr_SEX_ALL
	"usuari/a",
	"Benutzer",
	"user",
	"usuario/a",
	"utilisateur/se",
	"puruhra",
	"utente",
	"uzytkownika",
	"utilizador/a",
	}
};

const char *Txt_Users_NO_HTML =
#if   L==0
	"Usuaris";
#elif L==1
	"Benutzer";
#elif L==2
	"Users";
#elif L==3
	"Usuarios";
#elif L==4
	"Utilisateurs";
#elif L==5
	"Puruh&aacute;ra";
#elif L==6
	"Utenti";
#elif L==7
	"Uzytkownikw";
#elif L==8
	"Utilizadores";
#endif

/***** Messages translated (use HTML entities) *****/

const char *Txt_A_face_marked_in_green_has_been_detected_ =
#if   L==0
	"S'ha detectat un rostre (senyalat en verd) en posici&oacute; frontal"
	" i amb el fons darrere del cap prou clar.<br />"
	"Premeu sobre ell per confirmar la actualizaci&oacute; de la fotografia.";
#elif L==1
	"A face (marked in green) has been detected in front position"
	" and the background behind the head is light enough.<br />"
	"Click on it to confirm the update of the picture.";						// Need bersetzung
#elif L==2
	"A face (marked in green) has been detected in front position"
	" and the background behind the head is light enough.<br />"
	"Click on it to confirm the update of the picture.";
#elif L==3
	"Se ha detectado un rostro (se&ntilde;alado en verde) en posici&oacute;n frontal"
	" y con el fondo detr&aacute;s de la cabeza suficientemente claro.<br />"
	"Pulse sobre &eacute;l para confirmar la actualizaci&oacute;n de la fotograf&iacute;a.";
#elif L==4
	"A face (marked in green) has been detected in front position"
	" and the background behind the head is light enough.<br />"
	"Click on it to confirm the update of the picture.";						// Besoin de traduction
#elif L==5
	"Se ha detectado un rostro (se&ntilde;alado en verde) en posici&oacute;n frontal"
	" y con el fondo detr&aacute;s de la cabeza suficientemente claro.<br />"
	"Pulse sobre &eacute;l para confirmar la actualizaci&oacute;n de la fotograf&iacute;a.";	// Okoteve traduccin
#elif L==6
	"Un viso (segnato in verde) &egrave; stato individuato nella posizione centrale"
	" e lo sfondo dietro la testa &egrave; abbastanza chiaro.<br />"
	"Clicca su di esso per confermare l'aggiornamento dell'immagine.";
#elif L==7
	"twarzy (zaznaczone na zielono) zostal wykryty w prz&oacute;d w pozycji "
	"a tlo z tylu glowy jest wystarczajaco lekki.<br />"
	"Kliknij na nia, aby potwierdzic aktualizacje obrazu.";
#elif L==8
	"A face (marked in green) has been detected in front position"
	" and the background behind the head is light enough.<br />"
	"Click on it to confirm the update of the picture.";						// Necessita de traduo
#endif

const char *Txt_A_face_marked_in_red_has_been_detected_ =
#if   L==0
	"S'ha detectat un rostre (senyalat en vermell) en posici&oacute; frontal,"
	" per&ograve; el fons darrere del cap s massa fosc.";
#elif L==1
	"A face (marked in green) has been detected in front position,"
	" but the background behind the head is too dark.";			// Need bersetzung
#elif L==2
	"A face (marked in green) has been detected in front position,"
	" but the background behind the head is too dark.";
#elif L==3
	"Se ha detectado un rostro (se&ntilde;alado en rojo) en posici&oacute;n frontal,"
	" pero el fondo detr&aacute;s de la cabeza es demasiado oscuro.";
#elif L==4
	"A face (marked in green) has been detected in front position,"
	" but the background behind the head is too dark.";			// Besoin de traduction
#elif L==5
	"Se ha detectado un rostro (se&ntilde;alado en rojo) en posici&oacute;n frontal,"
	" pero el fondo detr&aacute;s de la cabeza es demasiado oscuro.";	// Okoteve traduccin
#elif L==6
	"Un viso (segnato in verde) &egrave; stato individuato nella posizione centrale,"
	" ma lo sfondo dietro la testa &egrave; troppo scuro.";
#elif L==7
	"twarzy (zaznaczone na zielono) zostal wykryty w przednim"
	"Ale w tle z tylu glowy jest zbyt ciemny.";
#elif L==8
	"A face (marked in green) has been detected in front position,"
	" but the background behind the head is too dark.";			// Necessita de traduo
#endif

const char *Txt_A_message_has_been_sent_to_email_address_X_to_confirm_that_address =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"Se ha enviado un mensaje"
	" a la direcci&oacute;n de correo electr&oacute;nico %s"
	" para confirmar dicha direcci&oacute;n.";	// Necessita traduccio
#elif L==1
	"A message has been sent"
	" to e-mail address %s"
	" to confirm that address.";			// Need bersetzung
#elif L==2
	"A message has been sent"
	" to e-mail address %s"
	" to confirm that address.";
#elif L==3
	"Se ha enviado un mensaje"
	" a la direcci&oacute;n de correo electr&oacute;nico %s"
	" para confirmar dicha direcci&oacute;n.";
#elif L==4
	"A message has been sent"
	" to e-mail address %s"
	" to confirm that address.";			// Besoin de traduction
#elif L==5
	"Se ha enviado un mensaje"
	" a la direcci&oacute;n de correo electr&oacute;nico %s"
	" para confirmar dicha direcci&oacute;n.";	// Okoteve traduccin
#elif L==6
	"A message has been sent"
	" to e-mail address %s"
	" to confirm that address.";			// Bisogno di traduzione
#elif L==7
	"A message has been sent"
	" to e-mail address %s"
	" to confirm that address.";			// Potrzebujesz tlumaczenie
#elif L==8
	"A message has been sent"
	" to e-mail address %s"
	" to confirm that address.";			// Necessita de traduo
#endif

const char *Txt_A_student_can_belong_to_several_groups =
#if   L==0
	"Un estudiant pot pert&agrave;nyer a diversos grups";
#elif L==1
	"Ein Student kann mehreren Gruppen angehren";
#elif L==2
	"A student can belong to several groups";
#elif L==3
	"Un estudiante puede pertenecer a varios grupos";
#elif L==4
	"Un &eacute;tudiant peut appartenir &agrave; plusieurs groupes";
#elif L==5
	"Un estudiante puede pertenecer a varios grupos";	// Okoteve traduccin
#elif L==6
	"Uno studente pu&ograve; appartenere a pi&ugrave; gruppi";
#elif L==7
	"Kazdy student moze nalezec do wielu grup";
#elif L==8
	"Um aluno pode pertencer a v&aacute;rios grupos";
#endif

const char *Txt_A_student_can_only_belong_to_one_group =
#if   L==0
	"Un estudiant nom&eacute;s pot pert&agrave;nyer a un grup";
#elif L==1
	"Ein Student kann nur einer Gruppe angeh&ouml;ren";
#elif L==2
	"A student can only belong to one group";
#elif L==3
	"Un estudiante s&oacute;lo puede pertenecer a un grupo";
#elif L==4
	"Un &eacute;tudiant ne peut appartenir &agrave; un groupe";
#elif L==5
	"Un estudiante s&oacute;lo puede pertenecer a un grupo";	// Okoteve traduccin
#elif L==6
	"Uno studente pu&ograve; appartenere solo a un gruppo";
#elif L==7
	"student moze nalezec tylko do jednej grupy";
#elif L==8
	"Um aluno pode pertencer a um grupo";
#endif

const char *Txt_A_teacher_or_administrator_has_enrolled_you_into_the_course_ =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"Un profesor/a o un administrador/a le ha inscrito"
	" en la asignatura <strong>%s</strong>."
	" &iquest;Confirma la inscripci&oacute;n"
	" o prefiere ser eliminado de la asignatura?";		// Necessita traduccio
#elif L==1
	"Ein Lehrer oder Administrator hat Sie"
	" im Kurs <strong>%s</strong> eingeschrieben."
	" M&ouml;chten Sie, dass die Registrierung"
	" oder bevorzugen Sie, um aus dem Kurs?";
#elif L==2
	"A teacher or administrator has enrolled you"
	" into the course <strong>%s</strong>."
	" Do you want to confirm the enrollment"
	" or do you prefer to be removed from the course?";
#elif L==3
	"Un profesor/a o un administrador/a le ha inscrito"
	" en la asignatura <strong>%s</strong>."
	" &iquest;Confirma la inscripci&oacute;n"
	" o prefiere ser eliminado de la asignatura?";
#elif L==4
	"Un professeur ou un administrateur vous a enregistr&eacute;"
	" dans la mati&egrave;re <strong>%s</strong>."
	" Voulez-vous confirmer l'enregistrement"
	" ou pr&eacute;f&eacute;rez-vous &ecirc;tre enlev&eacute; de la mati&egrave;re?";
#elif L==5
	"Un profesor/a o un administrador/a le ha inscrito "
	" en la asignatura <strong>%s</strong>."
	" &iquest;Confirma la inscripci&oacute;n"
	" o prefiere ser eliminado de la asignatura?";		// Okoteve traduccin
#elif L==6
	"Un insegnante o amministratore ti a registrato"
	" nel corso <strong>%s</strong>."
	" Vuoi confermare la registrazione"
	" o preferisci essere rimosso da questo corso?";
#elif L==7
	"A teacher or administrator has enrolled you"
	" into the course <strong>%s</strong>."
	" Do you want to confirm the enrollment"
	" or do you prefer to be removed from the course?";	// Potrzebujesz tlumaczenie
#elif L==8
	"Um professor ou administrador tem registrado voc&ecirc;"
	" na disciplina <strong>%s</strong>."
	" Voc&ecirc; quer para confirmar a inscri&ccedil;&atilde;o"
	" ou voc&ecirc; prefere para ser removido da disciplina?";
#endif

const char *Txt_A_user_can_not_have_more_than_X_IDs =
#if   L==0
	"Un usuari no pot tenir m&eacute;s de %u IDs (DNI/c&egrave;dulas)";
#elif L==1
	"Ein Benutzer kann nicht mehr als %u Ausweis-Nr.";
#elif L==2
	"A user can not have more than %u IDs.";
#elif L==3
	"Un usuario no puede tener m&aacute;s de %u ID (DNI/c&eacute;dulas).";
#elif L==4
	"Un utilisateur ne peut pas avoir plus de %u IDs.";
#elif L==5
	"Un usuario no puede tener m&aacute;s de %u ID (DNI/c&eacute;dulas).";	// Okoteve traduccin
#elif L==6
	"Un utente non pu&ograve; avere pi&ugrave; di %u ID.";
#elif L==7
	"U&zdot;ytkownik nie mo&zdot;e mie&cacute; wi&eogon;cej ni&zdot; %u identyfikatory.";
#elif L==8
	"Um usu&aacute;rio n&atilde;o pode ter mais de %u n&ordm;s. identif.";
#endif

const char *Txt_About_X =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"Quant a %s";
#elif L==1
	"&Uuml;ber uns %s";
#elif L==2
	"About %s";
#elif L==3
	"Acerca de %s";
#elif L==4
	"&Agrave; propos de %s";
#elif L==5
	"%s rehegua";
#elif L==6
	"Riguardo %s";
#elif L==7
	"O %s";
#elif L==8
	"Sobre %s";
#endif

const char *Txt_Absent =
#if   L==0
	"Absent";
#elif L==1
	"Nicht vorhanden";
#elif L==2
	"Absent";
#elif L==3
	"Ausente";
#elif L==4
	"Absent";
#elif L==5
	"Ausente";	// Okoteve traduccin
#elif L==6
	"Assente";
#elif L==7
	"Nieobecny";
#elif L==8
	"Assente";
#endif

const char *Txt_Absents =
#if   L==0
	"Absents";
#elif L==1
	"Nicht vorhanden";
#elif L==2
	"Absents";
#elif L==3
	"Ausentes";
#elif L==4
	"Absents";
#elif L==5
	"Ausentes";	// Okoteve traduccin
#elif L==6
	"Assenti";
#elif L==7
	"Nieobecny";
#elif L==8
	"Assentes";
#endif

const char *Txt_accessible_for_reading_and_writing_by_students_and_teachers_of_the_course =
#if   L==0
	"accessible per a lectura i escriptura per estudiants i professors de l'assignatura";
#elif L==1
	"zum Schreiben zug&auml;nglich f&uuml;r Studenten und Lehrkr&auml;fte der Kurs";
#elif L==2
	"accessible for reading and writing by students and teachers of the course";
#elif L==3
	"accesible para lectura y escritura por estudiantes y profesores de la asignatura";
#elif L==4
	"accessible en lecture et en &eacute;criture par les &eacute;tudiants et les enseignants de la mati&egrave;re";
#elif L==5
	"accesible para lectura y escritura por estudiantes y profesores de la asignatura";	// Okoteve traduccin
#elif L==6
	"accessibile per la lettura e la scrittura da parte degli studenti e dei professori del corso";
#elif L==7
	"dostep do odczytu i zapisu przez uczni&oacute;w i nauczycieli zajec";
#elif L==8
	"acess&iacute;vel para leitura e escrita pelos alunos e professores da disciplina";
#endif

const char *Txt_accessible_for_reading_and_writing_by_students_of_the_group_and_teachers_of_the_course =
#if   L==0
	"accesible per a lectura i escriptura per estudiants del grup i professors de l'assignatura";
#elif L==1
	"zum Schreiben zug&auml;nglich f&uuml;r Studenten der Gruppe und Lehrkr&auml;fte der Kurs";
#elif L==2
	"accessible for reading and writing by students of the group and teachers of the course";
#elif L==3
	"accesible para lectura y escritura por estudiantes del grupo y profesores de la asignatura";
#elif L==4
	"accessible en lecture et en &eacute;criture par les &eacute;tudiants du groupe et les enseignants de la mati&egrave;re";
#elif L==5
	"accesible para lectura y escritura por estudiantes del grupo y profesores de la asignatura";	// Okoteve traduccin
#elif L==6
	"accessibile per la lettura e la scrittura da parte degli studenti del gruppo e dei professori del corso";
#elif L==7
	"dostep do odczytu i zapisu przez student&oacute;w i nauczycieli, grupy kursu";
#elif L==8
	"acess&iacute;vel para leitura e escrita pelos alunos do grupo e professores da disciplina";
#endif

const char *Txt_accessible_for_reading_and_writing_by_teachers_of_the_course =
#if   L==0
	"accessible per a lectura i escriptura per professors de l'assignatura";
#elif L==1
	"zum Schreiben zug&auml;nglich f&uuml;r Lehrkr&auml;fte der Kurs";
#elif L==2
	"accessible for reading and writing by teachers of the course";
#elif L==3
	"accesible para lectura y escritura por profesores de la asignatura";
#elif L==4
	"accessible en lecture et en &eacute;criture par les enseignants de la mati&egrave;re";
#elif L==5
	"accesible para lectura y escritura por profesores de la asignatura";	// Okoteve traduccin
#elif L==6
	"accessibile per la lettura e la scrittura da parte dei professori del corso";
#elif L==7
	"dostep do odczytu i zapisu przez nauczycieli zajec";
#elif L==8
	"acess&iacute;vel para leitura e escrita pelos professores da disciplina";
#endif

const char *Txt_accessible_for_reading_and_writing_by_you_and_the_teachers_of_the_course =
#if   L==0
	"accessible per a lectura per vost&egrave; i els professors de l'assignatura";
#elif L==1
	"zum Schreiben zug&auml;nglich f&uuml;r Sie und die Lehrkr&auml;fte der Kurs";
#elif L==2
	"accessible for reading and writing by you and the teachers of the course";
#elif L==3
	"accesible para lectura por usted y los profesores de la asignatura";
#elif L==4
	"accessible en lecture et en &eacute;criture par vous et par les &eacute;tudiants et les enseignants de la mati&egrave;re";
#elif L==5
	"accesible para lectura por usted y los profesores de la asignatura";	// Okoteve traduccin
#elif L==6
	"accessibile per la lettura e la scrittura per te e i professori del corso";
#elif L==7
	"dostep do odczytu i zapisu przez Ciebie i nauczycieli zajec";
#elif L==8
	"acess&iacute;vel para leitura e escrita por voc&ecirc; e os professores da disciplina";
#endif

const char *Txt_accessible_only_for_reading_by_you_and_the_teachers_of_the_course =
#if   L==0
	"accesible nom&egrave; per a lectura per vost&eacute; i els professors de l'assignatura";
#elif L==1
	"zum Lesen zug&auml;nglich f&uuml;r Sie und die Lehrkr&auml;fte der Kurs";
#elif L==2
	"accessible only for reading by you and the teachers of the course";
#elif L==3
	"accesible s&oacute;lo para lectura por usted y los profesores de la asignatura";
#elif L==4
	"accessible uniquement pour la lecture par vous et par les &eacute;tudiants et les enseignants de la mati&egrave;re";
#elif L==5
	"accesible s&oacute;lo para lectura por usted y los profesores de la asignatura";	// Okoteve traduccin
#elif L==6
	"accessibile solo per la lettura per te e i professori del corso";
#elif L==7
	"dostepne tylko do czytania przez Ciebie i nauczycieli prowadzacych ten kurs";
#elif L==8
	"acess&iacute;vel apenas para leitura por voc&ecirc; e os professores da disciplina";
#endif

const char *Txt_accessible_only_for_reading_by_students_and_teachers_of_the_course =
#if   L==0
	"accessible nom&eacute;s per a lectura per estudiants i professors de l'assignatura";
#elif L==1
	"zum Lesen zug&auml;nglich f&uuml;r Studenten und Lehrkr&auml;fte der Kurs";
#elif L==2
	"accessible only for reading by students and teachers of the course";
#elif L==3
	"accesible s&oacute;lo para lectura por estudiantes y profesores de la asignatura";
#elif L==4
	"accessible uniquement pour la lecture par les &eacute;tudiants et les enseignants de la mati&egrave;re";
#elif L==5
	"accesible s&oacute;lo para lectura por estudiantes y profesores de la asignatura";	// Okoteve traduccin
#elif L==6
	"accessibile solo per la lettura da parte degli studenti e dei professori del corso";
#elif L==7
	"dostepne tylko do czytania przez uczni&oacute;w i nauczycieli zajec";
#elif L==8
	"acess&iacute;vel apenas para leitura pelos alunos e professores da disciplina";
#endif

const char *Txt_accessible_only_for_reading_by_students_of_the_group_and_teachers_of_the_course =
#if   L==0
	"accesible nom&eacute;s per a lectura per estudiants del grup i professors de l'assignatura";
#elif L==1
	"zum Lesen zug&auml;nglich f&uuml;r Studenten der Gruppe und Lehrkr&auml;fte der Kurs";
#elif L==2
	"accessible only for reading by students of the group and teachers of the course";
#elif L==3
	"accesible s&oacute;lo para lectura por estudiantes del grupo y profesores de la asignatura";
#elif L==4
	"accessible uniquement pour la lecture par les &eacute;tudiants du groupe et les enseignants de la mati&egrave;re";
#elif L==5
	"accesible s&oacute;lo para lectura por estudiantes del grupo y profesores de la asignatura";	// Okoteve traduccin
#elif L==6
	"accessibile solo per la lettura da parte degli studenti del gruppo e dei professori del corso";
#elif L==7
	"dostepne tylko do czytania przez uczni&oacute;w i nauczycieli grupy kursu";
#elif L==8
	"acess&iacute;vel apenas para leitura pelos alunos do grupo e professores da disciplina";
#endif

const char *Txt_Action =
#if   L==0
	"Acci&oacute;";
#elif L==1
	"Handlung";
#elif L==2
	"Action";
#elif L==3
	"Acci&oacute;n";
#elif L==4
	"Action";
#elif L==5
	"Tembiapo";
#elif L==6
	"Azione";
#elif L==7
	"Dzia&lstrok;anie";
#elif L==8
	"A&ccedil;&atilde;o";
#endif

const char *Txt_Admin_one_user =
#if   L==0
	"Administrar un usuari";
#elif L==1
	"Verwalten eines Benutzer";
#elif L==2
	"Admin one user";
#elif L==3
	"Administrar un usuario";
#elif L==4
	"G&eacute;rer un utilisateur";
#elif L==5
	"Administrar un usuario";	// Okoteve traduccin
#elif L==6
	"Gestire un utente";
#elif L==7
	"Zarz&aogon;dzaj u&zdot;ytkownika";
#elif L==8
	"Gerenciar um utilizador";
#endif

const char *Txt_Admin_user =
#if   L==0
	"Administrar usuari";
#elif L==1
	"Verwalten Benutzer";
#elif L==2
	"Admin user";
#elif L==3
	"Administrar usuario";
#elif L==4
	"G&eacute;rer utilisateur";
#elif L==5
	"Administrar usuario";	// Okoteve traduccin
#elif L==6
	"Gestire utente";
#elif L==7
	"Zarz&aogon;dzaj u&zdot;ytkownika";
#elif L==8
	"Gerenciar utilizador";
#endif

const char *Txt_All =
#if   L==0
	"Tot";
#elif L==1
	"Alle";
#elif L==2
	"All";
#elif L==3
	"Todo";
#elif L==4
	"Tout";
#elif L==5
	"Opakatu";
#elif L==6
	"Tutto";
#elif L==7
	"Wszystko";
#elif L==8
	"Tudo";
#endif

const char *Txt_all_degrees =
#if   L==0
	"Totes les titulacions";
#elif L==1
	"all degrees";			// Need bersetzung
#elif L==2
	"all degrees";
#elif L==3
	"todas las titulaciones";
#elif L==4
	"tous les degr&eacute;s";
#elif L==5
	"todas las titulaciones";	// Okoteve traduccin
#elif L==6
	"tutte le lauree";
#elif L==7
	"wszystkich stopni";
#elif L==8
	"all degrees";			// Necessita de traduo
#endif

const char *Txt_all_files =
#if   L==0
	"tots els fitxers";
#elif L==1
	"alle Dateien";
#elif L==2
	"all files";
#elif L==3
	"todos los archivos";
#elif L==4
	"tous les fichiers";
#elif L==5
	"todos los archivos";	// Okoteve traduccin
#elif L==6
	"tutti i file";
#elif L==7
	"wszystkich plik&oacute;w";
#elif L==8
	"todos os arquivos";
#endif

const char *Txt_All_groups =
#if   L==0
	"Tots els grups";
#elif L==1
	"Alle Gruppen";
#elif L==2
	"All groups";
#elif L==3
	"Todos los grupos";
#elif L==4
	"Tous les groupes";
#elif L==5
	"Todos los grupos";	// Okoteve traduccin
#elif L==6
	"Tutti i gruppi";
#elif L==7
	"Wszystkie grupy";
#elif L==8
	"Todos os grupos";
#endif

const char *Txt_All_notices =
#if   L==0
	"Tots els avisos";
#elif L==1
	"Alle Ank&uuml;ndigungen";
#elif L==2
	"All notices";
#elif L==3
	"Todos los avisos";
#elif L==4
	"Tous les avis";
#elif L==5
	"Todos los avisos";	// Okoteve traduccin
#elif L==6
	"Tutti i avvisi";
#elif L==7
	"Wszystkie ogloszenia";
#elif L==8
	"Todos os avisos";
#endif

const char *Txt_All_places =
#if   L==0
	"Tots els llocs";
#elif L==1
	"Alle Standorte";
#elif L==2
	"All places";
#elif L==3
	"Todos los lugares";
#elif L==4
	"Tous les emplacements";
#elif L==5
	"Todos los lugares";	// Okoteve traduccin
#elif L==6
	"Tutte le localit&agrave;";
#elif L==7
	"Wszystkie miejsca";
#elif L==8
	"Todas as localiza&ccedil;oes";
#endif

const char *Txt_All_tags =		// Tags used in test
#if   L==0
	"Tots els descriptors";
#elif L==1
	"Alle Tags";
#elif L==2
	"All tags";
#elif L==3
	"Todos los descriptores";
#elif L==4
	"Tous les descripteurs";
#elif L==5
	"Todos los descriptores";	// Okoteve traduccin
#elif L==6
	"Tutte le etichette";
#elif L==7
	"Wszystkie tagi";
#elif L==8
	"Todos os descritores";
#endif

const char *Txt_All_types_of_answers =		// Answers of a test
#if   L==0
	"Tots els tipus de respostes";
#elif L==1
	"Alle Antworttypen";
#elif L==2
	"All types of answers";
#elif L==3
	"Todos los tipos de respuestas";
#elif L==4
	"Tous les types de r&eacute;ponses";
#elif L==5
	"Todos los tipos de respuestas";	// Okoteve traduccin
#elif L==6
	"Tutti i tipi di risposte";
#elif L==7
	"Wszystkie typy odpowiedzi";
#elif L==8
	"Todos os tipos de respostas";
#endif

const char *Txt_Allow_teachers_to_consult_this_exam =
#if   L==0
	"Permetre als professors consultar aquest examen";
#elif L==1
	"Erm&ouml;glichen es den Lehrern, diese Pr&uuml;fung zu konsultieren";
#elif L==2
	"Allow teachers to consult this exam";
#elif L==3
	"Permitir a los profesores consultar este examen";
#elif L==4
	"Permitir a los profesores consultar este examen";
#elif L==5
	"Permitir a los profesores consultar este examen";
#elif L==6
	"Lasciare gli insegnanti di consultare questo esame";
#elif L==7
	"Pozwalaj&aogon; nauczycielom skonsultowa&cacute; ten egzamin";
#elif L==8
	"Permitir que os professores de consultar este exame";
#endif

const char *Txt_Alphabetic_BR_code_BR_ISO_3166_1 =
#if   L==0
	"Codi<br />alfab&egrave;tic<br />ISO 3166-1";
#elif L==1
	"Alphabetischer<br />Code<br />ISO 3166-1";
#elif L==2
	"Alphabetic<br />code<br />ISO 3166-1";
#elif L==3
	"C&oacute;d.<br />alfab&eacute;tico<br />ISO 3166-1";
#elif L==4
	"Code<br />alphab&eacute;tique<br />ISO 3166-1";
#elif L==5
	"C&oacute;d.<br />alfab&eacute;tico<br />ISO 3166-1";	// Okoteve traduccin
#elif L==6
	"Codice<br />alfabetico<br />ISO 3166-1";
#elif L==7
	"Alfabetycznego<br />kodu<br />ISO 3166-1";
#elif L==8
	"C&oacute;d.<br />alfab&eacute;tico<br />ISO 3166-1";
#endif

const char *Txt_Already_existed_an_assignment_with_the_folder_X =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"Ja existia una activitat amb la carpeta <strong>%s</strong>.";
#elif L==1
	"Es gibt bereits einen Aufgabe im Verzeichnis <strong>%s</strong>.";
#elif L==2
	"There is already an assignment with folder <strong>%s</strong>.";
#elif L==3
	"Ya exist&iacute;a una actividad con la carpeta <strong>%s</strong>.";
#elif L==4
	"Il existe d&eacute;j&agrave; une activit&eacute; avec le r&eacute;pertoire <strong>%s</strong>.";
#elif L==5
	"Ya exist&iacute;a una actividad con la carpeta <strong>%s</strong>.";	// Okoteve traduccin
#elif L==6
	"Esiste gi&agrave; un'attivit&agrave; con la cartella <strong>%s</strong>.";
#elif L==7
	"Jest juz zadanie z folderu <strong>%s</strong>.";
#elif L==8
	"J&aacute; existe uma atividade com o diret&oacute;rio <strong>%s</strong>.";
#endif

const char *Txt_Already_existed_an_assignment_with_the_title_X =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"Ja existia una activitat amb el t&iacute;tol <strong>%s</strong>.";
#elif L==1
	"Es gibt bereits einen Aufgabe mit dem Namen <strong>%s</strong>.";
#elif L==2
	"Already existed an assignment with the title <strong>%s</strong>.";
#elif L==3
	"Ya exist&iacute;a una actividad con el t&iacute;tulo <strong>%s</strong>.";
#elif L==4
	"Il existe d&eacute;j&agrave; une activit&eacute; du titre <strong>%s</strong>.";
#elif L==5
	"Ya exist&iacute;a una actividad con el t&iacute;tulo <strong>%s</strong>.";	// Okoteve traduccin
#elif L==6
	"Esiste gi&agrave; un'attivit&agrave; con il titolo <strong>%s</strong>.";
#elif L==7
	"Istniala juz przydzialu z tytulu <strong>%s</strong>.";
#elif L==8
	"J&aacute; existe uma atividade com o t&iacute;tulo <strong>%s</strong>.";
#endif

const char *Txt_Already_existed_an_event_with_the_title_X =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"Ja existia un esdeveniment amb el t&iacute;tol <strong>%s</strong>.";
#elif L==1
	"Es gibt bereits einen Ereignis mit dem Namen <strong>%s</strong>.";
#elif L==2
	"Already existed an event with the title <strong>%s</strong>.";
#elif L==3
	"Ya exist&iacute;a un evento con el t&iacute;tulo <strong>%s</strong>.";
#elif L==4
	"Il existe d&eacute;j&agrave; un &eacute;v&eacute;nement du titre <strong>%s</strong>.";
#elif L==5
	"Ya exist&iacute;a un evento con el t&iacute;tulo <strong>%s</strong>.";	// Okoteve traduccin
#elif L==6
	"Esiste gi&agrave; un evento con il titolo <strong>%s</strong>.";
#elif L==7
	"Istniala juz wydarzenie z tytulu <strong>%s</strong>.";
#elif L==8
	"J&aacute; existe um evento com o t&iacute;tulo <strong>%s</strong>.";
#endif

const char *Txt_Already_existed_a_survey_with_the_title_X =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"Ja existia una enquesta amb el t&iacute;tol <strong>%s</strong>.";
#elif L==1
	"Es gibt bereits einen Umfrage mit dem Namen <strong>%s</strong>.";
#elif L==2
	"Already existed a survey with the title <strong>%s</strong>.";
#elif L==3
	"Ya exist&iacute;a una encuesta con el t&iacute;tulo <strong>%s</strong>.";
#elif L==4
	"Il existe d&eacute;j&agrave; un sondage du titre <strong>%s</strong>.";
#elif L==5
	"Ya exist&iacute;a una encuesta con el t&iacute;tulo <strong>%s</strong>.";	// Okoteve traduccin
#elif L==6
	"Esiste gi&agrave; un sondaggio con il titolo <strong>%s</strong>.";
#elif L==7
	"Already existed a survey with the title <strong>%s</strong>.";	// Potrzebujesz tlumaczenie
#elif L==8
	"J&aacute; existe um inqu&eacute;rito com o t&iacute;tulo <strong>%s</strong>.";
#endif

const char *Txt_and =
#if   L==0
	"i";
#elif L==1
	"und";
#elif L==2
	"and";
#elif L==3
	"y";
#elif L==4
	"et";
#elif L==5
	"y";	// Okoteve traduccin
#elif L==6
	"e";
#elif L==7
	"i";
#elif L==8
	"e";
#endif

const char *Txt_and_X_other_recipients =	// Warning: it is very important to include %u in the following sentences
#if   L==0
	"...i altres %u destinataris";
#elif L==1
	"...und %u andere Empf&auml;nger";
#elif L==2
	"...and %u other recipients";
#elif L==3
	"...y otros %u destinatarios";
#elif L==4
	"...et %u autres destinataires";
#elif L==5
	"...y otros %u destinatarios";	// Okoteve traduccin
#elif L==6
	"...e altri %u destinatari";
#elif L==7
	"...i %u innych odbiorc&oacute;w";
#elif L==8
	"...e %u outros destinat&aacute;rios";
#endif

const char *Txt_Announcement_of_exam_removed =
#if   L==0
	"Convocatoria d'examen eliminada.";
#elif L==1
	"Aufruf zur Pr&uuml;fung entfernt.";
#elif L==2
	"Announcement of exam removed.";
#elif L==3
	"Convocatoria de examen eliminada.";
#elif L==4
	"Convocation pour l'examen supprim&eacute;.";
#elif L==5
	"Convocatoria de examen eliminada.";	// Okoteve traduccin
#elif L==6
	"Appello d'esame rimosso.";
#elif L==7
	"Ogloszenie egzamin usuniete.";
#elif L==8
	"Chamada para exame removida.";
#endif

const char *Txt_Another_centre =
#if   L==0
	"Un altre centre";
#elif L==1
	"Ein weiteres Lehrinstitut";
#elif L==2
	"Another centre";
#elif L==3
	"Otro centro";
#elif L==4
	"Un autre centre";
#elif L==5
	"Otro centro";	// Okoteve traduccin
#elif L==6
	"Un altro centro";
#elif L==7
	"Innym centrum";
#elif L==8
	"Outro centro";
#endif

const char *Txt_Another_country =
#if   L==0
	"Un altre pa&iacute;s";
#elif L==1
	"Ein weiterer Land";
#elif L==2
	"Another country";
#elif L==3
	"Otro pa&iacute;s";
#elif L==4
	"Un autre pays";
#elif L==5
	"Otro pa&iacute;s";	// Okoteve traduccin
#elif L==6
	"Un altro paese";
#elif L==7
	"Kolejny kraj";
#elif L==8
	"Outro pa&iacute;s";
#endif

const char *Txt_Another_department =
#if   L==0
	"Un altre departament";
#elif L==1
	"Eine weitere Abteilung";
#elif L==2
	"Another department";
#elif L==3
	"Otro departamento";
#elif L==4
	"Une autre d&eacute;partement";
#elif L==5
	"Otro departamento";	// Okoteve traduccin
#elif L==6
	"Un altro dipartimento";
#elif L==7
	"Kolejny dzia&lstrok;";
#elif L==8
	"Outro departamento";
#endif

const char *Txt_Another_ID =
#if   L==0
	"Un altre ID (DNI/c&egrave;dula)";
#elif L==1
	"Ein anderer Ausweis-Nr.";
#elif L==2
	"Another ID";
#elif L==3
	"Otro ID (DNI/c&eacute;dula)";
#elif L==4
	"Un autre num&eacute;ro d'identit&eacute;";
#elif L==5
	"Otro ID (DNI/c&eacute;dula)";		// Okoteve traduccin
#elif L==6
	"Un'altra carta d'identit&agrave;";
#elif L==7
	"Another ID";				// Potrzebujesz tlumaczenie
#elif L==8
	"Outro n&ordm; de identif";
#endif

const char *Txt_Add_this_ID =
#if   L==0
	"Afegir aquest ID (DNI/c&egrave;dula)";
#elif L==1
	"F&uuml;gen Sie diese Ausweis-Nr.";
#elif L==2
	"Add this ID";
#elif L==3
	"A&ntilde;adir este ID (DNI/c&eacute;dula)";
#elif L==4
	"Ajouter ce num&eacute;ro d'identit&eacute;";
#elif L==5
	"A&ntilde;adir este ID (DNI/c&eacute;dula)";	// Okoteve traduccin
#elif L==6
	"Aggiungi questa carta d'identit&agrave;";
#elif L==7
	"Add this ID";					// Potrzebujesz tlumaczenie
#elif L==8
	"Adicionar este n&ordm; de identif";
#endif

const char *Txt_Announcement_created =
#if   L==0
	"Anunci creat.";
#elif L==1
	"Bekanntmachung erstellt.";
#elif L==2
	"Announcement created.";
#elif L==3
	"Anuncio creado.";
#elif L==4
	"Annonce cr&eacute;&eacute;.";
#elif L==5
	"Anuncio creado.";		// Okoteve traduccin
#elif L==6
	"Annuncio creato.";
#elif L==7
	"Og&lstrok;oszenia stworzony.";
#elif L==8
	"An&uacute;ncio criado.";
#endif

const char *Txt_Announcement_removed =
#if   L==0
	"Anunci eliminat.";
#elif L==1
	"Bekanntmachung entfernt.";
#elif L==2
	"Announcement removed.";
#elif L==3
	"Anuncio eliminado.";
#elif L==4
	"Annonce supprim&eacute;.";
#elif L==5
	"Anuncio eliminado.";	// Okoteve traduccin
#elif L==6
	"Annuncio rimosso.";
#elif L==7
	"Og&lstrok;oszenia usuniete.";
#elif L==8
	"An&uacute;ncio removido.";
#endif

const char *Txt_Another_institution =
#if   L==0
	"Una altra instituci&oacute;";
#elif L==1
	"Eine weitere Hochschule";
#elif L==2
	"Another institution";
#elif L==3
	"Otra instituci&oacute;n";
#elif L==4
	"Un autre &eacute;tablissement";
#elif L==5
	"Otra instituci&oacute;n";	// Okoteve traduccin
#elif L==6
	"Un'altra istituzione";
#elif L==7
	"Kolejna instytucja";
#elif L==8
	"Outra institui&ccedil;&atilde;o";
#endif

const char *Txt_Another_place =
#if   L==0
	"Un altre lloc";
#elif L==1
	"Ein weiterer Standort";
#elif L==2
	"Another place";
#elif L==3
	"Otro lugar";
#elif L==4
	"Un autre emplacement";
#elif L==5
	"Otro lugar";	// Okoteve traduccin
#elif L==6
	"Un'altra localit&grave;";
#elif L==7
	"Innym miejscem";
#elif L==8
	"Outra localiza&ccedil;&atilde;o";
#endif

const char *Txt_Answer =
#if   L==0
	"Resposta";
#elif L==1
	"Answer";		// Need bersetzung
#elif L==2
	"Answer";
#elif L==3
	"Respuesta";
#elif L==4
	"Answer";		// Besoin de traduction
#elif L==5
	"Respuesta";		// Okoteve traduccin
#elif L==6
	"Risposta";
#elif L==7
	"Odpowiedz";
#elif L==8
	"Answer";		// Necessita de traduo
#endif

const char *Txt_Answer_survey =
#if   L==0
	"Respondre enquesta";
#elif L==1
	"Beantwortung Umfrage";
#elif L==2
	"Answer survey";
#elif L==3
	"Responder encuesta";
#elif L==4
	"R&eacute;pondre sondage";
#elif L==5
	"Responder encuesta";	// Okoteve traduccin
#elif L==6
	"Rispondere sondaggio";
#elif L==7
	"Badanie odpowiedzi";
#elif L==8
	"Responder inqu&eacute;rito";
#endif

const char *Txt_Answers =		// Answers of test
#if   L==0
	"Respostes";
#elif L==1
	"Antworten";
#elif L==2
	"Answers";
#elif L==3
	"Respuestas";
#elif L==4
	"R&eacute;ponses";
#elif L==5
	"&Ntilde;embohov&aacute;i";
#elif L==6
	"Risposte";
#elif L==7
	"Odpowiedzi";
#elif L==8
	"Respostas";
#endif

const char *Txt_any_course =
#if   L==0
	"qualsevol assignatura";
#elif L==1
	"alle Kursen";
#elif L==2
	"any course";
#elif L==3
	"cualquier asignatura";
#elif L==4
	"toute mati&egrave;re";
#elif L==5
	"cualquier asignatura";	// Okoteve traduccin
#elif L==6
	"qualsiasi corso";
#elif L==7
	"kurs";
#elif L==8
	"qualquer disciplina";
#endif

const char *Txt_Any_type_of_degree =
#if   L==0
	"Qualsevol tipus de titulaci&oacute;";
#elif L==1
	"Alle Abschlussart";
#elif L==2
	"Any type of degree";
#elif L==3
	"Cualquier tipo de titulaci&oacute;n";
#elif L==4
	"Tout type d'&eacute;tude";
#elif L==5
	"Cualquier tipo de titulaci&oacute;n";	// Okoteve traduccin
#elif L==6
	"Qualsiasi tipo di laurea";
#elif L==7
	"Any type of degree";			// Potrzebujesz tlumaczenie
#elif L==8
	"Qualquer tipo de titula&ccedil;&atilde;o";
#endif

const char *Txt_Any_department =
#if   L==0
	"Qualsevol departament";
#elif L==1
	"Alle Abteilung";
#elif L==2
	"Any department";
#elif L==3
	"Cualquier departamento";
#elif L==4
	"Tout d&eacute;partement";
#elif L==5
	"Cualquier departamento";	// Okoteve traduccin
#elif L==6
	"Qualsiasi dipartimento";
#elif L==7
	"Wszystkie dzia&lstrok;y";
#elif L==8
	"Qualquer departamento";
#endif

const char *Txt_Any_number =
#if   L==0
	"Qualsevol nombre";
#elif L==1
	"Alle Anzahl";
#elif L==2
	"Any number";
#elif L==3
	"Cualquier n&uacute;mero";
#elif L==4
	"Tout nombre";
#elif L==5
	"Cualquier n&uacute;mero";	// Okoteve traduccin
#elif L==6
	"Qualsiasi numero";
#elif L==7
	"Wszystkie liczba";
#elif L==8
	"Qualquer n&uacute;mero";
#endif

const char *Txt_any_user =
#if   L==0
	"qualsevol usuari";
#elif L==1
	"alle Benutzer";
#elif L==2
	"any user";
#elif L==3
	"cualquier usuario";
#elif L==4
	"n'importe qu'utilisateur";
#elif L==5
	"cualquier usuario";	// Okoteve traduccin
#elif L==6
	"qualsiasi utente";
#elif L==7
	"wszystkich uzytkownik&oacute;w";
#elif L==8
	"qualquer utilizador";
#endif

const char *Txt_Application_key =
#if   L==0
	"Clau de aplicaci&oacute;";
#elif L==1
	"Application key";		// Need bersetzung
#elif L==2
	"Application key";
#elif L==3
	"Clave de aplicaci&oacute;n";
#elif L==4
	"Cl&eacute; d'application";
#elif L==5
	"Clave de aplicaci&oacute;n";	// Okoteve traduccin
#elif L==6
	"Codice di applicazione";
#elif L==7
	"Klawisz aplikacji";
#elif L==8
	"Application key";		// Necessita de traduo
#endif

const char *Txt_Assessment_criteria =
#if   L==0
	"Criteris d'avaluaci&oacute;";
#elif L==1
	"Bewertungskriterien";
#elif L==2
	"Assessment criteria";
#elif L==3
	"Criterios de evaluaci&oacute;n";
#elif L==4
	"Crit&egrave;res d'&eacute;valuation";
#elif L==5
	"Criterios de evaluaci&oacute;n";	// Okoteve traduccin
#elif L==6
	"Criteri di valutazione";
#elif L==7
	"Kryteria oceny";
#elif L==8
	"Crit&eacute;rios de avalia&ccedil;&atilde;o";
#endif

const char *Txt_Assignment =
#if   L==0
	"Activitat";
#elif L==1
	"Aufgabe";
#elif L==2
	"Assignment";
#elif L==3
	"Actividad";
#elif L==4
	"Activit&eacute;";
#elif L==5
	"Tar&eacute;a";
#elif L==6
	"Attivit&agrave;";
#elif L==7
	"Przyporzadkowanie";
#elif L==8
	"Atividade";
#endif

const char *Txt_ASSIGNMENT_TYPES[Asg_NUM_TYPES_SEND_WORK] =
	 {
#if   L==0
	   "No cal enviar treballs (fitxers)"
#elif L==1
	   "Keine Hausarbeiten vorgesehen (Dateien)"
#elif L==2
	   "No homework to be submitted (files)"
#elif L==3
	   "No hay que enviar trabajos (archivos)"
#elif L==4
	   "Ne soumettez pas de travail (fichiers)"
#elif L==5
	   "No hay que enviar trabajos (archivos)"	// Okoteve traduccin
#elif L==6
	   "Non ci sono lavori da presentare (files)"
#elif L==7
	   "Nie ma pracy domowej, kt&oacute;re nalezy przedlozyc (pliki)"
#elif L==8
	   "N&atilde;o submeter trabalhos (arquivos)"
#endif
	   ,
#if   L==0
	   "Es obligatori enviar treballs (fitxers)"
#elif L==1
	   "Hausarbeiten vorgesehen (files)"
#elif L==2
	   "It is mandatory to submit homework (files)"
#elif L==3
	   "Es obligatorio enviar trabajos (archivos)"
#elif L==4
	   "Il est obligatoire de soumetter le travail (fichiers)"
#elif L==5
	   "Es obligatorio enviar trabajos (archivos)"// Okoteve traduccin
#elif L==6
	   "E' obbligatorio presentare il lavoro (files)"
#elif L==7
	   "Jest to obowiazkowe do przedstawienia prac domowych (pliki)"
#elif L==8
	   "&Eacute; obrigat&oacute;ria a apresenta&ccedil;&atilde;o de trabalhos (arquivos)"
#endif
	};

const char *Txt_Assignment_X_is_now_hidden =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"La activitat <strong>%s</strong> ara est&agrave; oculta.";
#elif L==1
	"Die Aufgabe <strong>%s</strong> ist jetzt ausgeblendet.";
#elif L==2
	"Assignment <strong>%s</strong> is now hidden.";
#elif L==3
	"La actividad <strong>%s</strong> ahora est&aacute; oculta.";
#elif L==4
	"L'activit&eacute; <strong>%s</strong> est maintenant cach&eacute;e.";
#elif L==5
	"La actividad <strong>%s</strong> ahora est&aacute; oculta.";	// Okoteve traduccin
#elif L==6
	"L'attivit&agrave; <strong>%s</strong> &egrave; ora nascosta.";
#elif L==7
	"Zadanie <strong>%s</strong> jest ukryte.";
#elif L==8
	"A atividade <strong>%s</strong> &eacute; agora oculta.";
#endif

const char *Txt_Assignment_X_is_now_visible =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"La activitat <strong>%s</strong> ara est&agrave; visible.";
#elif L==1
	"Die Aufgabe <strong>%s</strong> ist jetzt sichtbar.";
#elif L==2
	"Assignment <strong>%s</strong> is now visible.";
#elif L==3
	"La actividad <strong>%s</strong> ahora est&aacute; visible.";
#elif L==4
	"L'activit&eacute; <strong>%s</strong> est maintenant visible.";
#elif L==5
	"La actividad <strong>%s</strong> ahora est&aacute; visible.";	// Okoteve traduccin
#elif L==6
	"L'attivit&agrave; <strong>%s</strong> &egrave; ora visibile.";
#elif L==7
	"Zadanie <strong>%s</strong> jest teraz widoczny.";
#elif L==8
	"A atividade <strong>%s</strong> &eacute; agora vis&iacute;vel.";
#endif

const char *Txt_Assignment_X_removed =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"Activitat <strong>%s</strong> eliminada.";
#elif L==1
	"Aufgabe <strong>%s</strong> entfernt.";
#elif L==2
	"Assignment <strong>%s</strong> removed.";
#elif L==3
	"Actividad <strong>%s</strong> eliminada.";
#elif L==4
	"Activit&eacute; <strong>%s</strong> supprim&eacute;e.";
#elif L==5
	"Actividad <strong>%s</strong> eliminada.";	// Okoteve traduccin
#elif L==6
	"Attivit&agrave; <strong>%s</strong> rimossa.";
#elif L==7
	"Zadanie <strong>%s</strong> usuniete.";
#elif L==8
	"Atividade <strong>%s</strong> removida.";
#endif

const char *Txt_Assignments =
#if   L==0
	"Activitats";
#elif L==1
	"Aufgaben";
#elif L==2
	"Assignments";
#elif L==3
	"Actividades";
#elif L==4
	"Activit&eacute;s";
#elif L==5
	"Tar&eacute;a";
#elif L==6
	"Attivit&agrave;";
#elif L==7
	"Zadania";
#elif L==8
	"Atividades";
#endif

const char *Txt_ASG_ATT_OR_SVY_HELP_ORDER[2] =
   {
#if   L==0
	 "Ordenar per data de comen&ccedil;ament"
#elif L==1
	 "Sortieren nach Startdatum"
#elif L==2
	 "Sort by start date"
#elif L==3
	 "Ordenar por fecha de comienzo"
#elif L==4
	 "Trier par date de d&eacute;but"
#elif L==5
	 "Ordenar por fecha de comienzo"	// Okoteve traduccin
#elif L==6
	 "Ordina per data di inizio"
#elif L==7
	 "Sortuj wedlug daty rozpoczecia"
#elif L==8
	 "Classificar por data de in&iacute;cio"
#endif
	 ,
#if   L==0
	 "Ordenar per data de finalitzaci&oacute;"
#elif L==1
	 "Sortieren nach Enddatum"
#elif L==2
	 "Sort by end date"
#elif L==3
	 "Ordenar por fecha de finalizaci&oacute;n"
#elif L==4
	 "Trier par date de fin"
#elif L==5
	 "Ordenar por fecha de finalizaci&oacute;n"	// Okoteve traduccin
#elif L==6
	 "Ordina per data di fine"
#elif L==7
	 "Sortuj wedlug daty zakonczenia"
#elif L==8
	 "Classificar por data de t&eacute;rmino"
#endif
	};

const char *Txt_ASG_ATT_OR_SVY_ORDER[2] =
   {
#if   L==0
	 "Data<br />inicial"
#elif L==1
	 "Startdatum"
#elif L==2
	 "Start<br />date"
#elif L==3
	 "Fecha<br />inicial"
#elif L==4
	 "Date de<br />d&eacute;but"
#elif L==5
	 "Fecha<br />inicial"	// Okoteve traduccin
#elif L==6
	 "Data di<br />inizio"
#elif L==7
	 "Start<br />data"
#elif L==8
	 "Data de<br />in&iacute;cio"
#endif
	 ,
#if   L==0
	 "Data<br />final"
#elif L==1
	 "Enddatum"
#elif L==2
	 "End<br />date"
#elif L==3
	 "Fecha<br />final"
#elif L==4
	 "Date de<br />fin"
#elif L==5
	 "Fecha<br />final"	// Okoteve traduccin
#elif L==6
	 "Data di<br />fine"
#elif L==7
	 "Koniec<br />data"
#elif L==8
	 "Data de<br />t&eacute;rmino"
#endif
	};

const char *Txt_Assignments_zone =
#if   L==0
	"Zona d'activitats";
#elif L==1
	"Aufgaben-Bereich";
#elif L==2
	"Assignments area";
#elif L==3
	"Zona de actividades";
#elif L==4
	"Zone d'activit&eacute;s";
#elif L==5
	"Zona de actividades";	// Okoteve traduccin
#elif L==6
	"Area di attivit&agrave;";
#elif L==7
	"Obszarze Zadania";
#elif L==8
	"Zona de atividades";
#endif

const char *Txt_assignments_zone =
#if   L==0
	"zona d'activitats";	// Necessita traduccio
#elif L==1
	"Aufgaben-Bereich";
#elif L==2
	"assignments area";
#elif L==3
	"zona de actividades";
#elif L==4
	"zone d'activit&eacute;s";
#elif L==5
	"zona de actividades";	// Okoteve traduccin
#elif L==6
	"area di attivit&agrave;";
#elif L==7
	"obszarze zadan";
#elif L==8
	"zona de atividades";
#endif

const char *Txt_Attendance =	// Number of times a students has attended to face-to-face class
#if   L==0
	"Assist&egrave;ncia";
#elif L==1
	"Anwesenheit";
#elif L==2
	"Attendance";
#elif L==3
	"Asistencia";
#elif L==4
	"Pr&eacute;sence";
#elif L==5
	"Asistencia";	// Okoteve traduccin
#elif L==6
	"Presenza";
#elif L==7
	"Bytno&sacute;&cacute;";
#elif L==8
	"Presen&ccedil;a";
#endif

const char *Txt_Attendance_list =
#if   L==0
	"Llista assistents";
#elif L==1
	"Anwesenheitsliste";
#elif L==2
	"Attendance list";
#elif L==3
	"Lista asistentes";
#elif L==4
	"Liste participants";
#elif L==5
	"Lista asistentes";	// Okoteve traduccin
#elif L==6
	"Elenco presenze";
#elif L==7
	"Lista obecno&sacute;ci";
#elif L==8
	"Lista presen&ccedil;as";
#endif

const char *Txt_Availability =
#if   L==0
	"Disponibilitat";
#elif L==1
	"Verf&uuml;gbarkeit";
#elif L==2
	"Availability";
#elif L==3
	"Disponibilidad";
#elif L==4
	"Disponibilit&eacute;";
#elif L==5
	"Disponibilidad";	// Okoteve traduccin
#elif L==6
	"Disponibilit&agrave;";
#elif L==7
	"Dost&eogon;pno&sacute;&cacute;";
#elif L==8
	"Disponibilidade";
#endif

const char *Txt_Average_number_BR_of_assignments_BR_per_course =
#if   L==0
	"Nombre mitj&acute;<br />d'activitats<br />per assignatura";
#elif L==1
	"Durchschnittliche Anzahl<br />von Aufgaben<br />pro Kurs";
#elif L==2
	"Average number<br />of assignments<br />per course";
#elif L==3
	"N&ordm; medio<br />de actividades<br />por asignatura";
#elif L==4
	"Nombre moyen<br />d'activit&eacute;s<br />par mati&egrave;re";
#elif L==5
	"N&ordm; medio<br />de actividades<br />por asignatura";	// Okoteve traduccin
#elif L==6
	"Numero medio<br />di attivit&agrave;<br />per corso";
#elif L==7
	"&Sacute;rednia liczba<br />zadania<br />na kurs";
#elif L==8
	"N&ordm; medio<br />de atividades<br />por disciplina";
#endif

const char *Txt_Average_number_BR_of_questions_BR_per_survey =
#if   L==0
	"Nombre mitj&acute;<br />de preguntes<br />per enquesta";
#elif L==1
	"Durchschnittliche Anzahl<br />von Fragen<br />pro Umfrage";
#elif L==2
	"Average number<br />of questions<br />per survey";
#elif L==3
	"N&ordm; medio<br />de preguntas<br />por encuesta";
#elif L==4
	"Nombre moyen<br />de questions<br />par sondage";
#elif L==5
	"N&ordm; medio<br />de preguntas<br />por encuesta";	// Okoteve traduccin
#elif L==6
	"Numero medio<br />di domande<br />per sondaggio";
#elif L==7
	"&Sacute;rednia liczba<br />pyta&nacute;<br />na ankiety";
#elif L==8
	"N&ordm; medio<br />de quest&otilde;es<br />por inqu&eacute;rito";
#endif

const char *Txt_Average_BR_number_BR_of_test_BR_questions_BR_per_course =
#if   L==0
	"Nombre mitj&acute;<br />de preguntes<br />de test per<br />assignatura";
#elif L==1
	"Durchschnittliche Anzahl<br />von Testfragen<br />pro Kurs";
#elif L==2
	"Average<br />number<br />of test<br />questions<br />per course";
#elif L==3
	"N&ordm; medio<br />de preguntas<br />de test por<br />asignatura";
#elif L==4
	"Nombre moyen<br />de questions<br />de test par<br />mati&egrave;re";
#elif L==5
	"N&ordm; medio<br />de preguntas<br />de test por<br />asignatura";	// Okoteve traduccin
#elif L==6
	"Numero medio<br />di domande<br />di test<br />per corso";
#elif L==7
	"&Sacute;rednia liczba<br />pyta&nacute;<br />testowych<br />na kurs";
#elif L==8
	"N&ordm; medio<br />de quest&otilde;es<br />de test por<br />disciplina";
#endif

const char *Txt_Average_number_BR_of_surveys_BR_per_course =
#if   L==0
	"Nombre mitj&acute;<br />d'enquestes<br />per assignatura";
#elif L==1
	"Durchschnittliche Anzahl<br />von Umfragen<br />pro Kurs";
#elif L==2
	"Average number<br />of surveys<br />per course";
#elif L==3
	"N&ordm; medio<br />de encuestas<br />por asignatura";
#elif L==4
	"Nombre moyen<br />de sondages<br />par mati&egrave;re";
#elif L==5
	"N&ordm; medio<br />de encuestas<br />por asignatura";	// Okoteve traduccin
#elif L==6
	"Numero medio<br />di sondaggi<br />per corso";
#elif L==7
	"&Sacute;rednia liczba<br />badania<br />na kurs";
#elif L==8
	"N&ordm; medio<br />de inqu&eacute;ritos<br />por disciplina";
#endif

const char *Txt_Average_number_of_courses_to_which_a_user_belongs =
#if   L==0
	"Nombre mitj&acute; d'assignatures a las quals pertany cada usuari";
#elif L==1
	"Durchschnittliche Anzahl von Kursen, an denen der Benutzer teilnimmt";
#elif L==2
	"Average number of courses to which a user belongs";
#elif L==3
	"N&ordm; medio de asignaturas a las que pertenece cada usuario";
#elif L==4
	"Nombre moyen de cours &agrave; qui appartient un utilisateur";
#elif L==5
	"N&ordm; medio de asignaturas a las que pertenece cada usuario";	// Okoteve traduccin
#elif L==6
	"Numero medio di corsi a cui appartiene un utente";
#elif L==7
	"&Sacute;rednia liczba kurs&oacute;w do kt&oacute;rej nalezy uzytkownik";
#elif L==8
	"N&ordm; medio de disciplinas em que cada utilizador pertence";
#endif

const char *Txt_Average_BR_number_of_BR_times_that_BR_a_question_BR_has_been_BR_responded =
#if   L==0
	"N&ordm; medio<br />de veces<br />que se ha<br />respondido<br />cada pregunta";	// Necessita traduccio
#elif L==1
	"Average<br />number of<br />times that<br />a question<br />has been<br />responded";	// Need bersetzung
#elif L==2
	"Average<br />number of<br />times that<br />a question<br />has been<br />responded";
#elif L==3
	"N&ordm; medio<br />de veces<br />que se ha<br />respondido<br />cada pregunta";
#elif L==4
	"Average<br />number of<br />times that<br />a question<br />has been<br />responded";	// Besoin de traduction
#elif L==5
	"N&ordm; medio<br />de veces<br />que se ha<br />respondido<br />cada pregunta";	// Okoteve traduccin
#elif L==6
	"Average<br />number of<br />times that<br />a question<br />has been<br />responded";	// Bisogno di traduzione
#elif L==7
	"Average<br />number of<br />times that<br />a question<br />has been<br />responded";	// Potrzebujesz tlumaczenie
#elif L==8
	"Average<br />number of<br />times that<br />a question<br />has been<br />responded";	// Necessita de traduo
#endif

const char *Txt_Average_BR_number_of_BR_times_that_BR_questions_BR_have_been_BR_responded_BR_per_course =
#if   L==0
	"N&ordm; medio<br />de veces<br />que se ha<br />respondido<br />por asignatura";	// Necessita traduccio
#elif L==1
	"Average<br />number of<br />times that<br />questions<br />have been<br />responded<br />per course";	// Need bersetzung
#elif L==2
	"Average<br />number of<br />times that<br />questions<br />have been<br />responded<br />per course";
#elif L==3
	"N&ordm; medio<br />de veces<br />que se ha<br />respondido<br />por asignatura";
#elif L==4
	"Average<br />number of<br />times that<br />questions<br />have been<br />responded<br />per course";	// Besoin de traduction
#elif L==5
	"N&ordm; medio<br />de veces<br />que se ha<br />respondido<br />por asignatura";			// Okoteve traduccin
#elif L==6
	"Average<br />number of<br />times that<br />questions<br />have been<br />responded<br />per course";	// Bisogno di traduzione
#elif L==7
	"Average<br />number of<br />times that<br />questions<br />have been<br />responded<br />per course";	// Potrzebujesz tlumaczenie
#elif L==8
	"Average<br />number of<br />times that<br />questions<br />have been<br />responded<br />per course";	// Necessita de traduo
#endif

const char *Txt_Average_number_of_users_belonging_to_a_course =
#if   L==0
	"Nombre mitj&agrave; de usuaris que pertanyen a cada assignatura";
#elif L==1
	"Durchschnittliche Anzahl der Benutzer, die an einer Kurs teilnehmen";
#elif L==2
	"Average number of users belonging to a course";
#elif L==3
	"N&ordm; medio de usuarios que pertenecen a cada asignatura";
#elif L==4
	"Nombre moyen d'utilisateurs appartenant &agrave; une mati&egrave;re";
#elif L==5
	"N&ordm; medio de usuarios que pertenecen a cada asignatura";	// Okoteve traduccin
#elif L==6
	"Numero medio di utenti appartenenti a un corso";
#elif L==7
	"&Sacute;rednia liczba uzytkownik&oacute;w nalezacych do kursu";
#elif L==8
	"N&ordm; medio de utilizadores que pertencem a uma disciplina";
#endif

const char *Txt_AVERAGE_PHOTO_TYPES[Pho_NUM_AVERAGE_PHOTO_TYPES] =
   {
#if   L==0
	 "mitjana de totes les fotos"
#elif L==1
	 "median of all the photos"			// Need bersetzung
#elif L==2
	 "median of all the photos"
#elif L==3
	 "mediana de todas las fotos"
#elif L==4
	 "m&eacute;dian de toutes les photos"
#elif L==5
	 "mediana de todas las fotos"			// Okoteve traduccin
#elif L==6
	 "mediana di tutte le foto"
#elif L==7
	 "mediana wszystkich zdjec"
#elif L==8
	 "median of all the photos"			// Necessita de traduo
#endif
	 ,
#if   L==0
	 "mitjana aritm&egrave;tica de totes les fotos"
#elif L==1
	 "arithmetic mean of all the photos"		// Need bersetzung
#elif L==2
	 "arithmetic mean of all the photos"
#elif L==3
	 "media aritm&eacute;tica de todas las fotos"
#elif L==4
	 "moyenne arithm&eacute;tique de toutes les photos"
#elif L==5
	 "media aritm&eacute;tica de todas las fotos"	// Okoteve traduccin
#elif L==6
	 "media aritmetica di tutte le foto"
#elif L==7
	 "&sacute;rednia arytmetyczna wszystkich zdjec"
#elif L==8
	 "arithmetic mean of all the photos"		// Necessita de traduo
#endif
	};

const char *Txt_Average_BR_score_BR_per_question_BR_from_0_to_1 =
#if   L==0
	"Puntuaci&oacute;n<br />media<br />por pregunta<br />(de 0 a 1)";	// Necessita traduccio
#elif L==1
	"Average<br />score<br />per question<br />(from 0 to 1)";		// Need bersetzung
#elif L==2
	"Average<br />score<br />per question<br />(from 0 to 1)";
#elif L==3
	"Puntuaci&oacute;n<br />media<br />por pregunta<br />(de 0 a 1)";
#elif L==4
	"Average<br />score<br />per question<br />(from 0 to 1)";		// Besoin de traduction
#elif L==5
	"Puntuaci&oacute;n<br />media<br />por pregunta<br />(de 0 a 1)";	// Okoteve traduccin
#elif L==6
	"Average<br />score<br />per question<br />(from 0 to 1)";		// Bisogno di traduzione
#elif L==7
	"Average<br />score<br />per question<br />(from 0 to 1)";		// Potrzebujesz tlumaczenie
#elif L==8
	"Average<br />score<br />per question<br />(from 0 to 1)";		// Necessita de traduo
#endif

const char *Txt_Average_type =
#if   L==0
	"Tipus de mitjana";
#elif L==1
	"Art des Mittelwerts";
#elif L==2
	"Average type";
#elif L==3
	"Tipo de promedio";
#elif L==4
	"Type de moyenne";
#elif L==5
	"Tipo de promedio";	// Okoteve traduccin
#elif L==6
	"Tipo di media";
#elif L==7
	"&Sacute;rednia typu";
#elif L==8
	"Tipo de m&eacute;dia";
#endif

const char *Txt_Banned_users =
#if   L==0
	"Usuaris bloquejats";
#elif L==1
	"Gebannt Benutzer";
#elif L==2
	"Banned users";
#elif L==3
	"Usuarios bloqueados";
#elif L==4
	"Utilisateurs bannis";
#elif L==5
	"Usuarios bloqueados";	// Okoteve traduccin
#elif L==6
	"Utenti vietati";
#elif L==7
	"Zablokowani u&zdot;ytkownicy";
#elif L==8
	"Usu&aacute;rios banidos";
#endif

const char *Txt_Banner =
#if   L==0
	"Banner";
#elif L==1
	"Werbebanner";
#elif L==2
	"Banner";
#elif L==3
	"Banner";
#elif L==4
	"Banni&eacute;re";
#elif L==5
	"Banner";
#elif L==6
	"Banner";
#elif L==7
	"Banner";
#elif L==8
	"Banner";
#endif

const char *Txt_Banner_X_removed =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"Banner <strong>%s</strong> eliminado.";	// Necessita traduccio
#elif L==1
	"Werbebanner <strong>%s</strong> entfernt.";
#elif L==2
	"Banner <strong>%s</strong> removed.";
#elif L==3
	"Banner <strong>%s</strong> eliminado.";
#elif L==4
	"Banni&eacute;re <strong>%s</strong> supprim&eacute;e.";
#elif L==5
	"Banner <strong>%s</strong> eliminado.";	// Okoteve traduccin
#elif L==6
	"Banner <strong>%s</strong> rimosso.";
#elif L==7
	"Banner <strong>%s</strong> usuniety.";
#elif L==8
	"Banner <strong>%s</strong> removed.";		// Necessita de traduo
#endif

const char *Txt_Banners =
#if   L==0
	"Banners";
#elif L==1
	"Werbebanners";
#elif L==2
	"Banners";
#elif L==3
	"Banners";
#elif L==4
	"Banni&eacute;res";
#elif L==5
	"Banners";
#elif L==6
	"Banners";
#elif L==7
	"Banners";
#elif L==8
	"Banners";
#endif

const char *Txt_Before_going_to_any_other_option_you_must_create_your_password =
#if   L==0
	"Abans d'accedir a qualsevol altra opci&oacute;"
	" ha de crear la seva contrasenya.";
#elif L==1
	"Bevor ich auf jede andere Option"
	" m&uuml;ssen Sie Ihr Passwort erstellen.";
#elif L==2
	"Before going to any other option"
	" you must create your password.";
#elif L==3
	"Antes de acceder a cualquier otra opci&oacute;n"
	" debe crear su contrase&ntilde;a.";
#elif L==4
	"Avant d'aller &agrave; une autre option"
	" vous devez cr&eacute;er votre mot de passe.";
#elif L==5
	"Antes de acceder a cualquier otra opci&oacute;n"
	" debe crear su contrase&ntilde;a.";		// Okoteve traduccin
#elif L==6
	"Prima di scegliere qualsiasi altra opzione"
	" devi creare una password.";
#elif L==7
	"Przed p&oacute;jsciem do jakiejkolwiek"
	" innej opcji nalezy utworzy&cacute; has&lstrok;o.";
#elif L==8
	"Antes de ir para qualquer outra op&ccedil;&atilde;o"
	"voc&ecirc; deve criar sua senha.";
#endif

const char *Txt_Before_going_to_any_other_option_you_must_fill_your_centre_and_department =
#if   L==0
	"Abans d'accedir a qualsevol altra opci&oacute;"
	" ha de omplir el seu centre i department.";
#elif L==1
	"Bevor ich auf jede andere Option"
	" m&uuml;ssen Sie Ihre Zentrum und Abteilung zu f&uuml;llen.";
#elif L==2
	"Before going to any other option"
	" you must fill your centre and department.";
#elif L==3
	"Antes de acceder a cualquier otra opci&oacute;n"
	" debe rellenar su centro y departamento.";
#elif L==4
	"Avant d'aller &agrave; une autre option"
	" vous devez remplir votre centre et d&eacute;partement.";
#elif L==5
	"Antes de acceder a cualquier otra opci&oacute;n"
	" debe rellenar su centro y departamento.";	// Okoteve traduccin
#elif L==6
	"Prima di scegliere qualsiasi altra opzione"
	" devi compilare il tuo centro e dipartimento.";
#elif L==7
	"Zanim przejdziemy do innych opcji"
	" nalezy wypelnic tym centrum i dzia&lstrok;u.";
#elif L==8
	"Antes de ir para qualquer outra op&ccedil;&atilde;o"
	" voc&ecirc; deve preencher o seu centro e departamento.";
#endif

const char *Txt_Before_going_to_any_other_option_you_must_fill_your_record_card_including_your_country_nationality =
#if   L==0
	"Abans d'accedir a qualsevol altra opci&oacute;"
	" ha de omplir la seva fitxa,"
	" incloent el seu pais (nacionalitat).";
#elif L==1
	"Bevor ich auf jede andere Option"
	" m&uuml;ssen Sie Ihre Karteikarte zu f&uuml;llen,"
	" einschlie&szlig;lich Ihrem Land (Nationalit&auml;t).";
#elif L==2
	"Before going to any other option"
	" you must fill your record card,"
	" including your country (nationality).";
#elif L==3
	"Antes de acceder a cualquier otra opci&oacute;n"
	" debe rellenar su ficha,"
	" incluyendo su pa&iacute;s (nacionalidad).";
#elif L==4
	"Avant d'aller &agrave; une autre option"
	" vous devez remplir votre carte d'enregistrement,"
	" y compris votre pays (nationalit&eacute;).";
#elif L==5
	"Antes de acceder a cualquier otra opci&oacute;n"
	" debe rellenar su ficha,"
	" incluyendo su pa&iacute;s (nacionalidad).";	// Okoteve traduccin
#elif L==6
	"Prima di scegliere qualsiasi altra opzione"
	" devi compilare la tua scheda,"
	" compreso il tuo paese (nazionalit&agrave;).";
#elif L==7
	"Zanim przejdziemy do innych opcji"
	" nalezy wypelnic karte zapisu,"
	" w tym kraju (obywatelstwo";
#elif L==8
	"Antes de ir para qualquer outra op&ccedil;&atilde;o"
	" voc&ecirc; deve preencher o seu cart&atilde;o,"
	" incluindo seu pa&iacute;s (nacionalidade).";
#endif

const char *Txt_Before_going_to_any_other_option_you_must_fill_your_email_address =
#if   L==0
	"Abans d'accedir a qualsevol altra opci&oacute;"
	" ha de omplir la seva adre&ccedil;a de correu electr&ograve;nic.";
#elif L==1
	"Bevor ich auf jede andere Option"
	" m&uuml;ssen Sie Ihre E-Mail-Adresse ein.";
#elif L==2
	"Before going to any other option"
	" you must fill your email address.";
#elif L==3
	"Antes de acceder a cualquier otra opci&oacute;n"
	" debe rellenar su direcci&oacute;n de correo electr&oacute;nico.";
#elif L==4
	"Avant d'aller &agrave; une autre option"
	" vous devez remplir votre adresse e-mail.";
#elif L==5
	"Antes de acceder a cualquier otra opci&oacute;n"
	" debe rellenar su direcci&oacute;n de correo electr&oacute;nico.";	// Okoteve traduccin
#elif L==6
	"Prima di scegliere qualsiasi altra opzione"
	" devi compilare il tuo indirizzo e-mail.";
#elif L==7
	"Zanim przejdziemy do innych opcji"
	" nalezy w tym adresu e-mail.";
#elif L==8
	"Antes de ir para qualquer outra op&ccedil;&atilde;o"
	" voc&ecirc; deve preencher o seu endere&ccedil;o de e-mail.";
#endif

const char *Txt_Before_going_to_any_other_option_you_must_fill_your_ID =
#if   L==0
	"Abans d'accedir a qualsevol altra opci&oacute;"
	" ha de omplir el seu ID (DNI/c&egrave;dula).";
#elif L==1
	"Bevor ich auf jede andere Option"
	" m&uuml;ssen Sie Ihren Ausweis-Nr. &auml;nder.";
#elif L==2
	"Before going to any other option"
	" you must fill your ID.";
#elif L==3
	"Antes de acceder a cualquier otra opci&oacute;n"
	" debe rellenar su ID (DNI/c&eacute;dula).";
#elif L==4
	"Avant d'aller &agrave; une autre option"
	" vous devez remplir votre ID.";
#elif L==5
	"Antes de acceder a cualquier otra opci&oacute;n"
	" debe rellenar su ID (DNI/c&eacute;dula).";	// Okoteve traduccin
#elif L==6
	"Prima di scegliere qualsiasi altra opzione"
	" devi compilare il tuo ID.";
#elif L==7
	"Zanim przejdziemy do innych opcji"
	" nalezy w tym nick swoje zmie&nacute; identyfikatory.";
#elif L==8
	"Antes de ir para qualquer outra op&ccedil;&atilde;o"
	" voc&ecirc; deve preencher o seu n&ordm;. identif.";
#endif

const char *Txt_Before_going_to_any_other_option_you_must_fill_your_nickname =
#if   L==0
	"Abans d'accedir a qualsevol altra opci&oacute;"
	" ha de omplir el seu sobrenom.";
#elif L==1
	"Bevor ich auf jede andere Option"
	" m&uuml;ssen Sie Ihren Namen zu f&uuml;llen.";
#elif L==2
	"Before going to any other option"
	" you must fill your nickname.";
#elif L==3
	"Antes de acceder a cualquier otra opci&oacute;n"
	" debe rellenar su apodo.";
#elif L==4
	"Avant d'aller &agrave; une autre option"
	" vous devez remplir votre surnom.";
#elif L==5
	"Antes de acceder a cualquier otra opci&oacute;n"
	" debe rellenar su apodo.";			// Okoteve traduccin
#elif L==6
	"Prima di scegliere qualsiasi altra opzione"
	" devi compilare il tuo soprannome.";
#elif L==7
	"Zanim przejdziemy do innych opcji"
	" nalezy w tym nick.";
#elif L==8
	"Antes de ir para qualquer outra op&ccedil;&atilde;o"
	" voc&ecirc; deve preencher a seu alcunha.";
#endif

const char *Txt_Before_going_to_any_other_option_you_must_fill_your_institution =
#if   L==0
	"Abans d'accedir a qualsevol altra opci&oacute;"
	" ha de omplir la seva instituci&oacute;.";
#elif L==1
	"Bevor ich auf jede andere Option"
	" m&uuml;ssen Sie Ihre Einrichtung zu f&uuml;llen.";
#elif L==2
	"Before going to any other option"
	" you must fill your institution.";
#elif L==3
	"Antes de acceder a cualquier otra opci&oacute;n"
	" debe rellenar su instituci&oacute;n.";
#elif L==4
	"Avant d'aller &agrave; une autre option"
	" vous devez remplir votre &eacute;tablissement.";
#elif L==5
	"Antes de acceder a cualquier otra opci&oacute;n"
	" debe rellenar su instituci&oacute;n.";	// Okoteve traduccin
#elif L==6
	"Prima di scegliere qualsiasi altra opzione"
	" devi compilare la tua istituzione accademica.";
#elif L==7
	"Zanim przejdziemy do innych opcji"
	" nalezy wypelnic tym instytucji.";
#elif L==8
	"Antes de ir para qualquer outra op&ccedil;&atilde;o"
	" voc&ecirc; deve preencher sua institui&ccedil;&atilde;o.";
#endif

const char *Txt_Before_going_to_any_other_option_you_must_fill_your_record_card_including_your_name =
#if   L==0
	"Abans d'accedir a qualsevol altra opci&oacute;"
	" ha de omplir la seva fitxa,"
	" incloent el seu nom i cognom(s).";
#elif L==1
	"Bevor ich auf jede andere Option"
	" m&uuml;ssen Sie Ihre Karteikarte zu f&uuml;llen,"
	" mit Ihrem Namen.";
#elif L==2
	"Before going to any other option"
	" you must fill your record card,"
	" including your name.";
#elif L==3
	"Antes de acceder a cualquier otra opci&oacute;n"
	" debe rellenar su ficha,"
	" incluyendo su nombre y apellido(s).";
#elif L==4
	"Avant d'aller &agrave; une autre option"
	" vous devez remplir votre carte d'enregistrement,"
	" y compris votre nom.";
#elif L==5
	"Antes de acceder a cualquier otra opci&oacute;n"
	" debe rellenar su ficha,"
	" incluyendo su nombre y apellido(s).";		// Okoteve traduccin
#elif L==6
	"Prima di scegliere qualsiasi altra opzione"
	" devi compilare la tua scheda,"
	" compresi il tuo nome e cognome.";
#elif L==7
	"Zanim przejdziemy do innych opcji"
	" nalezy wypelnic karte zapisu,"
	" w tym imie i nazwisko.";
#elif L==8
	"Antes de ir para qualquer outra op&ccedil;&atilde;o"
	" voc&ecirc; deve preencher o cart&atilde;o,"
	" incluindo seu nome.";
#endif

const char *Txt_Before_going_to_any_other_option_you_must_fill_your_record_card_including_your_sex =
#if   L==0
	"Abans d'accedir a qualsevol altra opci&oacute;"
	" ha de omplir la seva fitxa,"
	" incloent el sexe.";
#elif L==1
	"Bevor ich auf jede andere Option"
	" m&uuml;ssen Sie Ihre Karteikarte zu f&uuml;llen,"
	" einschlielich Ihrer Sex.";
#elif L==2
	"Before going to any other option"
	" you must fill your record card,"
	" including your sex.";
#elif L==3
	"Antes de acceder a cualquier otra opci&oacute;n"
	" debe rellenar su ficha,"
	" incluyendo el sexo.";
#elif L==4
	"Avant d'aller &agrave; une autre option"
	" vous devez remplir votre carte d'enregistrement,"
	" y compris votre sexe.";
#elif L==5
	"Antes de acceder a cualquier otra opci&oacute;n"
	" debe rellenar su ficha,"
	" incluyendo el sexo.";				// Okoteve traduccin
#elif L==6
	"Prima di scegliere qualsiasi altra opzione"
	" devi compilare la tua scheda,"
	" compreso il sesso.";
#elif L==7
	"Zanim przejdziemy do innych opcji"
	" nalezy wypelnic karte zapisu,"
	" w tym ple&cacute;.";
#elif L==8
	"Antes de ir para qualquer outra op&ccedil;&atilde;o"
	" voc&ecirc; deve preencher o cart&atilde;o,"
	" incluindo o seu sexo.";
#endif

const char *Txt_Calculate_average_photo_of_a_degree =
#if   L==0
	"Calcular fotografia mitjana d'una titulaci&oacute;";
#elif L==1
	"Calculate average photo of a degree";				// Need bersetzung
#elif L==2
	"Calculate average photo of a degree";
#elif L==3
	"Calcular fotograf&iacute;a promedio de una titulaci&oacute;n";
#elif L==4
	"Calculate average photo of a degree";				// Besoin de traduction
#elif L==5
	"Calcular fotograf&iacute;a promedio de una titulaci&oacute;n";	// Okoteve traduccin
#elif L==6
	"Calcolare la media delle foto di laurea";
#elif L==7
	"Obliczyc &sacute;rednia zdjec stopnia";
#elif L==8
	"Calculate average photo of a degree";				// Necessita de traduo
#endif

const char *Txt_Calculate_average_photo_of_THE_DEGREE_X =
#if   L==0
	"Calcular fotografia mitjana de ";
#elif L==1
	"Calculate average photo of ";			// Need bersetzung
#elif L==2
	"Calculate average photo of ";
#elif L==3
	"Calcular fotograf&iacute;a promedio de ";
#elif L==4
	"Calculate average photo of ";			// Besoin de traduction
#elif L==5
	"Calcular fotograf&iacute;a promedio de ";	// Okoteve traduccin
#elif L==6
	"Calcolare la media della foto di ";
#elif L==7
	"Obliczyc &sacute;rednia zdjecie ";
#elif L==8
	"Calculate average photo of ";			// Necessita de traduo
#endif

const char *Txt_Call_the_roll =
#if   L==0
	"Passar llista";
#elif L==1
	"Anwesenheit feststellen";
#elif L==2
	"Call the roll";
#elif L==3
	"Pasar lista";
#elif L==4
	"Faire l'appel";
#elif L==5
	"Pasar lista";	// Okoteve traduccin
#elif L==6
	"Appello";
#elif L==7
	"Imienne";
#elif L==8
	"Lista de presen&ccedil;a";
#endif

const char *Txt_Can_not_create_the_folder_X_because_it_would_exceed_the_disk_quota =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"No s'ha creat la carpeta <strong>%s</strong>"
	" perque s'excediria la quota de disc.";
#elif L==1
	"Verzeichnis <strong>%s</strong> kann aus Platzgrnden"
	" nicht erstellt werden.";
#elif L==2
	"Cannot create the folder <strong>%s</strong>"
	" because it would exceed the disk quota.";
#elif L==3
	"No se ha creado la carpeta <strong>%s</strong>"
	" porque se exceder&iacute;a la cuota de disco.";
#elif L==4
	"Impossible de cr&eacute;er le r&eacute;pertoire <strong>%s</strong>"
	" parce qu'il d&eacute;passerait le quota de disque.";
#elif L==5
	"No se ha creado la carpeta <strong>%s</strong>"
	" porque se exceder&iacute;a la cuota de disco.";	// Okoteve traduccin
#elif L==6
	"Non si pu&ograve; creare la cartella <strong>%s</strong>"
	" perch&egrave; si &egrave; superata la quota del disco.";
#elif L==7
	"Nie mozna utworzy&cacute; folder <strong>%s</strong>,"
	" poniewaz to przekracza kontyngentu na dysku.";
#elif L==8
	"N&atilde;o &eacute; poss&iacute;vel criar o diret&oacute;rio <strong>%s</strong>"
	" porque iria exceder a cota de disco.";
#endif

const char *Txt_Can_not_create_the_folder_X_because_there_is_already_a_folder_or_a_file_with_that_name =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"No s'ha creat la carpeta <strong>%s</strong>"
	" perque ja existeix una carpeta o un fitxer amb aquest nom.";
#elif L==1
	"Cannot create the folder <strong>%s</strong>"
	" because there is already a folder or a file with that name.";	// Need bersetzung
#elif L==2
	"Cannot create the folder <strong>%s</strong>"
	" because there is already a folder or a file with that name.";
#elif L==3
	"No se ha creado la carpeta <strong>%s</strong>"
	" porque ya existe una carpeta o un archivo con ese nombre.";
#elif L==4
	"Impossible de cr&eacute;er le r&eacute;pertoire <strong>%s</strong>"
	" car il existe d&eacute;j&agrave; un r&eacute;pertoire ou un fichier avec ce nom.";
#elif L==5
	"No se ha creado la carpeta <strong>%s</strong>"
	" porque ya existe una carpeta o un archivo con ese nombre.";	// Okoteve traduccin
#elif L==6
	"Non puoi creare la cartella <strong>%s</strong>"
	" perch&egrave; gi&agrave; esiste una cartella o un file con quel nome.";
#elif L==7
	"Nie mozna utworzy&cacute; folder <strong>%s</strong>"
	" poniewaz istnieje juz folder lub plik o tej nazwie.";
#elif L==8
	"N&atilde;o &eacute; poss&iacute;vel criar o diret&oacute;rio <strong>%s</strong>"
	" porque j&aacute; existe um diret&oacute;rio ou um arquivo com esse nome.";
#endif

const char *Txt_Can_not_create_the_link_X_because_it_would_exceed_the_disk_quota =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"No s'ha creat l'enlla&ccedil; <strong>%s</strong>"
	" perque s'excediria la quota de disc.";
#elif L==1
	"Link <strong>%s</strong> kann aus Platzgr&uuml;nden"
	" nicht erstellt werden.";
#elif L==2
	"Cannot create the link <strong>%s</strong>"
	" because it would exceed the disk quota.";
#elif L==3
	"No se ha creado el enlace <strong>%s</strong>"
	" porque se exceder&iacute;a la cuota de disco.";
#elif L==4
	"Impossible de cr&eacute;er le lien <strong>%s</strong>"
	" parce qu'il d&eacute;passerait le quota de disque.";
#elif L==5
	"No se ha creado el enlace <strong>%s</strong>"
	" porque se exceder&iacute;a la cuota de disco.";	// Okoteve traduccin
#elif L==6
	"Non si pu&ograve; creare il link <strong>%s</strong>"
	" perch&egrave; si &egrave; superata la quota del disco.";
#elif L==7
	"Nie mozna utworzy&cacute; link <strong>%s</strong>,"
	" poniewaz to przekracza kontyngentu na dysku.";
#elif L==8
	"N&atilde;o &eacute; poss&iacute;vel criar a liga&ccedil;&atilde;o <strong>%s</strong>"
	" porque iria exceder a cota de disco.";
#endif

const char *Txt_Can_not_create_the_link_X_because_there_is_already_a_folder_or_a_link_with_that_name =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"No s'ha creat l'enlla&ccedil; <strong>%s</strong>"
	" perque ja existeix una carpeta o un enlla&ccedil; amb aquest nom.";
#elif L==1
	"Cannot create the link <strong>%s</strong>"
	" because there is already a folder or a link with that name.";	// Need bersetzung
#elif L==2
	"Cannot create the link <strong>%s</strong>"
	" because there is already a folder or a link with that name.";
#elif L==3
	"No se ha creado el enlace <strong>%s</strong>"
	" porque ya existe una carpeta o un enlace con ese nombre.";
#elif L==4
	"Impossible de cr&eacute;er le lien <strong>%s</strong>"
	" car il existe d&eacute;j&agrave; un r&eacute;pertoire ou un lien avec ce nom.";
#elif L==5
	"No se ha creado el enlace <strong>%s</strong>"
	" porque ya existe una carpeta o un enlace con ese nombre.";	// Okoteve traduccin
#elif L==6
	"Non puoi creare il link <strong>%s</strong>"
	" perch&egrave; gi&agrave; esiste una cartella o un link con quel nome.";
#elif L==7
	"Nie mozna utworzy&cacute; link <strong>%s</strong>"
	" poniewaz istnieje juz folder lub link o tej nazwie.";
#elif L==8
	"N&atilde;o &eacute; poss&iacute;vel criar a liga&ccedil;&atilde;o <strong>%s</strong>"
	" porque j&aacute; existe um diret&oacute;rio ou uma liga&ccedil;&atilde;o com esse nome.";
#endif

const char *Txt_Can_not_rename_a_folder_of_assignment =
#if   L==0
	"No es pot canviar el nom de una carpeta de activitat.";
#elif L==1
	"Das Verzeichnis der Aufgabe kann nicht umbenannt werden.";
#elif L==2
	"Cannot rename a folder of assignment.";
#elif L==3
	"No se puede cambiar el nombre de una carpeta de actividad.";
#elif L==4
	"Impossible de renommer un r&eacute;pertoire d'activit&eacute;.";
#elif L==5
	"No se puede cambiar el nombre de una carpeta de actividad.";	// Okoteve traduccin
#elif L==6
	"Non &egrave; possibile rinominare una cartella di attivit&agrave;.";
#elif L==7
	"Nie mozna zmienic nazwy folderu zadania.";
#elif L==8
	"N&atilde;o &eacute; poss&iacute;vel renomear um diret&oacute;rio de actividade.";
#endif

const char *Txt_Centre =
#if   L==0
	"Centre";
#elif L==1
	"Lehrinstitut";
#elif L==2
	"Centre";
#elif L==3
	"Centro";
#elif L==4
	"Centre";
#elif L==5
	"Mbo'ehao";
#elif L==6
	"Centro";
#elif L==7
	"Centrum";
#elif L==8
	"Centro";
#endif

const char *Txt_centre =
#if   L==0
	"centre";
#elif L==1
	"Lehrinstitut";
#elif L==2
	"centre";
#elif L==3
	"centro";
#elif L==4
	"centre";
#elif L==5
	"mbo'ehao";
#elif L==6
	"centro";
#elif L==7
	"centrum";
#elif L==8
	"centro";
#endif

const char *Txt_Centre_X_removed =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"Centre <strong>%s</strong> eliminat.";
#elif L==1
	"Lehrinstitut <strong>%s</strong> entfernt.";
#elif L==2
	"Centre <strong>%s</strong> removed.";
#elif L==3
	"Centro <strong>%s</strong> eliminado.";
#elif L==4
	"Centre <strong>%s</strong> removed.";		// Besoin de traduction
#elif L==5
	"Centro <strong>%s</strong> eliminado.";	// Okoteve traduccin
#elif L==6
	"Centro <strong>%s</strong> rimosso.";
#elif L==7
	"Centrum <strong>%s</strong> usuniete.";
#elif L==8
	"Centre <strong>%s</strong> removed.";		// Necessita de traduo
#endif

const char *Txt_CENTRE_STATUS[Crs_NUM_STATUS_TXT] =
   {
#if   L==0	// Crs_STATUS_UNKNOWN
	 "Desconegudo"
#elif L==1
	 "Unbekannt"
#elif L==2
	 "Unknown"
#elif L==3
	 "Desconocido"
#elif L==4
	 "Inconnu"
#elif L==5
	 "Desconocido"			// Okoteve traduccin
#elif L==6
	 "Sconosciuto"
#elif L==7
	 "Nieznany"
#elif L==8
	 "Desconhecido"
#endif
	 ,
#if   L==0	// Crs_STATUS_ACTIVE
	 "Activo"
#elif L==1
	 "Aktive"
#elif L==2
	 "Active"
#elif L==3
	 "Activo"
#elif L==4
	 "Actif"
#elif L==5
	 "Activo"	// Okoteve traduccin
#elif L==6
	 "Attivo"
#elif L==7
	 "Aktywne"
#elif L==8
	 "Ativo"
#endif
	 ,
#if   L==0	// Crs_STATUS_PENDING
	 "Pendent"
#elif L==1
	 "Offen"
#elif L==2
	 "Pending"
#elif L==3
	 "Pendiente"
#elif L==4
	 "En attente"
#elif L==5
	 "Pendiente"	// Okoteve traduccin
#elif L==6
	 "In attesa"
#elif L==7
	 "Do czasu"
#elif L==8
	 "Pendente"
#endif
	 ,
#if   L==0	// Crs_STATUS_REMOVED
	 "Eliminado"
#elif L==1
	 "Entfernt"
#elif L==2
	 "Removed"
#elif L==3
	 "Eliminado"
#elif L==4
	 "Supprim&eacute;"
#elif L==5
	 "Eliminado"	// Okoteve traduccin
#elif L==6
	 "Rimosso"
#elif L==7
	 "Usuniete"
#elif L==8
	 "Suprimido"
#endif
	};

const char *Txt_Centres =
#if   L==0
	"Centres";
#elif L==1
	"Lehrinstitute";
#elif L==2
	"Centres";
#elif L==3
	"Centros";
#elif L==4
	"Centres";
#elif L==5
	"Mbo'ehao";
#elif L==6
	"Centri";
#elif L==7
	"Centra";
#elif L==8
	"Centros";
#endif

const char *Txt_Centres_ABBREVIATION =
#if   L==0
	"Centres";
#elif L==1
	"Lehrinst.";
#elif L==2
	"Centres";
#elif L==3
	"Centros";
#elif L==4
	"Centres";
#elif L==5
	"Mbo'ehao";
#elif L==6
	"Centri";
#elif L==7
	"Centra";
#elif L==8
	"Centros";
#endif

const char *Txt_centres =
#if   L==0
	"centres";
#elif L==1
	"Lehrinstitute";
#elif L==2
	"centres";
#elif L==3
	"centros";
#elif L==4
	"centres";
#elif L==5
	"mbo'ehao";
#elif L==6
	"centri";
#elif L==7
	"centra";
#elif L==8
	"centros";
#endif

const char *Txt_CENTRES_HELP_ORDER[2] =
   {
#if   L==0
	 "Ordenar per centre"
#elif L==1
	 "Nach Lehrinstitut Sortieren"
#elif L==2
	 "Sort by centre"
#elif L==3
	 "Ordenar por centro"
#elif L==4
	 "Trier par centre"
#elif L==5
	 "Ordenar por centro"	// Okoteve traduccin
#elif L==6
	 "Ordina per centro"
#elif L==7
	 "Sortuj centrum"
#elif L==8
	 "Classificar por centro"
#endif
	 ,
#if   L==0
	 "Ordenar per nombre de professors"
#elif L==1
	 "Sortieren nach Anzahl der Lehrkr&auml;fte"
#elif L==2
	 "Sort by no. of teachers"
#elif L==3
	 "Ordenar por n&ordm; de profesores"
#elif L==4
	 "Trier par nombre d'enseignants"
#elif L==5
	 "Ordenar por n&ordm; de profesores"	// Okoteve traduccin
#elif L==6
	 "Ordina per numero di professori"
#elif L==7
	 "Sortuj nie. nauczycieli"
#elif L==8
	 "Classificar por n&ordm; de professores"
#endif
	};

const char *Txt_Centres_of_INSTITUTION_X =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"Centres de %s";
#elif L==1
	"Lehrinstitute von %s";
#elif L==2
	"Centres of %s";
#elif L==3
	"Centros de %s";
#elif L==4
	"Centres de %s";
#elif L==5
	"Centros de %s";	// Okoteve traduccin
#elif L==6
	"Centri di %s";
#elif L==7
	"Centra %s";
#elif L==8
	"Centros de %s";
#endif

const char *Txt_CENTRES_ORDER[2] =
   {
#if   L==0
	 "Centre"
#elif L==1
	 "Lehrinstitut"
#elif L==2
	 "Centre"
#elif L==3
	 "Centro"
#elif L==4
	 "Centre"
#elif L==5
	 "Mbo'ehao"
#elif L==6
	 "Centro"
#elif L==7
	 "Centrum"
#elif L==8
	 "Centro"
#endif
	 ,
#if   L==0
	 "Prof."
#elif L==1
	 "Lehrkr&auml;fte"
#elif L==2
	 "Teachers"
#elif L==3
	 "Prof."
#elif L==4
	 "Enseign."
#elif L==5
	 "Prof."	// Okoteve traduccin
#elif L==6
	 "Prof."
#elif L==7
	 "Nauczycieli"
#elif L==8
	 "Prof."
#endif
	};

const char *Txt_Centres_with_pending_degrees =
#if   L==0
	"Centres amb titulacions pendents d'activar";
#elif L==1
	"Lehrinstitute mit anh&auml;ngig Studiengang";
#elif L==2
	"Centres with pending degrees";
#elif L==3
	"Centros con titulaciones pendientes de activar";
#elif L==4
	"Centres avec des &eacute;tudes en attente";
#elif L==5
	"Centros con titulaciones pendientes de activar";	// Okoteve traduccin
#elif L==6
	"Centri con lauree in sospeso";
#elif L==7
	"Centra z stopnie oczekuj&aogon;cy";
#elif L==8
	"Centros com titula&ccedil;&otilde;es pendentes";
#endif

const char *Txt_Change_email =
#if   L==0
	"Canviar correu";
#elif L==1
	"E-Mail &auml;ndern";
#elif L==2
	"Change e-mail";
#elif L==3
	"Cambiar correo";
#elif L==4
	"Changer courrier";
#elif L==5
	"Cambiar correo";	// Okoteve traduccin
#elif L==6
	"Cambiare e-mail";
#elif L==7
	"Zmie&nacute; e-mail";
#elif L==8
	"Alterar e-mail";
#endif

const char *Txt_Change_IDs =
#if   L==0
	"Canviar IDs (DNI/c&egrave;dulas)";
#elif L==1
	"Ausweis-Nr. &auml;ndern";
#elif L==2
	"Change IDs";
#elif L==3
	"Cambiar ID (DNI/c&eacute;dulas)";
#elif L==4
	"Changer IDs";
#elif L==5
	"Cambiar ID (DNI/c&eacute;dulas)";		// Okoteve traduccin
#elif L==6
	"Cambiare ID";
#elif L==7
	"Zmie&nacute; identyfikatory";
#elif L==8
	"Alterar n&ordm;s. identif.";
#endif

const char *Txt_Change_my_groups =
#if   L==0
	"Canviar de grups";
#elif L==1
	"Gruppen &auml;ndern";
#elif L==2
	"Change my groups";
#elif L==3
	"Cambiar de grupos";
#elif L==4
	"Modifier mes groupes";
#elif L==5
	"Cambiar de grupos";	// Okoteve traduccin
#elif L==6
	"Cambia i miei gruppi";
#elif L==7
	"Zmie&nacute;moje grupy";
#elif L==8
	"Alterar meus grupos";
#endif

const char *Txt_Change_nickname =
#if   L==0
	"Canviar sobrenom";
#elif L==1
	"Benutzername &auml;ndern";
#elif L==2
	"Change nickname";
#elif L==3
	"Cambiar apodo";
#elif L==4
	"Changer le surnom";
#elif L==5
	"Cambiar apodo";	// Okoteve traduccin
#elif L==6
	"Cambiare nome utente";
#elif L==7
	"Zmie&nacute; nick";
#elif L==8
	"Alterar alcunha";
#endif

const char *Txt_Change_password =
#if   L==0
	"Canviar contrasenya";
#elif L==1
	"Passwort &auml;ndern";
#elif L==2
	"Change password";
#elif L==3
	"Cambiar contrase&ntilde;a";
#elif L==4
	"Changer le mot de passe";
#elif L==5
	"Cambiar contrase&ntilde;a";	// Okoteve traduccin
#elif L==6
	"Cambiare password";
#elif L==7
	"Zmie&nacute; has&lstrok;o";
#elif L==8
	"Alterar senha";
#endif

const char *Txt_Change_photo =
#if   L==0
	"Canviar foto";
#elif L==1
	"Foto &auml;ndern";
#elif L==2
	"Change photo";
#elif L==3
	"Cambiar foto";
#elif L==4
	"Changer photo";
#elif L==5
	"Cambiar foto";		// Okoteve traduccin
#elif L==6
	"Cambiare foto";
#elif L==7
	"Zmie&nacute; zdjecie";
#elif L==8
	"Alterar foto";
#endif

const char *Txt_Change_preferences =
#if   L==0
	"Canviar prefer&egrave;ncies";
#elif L==1
	"Einstellungen &auml;ndern";
#elif L==2
	"Change preferences";
#elif L==3
	"Cambiar preferencias";
#elif L==4
	"Changer mes pr&eacute;f&eacute;rences";
#elif L==5
	"Cambiar preferencias";	// Okoteve traduccin
#elif L==6
	"Cambiare preferenze";
#elif L==7
	"Zmiana preferencji";
#elif L==8
	"Alterar prefer&ecirc;ncias";
#endif

const char *Txt_Changing_the_IDs_for_the_following_user =
#if   L==0
	"Canvi de IDs (DNI/c&egrave;dulas) del seg&uuml;ent usuari:";
#elif L==1
	"Changing the IDs for the following user:";			// Need bersetzung
#elif L==2
	"Changing the IDs for the following user:";
#elif L==3
	"Cambio de los ID (DNI/c&eacute;dulas) del siguiente usuario:";
#elif L==4
	"Changing the IDs for the following user:";			// Besoin de traduction
#elif L==5
	"Cambio de los ID (DNI/c&eacute;dulas) del siguiente usuario:";	// Okoteve traduccin
#elif L==6
	"Cambiare gli ID per il seguente utente:";
#elif L==7
	"Zmiana hasla dla identyfikatory uzytkownik&oacute;w:";
#elif L==8
	"Changing the IDs for the following user:";			// Necessita de traduo
#endif

const char *Txt_Changing_the_password_for_the_following_user =
#if   L==0
	"Canvi de la contrasenya del seg&uuml;ent usuari:";
#elif L==1
	"Changing the password for the following user:";		// Need bersetzung
#elif L==2
	"Changing the password for the following user:";
#elif L==3
	"Cambio de la contrase&ntilde;a del siguiente usuario:";
#elif L==4
	"Changing the password for the following user:";		// Besoin de traduction
#elif L==5
	"Cambio de la contrase&ntilde;a del siguiente usuario:";	// Okoteve traduccin
#elif L==6
	"Cambiare la password per il seguente utente:";
#elif L==7
	"Zmiana hasla dla nastepujacych uzytkownik&oacute;w:";
#elif L==8
	"Changing the password for the following user:";		// Necessita de traduo
#endif

const char *Txt_CHAT_Room =
#if   L==0
	"Sala";
#elif L==1
	"Raum";
#elif L==2
	"Room";
#elif L==3
	"Sala";
#elif L==4
	"Salle";
#elif L==5
	"Sala";	// Okoteve traduccin
#elif L==6
	"Room";
#elif L==7
	"Pok&oacute;j";
#elif L==8
	"Room";
#endif

const char *Txt_CHAT_Room_code =
#if   L==0
	"Codi de sala";
#elif L==1
	"Raum-code";
#elif L==2
	"Room code";
#elif L==3
	"C&oacute;digo de sala";
#elif L==4
	"Code de salle";
#elif L==5
	"C&oacute;digo de sala";	// Okoteve traduccin
#elif L==6
	"Chiave di room";
#elif L==7
	"Kod pokoju";
#elif L==8
	"C&oacute;digo de room";
#endif

const char *Txt_Chat_rooms =
#if   L==0
	"Sales de xat";
#elif L==1
	"Chat-R&auml;ume";
#elif L==2
	"Chat rooms";
#elif L==3
	"Salas de chat";
#elif L==4
	"Salles de chat";
#elif L==5
	"Salas de chat";	// Okoteve traduccin
#elif L==6
	"Chat rooms";
#elif L==7
	"Czat";
#elif L==8
	"Chat rooms";
#endif

const char *Txt_Check_marks_in_file_X =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"Consultar qualificacions en el fitxer %s";
#elif L==1
	"Bewertung in Datei %s abrufen";
#elif L==2
	"Check marks in file %s";
#elif L==3
	"Consultar calificaciones en el archivo %s";
#elif L==4
	"Check marks in file %s";			// Besoin de traduction
#elif L==5
	"Consultar calificaciones en el archivo %s";	// Okoteve traduccin
#elif L==6
	"Consultare i risultati nel file %s";
#elif L==7
	"Zaznaczenia w pliku  %s";
#elif L==8
	"Check marks in file %s";			// Necessita de traduo
#endif

const char *Txt_Classroom =
#if   L==0
	"Aula";
#elif L==1
	"Klassenzimmer";
#elif L==2
	"Classroom";
#elif L==3
	"Aula";
#elif L==4
	"Classe";
#elif L==5
	"Aula";	// Okoteve traduccin
#elif L==6
	"Aula";
#elif L==7
	"Klasa";
#elif L==8
	"Sala de aula";
#endif

const char *Txt_Click =
#if   L==0
	"Clic";
#elif L==1
	"Klick";
#elif L==2
	"Click";
#elif L==3
	"Clic";
#elif L==4
	"Clic";
#elif L==5
	"Clic";	// Okoteve traduccin
#elif L==6
	"Clic";
#elif L==7
	"Kliknij";
#elif L==8
	"Clique";
#endif

const char *Txt_Clicks =
#if   L==0
	"Clics";
#elif L==1
	"Klicks";
#elif L==2
	"Clicks";
#elif L==3
	"Clics";
#elif L==4
	"Clics";
#elif L==5
	"Clics";	// Okoteve traduccin
#elif L==6
	"Clic";
#elif L==7
	"Klikni&eogon;cia";
#elif L==8
	"Cliques";
#endif

const char *Txt_Clicks_without_degree_selected =
#if   L==0
	"Clics sense seleccionar titulaci&oacute;";
#elif L==1
	"Clicks without degree selected";		// Need bersetzung
#elif L==2
	"Clicks without degree selected";
#elif L==3
	"Clics sin seleccionar titulaci&oacute;n";
#elif L==4
	"Clicks without degree selected";		// Besoin de traduction
#elif L==5
	"Clics sin seleccionar titulaci&oacute;n";	// Okoteve traduccin
#elif L==6
	"Clicca senza selezionare laurea";
#elif L==7
	"Klikniec bez stopnia wybranych";
#elif L==8
	"Clicks without degree selected";		// Necessita de traduo
#endif

const char *Txt_Closed_survey =
#if   L==0
	"Enquesta tancada";
#elif L==1
	"Closed survey";		// Need bersetzung
#elif L==2
	"Closed survey";
#elif L==3
	"Encuesta cerrada";
#elif L==4
	"Sondage ferm&eacute;";
#elif L==5
	"Encuesta cerrada";		// Okoteve traduccin
#elif L==6
	"Sondaggio chiuso";
#elif L==7
	"Zamkniete badania";
#elif L==8
	"Closed survey";		// Necessita de traduo
#endif

const char *Txt_Code =
#if   L==0
	"Codi";
#elif L==1
	"Code";
#elif L==2
	"Code";
#elif L==3
	"C&oacute;d.";
#elif L==4
	"Code";
#elif L==5
	"Papanga";
#elif L==6
	"Cod.";
#elif L==7
	"Kod";
#elif L==8
	"C&oacute;d.";
#endif

const char *Txt_Color_of_the_graphic =
#if   L==0
	"Color de la gr&agrave;fica";
#elif L==1
	"Farbe der Grafik";
#elif L==2
	"Color of the graphic";
#elif L==3
	"Color de la gr&aacute;fica";
#elif L==4
	"Couleur de l'image";
#elif L==5
	"Color de la gr&aacute;fica";	// Okoteve traduccin
#elif L==6
	"Colore del grafico";
#elif L==7
	"Kolor grafiki";
#elif L==8
	"Cor do gr&aacute;fico";
#endif

const char *Txt_Columns =
#if   L==0
	"Columnes";
#elif L==1
	"Spalten";
#elif L==2
	"Columns";
#elif L==3
	"Columnas";
#elif L==4
	"Colonnes";
#elif L==5
	"Columnas";	// Okoteve traduccin
#elif L==6
	"Colonne";
#elif L==7
	"Kolumny";
#elif L==8
	"Colunas";
#endif

const char *Txt_columns =
#if   L==0
	"columnes";
#elif L==1
	"Spalten";
#elif L==2
	"columns";
#elif L==3
	"columnas";
#elif L==4
	"colonnes";
#elif L==5
	"columnas";	// Okoteve traduccin
#elif L==6
	"colonne";
#elif L==7
	"kolumny";
#elif L==8
	"colunas";
#endif

const char *Txt_Comments =
#if   L==0
	"Comentaris";
#elif L==1
	"Kommentare";
#elif L==2
	"Comments";
#elif L==3
	"Comentarios";
#elif L==4
	"Commentaires";
#elif L==5
	"Comentarios";	// Okoteve traduccin
#elif L==6
	"Commenti";
#elif L==7
	"Komentarze";
#elif L==8
	"Coment&aacute;rios";
#endif

const char *Txt_Teachers_comment =
#if   L==0
	"Comentari de professors";
#elif L==1
	"Lehrern-Kommentar";
#elif L==2
	"Teachers' comment";
#elif L==3
	"Comentario de profesores";
#elif L==4
	"Commentaire des enseignants";
#elif L==5
	"Comentario de profesores";	// Okoteve traduccin
#elif L==6
	"Commento degli professori";
#elif L==7
	"Komentarz nauczycieli";
#elif L==8
	"Coment&aacute;rio dos professores";
#endif

const char *Txt_common_zone =
#if   L==0
	"zona d'arxius compartits";
#elif L==1
	"Freigegebene Dateien-Bereich";
#elif L==2
	"shared files area";
#elif L==3
	"zona de archivos compartidos";
#elif L==4
	"zone de fichiers partag&eacute;s";
#elif L==5
	"zona de archivos compartidos";	// Okoteve traduccin
#elif L==6
	"zona di file condivisi";
#elif L==7
	"Powierzchnia udost&eogon;pnionych plik&oacute;w";
#elif L==8
	"zona de arquivos compartilhados";
#endif

const char *Txt_Completely_eliminate_user =
#if   L==0
	"Eliminar usuari completament";
#elif L==1
	"Completely eliminate user";		// Need bersetzung
#elif L==2
	"Completely eliminate user";
#elif L==3
	"Eliminar usuario completamente";
#elif L==4
	"Completely eliminate user";		// Besoin de traduction
#elif L==5
	"Eliminar usuario completamente";	// Okoteve traduccin
#elif L==6
	"Rimuovere utente completamente";
#elif L==7
	"Calkowicie wyeliminowac u&zdot;ytkownika";
#elif L==8
	"Completely eliminate user";		// Necessita de traduo
#endif

const char *Txt_Completely_eliminate_me =
#if   L==0
	"Eliminar-me completament";
#elif L==1
	"Completely eliminate me";		// Need bersetzung
#elif L==2
	"Completely eliminate me";
#elif L==3
	"Eliminarme completamente";
#elif L==4
	"Completely eliminate me";		// Besoin de traduction
#elif L==5
	"Eliminarme completamente";		// Okoteve traduccin
#elif L==6
	"Rimuovimi completamente";
#elif L==7
	"Calkowicie mnie wyeliminowac";
#elif L==8
	"Completely eliminate me";		// Necessita de traduo
#endif

const char *Txt_Configure =
#if   L==0
	"Configurar";
#elif L==1
	"Konfigurieren";
#elif L==2
	"Configure";
#elif L==3
	"Configurar";
#elif L==4
	"Configurer";
#elif L==5
	"Configurar";	// Okoteve traduccin
#elif L==6
	"Configurare";
#elif L==7
	"Konfiguracja";
#elif L==8
	"Configurar";
#endif

const char *Txt_Configure_tests =
#if   L==0
	"Configurar tests";
#elif L==1
	"Konfigurieren Tests";
#elif L==2
	"Configure tests";
#elif L==3
	"Configurar tests";
#elif L==4
	"Configurer tests";
#elif L==5
	"Configurar tests";	// Okoteve traduccin
#elif L==6
	"Configurare test";
#elif L==7
	"Test Konfiguracja";
#elif L==8
	"Configurar testes";
#endif

const char *Txt_Confirm =
#if   L==0
	"Confirmar";
#elif L==1
	"Best&auml;tigen";
#elif L==2
	"Confirm";
#elif L==3
	"Confirmar";
#elif L==4
	"Confirmer";
#elif L==5
	"Confirmar";	// Okoteve traduccin
#elif L==6
	"Confermare";
#elif L==7
	"Potwierdzenie";
#elif L==8
	"Confirmar";
#endif

const char *Txt_Confirm_email =
#if   L==0
	"Confirmar correu electr&ograve;nic";
#elif L==1
	"E-Mail best&auml;tigen";
#elif L==2
	"Confirm e-mail";
#elif L==3
	"Confirmar correo electr&oacute;nico";
#elif L==4
	"Confirmer courrier &eacute;lectronique";
#elif L==5
	"Confirmar correo electr&oacute;nico";	// Okoteve traduccin
#elif L==6
	"Confermare e-mail";
#elif L==7
	"Potwierdzenie e-mail";
#elif L==8
	"Confirmar e-mail";
#endif

const char *Txt_Confirm_my_enrollment =
#if   L==0
	"Confirmar la meva inscripci&oacute;";
#elif L==1
	"Best&auml;tigen Sie meine Registrierung";
#elif L==2
	"Confirm my enrollment";
#elif L==3
	"Confirmar mi inscripci&oacute;n";
#elif L==4
	"Confirmer mon inscription";
#elif L==5
	"Confirmar mi inscripci&oacute;n";	// Okoteve traduccin
#elif L==6
	"Confermare la mia iscrizione";
#elif L==7
	"Potwierdzenie mojej rejestracji";
#elif L==8
	"Confirmar o meu registo";
#endif

const char *Txt_Connected_PLURAL =
#if   L==0
	"Connectats";
#elif L==1
	"Angemeldet";
#elif L==2
	"Connected";
#elif L==3
	"Conectados";
#elif L==4
	"Connect&eacute;s";
#elif L==5
	"Conectados";	// Okoteve traduccin
#elif L==6
	"Connessi";
#elif L==7
	"Online";
#elif L==8
	"Conectados";
#endif

const char *Txt_connected_PLURAL =
#if   L==0
	"connectats";
#elif L==1
	"angemeldet";
#elif L==2
	"connected";
#elif L==3
	"conectados";
#elif L==4
	"connect&eacute;s";
#elif L==5
	"conectados";	// Okoteve traduccin
#elif L==6
	"connessi";
#elif L==7
	"online";
#elif L==8
	"conectados";
#endif

const char *Txt_connected_SINGULAR =
#if   L==0
	"connectat";
#elif L==1
	"angemeldet";
#elif L==2
	"connected";
#elif L==3
	"conectado";
#elif L==4
	"connect&eacute;";
#elif L==5
	"conectado";	// Okoteve traduccin
#elif L==6
	"connesso";
#elif L==7
	"online";
#elif L==8
	"conectado";
#endif

const char *Txt_Connected_users =
#if   L==0
	"Usuaris connectats";
#elif L==1
	"Angemeldet Benutzer";
#elif L==2
	"Connected users";
#elif L==3
	"Usuarios conectados";
#elif L==4
	"Utilisateurs connect&eacute;s";
#elif L==5
	"Usuarios conectados";	// Okoteve traduccin
#elif L==6
	"Utenti connessi";
#elif L==7
	"U&zdot;ytkownicy online";
#elif L==8
	"Utilizadores conectados";
#endif

const char *Txt_Continue =
#if   L==0
	"Continuar";
#elif L==1
	"Weiter";
#elif L==2
	"Continue";
#elif L==3
	"Continuar";
#elif L==4
	"Continuer";
#elif L==5
	"Hemo&ntilde;a";
#elif L==6
	"Continua";
#elif L==7
	"Kontynuuj";
#elif L==8
	"Continuar";
#endif

const char *Txt_Contract_FOLDER_X =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"Contreure %s";
#elif L==1
	"Zusammenziehen %s";
#elif L==2
	"Contract %s";
#elif L==3
	"Contraer %s";
#elif L==4
	"Contracter %s";
#elif L==5
	"Contraer %s";	// Okoteve traduccin
#elif L==6
	"Contratto %s";
#elif L==7
	"Umowy %s";
#elif L==8
	"Contrair %s";
#endif

const char *Txt_Control_of_class_attendance =
#if   L==0
	"Control d'assistencia a classe";
#elif L==1
	"Anwesenheitskontrolle";
#elif L==2
	"Control of class attendance";
#elif L==3
	"Control de asistencia a clase";
#elif L==4
	"Contr&ouml;le de la fr&eacute;quentation scolaire";
#elif L==5
	"Control de asistencia a clase";	// Okoteve traduccin
#elif L==6
	"Controllo delle presenze in classe";
#elif L==7
	"Kontrola obecnosc na zajeciach";
#elif L==8
	"Controlo de assiduidade de classe";
#endif

const char *Txt_Copy_FOLDER_FILE_OR_LINK_X =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"Copiar %s";
#elif L==1
	"Kopieren %s";
#elif L==2
	"Copy %s";
#elif L==3
	"Copiar %s";
#elif L==4
	"Copier %s";
#elif L==5
	"Ekopia %s";
#elif L==6
	"Copiare %s";
#elif L==7
	"Kopia %s";
#elif L==8
	"Copiar %s";
#endif

const char *Txt_Copy_source =
#if   L==0
	"Origen de c&ograve;pia";
#elif L==1
	"Kopier-Quelle";
#elif L==2
	"Copy source";
#elif L==3
	"Origen de copia";
#elif L==4
	"Source de copie";
#elif L==5
	"Origen de copia";	// Okoteve traduccin
#elif L==6
	"Copia sorgente";
#elif L==7
	"Kopiowanie zr&oacute;dla";
#elif L==8
	"Origem da copia";
#endif

const char *Txt_Could_not_detect_any_face_in_front_position_ =
#if   L==0
	"No s'ha detectat cap rostre en posici&oacute; frontal.";
#elif L==1
	"Could not detect any face in front position.";				// Need bersetzung
#elif L==2
	"Could not detect any face in front position.";
#elif L==3
	"No se ha detectado ning&uacute;n rostro en posici&oacute;n frontal.";
#elif L==4
	"Could not detect any face in front position.";				// Besoin de traduction
#elif L==5
	"No se ha detectado ning&uacute;n rostro en posici&oacute;n frontal.";	// Okoteve traduccin
#elif L==6
	"Non puoi rilevare qualsiasi viso in posizione frontale.";
#elif L==7
	"Nie wykryto zadnej twarzy w przednim.";
#elif L==8
	"Could not detect any face in front position.";				// Necessita de traduo
#endif

const char *Txt_Could_not_get_the_official_list_of_your_groups_and_students_ =	// Warning: it is very important to include three %s in the following sentences
#if   L==0
	"No s'ha pogut obtenir la llista oficial dels seus grups i estudiants.<br />"
	"Es posible que haya expirado el plazo para obtener dicha lista.<br />"
	"Entre de nuevo a %s desde <a href=\"%s\" target=\"_blank\">%s</a> y reint&eacute;ntelo.";	// Necessita traduccio
#elif L==1
	"Could not get the official list of your groups and students.<br />"
	"It is possible that the deadline expired for getting the list.<br />"
	"Log in again to %s from <a href=\"%s\" target=\"_blank\">%s</a> and try again.";		// Need bersetzung
#elif L==2
	"Could not get the official list of your groups and students.<br />"
	"It is possible that the deadline expired for getting the list.<br />"
	"Log in again to %s from <a href=\"%s\" target=\"_blank\">%s</a> and try again.";
#elif L==3
	"No se ha podido obtener la lista oficial de sus grupos y estudiantes.<br />"
	"Es posible que haya expirado el plazo para obtener dicha lista.<br />"
	"Entre de nuevo a %s desde <a href=\"%s\" target=\"_blank\">%s</a> y reint&eacute;ntelo.";
#elif L==4
	"Could not get the official list of your groups and students.<br />"
	"It is possible that the deadline expired for getting the list.<br />"
	"Log in again to %s from <a href=\"%s\" target=\"_blank\">%s</a> and try again.";		// Besoin de traduction
#elif L==5
	"No se ha podido obtener la lista oficial de sus grupos y estudiantes.<br />"
	"Es posible que haya expirado el plazo para obtener dicha lista.<br />"
	"Entre de nuevo a %s desde <a href=\"%s\" target=\"_blank\">%s</a> y reint&eacute;ntelo.";	// Okoteve traduccin
#elif L==6
	"Non puoi prendere la lista ufficiale dei tuoi gruppi e degli studenti.<br />"
	"E' possibile che il tempo per acquisire la lista sia scaduto.<br />"
	"Loggati nuovamente a %s da %s e prova ancora.";
#elif L==7
	"Nie udalo sie uzyskac oficjalne liste grup i student&oacute;w.<br />"
	"Jest mozliwe, ze uplynal termin uzyskania listy<br />"
	"Zaloguj sie ponownie do %s <a href=\"%s\" target=\"_blank\">%s</a> i spr&oacute;buj ponownie";	// Please, check this!
#elif L==8
	"Could not get the official list of your groups and students.<br />"
	"It is possible that the deadline expired for getting the list.<br />"
	"Log in again to %s from <a href=\"%s\" target=\"_blank\">%s</a> and try again.";		// Necessita de traduo
#endif

const char *Txt_Countries =
#if   L==0
	"Pa&iuml;sos";
#elif L==1
	"L&auml;nder";
#elif L==2
	"Countries";
#elif L==3
	"Pa&iacute;ses";
#elif L==4
	"Pays";
#elif L==5
	"Tet&atilde;";
#elif L==6
	"Paesi";
#elif L==7
	"Kraje";
#elif L==8
	"Pa&iacute;ses";
#endif

const char *Txt_COUNTRIES_HELP_ORDER[2] =
   {
#if   L==0
	 "Ordenar por pa&iacute;s"	// Necessita traduccio
#elif L==1
	 "Nach Land sortieren"
#elif L==2
	 "Sort by country"
#elif L==3
	 "Ordenar por pa&iacute;s"
#elif L==4
	 "Trier par pays"
#elif L==5
	 "Ordenar por pa&iacute;s"	// Okoteve traduccin
#elif L==6
	 "Ordina per paese"
#elif L==7
	 "Sortuj wedlug kraju"
#elif L==8
	 "Classificar por pa&iacute;s"
#endif
	 ,
#if   L==0
	 "Ordenar por n&ordm; de usuarios"	// Necessita traduccio
#elif L==1
	 "Nach Anzahl der Benutzer sortieren"
#elif L==2
	 "Sort by no. of users"
#elif L==3
	 "Ordenar por n&ordm; de usuarios"
#elif L==4
	 "Trier par nombre d'utilisateurs"
#elif L==5
	 "Ordenar por n&ordm; de usuarios"	// Okoteve traduccin
#elif L==6
	 "Ordina per numero di utenti"
#elif L==7
	 "Sortuj wedlug liczby uzytkownikow"
#elif L==8
	 "Classificar por n&ordm; de utilizadores"
#endif
	};

const char *Txt_Countries_with_pending_institutions =
#if   L==0
	"Pa&iuml;sos amb institucions pendents d'activar";
#elif L==1
	"L&auml;nder mit anh&auml;ngig Hochschulen";
#elif L==2
	"Countries with pending institutions";
#elif L==3
	"Pa&iacute;ses con instituciones pendientes de activar";
#elif L==4
	"Pays avec des &eacute;tablissements en attente";
#elif L==5
	"Pa&iacute;ses con instituciones pendientes de activar";	// Okoteve traduccin
#elif L==6
	"Paesi con istituzioni in sospeso";
#elif L==7
	"Kraje z instytucje oczekuj&aogon;cy";
#elif L==8
	"Pa&iacute;ses com institu&ccedil;&otilde;es pendentes";
#endif

const char *Txt_COUNTRIES_ORDER[2] =
   {
#if   L==0
	 "Pa&iacute;s"
#elif L==1
	 "Land"
#elif L==2
	 "Country"
#elif L==3
	 "Pa&iacute;s"
#elif L==4
	 "Pays"
#elif L==5
	 "Tet&atilde;"
#elif L==6
	 "Paese"
#elif L==7
	 "Kraj"
#elif L==8
	 "Pa&iacute;s"
#endif
	 ,
#if   L==0
	 "Usuaris"
#elif L==1
	 "Benutzer"
#elif L==2
	 "Users"
#elif L==3
	 "Usuarios"
#elif L==4
	 "Utilisateurs"
#elif L==5
	 "Usuarios"	// Okoteve traduccin
#elif L==6
	 "Utenti"
#elif L==7
	 "Users"
#elif L==8
	 "Utilizadores"
#endif
	};

const char *Txt_Country =
#if   L==0
	"Pa&iacute;s";
#elif L==1
	"Land";
#elif L==2
	"Country";
#elif L==3
	"Pa&iacute;s";
#elif L==4
	"Pays;";
#elif L==5
	"Tet&atilde;";
#elif L==6
	"Paese";
#elif L==7
	"Kraj";
#elif L==8
	"Pa&iacute;s";
#endif

const char *Txt_Country_of_institution =
#if   L==0
	"Pa&iacute;s de la instituci&oacute;";
#elif L==1
	"Land der Hochschule";
#elif L==2
	"Country of institution";
#elif L==3
	"Pa&iacute;s de la instituci&oacute;n";
#elif L==4
	"Pays de l'&eacute;tablissement";
#elif L==5
	"Tet&atilde; mbo'ehao";
#elif L==6
	"Paese di istituzione";
#elif L==7
	"Kraj instytucji";
#elif L==8
	"Pa&iacute;s de institu&ccedil;&atilde;o";
#endif

const char *Txt_Country_unspecified =
#if   L==0
	"Pa&iacute;s sense especificar";
#elif L==1
	"Land nicht angegeben";
#elif L==2
	"Country unspecified";
#elif L==3
	"Pa&iacute;s sin especificar";
#elif L==4
	"Pays non sp&eacute;cifi&eacute;";
#elif L==5
	"Pa&iacute;s sin especificar";	// Okoteve traduccin
#elif L==6
	"Paese non specificato";
#elif L==7
	"Kraj nieokreslony";
#elif L==8
	"Pa&iacute;s indeterminado";
#endif

const char *Txt_Country_X_removed =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"Pa&iacute;s <strong>%s</strong> eliminado.";	// Necessita traduccio
#elif L==1
	"Land <strong>%s</strong> entfernt.";
#elif L==2
	"Country <strong>%s</strong> removed.";
#elif L==3
	"Pa&iacute;s <strong>%s</strong> eliminado.";
#elif L==4
	"Pays <strong>%s</strong> removed.";		// Besoin de traduction
#elif L==5
	"Pa&iacute;s <strong>%s</strong> eliminado.";	// Okoteve traduccin
#elif L==6
	"Paese <strong>%s</strong> rimosso.";
#elif L==7
	"Kraj <strong>%s</strong> usuniete.";
#elif L==8
	"Pa&iacute;s <strong>%s</strong> removed.";	// Necessita de traduo
#endif

const char *Txt_Course =
#if   L==0
	"Assignatura";
#elif L==1
	"Kurs";
#elif L==2
	"Course";
#elif L==3
	"Asignatura";
#elif L==4
	"Mati&egrave;re";
#elif L==5
	"Mbo'esyry";
#elif L==6
	"Corso";
#elif L==7
	"Kurs";
#elif L==8
	"Disciplina";
#endif

const char *Txt_course =
#if   L==0
	"assignatura";
#elif L==1
	"Kurs";
#elif L==2
	"course";
#elif L==3
	"asignatura";
#elif L==4
	"mati&egrave;re";
#elif L==5
	"mbo'esyry";
#elif L==6
	"corso";
#elif L==7
	"kurs";
#elif L==8
	"disciplina";
#endif

const char *Txt_COURSE_STATUS[Crs_NUM_STATUS_TXT] =
   {
#if   L==0	// Crs_STATUS_UNKNOWN
	 "Desconegudo"
#elif L==1
	 "Unbekannt"
#elif L==2
	 "Unknown"
#elif L==3
	 "Desconocido"
#elif L==4
	 "Inconnu"
#elif L==5
	 "Desconocido"			// Okoteve traduccin
#elif L==6
	 "Sconosciuto"
#elif L==7
	 "Nieznany"
#elif L==8
	 "Desconhecido"
#endif
	 ,
#if   L==0	// Crs_STATUS_ACTIVE
	 "Activa"
#elif L==1
	 "Aktive"
#elif L==2
	 "Active"
#elif L==3
	 "Activa"
#elif L==4
	 "Actif"
#elif L==5
	 "Activa"	// Okoteve traduccin
#elif L==6
	 "Attivo"
#elif L==7
	 "Aktywne"
#elif L==8
	 "Ativa"
#endif
	 ,
#if   L==0	// Crs_STATUS_PENDING
	 "Pendent"
#elif L==1
	 "Offen"
#elif L==2
	 "Pending"
#elif L==3
	 "Pendiente"
#elif L==4
	 "En attente"
#elif L==5
	 "Pendiente"	// Okoteve traduccin
#elif L==6
	 "In attesa"
#elif L==7
	 "Do czasu"
#elif L==8
	 "Pendente"
#endif
	 ,
#if   L==0	// Crs_STATUS_REMOVED
	 "Eliminada"
#elif L==1
	 "Entfernt"
#elif L==2
	 "Removed"
#elif L==3
	 "Eliminada"
#elif L==4
	 "Supprim&eacute;"
#elif L==5
	 "Eliminada"	// Okoteve traduccin
#elif L==6
	 "Rimosso"
#elif L==7
	 "Usuniete"
#elif L==8
	 "Suprimida"
#endif
	};

const char *Txt_COURSE_With_users =
#if   L==0
	"Amb usuaris";
#elif L==1
	"Mit Benutzer";
#elif L==2
	"With users";
#elif L==3
	"Con usuarios";
#elif L==4
	"Avec utilisateurs";
#elif L==5
	"Con usuarios";	// Okoteve traduccin
#elif L==6
	"Con utenti";
#elif L==7
	"Z uzytkownikow";
#elif L==8
	"Com utilizadores";
#endif

const char *Txt_COURSE_Without_users =
#if   L==0
	"Sense usuaris";
#elif L==1
	"Ohne Benutzer";
#elif L==2
	"Without users";
#elif L==3
	"Sin usuarios";
#elif L==4
	"Sans utilisateurs";
#elif L==5
	"Sin usuarios";	// Okoteve traduccin
#elif L==6
	"Senza utenti";
#elif L==7
	"Bez uzytkownikow";
#elif L==8
	"Sem utilizadores";
#endif

const char *Txt_Course_X_removed =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"Asignatura <strong>%s</strong> eliminada.";	// Necessita traduccio
#elif L==1
	"Kurs <strong>%s</strong> entfernt.";
#elif L==2
	"Course <strong>%s</strong> removed.";
#elif L==3
	"Asignatura <strong>%s</strong> eliminada.";
#elif L==4
	"Mati&egrave;re <strong>%s</strong> supprim&eacute;.";
#elif L==5
	"Asignatura <strong>%s</strong> eliminada.";	// Okoteve traduccin
#elif L==6
	"Corso <strong>%s</strong> rimosso.";
#elif L==7
	"Kurs <strong>%s</strong> usuniete.";
#elif L==8
	"Course <strong>%s</strong> removed.";		// Necessita de traduo
#endif

const char *Txt_Courses =
#if   L==0
	"Assignatures";
#elif L==1
	"Kursen";
#elif L==2
	"Courses";
#elif L==3
	"Asignaturas";
#elif L==4
	"Mati&egrave;res";
#elif L==5
	"Mbo'esyry";
#elif L==6
	"Corsi";
#elif L==7
	"Kurs&oacute;w";
#elif L==8
	"Disciplinas";
#endif

const char *Txt_Courses_ABBREVIATION =
#if   L==0
	"Assignat.";
#elif L==1
	"Veranst.";
#elif L==2
	"Courses";
#elif L==3
	"Asignat.";
#elif L==4
	"Mati&egrave;.";
#elif L==5
	"Asignat.";	// Okoteve traduccin
#elif L==6
	"Corsi";
#elif L==7
	"Kurs&oacute;w";
#elif L==8
	"Discip.";
#endif

const char *Txt_courses =
#if   L==0
	"assignatures";
#elif L==1
	"Kursen";
#elif L==2
	"courses";
#elif L==3
	"asignaturas";
#elif L==4
	"mati&egrave;res";
#elif L==5
	"mbo'esyry";
#elif L==6
	"corsi";
#elif L==7
	"kurs&oacute;w";
#elif L==8
	"disciplinas";
#endif

const char *Txt_X_courses_have_been_eliminated =	// Warning: it is very important to include %u in the following sentences
#if   L==0
	"Se ha(n) eliminado %u asignatura(s).";		// Necessita traduccio
#elif L==1
	"%u course(s) have been eliminated.";		// Need bersetzung
#elif L==2
	"%u course(s) have been eliminated.";
#elif L==3
	"Se ha(n) eliminado %u asignatura(s).";
#elif L==4
	"%u course(s) have been eliminated.";		// Besoin de traduction
#elif L==5
	"Se ha(n) eliminado %u asignatura(s).";		// Okoteve traduccin
#elif L==6
	"%u corsi sono stati rimossi.";
#elif L==7
	"%u course(s) have been eliminated.";		// Potrzebujesz tlumaczenie
#elif L==8
	"%u course(s) have been eliminated.";		// Necessita de traduo
#endif

const char *Txt_Courses_of_DEGREE_X =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"Assignatures de %s";
#elif L==1
	"Kursen der %s";
#elif L==2
	"Courses of %s";
#elif L==3
	"Asignaturas de %s";
#elif L==4
	"Mati&egrave;res de %s";
#elif L==5
	"Asignaturas de %s";	// Okoteve traduccin
#elif L==6
	"Corsi di %s";
#elif L==7
	"Kurs&oacute;w %s";
#elif L==8
	"Disciplinas de %s";
#endif

const char *Txt_Create_account =
#if   L==0
	"Crea compte";
#elif L==1
	"Benutzerkonto anlegen";
#elif L==2
	"Create account";
#elif L==3
	"Crear cuenta";
#elif L==4
	"Cr&eacute;er un compte";
#elif L==5
	"Crear cuenta";		// Okoteve traduccin
#elif L==6
	"Registrati";
#elif L==7
	"Za&lstrok;&oacute;&zdot; nowe konto";
#elif L==8
	"Criar conta";
#endif

const char *Txt_Create_announcement =
#if   L==0
	"Crear anunci";
#elif L==1
	"Erstelle Bekanntmachung";
#elif L==2
	"Create announcement";
#elif L==3
	"Crear anuncio";
#elif L==4
	"Cr&eacute;er annonce";
#elif L==5
	"Crear anuncio";		// Okoteve traduccin
#elif L==6
	"Crea annuncio";
#elif L==7
	"Stw&oacute;rz og&lstrok;oszenia";
#elif L==8
	"Criar an&uacute;ncio";
#endif

const char *Txt_Create_assignment =
#if   L==0
	"Crear actividad";	// Necessita traduccio
#elif L==1
	"Aufgabe erstellen";
#elif L==2
	"Create assignment";
#elif L==3
	"Crear actividad";
#elif L==4
	"Cr&eacute;er activit&eacute;";
#elif L==5
	"Crear actividad";	// Okoteve traduccin
#elif L==6
	"Crea attivit&agrave;";
#elif L==7
	"Utw&oacute;rz zadanie";
#elif L==8
	"Criar atividade";
#endif

const char *Txt_Create_banner =
#if   L==0
	"Crear banner";	// Necessita traduccio
#elif L==1
	"Werbebanner erstellen";
#elif L==2
	"Create banner";
#elif L==3
	"Crear banner";
#elif L==4
	"Cr&eacute;er banni&eacute;re";
#elif L==5
	"Crear banner";	// Okoteve traduccin
#elif L==6
	"Crea banner";
#elif L==7
	"Utw&oacute;rz banner";
#elif L==8
	"Criar banner";
#endif

const char *Txt_Create_centre =
#if   L==0
	"Crear centro";		// Necessita traduccio
#elif L==1
	"Lehrinstitut eingeben";
#elif L==2
	"Create centre";
#elif L==3
	"Crear centro";
#elif L==4
	"Cr&eacute;er centre";
#elif L==5
	"Crear centro";		// Okoteve traduccin
#elif L==6
	"Crea centro";
#elif L==7
	"Tworzenie centrum";
#elif L==8
	"Criar centro";
#endif

const char *Txt_Create_country =
#if   L==0
	"Crear pa&iacute;s";	// Necessita traduccio
#elif L==1
	"Land eingeben";
#elif L==2
	"Create country";
#elif L==3
	"Crear pa&iacute;s";
#elif L==4
	"Cr&eacute;er pays";
#elif L==5
	"Crear pa&iacute;s";	// Okoteve traduccin
#elif L==6
	"Crea paese";
#elif L==7
	"Tworzenie kraju";
#elif L==8
	"Criar pa&iacute;s";
#endif

const char *Txt_Create_course =
#if   L==0
	"Crear asignatura";	// Necessita traduccio
#elif L==1
	"Kurs eingeben";
#elif L==2
	"Create course";
#elif L==3
	"Crear asignatura";
#elif L==4
	"Cr&eacute;er mati&egrave;re";
#elif L==5
	"Crear asignatura";	// Okoteve traduccin
#elif L==6
	"Crea corso";
#elif L==7
	"Tworzenie kursu";
#elif L==8
	"Criar disciplina";
#endif

const char *Txt_Create_degree =
#if   L==0
	"Crear titulaci&oacute;n";	// Necessita traduccio
#elif L==1
	"Studiengang eingeben";
#elif L==2
	"Create degree";
#elif L==3
	"Crear titulaci&oacute;n";
#elif L==4
	"Cr&eacute;er &eacute;tude";
#elif L==5
	"Crear titulaci&oacute;n";	// Okoteve traduccin
#elif L==6
	"Crea laurea";
#elif L==7
	"Tworzenie stopnia";
#elif L==8
	"Criar titula&ccedil;&atilde;o";
#endif

const char *Txt_Create_department =
#if   L==0
	"Crear departamento";	// Necessita traduccio
#elif L==1
	"Abteilung eingeben";
#elif L==2
	"Create department";
#elif L==3
	"Crear departamento";
#elif L==4
	"Cr&eacute;er d&eacute;partement";
#elif L==5
	"Crear departamento";	// Okoteve traduccin
#elif L==6
	"Crea dipartimento";
#elif L==7
	"Tworzenie dzia&lstrok;u";
#elif L==8
	"Criar departamento";
#endif

const char *Txt_Create_e_mail_message =
#if   L==0
	"Crear mensaje de correo electr&oacute;nico";	// Necessita traduccio
#elif L==1
	"Neue E-Mail-Nachricht";
#elif L==2
	"Create e-mail message";
#elif L==3
	"Crear mensaje de correo electr&oacute;nico";
#elif L==4
	"Cr&eacute;er un e-mail message";
#elif L==5
	"Crear mensaje de correo electr&oacute;nico";	// Okoteve traduccin
#elif L==6
	"Crea messaggio e-mail";
#elif L==7
	"Tworzenie wiadomosci e-mail";
#elif L==8
	"Criar uma mensagem de e-mail";
#endif

const char *Txt_Create_event =
#if   L==0
	"Crear evento";	// Necessita traduccio
#elif L==1
	"Ereignis erstellen";
#elif L==2
	"Create event";
#elif L==3
	"Crear evento";
#elif L==4
	"Cr&eacute;er &eacute;v&eacute;nement";
#elif L==5
	"Crear evento";	// Okoteve traduccin
#elif L==6
	"Crea evento";
#elif L==7
	"Utw&oacute;rz wydarzenie";
#elif L==8
	"Criar evento";
#endif

const char *Txt_Create_folder =
#if   L==0
	"Crear carpeta";	// Necessita traduccio
#elif L==1
	"Verzeichnis anlegen";
#elif L==2
	"Create folder";
#elif L==3
	"Crear carpeta";
#elif L==4
	"Cr&eacute;er r&eacute;pertoire";
#elif L==5
	"Crear carpeta";	// Okoteve traduccin
#elif L==6
	"Crea cartella";
#elif L==7
	"Utw&oacute;rz folder";
#elif L==8
	"Criar diret&oacute;rio";
#endif

const char *Txt_Create_group =
#if   L==0
	"Crear grupo";		// Necessita traduccio
#elif L==1
	"Gruppe erstellen";
#elif L==2
	"Create group";
#elif L==3
	"Crear grupo";
#elif L==4
	"Cr&eacute;er groupe";
#elif L==5
	"Crear grupo";		// Okoteve traduccin
#elif L==6
	"Crea gruppo";
#elif L==7
	"Create group";		// Potrzebujesz tlumaczenie
#elif L==8
	"Criar grupo";
#endif

const char *Txt_Create_holiday =
#if   L==0
	"Crear festividad";	// Necessita traduccio
#elif L==1
	"Feiertag eingeben";
#elif L==2
	"Create holiday";
#elif L==3
	"Crear festividad";
#elif L==4
	"Cr&eacute;er f&ecirc;te";
#elif L==5
	"Crear festividad";	// Okoteve traduccin
#elif L==6
	"Crea festivit&agrave;";
#elif L==7
	"Tworzenie wakacje";
#elif L==8
	"Criar f&eacute;ria";
#endif

const char *Txt_Create_institution =
#if   L==0
	"Crear instituci&oacute;n";	// Necessita traduccio
#elif L==1
	"Hochschule eingeben";
#elif L==2
	"Create institution";
#elif L==3
	"Crear instituci&oacute;n";
#elif L==4
	"Cr&eacute;er &eacute;tablissement";
#elif L==5
	"Crear instituci&oacute;n";	// Okoteve traduccin
#elif L==6
	"Crea istituzione";
#elif L==7
	"Tworzenie instytucji";
#elif L==8
	"Criar institu&ccedil;&atilde;o";
#endif

const char *Txt_Create_link =
#if   L==0
	"Crear enlla&ccedil;";
#elif L==1
	"Link erstellen";
#elif L==2
	"Create link";
#elif L==3
	"Crear enlace";
#elif L==4
	"Cr&eacute;er lien";
#elif L==5
	"Crear enlace";	// Okoteve traduccin
#elif L==6
	"Crea link";
#elif L==7
	"Utw&oacute;rz link";
#elif L==8
	"Criar liga&ccedil;&atilde;o";
#endif

const char *Txt_Create_mail_domain =
#if   L==0
	"Crear dominio de correo";	// Necessita traduccio
#elif L==1
	"Mail-Dom&auml;ne anlegen";
#elif L==2
	"Create mail domain";
#elif L==3
	"Crear dominio de correo";
#elif L==4
	"Cr&eacute;er domaine de messagerie";
#elif L==5
	"Crear dominio de correo";	// Okoteve traduccin
#elif L==6
	"Crea campo mail";
#elif L==7
	"Tworzenie domeny mail";
#elif L==8
	"Criar dom&iacute;nio de email";
#endif

const char *Txt_Create_notice =
#if   L==0
	"Crear av&iacute;s";
#elif L==1
	"Erstelle Ank&uuml;ndigung";
#elif L==2
	"Create notice";
#elif L==3
	"Crear aviso";
#elif L==4
	"Cr&eacute;er avis";
#elif L==5
	"Crear aviso";		// Okoteve traduccin
#elif L==6
	"Crea avviso";
#elif L==7
	"Stw&oacute;rz powiadomienie";
#elif L==8
	"Criar aviso";
#endif

const char *Txt_Create_BR_notification =
#if   L==0
	"Crear<br />notificaci&oacute;";
#elif L==1
	"Anmeldung<br />eingeben";
#elif L==2
	"Create<br />notification";
#elif L==3
	"Crear<br />notificaci&oacute;n";
#elif L==4
	"Cr&eacute;er<br />notification";
#elif L==5
	"Crear<br />notificaci&oacute;n";	// Okoteve traduccin
#elif L==6
	"Crea<br />notifica";
#elif L==7
	"Tworzenie<br />powiadomienie";
#elif L==8
	"Criar<br />notifica&ccedil;&atilde;o";
#endif

const char *Txt_Create_password =
#if   L==0
	"Crear contrasenya";
#elif L==1
	"Passwort erstellen";
#elif L==2
	"Crear password";
#elif L==3
	"Cambiar contrase&ntilde;a";
#elif L==4
	"Cr&eacute;er le mot de passe";
#elif L==5
	"Crear contrase&ntilde;a";	// Okoteve traduccin
#elif L==6
	"Cambiare password";
#elif L==7
	"Utw&oacute;rz has&lstrok;o";
#elif L==8
	"Criar senha";
#endif

const char *Txt_Create_place =
#if   L==0
	"Crear lugar";	// Necessita traduccio
#elif L==1
	"Standort eingeben";
#elif L==2
	"Create place";
#elif L==3
	"Crear lugar";
#elif L==4
	"Cr&eacute;er emplacement";
#elif L==5
	"Crear lugar";	// Okoteve traduccin
#elif L==6
	"Crea localit&grave;";
#elif L==7
	"Tworzenie miejsce";
#elif L==8
	"Criar localiza&ccedil;&atilde;o";
#endif

const char *Txt_Create_plugin =
#if   L==0
	"Crear complemento";	// Necessita traduccio
#elif L==1
	"Plugin erstellen";
#elif L==2
	"Create plugin";
#elif L==3
	"Crear complemento";
#elif L==4
	"Cr&eacute;er plugin";
#elif L==5
	"Crear complemento";	// Okoteve traduccin
#elif L==6
	"Crea plugin";
#elif L==7
	"Tworzenie plugin";
#elif L==8
	"Criar plugin";
#endif

const char *Txt_Create_record_field =
#if   L==0
	"Crear campo de ficha";	// Necessita traduccio
#elif L==1
	"Datenfeld erstellen";
#elif L==2
	"Create record field";
#elif L==3
	"Crear campo de ficha";
#elif L==4
	"Cr&eacute;er champ de carte";
#elif L==5
	"Crear campo de ficha";	// Okoteve traduccin
#elif L==6
	"Crea campo di scheda";
#elif L==7
	"Utw&oacute;rz rekord pola";
#elif L==8
	"Criar campo de cart&atilde;o";
#endif

const char *Txt_Create_survey =
#if   L==0
	"Crear enquesta";
#elif L==1
	"Umfrage erstellen";
#elif L==2
	"Create survey";
#elif L==3
	"Crear encuesta";
#elif L==4
	"Cr&eacute;er sondage";
#elif L==5
	"Crear encuesta";	// Okoteve traduccin
#elif L==6
	"Crea sondaggio";
#elif L==7
	"Tworzenie ankiety";
#elif L==8
	"Criar inqu&eacute;rito";
#endif

const char *Txt_Create_type_of_degree =
#if   L==0
	"Crear tipo de titulaci&oacute;n";	// Necessita traduccio
#elif L==1
	"Abschlussart definieren";
#elif L==2
	"Create type of degree";
#elif L==3
	"Crear tipo de titulaci&oacute;n";
#elif L==4
	"Cr&eacute;er type d'&eacute;tude";
#elif L==5
	"Crear tipo de titulaci&oacute;n";	// Okoteve traduccin
#elif L==6
	"Crea tipo di laurea";
#elif L==7
	"Tworzenie typu stopnia";
#elif L==8
	"Criar tipo de titula&ccedil;&atilde;o";
#endif

const char *Txt_Create_type_of_group =
#if   L==0
	"Crear tipo de grupo";	// Necessita traduccio
#elif L==1
	"Gruppen-Typ definieren";
#elif L==2
	"Create type of group";
#elif L==3
	"Crear tipo de grupo";
#elif L==4
	"Cr&eacute;er type de groupe";
#elif L==5
	"Crear tipo de grupo";	// Okoteve traduccin
#elif L==6
	"Crea tipo di gruppo";
#elif L==7
	"Tworzenie typu grupy";
#elif L==8
	"Criar tipo de grupo";
#endif

const char *Txt_Create_ZIP_file =
#if   L==0
	"Crear archivo ZIP";	// Necessita traduccio
#elif L==1
	"ZIP-Datei erstellen";
#elif L==2
	"Create ZIP file";
#elif L==3
	"Crear archivo ZIP";
#elif L==4
	"Cr&eacute;er fichier ZIP";
#elif L==5
	"Crear archivo ZIP";	// Okoteve traduccin
#elif L==6
	"Crea file ZIP";
#elif L==7
	"Tworzenie pliku ZIP";
#elif L==8
	"Criar arquivo ZIP";
#endif

const char *Txt_Created_new_announcement_of_exam =
#if   L==0
	"Creada nueva convocatoria de examen.";	// Necessita traduccio
#elif L==1
	"Neuer Aufruf zur Pr&uuml;fung erstellt.";
#elif L==2
	"Created new announcement of exam.";
#elif L==3
	"Creada nueva convocatoria de examen.";
#elif L==4
	"Cr&eacute;&eacute; nouvelle convocation &agrave; un examen.";
#elif L==5
	"Creada nueva convocatoria de examen.";	// Okoteve traduccin
#elif L==6
	"Creata nuova sessione d'esame.";
#elif L==7
	"Utworzono nowe ogloszenie egzamin.";
#elif L==8
	"Criado nova chamada para exame.";
#endif

const char *Txt_Created_new_assignment_X =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"Creada nueva actividad <strong>%s</strong>.";	// Necessita traduccio
#elif L==1
	"Neue Aufgabe <strong>%s</strong> erstellt.";
#elif L==2
	"Created new assignment <strong>%s</strong>.";
#elif L==3
	"Creada nueva actividad <strong>%s</strong>.";
#elif L==4
	"Cr&eacute;&eacute;e nouvelle activit&eacute; <strong>%s</strong>.";
#elif L==5
	"Creada nueva actividad <strong>%s</strong>.";	// Okoteve traduccin
#elif L==6
	"Creata nuova attivit&agrave; <strong>%s</strong>.";
#elif L==7
	"Utworzono nowa zadania <strong>%s</strong>.";
#elif L==8
	"Criada nova atividade <strong>%s</strong>.";
#endif

const char *Txt_Created_new_banner_X =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"Creado nuevo banner <strong>%s</strong>.";	// Necessita traduccio
#elif L==1
	"Neuer Werbebanner <strong>%s</strong> erstellt.";
#elif L==2
	"Created new banner <strong>%s</strong>.";
#elif L==3
	"Creado nuevo banner <strong>%s</strong>.";
#elif L==4
	"Cr&eacute;&eacute; nouvelle banni&eacute;re <strong>%s</strong>.";
#elif L==5
	"Creado nuevo banner <strong>%s</strong>.";	// Okoteve traduccin
#elif L==6
	"Creato nuovo banner <strong>%s</strong>.";
#elif L==7
	"Utworzono nowy banner <strong>%s</strong>.";
#elif L==8
	"Criado novo banner <strong>%s</strong>.";
#endif

const char *Txt_Created_new_centre_X =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"Creado nuevo centro <strong>%s</strong>.";	// Necessita traduccio
#elif L==1
	"Neues Lehrinstitut <strong>%s</strong> eingegeben.";
#elif L==2
	"Created new centre <strong>%s</strong>.";
#elif L==3
	"Creado nuevo centro <strong>%s</strong>.";
#elif L==4
	"Cr&eacute;&eacute; nouveau centre <strong>%s</strong>.";
#elif L==5
	"Creado nuevo centro <strong>%s</strong>.";	// Okoteve traduccin
#elif L==6
	"Creato nuovo centro <strong>%s</strong>.";
#elif L==7
	"Utworzono nowe centrum <strong>%s</strong>.";
#elif L==8
	"Criado novo centro <strong>%s</strong>.";
#endif

const char *Txt_Created_new_country_X =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"Creado nuevo pa&iacute;s <strong>%s</strong>.";	// Necessita traduccio
#elif L==1
	"Neues Land <strong>%s</strong> eingegeben.";
#elif L==2
	"Created new country <strong>%s</strong>.";
#elif L==3
	"Creado nuevo pa&iacute;s <strong>%s</strong>.";
#elif L==4
	"Cr&eacute;&eacute; nouveau pays <strong>%s</strong>.";
#elif L==5
	"Creado nuevo pa&iacute;s <strong>%s</strong>.";	// Okoteve traduccin
#elif L==6
	"Creato nuovo paese <strong>%s</strong>.";
#elif L==7
	"Utworzono nowa kraju <strong>%s</strong>.";
#elif L==8
	"Criado novo pa&iacute;s <strong>%s</strong>.";
#endif

const char *Txt_Created_new_course_X =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"Creada nueva asignatura <strong>%s</strong>.";	// Necessita traduccio
#elif L==1
	"Neue Kurs <strong>%s</strong> eingegeben.";
#elif L==2
	"Created new course <strong>%s</strong>.";
#elif L==3
	"Creada nueva asignatura <strong>%s</strong>.";
#elif L==4
	"Cr&eacute;&eacute;e nouvelle mati&egrave;re <strong>%s</strong>.";
#elif L==5
	"Creada nueva asignatura <strong>%s</strong>.";	// Okoteve traduccin
#elif L==6
	"Creato nuovo corso <strong>%s</strong>.";
#elif L==7
	"Utworzono nowa oczywiscie <strong>%s</strong>.";
#elif L==8
	"Criada nova disciplina <strong>%s</strong>.";
#endif

const char *Txt_Created_new_degree_X =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"Creada nueva titulaci&oacute;n <strong>%s</strong>.";	// Necessita traduccio
#elif L==1
	"Neuer Studiengang <strong>%s</strong> eingegeben.";
#elif L==2
	"Created new degree <strong>%s</strong>.";
#elif L==3
	"Creada nueva titulaci&oacute;n <strong>%s</strong>.";
#elif L==4
	"Cr&eacute;&eacute;e nouvel &eacute;tude <strong>%s</strong>.";
#elif L==5
	"Creada nueva titulaci&oacute;n <strong>%s</strong>.";	// Okoteve traduccin
#elif L==6
	"Creata nuova laurea <strong>%s</strong>.";
#elif L==7
	"Utworzono nowy poziom <strong>%s</strong>.";
#elif L==8
	"Criada nova titula&ccedil;&atilde;o <strong>%s</strong>.";
#endif

const char *Txt_Created_new_department_X =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"Creado nuevo departamento <strong>%s</strong>.";	// Necessita traduccio
#elif L==1
	"Neue Abteilung <strong>%s</strong> eingegeben.";
#elif L==2
	"Created new department <strong>%s</strong>.";
#elif L==3
	"Creado nuevo departamento <strong>%s</strong>.";
#elif L==4
	"Cr&eacute;&eacute; nouveau d&eacute;partement <strong>%s</strong>.";
#elif L==5
	"Creado nuevo departamento <strong>%s</strong>.";	// Okoteve traduccin
#elif L==6
	"Creato nuovo dipartimento <strong>%s</strong>.";
#elif L==7
	"Utworzono nowy dzia&lstrok; <strong>%s</strong>.";
#elif L==8
	"Criado novo departamento <strong>%s</strong>.";
#endif

const char *Txt_Created_new_event_X =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"Creado nuevo evento <strong>%s</strong>.";	// Necessita traduccio
#elif L==1
	"Neues Ereignis <strong>%s</strong> eingegeben.";
#elif L==2
	"Created new event <strong>%s</strong>.";
#elif L==3
	"Creado nuevo evento <strong>%s</strong>.";
#elif L==4
	"Cr&eacute;&eacute; nouveau &eacute;v&eacute;nement <strong>%s</strong>.";
#elif L==5
	"Creado nuevo evento <strong>%s</strong>.";	// Okoteve traduccin
#elif L==6
	"Creato nuovo evento <strong>%s</strong>.";
#elif L==7
	"Utworzono nowe wydarzenie <strong>%s</strong>.";
#elif L==8
	"Criado novo evento <strong>%s</strong>.";
#endif

const char *Txt_Created_new_group_X =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"Creado nuevo grupo <strong>%s</strong>.";	// Necessita traduccio
#elif L==1
	"Neue Gruppe <strong>%s</strong> erstellt.";
#elif L==2
	"Created new group <strong>%s</strong>.";
#elif L==3
	"Creado nuevo grupo <strong>%s</strong>.";
#elif L==4
	"Cr&eacute;&eacute; nouveau groupe <strong>%s</strong>.";
#elif L==5
	"Creado nuevo grupo <strong>%s</strong>.";	// Okoteve traduccin
#elif L==6
	"Creato nuovo gruppo <strong>%s</strong>.";
#elif L==7
	"Stworzyl nowa grupe <strong>%s</strong>.";
#elif L==8
	"Criado novo grupo <strong>%s</strong>.";
#endif

const char *Txt_Created_new_holiday_X =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"Creada nueva festividad <strong>%s</strong>.";	// Necessita traduccio
#elif L==1
	"Neuer Feiertag <strong>%s</strong> eingegeben.";
#elif L==2
	"Created new holiday <strong>%s</strong>.";
#elif L==3
	"Creada nueva festividad <strong>%s</strong>.";
#elif L==4
	"Cr&eacute;&eacute; nouvelle f&ecirc;te <strong>%s</strong>.";
#elif L==5
	"Creada nueva festividad <strong>%s</strong>.";	// Okoteve traduccin
#elif L==6
	"Creata nuova festivit&agrave; <strong>%s</strong>.";
#elif L==7
	"Utworzono nowe swieto <strong>%s</strong>.";
#elif L==8
	"Criado nova f&eacute;ria <strong>%s</strong>.";
#endif

const char *Txt_Created_new_institution_X =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"Creada nueva instituci&oacute;n <strong>%s</strong>.";	// Necessita traduccio
#elif L==1
	"Neue Hochschule <strong>%s</strong> eingegeben.";
#elif L==2
	"Created new institution <strong>%s</strong>.";
#elif L==3
	"Creada nueva instituci&oacute;n <strong>%s</strong>.";
#elif L==4
	"Cr&eacute;&eacute;e nouvel &eacute;tablissement <strong>%s</strong>.";
#elif L==5
	"Creada nueva instituci&oacute;n <strong>%s</strong>.";	// Okoteve traduccin
#elif L==6
	"Creata nuova istituzione <strong>%s</strong>.";
#elif L==7
	"Utworzono nowa instytucje <strong>%s</strong>.";
#elif L==8
	"Criada nova institu&ccedil;&atilde;o <strong>%s</strong>.";
#endif

const char *Txt_Created_new_link_X =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"Creado nuevo enlace <strong>%s</strong>.";	// Necessita traduccio
#elif L==1
	"Neuer Link <strong>%s</strong> erstellt.";
#elif L==2
	"Created new link <strong>%s</strong>.";
#elif L==3
	"Creado nuevo enlace <strong>%s</strong>.";
#elif L==4
	"Cr&eacute;&eacute; nouveau lien <strong>%s</strong>.";
#elif L==5
	"Creado nuevo enlace <strong>%s</strong>.";	// Okoteve traduccin
#elif L==6
	"Creato nuovo link <strong>%s</strong>.";
#elif L==7
	"Utworzono nowy link <strong>%s</strong>.";
#elif L==8
	"Criado nova liga&ccedil;&atilde;o <strong>%s</strong>.";
#endif

const char *Txt_Created_new_mail_domain_X =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"Creado nuevo dominio de correo <strong>%s</strong>.";	// Necessita traduccio
#elif L==1
	"Neue Mail-Dom&auml;ne <strong>%s</strong> angelegt.";
#elif L==2
	"Created new mail domain <strong>%s</strong>.";
#elif L==3
	"Creado nuevo dominio de correo <strong>%s</strong>.";
#elif L==4
	"Cr&eacute;&eacute; nouvel domaine de messagerie <strong>%s</strong>.";
#elif L==5
	"Creado nuevo dominio de correo <strong>%s</strong>.";	// Okoteve traduccin
#elif L==6
	"Creato nuovo campo mail <strong>%s</strong>.";
#elif L==7
	"Utworzono nowa domena mail <strong>%s</strong>.";
#elif L==8
	"Criado novo dom&iacute;nio de email <strong>%s</strong>.";
#endif

const char *Txt_Created_new_place_X =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"Creado nuevo lugar <strong>%s</strong>.";	// Necessita traduccio
#elif L==1
	"Neuer Standort <strong>%s</strong> eingegeben.";
#elif L==2
	"Created new place <strong>%s</strong>.";
#elif L==3
	"Creado nuevo lugar <strong>%s</strong>.";
#elif L==4
	"Cr&eacute;&eacute; nouvel emplacement <strong>%s</strong>.";
#elif L==5
	"Creado nuevo lugar <strong>%s</strong>.";	// Okoteve traduccin
#elif L==6
	"Creata nuova localit&grave; <strong>%s</strong>.";
#elif L==7
	"Utworzono nowe miejsce <strong>%s</strong>.";
#elif L==8
	"Criado nova localiza&ccedil;&atilde;o <strong>%s</strong>.";
#endif

const char *Txt_Created_new_plugin_X =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"Creado nuevo complemento <strong>%s</strong>.";	// Necessita traduccio
#elif L==1
	"Neuer Plugin <strong>%s</strong> erstellt.";
#elif L==2
	"Created new plugin <strong>%s</strong>.";
#elif L==3
	"Creado nuevo complemento <strong>%s</strong>.";
#elif L==4
	"Cr&eacute;&eacute; nouveau plugin <strong>%s</strong>.";
#elif L==5
	"Creado nuevo complemento <strong>%s</strong>.";	// Okoteve traduccin
#elif L==6
	"Creato nuovo plugin <strong>%s</strong>.";
#elif L==7
	"Utworzono nowy plugin <strong>%s</strong>.";
#elif L==8
	"Criado novo plugin <strong>%s</strong>.";
#endif

const char *Txt_Created_new_record_field_X =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"Creado nuevo campo de ficha <strong>%s</strong>.";	// Necessita traduccio
#elif L==1
	"Neuer Datenfeld <strong>%s</strong> erstellt.";
#elif L==2
	"Created new record field <strong>%s</strong>.";
#elif L==3
	"Creado nuevo campo de ficha <strong>%s</strong>.";
#elif L==4
	"Cr&eacute;&eacute; nouveau champ de carte <strong>%s</strong>.";
#elif L==5
	"Creado nuevo campo de ficha <strong>%s</strong>.";	// Okoteve traduccin
#elif L==6
	"Creato nuovo campo di scheda <strong>%s</strong>.";
#elif L==7
	"Utworzono nowe pola rekordu <strong>%s</strong>.";
#elif L==8
	"Created new record field <strong>%s</strong>.";	// Necessita de traduo
#endif

const char *Txt_Created_new_survey_X =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"Creada nueva encuesta <strong>%s</strong>.";	// Necessita traduccio
#elif L==1
	"Neuer Umfrage <strong>%s</strong> erstellt.";
#elif L==2
	"Created new survey <strong>%s</strong>.";
#elif L==3
	"Creada nueva encuesta <strong>%s</strong>.";
#elif L==4
	"Cr&eacute;&eacute; nouveau sondage <strong>%s</strong>.";
#elif L==5
	"Creada nueva encuesta <strong>%s</strong>.";	// Okoteve traduccin
#elif L==6
	"Creato nuovo sondaggio <strong>%s</strong>.";
#elif L==7
	"Utworzono nowe badania <strong>%s</strong>.";
#elif L==8
	"Criado novo inqu&eacute;rito <strong>%s</strong>.";
#endif

const char *Txt_Created_new_type_of_degree_X =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"Creado nuevo tipo de titulaci&oacute;n <strong>%s</strong>.";	// Necessita traduccio
#elif L==1
	"Neue Abschlussart <strong>%s</strong> definiert.";
#elif L==2
	"Created new type of degree <strong>%s</strong>.";
#elif L==3
	"Creado nuevo tipo de titulaci&oacute;n <strong>%s</strong>.";
#elif L==4
	"Cr&eacute;&eacute; nouveau type d'&eacute;tude <strong>%s</strong>.";
#elif L==5
	"Creado nuevo tipo de titulaci&oacute;n <strong>%s</strong>.";	// Okoteve traduccin
#elif L==6
	"Creato nuovo tipo di laurea <strong>%s</strong>.";
#elif L==7
	"Stworzyl nowy typ stopnia <strong>%s</strong>.";
#elif L==8
	"Criado novo tipo de titula&ccedil;&atilde;o <strong>%s</strong>.";
#endif

const char *Txt_Created_new_type_of_group_X =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"Creado nuevo tipo de grupo <strong>%s</strong>.";	// Necessita traduccio
#elif L==1
	"Neuer Gruppen-Typ <strong>%s</strong> definiert.";
#elif L==2
	"Created new type of group <strong>%s</strong>.";
#elif L==3
	"Creado nuevo tipo de grupo <strong>%s</strong>.";
#elif L==4
	"Cr&eacute;&eacute; nouveau type de groupe <strong>%s</strong>.";
#elif L==5
	"Creado nuevo tipo de grupo <strong>%s</strong>.";	// Okoteve traduccin
#elif L==6
	"Creato nuovo tipo di gruppo <strong>%s</strong>.";
#elif L==7
	"Stworzyl nowy typ grupy <strong>%s</strong>.";
#elif L==8
	"Criado novo tipo de grupo <strong>%s</strong>.";
#endif

const char *Txt_Created_tables_in_the_database_that_did_not_exist =
#if   L==0
	"Creadas las tablas de la base de datos que no exist&iacute;an.";	// Necessita traduccio
#elif L==1
	"Neue Tabellen in der Datenbank erstellt.";
#elif L==2
	"Created tables in the database that did not exist.";
#elif L==3
	"Creadas las tablas de la base de datos que no exist&iacute;an.";
#elif L==4
	"Tables cr&eacute;&eacute;es dans la base de donn&eacute;es qui n'existaient pas.";
#elif L==5
	"Creadas las tablas de la base de datos que no exist&iacute;an.";	// Okoteve traduccin
#elif L==6
	"Tabelle create in un database inesistente.";
#elif L==7
	"Wykonawca tabel w bazie danych, kt&oacute;re nie istnialy.";
#elif L==8
	"Criadas tabelas no banco de dados que n&atilde;o existiam.";
#endif

const char *Txt_Creating_database_tables_if_they_do_not_exist =
#if   L==0
	"Creando tablas de la base de datos si no existen...";	// Necessita traduccio
#elif L==1
	"Creating database tables if they do not exist...";	// Need bersetzung
#elif L==2
	"Creating database tables if they do not exist...";
#elif L==3
	"Creando tablas de la base de datos si no existen...";
#elif L==4
	"Creating database tables if they do not exist...";	// Besoin de traduction
#elif L==5
	"Creando tablas de la base de datos si no existen...";	// Okoteve traduccin
#elif L==6
	"Creazione tabelle della base di dati se non esistono...";
#elif L==7
	"Tworzenie tabel bazy danych, jesli ich nie ma ...";
#elif L==8
	"Creating database tables if they do not exist...";	// Necessita de traduo
#endif

const char *Txt_crs =	// Abbreviation of "course" (three characters + dot)
#if   L==0
	"asg.";		// Necessita traduccio
#elif L==1
	"Ver.";
#elif L==2
	"crs.";
#elif L==3
	"asg.";
#elif L==4
	"mat.";
#elif L==5
	"mbo'esyry";
#elif L==6
	"crs.";
#elif L==7
	"Oczywiscie.";
#elif L==8
	"crs.";
#endif

const char *Txt_Current_email =
#if   L==0
	"Correu electr&ograve;nic actual";
#elif L==1
	"Aktueller E-Mail";
#elif L==2
	"Current e-mail";
#elif L==3
	"Correo electr&oacute;nico actual";
#elif L==4
	"Courrier &eacute;lectronique actuel";
#elif L==5
	"Correo electr&oacute;nico actual";	// Okoteve traduccin
#elif L==6
	"E-mail corrente";
#elif L==7
	"Obecny e-mail";
#elif L==8
	"E-mail atual";
#endif

const char *Txt_Current_nickname =
#if   L==0
	"Sobrenom actual";
#elif L==1
	"Aktueller Benutzername";
#elif L==2
	"Current nickname";
#elif L==3
	"Apodo actual";
#elif L==4
	"Surnom actuel";
#elif L==5
	"Apodo actual";	// Okoteve traduccin
#elif L==6
	"Nome utente corrente";
#elif L==7
	"Obecny pseudonim";
#elif L==8
	"Alcunha atual";
#endif

const char *Txt_Current_password =
#if   L==0
	"Contrasenya actual";
#elif L==1
	"Aktuelles Passwort";
#elif L==2
	"Current password";
#elif L==3
	"Contrase&ntilde;a actual";
#elif L==4
	"Mot de passe actuel";
#elif L==5
	"Contrase&ntilde;a actual";	// Okoteve traduccin
#elif L==6
	"Password corrente";
#elif L==7
	"Obecne has&lstrok;o";
#elif L==8
	"Senha atual";
#endif

const char *Txt_DATA_PROTECTION_CLAUSE =
#if   L==0
	"Pol&iacute;tica de protecci&oacute;n de datos";	// Necessita traduccio
#elif L==1
	"Pol&iacute;tica de protecci&oacute;n de datos";	// Need bersetzung
#elif L==2
	"Data protection clause";
#elif L==3
	"Pol&iacute;tica de protecci&oacute;n de datos";
#elif L==4
	"Clause de protection des donn&eacute;es";
#elif L==5
	"Pol&iacute;tica de protecci&oacute;n de datos";	// Okoteve traduccin
#elif L==6
	"Pol&iacute;tica de protecci&oacute;n de datos";	// Bisogno di traduzione
#elif L==7
	"Pol&iacute;tica de protecci&oacute;n de datos";	// Potrzebujesz tlumaczenie
#elif L==8
	"Pol&iacute;tica de protecci&oacute;n de datos";	// Necessita de traduo
#endif

const char *Txt_Date =
#if   L==0
	"Data";
#elif L==1
	"Datum";
#elif L==2
	"Date";
#elif L==3
	"Fecha";
#elif L==4
	"Date";
#elif L==5
	"Arange";
#elif L==6
	"Data";
#elif L==7
	"Data";
#elif L==8
	"Data";
#endif

const char *Txt_Date_of_birth =
#if   L==0
	"Data naixement";
#elif L==1
	"Geburtsdatum";
#elif L==2
	"Date of birth";
#elif L==3
	"Fecha nacimiento";
#elif L==4
	"Date naissance";
#elif L==5
	"Fecha nacimiento";	// Okoteve traduccin
#elif L==6
	"Data di nascita";
#elif L==7
	"Data urodzenia";
#elif L==8
	"Data nascimento";
#endif

const char *Txt_Date_of_creation =
#if   L==0
	"Data de creaci&oacute;";
#elif L==1
	"Datum der Erstellung";
#elif L==2
	"Date of creation";
#elif L==3
	"Fecha de creaci&oacute;n";
#elif L==4
	"Date de cr&eacute;ation";
#elif L==5
	"Fecha de creaci&oacute;n";	// Okoteve traduccin
#elif L==6
	"Data di creazione";
#elif L==7
	"Data utworzenia";
#elif L==8
	"Data da cria&ccedil;&atilde;o";
#endif

const char *Txt_Dates =
#if   L==0
	"Dates";
#elif L==1
	"Datum";
#elif L==2
	"Dates";
#elif L==3
	"Fechas";
#elif L==4
	"Dates";
#elif L==5
	"Fechas";	// Okoteve traduccin
#elif L==6
	"Date";
#elif L==7
	"Daty";
#elif L==8
	"Datas";
#endif

const char *Txt_DATES_RANGE =	// Warning: it is very important to include two %s in the following sentences
#if   L==0
	"desde el %s hasta el %s";	// Necessita traduccio
#elif L==1
	"von %s bis %s";
#elif L==2
	"from %s to %s";
#elif L==3
	"desde el %s hasta el %s";
#elif L==4
	"de %s jusqu'&agrave; %s";
#elif L==5
	"desde el %s hasta el %s";	// Okoteve traduccin
#elif L==6
	"dal %s al %s";
#elif L==7
	"z %s do %s";
#elif L==8
	"%s at&eacute; %s";
#endif

const char *Txt_Day =
#if   L==0
	"D&iacute;a";	// Necessita traduccio
#elif L==1
	"Tag";
#elif L==2
	"Day";
#elif L==3
	"D&iacute;a";
#elif L==4
	"Jour";
#elif L==5
	"D&iacute;a";	// Okoteve traduccin
#elif L==6
	"Giorno";
#elif L==7
	"Dzien";
#elif L==8
	"Dia";
#endif

const char *Txt_days =
#if   L==0
	"d&iacute;as";	// Necessita traduccio
#elif L==1
	"Tage";
#elif L==2
	"days";
#elif L==3
	"d&iacute;as";
#elif L==4
	"jours";
#elif L==5
	"d&iacute;as";	// Okoteve traduccin
#elif L==6
	"giorni";
#elif L==7
	"dni";
#elif L==8
	"dias";
#endif

const char *Txt_DAYS_CAPS[7] =
   {
#if   L==0
	 "DILLUNS"
#elif L==1
	 "MONTAG"
#elif L==2
	 "MONDAY"
#elif L==3
	 "LUNES"
#elif L==4
	 "LUNDI"
#elif L==5
	 "ARAKOI"
#elif L==6
	 "LUNED&Igrave;"
#elif L==7
	 "Poniedzialek"
#elif L==8
	 "SEGUNDA-FEIRA"
#endif
	 ,
#if   L==0
	 "DIMARTS"
#elif L==1
	 "DIENSTAG"
#elif L==2
	 "TUESDAY"
#elif L==3
	 "MARTES"
#elif L==4
	 "MARDI"
#elif L==5
	 "ARAAPY"
#elif L==6
	 "MARTED&Igrave;"
#elif L==7
	 "Wtorek"
#elif L==8
	 "TER&Ccedil;A-FEIRA"
#endif
	 ,
#if   L==0
	 "DIMECRES"
#elif L==1
	 "MITTWOCH"
#elif L==2
	 "WEDNESDAY"
#elif L==3
	 "MI&Eacute;RCOLES"
#elif L==4
	 "MERCREDI"
#elif L==5
	 "ARARUNDY"
#elif L==6
	 "MERCOLED&Igrave;"
#elif L==7
	 "Sroda"
#elif L==8
	 "QUARTA-FEIRA"
#endif
	 ,
#if   L==0
	 "DIJOUS"
#elif L==1
	 "DONNERSTAG"
#elif L==2
	 "THURSDAY"
#elif L==3
	 "JUEVES"
#elif L==4
	 "JEUDI"
#elif L==5
	 "ARAPO"
#elif L==6
	 "GIOVED&Igrave;"
#elif L==7
	 "Czwartek"
#elif L==8
	 "QUINTA-FEIRA"
#endif
	 ,
#if   L==0
	 "DIVENDRES"
#elif L==1
	 "FREITAG"
#elif L==2
	 "FRIDAY"
#elif L==3
	 "VIERNES"
#elif L==4
	 "VENDREDI"
#elif L==5
	 "ARAPOTE&#296;"
#elif L==6
	 "VENERD&Igrave;"
#elif L==7
	 "Piatek"
#elif L==8
	 "SEXTA-FEIRA"
#endif
	 ,
#if   L==0
	 "DISSABTE"
#elif L==1
	 "SAMSTAG"
#elif L==2
	 "SATURDAY"
#elif L==3
	 "S&Aacute;BADO"
#elif L==4
	 "SAMEDI"
#elif L==5
	 "ARAPOK&Otilde;I"
#elif L==6
	 "SABATO"
#elif L==7
	 "Sobota"
#elif L==8
	 "SABADO"
#endif
	 ,
#if   L==0
	 "DIUMENGE"
#elif L==1
	 "SONNTAG"
#elif L==2
	 "SUNDAY"
#elif L==3
	 "DOMINGO"
#elif L==4
	 "DIMANCHE"
#elif L==5
	 "ARATE&#296;"
#elif L==6
	 "DOMENICA"
#elif L==7
	 "Niedziela"
#elif L==8
	 "DOMINGO"
#endif
	};

const char *Txt_DAYS_SMALL[7] =
   {
#if   L==0
	 "dilluns"
#elif L==1
	 "Montag"
#elif L==2
	 "Monday"
#elif L==3
	 "lunes"
#elif L==4
	 "Lundi"
#elif L==5
	 "arak&otilde;i"
#elif L==6
	 "Luned&igrave;"
#elif L==7
	 "Poniedzialek"
#elif L==8
	 "Segunda-feira"
#endif
	 ,
#if   L==0
	 "dimarts"
#elif L==1
	 "Dienstag"
#elif L==2
	 "Tuesday"
#elif L==3
	 "martes"
#elif L==4
	 "Mardi"
#elif L==5
	 "araapy"
#elif L==6
	 "Marted&igrave;"
#elif L==7
	 "Wtorek"
#elif L==8
	 "Ter&ccedil;a-feira"
#endif
	 ,
#if   L==0
	 "dimecres"
#elif L==1
	 "Mittwoch"
#elif L==2
	 "Wednesday"
#elif L==3
	 "mi&eacute;rcoles"
#elif L==4
	 "Mercredi"
#elif L==5
	 "ararundy"
#elif L==6
	 "Mercoled&igrave;"
#elif L==7
	 "Sroda"
#elif L==8
	 "Quarta-feira"
#endif
	 ,
#if   L==0
	 "dijous"
#elif L==1
	 "Donnerstag"
#elif L==2
	 "Thursday"
#elif L==3
	 "jueves"
#elif L==4
	 "Jeudi"
#elif L==5
	 "arapo"
#elif L==6
	 "Gioved&igrave;"
#elif L==7
	 "Czwartek"
#elif L==8
	 "Quinta-feira"
#endif
	 ,
#if   L==0
	 "divendres"
#elif L==1
	 "Freitag"
#elif L==2
	 "Friday"
#elif L==3
	 "viernes"
#elif L==4
	 "Vendredi"
#elif L==5
	 "arapote&#297;"
#elif L==6
	 "Venerd&igrave;"
#elif L==7
	 "Piatek"
#elif L==8
	 "Sexta-feira"
#endif
	 ,
#if   L==0
	 "dissabte"
#elif L==1
	 "Samstag"
#elif L==2
	 "Saturday"
#elif L==3
	 "s&aacute;bado"
#elif L==4
	 "Samedi"
#elif L==5
	 "arapok&otilde;i"
#elif L==6
	 "Sabato"
#elif L==7
	 "Sobota"
#elif L==8
	 "Sabado"
#endif
	 ,
#if   L==0
	 "diumenge"
#elif L==1
	 "Sonntag"
#elif L==2
	 "Sunday"
#elif L==3
	 "domingo"
#elif L==4
	 "Dimanche"
#elif L==5
	 "arate&#297;"
#elif L==6
	 "Domenica"
#elif L==7
	 "Niedziela"
#elif L==8
	 "Domingo"
#endif
	};

const char *Txt_Decrease_level_of_X =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"Disminuir nivel de %s";	// Necessita traduccio
#elif L==1
	"Decrease level of %s";		// Need bersetzung
#elif L==2
	"Decrease level of %s";
#elif L==3
	"Disminuir nivel de %s";
#elif L==4
	"Decrease level of %s";		// Besoin de traduction
#elif L==5
	"Disminuir nivel de %s";	// Okoteve traduccin
#elif L==6
	"Decrementare livello di %s";
#elif L==7
	"Spadek poziomu %s";
#elif L==8
	"Decrease level of %s";		// Necessita de traduo
#endif

const char *Txt_default =
#if   L==0
	"predeterminado";	// Necessita traduccio
#elif L==1
	"Vorgabe";
#elif L==2
	"default";
#elif L==3
	"predeterminado";
#elif L==4
	"d&eacute;faut";
#elif L==5
	"predeterminado";	// Okoteve traduccin
#elif L==6
	"predefinito";
#elif L==7
	"Domyslnie";
#elif L==8
	"padr&atilde;o";
#endif

const char *Txt_Degree =
#if   L==0
	"Titulaci&oacute;";
#elif L==1
	"Studiengang";
#elif L==2
	"Degree";
#elif L==3
	"Titulaci&oacute;n";
#elif L==4
	"&Eacute;tude";
#elif L==5
	"Arandur&atilde;";
#elif L==6
	"Laurea";
#elif L==7
	"Stopien";
#elif L==8
	"Titula&ccedil;&atilde;o";
#endif

const char *Txt_degree =
#if   L==0
	"titulaci&oacute;";
#elif L==1
	"Studiengang";
#elif L==2
	"degree";
#elif L==3
	"titulaci&oacute;n";
#elif L==4
	"&eacute;tude";
#elif L==5
	"arandur&atilde;";
#elif L==6
	"laurea";
#elif L==7
	"stopien";
#elif L==8
	"titula&ccedil;&atilde;o";
#endif

const char *Txt_DEGREE_STATUS[Deg_NUM_STATUS_TXT] =
   {
#if   L==0	// Deg_STATUS_UNKNOWN
	 "Desconegudo"
#elif L==1
	 "Unbekannt"
#elif L==2
	 "Unknown"
#elif L==3
	 "Desconocido"
#elif L==4
	 "Inconnu"
#elif L==5
	 "Desconocido"			// Okoteve traduccin
#elif L==6
	 "Sconosciuto"
#elif L==7
	 "Nieznany"
#elif L==8
	 "Desconhecido"
#endif
	 ,
#if   L==0	// Deg_STATUS_ACTIVE
	 "Activa"
#elif L==1
	 "Aktive"
#elif L==2
	 "Active"
#elif L==3
	 "Activa"
#elif L==4
	 "Actif"
#elif L==5
	 "Activa"	// Okoteve traduccin
#elif L==6
	 "Attiva"
#elif L==7
	 "Aktywne"
#elif L==8
	 "Ativa"
#endif
	 ,
#if   L==0	// Deg_STATUS_PENDING
	 "Pendent"
#elif L==1
	 "Offen"
#elif L==2
	 "Pending"
#elif L==3
	 "Pendiente"
#elif L==4
	 "En attente"
#elif L==5
	 "Pendiente"	// Okoteve traduccin
#elif L==6
	 "In attesa"
#elif L==7
	 "Do czasu"
#elif L==8
	 "Pendente"
#endif
	 ,
#if   L==0	// Deg_STATUS_REMOVED
	 "Eliminada"
#elif L==1
	 "Entfernt"
#elif L==2
	 "Removed"
#elif L==3
	 "Eliminada"
#elif L==4
	 "Supprim&eacute;"
#elif L==5
	 "Eliminada"	// Okoteve traduccin
#elif L==6
	 "Rimossa"
#elif L==7
	 "Usuniete"
#elif L==8
	 "Suprimida"
#endif
	};

const char *Txt_DEGREE_With_courses =
#if   L==0
	"Amb assignatures";
#elif L==1
	"Mit Kursen";
#elif L==2
	"With courses";
#elif L==3
	"Con asignaturas";
#elif L==4
	"Avec des mati&egrave;res";
#elif L==5
	"Con asignaturas";	// Okoteve traduccin
#elif L==6
	"Con corsi";
#elif L==7
	"Z kurs&oacute;w";
#elif L==8
	"Com disciplinas";
#endif

const char *Txt_DEGREE_With_year_for_optional_courses =
#if   L==0
	"Con curso de optativas.";			// Necessita traduccio
#elif L==1
	"With year for optional courses.";		// Need bersetzung
#elif L==2
	"With year for optional courses.";
#elif L==3
	"La titulaci&oacute;n ya no tiene un curso de optativas.";
#elif L==4
	"With year for optional courses.";		// Besoin de traduction
#elif L==5
	"La titulaci&oacute;n ya no tiene un curso de optativas.";	// Okoteve traduccin
#elif L==6
	"Con anno per corsi opzionali.";
#elif L==7
	"With year for optional courses.";	// Potrzebujesz tlumaczenie
#elif L==8
	"With year for optional courses.";		// Necessita de traduo
#endif

const char *Txt_DEGREE_Without_courses =
#if   L==0
	"Sense assignatures";
#elif L==1
	"Ohne Kursen";
#elif L==2
	"Without courses";
#elif L==3
	"Sin asignaturas";
#elif L==4
	"Sans mati&egrave;res";
#elif L==5
	"Sin asignaturas";	// Okoteve traduccin
#elif L==6
	"Senza corsi";
#elif L==7
	"Bez kurs&oacute;w";
#elif L==8
	"Sem disciplinas";
#endif

const char *Txt_DEGREE_Without_year_for_optional_courses =
#if   L==0
	"Sin curso de optativas.";					// Necessita traduccio
#elif L==1
	"Without year for optional courses.";				// Need bersetzung
#elif L==2
	"Without year for optional courses.";
#elif L==3
	"La titulaci&oacute;n ya no tiene un curso de optativas.";
#elif L==4
	"Without year for optional courses.";				// Besoin de traduction
#elif L==5
	"La titulaci&oacute;n ya no tiene un curso de optativas.";	// Okoteve traduccin
#elif L==6
	"Senza anno per corsi opzionali.";
#elif L==7
	"Without year for optional courses.";				// Potrzebujesz tlumaczenie
#elif L==8
	"Without year for optional courses.";				// Necessita de traduo
#endif

const char *Txt_Degree_X_removed =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"Titulaci&oacute;n <strong>%s</strong> eliminada.";	// Necessita traduccio
#elif L==1
	"Studiengang <strong>%s</strong> entfernt.";
#elif L==2
	"Degree <strong>%s</strong> removed.";
#elif L==3
	"Titulaci&oacute;n <strong>%s</strong> eliminada.";
#elif L==4
	"Degree <strong>%s</strong> removed.";			// Besoin de traduction
#elif L==5
	"Titulaci&oacute;n <strong>%s</strong> eliminada.";	// Okoteve traduccin
#elif L==6
	"Laurea <strong>%s</strong> rimossa.";
#elif L==7
	"Studia <strong>%s</strong> usuniete.";
#elif L==8
	"Degree <strong>%s</strong> removed.";			// Necessita de traduo
#endif

const char *Txt_Degrees =
#if   L==0
	"Titulacions";
#elif L==1
	"Studiengang";
#elif L==2
	"Degrees";
#elif L==3
	"Titulaciones";
#elif L==4
	"&Eacute;tudes";
#elif L==5
	"Arandur&atilde;";
#elif L==6
	"Lauree";
#elif L==7
	"Stopnie";
#elif L==8
	"Titula&ccedil;oes";
#endif

const char *Txt_Degrees_ABBREVIATION =
#if   L==0
	"Titul.";
#elif L==1
	"Studieng.";
#elif L==2
	"Degrees";
#elif L==3
	"Titul.";
#elif L==4
	"&Eacute;tudes";
#elif L==5
	"Titul.";	// Okoteve traduccin
#elif L==6
	"Lauree";
#elif L==7
	"Stopni";
#elif L==8
	"Titul.";
#endif

const char *Txt_degrees =
#if   L==0
	"titulacions";
#elif L==1
	"Studiengang";
#elif L==2
	"degrees";
#elif L==3
	"titulaciones";
#elif L==4
	"&eacute;tudes";
#elif L==5
	"arandur&atilde;";
#elif L==6
	"lauree";
#elif L==7
	"stopnie";
#elif L==8
	"titula&ccedil;oes";
#endif

const char *Txt_Degrees_of_CENTRE_X =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"Titulacions de %s";
#elif L==1
	"Studiengang von %s";
#elif L==2
	"Degrees of %s";
#elif L==3
	"Titulaciones de %s";
#elif L==4
	"&Eacute;tudes de %s";
#elif L==5
	"Titulaciones de %s";	// Okoteve traduccin
#elif L==6
	"Lauree di %s";
#elif L==7
	"Stopnie %s";
#elif L==8
	"Titula&ccedil;oes de %s";
#endif

const char *Txt_Degrees_with_pending_courses =
#if   L==0
	"Titulacions amb assignatures pendents d'activar";
#elif L==1
	"Studiengang mit anh&auml;ngig Kursen";
#elif L==2
	"Degrees with pending courses";
#elif L==3
	"Titulaciones con asignaturas pendientes de activar";
#elif L==4
	"&Eacute;tudes avec des mati&egrave;res en attente";
#elif L==5
	"Titulaciones con asignaturas pendientes de activar";	// Okoteve traduccin
#elif L==6
	"Lauree con corsi in sospeso";
#elif L==7
	"Stopnie z kurs&otilde;w oczekuj&aogon;cy";
#elif L==8
	"Titula&ccedil;&atilde;os com disciplinas pendentes";
#endif

const char *Txt_Delete_message =
#if   L==0
	"Eliminar mensaje";	// Necessita traduccio
#elif L==1
	"Entfernen Nachricht";
#elif L==2
	"Delete message";
#elif L==3
	"Eliminar mensaje";
#elif L==4
	"Supprimer message";
#elif L==5
	"Eliminar mensaje";	// Okoteve traduccin
#elif L==6
	"Rimuovere messaggio";
#elif L==7
	"Usu&nacute; wiadomosc";
#elif L==8
	"Remover mensagem";
#endif

const char *Txt_Delete_messages_received =
#if   L==0
	"Eliminar missatges rebuts";
#elif L==1
	"Entfernen empfangene Nachrichten";
#elif L==2
	"Delete messages received";
#elif L==3
	"Eliminar mensajes recibidos";
#elif L==4
	"&Eacute;liminer messages re&ccedil;us";
#elif L==5
	"Eliminar mensajes recibidos";	// Okoteve traduccin
#elif L==6
	"Rimuovere messaggi ricevuti";
#elif L==7
	"Usu&nacute; wiadomo&sacute;ci otrzymane";
#elif L==8
	"Eliminar mensagens recebidas";
#endif

const char *Txt_Delete_messages_sent =
#if   L==0
	"Eliminar mensajes enviados";	// Necessita traduccio
#elif L==1
	"Entfernen gesendete Nachrichten";
#elif L==2
	"Delete messages sent";
#elif L==3
	"Eliminar mensajes enviados";
#elif L==4
	"&Eacute;liminer messages envoy&eacute;s";
#elif L==5
	"Eliminar mensajes enviados";	// Okoteve traduccin
#elif L==6
	"Rimuovere messaggi inviati";
#elif L==7
	"Usu&nacute; wiadomosci wyslanych";
#elif L==8
	"Eliminar mensagens enviadas";
#endif

const char *Txt_Department =
#if   L==0
	"Departament";
#elif L==1
	"Abteilung";
#elif L==2
	"Department";
#elif L==3
	"Departamento";
#elif L==4
	"D&eacute;partement";
#elif L==5
	"Peh&#7869;";
#elif L==6
	"Dipartimento";
#elif L==7
	"Wydzia&lstrok;";
#elif L==8
	"Departamento";
#endif

const char *Txt_Department_X_removed =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"Departamento <strong>%s</strong> eliminado.";	// Necessita traduccio
#elif L==1
	"Abteilung <strong>%s</strong> entfernt.";
#elif L==2
	"Department <strong>%s</strong> removed.";
#elif L==3
	"Departamento <strong>%s</strong> eliminado.";
#elif L==4
	"D&eacute;partement <strong>%s</strong> supprim&eacute;.";
#elif L==5
	"Departamento <strong>%s</strong> eliminado.";	// Okoteve traduccin
#elif L==6
	"Dipartamento <strong>%s</strong> rimosso.";
#elif L==7
	"Zaklad <strong>%s</strong> usuniete.";
#elif L==8
	"Department <strong>%s</strong> removed.";	// Necessita de traduo
#endif

const char *Txt_Department_unspecified =
#if   L==0
	"Departament sense especificar";
#elif L==1
	"Abteilung nicht angegeben";
#elif L==2
	"Department unspecified";
#elif L==3
	"Departamento sin especificar";
#elif L==4
	"D&eacute;partement non sp&eacute;cifi&eacute;";
#elif L==5
	"Departamento sin especificar";	// Okoteve traduccin
#elif L==6
	"Dipartimento non specificato";
#elif L==7
	"Departament nieokreslone";
#elif L==8
	"Departamento indeterminado";
#endif

const char *Txt_Departments =
#if   L==0
	"Departaments";
#elif L==1
	"Abteilungen";
#elif L==2
	"Departments";
#elif L==3
	"Departamentos";
#elif L==4
	"D&eacute;partements";
#elif L==5
	"Peh&#7869;";
#elif L==6
	"Dipartimenti";
#elif L==7
	"Wydzia&lstrok;y";
#elif L==8
	"Departamentos";
#endif

const char *Txt_Departments_ABBREVIATION =
#if   L==0
	"Depar.";
#elif L==1
	"Abteilung";
#elif L==2
	"Depar";
#elif L==3
	"Depar.";
#elif L==4
	"Depar.";
#elif L==5
	"Depar.";	// Okoteve traduccin
#elif L==6
	"Dipar.";
#elif L==7
	"Wydzialy";
#elif L==8
	"Depar.";
#endif

const char *Txt_DEPARTMENTS_HELP_ORDER[2] =
   {
#if   L==0
	 "Ordenar por departamento"	// Necessita traduccio
#elif L==1
	 "Sortieren nach Abteilung"
#elif L==2
	 "Sort by department"
#elif L==3
	 "Ordenar por departamento"
#elif L==4
	 "Trier par d&eacute;partement"
#elif L==5
	 "Ordenar por departamento"	// Okoteve traduccin
#elif L==6
	 "Ordina per dipartimento"
#elif L==7
	 "Sortuj wedlug dzial&oacute;w"
#elif L==8
	 "Classificar por departamento"
#endif
	 ,
#if   L==0
	 "Ordenar por n&ordm; de profesores"	// Necessita traduccio
#elif L==1
	 "Sortieren nach Anzahl der Lehrkr&auml;fte"
#elif L==2
	 "Sort by no. of teachers"
#elif L==3
	 "Ordenar por n&ordm; de profesores"
#elif L==4
	 "Trier par nombre d'enseignants"
#elif L==5
	 "Ordenar por n&ordm; de profesores"	// Okoteve traduccin
#elif L==6
	 "Ordina per numero di professori"
#elif L==7
	 "Sortuj wedlug liczby nauczycieli"
#elif L==8
	 "Classificar por n&ordm; de professores"
#endif
	};

const char *Txt_DEPARTMENTS_ORDER[2] =
   {
#if   L==0
	 "Departament"
#elif L==1
	 "Abteilung"
#elif L==2
	 "Department"
#elif L==3
	 "Departamento"
#elif L==4
	 "D&eacute;partement"
#elif L==5
	 "Peh&#7869;"
#elif L==6
	 "Dipartimento"
#elif L==7
	 "Wydzia&lstrok;"
#elif L==8
	 "Departamento"
#endif
	 ,
#if   L==0
	 "Professors"
#elif L==1
	 "Lehrkr&auml;fte"
#elif L==2
	 "Teachers"
#elif L==3
	 "Profesores"
#elif L==4
	 "Enseignants"
#elif L==5
	 "Profesores"	// Okoteve traduccin
#elif L==6
	 "Professori"
#elif L==7
	 "Nauczycieli"
#elif L==8
	 "Professores"
#endif
	};

const char *Txt_Description =
#if   L==0
	"Descripci&oacute;n";	// Necessita traduccio
#elif L==1
	"Beschreibung";
#elif L==2
	"Description";
#elif L==3
	"Descripci&oacute;n";
#elif L==4
	"Description";
#elif L==5
	"Descripci&oacute;n";	// Okoteve traduccin
#elif L==6
	"Descrizione";
#elif L==7
	"Opis";
#elif L==8
	"Descri&ccedil;&atilde;o";
#endif

const char *Txt_Details =
#if   L==0
	"Detalls";
#elif L==1
	"Einzelheiten";
#elif L==2
	"Details";
#elif L==3
	"Detalles";
#elif L==4
	"D&eacute;tails";
#elif L==5
	"Detalles";	// Okoteve traduccin
#elif L==6
	"Dettagli";
#elif L==7
	"Szczeg&oacute;&lstrok;y";
#elif L==8
	"Detalhes";
#endif

const char *Txt_Direct_authentication =
#if   L==0
	"&iquest;Autenticaci&oacute; directa?";
#elif L==1
	"Direkte Authentifizierung?";
#elif L==2
	"Direct authentication?";
#elif L==3
	"&iquest;Autenticaci&oacute;n directa?";
#elif L==4
	"Authentification directe?";
#elif L==5
	"&iquest;Autenticaci&oacute;n directa?";	// Okoteve traduccin
#elif L==6
	"Autenticazione diretta?";
#elif L==7
	"Bezpo&sacute;redni uwierzytelniania?";
#elif L==8
	"Autentica&ccedil;&atilde;o direta?";
#endif

const char *Txt_Direct_authentication_allowed =
#if   L==0
	"Autenticaci&oacute; directa permesa";
#elif L==1
	"Direkte Authentifizierung erlaubt";
#elif L==2
	"Direct authentication allowed";
#elif L==3
	"Autenticaci&oacute;n directa permitida";
#elif L==4
	"Authentification directe autoris&eacute;e";
#elif L==5
	"Autenticaci&oacute;n directa permitida";	// Okoteve traduccin
#elif L==6
	"Autenticazione diretta permessa";
#elif L==7
	"Bezpo&sacute;redni uwierzytelniania dozwolone";
#elif L==8
	"Autentica&ccedil;&atilde;o direta permitida";
#endif

const char *Txt_Direct_authentication_not_allowed =
#if   L==0
	"Autenticaci&oacute; directa no permesa";
#elif L==1
	"Direkte Authentifizierung nicht erlaubt";
#elif L==2
	"Direct authentication not allowed";
#elif L==3
	"Autenticaci&oacute;n directa no permitida";
#elif L==4
	"Authentification directe n'est pas autoris&eacute;e";
#elif L==5
	"Autenticaci&oacute;n directa no permitida";	// Okoteve traduccin
#elif L==6
	"Autenticazione diretta non permessa";
#elif L==7
	"Bezpo&sacute;redni uwierzytelniania nie dozwolone";
#elif L==8
	"Autentica&ccedil;&atilde;o direta n&atilde;o permitida";
#endif

const char *Txt_Disclaimer_the_files_hosted_here_ =	// Warning: it is very important to include two %s in the following sentences
#if   L==0
	"Aviso legal: los archivos alojados aqu&iacute;, salvo que se indique lo contrario,"
	" est&aacute;n sujetos a derechos de propiedad intelectual"
	" y su titularidad corresponde a los usuarios que los han subido."
	" %s no se responsabiliza de la informaci&oacute;n contenida en dichos archivos."
	" Si usted cree conveniente retirar cualquier archivo cuyo contenido le pertenezca"
	" o que infrinja la ley, puede comunicarlo a %s.";	// Necessita traduccio
#elif L==1
	"Disclaimer: the files hosted here, unless otherwise noted,"
	" are subject to intellectual property rights"
	" and their ownership belongs to the users who have uploaded them."
	" %s is not responsible for the information contained in those files."
	" If you think convenient to remove any file whose contents belongs to you"
	" or that violates the law, you can contact %s.";	// Need bersetzung
#elif L==2
	"Disclaimer: the files hosted here, unless otherwise noted,"
	" are subject to intellectual property rights"
	" and their ownership belongs to the users who have uploaded them."
	" %s is not responsible for the information contained in those files."
	" If you think convenient to remove any file whose contents belongs to you"
	" or that violates the law, you can contact %s.";
#elif L==3
	"Aviso legal: los archivos alojados aqu&iacute;, salvo que se indique lo contrario,"
	" est&aacute;n sujetos a derechos de propiedad intelectual"
	" y su titularidad corresponde a los usuarios que los han subido."
	" %s no se responsabiliza de la informaci&oacute;n contenida en dichos archivos."
	" Si usted cree conveniente retirar cualquier archivo cuyo contenido le pertenezca"
	" o que infrinja la ley, puede comunicarlo a %s.";
#elif L==4
	"Disclaimer: the files hosted here, unless otherwise noted,"
	" are subject to intellectual property rights"
	" and their ownership belongs to the users who have uploaded them."
	" %s is not responsible for the information contained in those files."
	" If you think convenient to remove any file whose contents belongs to you"
	" or that violates the law, you can contact %s.";	// Besoin de traduction
#elif L==5
	"Aviso legal: los archivos alojados aqu&iacute;, salvo que se indique lo contrario,"
	" est&aacute;n sujetos a derechos de propiedad intelectual"
	" y su titularidad corresponde a los usuarios que los han subido."
	" %s no se responsabiliza de la informaci&oacute;n contenida en dichos archivos."
	" Si usted cree conveniente retirar cualquier archivo cuyo contenido le pertenezca"
	" o que infrinja la ley, puede comunicarlo a %s.";	// Okoteve traduccin
#elif L==6
	"Disclaimer: the files hosted here, unless otherwise noted,"
	" are subject to intellectual property rights"
	" and their ownership belongs to the users who have uploaded them."
	" %s is not responsible for the information contained in those files."
	" If you think convenient to remove any file whose contents belongs to you"
	" or that violates the law, you can contact %s.";	// Bisogno di traduzione
#elif L==7
	"Disclaimer: the files hosted here, unless otherwise noted,"
	" are subject to intellectual property rights"
	" and their ownership belongs to the users who have uploaded them."
	" %s is not responsible for the information contained in those files."
	" If you think convenient to remove any file whose contents belongs to you"
	" or that violates the law, you can contact %s.";	// Potrzebujesz tlumaczenie
#elif L==8
	"Disclaimer: the files hosted here, unless otherwise noted,"
	" are subject to intellectual property rights"
	" and their ownership belongs to the users who have uploaded them."
	" %s is not responsible for the information contained in those files."
	" If you think convenient to remove any file whose contents belongs to you"
	" or that violates the law, you can contact %s.";	// Necessita de traduo
#endif

const char *Txt_Display_photos =
#if   L==0
	"Incloure fotos";
#elif L==1
	"Bilder anzeigen";
#elif L==2
	"Display photos";
#elif L==3
	"Incluir fotos";
#elif L==4
	"Inclure des photos";
#elif L==5
	"Incluir fotos";	// Okoteve traduccin
#elif L==6
	"Mostra foto";
#elif L==7
	"Wyswietlanie zdjec";
#elif L==8
	"Incluir fotos";
#endif

const char *Txt_distributed_by =
#if   L==0
	"distributs per";
#elif L==1
	"vertrieben von";
#elif L==2
	"distributed by";
#elif L==3
	"distribuidos por";
#elif L==4
	"distribu&eacute; par";
#elif L==5
	"distribuidos por";	// Okoteve traduccin
#elif L==6
	"distribuito da";
#elif L==7
	"wyplacanych przez";
#elif L==8
	"distribu&iacute;do por";
#endif

const char *Txt_Do_not_reload_this_page_because_the_message_will_be_sent_again_ =
#if   L==0
	"No recargue esta p&aacute;gina, pues el mensaje se enviar&iacute;a de nuevo y llegar&iacute;a repetido.";	// Necessita traduccio
#elif L==1
	"Do not reload this page because the message would be sent again and would arrive repeated.";			// Need bersetzung
#elif L==2
	"Do not reload this page because the message would be sent again and would arrive repeated.";
#elif L==3
	"No recargue esta p&aacute;gina, pues el mensaje se enviar&iacute;a de nuevo y llegar&iacute;a repetido.";
#elif L==4
	"Do not reload this page because the message would be sent again and would arrive repeated.";			// Besoin de traduction
#elif L==5
	"No recargue esta p&aacute;gina, pues el mensaje se enviar&iacute;a de nuevo y llegar&iacute;a repetido.";	// Okoteve traduccin
#elif L==6
	"Non ricaricare questa pagina perch&egrave; il messaggio sar&agrave; nuovamente inviato.";
#elif L==7
	"Nie przeladuj strone, poniewaz wiadomosc zostanie wyslana ponownie przyjedzie powtarzane.";
#elif L==8
	"Do not reload this page because the message would be sent again and would arrive repeated.";			// Necessita de traduo
#endif

const char *Txt_Do_not_reload_this_page_because_the_post_will_be_sent_again_ =
#if   L==0
	"No recargue esta p&aacute;gina, pues el mensaje se enviar&iacute;a de nuevo y llegar&iacute;a repetido.";	// Necessita traduccio
#elif L==1
	"Do not reload this page because the post would be sent again and would arrive repeated.";			// Need bersetzung
#elif L==2
	"Do not reload this page because the post would be sent again and would arrive repeated.";
#elif L==3
	"No recargue esta p&aacute;gina, pues el mensaje se enviar&iacute;a de nuevo y llegar&iacute;a repetido.";
#elif L==4
	"Do not reload this page because the post would be sent again and would arrive repeated.";			// Besoin de traduction
#elif L==5
	"No recargue esta p&aacute;gina, pues el mensaje se enviar&iacute;a de nuevo y llegar&iacute;a repetido.";	// Okoteve traduccin
#elif L==6
	"Non ricaricare questa pagina perch&egrave; il post sar&agrave; nuovamente inviato.";
#elif L==7
	"Nie przeladuj strone, poniewaz post zostanie wyslany ponownie przyjedzie powtarzane.";
#elif L==8
	"Do not reload this page because the post would be sent again and would arrive repeated.";			// Necessita de traduo
#endif

const char *Txt_Do_not_show_again =
#if   L==0
	"No mostrar de nou";
#elif L==1
	"Nicht mehr anzeigen";
#elif L==2
	"Do not show again";
#elif L==3
	"No mostrar de nuevo";
#elif L==4
	"Ne plus afficher";
#elif L==5
	"No mostrar de nuevo";	// Okoteve traduccin
#elif L==6
	"Non mostrare pi&ugrave;";
#elif L==7
	"Nie pokazuj ponownie";
#elif L==8
	"N&atilde;o mostrar novamente";
#endif

const char *Txt_Do_you_really_want_to_be_removed_as_an_administrator_of_the_degree_X =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"&iquest;Realmente desea dejar de ser administrador/a de la titulaci&oacute;n <strong>%s</strong>?";	// Necessita traduccio
#elif L==1
	"Do you really want to be removed as an administrator of the degree <strong>%s</strong>?";		// Need bersetzung
#elif L==2
	"Do you really want to be removed as an administrator of the degree <strong>%s</strong>?";
#elif L==3
	"&iquest;Realmente desea dejar de ser administrador/a de la titulaci&oacute;n <strong>%s</strong>?";
#elif L==4
	"Do you really want to be removed as an administrator of the degree <strong>%s</strong>?";		// Besoin de traduction
#elif L==5
	"&iquest;Realmente desea dejar de ser administrador/a de la titulaci&oacute;n <strong>%s</strong>?";	// Okoteve traduccin
#elif L==6
	"Vuole realmente essere rimosso come amministratore della laurea <strong>%s</strong>?";
#elif L==7
	"Czy naprawde chcesz byc usuniety jako administrator stopnia <strong>%s</strong>?";
#elif L==8
	"Do you really want to be removed as an administrator of the degree <strong>%s</strong>?";		// Necessita de traduo
#endif

const char *Txt_Do_you_really_want_to_be_removed_from_the_course_X =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"&iquest;Realmente desea ser eliminado de la asignatura <strong>%s</strong>"
	" (eliminando sus trabajos, adscripci&oacute;n a grupos, etc.)?";	// Necessita traduccio
#elif L==1
	"Do you really want to be removed from the course <strong>%s</strong>"
	" (removing your works, enrollment in groups, etc.)?";			// Need bersetzung
#elif L==2
	"Do you really want to be removed from the course <strong>%s</strong>"
	" (removing your works, enrollment in groups, etc.)?";
#elif L==3
	"&iquest;Realmente desea ser eliminado de la asignatura <strong>%s</strong>"
	" (eliminando sus trabajos, adscripci&oacute;n a grupos, etc.)?";
#elif L==4
	"Do you really want to be removed from the course <strong>%s</strong>"
	" (removing your works, enrollment in groups, etc.)?";			// Besoin de traduction
#elif L==5
	"&iquest;Realmente desea ser eliminado de la asignatura <strong>%s</strong>"
	" (eliminando sus trabajos, adscripci&oacute;n a grupos, etc.)?";	// Okoteve traduccin
#elif L==6
	"Vuoi realmente essere rimosso dal corso <strong>%s</strong>"
	" (rimuovendo i tuoi lavori, registrazione in gruppi, etc.)?";
#elif L==7
	"Czy na pewno chcesz zostac usuniete z kursu <strong>%s</strong>"
	" (usuwanie swoja prace, rejestracja w grupach, itp.)?";
#elif L==8
	"Do you really want to be removed from the course <strong>%s</strong>"
	" (removing your works, enrollment in groups, etc.)?";			// Necessita de traduo
#endif

const char *Txt_Do_you_really_want_to_completely_eliminate_the_following_user =
#if   L==0
	"&iquest;Realmente desea eliminar completamente el siguiente usuario?";	// Necessita traduccio
#elif L==1
	"Do you really want to completely eliminate the following user?";	// Need bersetzung
#elif L==2
	"Do you really want to completely eliminate the following user?";
#elif L==3
	"&iquest;Realmente desea eliminar completamente el siguiente usuario?";
#elif L==4
	"Do you really want to completely eliminate the following user?";	// Besoin de traduction
#elif L==5
	"&iquest;Realmente desea eliminar completamente el siguiente usuario?";	// Okoteve traduccin
#elif L==6
	"Vuoi realmente rimuovere totalmente il seguente utente?";
#elif L==7
	"Czy na pewno chcesz, aby calkowicie wyeliminowac nastepujacych uzytkownik&oacute;w?";
#elif L==8
	"Do you really want to completely eliminate the following user?";	// Necessita de traduo
#endif

const char *Txt_Do_you_really_want_to_completely_eliminate_your_user_account =
#if   L==0
	"&iquest;Realmente desea eliminar completamente su cuenta de usuario?";	// Necessita traduccio
#elif L==1
	"Do you really want to completely eliminate your user account?";	// Need bersetzung
#elif L==2
	"Do you really want to completely eliminate your user account?";
#elif L==3
	"&iquest;Realmente desea eliminar completamente su cuenta de usuario?";
#elif L==4
	"Do you really want to completely eliminate your user account?";	// Besoin de traduction
#elif L==5
	"&iquest;Realmente desea eliminar completamente su cuenta de usuario?";	// Okoteve traduccin
#elif L==6
	"Vuoi realmente rimuovere totalmente il tuo account?";
#elif L==7
	"Czy na pewno chcesz, aby calkowicie wyeliminowac konta uzytkownika?";
#elif L==8
	"Do you really want to completely eliminate your user account?";	// Necessita de traduo
#endif

const char *Txt_Do_you_really_want_to_create_a_new_user_account_with_the_following_data =
#if   L==0
	"Realment voleu crear un nou compte d'usuari"
	" amb les seg&uuml;ents dades?";
#elif L==1
	"Wollen Sie wirklich, um ein neues Benutzerkonto"
	" mit den folgenden Daten zu erstellen?";
#elif L==2
	"Do you really want to create a new user account"
	" with the following data?";
#elif L==3
	"&iquest;Realmente desea crear una nueva cuenta de usuario"
	" con los siguientes datos?";
#elif L==4
	"Voulez-vous vraiment cr&eacute;er un nouveau compte d'utilisateur"
	" avec les donn&eacute;es suivantes?";
#elif L==5
	"&iquest;Realmente desea crear una nueva cuenta de usuario"
	" con los siguientes datos?";	// Okoteve traduccin
#elif L==6
	"Vuoi realmente creare un nuovo account utente"
	" con i seguenti dati?";
#elif L==7
	"Czy na pewno chcesz, aby utworzy&cacute; nowe konto u&zdot;ytkownika"
	" z nast&eogon;puj&aogon;cymi danymi?";
#elif L==8
	"Voc&ecirc; realmente deseja criar uma nova conta de usu&aacute;rio"
	" com os seguintes dados?";
#endif

const char *Txt_Do_you_really_want_to_delete_all_messages_received_from_USER_X_from_COURSE_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==0
	"&iquest;Realmente desea eliminar todos los mensajes recibidos de %s desde %s?";	// Necessita traduccio
#elif L==1
	"Do you really want to delete all messages received from %s from %s?";			// Need bersetzung
#elif L==2
	"Do you really want to delete all messages received from %s from %s?";
#elif L==3
	"&iquest;Realmente desea eliminar todos los mensajes recibidos de %s desde %s?";
#elif L==4
	"Do you really want to delete all messages received from %s from %s?";			// Besoin de traduction
#elif L==5
	"&iquest;Realmente desea eliminar todos los mensajes recibidos de %s desde %s?";	// Okoteve traduccin
#elif L==6
	"Vuoi realmente rimuovere tutti i messaggi ricevuti da %s da %s?";
#elif L==7
	"Czy na pewno chcesz usunac wszystkie wiadomosci otrzymane od %s z %s?";
#elif L==8
	"Do you really want to delete all messages received from %s from %s?";	// Necessita de traduo
#endif

const char *Txt_Do_you_really_want_to_delete_all_messages_received_from_USER_X_from_COURSE_Y_related_to_CONTENT_Z =	// Warning: it is very important to include three %s in the following sentences
#if   L==0
	"&iquest;Realmente desea eliminar todos los mensajes recibidos de %s desde %s relacionados con <strong>%s</strong>?";	// Necessita traduccio
#elif L==1
	"Do you really want to delete all messages received from %s from %s related to <strong>%s</strong>?";			// Need bersetzung
#elif L==2
	"Do you really want to delete all messages received from %s from %s related to <strong>%s</strong>?";
#elif L==3
	"&iquest;Realmente desea eliminar todos los mensajes recibidos de %s desde %s relacionados con <strong>%s</strong>?";
#elif L==4
	"Do you really want to delete all messages received from %s from %s related to <strong>%s</strong>?";			// Besoin de traduction
#elif L==5
	"&iquest;Realmente desea eliminar todos los mensajes recibidos de %s desde %s relacionados con <strong>%s</strong>?";	// Okoteve traduccin
#elif L==6
	"Vuoi realmente rimuovere tutti i messaggi ricevuti da %s da %s relativi a <strong>%s</strong>?";
#elif L==7
	"Czy na pewno chcesz usunac wszystkie wiadomosci otrzymane od %s z %s podobne do <strong>%s</strong>?";
#elif L==8
	"Do you really want to delete all messages received from %s from %s related to <strong>%s</strong>?";			// Necessita de traduo
#endif

const char *Txt_Do_you_really_want_to_delete_all_messages_sent_to_USER_X_from_COURSE_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==0
	"&iquest;Realmente desea eliminar todos los mensajes enviados a %s desde %s?";	// Necessita traduccio
#elif L==1
	"Do you really want to delete all messages sent to %s from %s?";		// Need bersetzung
#elif L==2
	"Do you really want to delete all messages sent to %s from %s?";
#elif L==3
	"&iquest;Realmente desea eliminar todos los mensajes enviados a %s desde %s?";
#elif L==4
	"Do you really want to delete all messages sent to %s from %s?";		// Besoin de traduction
#elif L==5
	"&iquest;Realmente desea eliminar todos los mensajes enviados a %s desde %s?";	// Okoteve traduccin
#elif L==6
	"Vuoi realmente rimuovere tutti i messaggi inviati a %s da %s?";
#elif L==7
	"Czy na pewno chcesz usunac wszystkie wiadomosci wyslane do %s z %s?";
#elif L==8
	"Do you really want to delete all messages sent to %s from %s?";		// Necessita de traduo
#endif

const char *Txt_Do_you_really_want_to_delete_all_messages_sent_to_USER_X_from_COURSE_Y_related_to_CONTENT_Z =	// Warning: it is very important to include three %s in the following sentences
#if   L==0
	"&iquest;Realmente desea eliminar todos los mensajes enviados a %s desde %s relacionados con <strong>%s</strong>?";	// Necessita traduccio
#elif L==1
	"Do you really want to delete all messages sent to %s from %s related to <strong>%s</strong>?";				// Need bersetzung
#elif L==2
	"Do you really want to delete all messages sent to %s from %s related to <strong>%s</strong>?";
#elif L==3
	"&iquest;Realmente desea eliminar todos los mensajes enviados a %s desde %s relacionados con <strong>%s</strong>?";
#elif L==4
	"Do you really want to delete all messages sent to %s from %s related to <strong>%s</strong>?";				// Besoin de traduction
#elif L==5
	"&iquest;Realmente desea eliminar todos los mensajes enviados a %s desde %s relacionados con <strong>%s</strong>?";	// Okoteve traduccin
#elif L==6
	"Vuoi realmente rimuovere tutti i messaggi inviati a %s da %s relativi a<strong>%s</strong>?";
#elif L==7
	"Czy na pewno chcesz usunac wszystkie wiadomosci wyslane do %s z %s podobne do <strong>%s</strong>?";
#elif L==8
	"Do you really want to delete all messages sent to %s from %s related to <strong>%s</strong>?";				// Necessita de traduo
#endif

const char *Txt_Do_you_really_want_to_delete_the_unread_messages_received_from_USER_X_from_COURSE_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==0
	"&iquest;Realmente desea eliminar los mensajes no le&iacute;dos recibidos de %s desde %s?";	// Necessita traduccio
#elif L==1
	"Do you really want to delete the unread messages received from %s from %s?";			// Need bersetzung
#elif L==2
	"Do you really want to delete the unread messages received from %s from %s?";
#elif L==3
	"&iquest;Realmente desea eliminar los mensajes no le&iacute;dos recibidos de %s desde %s?";
#elif L==4
	"Do you really want to delete the unread messages received from %s from %s?";			// Besoin de traduction
#elif L==5
	"&iquest;Realmente desea eliminar los mensajes no le&iacute;dos recibidos de %s desde %s?";	// Okoteve traduccin
#elif L==6
	"Vuoi realmente rimuovere tutti i messaggi non letti ricevuti da %s da %s?";
#elif L==7
	"Czy na pewno chcesz usunac nieprzeczytanych wiadomosci otrzymane od %s z %s?";
#elif L==8
	"Do you really want to delete the unread messages received from %s from %s?";			// Necessita de traduo
#endif

const char *Txt_Do_you_really_want_to_delete_the_unread_messages_received_from_USER_X_from_COURSE_Y_related_to_CONTENT_Z =	// Warning: it is very important to include three %s in the following sentences
#if   L==0
	"&iquest;Realmente desea eliminar los mensajes no le&iacute;dos recibidos"
	" de %s desde %s relacionados con <strong>%s</strong>?";	// Necessita traduccio
#elif L==1
	"Do you really want to delete the unread messages received"
	" from %s from %s related to <strong>%s</strong>?";		// Need bersetzung
#elif L==2
	"Do you really want to delete the unread messages received"
	" from %s from %s related to <strong>%s</strong>?";
#elif L==3
	"&iquest;Realmente desea eliminar los mensajes no le&iacute;dos recibidos"
	" de %s desde %s relacionados con <strong>%s</strong>?";
#elif L==4
	"Do you really want to delete the unread messages received"
	" from %s from %s related to <strong>%s</strong>?";		// Besoin de traduction
#elif L==5
	"&iquest;Realmente desea eliminar los mensajes no le&iacute;dos recibidos"
	" de %s desde %s relacionados con <strong>%s</strong>?";	// Okoteve traduccin
#elif L==6
	"Vuoi realmente rimuovere tutti i messaggi non letti ricevuti"
	" da %s da %s relativi a <strong>%s</strong>?";
#elif L==7
	"Czy na pewno chcesz usunac nieprzeczytanych wiadomosci otrzymane"
	" od %s z %s podobne do <strong>%s</strong>?";
#elif L==8
	"Do you really want to delete the unread messages received"
	" from %s from %s related to <strong>%s</strong>?";		// Necessita de traduo
#endif

const char *Txt_Do_you_really_want_to_register_the_following_user_as_an_administrator_of_the_degree_X =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"&iquest;Realmente desea inscribir el siguiente usuario como administrador de la titulaci&oacute;n <strong>%s</strong>?";	// Necessita traduccio
#elif L==1
	"Do you really want to register the following user as an administrator of the degree <strong>%s</strong>?";	// Need bersetzung
#elif L==2
	"Do you really want to register the following user as an administrator of the degree <strong>%s</strong>?";
#elif L==3
	"&iquest;Realmente desea inscribir el siguiente usuario como administrador de la titulaci&oacute;n <strong>%s</strong>?";
#elif L==4
	"Do you really want to register the following user as an administrator of the degree <strong>%s</strong>?";	// Besoin de traduction
#elif L==5
	"&iquest;Realmente desea inscribir el siguiente usuario como administrador de la titulaci&oacute;n <strong>%s</strong>?";	// Okoteve traduccin
#elif L==6
	"Vuoi realmente registrare il seguente utente come amministratore della laurea <strong>%s</strong>?";
#elif L==7
	"Czy na pewno chcesz sie zarejestrowac nastepujace u&zdot;ytkownika jako administratora stopnia <strong>%s</strong>?";
#elif L==8
	"Do you really want to register the following user as an administrator of the degree <strong>%s</strong>?";	// Necessita de traduo
#endif

const char *Txt_Do_you_really_want_to_reject_the_enrollment_request_ =	// Warning: it is very important to include three %s in the following sentences
#if   L==0
	"&iexcl;Realmente desea rechazar la solicitud de inscripci&oacute;n"
	" de <strong>%s</strong> como <strong>%s</strong>"
	" en la asignatura <strong>%s</strong>?";		// Necessita traduccio
#elif L==1
	"Do you really want to reject the enrollment request"
	" from <strong>%s</strong> as <strong>%s</strong>"
	" in the course <strong>%s</strong>?";			// Need bersetzung
#elif L==2
	"Do you really want to reject the enrollment request"
	" from <strong>%s</strong> as <strong>%s</strong>"
	" in the course <strong>%s</strong>?";
#elif L==3
	"&iexcl;Realmente desea rechazar la solicitud de inscripci&oacute;n"
	" de <strong>%s</strong> como <strong>%s</strong>"
	" en la asignatura <strong>%s</strong>?";
#elif L==4
	"Do you really want to reject the enrollment request"
	" from <strong>%s</strong> as <strong>%s</strong>"
	" in the course <strong>%s</strong>?";			// Besoin de traduction
#elif L==5
	"&iexcl;Realmente desea rechazar la solicitud de inscripci&oacute;n"
	" de <strong>%s</strong> como <strong>%s</strong>"
	" en la asignatura <strong>%s</strong>?";		// Okoteve traduccin
#elif L==6
	"Do you really want to reject the enrollment request"
	" from <strong>%s</strong> as <strong>%s</strong>"
	" in the course <strong>%s</strong>?";			// Bisogno di traduzione
#elif L==7
	"Do you really want to reject the enrollment request"
	" from <strong>%s</strong> as <strong>%s</strong>"
	" in the course <strong>%s</strong>?";			// Potrzebujesz tlumaczenie
#elif L==8
	"Do you really want to reject the enrollment request"
	" from <strong>%s</strong> as <strong>%s</strong>"
	" in the course <strong>%s</strong>?";			// Necessita de traduo
#endif

const char *Txt_Do_you_really_want_to_remove_the_X_students_from_the_course_Y_ =	// Warning: it is very important to include %u and %s in the following sentences
#if   L==0
	"&iquest;Realmente desea eliminar los %u estudiantes de la asignatura <strong>%s</strong>?<br />"
	"Aunque no se eliminar&aacute;n los datos de los estudiantes en otras asignaturas,"
	" ya no aparecer&aacute;n inscritos en esta asignatura,"
	" y adem&aacute;s <strong>se eliminar&aacute;n todos los trabajos enviados por ellos en esta asignatura,"
	" sus fichas personalizadas para esta asignatura,"
	" y su inscripci&oacute;n en grupos de esta asignatura.</strong>.";		// Necessita traduccio
#elif L==1
	"Do you really want to remove the %u students from the course <strong>%s</strong>?<br />"
	"Although students' data will not be removed in other courses,"
	" they will no longer enrolled in this course,"
	" and also <strong>all the works submitted by them in this course,"
	" their record cards for this course,"
	" and their enrollment in groups of this course will be removed.</strong>.";	// Need bersetzung
#elif L==2
	"Do you really want to remove the %u students from the course <strong>%s</strong>?<br />"
	"Although students' data will not be removed in other courses,"
	" they will no longer enrolled in this course,"
	" and also <strong>all the works submitted by them in this course,"
	" their record cards for this course,"
	" and their enrollment in groups of this course will be removed.</strong>.";
#elif L==3
	"&iquest;Realmente desea eliminar los %u estudiantes de la asignatura <strong>%s</strong>?<br />"
	"Aunque no se eliminar&aacute;n los datos de los estudiantes en otras asignaturas,"
	" ya no aparecer&aacute;n inscritos en esta asignatura,"
	" y adem&aacute;s <strong>se eliminar&aacute;n todos los trabajos enviados por ellos en esta asignatura,"
	" sus fichas personalizadas para esta asignatura,"
	" y su inscripci&oacute;n en grupos de esta asignatura.</strong>.";
#elif L==4
	"Do you really want to remove the %u students from the course <strong>%s</strong>?<br />"
	"Although students' data will not be removed in other courses,"
	" they will no longer enrolled in this course,"
	" and also <strong>all the works submitted by them in this course,"
	" their record cards for this course,"
	" and their enrollment in groups of this course will be removed.</strong>.";	// Besoin de traduction
#elif L==5
	"&iquest;Realmente desea eliminar los %u estudiantes de la asignatura <strong>%s</strong>?<br />"
	"Aunque no se eliminar&aacute;n los datos de los estudiantes en otras asignaturas,"
	" ya no aparecer&aacute;n inscritos en esta asignatura,"
	" y adem&aacute;s <strong>se eliminar&aacute;n todos los trabajos enviados por ellos en esta asignatura,"
	" sus fichas personalizadas para esta asignatura,"
	" y su inscripci&oacute;n en grupos de esta asignatura.</strong>.";		// Okoteve traduccin
#elif L==6
	"Vuoi realmente rimuovere gli %u studenti dal corso <strong>%s</strong>?<br />"
	"Sebbene i dati degli studenti non saranno rimossi in altri corsi,"
	" essi non saranno pi&ugrave; registrati in questo corso,"
	" e cos <strong>tutti i lavori presentati da loro in questo corso,"
	" le loro schede per questo corso,"
	" e la loro registrazione nei gruppi di questo corso saranno rimossi.</strong>.";
#elif L==7
	"Czy na pewno chcesz usunac %u student&oacute;w z kursu <strong>%s</strong>?<br />"
	"Although students' data will not be removed in other courses,"
	" nie beda juz zarejestrowane w tym kurs"
	" a takze <strong>wszystkie prace zlozone przez nich w tym kursie,"
	" swoich kart zapisu na ten kurs,"
	" i ich rejestracja w grupach zajec zostana usuniete.</strong>.";
#elif L==8
	"Do you really want to remove the %u students from the course <strong>%s</strong>?<br />"
	"Although students' data will not be removed in other courses,"
	" they will no longer enrolled in this course,"
	" and also <strong>all the works submitted by them in this course,"
	" their record cards for this course,"
	" and their enrollment in groups of this course will be removed.</strong>.";	// Necessita de traduo
#endif

const char *Txt_Do_you_really_want_to_remove_the_assignment_X =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"&iquest;Realmente desea eliminar la actividad <strong>%s</strong>?";	// Necessita traduccio
#elif L==1
	"Wollen Sie die Aufgabe <strong>%s</strong> wirklich entfernen?";
#elif L==2
	"Do you really want to remove the assignment <strong>%s</strong>?";
#elif L==3
	"&iquest;Realmente desea eliminar la actividad <strong>%s</strong>?";
#elif L==4
	"Voulez-vous vraiment supprimer l'activit&eacute; <strong>%s</strong>?";
#elif L==5
	"&iquest;Realmente desea eliminar la actividad <strong>%s</strong>?";	// Okoteve traduccin
#elif L==6
	"Vuoi realmente rimuovere l'attivit&agrave; <strong>%s</strong>?";
#elif L==7
	"Czy na pewno chcesz usunac przypisanie <strong>%s</strong>?";
#elif L==8
	"Voc&ecirc; realmente deseja remover a atividade <strong>%s</strong>?";
#endif

const char *Txt_Do_you_really_want_to_remove_the_entire_thread =
#if   L==0
	"&iquest;Realmente desea eliminar toda la discusi&oacute;n?";	// Necessita traduccio
#elif L==1
	"Wollen Sie die gesamten Thread wirklich entfernen?";
#elif L==2
	"Do you really want to remove the entire thread?";
#elif L==3
	"&iquest;Realmente desea eliminar toda la discusi&oacute;n?";
#elif L==4
	"Voulez-vous vraiment supprimer tout le fil?";
#elif L==5
	"&iquest;Realmente desea eliminar toda la discusi&oacute;n?";	// Okoteve traduccin
#elif L==6
	"Vuoi realmente rimuovere l'intera discussione?";
#elif L==7
	"Czy na pewno chcesz usunac caly watek?";
#elif L==8
	"Voc&ecirc; realmente deseja remover todo o thread?";
#endif

const char *Txt_Do_you_really_want_to_remove_the_entire_thread_X =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"&iquest;Realmente desea eliminar toda la discusi&oacute;n <strong>%s</strong>?";	// Necessita traduccio
#elif L==1
	"Wollen Sie die gesamten Thread <strong>%s</strong> wirklich entfernen?";
#elif L==2
	"Do you really want to remove the entire thread <strong>%s</strong>?";
#elif L==3
	"&iquest;Realmente desea eliminar toda la discusi&oacute;n <strong>%s</strong>?";
#elif L==4
	"Voulez-vous vraiment supprimer tout le fil <strong>%s</strong>?";
#elif L==5
	"&iquest;Realmente desea eliminar toda la discusi&oacute;n <strong>%s</strong>?";	// Okoteve traduccin
#elif L==6
	"Vuoi realmente rimuovere l'intera discussione <strong>%s</strong>?";
#elif L==7
	"Czy na pewno chcesz usunac caly watek <strong>%s</strong>?";
#elif L==8
	"Voc&ecirc; realmente deseja remover todo o thread <strong>%s</strong>?";
#endif

const char *Txt_Do_you_really_want_to_remove_the_event_X =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"&iquest;Realmente desea eliminar el evento <strong>%s</strong>?";	// Necessita traduccio
#elif L==1
	"Wollen Sie der Ereignis <strong>%s</strong> wirklich entfernen?";
#elif L==2
	"Do you really want to remove the event <strong>%s</strong>?";
#elif L==3
	"&iquest;Realmente desea eliminar el evento <strong>%s</strong>?";
#elif L==4
	"Voulez-vous vraiment supprimer l'&eacute;v&eacute;nement <strong>%s</strong>?";
#elif L==5
	"&iquest;Realmente desea eliminar el evento <strong>%s</strong>?";	// Okoteve traduccin
#elif L==6
	"Vuoi realmente rimuovere l'evento <strong>%s</strong>?";
#elif L==7
	"Czy na pewno chcesz usunac wydarzenie <strong>%s</strong>?";
#elif L==8
	"Voc&ecirc; realmente deseja remover o evento <strong>%s</strong>?";
#endif

const char *Txt_Do_you_really_want_to_remove_the_field_X_from_the_records_of_X =	// Warning: it is very important to include two %s in the following sentences
#if   L==0
	"&iquest;Realmente desea eliminar el campo <strong>%s</strong> de las fichas de <strong>%s</strong>?";	// Necessita traduccio
#elif L==1
	"Do you really want to remove the field <strong>%s</strong> from the records of <strong>%s</strong>?";	// Need bersetzung
#elif L==2
	"Do you really want to remove the field <strong>%s</strong> from the records of <strong>%s</strong>?";
#elif L==3
	"&iquest;Realmente desea eliminar el campo <strong>%s</strong> de las fichas de <strong>%s</strong>?";
#elif L==4
	"Do you really want to remove the field <strong>%s</strong> from the records of <strong>%s</strong>?";	// Besoin de traduction
#elif L==5
	"&iquest;Realmente desea eliminar el campo <strong>%s</strong> de las fichas de <strong>%s</strong>?";	// Okoteve traduccin
#elif L==6
	"Vuoi realmente rimuovere il campo <strong>%s</strong> dalle schede di <strong>%s</strong>?";
#elif L==7
	"Czy na pewno chcesz usunac pole <strong>%s</strong> z ewidencji <strong>%s</strong>?";
#elif L==8
	"Do you really want to remove the field <strong>%s</strong> from the records of <strong>%s</strong>?";	// Necessita de traduo
#endif

const char *Txt_Do_you_really_want_to_remove_FILE_OR_LINK_X =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"&iquest;Realmente desea eliminar <strong>%s</strong>?";	// Necessita traduccio
#elif L==1
	"Wollen Sie <strong>%s</strong> wirklich entfernen?";
#elif L==2
	"Do you really want to remove <strong>%s</strong>?";
#elif L==3
	"&iquest;Realmente desea eliminar <strong>%s</strong>?";
#elif L==4
	"Voulez-vous vraiment supprimer <strong>%s</strong>?";
#elif L==5
	"&iquest;Realmente desea eliminar <strong>%s</strong>?";	// Okoteve traduccin
#elif L==6
	"Vuoi realmente rimuovere <strong>%s</strong>?";
#elif L==7
	"Czy na pewno chcesz usunac <strong>%s</strong>?";
#elif L==8
	"Voc&ecirc; realmente deseja remover <strong>%s</strong>?";
#endif

const char *Txt_Do_you_really_want_to_remove_the_folder_X =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"&iquest;Realmente desea eliminar la carpeta <strong>%s</strong> y todo su contenido?";	// Necessita traduccio
#elif L==1
	"Wollen Sie wirklich das Verzeichnis <strong>%s</strong> und alle Inhalte entfernen?";
#elif L==2
	"Do you really want to remove the folder <strong>%s</strong> and all its contents?";
#elif L==3
	"&iquest;Realmente desea eliminar la carpeta <strong>%s</strong> y todo su contenido?";
#elif L==4
	"Voulez-vous vraiment supprimer le r&eacute;pertoire <strong>%s</strong> et l'ensemble de son contenu?";
#elif L==5
	"&iquest;Realmente desea eliminar la carpeta <strong>%s</strong> y todo su contenido?";	// Okoteve traduccin
#elif L==6
	"Vuoi realmente rimuovere la cartella <strong>%s</strong> e tutto il suo contenuto?";
#elif L==7
	"Czy na pewno chcesz usunac folder <strong>%s</strong> i wszystkie jego skladniki?";
#elif L==8
	"Voc&ecirc; realmente deseja remover o diret&oacute;rio <strong>%s</strong> e todo o seu conte&uacute;do?";
#endif

const char *Txt_Do_you_really_want_to_remove_the_following_user_as_an_administrator_of_the_degree_X =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"&iquest;Realmente desea eliminar el siguiente usuario como administrador de la titulaci&oacute;n <strong>%s</strong>?";	// Necessita traduccio
#elif L==1
	"Do you really want to remove the following user as an administrator of the degree <strong>%s</strong>?";	// Need bersetzung
#elif L==2
	"Do you really want to remove the following user as an administrator of the degree <strong>%s</strong>?";
#elif L==3
	"&iquest;Realmente desea eliminar el siguiente usuario como administrador de la titulaci&oacute;n <strong>%s</strong>?";
#elif L==4
	"Do you really want to remove the following user as an administrator of the degree <strong>%s</strong>?";	// Besoin de traduction
#elif L==5
	"&iquest;Realmente desea eliminar el siguiente usuario como administrador de la titulaci&oacute;n <strong>%s</strong>?";	// Okoteve traduccin
#elif L==6
	"Vuoi realmente rimuovere il seguente utente come amministratore della laurea <strong>%s</strong>?";
#elif L==7
	"Czy na pewno chcesz usunac nastepujace u&zdot;ytkownika jako administratora stopnia <strong>%s</strong>?";
#elif L==8
	"Do you really want to remove the following user as an administrator of the degree <strong>%s</strong>?";	// Necessita de traduo
#endif

const char *Txt_Do_you_really_want_to_remove_the_following_user_from_the_course_X =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"&iquest;Realmente desea eliminar el siguiente usuario de la asignatura <strong>%s</strong>"
	" (eliminando sus trabajos, adscripci&oacute;n a grupos, etc.)?";	// Necessita traduccio
#elif L==1
	"Do you really want to remove the following user from the course <strong>%s</strong>"
	" (removing her/his works, enrollment in groups, etc.)?";		// Need bersetzung
#elif L==2
	"Do you really want to remove the following user from the course <strong>%s</strong>"
	" (removing her/his works, enrollment in groups, etc.)?";
#elif L==3
	"&iquest;Realmente desea eliminar el siguiente usuario de la asignatura <strong>%s</strong>"
	" (eliminando sus trabajos, adscripci&oacute;n a grupos, etc.)?";
#elif L==4
	"Do you really want to remove the following user from the course <strong>%s</strong>"
	" (removing her/his works, enrollment in groups, etc.)?";		// Besoin de traduction
#elif L==5
	"&iquest;Realmente desea eliminar el siguiente usuario de la asignatura <strong>%s</strong>"
	" (eliminando sus trabajos, adscripci&oacute;n a grupos, etc.)?";	// Okoteve traduccin
#elif L==6
	"Vuoi realmente rimuovere il seguente utente dal corso <strong>%s</strong>"
	" (rimuovendo i suoi lavori, registrazione in gruppi, etc.)?";
#elif L==7
	"Do you really want to remove the following user z kursu <strong>%s</strong>"
	" (removing her/his works, enrollment in groups, etc.)?";
#elif L==8
	"Do you really want to remove the following user from the course <strong>%s</strong>"
	" (removing her/his works, enrollment in groups, etc.)?";		// Necessita de traduo
#endif

const char *Txt_Do_you_really_want_to_remove_the_group_X =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"&iquest;Realmente desea eliminar el grupo <strong>%s</strong>?";	// Necessita traduccio
#elif L==1
	"Do you really want to remove the group <strong>%s</strong>?";		// Need bersetzung
#elif L==2
	"Do you really want to remove the group <strong>%s</strong>?";
#elif L==3
	"&iquest;Realmente desea eliminar el grupo <strong>%s</strong>?";
#elif L==4
	"Do you really want to remove the group <strong>%s</strong>?";		// Besoin de traduction
#elif L==5
	"&iquest;Realmente desea eliminar el grupo <strong>%s</strong>?";	// Okoteve traduccin
#elif L==6
	"Vuoi realmente rimuovere il gruppo <strong>%s</strong>?";
#elif L==7
	"Czy na pewno chcesz usunac grupe <strong>%s</strong>?";
#elif L==8
	"Do you really want to remove the group <strong>%s</strong>?";	// Necessita de traduo
#endif

const char *Txt_Do_you_really_want_to_remove_the_group_X_1_student_ =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"&iquest;Realmente desea eliminar el grupo <strong>%s</strong>?<br />"
	"Si lo hace, un/a estudiante dejar&aacute; de pertenecer a ese grupo.";	// Necessita traduccio
#elif L==1
	"Do you really want to remove the group <strong>%s</strong>?<br />"
	"Doing so will remove one student from that group.";			// Need bersetzung
#elif L==2
	"Do you really want to remove the group <strong>%s</strong>?<br />"
	"Doing so will remove one student from that group.";
#elif L==3
	"&iquest;Realmente desea eliminar el grupo <strong>%s</strong>?<br />"
	"Si lo hace, un/a estudiante dejar&aacute; de pertenecer a ese grupo.";
#elif L==4
	"Do you really want to remove the group <strong>%s</strong>?<br />"
	"Doing so will remove one student from that group.";			// Besoin de traduction
#elif L==5
	"&iquest;Realmente desea eliminar el grupo <strong>%s</strong>?<br />"
	"Si lo hace, un/a estudiante dejar&aacute; de pertenecer a ese grupo.";	// Okoteve traduccin
#elif L==6
	"Vuoi realmente rimuovere il gruppo <strong>%s</strong>?<br />"
	"Facendo cos rimuoverai uno studente da questo gruppo.";
#elif L==7
	"Do you really want to remove the group <strong>%s</strong>?<br />"
	"W ten spos&oacute;b usunie jeden uczen z tej grupy.";
#elif L==8
	"Do you really want to remove the group <strong>%s</strong>?<br />"
	"Doing so will remove one student from that group.";			// Necessita de traduo
#endif

const char *Txt_Do_you_really_want_to_remove_the_group_X_Y_students_ =	// Warning: it is very important to include %s and %u in the following sentences
#if   L==0
	"&iquest;Realmente desea eliminar el grupo <strong>%s</strong>?<br />"
	"Si lo hace, %u estudiantes dejar&aacute;n de pertenecer a ese grupo.";	// Necessita traduccio
#elif L==1
	"Do you really want to remove the group <strong>%s</strong>?<br />"
	"Doing so will remove %u students from that group.";			// Need bersetzung
#elif L==2
	"Do you really want to remove the group <strong>%s</strong>?<br />"
	"Doing so will remove %u students from that group.";
#elif L==3
	"&iquest;Realmente desea eliminar el grupo <strong>%s</strong>?<br />"
	"Si lo hace, %u estudiantes dejar&aacute;n de pertenecer a ese grupo.";
#elif L==4
	"Do you really want to remove the group <strong>%s</strong>?<br />"
	"Doing so will remove %u students from that group.";			// Besoin de traduction
#elif L==5
	"&iquest;Realmente desea eliminar el grupo <strong>%s</strong>?<br />"
	"Si lo hace, %u estudiantes dejar&aacute;n de pertenecer a ese grupo.";	// Okoteve traduccin
#elif L==6
	"Vuoi realmente rimuovere il gruppo <strong>%s</strong>?<br />"
	"Facendo cos rimuoverai %u studenti da questo gruppo.";
#elif L==7
	"Do you really want to remove the group <strong>%s</strong>?<br />"
	"W ten spos&oacute;b usunie %u uczni&oacute;w z tej grupy.";
#elif L==8
	"Do you really want to remove the group <strong>%s</strong>?<br />"
	"Doing so will remove %u students from that group.";			// Necessita de traduo
#endif

const char *Txt_Do_you_really_want_to_remove_the_survey_X =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"&iquest;Realmente desea eliminar la encuesta <strong>%s</strong>?";	// Necessita traduccio
#elif L==1
	"Wollen Sie die Umfrage <strong>%s</strong> wirklich entfernen?";
#elif L==2
	"Do you really want to remove the survey <strong>%s</strong>?";
#elif L==3
	"&iquest;Realmente desea eliminar la encuesta <strong>%s</strong>?";
#elif L==4
	"Voulez-vous vraiment supprimer le sondage <strong>%s</strong>?";
#elif L==5
	"&iquest;Realmente desea eliminar la encuesta <strong>%s</strong>?";	// Okoteve traduccin
#elif L==6
	"Vuoi realmente rimuovere il sondaggio <strong>%s</strong>?";
#elif L==7
	"Czy na pewno chcesz usunac badania <strong>%s</strong>?";
#elif L==8
	"Voc&ecirc; realmente deseja remover o inqu&eacute;rito <strong>%s</strong>?";
#endif

const char *Txt_Do_you_really_want_to_remove_the_type_of_group_X_1_group_ =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"&iquest;Realmente desea eliminar el tipo de grupo <strong>%s</strong>?<br />"
	"Si lo hace, eliminar&aacute; tambi&eacute;n un grupo de ese tipo.";	// Necessita traduccio
#elif L==1
	"Do you really want to remove the type of group <strong>%s</strong>?<br />"
	"Doing so will also remove one group of that type.";			// Need bersetzung
#elif L==2
	"Do you really want to remove the type of group <strong>%s</strong>?<br />"
	"Doing so will also remove one group of that type.";
#elif L==3
	"&iquest;Realmente desea eliminar el tipo de grupo <strong>%s</strong>?<br />"
	"Si lo hace, eliminar&aacute; tambi&eacute;n un grupo de ese tipo.";
#elif L==4
	"Do you really want to remove the type of group <strong>%s</strong>?<br />"
	"Doing so will also remove one group of that type.";			// Besoin de traduction
#elif L==5
	"&iquest;Realmente desea eliminar el tipo de grupo <strong>%s</strong>?<br />"
	"Si lo hace, eliminar&aacute; tambi&eacute;n un grupo de ese tipo.";	// Okoteve traduccin
#elif L==6
	"Vuoi realmente rimuovere il tipo di gruppo <strong>%s</strong>?<br />"
	"Facendo cos rimuoverai anche un gruppo di questo tipo.";
#elif L==7
	"Do you really want to remove the type of group <strong>%s</strong>?<br />"
	"Spowoduje to takze usuniecie jednej grupie tego typu.";
#elif L==8
	"Do you really want to remove the type of group <strong>%s</strong>?<br />"
	"Doing so will also remove one group of that type.";			// Necessita de traduo
#endif

const char *Txt_Do_you_really_want_to_remove_the_type_of_group_X_Y_groups_ =	// Warning: it is very important to include %s and %u in the following sentences
#if   L==0
	"&iquest;Realmente desea eliminar el tipo de grupo <strong>%s</strong>?<br />"
	"Si lo hace, eliminar&aacute; tambi&eacute;n %u grupos de ese tipo.";	// Necessita traduccio
#elif L==1
	"Do you really want to remove the type of group <strong>%s</strong>?<br />"
	"Doing so will also remove %u groups of that type.";			// Need bersetzung
#elif L==2
	"Do you really want to remove the type of group <strong>%s</strong>?<br />"
	"Doing so will also remove %u groups of that type.";
#elif L==3
	"&iquest;Realmente desea eliminar el tipo de grupo <strong>%s</strong>?<br />"
	"Si lo hace, eliminar&aacute; tambi&eacute;n %u grupos de ese tipo.";
#elif L==4
	"Do you really want to remove the type of group <strong>%s</strong>?<br />"
	"Doing so will also remove %u groups of that type.";			// Besoin de traduction
#elif L==5
	"&iquest;Realmente desea eliminar el tipo de grupo <strong>%s</strong>?<br />"
	"Si lo hace, eliminar&aacute; tambi&eacute;n %u grupos de ese tipo.";	// Okoteve traduccin
#elif L==6
	"Vuoi realmente rimuovere il tipo di gruppo <strong>%s</strong>?<br />"
	"Facendo cos rimuoverai anche %u gruppi di questo tipo.";
#elif L==7
	"Do you really want to remove the type of group <strong>%s</strong>?<br />"
	"Spowoduje to r&oacute;wniez usuniecie grup %u tego typu.";
#elif L==8
	"Do you really want to remove the type of group <strong>%s</strong>?<br />"
	"Doing so will also remove %u groups of that type.";			// Necessita de traduo
#endif

const char *Txt_Do_you_really_want_to_reset_the_survey_X =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"&iquest;Realmente desea poner a cero la encuesta <strong>%s</strong>?";	// Necessita traduccio
#elif L==1
	"Wollen Sie die Umfrage <strong>%s</strong> wirklich reset?";
#elif L==2
	"Do you really want to reset the survey <strong>%s</strong>?";
#elif L==3
	"&iquest;Realmente desea poner a cero la encuesta <strong>%s</strong>?";
#elif L==4
	"Voulez-vous vraiment reset le sondage <strong>%s</strong>?";
#elif L==5
	"&iquest;Realmente desea poner a cero la encuesta <strong>%s</strong>?";	// Okoteve traduccin
#elif L==6
	"Vuoi realmente resettare il sondaggio <strong>%s</strong>?";
#elif L==7
	"Czy na pewno chcesz zresetowac badania <strong>%s</strong>?";
#elif L==8
	"Voc&ecirc; realmente deseja reiniciar o inqu&eacute;rito <strong>%s</strong>?";
#endif

const char *Txt_document =
#if   L==0
	"document";
#elif L==1
	"Dokument";
#elif L==2
	"document";
#elif L==3
	"documento";
#elif L==4
	"document";
#elif L==5
	"documento";	// Okoteve traduccin
#elif L==6
	"documento";
#elif L==7
	"document";	// Potrzebujesz tlumaczenie
#elif L==8
	"documento";
#endif

const char *Txt_Document =
#if   L==0
	"Document";
#elif L==1
	"Dokument";
#elif L==2
	"Document";
#elif L==3
	"Documento";
#elif L==4
	"Document";
#elif L==5
	"Documento";	// Okoteve traduccin
#elif L==6
	"Documento";
#elif L==7
	"Document";	// Potrzebujesz tlumaczenie
#elif L==8
	"Documento";
#endif

const char *Txt_document_hidden =
#if   L==0
	"document ocult";
#elif L==1
	"Dokument verborgen";
#elif L==2
	"documen hidden";
#elif L==3
	"documento oculto";
#elif L==4
	"document cach&eacute;";
#elif L==5
	"documento oculto";	// Okoteve traduccin
#elif L==6
	"documento nascosto";
#elif L==7
	"document hidden";	// Potrzebujesz tlumaczenie
#elif L==8
	"documento oculto";
#endif

const char *Txt_Documents =
#if   L==0
	"Documents";
#elif L==1
	"Dokumente";
#elif L==2
	"Documents";
#elif L==3
	"Documentos";
#elif L==4
	"Documents";
#elif L==5
	"Documentos";	// Okoteve traduccin
#elif L==6
	"Documenti";
#elif L==7
	"Documents";	// Potrzebujesz tlumaczenie
#elif L==8
	"Documentos";
#endif

const char *Txt_documents =
#if   L==0
	"documents";
#elif L==1
	"Dokumente";
#elif L==2
	"documents";
#elif L==3
	"documentos";
#elif L==4
	"documents";
#elif L==5
	"documentos";	// Okoteve traduccin
#elif L==6
	"documenti";
#elif L==7
	"documents";	// Potrzebujesz tlumaczenie
#elif L==8
	"documentos";
#endif

const char *Txt_Documents_in_my_courses =
#if   L==0
	"Documents en les meves assignatures";
#elif L==1
	"Dokumente in meinen Kursen";
#elif L==2
	"Documents in my courses";
#elif L==3
	"Documentos en mis asignaturas";
#elif L==4
	"Documents &agrave; mes mati&egrave;res";
#elif L==5
	"Documentos en mis asignaturas";	// Okoteve traduccin
#elif L==6
	"Documenti in miei corsi";
#elif L==7
	"Dokumenty w moich kursach";
#elif L==8
	"Documentos em minha disciplinas";
#endif

const char *Txt_documents_hidden =
#if   L==0
	"documents ocults";
#elif L==1
	"Dokumente verborgen";
#elif L==2
	"documents hidden";
#elif L==3
	"documentos ocultos";
#elif L==4
	"documents cach&eacute;s";
#elif L==5
	"documentos ocultos";	// Okoteve traduccin
#elif L==6
	"documenti nascosti";
#elif L==7
	"documents hidden";	// Potrzebujesz tlumaczenie
#elif L==8
	"documentos ocultos";
#endif

const char *Txt_Documents_zone =
#if   L==0
	"Zona de documents";
#elif L==1
	"Dokumente-Bereich";
#elif L==2
	"Documents area";
#elif L==3
	"Zona de documentos";
#elif L==4
	"Zone de documents";
#elif L==5
	"Zona de documentos";	// Okoteve traduccin
#elif L==6
	"Area documenti";
#elif L==7
	"Obszar Dokument&oacute;w";
#elif L==8
	"Zona de documentos";
#endif

const char *Txt_Documents_management_zone =
#if   L==0
	"Zona d'administraci&oacute; de documents";
#elif L==1
	"Dokumente-Verwaltung";
#elif L==2
	"Documents management area";
#elif L==3
	"Zona de administraci&oacute;n de documentos";
#elif L==4
	"Zone de gestion de documents";
#elif L==5
	"Zona de administraci&oacute;n de documentos";	// Okoteve traduccin
#elif L==6
	"Area di gestione dei documenti";
#elif L==7
	"Obszar zarzadzania Dokumenty";
#elif L==8
	"Zona de gest&atilde;o de documentos";
#endif

const char *Txt_documents_management_zone =
#if   L==0
	"zona de administraci&oacute;n de documentos";	// Necessita traduccio
#elif L==1
	"Dokumente-Verwaltung";
#elif L==2
	"documents management area";
#elif L==3
	"zona de administraci&oacute;n de documentos";
#elif L==4
	"zone de gestion de documents";
#elif L==5
	"zona de administraci&oacute;n de documentos";	// Okoteve traduccin
#elif L==6
	"area di gestione dei documenti";
#elif L==7
	"obszaru zarzadzania dokumentami";
#elif L==8
	"zona de gest&atilde;o de documentos";
#endif

const char *Txt_Done_assess_exam =
#if   L==0
	"He acabat; corregir examen";
#elif L==1
	"Geschehen; beurteilen Pr&uuml;fung";
#elif L==2
	"Done; assess exam";
#elif L==3
	"He terminado; corregir examen";
#elif L==4
	"Fait; corriger l'examen";
#elif L==5
	"He terminado; corregir examen";	// Okoteve traduccin
#elif L==6
	"Finito; correggere l'esame";
#elif L==7
	"Sporz&aogon;dzono; oceny egzaminu";
#elif L==8
	"Feito; corrigir exame";
#endif

const char *Txt_Download =
#if   L==0
	"Descarregar";
#elif L==1
	"Download";
#elif L==2
	"Download";
#elif L==3
	"Descargar";
#elif L==4
	"T&eacute;l&eacute;charger";
#elif L==5
	"Descargar";	// Okoteve traduccin
#elif L==6
	"Scarica";
#elif L==7
	"Pobierz";
#elif L==8
	"Baixar";
#endif

const char *Txt_Download_FILE_OR_LINK_X =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"Descarregar %s";
#elif L==1
	"Laden Sie %s";
#elif L==2
	"Download %s";
#elif L==3
	"Descargar %s";
#elif L==4
	"T&eacute;l&eacute;charger %s";
#elif L==5
	"Descargar %s";		// Okoteve traduccin
#elif L==6
	"Scarica %s";
#elif L==7
	"Pobierz %s";
#elif L==8
	"Download %s";		// Necessita de traduo
#endif

const char *Txt_Edit =
#if   L==0
	"Editar";
#elif L==1
	"Bearbeiten";
#elif L==2
	"Edit";
#elif L==3
	"Editar";
#elif L==4
	"&Eacute;diter";
#elif L==5
	"Jehaijey";
#elif L==6
	"Editare";
#elif L==7
	"Edytuj";
#elif L==8
	"Editar";
#endif

const char *Txt_Edit_announcement_of_exam =
#if   L==0
	"Editar convocatoria de examen";	// Necessita traduccio
#elif L==1
	"Aufruf f&uuml;r Pr&uuml;fung bearbeiten";
#elif L==2
	"Edit announcement of exam";
#elif L==3
	"Editar convocatoria de examen";
#elif L==4
	"&Eacute;diter convocation &agrave; l'examen";
#elif L==5
	"Editar convocatoria de examen";	// Okoteve traduccin
#elif L==6
	"Editare sessione d'esame";
#elif L==7
	"Edytuj ogloszenie egzaminu";
#elif L==8
	"Editar chamada para o exame";
#endif

const char *Txt_Edit_assignment =
#if   L==0
	"Editar actividad";	// Necessita traduccio
#elif L==1
	"Aufgabe bearbeiten";
#elif L==2
	"Edit assignment";
#elif L==3
	"Editar actividad";
#elif L==4
	"&Eacute;diter activit&eacute;";
#elif L==5
	"Editar actividad";	// Okoteve traduccin
#elif L==6
	"Editare attivit&agrave;";
#elif L==7
	"Edytuj zadanie";
#elif L==8
	"Editar atividade";
#endif

const char *Txt_Edit_event =
#if   L==0
	"Editar evento";	// Necessita traduccio
#elif L==1
	"Ereignis bearbeiten";
#elif L==2
	"Edit event";
#elif L==3
	"Editar evento";
#elif L==4
	"&Eacute;diter &eacute;v&eacute;nement";
#elif L==5
	"Editar evento";	// Okoteve traduccin
#elif L==6
	"Editare evento";
#elif L==7
	"Edytuj wydarzenie";
#elif L==8
	"Editar evento";
#endif

const char *Txt_Edit_my_institution =
#if   L==0
	"Edita la meva instituci&oacute;";
#elif L==1
	"Bearbeiten meine Hochschule";
#elif L==2
	"Edit my institution";
#elif L==3
	"Editar mi instituci&oacute;n";
#elif L==4
	"&Eacute;diter mon &eacute;tablissement";
#elif L==5
	"Editar mi instituci&oacute;n";		// Okoteve traduccin
#elif L==6
	"Editare la mia istituzione";
#elif L==7
	"Edycja moich instytucji";
#elif L==8
	"Editar minha institu&ccedil;&atilde;o";
#endif

const char *Txt_Edit_my_personal_data =
#if   L==0
	"Edita meves dades personals";
#elif L==1
	"Bearbeiten meiner pers&ouml;nlichen Daten";
#elif L==2
	"Edit my personal data";
#elif L==3
	"Editar mis datos personales";
#elif L==4
	"&Eacute;diter mes donn&eacute;es personnelles";
#elif L==5
	"Editar mis datos personales";		// Okoteve traduccin
#elif L==6
	"Editare i miei dati personali";
#elif L==7
	"Edycja moich danych osobowych";
#elif L==8
	"Editar os meus dados pessoais";
#endif

const char *Txt_Edit_my_webs_networks =
#if   L==0
	"Edita meves webs / xarxes";
#elif L==1
	"Bearbeiten meiner Webs / Netzwerke";
#elif L==2
	"Edit my webs / networks";
#elif L==3
	"Editar mis webs / redes";
#elif L==4
	"&Eacute;diter mes webs / r&eacute;seaux";
#elif L==5
	"Editar mis webs / redes";	// Okoteve traduccin
#elif L==6
	"Editare i miei webs / reti";
#elif L==7
	"Edycja moje webs / sieci";
#elif L==8
	"Editar as minhas webs / redes";
#endif

const char *Txt_Edit_office_hours =
#if   L==0
	"Editar tutories";
#elif L==1
	"Gesch&auml;ftszeiten bearbeiten";
#elif L==2
	"Edit office hours";
#elif L==3
	"Editar tutor&iacute;as";
#elif L==4
	"&Eacute;diter tuteur";
#elif L==5
	"Editar tutor&iacute;as";	// Okoteve traduccin
#elif L==6
	"Editare tutoraggio";
#elif L==7
	"Edytuj godzin pracy biura";
#elif L==8
	"Editar tutor";
#endif

const char *Txt_Edit_question =
#if   L==0
	"Editar pregunta";
#elif L==1
	"Frage bearbeiten";
#elif L==2
	"Edit question";
#elif L==3
	"Editar pregunta";
#elif L==4
	"&Eacute;diter question";
#elif L==5
	"Editar pregunta";	// Okoteve traduccin
#elif L==6
	"Editare domanda";
#elif L==7
	"Edytuj pytanie";
#elif L==8
	"Editar quest&atilde;o";
#endif

const char *Txt_Edit_record_fields =
#if   L==0
	"Editar camps de fitxes";
#elif L==1
	"Datensatzfelder bearbeiten";
#elif L==2
	"Edit record fields";
#elif L==3
	"Editar campos de fichas";
#elif L==4
	"Modifier champs de cartes";
#elif L==5
	"Editar campos de fichas";	// Okoteve traduccin
#elif L==6
	"Editare campo schede";
#elif L==7
	"Edycja rekord w dziedzinie";
#elif L==8
	"Editar campos cart&otilde;es";
#endif

const char *Txt_Edit_survey =
#if   L==0
	"Editar enquesta";
#elif L==1
	"Umfrage bearbeiten";
#elif L==2
	"Edit survey";
#elif L==3
	"Editar encuesta";
#elif L==4
	"&Eacute;diter sondage";
#elif L==5
	"Editar encuesta";	// Okoteve traduccin
#elif L==6
	"Editare sondaggio";
#elif L==7
	"Edycja ankiety";
#elif L==8
	"Editar inqu&eacute;rito";
#endif

const char *Txt_Edit_text =
#if   L==0
	"Editar text";
#elif L==1
	"Text bearbeiten";
#elif L==2
	"Edit text";
#elif L==3
	"Editar texto";
#elif L==4
	"&Eacute;diter texte";
#elif L==5
	"Editar texto";	// Okoteve traduccin
#elif L==6
	"Editare testo";
#elif L==7
	"Edycja tekstu";
#elif L==8
	"Editar texto";
#endif

const char *Txt_eg_A_B =
#if   L==0
	"per ex.: A, B,...";
#elif L==1
	"z. B.: A, B,...";
#elif L==2
	"e.g., A, B,...";
#elif L==3
	"por ej.: A, B,...";
#elif L==4
	"par exemple: A, B,...";
#elif L==5
	"por ej.: A, B,...";	// Okoteve traduccin
#elif L==6
	"ad esempio: A, B,...";
#elif L==7
	"np. A, B, ...";
#elif L==8
	"ex.: A, B,...";
#endif

const char *Txt_eg_Lectures_Practicals =
#if   L==0
	"per ex.: Teoria, Pr&agrave;ctiques,...";
#elif L==1
	"z. B.: Vorlesung, &Uuml;bung,...";
#elif L==2
	"e.g., Lectures, Practicals,...";
#elif L==3
	"por ej.: Teor&iacute;a, Pr&aacute;cticas,...";
#elif L==4
	"par exemple: Th&eacute;orie, Pratique,...";
#elif L==5
	"por ej.: Teor&iacute;a, Pr&aacute;cticas,...";	// Okoteve traduccin
#elif L==6
	"ad esempio: Teoria, Pratica,...";
#elif L==7
	"np. Wyklady, Praktyki ...";
#elif L==8
	"ex.: Teoria, Pr&aacute;tica,...";
#endif

const char *Txt_Eliminate =
#if   L==0
	"Eliminar";
#elif L==1
	"Entfernen";
#elif L==2
	"Eliminate";
#elif L==3
	"Eliminar";
#elif L==4
	"&Eacute;liminer";
#elif L==5
	"Mbogue";
#elif L==6
	"Rimuovere";
#elif L==7
	"Wyeliminowac";
#elif L==8
	"Eliminar";
#endif

const char *Txt_Eliminate_all_courses_whithout_users_PART_1_OF_2 =
#if   L==0
	"Eliminar todas las asignaturas sin usuarios y sin accesos en los &uacute;ltimos";	// Necessita traduccio
#elif L==1
	"Eliminate all courses without users and without accesses in the last";			// Need bersetzung
#elif L==2
	"Eliminate all courses without users and without accesses in the last";
#elif L==3
	"Eliminar todas las asignaturas sin usuarios y sin accesos en los &uacute;ltimos";
#elif L==4
	"Eliminate all courses without users and without accesses in the last";			// Besoin de traduction
#elif L==5
	"Eliminar todas las asignaturas sin usuarios y sin accesos en los &uacute;ltimos";	// Okoteve traduccin
#elif L==6
	"Eliminate all courses without users and without accesses in the last";			// Bisogno di traduzione
#elif L==7
	"Eliminate all courses without users and without accesses in the last";			// Potrzebujesz tlumaczenie
#elif L==8
	"Eliminate all courses without users and without accesses in the last";			// Necessita de traduo
#endif

const char *Txt_Eliminate_all_courses_whithout_users_PART_2_OF_2 =
#if   L==0
	"meses.";		// Necessita traduccio
#elif L==1
	"months.";		// Need bersetzung
#elif L==2
	"months.";
#elif L==3
	"meses.";
#elif L==4
	"months.";		// Besoin de traduction
#elif L==5
	"meses.";		// Okoteve traduccin
#elif L==6
	"mesi.";
#elif L==7
	"months.";		// Potrzebujesz tlumaczenie
#elif L==8
	"months.";		// Necessita de traduo
#endif

const char *Txt_Eliminate_all_users_who_are_not_enrolled_on_any_courses_PART_1_OF_2 =
#if   L==0
	"Eliminar todos los usuarios que no est&eacute;n inscritos en ninguna asignatura y que lleven m&aacute;s de";	// Necessita traduccio
#elif L==1
	"Eliminate all users who are not enrolled on any courses and with more than";					// Need bersetzung
#elif L==2
	"Eliminate all users who are not enrolled on any courses and with more than";
#elif L==3
	"Eliminar todos los usuarios que no est&eacute;n inscritos en ninguna asignatura y que lleven m&aacute;s de";
#elif L==4
	"Eliminate all users who are not enrolled on any courses and with more than";					// Besoin de traduction
#elif L==5
	"Eliminar todos los usuarios que no est&eacute;n inscritos en ninguna asignatura y que lleven m&aacute;s de";	// Okoteve traduccin
#elif L==6
	"Rimuovere tutti gli utenti che non sono registrati ad alcun corso e con pi&ugrave; di";
#elif L==7
	"Wyeliminuj wszystkich uzytkownik&oacute;w, kt&oacute;rzy nie sa zarejestrowani na kazdym z kurs&oacute;w";
#elif L==8
	"Eliminate all users who are not enrolled on any courses and with more than";					// Necessita de traduo
#endif

const char *Txt_Eliminate_all_users_who_are_not_enrolled_on_any_courses_PART_2_OF_2 =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"meses sin acceder a %s.";		// Necessita traduccio
#elif L==1
	"months without access to %s.";		// Need bersetzung
#elif L==2
	"months without access to %s.";
#elif L==3
	"meses sin acceder a %s.";
#elif L==4
	"months without access to %s.";		// Besoin de traduction
#elif L==5
	"meses sin acceder a %s.";		// Okoteve traduccin
#elif L==6
	"mesi senza accedere a %s.";
#elif L==7
	"months without access to %s.";		// Potrzebujesz tlumaczenie
#elif L==8
	"months without access to %s.";		// Necessita de traduo
#endif

const char *Txt_Eliminate_from_the_platform_the_users_indicated_in_step_1 =
#if   L==0
	"Eliminar completamente de la plataforma"
	" los usuarios indicados en el paso 1";		// Necessita traduccio
#elif L==1
	"Die Benutzer auf der Liste aus dem System entfernen";
#elif L==2
	"Eliminate from the platform"
	" the users indicated on step 1";
#elif L==3
	"Eliminar completamente de la plataforma"
	" los usuarios indicados en el paso 1";
#elif L==4
	"&Eacute;liminer de la plate-forme"
	" les utilisateurs qui sont sur la liste";
#elif L==5
	"Eliminar completamente de la plataforma"
	" los usuarios indicados en el paso 1";		// Okoteve traduccin
#elif L==6
	"Rimuovere dalla piattaforma"
	" gli utenti che sono in lista";
#elif L==7
	"Wyeliminuj z platformy"
	" uzytkownikow kt&oacute;rzy sa na liscie";
#elif L==8
	"Eliminar da plataforma"
	" os utilizadores que est&atilde;o na lista";
#endif

const char *Txt_Eliminating_X_courses_whithout_users_and_with_more_than_Y_months_without_access =	// Warning: it is very important to include %lu and %u in the following sentences
#if   L==0
	"Eliminando %lu asignatura(s) sin usuarios"
	" y que llevaba(n) m&aacute;s de %u meses sin accesos.";	// Necessita traduccio
#elif L==1
	"Eliminating %lu course(s) without users"
	" and with more than %u months without access.";		// Need bersetzung
#elif L==2
	"Eliminating %lu course(s) without users"
	" and with more than %u months without access.";
#elif L==3
	"Eliminando %lu asignatura(s) sin usuarios"
	" y que llevaba(n) m&aacute;s de %u meses sin accesos.";
#elif L==4
	"Eliminating %lu course(s) without users"
	" and with more than %u months without access.";		// Besoin de traduction
#elif L==5
	"Eliminando %lu asignatura(s) sin usuarios"
	" y que llevaba(n) m&aacute;s de %u meses sin accesos.";	// Okoteve traduccin
#elif L==6
	"Eliminating %lu course(s) without users"
	" and with more than %u months without access.";		// Bisogno di traduzione
#elif L==7
	"Eliminating %lu course(s) without users"
	" and with more than %u months without access.";		// Potrzebujesz tlumaczenie
#elif L==8
	"Eliminating %lu course(s) without users"
	" and with more than %u months without access.";		// Necessita de traduo
#endif

const char *Txt_Eliminating_X_users_who_were_not_enrolled_in_any_course_and_with_more_than_Y_months_without_access_to_Z =	// Warning: it is very important to include %lu, %u and %s in the following sentences
#if   L==0
	"Eliminando %lu usuario(s) que no estaba(n) inscrito(s) en ninguna asignatura"
	" y que llevaba(n) m&aacute;s de %u meses sin acceder a %s.";	// Necessita traduccio
#elif L==1
	"Eliminating %lu user(s) who were not enrolled in any course"
	" and with more than %u months without access to %s.";		// Need bersetzung
#elif L==2
	"Eliminating %lu user(s) who were not enrolled in any course"
	" and with more than %u months without access to %s.";
#elif L==3
	"Eliminando %lu usuario(s) que no estaba(n) inscrito(s) en ninguna asignatura"
	" y que llevaba(n) m&aacute;s de %u meses sin acceder a %s.";
#elif L==4
	"Eliminating %lu user(s) who were not enrolled in any course"
	" and with more than %u months without access to %s.";		// Besoin de traduction
#elif L==5
	"Eliminando %lu usuario(s) que no estaba(n) inscrito(s) en ninguna asignatura"
	" y que llevaba(n) m&aacute;s de %u meses sin acceder a %s.";	// Okoteve traduccin
#elif L==6
	"Rimuovendo %lu utenti che non siano registrati in alcun corso"
	" e con pi&ugrave; di %u mesi senza accedere a %s.";
#elif L==7
	"Eliminowanie uzytkownik&oacute;w, kt&oacute;rzy nie lu% odnotowano w kurs"
	" i ponad miesiac u% bez dostepu do %s.";
#elif L==8
	"Eliminating %lu user(s) who were not enrolled in any course"
	" and with more than %u months without access to %s.";		// Necessita de traduo
#endif

const char *Txt_Email =
#if   L==0
	"Correu electr&ograve;nic";
#elif L==1
	"E-Mail";
#elif L==2
	"E-mail";
#elif L==3
	"Correo electr&oacute;nico";
#elif L==4
	"Courrier &eacute;lectronique";
#elif L==5
	"&Ntilde;e'&#7869;veve";
#elif L==6
	"E-mail";
#elif L==7
	"E-mail";
#elif L==8
	"E-mail";
#endif

const char *Txt_Email_X_confirmed =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"Correo electr&oacute;nico %s confirmado";	// Necessita traduccio
#elif L==1
	"E-mail %s confirmed";				// Need bersetzung
#elif L==2
	"E-mail %s confirmed";
#elif L==3
	"Correo electr&oacute;nico %s confirmado";
#elif L==4
	"E-mail %s confirmed";				// Besoin de traduction
#elif L==5
	"Correo electr&oacute;nico %s confirmado";	// Okoteve traduccin
#elif L==6
	"E-mail %s confirmed";				// Bisogno di traduzione
#elif L==7
	"E-mail %s confirmed";				// Potrzebujesz tlumaczenie
#elif L==8
	"E-mail %s confirmed";				// Necessita de traduo
#endif

const char *Txt_Email_X_has_already_been_confirmed_before =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"El correo electr&oacute;nico <strong>%s</strong>"
	" ya hab&iacute;a sido confirmado anteriormente.";	// Necessita traduccio
#elif L==1
	"E-mail <strong>%s</strong>"
	" has already been confirmed before.";			// Need bersetzung
#elif L==2
	"E-mail <strong>%s</strong>"
	" has already been confirmed before.";
#elif L==3
	"El correo electr&oacute;nico <strong>%s</strong>"
	" ya hab&iacute;a sido confirmado anteriormente.";
#elif L==4
	"E-mail <strong>%s</strong>"
	" has already been confirmed before.";			// Besoin de traduction
#elif L==5
	"El correo electr&oacute;nico <strong>%s</strong>"
	" ya hab&iacute;a sido confirmado anteriormente.";	// Okoteve traduccin
#elif L==6
	"E-mail <strong>%s</strong>"
	" has already been confirmed before.";			// Bisogno di traduzione
#elif L==7
	"E-mail <strong>%s</strong>"
	" has already been confirmed before.";			// Potrzebujesz tlumaczenie
#elif L==8
	"E-mail <strong>%s</strong>"
	" has already been confirmed before.";			// Necessita de traduo
#endif

const char *Txt_Email_new_password =
#if   L==0
	"Enviarme una nueva contrase&ntilde;a";	// Necessita traduccio
#elif L==1
	"Neues Passwort beantragen";
#elif L==2
	"E-mail new password";
#elif L==3
	"Enviarme una nueva contrase&ntilde;a";
#elif L==4
	"Envoyez-moi un nouveau mot de passe";
#elif L==5
	"Enviarme una nueva contrase&ntilde;a";	// Okoteve traduccin
#elif L==6
	"Invia una nuova password";
#elif L==7
	"E-mail nowe has&lstrok;o";
#elif L==8
	"Enviar uma nova senha";
#endif

const char *Txt_Email_X_removed =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"Correu electr&ograve;nic <strong>%s</strong> eliminat.";
#elif L==1
	"E-Mail <strong>%s</strong> entfernt.";
#elif L==2
	"E-mail <strong>%s</strong> removed.";
#elif L==3
	"Correo electr&oacute;nico <strong>%s</strong> eliminado.";
#elif L==4
	"Courrier &eacute;lectronique <strong>%s</strong> supprim&eacute;.";
#elif L==5
	"Correo electr&oacute;nico <strong>%s</strong> eliminado.";	// Okoteve traduccin
#elif L==6
	"E-mail <strong>%s</strong> rimosso.";
#elif L==7
	"E-mail <strong>%s</strong> usuniete.";
#elif L==8
	"E-mail <strong>%s</strong> removido.";
#endif

const char *Txt_End_date =
#if   L==0
	"Data final";
#elif L==1
	"Enddatum";
#elif L==2
	"End date";
#elif L==3
	"Fecha final";
#elif L==4
	"Date finale";
#elif L==5
	"Fecha final";	// Okoteve traduccin
#elif L==6
	"Data finale";
#elif L==7
	"Data koncowa";
#elif L==8
	"Data final";
#endif

const char *Txt_Enroll_in_groups =
#if   L==0
	"Inscriure en grups";
#elif L==1
	"Anmeldung mich in Gruppen";
#elif L==2
	"Enroll in groups";
#elif L==3
	"Inscribirme en grupos";
#elif L==4
	"Inscriver moi dans groupes";
#elif L==5
	"Inscribirme en grupos";	// Okoteve traduccin
#elif L==6
	"Iscriver in gruppi";
#elif L==7
	"Zarejestruj mnie w grupach";
#elif L==8
	"Inscreva-me em grupos ";
#endif

const char *Txt_Enrollment_of_X_rejected =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"Inscripci&oacute;n de <strong>%s</strong> rechazada.";	// Necessita traduccio
#elif L==1
	"Enrollment of <strong>%s</strong> rejected.";		// Need bersetzung
#elif L==2
	"Enrollment of <strong>%s</strong> rejected.";
#elif L==3
	"Inscripci&oacute;n de <strong>%s</strong> rechazada.";
#elif L==4
	"Enrollment of <strong>%s</strong> rejected.";		// Besoin de traduction
#elif L==5
	"Inscripci&oacute;n de <strong>%s</strong> rechazada.";	// Okoteve traduccin
#elif L==6
	"Enrollment of <strong>%s</strong> rejected.";		// Bisogno di traduzione
#elif L==7
	"Enrollment of <strong>%s</strong> rejected.";		// Potrzebujesz tlumaczenie
#elif L==8
	"Enrollment of <strong>%s</strong> rejected.";		// Necessita de traduo
#endif

const char *Txt_Enter_a_new_item_here =
#if   L==0
	"Escriba aqu&iacute; un nuevo apartado";	// Necessita traduccio
#elif L==1
	"Neuen Eintrag hier eingeben";
#elif L==2
	"Enter a new item here";
#elif L==3
	"Escriba aqu&iacute; un nuevo apartado";
#elif L==4
	"Ecrivez ici un nouveau point";
#elif L==5
	"Escriba aqu&iacute; un nuevo apartado";	// Okoteve traduccin
#elif L==6
	"Inserisci qui un nuovo oggetto(tambi&egrave;n item est&agrave; bien)";
#elif L==7
	"Wpisz nowy element o";
#elif L==8
	"Escreva aqui uma nova rubrica";
#endif

const char *Txt_Enter_from_X =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"Entrar des %s";
#elif L==1
	"Geben Sie %s";
#elif L==2
	"Enter from %s";
#elif L==3
	"Entrar desde %s";
#elif L==4
	"Entrez du %s";
#elif L==5
	"Entrar desde %s";	// Okoteve traduccin
#elif L==6
	"Inserisci da %s";
#elif L==7
	"Wprowadzi&cacute; z %s";
#elif L==8
	"Digite partir %s";
#endif

const char *Txt_Enter_the_ID_of_the_user_you_want_to_register_remove_ =
#if   L==0
	"Escriu el @sobrenom, correu o ID (DNI/c&egrave;dula)"
	" de l'usuari que desitja inscriure/eliminar"
	" o les dades voleu modificar.";
#elif L==1
	"Enter the @nick, e-mail or ID"
	" of the user you want to register/remove"
	" or whose data you want to modify.";		// Need bersetzung
#elif L==2
	"Enter the @nick, e-mail or ID"
	" of the user you want to register/remove"
	" or whose data you want to modify.";
#elif L==3
	"Escriba el @apodo, correo o ID (DNI/c&eacute;dula)"
	" del usuario que desea inscribir/eliminar"
	" o cuyos datos desea modificar.";
#elif L==4
	"Enter the @nick, e-mail or ID"
	" of the user you want to register/remove"
	" or whose data you want to modify.";		// Besoin de traduction
#elif L==5
	"Escriba el @apodo, correo o ID (DNI/c&eacute;dula)"
	" del usuario que desea inscribir/eliminar"
	" o cuyos datos desea modificar.";		// Okoteve traduccin
#elif L==6
	"Introduci il @nome-utente, e-mail o ID"
	" dell'utente che vuoi registrare/rimuovere"
	" o la data di chi vuoi modificare.";
#elif L==7
	"Enter the @nick, e-mail or ID"
	" of the user you want to register/remove"
	" or whose data you want to modify.";		// Potrzebujesz tlumaczenie
#elif L==8
	"Enter the @nick, e-mail or ID"
	" of the user you want to register/remove"
	" or whose data you want to modify.";		// Necessita de traduo
#endif

const char *Txt_Error_getting_data_from_a_recipient =
#if   L==0
	"Error obteniendo datos de un destinatario.";	// Necessita traduccio
#elif L==1
	"Error getting data from a recipient.";		// Need bersetzung
#elif L==2
	"Error getting data from a recipient.";
#elif L==3
	"Error obteniendo datos de un destinatario.";
#elif L==4
	"Error getting data from a recipient.";		// Besoin de traduction
#elif L==5
	"Error obteniendo datos de un destinatario.";	// Okoteve traduccin
#elif L==6
	"Errore acquisizione dati da un destinatario.";
#elif L==7
	"Blad podczas pobierania danych z odbiorca.";
#elif L==8
	"Error getting data from a recipient.";		// Necessita de traduo
#endif

const char *Txt_Event =
#if   L==0
	"Esdeveniment";
#elif L==1
	"Ereignis";
#elif L==2
	"Event";
#elif L==3
	"Evento";
#elif L==4
	"&Eacute;v&eacute;nement";
#elif L==5
	"Evento";	// Okoteve traduccin
#elif L==6
	"Evento";
#elif L==7
	"Wydarzenie";
#elif L==8
	"Evento";
#endif

const char *Txt_Event_X_is_now_hidden =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"L'esdeveniment <strong>%s</strong> ara est&agrave; ocult.";
#elif L==1
	"Der Ereignis <strong>%s</strong> ist jetzt ausgeblendet.";
#elif L==2
	"Event <strong>%s</strong> is now hidden.";
#elif L==3
	"El evento <strong>%s</strong> ahora est&aacute; oculto.";
#elif L==4
	"L'&eacute;v&eacute;nement <strong>%s</strong> est maintenant cach&eacute;.";
#elif L==5
	"El evento <strong>%s</strong> ahora est&aacute; oculto.";	// Okoteve traduccin
#elif L==6
	"L'evento <strong>%s</strong> &egrave; ora nascosto.";
#elif L==7
	"Wydarzenie <strong>%s</strong> jest ukryte.";
#elif L==8
	"O evento <strong>%s</strong> &eacute; agora oculto.";
#endif

const char *Txt_Event_X_is_now_visible =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"L'esdevenimen <strong>%s</strong> ara est&agrave; visible.";
#elif L==1
	"Der Ereignis <strong>%s</strong> ist jetzt sichtbar.";
#elif L==2
	"Event <strong>%s</strong> is now visible.";
#elif L==3
	"El evento <strong>%s</strong> ahora est&aacute; visible.";
#elif L==4
	"L'&eacute;v&eacute;nement <strong>%s</strong> est maintenant visible.";
#elif L==5
	"El evento <strong>%s</strong> ahora est&aacute; visible.";	// Okoteve traduccin
#elif L==6
	"L'evento <strong>%s</strong> &egrave; ora visibile.";
#elif L==7
	"Wydarzenie <strong>%s</strong> jest teraz widoczny.";
#elif L==8
	"O evento <strong>%s</strong> &eacute; agora vis&iacute;vel.";
#endif

const char *Txt_Event_X_removed =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"Esdeveniment <strong>%s</strong> eliminat.";
#elif L==1
	"Ereignis <strong>%s</strong> entfernt.";
#elif L==2
	"Event <strong>%s</strong> removed.";
#elif L==3
	"Evento <strong>%s</strong> eliminado.";
#elif L==4
	"&Eacute;v&eacute;nement <strong>%s</strong> supprim&eacute;.";
#elif L==5
	"Event <strong>%s</strong> eliminado.";	// Okoteve traduccin
#elif L==6
	"Evento <strong>%s</strong> rimosso.";
#elif L==7
	"Wydarzenie <strong>%s</strong> usuniete.";
#elif L==8
	"Evento <strong>%s</strong> removido.";
#endif

const char *Txt_Events =
#if   L==0
	 "Esdeveniments";
#elif L==1
	 "Ereignisse";
#elif L==2
	 "Events";
#elif L==3
	 "Eventos";
#elif L==4
	 "&Eacute;v&eacute;nements";
#elif L==5
	 "Eventos";	// Okoteve traduccin
#elif L==6
	 "Eventi";
#elif L==7
	 "Wydarzen";
#elif L==8
	 "Eventos";
#endif

const char *Txt_Exam =
#if   L==0
	"ex&agrave;men";
#elif L==1
	"Pr&uuml;fung";
#elif L==2
	"Exam";
#elif L==3
	"Examen";
#elif L==4
	"Examen";
#elif L==5
	"Kuaara'&atilde;";
#elif L==6
	"Esame";
#elif L==7
	"Egzamin";
#elif L==8
	"Exame";
#endif

const char *Txt_exam =
#if   L==0
	"ex&agrave;men";
#elif L==1
	"Pr&uuml;fung";
#elif L==2
	"exam";
#elif L==3
	"examen";
#elif L==4
	"examen";
#elif L==5
	"kuaara'&atilde;";
#elif L==6
	"esame";
#elif L==7
	"egzamin";
#elif L==8
	"exame";
#endif

const char *Txt_EXAM_ANNOUNCEMENT =
#if   L==0
	"CONVOCAT&Ograve;RIA D'EXAMEN";
#elif L==1
	"PR&Uuml;FUNGSAUFRUF";
#elif L==2
	"ANNOUNCEMENT OF EXAM";
#elif L==3
	"CONVOCATORIA DE EXAMEN";
#elif L==4
	"CONVOCATION &Agrave; UN EXAMEN";
#elif L==5
	"CONVOCATORIA DE EXAMEN";	// Okoteve traduccin
#elif L==6
	"SESSIONE D'ESAME";
#elif L==7
	"OGLOSZENIE egzaminu";
#elif L==8
	"CHAMADA DE EXAME";
#endif

const char *Txt_EXAM_ANNOUNCEMENT_Approximate_duration =
#if   L==0
	"Duraci&oacute;n aproximada";	// Necessita traduccio
#elif L==1
	"Voraussichtliche Dauer";
#elif L==2
	"Approximate duration";
#elif L==3
	"Duraci&oacute;n aproximada";
#elif L==4
	"Dur&eacute;e approximative";
#elif L==5
	"Duraci&oacute;n aproximada";	// Okoteve traduccin
#elif L==6
	"Durata approssimativa";
#elif L==7
	"Przyblizony czas trwania";
#elif L==8
	"Dura&ccedil;&atilde;o aproximada";
#endif

const char *Txt_EXAM_ANNOUNCEMENT_Course =
#if   L==0
	"Assignatura";
#elif L==1
	"Kurs";
#elif L==2
	"Course";
#elif L==3
	"Asignatura";
#elif L==4
	"Mati&egrave;re";
#elif L==5
	"Mbo'esyry";
#elif L==6
	"Corso";
#elif L==7
	"Kurs";
#elif L==8
	"Disciplina";
#endif

const char *Txt_EXAM_ANNOUNCEMENT_Documentation_required =
#if   L==0
	"Documentaci&oacute;n exigida";	// Necessita traduccio
#elif L==1
	"Ausweis erforderlich";
#elif L==2
	"ID required";
#elif L==3
	"Documentaci&oacute;n exigida";
#elif L==4
	"Pi&egrave;ce d'identit obligatoire";
#elif L==5
	"Documentaci&oacute;n exigida";	// Okoteve traduccin
#elif L==6
	"Documentazione richiesta";
#elif L==7
	"ID wymagane";
#elif L==8
	"Documenta&ccedil;&atilde;o necess&aacute;ria";
#endif

const char *Txt_EXAM_ANNOUNCEMENT_Exam_date =
#if   L==0
	"Fecha del examen";	// Necessita traduccio
#elif L==1
	"Pr&uuml;fungsdatum";
#elif L==2
	"Exam date";
#elif L==3
	"Fecha del examen";
#elif L==4
	"Date de l'examen";
#elif L==5
	"Fecha del examen";	// Okoteve traduccin
#elif L==6
	"Data d'esame";
#elif L==7
	"Termin egzaminu";
#elif L==8
	"Exame data";
#endif

const char *Txt_EXAM_ANNOUNCEMENT_Material_allowed =
#if   L==0
	"Material permitido";	// Necessita traduccio
#elif L==1
	"Erlaubte Unterlagen";
#elif L==2
	"Material allowed";
#elif L==3
	"Material permitido";
#elif L==4
	"Mat&eacute;riel autoris&eacute;";
#elif L==5
	"Material permitido";	// Okoteve traduccin
#elif L==6
	"Materiale permesso";
#elif L==7
	"Material dozwolone";
#elif L==8
	"Material autorizado";
#endif

const char *Txt_EXAM_ANNOUNCEMENT_Material_required =
#if   L==0
	"Material obligatorio";	// Necessita traduccio
#elif L==1
	"Erforderliche Unterlagen";
#elif L==2
	"Material required";
#elif L==3
	"Material obligatorio";
#elif L==4
	"Mat&eacute;riel n&eacute;cessaire";
#elif L==5
	"Material obligatorio";	// Okoteve traduccin
#elif L==6
	"Materiale richiesto";
#elif L==7
	"Materialy niezbedne do";
#elif L==8
	"Material obrigat&oacute;rio";
#endif

const char *Txt_EXAM_ANNOUNCEMENT_Mode =
#if   L==0
	"Modalidad<br />(escrito, oral,...)";	// Necessita traduccio
#elif L==1
	"Pr&uuml;fungsart<br />(schriftlich, m&uuml;ndlich,...)";
#elif L==2
	"Mode<br />(written, oral,...)";
#elif L==3
	"Modalidad<br />(escrito, oral,...)";
#elif L==4
	"Mode<br />(&eacute;crit, oral,...)";
#elif L==5
	"Modalidad<br />(escrito, oral,...)";	// Okoteve traduccin
#elif L==6
	"Modalit&agrave;<br />(scritto, orale,...)";
#elif L==7
	"Tryb<br />(pisemnej, ustnej,...)";
#elif L==8
	"Modalidade<br />(escrita, oral,...)";
#endif

const char *Txt_EXAM_ANNOUNCEMENT_Other_information =
#if   L==0
	"Otras indicaciones";	// Necessita traduccio
#elif L==1
	"Weitere Informationen";
#elif L==2
	"Other information";
#elif L==3
	"Otras indicaciones";
#elif L==4
	"Autres informations";
#elif L==5
	"Otras indicaciones";	// Okoteve traduccin
#elif L==6
	"Altre informazioni";
#elif L==7
	"Inne informacje";
#elif L==8
	"Outras informa&ccedil;&otilde;es";
#endif

const char *Txt_EXAM_ANNOUNCEMENT_Place_of_exam =
#if   L==0
	"Lugar de realizaci&oacute;n";	// Necessita traduccio
#elif L==1
	"Pr&uuml;fungsort";
#elif L==2
	"Place of exam";
#elif L==3
	"Lugar de realizaci&oacute;n";
#elif L==4
	"Lieu de l'examen";
#elif L==5
	"Lugar de realizaci&oacute;n";	// Okoteve traduccin
#elif L==6
	"Luogo d'esame";
#elif L==7
	"Miejsce egzaminu";
#elif L==8
	"Local do exame";
#endif

const char *Txt_EXAM_ANNOUNCEMENT_Session =
#if   L==0
	"Convocatoria<br />(junio, septiembre,...)";	// Necessita traduccio
#elif L==1
	"Einberufung<br />(Juni, September,...)";
#elif L==2
	"Session<br />(june, september,...)";
#elif L==3
	"Convocatoria<br />(junio, septiembre,...)";
#elif L==4
	"P&eacute;riode<br />(juin, septembre,...)";
#elif L==5
	"Convocatoria<br />(junio, septiembre,...)";	// Okoteve traduccin
#elif L==6
	"Sessione<br />(giugno, settembre,...)";
#elif L==7
	"Sesja<br />(czerwiec, wrzesien,...)";
#elif L==8
	"Per&iacute;odo<br />(junho, setembro,...)";
#endif

const char *Txt_EXAM_ANNOUNCEMENT_Start_time =
#if   L==0
	"Hora de inicio";	// Necessita traduccio
#elif L==1
	"Beginn um";
#elif L==2
	"Start time";
#elif L==3
	"Hora de inicio";
#elif L==4
	"Heure de d&eacute;but";
#elif L==5
	"Hora de inicio";	// Okoteve traduccin
#elif L==6
	"Ora d'inizio";
#elif L==7
	"Czas rozpoczecia";
#elif L==8
	"Hora de in&iacute;cio";
#endif

const char *Txt_EXAM_ANNOUNCEMENT_Structure_of_the_exam =
#if   L==0
	"Estructura del examen";	// Necessita traduccio
#elif L==1
	"Pr&uuml;fungsstruktur";
#elif L==2
	"Structure of the exam";
#elif L==3
	"Estructura del examen";
#elif L==4
	"Structure de l'examen";
#elif L==5
	"Estructura del examen";	// Okoteve traduccin
#elif L==6
	"Struttura d'esame";
#elif L==7
	"Struktura egzaminu";
#elif L==8
	"Estrutura do exame";
#endif

const char *Txt_EXAM_ANNOUNCEMENT_Year =	// 1, 2, 3..., not 1984, 2010...
#if   L==0
	"Curs";
#elif L==1
	"Jahr";
#elif L==2
	"Year";
#elif L==3
	"Curso";
#elif L==4
	"Ann&eacute;e";
#elif L==5
	"Curso";		// Okoteve traduccin
#elif L==6
	"Anno";
#elif L==7
	"Rok";
#elif L==8
	"Ano";
#endif

const char *Txt_Exam_of_X =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"Examen de %s";		// Necessita traduccio
#elif L==1
	"Exam of %s";		// Need bersetzung
#elif L==2
	"Exam of %s";
#elif L==3
	"Examen de %s";
#elif L==4
	"Exam of %s";		// Besoin de traduction
#elif L==5
	"Examen de %s";		// Okoteve traduccin
#elif L==6
	"Esame di %s";
#elif L==7
	"Egzamin z %s";
#elif L==8
	"Exam of %s";		// Necessita de traduo
#endif

const char *Txt_exams =
#if   L==0
	"ex&agrave;mens";
#elif L==1
	"Pr&uuml;fungen";
#elif L==2
	"exams";
#elif L==3
	"ex&aacute;menes";
#elif L==4
	"examens";
#elif L==5
	"ex&aacute;menes";	// Okoteve traduccin
#elif L==6
	"esami";
#elif L==7
	"egzaminy";
#elif L==8
	"exames";
#endif

const char *Txt_Existing_question =
#if   L==0
	"Pregunta ja existent";
#elif L==1
	"Bestehende Frage";
#elif L==2
	"Existing question";
#elif L==3
	"Pregunta ya existente";
#elif L==4
	"Question existante";
#elif L==5
	"Pregunta ya existente";	// Okoteve traduccin
#elif L==6
	"Domanda esistente";
#elif L==7
	"Istniej&aogon;ca pytanie";
#elif L==8
	"Quest&atilde;o existente";
#endif

const char *Txt_Expand_FOLDER_X =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"Expandir %s";
#elif L==1
	"Erweitern %s";
#elif L==2
	"Expand %s";
#elif L==3
	"Expandir %s";
#elif L==4
	"&Eacute;largir %s";
#elif L==5
	"Expandir %s";	// Okoteve traduccin
#elif L==6
	"Espandere %s";
#elif L==7
	"Rozwin %s";
#elif L==8
	"Expandir %s";
#endif

const char *Txt_Export_questions =
#if   L==0
	"Exportar preguntas";	// Necessita traduccio
#elif L==1
	"Export questions";	// Need bersetzung
#elif L==2
	"Export questions";
#elif L==3
	"Exportar preguntas";
#elif L==4
	"Exporter questions";
#elif L==5
	"Exportar preguntas";	// Okoteve traduccin
#elif L==6
	"Export questions";	// Bisogno di traduzione
#elif L==7
	"Export questions";	// Potrzebujesz tlumaczenie
#elif L==8
	"Export questions";	// Necessita de traduo
#endif

const char *Txt_Export_questions_to_XML_file =
#if   L==0
	"Exportar preguntas a un archivo XML";	// Necessita traduccio
#elif L==1
	"Export questions to XML file";		// Need bersetzung
#elif L==2
	"Export questions to XML file";
#elif L==3
	"Exportar preguntas a un archivo XML";
#elif L==4
	"Exporter des questions vers un fichier XML";
#elif L==5
	"Exportar preguntas a un archivo XML";	// Okoteve traduccin
#elif L==6
	"Export questions to XML file";		// Bisogno di traduzione
#elif L==7
	"Export questions to XML file";		// Potrzebujesz tlumaczenie
#elif L==8
	"Export questions to XML file";		// Necessita de traduo
#endif

const char *Txt_X_faces_have_been_detected_in_front_position_1_Z_ =	// Warning: it is very important to include two %u in the following sentences
#if   L==0
	"Se han detectado %u rostros en posici&oacute;n frontal:<br />"
	"1 con el fondo suficientemente claro (en color verde),<br />"
	"%u con el fondo demasiado oscuro (en color rojo).<br />"
	"Si desea elegir el rostro se&ntilde;alado en verde, pulse sobre &eacute;l.";	// Necessita traduccio
#elif L==1
	"%u faces have been detected in front position:<br />"
	"1 with the background light enough (marked in green).<br />"
	"%u with the background too dark (marked in red).<br />"
	"If you want to choose the face marked in green, click on it.";			// Need bersetzung
#elif L==2
	"%u faces have been detected in front position:<br />"
	"1 with the background light enough (marked in green).<br />"
	"%u with the background too dark (marked in red).<br />"
	"If you want to choose the face marked in green, click on it.";
#elif L==3
	"Se han detectado %u rostros en posici&oacute;n frontal:<br />"
	"1 con el fondo suficientemente claro (en color verde),<br />"
	"%u con el fondo demasiado oscuro (en color rojo).<br />"
	"Si desea elegir el rostro se&ntilde;alado en verde, pulse sobre &eacute;l.";
#elif L==4
	"%u faces have been detected in front position:<br />"
	"1 with the background light enough (marked in green).<br />"
	"%u with the background too dark (marked in red).<br />"
	"If you want to choose the face marked in green, click on it.";			// Besoin de traduction
#elif L==5
	"Se han detectado %u rostros en posici&oacute;n frontal:<br />"
	"1 con el fondo suficientemente claro (en color verde),<br />"
	"%u con el fondo demasiado oscuro (en color rojo).<br />"
	"Si desea elegir el rostro se&ntilde;alado en verde, pulse sobre &eacute;l.";	// Okoteve traduccin
#elif L==6
	"%u visi sono stati individuati in posizione centrale:<br />"
	"1 con lo sfondo abbastanza chiaro (segnato in verde).<br />"
	"%u con lo sfondo troppo scuro (segnato in rosso).<br />"
	"Se vuoi scegliere il viso segnato in verde, clicca su di esso.";
#elif L==7
	"%u twarze zostaly wykryte w przednim:<br />"
	"1 z dosc jasnym tle (zaznaczone na zielono).<br />"
	"%u z tlem zbyt ciemny (zaznaczone na czerwono).<br />"
	"Jesli chcesz wybrac twarz zaznaczone na zielono, kliknij na niego.";
#elif L==8
	"%u faces have been detected in front position:<br />"
	"1 with the background light enough (marked in green).<br />"
	"%u with the background too dark (marked in red).<br />"
	"If you want to choose the face marked in green, click on it.";	// Necessita de traduo
#endif

const char *Txt_X_faces_have_been_detected_in_front_position_Y_Z_ =	// Warning: it is very important to include three %u in the following sentences
#if   L==0
	"Se han detectado %u rostros en posici&oacute;n frontal:<br />"
	"%u con el fondo suficientemente claro (en color verde),<br />"
	"%u con el fondo demasiado oscuro (en color rojo).<br />"
	"Si desea elegir uno de los rostros se&ntilde;alados en verde, pulse sobre &eacute;l.";	// Necessita traduccio
#elif L==1
	"%u faces have been detected in front position:<br />"
	"%u with the background light enough (marked in green).<br />"
	"%u with the background too dark (marked in red).<br />"
	"If you want to choose one of the faces marked in green, click on it.";			// Need bersetzung
#elif L==2
	"%u faces have been detected in front position:<br />"
	"%u with the background light enough (marked in green).<br />"
	"%u with the background too dark (marked in red).<br />"
	"If you want to choose one of the faces marked in green, click on it.";
#elif L==3
	"Se han detectado %u rostros en posici&oacute;n frontal:<br />"
	"%u con el fondo suficientemente claro (en color verde),<br />"
	"%u con el fondo demasiado oscuro (en color rojo).<br />"
	"Si desea elegir uno de los rostros se&ntilde;alados en verde, pulse sobre &eacute;l.";
#elif L==4
	"%u faces have been detected in front position:<br />"
	"%u with the background light enough (marked in green).<br />"
	"%u with the background too dark (marked in red).<br />"
	"If you want to choose one of the faces marked in green, click on it.";			// Besoin de traduction
#elif L==5
	"Se han detectado %u rostros en posici&oacute;n frontal:<br />"
	"%u con el fondo suficientemente claro (en color verde),<br />"
	"%u con el fondo demasiado oscuro (en color rojo).<br />"
	"Si desea elegir uno de los rostros se&ntilde;alados en verde, pulse sobre &eacute;l.";	// Okoteve traduccin
#elif L==6
	"%u visi sono stati individuati in posizione centrale:<br />"
	"%u con lo sfondo abbastanza chiaro (segnato il verde).<br />"
	"%u con lo sfondo troppo scuro (segnato in rosso).<br />"
	"Se vuoi scegliere uno dei visi segnati in verde, clicca su di esso.";
#elif L==7
	"%u twarze zostaly wykryte w przednim:<br />"
	"%u z dosc jasnym tlem (zaznaczone na zielono).<br />"
	"%u z tlem zbyt ciemnym (zaznaczone na czerwono).<br />"
	"Jesli chcesz wybrac jedna z twarzy zaznaczone na zielono, kliknij na niego.";
#elif L==8
	"%u faces have been detected in front position:<br />"
	"%u with the background light enough (marked in green).<br />"
	"%u with the background too dark (marked in red).<br />"
	"If you want to choose one of the faces marked in green, click on it.";			// Necessita de traduo
#endif

const char *Txt_X_faces_marked_in_green_have_been_detected_ =	// Warning: it is very important to include %u in the following sentences
#if   L==0
	"Se han detectado %u rostros (se&ntilde;alados en verde) en posici&oacute;n frontal"
	" y con el fondo detr&aacute;s de las cabezas suficientemente claro.<br />"
	"Si desea elegir uno de esos rostros, pulse sobre &eacute;l.";	// Necessita traduccio
#elif L==1
	"%u faces (marked in green) have been detected in front position,"
	" and the background behind the heads is light enough.<br />"
	"If you want to choose one of those faces, click on it.";	// Need bersetzung
#elif L==2
	"%u faces (marked in green) have been detected in front position,"
	" and the background behind the heads is light enough.<br />"
	"If you want to choose one of those faces, click on it.";
#elif L==3
	"Se han detectado %u rostros (se&ntilde;alados en verde) en posici&oacute;n frontal"
	" y con el fondo detr&aacute;s de las cabezas suficientemente claro.<br />"
	"Si desea elegir uno de esos rostros, pulse sobre &eacute;l.";
#elif L==4
	"%u faces (marked in green) have been detected in front position,"
	" and the background behind the heads is light enough.<br />"
	"If you want to choose one of those faces, click on it.";	// Besoin de traduction
#elif L==5
	"Se han detectado %u rostros (se&ntilde;alados en verde) en posici&oacute;n frontal"
	" y con el fondo detr&aacute;s de las cabezas suficientemente claro.<br />"
	"Si desea elegir uno de esos rostros, pulse sobre &eacute;l.";	// Okoteve traduccin
#elif L==6
	"%u visi (segnati in verde) sono stati individuati in posizione frontale,"
	" e lo sfondo dietro la testa &egrave; abbastanza chiaro.<br />"
	"Se vuoi scegliere uno di questi visi, clicca su di esso.";
#elif L==7
	"%u twarze (zaznaczone na zielono) wykryto w przodu,"
	" i tlo znajdujace sie za glowy jest wystarczajaco lekki,.<br />"
	"Jesli chcesz wybrac jedno z tych twarzy, kliknij na nia.";
#elif L==8
	"%u faces (marked in green) have been detected in front position,"
	" and the background behind the heads is light enough.<br />"
	"If you want to choose one of those faces, click on it.";	// Necessita de traduo
#endif

const char *Txt_X_faces_marked_in_red_have_been_detected_ =	// Warning: it is very important to include %u in the following sentences
#if   L==0
	"Se han detectado %u rostros (se&ntilde;alados en rojo) en posici&oacute;n frontal,"
	" pero el fondo detr&aacute;s de las cabezas es demasiado oscuro.";	// Necessita traduccio
#elif L==1
	"%u faces (marked in green) have been detected in front position,"
	" but the background behind the heads is too dark.";			// Need bersetzung
#elif L==2
	"%u faces (marked in green) have been detected in front position,"
	" but the background behind the heads is too dark.";
#elif L==3
	"Se han detectado %u rostros (se&ntilde;alados en rojo) en posici&oacute;n frontal,"
	" pero el fondo detr&aacute;s de las cabezas es demasiado oscuro.";
#elif L==4
	"%u faces (marked in green) have been detected in front position,"
	" but the background behind the heads is too dark.";			// Besoin de traduction
#elif L==5
	"Se han detectado %u rostros (se&ntilde;alados en rojo) en posici&oacute;n frontal,"
	" pero el fondo detr&aacute;s de las cabezas es demasiado oscuro.";	// Okoteve traduccin
#elif L==6
	"%u visi (segnati in verde) sono stati individuati in posizione centrale,"
	" ma lo sfondo dietro la testa &egrave; troppo scuro.";
#elif L==7
	"%u twarze (zaznaczone na zielono) wykryto w przodu,"
	" ale w tle za glowy jest zbyt ciemny.";
#elif L==8
	"%u faces (marked in green) have been detected in front position,"
	" but the background behind the heads is too dark.";			// Necessita de traduo
#endif

const char *Txt_Failed_email_confirmation_key =
#if   L==0
	"Error en la clave de confirmaci&oacute;n"
	" de correo electr&oacute;nico.";		// Necessita traduccio
#elif L==1
	"Failed email confirmation key.";		// Need bersetzung
#elif L==2
	"Failed email confirmation key.";
#elif L==3
	"Error en la clave de confirmaci&oacute;n"
	" de correo electr&oacute;nico.";
#elif L==4
	"Failed email confirmation key.";		// Besoin de traduction
#elif L==5
	"Error en la clave de confirmaci&oacute;n"
	" de correo electr&oacute;nico.";		// Okoteve traduccin
#elif L==6
	"Failed email confirmation key.";		// Bisogno di traduzione
#elif L==7
	"Failed email confirmation key.";		// Potrzebujesz tlumaczenie
#elif L==8
	"Failed email confirmation key.";		// Necessita de traduo
#endif

const char *Txt_Family_address =
#if   L==0
	"Domicilio familiar";	// Necessita traduccio
#elif L==1
	"Heimatadresse";
#elif L==2
	"Family address";
#elif L==3
	"Domicilio familiar";
#elif L==4
	"Adresse famille";
#elif L==5
	"Domicilio familiar";	// Okoteve traduccin
#elif L==6
	"Domicilio familiare";
#elif L==7
	"Adres rodziny";
#elif L==8
	"Endere&ccedil;o Fam&iacute;lia";
#endif

const char *Txt_Feedback =
#if   L==0
	"Realimentaci&oacute;n";	// Necessita traduccio
#elif L==1
	"Feedback";
#elif L==2
	"Feedback";
#elif L==3
	"Realimentaci&oacute;n";
#elif L==4
	"Feedback";
#elif L==5
	"Realimentaci&oacute;n";	// Okoteve traduccin
#elif L==6
	"Feedback";
#elif L==7
	"Opinie";
#elif L==8
	"Feedback";
#endif

const char *Txt_Feedback_to_students =
#if   L==0
	"Realimentaci&oacute;n para estudiantes";	// Necessita traduccio
#elif L==1
	"Feedback an die Studenten";
#elif L==2
	"Feedback to students";
#elif L==3
	"Realimentaci&oacute;n para estudiantes";
#elif L==4
	"Feedback pour les &eacute;tudiants";
#elif L==5
	"Realimentaci&oacute;n para estudiantes";	// Okoteve traduccin
#elif L==6
	"Feedback per gli studenti";
#elif L==7
	"Opinie student&oacute;w";
#elif L==8
	"Feedback para estudantes";
#endif

const char *Txt_Field_BR_name =
#if   L==0
	"Nombre<br />del campo";	// Necessita traduccio
#elif L==1
	"Name des Feldes";
#elif L==2
	"Field<br />name";
#elif L==3
	"Nombre<br />del campo";
#elif L==4
	"Nom<br />du champ";
#elif L==5
	"Nombre<br />del campo";	// Okoteve traduccin
#elif L==6
	"Nome<br />del campo";
#elif L==7
	"Nazwa<br />pola";
#elif L==8
	"Nome<br />do campo";
#endif

const char *Txt_File =
#if   L==0
	"Fitxer";
#elif L==1
	"Datei";
#elif L==2
	"File";
#elif L==3
	"Archivo";
#elif L==4
	"Fichier";
#elif L==5
	"Jehai'aty";
#elif L==6
	"File";
#elif L==7
	"Plik";
#elif L==8
	"Arquivo";
#endif

const char *Txt_file =
#if   L==0
	"fitxer";
#elif L==1
	"Datei";
#elif L==2
	"file";
#elif L==3
	"archivo";
#elif L==4
	"fichier";
#elif L==5
	"jehai'aty";
#elif L==6
	"file";
#elif L==7
	"Plik";
#elif L==8
	"arquivo";
#endif

const char *Txt_X_file =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"Fitxer %s";
#elif L==1
	"%s-Datei";
#elif L==2
	"%s file";
#elif L==3
	"Archivo %s";
#elif L==4
	"Fichier %s";
#elif L==5
	"Archivo %s";	// Okoteve traduccin
#elif L==6
	"File %s";
#elif L==7
	"Plik %s";
#elif L==8
	"Arquivo %s";
#endif

const char *Txt_FILE_uncompressed =
#if   L==0
	"sense comprimir";
#elif L==1
	"unkomprimierte";
#elif L==2
	"uncrompressed";
#elif L==3
	"sin comprimir";
#elif L==4
	"non compress&eacute;";
#elif L==5
	"sin comprimir";	// Okoteve traduccin
#elif L==6
	"non compresso";
#elif L==7
	"nieskompresowane";
#elif L==8
	"descompactado";
#endif

const char *Txt_Filename =
#if   L==0
	"Nom del fitxer";
#elif L==1
	"Dateiname";
#elif L==2
	"Filename";
#elif L==3
	"Nombre del archivo";
#elif L==4
	"Nom du fichier";
#elif L==5
	"Nombre del archivo";	// Okoteve traduccin
#elif L==6
	"Nome del file";
#elif L==7
	"Nazwa";
#elif L==8
	"Nome do arquivo";
#endif

const char *Txt_file_folder =
#if   L==0
	"fitxer/carpeta";
#elif L==1
	"Datei/Verzeichnis";
#elif L==2
	"file/folder";
#elif L==3
	"archivo/carpeta";
#elif L==4
	"fichier/r&eacute;pertoire";
#elif L==5
	"archivo/carpeta";	// Okoteve traduccin
#elif L==6
	"file/cartella";
#elif L==7
	"Plik/folder";
#elif L==8
	"arquivo/diret&oacute;rio";
#endif

const char *Txt_FILE_FOLDER_OR_LINK_X_is_now_hidden =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"<strong>%s</strong> ahora est&aacute; oculto.";	// Necessita traduccio
#elif L==1
	"<strong>%s</strong> ist jetzt ausgeblendet.";
#elif L==2
	"<strong>%s</strong> is now hidden.";
#elif L==3
	"<strong>%s</strong> ahora est&aacute; oculto.";
#elif L==4
	"<strong>%s</strong> est maintenant cach&eacute;.";
#elif L==5
	"<strong>%s</strong> ahora est&aacute; oculto.";	// Okoteve traduccin
#elif L==6
	"<strong>%s</strong> &egrave; ora nascosto.";
#elif L==7
	"<strong>%s</strong> jest ukryty.";
#elif L==8
	"<strong>%s</strong> &eacute; agora oculto.";
#endif

const char *Txt_FILE_FOLDER_OR_LINK_X_is_now_visible =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"<strong>%s</strong> ahora est&aacute; visible.";	// Necessita traduccio
#elif L==1
	"<strong>%s</strong> ist jetzt sichtbar.";
#elif L==2
	"<strong>%s</strong> is now visible.";
#elif L==3
	"<strong>%s</strong> ahora est&aacute; visible.";
#elif L==4
	"<strong>%s</strong> est maintenant visible.";
#elif L==5
	"<strong>%s</strong> ahora est&aacute; visible.";	// Okoteve traduccin
#elif L==6
	"<strong>%s</strong> &egrave; ora visibile.";
#elif L==7
	"<strong>%s</strong> jest teraz widoczny.";
#elif L==8
	"<strong>%s</strong> &eacute; agora vis&iacute;vel.";
#endif

const char *Txt_FILE_X_removed =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"<strong>%s</strong> eliminat.";
#elif L==1
	"<strong>%s</strong> entfernt.";
#elif L==2
	"<strong>%s</strong> removed.";
#elif L==3
	"<strong>%s</strong> eliminado.";
#elif L==4
	"<strong>%s</strong> supprim&eacute;.";
#elif L==5
	"<strong>%s</strong> eliminado.";	// Okoteve traduccin
#elif L==6
	"<strong>%s</strong> rimosso.";
#elif L==7
	"<strong>%s</strong> usuniety.";
#elif L==8
	"<strong>%s</strong> removido.";
#endif

const char *Txt_File_size =
#if   L==0
	"Mida del fitxer";
#elif L==1
	"Dateigr&ouml;&szlig;e";
#elif L==2
	"File size";
#elif L==3
	"Tama&ntilde;o del archivo";
#elif L==4
	"Taille du fichier";
#elif L==5
	"Tama&ntilde;o del archivo";	// Okoteve traduccin
#elif L==6
	"Dimensioni del file";
#elif L==7
	"Rozmiar pliku";
#elif L==8
	"Tamanho do arquivo";
#endif

const char *Txt_FILE_UPLOAD_Done =
#if   L==0
	"Finalitzar";
#elif L==1
	"Gemacht";
#elif L==2
	"Done";
#elif L==3
	"Finalizar";
#elif L==4
	"Fini";
#elif L==5
	"Finalizar";	// Okoteve traduccin
#elif L==6
	"Fatto";
#elif L==7
	"Gotowe";
#elif L==8
	"Terminar";
#endif

const char *Txt_File_with_the_photo =
#if   L==0
	"Fitxer amb la foto";
#elif L==1
	"Datei mit dem Foto";
#elif L==2
	"File with the photo";
#elif L==3
	"Archivo con la foto";
#elif L==4
	"Fichier avec la photo";
#elif L==5
	"Archivo con la foto";	// Okoteve traduccin
#elif L==6
	"File con la foto";
#elif L==7
	"Plik ze zdjeciem";
#elif L==8
	"Arquivo com a foto";
#endif

const char *Txt_File_zone =
#if   L==0
	"Zona d'arxius";
#elif L==1
	"Dateienbereich";
#elif L==2
	"File zone";
#elif L==3
	"Zona de archivos";
#elif L==4
	"Zone de fichiers";
#elif L==5
	"Zona de archivos";	// Okoteve traduccin
#elif L==6
	"Area file";
#elif L==7
	"File zone";		// Potrzebujesz tlumaczenie
#elif L==8
	"Zona de arquivos";
#endif

const char *Txt_File_zones =
#if   L==0
	"Zones d'arxius";
#elif L==1
	"Dateienbereiche";
#elif L==2
	"File zones";
#elif L==3
	"Zonas de archivos";
#elif L==4
	"Zones de fichiers";
#elif L==5
	"Zonas de archivos";	// Okoteve traduccin
#elif L==6
	"Area file";
#elif L==7
	"Plik obszar&oacute;w";
#elif L==8
	"Zonas de arquivos";
#endif

const char *Txt_File_zones_disabled =
#if   L==0
	"Zonas de archivos inhabilitadas";	// Necessita traduccio
#elif L==1
	"File zones disabled";			// Need bersetzung
#elif L==2
	"File zones disabled";
#elif L==3
	"Zonas de archivos inhabilitadas";
#elif L==4
	"File zones disabled";			// Besoin de traduction
#elif L==5
	"Zonas de archivos inhabilitadas";	// Okoteve traduccin
#elif L==6
	"Area file disabilitata";
#elif L==7
	"Plik strefy niedostepnej";
#elif L==8
	"File zones disabled";			// Necessita de traduo
#endif

const char *Txt_File_zones_of_the_group_X_are_now_disabled =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"Las zonas de archivos del grupo %s est&aacute;n ahora inhabilitadas.";	// Necessita traduccio
#elif L==1
	"File zones of the group %s are now disabled.";				// Need bersetzung
#elif L==2
	"File zones of the group %s are now disabled.";
#elif L==3
	"Las zonas de archivos del grupo %s est&aacute;n ahora inhabilitadas.";
#elif L==4
	"File zones of the group %s are now disabled.";				// Besoin de traduction
#elif L==5
	"Las zonas de archivos del grupo %s est&aacute;n ahora inhabilitadas.";	// Okoteve traduccin
#elif L==6
	"Le aree file del gruppo %s sono ora disabilitate.";
#elif L==7
	"Plik strefy grupy %s jest teraz niedostepny.";
#elif L==8
	"File zones of the group %s are now disabled.";				// Necessita de traduo
#endif

const char *Txt_File_zones_of_the_group_X_are_now_enabled =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"Las zonas de archivos del grupo %s est&aacute;n ahora habilitadas.";	// Necessita traduccio
#elif L==1
	"File zones of the group %s are now enabled.";				// Need bersetzung
#elif L==2
	"File zones of the group %s are now enabled.";
#elif L==3
	"Las zonas de archivos del grupo %s est&aacute;n ahora habilitadas.";
#elif L==4
	"File zones of the group %s are now enabled.";				// Besoin de traduction
#elif L==5
	"Las zonas de archivos del grupo %s est&aacute;n ahora habilitadas.";	// Okoteve traduccin
#elif L==6
	"Le aree file del gruppo %s sono ora abilitate.";
#elif L==7
	"Plik strefy grupa %s sa obecnie dostepne.";
#elif L==8
	"File zones of the group %s are now enabled.";				// Necessita de traduo
#endif

const char *Txt_File_zones_of_the_group_X_enabled_click_to_disable_them =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"Zonas de archivos del grupo %s habilitadas; pulse para inhabilitarlas";	// Necessita traduccio
#elif L==1
	"File zones of the group %s enabled, click to disable them";			// Need bersetzung
#elif L==2
	"File zones of the group %s enabled, click to disable them";
#elif L==3
	"Zonas de archivos del grupo %s habilitadas; pulse para inhabilitarlas";
#elif L==4
	"File zones of the group %s enabled, click to disable them";			// Besoin de traduction
#elif L==5
	"Zonas de archivos del grupo %s habilitadas; pulse para inhabilitarlas";	// Okoteve traduccin
#elif L==6
	"Aree file del gruppo %s abilitate, clicca per disabilitarle";
#elif L==7
	"Plik strefy grupy %s aktywny, kliknij, aby je wylaczyc";
#elif L==8
	"File zones of the group %s enabled, click to disable them";			// Necessita de traduo
#endif

const char *Txt_File_zones_of_the_group_X_disabled_click_to_enable_them =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"Zonas de archivos del grupo %s inhabilitadas; pulse para habilitarlas";	// Necessita traduccio
#elif L==1
	"File zones of the group %s disabled, click to enable them";			// Need bersetzung
#elif L==2
	"File zones of the group %s disabled, click to enable them";
#elif L==3
	"Zonas de archivos del grupo %s inhabilitadas; pulse para habilitarlas";
#elif L==4
	"File zones of the group %s disabled, click to enable them";			// Besoin de traduction
#elif L==5
	"Zonas de archivos del grupo %s inhabilitadas; pulse para habilitarlas";	// Okoteve traduccin
#elif L==6
	"Aree file del gruppo %s disabilitate, clicca per abilitarle";
#elif L==7
	"Plik strefy grupy %s jes niedostepny, kliknij aby wlaczyc";
#elif L==8
	"File zones of the group %s disabled, click to enable them";			// Necessita de traduo
#endif

const char *Txt_File_zones_of_the_group_X_Y_removed =	// Warning: it is very important to include two %s in the following sentences
#if   L==0
	"Zonas de archivos del grupo <strong>%s %s</strong> eliminadas.";	// Necessita traduccio
#elif L==1
	"Dateienbereiche der Gruppe <strong>%s %s</strong> entfernt.";
#elif L==2
	"File areas of the group <strong>%s %s</strong> removed.";
#elif L==3
	"Zonas de archivos del grupo <strong>%s %s</strong> eliminadas.";
#elif L==4
	"File zones of the group <strong>%s %s</strong> removed.";		// Besoin de traduction
#elif L==5
	"Zonas de archivos del grupo <strong>%s %s</strong> eliminadas.";	// Okoteve traduccin
#elif L==6
	"Aree file del gruppo <strong>%s %s</strong> rimosse.";
#elif L==7
	"Plik obszarach grupy <strong>%s %s</strong> usuniety.";
#elif L==8
	"File zones of the group <strong>%s %s</strong> removed.";	// Necessita de traduo
#endif

const char *Txt_Files =
#if   L==0
	"Fitxers";
#elif L==1
	"Dateien";
#elif L==2
	"Files";
#elif L==3
	"Archivos";
#elif L==4
	"Fichiers";
#elif L==5
	"Archivos";	// Okoteve traduccin
#elif L==6
	"File";
#elif L==7
	"Pliki";
#elif L==8
	"Arquivos";
#endif

const char *Txt_files =
#if   L==0
	"fitxers";
#elif L==1
	"Dateien";
#elif L==2
	"files";
#elif L==3
	"archivos";
#elif L==4
	"fichiers";
#elif L==5
	"archivos";	// Okoteve traduccin
#elif L==6
	"file";
#elif L==7
	"pliki";
#elif L==8
	"arquivos";
#endif

const char *Txt_Files_assignments =
#if   L==0
	"Archivos actividades";	// Necessita traduccio
#elif L==1
	"Dateien der Aufgaben";
#elif L==2
	"Files assignments";
#elif L==3
	"Archivos actividades";
#elif L==4
	"Fichiers activites";
#elif L==5
	"Archivos actividades";	// Okoteve traduccin
#elif L==6
	"File attivit&agrave";
#elif L==7
	"Pliki zadan";
#elif L==8
	"Arquivos atividades";
#endif

const char *Txt_Files_copied =
#if   L==0
	"Arxius copiats";
#elif L==1
	"Dateien kopiert";
#elif L==2
	"Files copied";
#elif L==3
	"Archivos copiados";
#elif L==4
	"Fichiers copi&eacute;s";
#elif L==5
	"Archivos copiados";	// Okoteve traduccin
#elif L==6
	"File copiati";
#elif L==7
	"Pliki kopiowane";
#elif L==8
	"Arquivos copiados";
#endif

const char *Txt_Files_of_marks_must_contain_a_table_in_HTML_format_ =
#if   L==0
	"Los archivos de calificaciones deben contener una tabla en <strong>formato HTML</strong> sin im&aacute;genes."
	" Cada archivo HTML se puede generar, por ejemplo, desde una hoja de c&aacute;lculo,"
	" seleccionando la parte deseada de la tabla y guard&aacute;ndola como p&aacute;gina web.<br />"
	" Una tabla puede tener <strong>varias filas de cabecera y varias filas de pie</strong>."
	" Entre ellas debe haber <strong>una &uacute;nica fila para cada estudiante</strong>,"
	" con cualquier n&uacute;mero de columnas, conteniendo <strong>la primera columna el ID (DNI/c&eacute;dula) del estudiante</strong>.<br />"
	" Un estudiante ver&aacute; una tabla con las filas de cabecera,"
	" seguidas por la fila con sus calificaciones,"
	" y por las filas de pie.";			// Necessita traduccio
#elif L==1
	"Bewertungsdateien mssen eine Tabelle im <strong>HTML-Format</strong> ohne Bilder enthalten."
	" Die HTML-Dateien k&ouml;nnen z.B. aus einer Kalkulationstabelle erstellt werden,"
	" indem der gewnschte Bereich aus der Tabelle ausgew&auml;hlt und als HTML gespeichert wird.<br />"
	" Eine Tabelle kann <strong>mehrere Kopf- und Fuzeilen enthalten</strong>."
	" Zwischen solchen Zeilen muss eine <strong>einzige Zeile f&uuml; jeden Studenten bestehen</strong>,"
	" mit einer beliebigen Anzahl von Spalten, <strong>wovon die erste die Ausweis-Nr. enthalten muss</strong>.<br />"
	" Ruft ein Student die Tabelle auf, sieht er nur die f&uuml;r ihn bestimmte Zeile.";
#elif L==2
	"Files of marks must contain a table in <strong>HTML format</strong> without pictures."
	" Each HTML file may be generated, for example, from a spreadsheet,"
	" by selecting the desired portion of the table and saving it as web page.<br />"
	" A table can have <strong>multiple header and footer rows</strong>."
	" Between them it should be <strong>a single row for each student</strong>,"
	" with any number of columns, <strong>the first column containing the student's ID</strong>.<br />"
	" A student will see a table with header rows, followed by the row with their marks, and footer rows.";
#elif L==3
	"Los archivos de calificaciones deben contener una tabla en <strong>formato HTML</strong> sin im&aacute;genes."
	" Cada archivo HTML se puede generar, por ejemplo, desde una hoja de c&aacute;lculo,"
	" seleccionando la parte deseada de la tabla y guard&aacute;ndola como p&aacute;gina web.<br />"
	" Una tabla puede tener <strong>varias filas de cabecera y varias filas de pie</strong>."
	" Entre ellas debe haber <strong>una &uacute;nica fila para cada estudiante</strong>,"
	" con cualquier n&uacute;mero de columnas, conteniendo <strong>la primera columna el ID (DNI/c&eacute;dula) del estudiante</strong>.<br />"
	" Un estudiante ver&aacute; una tabla con las filas de cabecera,"
	" seguidas por la fila con sus calificaciones,"
	" y por las filas de pie.";
#elif L==4
	"Files of marks must contain a table in <strong>HTML format</strong> without pictures."
	" Each HTML file may be generated, for example, from a spreadsheet,"
	" by selecting the desired portion of the table and saving it as web page.<br />"
	" A table can have <strong>multiple header and footer rows</strong>."
	" Between them it should be <strong>a single row for each student</strong>,"
	" with any number of columns, <strong>the first column containing the student's ID</strong>.<br />"
	" A student will see a table with header rows,"
	" followed by the row with their marks,"
	" and footer rows.";				// Besoin de traduction
#elif L==5
	"Los archivos de calificaciones deben contener una tabla en <strong>formato HTML</strong> sin im&aacute;genes."
	" Cada archivo HTML se puede generar, por ejemplo, desde una hoja de c&aacute;lculo,"
	" seleccionando la parte deseada de la tabla y guard&aacute;ndola como p&aacute;gina web.<br />"
	" Una tabla puede tener <strong>varias filas de cabecera y varias filas de pie</strong>."
	" Entre ellas debe haber <strong>una &uacute;nica fila para cada estudiante</strong>,"
	" con cualquier n&uacute;mero de columnas, conteniendo <strong>la primera columna el ID (DNI/c&eacute;dula) del estudiante</strong>.<br />"
	" Un estudiante ver&aacute; una tabla con las filas de cabecera,"
	" seguidas por la fila con sus calificaciones,"
	" y por las filas de pie.";			// Okoteve traduccin
#elif L==6
	"I file dei risultati devono apparire in una tabella in <strong>formato HTML</strong> senza immagini."
	" Ogni file HTML pu&ograve; essere generato, per esempio, da un foglio di calcolo elettronico,"
	" selezionando la desiderata porzione di tabella e salvandolo come una pagina web.<br />"
	" Una tabella pu&ograve; avere <strong>intestazioni multiple e righe a pi&egrave; di pagina</strong>."
	" Tra di loro dovrebbe esserci <strong>una singola riga per ogni studente</strong>,"
	" con alcuni numeri di colonne, <strong>la prima colonna contiene l'ID dello studente</strong>.<br />"
	" Uno studente vedr&agrave; una tabella con righe intestate,"
	" seguite da una riga con i loro risultati,"
	" e righe a pi&egrave; di pagina.";
#elif L==7
	"Pliki znak&oacute;w musza zawierac tabele <strong>w formacie HTML</strong> bez zdjecia."
	" Kazdy plik HTML moze zostac wygenerowany, na przyklad z arkusza kalkulacyjnego,"
	" wybierajac odpowiednia czesc tabeli i zapisanie go jako strone sieci Web.<br />"
	" Tabela moze miec <strong>wielu wiersze nagl&oacute;wka i stopki</strong>."
	" Pomiedzy nimi nalezy <strong>jednym wierszu dla kazdego ucznia</strong>,"
	" z dowolna iloscia kolumn, <strong>pierwszej kolumnie zawierajacej studenta ID</strong>.<br />"
	" Student zobaczy tabele z wierszy nagl&oacute;wka, a nastepnie przez rzad z ich znak&oacute;w towarowych i stopki.";
#elif L==8
	"Files of marks must contain a table in <strong>HTML format</strong> without pictures."
	" Each HTML file may be generated, for example, from a spreadsheet,"
	" by selecting the desired portion of the table and saving it as web page.<br />"
	" A table can have <strong>multiple header and footer rows</strong>."
	" Between them it should be <strong>a single row for each student</strong>,"
	" with any number of columns, <strong>the first column containing the student's ID</strong>.<br />"
	" A student will see a table with header rows,"
	" followed by the row with their marks,"
	" and footer rows.";				// Necessita de traduo
#endif

const char *Txt_Files_works =
#if   L==0
	"Archivos trabajos";	// Necessita traduccio
#elif L==1
	"Dateien Hausarbeiten";
#elif L==2
	"Files homework";
#elif L==3
	"Archivos trabajos";
#elif L==4
	"Fichiers travaux";
#elif L==5
	"Archivos trabajos";	// Okoteve traduccin
#elif L==6
	"File lavori";
#elif L==7
	"Pliki domowe";
#elif L==8
	"Arquivos trabalhos";
#endif

const char *Txt_first_message_not_allowed =
#if   L==0
	"primer mensaje no permitido";	// Necessita traduccio
#elif L==1
	"erste Nachricht nicht erlaubt";
#elif L==2
	"first message not allowed";
#elif L==3
	"primer mensaje no permitido";
#elif L==4
	"premier message non admis";
#elif L==5
	"primer mensaje no permitido";	// Okoteve traduccin
#elif L==6
	"primo messaggio non ammesso";
#elif L==7
	"Pierwszy komunikat nie jest dozwolony";
#elif L==8
	"primeira mensagem n&atilde;o permitida";
#endif

const char *Txt_First_name =
#if   L==0
	"Nom";
#elif L==1
	"Vorname";
#elif L==2
	"First name";
#elif L==3
	"Nombre";
#elif L==4
	"Pr&eacute;nom";
#elif L==5
	"T&eacute;ra";
#elif L==6
	"Primo nome";
#elif L==7
	"Imi&eogon;";
#elif L==8
	"Nome";
#endif

const char *Txt_First_BR_year =	// "First academic year", the opposite to "Last academic year"
#if   L==0
	"Prim.<br />curso";		// Necessita traduccio
#elif L==1
	"First<br />year";		// Need bersetzung
#elif L==2
	"First<br />year";
#elif L==3
	"Prim.<br />curso";
#elif L==4
	"First<br />year";		// Besoin de traduction
#elif L==5
	"Prim.<br />curso";		// Okoteve traduccin
#elif L==6
	"Primo<br />anno";
#elif L==7
	"Pierwszy<br />rok";
#elif L==8
	"First<br />year";		// Necessita de traduo
#endif

const char *Txt_Folder =
#if   L==0
	"Carpeta";
#elif L==1
	"Verzeichnis";
#elif L==2
	"Folder";
#elif L==3
	"Carpeta";
#elif L==4
	"R&eacute;pertoire";
#elif L==5
	"Carpeta";	// Okoteve traduccin
#elif L==6
	"Cartella";
#elif L==7
	"Folder";
#elif L==8
	"Diret&oacute;rio";
#endif

const char *Txt_folder =
#if   L==0
	"carpeta";
#elif L==1
	"Verzeichnis";
#elif L==2
	"folder";
#elif L==3
	"carpeta";
#elif L==4
	"r&eacute;pertoire";
#elif L==5
	"carpeta";	// Okoteve traduccin
#elif L==6
	"cartella";
#elif L==7
	"folder";
#elif L==8
	"diret&oacute;rio";
#endif

const char *Txt_Folder_X_and_all_its_contents_removed =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"Carpeta <strong>%s</strong> y todo su contenido eliminados.";	// Necessita traduccio
#elif L==1
	"Verzeichnis <strong>%s</strong> und alle Inhalte wurden entfernt.";
#elif L==2
	"Folder <strong>%s</strong> and all its contents removed.";
#elif L==3
	"Carpeta <strong>%s</strong> y todo su contenido eliminados.";
#elif L==4
	"R&eacute;pertoire <strong>%s</strong> et l'ensemble de son contenu supprim&eacute;s.";
#elif L==5
	"Carpeta <strong>%s</strong> y todo su contenido eliminados.";	// Okoteve traduccin
#elif L==6
	"Cartella <strong>%s</strong> e tutto il suo contenuto rimossi.";
#elif L==7
	"Folder <strong>%s</strong> i cala jego zawartosc usunieta.";
#elif L==8
	"Diret&oacute;rio <strong>%s</strong> e todo o seu conte&uacute;do removidos.";
#endif

const char *Txt_Folder_X_removed =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"Carpeta <strong>%s</strong> eliminada.";	// Necessita traduccio
#elif L==1
	"Verzeichnis <strong>%s</strong> entfernt.";
#elif L==2
	"Folder <strong>%s</strong> removed.";
#elif L==3
	"Carpeta <strong>%s</strong> eliminada.";
#elif L==4
	"R&eacute;pertoire <strong>%s</strong> supprim&eacute;.";
#elif L==5
	"Carpeta <strong>%s</strong> eliminada.";	// Okoteve traduccin
#elif L==6
	"Cartella <strong>%s</strong> rimossa.";
#elif L==7
	"Folder <strong>%s</strong> usuniety.";
#elif L==8
	"Diret&oacute;rio <strong>%s</strong> removido.";
#endif

const char *Txt_Folders =
#if   L==0
	"Carpetes";
#elif L==1
	"Verzeichnisse";
#elif L==2
	"Folders";
#elif L==3
	"Carpetas";
#elif L==4
	"R&eacute;pertoires";
#elif L==5
	"Carpetas";	// Okoteve traduccin
#elif L==6
	"Cartelle";
#elif L==7
	"Foldery";
#elif L==8
	"Diret&oacute;rios";
#endif

const char *Txt_folders =
#if   L==0
	"carpetes";
#elif L==1
	"Verzeichnisse";
#elif L==2
	"folders";
#elif L==3
	"carpetas";
#elif L==4
	"r&eacute;pertoires";
#elif L==5
	"carpetas";	// Okoteve traduccin
#elif L==6
	"cartelle";
#elif L==7
	"foldery";
#elif L==8
	"diret&oacute;rios";
#endif

const char *Txt_Folders_copied =
#if   L==0
	"Carpetes copiades";
#elif L==1
	"Verzeichnisse kopiert";
#elif L==2
	"Folders copied";
#elif L==3
	"Carpetas copiadas";
#elif L==4
	"R&eacute;pertoires copi&eacute;s";
#elif L==5
	"Carpetas copiadas";	// Okoteve traduccin
#elif L==6
	"Cartelle copiate";
#elif L==7
	"foldery skopiowane";
#elif L==8
	"Diret&oacute;rios copiados";
#endif

const char *Txt_Folders_renamed =
#if   L==0
	"Carpetes anomenades";
#elif L==1
	"Verzeichnisse umbenannt";
#elif L==2
	"Folders renamed";
#elif L==3
	"Carpetas renombradas";
#elif L==4
	"R&eacute;pertoires renomm&eacute;s";
#elif L==5
	"Carpetas renombradas";	// Okoteve traduccin
#elif L==6
	"Cartelle rinominate";
#elif L==7
	"Foldery zmienione";
#elif L==8
	"Diret&oacute;rios renomeados";
#endif

const char *Txt_Folders_not_renamed =
#if   L==0
	"Carpetes no anomenades";
#elif L==1
	"Verzeichnisse nicht umbenannt";
#elif L==2
	"Folders not renamed";
#elif L==3
	"Carpetas no renombradas";
#elif L==4
	"R&eacute;pertoires pas renomm&eacute;s";
#elif L==5
	"Carpetas no renombradas";	// Okoteve traduccin
#elif L==6
	"Cartelle non rinominate";
#elif L==7
	"Foldery nie zmienione";
#elif L==8
	"Diret&oacute;rios n&atilde;o renomeados";
#endif

const char *Txt_For_security_enter_your_password =
#if   L==0
	"Per a m&eacute;s seguretat, introdueixi la seva contrasenya";
#elif L==1
	"Aus Sicherheitsgr&uuml;nden geben Sie Ihr Passwort";
#elif L==2
	"For security, enter your password";
#elif L==3
	"Para mayor seguridad, introduzca su contrase&ntilde;a";
#elif L==4
	"Pour plus de s&eacute;curit&eacute;, entrez votre mot de passe";
#elif L==5
	"Para mayor seguridad, introduzca su contrase&ntilde;a";	// Okoteve traduccin
#elif L==6
	"Per motivi di sicurezza, immettere la password";
#elif L==7
	"Dla bezpiecze&nacute;stwa, wprowad&zacute; has&lstrok;o";
#elif L==8
	"Por seguran&ccedil;a, digite a sua senha";
#endif

const char *Txt_Force_students_to_read_this_information =
#if   L==0
	"Obligar els estudiants a llegir aquesta informaci&oacute;";
#elif L==1
	"Erzwingen Studenten lesen diese Informationen";
#elif L==2
	"Force students to read this information";
#elif L==3
	"Obligar a los estudiantes a leer esta informaci&oacute;n";
#elif L==4
	"Forcer les &eacute;tudiants &agrave; lire cette information";
#elif L==5
	"Obligar a los estudiantes a leer esta informaci&oacute;n";	// Okoteve traduccin
#elif L==6
	"Obbligare gli studenti a leggere questa informazione";
#elif L==7
	"Studenci maja przeczytac ta informacje";
#elif L==8
	"For&ccedil;ar os estudantes a ler esta informa&ccedil;&atilde;o";
#endif

const char *Txt_Forum =
#if   L==0
	"Foro";		// Necessita traduccio
#elif L==1
	"Forum";
#elif L==2
	"Forum";
#elif L==3
	"Foro";
#elif L==4
	"Forum";
#elif L==5
	"Foro";		// Okoteve traduccin
#elif L==6
	"Forum";
#elif L==7
	"Forum";
#elif L==8
	"F&oacute;rum";
#endif

const char *Txt_Forum_posts =
#if   L==0
	"Mensajes de foro";	// Necessita traduccio
#elif L==1
	"Forum-Beitr&auml;ge";
#elif L==2
	"Forum posts";
#elif L==3
	"Mensajes de foro";
#elif L==4
	"Messages de forum";
#elif L==5
	"Mensajes de foro";	// Okoteve traduccin
#elif L==6
	"Post del forum";
#elif L==7
	"Forum postow";
#elif L==8
	"Posts de f&oacute;rum";
#endif

const char *Txt_Forum_threads =
#if   L==0
	"Discusiones de foro";	// Necessita traduccio
#elif L==1
	"Forum-Threads";
#elif L==2
	"Forum threads";
#elif L==3
	"Discusiones de foro";
#elif L==4
	"Fils de forum";
#elif L==5
	"Discusiones de foro";	// Okoteve traduccin
#elif L==6
	"Discussione del forum";
#elif L==7
	"Watki forum";
#elif L==8
	"Threads de f&oacute;rum";
#endif

const char *Txt_FORUM_THREAD_HELP_ORDER[2] =
   {
#if   L==0
	 "Ordenar por fecha del primer mensaje"	// Necessita traduccio
#elif L==1
	 "Nach Datum der Nachrichten sortieren (aufsteigend)"
#elif L==2
	 "Sort by date of first message"
#elif L==3
	 "Ordenar por fecha del primer mensaje"
#elif L==4
	 "Trier par date du premier message"
#elif L==5
	 "Ordenar por fecha del primer mensaje"	// Okoteve traduccin
#elif L==6
	 "Ordina per data del primo messaggio"
#elif L==7
	 "Sortuj wedlug daty pierwszej wiadomosci"
#elif L==8
	 "Classificar por data da primeira mensagem"
#endif
	 ,
#if   L==0
	 "Ordenar por fecha del &uacute;ltimo mensaje"	// Necessita traduccio
#elif L==1
	 "Nach Datum der Nachrichten sortieren (absteigend)"
#elif L==2
	 "Sort by date of the last message"
#elif L==3
	 "Ordenar por fecha del &uacute;ltimo mensaje"
#elif L==4
	 "Trier par date du dernier message"
#elif L==5
	 "Ordenar por fecha del &uacute;ltimo mensaje"	// Okoteve traduccin
#elif L==6
	 "Ordina per data dell'ultimo messaggio"
#elif L==7
	 "Sortuj wedlug daty ostatniego komunikatu"
#elif L==8
	 "Classificar pela data da &uacute;ltima mensagem"
#endif
	};

const char *Txt_FORUM_THREAD_ORDER[2] =
   {
#if   L==0
	 "Primer mensaje"	// Necessita traduccio
#elif L==1
	 "Erste Nachricht"
#elif L==2
	 "First message"
#elif L==3
	 "Primer mensaje"
#elif L==4
	 "Premier message"
#elif L==5
	 "Primer mensaje"	// Okoteve traduccin
#elif L==6
	 "Primo messaggio"
#elif L==7
	 "Pierwsza wiadomosc"
#elif L==8
	 "Primeira mensagem"
#endif
	 ,
#if   L==0
	 "&Uacute;ltimo mensaje"	// Necessita traduccio
#elif L==1
	 "Letzte Nachricht"
#elif L==2
	 "Last message"
#elif L==3
	 "&Uacute;ltimo mensaje"
#elif L==4
	 "Dernier message"
#elif L==5
	 "&Uacute;ltimo mensaje"	// Okoteve traduccin
#elif L==6
	 "Ultimo messaggio"
#elif L==7
	 "Ostatnia wiadomosc"
#elif L==8
	 "&Uacute;ltima mensagem"
#endif
	};

const char *Txt_FORUM_WHICH_FORUM[For_NUM_WHICH_FORUMS] =
   {
#if   L==0
	 "F&ograve;rums d'aquest lloc"
#elif L==1
	 "Foren zu diesem Standort"
#elif L==2
	 "Forums of this location"
#elif L==3
	 "Foros de este sitio"
#elif L==4
	 "Forums de cet emplacement"
#elif L==5
	 "Foros de este sitio"	// Okoteve traduccin
#elif L==6
	 "Forum di questo sito"
#elif L==7
	 "Forum tej lokalizacji"
#elif L==8
	 "F&oacute;runs deste site"
#endif
	 ,
#if   L==0
	 "Tots els meus f&ograve;rums"
#elif L==1
	 "Alle meine Foren"
#elif L==2
	 "All my forums"
#elif L==3
	 "Todos mis foros"
#elif L==4
	 "Tous mes forums"
#elif L==5
	 "Todos mis foros"	// Okoteve traduccin
#elif L==6
	 "Tutti i miei forum"
#elif L==7
	 "wszystkie moje forum"
#elif L==8
	 "Todos os meus f&oacute;runs"
#endif
	};

const char *Txt_Forums =
#if   L==0
	"F&ograve;rums";
#elif L==1
	"Foren";
#elif L==2
	"Forums";
#elif L==3
	"Foros";
#elif L==4
	"Forums";
#elif L==5
	"Foros";	// Okoteve traduccin
#elif L==6
	"Forum";
#elif L==7
	"Forum";
#elif L==8
	"F&oacute;runs";
#endif

const char *Txt_Found_an_index_htm_file =
#if   L==0
	"Se ha encontrado un archivo <em>index.htm</em>.";	// Necessita traduccio
#elif L==1
	"Found an <em>index.htm</em> file.";			// Need bersetzung
#elif L==2
	"Found an <em>index.htm</em> file.";
#elif L==3
	"Se ha encontrado un archivo <em>index.htm</em>.";
#elif L==4
	"Found an <em>index.htm</em> file.";			// Besoin de traduction
#elif L==5
	"Se ha encontrado un archivo <em>index.htm</em>.";	// Okoteve traduccin
#elif L==6
	"Si &egrave; trovato un file <em>index.htm</em>.";
#elif L==7
	"Znaleziono <em>index.htm</em> plik.";
#elif L==8
	"Found an <em>index.htm</em> file.";		// Necessita de traduo
#endif

const char *Txt_Found_an_index_html_file =
#if   L==0
	"Se ha encontrado un archivo <em>index.html</em>.";	// Necessita traduccio
#elif L==1
	"Found an <em>index.html</em> file.";			// Need bersetzung
#elif L==2
	"Found an <em>index.html</em> file.";
#elif L==3
	"Se ha encontrado un archivo <em>index.html</em>.";
#elif L==4
	"Found an <em>index.html</em> file.";			// Besoin de traduction
#elif L==5
	"Se ha encontrado un archivo <em>index.html</em>.";	// Okoteve traduccin
#elif L==6
	"Si &egrave; trovato un file <em>index.html</em>.";
#elif L==7
	"Znaleziono <em>index.html</em> plik.";
#elif L==8
	"Found an <em>index.html</em> file.";			// Necessita de traduo
#endif

const char *Txt_Frequent_actions =	// Check that this sentence fits within the allocated space
#if   L==0
	"Accions freq&uuml;ents";
#elif L==1
	"H&auml;ufige Handlungen";
#elif L==2
	"Frequent actions";
#elif L==3
	"Acciones frecuentes";
#elif L==4
	"Frequent actions";
#elif L==5
	"Acciones frecuentes";	// Okoteve traduccin
#elif L==6
	"Azioni frequenti";
#elif L==7
	"Czeste dzialania";
#elif L==8
	"A&ccedil;&otilde;es freq&uuml;entes";
#endif

const char *Txt_from =
#if   L==0
	"de";	// Necessita traduccio
#elif L==1
	"von";
#elif L==2
	"from";
#elif L==3
	"de";
#elif L==4
	"de";
#elif L==5
	"de";	// Okoteve traduccin
#elif L==6
	"da";
#elif L==7
	"z";
#elif L==8
	"de";
#endif

const char *Txt_from_A_COURSE =
#if   L==0
	"desde";	// Necessita traduccio
#elif L==1
	"von";
#elif L==2
	"from";
#elif L==3
	"desde";
#elif L==4
	"de";
#elif L==5
	"desde";	// Okoteve traduccin
#elif L==6
	"da";
#elif L==7
	"od";
#elif L==8
	"de";
#endif

const char *Txt_From_now_on_the_number_of_editing_lines_of_the_field_X_is_Y =	// Warning: it is very important to include %s and %u (in this order) in the following sentences
#if   L==0
	"A partir de ahora el n&ordm; de l&iacute;neas de edici&oacute;n del campo <strong>%s</strong> es <strong>%u</strong>.";	// Necessita traduccio
#elif L==1
	"From now on the number of editing lines of the field <strong>%s</strong> is <strong>%u</strong>.";	// Need bersetzung
#elif L==2
	"From now on the number of editing lines of the field <strong>%s</strong> is <strong>%u</strong>.";
#elif L==3
	"A partir de ahora el n&ordm; de l&iacute;neas de edici&oacute;n del campo <strong>%s</strong> es <strong>%u</strong>.";
#elif L==4
	"From now on the number of editing lines of the field <strong>%s</strong> is <strong>%u</strong>.";	// Besoin de traduction
#elif L==5
	"A partir de ahora el n&ordm; de l&iacute;neas de edici&oacute;n del campo <strong>%s</strong> es <strong>%u</strong>.";	// Okoteve traduccin
#elif L==6
	"A partire da ora il numero di linee editate <strong>%s</strong> &egrave; <strong>%u</strong>.";
#elif L==7
	"Od tej pory liczba linii w polu edycji <strong>%s</strong> jest <strong>%u</strong>.";
#elif L==8
	"From now on the number of editing lines of the field <strong>%s</strong> is <strong>%u</strong>.";	// Necessita de traduo
#endif

const char *Txt_from_this_course =
#if   L==0
	"desde esta asignatura";	// Necessita traduccio
#elif L==1
	"von dieser Kurs";
#elif L==2
	"from this course";
#elif L==3
	"desde esta asignatura";
#elif L==4
	"de cette mati&egrave;re";
#elif L==5
	"desde esta asignatura";	// Okoteve traduccin
#elif L==6
	"da questo corso";
#elif L==7
	"z tego kursu";
#elif L==8
	"desta disciplina";
#endif

const char *Txt_From_this_time_you_can_receive_messages_from_X =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"A partir de este momento, puede recibir mensajes de %s.";	// Necessita traduccio
#elif L==1
	"From this time, you can receive messages from %s.";		// Need bersetzung
#elif L==2
	"From this time, you can receive messages from %s.";
#elif L==3
	"A partir de este momento, puede recibir mensajes de %s.";
#elif L==4
	"From this time, you can receive messages from %s.";		// Besoin de traduction
#elif L==5
	"A partir de este momento, puede recibir mensajes de %s.";	// Okoteve traduccin
#elif L==6
	"From this time, you can receive messages from %s.";		// Bisogno di traduzione
#elif L==7
	"From this time, you can receive messages from %s.";		// Potrzebujesz tlumaczenie
#elif L==8
	"From this time, you can receive messages from %s.";		// Necessita de traduo
#endif

const char *Txt_From_this_time_you_will_not_receive_messages_from_X =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"A partir de este momento, no recibir&aacute; mensajes de %s.";	// Necessita traduccio
#elif L==1
	"From this time, you will not receive messages from %s.";	// Need bersetzung
#elif L==2
	"From this time, you will not receive messages from %s.";
#elif L==3
	"A partir de este momento, no recibir&aacute; mensajes de %s.";
#elif L==4
	"From this time, you will not receive messages from %s.";	// Besoin de traduction
#elif L==5
	"A partir de este momento, no recibir&aacute; mensajes de %s.";	// Okoteve traduccin
#elif L==6
	"From this time, you will not receive messages from %s.";	// Bisogno di traduzione
#elif L==7
	"From this time, you will not receive messages from %s.";	// Potrzebujesz tlumaczenie
#elif L==8
	"From this time, you will not receive messages from %s.";	// Necessita de traduo
#endif

const char *Txt_Full_Name =
#if   L==0
	"Nombre completo";	// Necessita traduccio
#elif L==1
	"Vollst&auml;ndiger Name";
#elif L==2
	"Full name";
#elif L==3
	"Nombre completo";
#elif L==4
	"Nom complet";
#elif L==5
	"Nombre completo";	// Okoteve traduccin
#elif L==6
	"Nome completo";
#elif L==7
	"Pelne imie i nazwisko";
#elif L==8
	"Nome completo";
#endif

const char *Txt_Full_BR_Name =
#if   L==0
	"Nombre<br />completo";	// Necessita traduccio
#elif L==1
	"Vollst&auml;ndiger<br />Name";
#elif L==2
	"Full<br />name";
#elif L==3
	"Nombre<br />completo";
#elif L==4
	"Nom<br />complet";
#elif L==5
	"Nombre<br />completo";	// Okoteve traduccin
#elif L==6
	"Nome<br />completo";
#elif L==7
	"Pelna<br />nazwa";
#elif L==8
	"Nome<br />completo";
#endif

const char *Txt_Function =
#if   L==0
	"Funci&oacute;";
#elif L==1
	"Funktion";
#elif L==2
	"Function";
#elif L==3
	"Funci&oacute;n";
#elif L==4
	"Fonction";
#elif L==5
	"Funci&oacute;n";	// Okoteve traduccin
#elif L==6
	"Funzione";
#elif L==7
	"Funkcja";
#elif L==8
	"Fun&ccedil;&atilde;o";
#endif

const char *Txt_General =
#if   L==0
	"General";
#elif L==1
	"Allgemein";
#elif L==2
	"General";
#elif L==3
	"General";
#elif L==4
	"G&eacute;n&eacute;ral";
#elif L==5
	"General";	// Okoteve traduccin
#elif L==6
	"Generale";
#elif L==7
	"Ogolne";
#elif L==8
	"Geral";
#endif

const char *Txt_general_topics =
#if   L==0
	"temas generales";	// Necessita traduccio
#elif L==1
	"allgemeine Themen";
#elif L==2
	"general topics";
#elif L==3
	"temas generales";
#elif L==4
	"affaires g&eacute;n&eacute;rales";
#elif L==5
	"temas generales";	// Okoteve traduccin
#elif L==6
	"temi generali";
#elif L==7
	"oglne tematy";
#elif L==8
	"temas gerais";
#endif

const char *Txt_Generate_exam =
#if   L==0
	"Generar examen";
#elif L==1
	"Generieren Pr&uuml;fung";
#elif L==2
	"Generate exam";
#elif L==3
	"Generar examen";
#elif L==4
	"G&eacute;n&eacute;rer l'examen";
#elif L==5
	"Generar examen";	// Okoteve traduccin
#elif L==6
	"Crea esame";
#elif L==7
	"Wygeneruj egzamin";
#elif L==8
	"Gerar teste";
#endif

const char *Txt_Go_back =
#if   L==0
	"Tornar enrere";
#elif L==1
	"Zur&uuml;ckgehen";
#elif L==2
	"Go back";
#elif L==3
	"Volver atr&aacute;s";
#elif L==4
	"Retour";
#elif L==5
	"Volver atr&aacute;s";	// Okoteve traduccin
#elif L==6
	"Tornare";
#elif L==7
	"Wraca&cacute;";
#elif L==8
	"Voltar";
#endif

const char *Txt_Go_to_X =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"Ir a %s";	// Necessita traduccio
#elif L==1
	"Gehe zum %s";
#elif L==2
	"Go to %s";
#elif L==3
	"Ir a %s";
#elif L==4
	"Aller &agrave; %s";
#elif L==5
	"Ir a %s";	// Okoteve traduccin
#elif L==6
	"Vai a %s";
#elif L==7
	"Przejdz do %s";
#elif L==8
	"Ir para %s";
#endif

const char *Txt_Go_to_course_and_reply =
#if   L==0
	"Ir a asignatura y responder";		// Necessita traduccio
#elif L==1
	"Kurs einblenden und antworten";
#elif L==2
	"Go to course and reply";
#elif L==3
	"Ir a asignatura y responder";
#elif L==4
	"Aller &agrave; la mati&egrave;re et r&eacute;pondre";
#elif L==5
	"Ir a asignatura y responder";		// Okoteve traduccin
#elif L==6
	"Vai a corso e rispondere";
#elif L==7
	"Idz do kursu i odpowiedz";
#elif L==8
	"Ir para a disciplina e responder";
#endif

const char *Txt_Go_to_course_and_reply_again =
#if   L==0
	"Ir a asignatura y responder otra vez";		// Necessita traduccio
#elif L==1
	"Kurs einblenden und erneut antworten";
#elif L==2
	"Go to course and reply again";
#elif L==3
	"Ir a asignatura y responder otra vez";
#elif L==4
	"Aller &agrave; la mati&egrave;re et r&eacute;pondre &agrave; nouveau";
#elif L==5
	"Ir a asignatura y responder otra vez";		// Okoteve traduccin
#elif L==6
	"Vai a corso e rispondere nuovamente";
#elif L==7
	"Idz do kursu i odpowiedz ponownie";
#elif L==8
	"Ir para a disciplina e responder novamente";
#endif

const char *Txt_Group =
#if   L==0
	"Grup";
#elif L==1
	"Gruppe";
#elif L==2
	"Group";
#elif L==3
	"Grupo";
#elif L==4
	"Groupe";
#elif L==5
	"Aty";
#elif L==6
	"Gruppo";
#elif L==7
	"Grupa";
#elif L==8
	"Grupo";
#endif

const char *Txt_group =
#if   L==0
	"grup";
#elif L==1
	"Gruppe";
#elif L==2
	"group";
#elif L==3
	"grupo";
#elif L==4
	"groupe";
#elif L==5
	"aty";
#elif L==6
	"gruppo";
#elif L==7
	"grupa";
#elif L==8
	"grupo";
#endif

const char *Txt_Group_closed =
#if   L==0
	"Grupo cerrado";	// Necessita traduccio
#elif L==1
	"Group closed";		// Need bersetzung
#elif L==2
	"Group closed";
#elif L==3
	"Grupo cerrado";
#elif L==4
	"Group closed";		// Besoin de traduction
#elif L==5
	"Grupo cerrado";	// Okoteve traduccin
#elif L==6
	"Gruppo chiuso";
#elif L==7
	"Grupa zamknieta";
#elif L==8
	"Group closed";		// Necessita de traduo
#endif

const char *Txt_Group_X_closed =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"Grupo %s cerrado";		// Necessita traduccio
#elif L==1
	"Group %s closed";		// Need bersetzung
#elif L==2
	"Group %s closed";
#elif L==3
	"Grupo %s cerrado";
#elif L==4
	"Group %s closed";		// Besoin de traduction
#elif L==5
	"Grupo %s cerrado";		// Okoteve traduccin
#elif L==6
	"Gruppo %s chiuso";
#elif L==7
	"Grupa %s zamknieta";
#elif L==8
	"Group %s closed";		// Necessita de traduo
#endif

const char *Txt_Group_X_closed_click_to_open_it =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"Grupo %s cerrado; pulse para abrirlo";		// Necessita traduccio
#elif L==1
	"Group %s closed, click to open it";		// Need bersetzung
#elif L==2
	"Group %s closed, click to open it";
#elif L==3
	"Grupo %s cerrado; pulse para abrirlo";
#elif L==4
	"Group %s closed, click to open it";		// Besoin de traduction
#elif L==5
	"Grupo %s cerrado; pulse para abrirlo";		// Okoteve traduccin
#elif L==6
	"Gruppo %s chiuso, clicca per aprirlo";
#elif L==7
	"Grupa %s zamknieta, kliknij zeby otworzyc";
#elif L==8
	"Group %s closed, click to open it";		// Necessita de traduo
#endif

const char *Txt_Group_name =
#if   L==0
	"Nombre del grupo";	// Necessita traduccio
#elif L==1
	"Name der Gruppe";
#elif L==2
	"Group name";
#elif L==3
	"Nombre del grupo";
#elif L==4
	"Nom du groupe";
#elif L==5
	"Nombre del grupo";	// Okoteve traduccin
#elif L==6
	"Nome del gruppo";
#elif L==7
	"Nazwa grupy";
#elif L==8
	"Nome do grupo";
#endif

const char *Txt_Group_X_open =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"Grupo %s abierto";		// Necessita traduccio
#elif L==1
	"Group %s open";		// Need bersetzung
#elif L==2
	"Group %s open";
#elif L==3
	"Grupo %s abierto";
#elif L==4
	"Group %s open";		// Besoin de traduction
#elif L==5
	"Grupo %s abierto";		// Okoteve traduccin
#elif L==6
	"Gruppo %s aperto";
#elif L==7
	"Grupa %s otwarta";
#elif L==8
	"Group %s open";		// Necessita de traduo
#endif

const char *Txt_Group_X_open_click_to_close_it =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"Grupo %s abierto; pulse para cerrarlo";	// Necessita traduccio
#elif L==1
	"Group %s open, click to close it";		// Need bersetzung
#elif L==2
	"Group %s open, click to close it";
#elif L==3
	"Grupo %s abierto; pulse para cerrarlo";
#elif L==4
	"Group %s open, click to close it";		// Besoin de traduction
#elif L==5
	"Grupo %s abierto; pulse para cerrarlo";	// Okoteve traduccin
#elif L==6
	"Gruppo %s aperto, clicca per chiuderlo";
#elif L==7
	"Grupa %s otwarta, kliknij zeby zamknac";
#elif L==8
	"Group %s open, click to close it";		// Necessita de traduo
#endif

const char *Txt_Group_X_removed =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"Grupo <strong>%s</strong> eliminado.";		// Necessita traduccio
#elif L==1
	"Gruppe <strong>%s</strong> entfernt.";
#elif L==2
	"Group <strong>%s</strong> removed.";
#elif L==3
	"Grupo <strong>%s</strong> eliminado.";
#elif L==4
	"Groupe <strong>%s</strong> supprim&eacute;.";
#elif L==5
	"Grupo <strong>%s</strong> eliminado.";		// Okoteve traduccin
#elif L==6
	"Gruppo <strong>%s</strong> rimosso.";
#elif L==7
	"Grupa <strong>%s</strong> usunieta.";
#elif L==8
	"Group <strong>%s</strong> removed.";		// Necessita de traduo
#endif

const char *Txt_Groups =
#if   L==0
	"Grups";
#elif L==1
	"Gruppen";
#elif L==2
	"Groups";
#elif L==3
	"Grupos";
#elif L==4
	"Groupes";
#elif L==5
	"Aty";
#elif L==6
	"Gruppi";
#elif L==7
	"Grupy";
#elif L==8
	"Grupos";
#endif

const char *Txt_Groups_OF_A_USER =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"Grups de %s";
#elif L==1
	"%s groups";		// Need bersetzung
#elif L==2
	"%s groups";
#elif L==3
	"Grupos de %s";
#elif L==4
	"Groupes de %s";
#elif L==5
	"Grupos de %s";		// Okoteve traduccin
#elif L==6
	"Gruppi di %s";
#elif L==7
	"%s grupy";
#elif L==8
	"%s groups";		// Necessita de traduo
#endif

const char *Txt_Guided_academic_assignments =
#if   L==0
	"Actividades acad&eacute;micas dirigidas";	// Necessita traduccio
#elif L==1
	"Gef&uuml;hrte akademische Aufgaben";
#elif L==2
	"Guided academic assignments";
#elif L==3
	"Actividades acad&eacute;micas dirigidas";
#elif L==4
	"Activit&eacute;s acad&eacute;miques guid&eacute;es";
#elif L==5
	"Actividades acad&eacute;micas dirigidas";	// Okoteve traduccin
#elif L==6
	"Attivit&agrave; accademiche guidate";
#elif L==7
	"Zwiedzanie dyplomowe";
#elif L==8
	"Atividades acad&eacute;micas guided";
#endif

const char *Txt_Hidden_MALE_PLURAL =
#if   L==0
	"Ocultos";	// Necessita traduccio
#elif L==1
	"Versteckt";
#elif L==2
	"Hidden";
#elif L==3
	"Ocultos";
#elif L==4
	"Cach&eacute;s";
#elif L==5
	"Ocultos";	// Okoteve traduccin
#elif L==6
	"Nascosti";
#elif L==7
	"Ukryty";
#elif L==8
	"Ocultos";
#endif

const char *Txt_Hidden_survey =
#if   L==0
	"Encuesta oculta";	// Necessita traduccio
#elif L==1
	"Ausgeblendet Umfrage";
#elif L==2
	"Hidden survey";
#elif L==3
	"Encuesta oculta";
#elif L==4
	"Sondage cach&eacute;";
#elif L==5
	"Encuesta oculta";	// Okoteve traduccin
#elif L==6
	"Sondaggio nascosto";
#elif L==7
	"Ukryte badania";
#elif L==8
	"Inqu&eacute;rito oculto";
#endif

const char *Txt_Hide =
#if   L==0
	"Amagar";
#elif L==1
	"Ausblenden";
#elif L==2
	"Hide";
#elif L==3
	"Ocultar";
#elif L==4
	"Masquer";
#elif L==5
	"Ocultar";	// Okoteve traduccin
#elif L==6
	"Nascondi";
#elif L==7
	"Ukryj";
#elif L==8
	"Ocultar";
#endif

const char *Txt_Hide_FOLDER_FILE_OR_LINK_X =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"Amagar %s";
#elif L==1
	"Ausblenden %s";
#elif L==2
	"Hide %s";
#elif L==3
	"Ocultar %s";
#elif L==4
	"Masquer %s";
#elif L==5
	"Ocultar %s";	// Okoteve traduccin
#elif L==6
	"Nascondi %s";
#elif L==7
	"Ukryj %s";
#elif L==8
	"Ocultar %s";
#endif

const char *Txt_Hide_left_column =
#if   L==0
	"Ocultar columna izquierda";	// Necessita traduccio
#elif L==1
	"Links Spalte ausblenden";
#elif L==2
	"Hide left column";
#elif L==3
	"Ocultar columna izquierda";
#elif L==4
	"Masquer la colonne de gauche";
#elif L==5
	"Ocultar columna izquierda";	// Okoteve traduccin
#elif L==6
	"Nascondi colonna di sinistra";
#elif L==7
	"Ukryj lewa kolumne";
#elif L==8
	"Ocultar coluna da esquerda";
#endif

const char *Txt_Hide_message =
#if   L==0
	"Amaga missatge";
#elif L==1
	"Ausblende Nachricht";
#elif L==2
	"Hide message";
#elif L==3
	"Ocultar mensaje";
#elif L==4
	"Masquer message";
#elif L==5
	"Ocultar mensaje";	// Okoteve traduccin
#elif L==6
	"Nascondi messaggio";
#elif L==7
	"Hide message";		// Potrzebujesz tlumaczenie
#elif L==8
	"Ocultar mensagem";
#endif

const char *Txt_Hide_right_column =
#if   L==0
	"Ocultar columna derecha";	// Necessita traduccio
#elif L==1
	"Rechte Spalte ausblenden";
#elif L==2
	"Hide right column";
#elif L==3
	"Ocultar columna derecha";
#elif L==4
	"Masquer la colonne de droite";
#elif L==5
	"Ocultar columna derecha";	// Okoteve traduccin
#elif L==6
	"Nascondi colonna di destra";
#elif L==7
	"Ukryj prawa kolumne";
#elif L==8
	"Ocultar coluna da direita";
#endif

const char *Txt_Holiday =
#if   L==0
	"Festivitat";
#elif L==1
	"Feiertag";
#elif L==2
	"Holiday";
#elif L==3
	"Festividad";
#elif L==4
	"F&ecirc;te";
#elif L==5
	"Festividad";
#elif L==6
	"Festivit&agrave;";
#elif L==7
	"Wakacje";
#elif L==8
	"F&eacute;ria";
#endif

const char *Txt_Holiday_X_removed =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"Festividad <strong>%s</strong> eliminada.";	// Necessita traduccio
#elif L==1
	"Feiertag <strong>%s</strong> entfernt.";
#elif L==2
	"Holiday <strong>%s</strong> removed.";
#elif L==3
	"Festividad <strong>%s</strong> eliminada.";
#elif L==4
	"Holiday <strong>%s</strong> removed.";		// Besoin de traduction
#elif L==5
	"Festividad <strong>%s</strong> eliminada.";	// Okoteve traduccin
#elif L==6
	"Festivit&agrave; <strong>%s</strong> rimossa.";
#elif L==7
	"Wakacje <strong>%s</strong> usuniete.";
#elif L==8
	"Holiday <strong>%s</strong> removed.";		// Necessita de traduo
#endif

const char *Txt_HOLIDAY_TYPES[Hld_NUM_TYPES_HOLIDAY] =
   {
#if   L==0
	 "D&iacute;a festivo"	// Necessita traduccio
#elif L==1
	 "Urlaub"
#elif L==2
	 "Holiday"
#elif L==3
	 "D&iacute;a festivo"
#elif L==4
	 "Jour f&eacute;ri&eacute;"
#elif L==5
	 "Arete"
#elif L==6
	 "Festivit&agrave;"
#elif L==7
	 "wakacje"
#elif L==8
	 "Feriado"
#endif
	 ,
#if   L==0
	 "Per&iacute;odo no lectivo"	// Necessita traduccio
#elif L==1
	 "Non-Schule Zeitraum"
#elif L==2
	 "Non school period"
#elif L==3
	 "Per&iacute;odo no lectivo"
#elif L==4
	 "P&eacute;riode sans &eacute;cole"
#elif L==5
	 "Per&iacute;odo no lectivo"	// Okoteve traduccin
#elif L==6
	 "Periodo senza lezione"
#elif L==7
	 "Okres nauki szkolnej dla"
#elif L==8
	 "Per&iacute;odo sem escola"
#endif
	};

const char *Txt_Holidays =
#if   L==0
	"Dies festius";
#elif L==1
	"Feiertage";
#elif L==2
	"Holidays";
#elif L==3
	"D&iacute;as festivos";
#elif L==4
	"Jours f&eacute;ri&eacute;s";
#elif L==5
	"Arete";
#elif L==6
	"Festivit&agrave;";
#elif L==7
	"Noclegi";
#elif L==8
	"F&eacute;rias";
#endif

const char *Txt_HOLIDAYS_HELP_ORDER[2] =
   {
#if   L==0
	 "Ordenar por lugar"	// Necessita traduccio
#elif L==1
	 "Nach Standort sortieren"
#elif L==2
	 "Sort by place"
#elif L==3
	 "Ordenar por lugar"
#elif L==4
	 "Trier par emplacement"
#elif L==5
	 "Ordenar por lugar"	// Okoteve traduccin
#elif L==6
	 "Ordina per localit&grave;"
#elif L==7
	 "Sortuj wedlug miejsca"
#elif L==8
	 "Classificar por localiza&ccedil;&atilde;o"
#endif
	 ,
#if   L==0
	 "Ordenar por fecha"	// Necessita traduccio
#elif L==1
	 "Sortieren nach Datum"
#elif L==2
	 "Sort by date"
#elif L==3
	 "Ordenar por fecha"
#elif L==4
	 "Trier par date"
#elif L==5
	 "Ordenar por fecha"	// Okoteve traduccin
#elif L==6
	 "Ordina per data"
#elif L==7
	 "Sortuj wedlug daty"
#elif L==8
	 "Classificar por data"
#endif
	};

const char *Txt_HOLIDAYS_ORDER[2] =
   {
#if   L==0
	 "Lloc"
#elif L==1
	 "Standort"
#elif L==2
	 "Place"
#elif L==3
	 "Lugar"
#elif L==4
	 "Emplacement"
#elif L==5
	 "Tenda"
#elif L==6
	 "Luogo"
#elif L==7
	 "Miejsce"
#elif L==8
	 "Localiza&ccedil;&atilde;o"
#endif
	 ,
#if   L==0
	 "Data inicial"
#elif L==1
	 "Startdatum"
#elif L==2
	 "Start date"
#elif L==3
	 "Fecha inicial"
#elif L==4
	 "Date initiale"
#elif L==5
	 "Fecha inicial"	// Okoteve traduccin
#elif L==6
	 "Data iniziale"
#elif L==7
	 "Data rozpoczecia"
#elif L==8
	 "Data de in&iacute;cio"
#endif
	};

const char *Txt_Home_PAGE =
#if   L==0
	"Inicio";	// Necessita traduccio
#elif L==1
	"Home";
#elif L==2
	"Home";
#elif L==3
	"Inicio";
#elif L==4
	"Accueil";
#elif L==5
	"Inicio";	// Okoteve traduccin
#elif L==6
	"Home";
#elif L==7
	"Dom";
#elif L==8
	"Home";
#endif

const char *Txt_Hour =
#if   L==0
	"Hora";
#elif L==1
	"Stunde";
#elif L==2
	"Hour";
#elif L==3
	"Hora";
#elif L==4
	"Heure";
#elif L==5
	"Aravo";
#elif L==6
	"Ora";
#elif L==7
	"Godzina";
#elif L==8
	"Hora";
#endif

const char *Txt_hour =
#if   L==0
	"hora";
#elif L==1
	"Stunde";
#elif L==2
	"hour";
#elif L==3
	"hora";
#elif L==4
	"heure";
#elif L==5
	"aravo";
#elif L==6
	"ora";
#elif L==7
	"godzina";
#elif L==8
	"hora";
#endif

const char *Txt_hours =
#if   L==0
	"horas";	// Necessita traduccio
#elif L==1
	"Stunden";
#elif L==2
	"hours";
#elif L==3
	"horas";
#elif L==4
	"heures";
#elif L==5
	"aravo";
#elif L==6
	"ore";
#elif L==7
	"godziny";
#elif L==8
	"horas";
#endif

const char *Txt_hours_ABBREVIATION =	// Abbreviation of "hours"
#if   L==0
	"h";	// Necessita traduccio
#elif L==1
	"S";
#elif L==2
	"h";
#elif L==3
	"h";
#elif L==4
	"h";
#elif L==5
	"a";
#elif L==6
	"o";
#elif L==7
	"g";
#elif L==8
	"h";
#endif

const char *Txt_I_forgot_my_password =
#if   L==0
	"He oblidat la meva contrasenya";
#elif L==1
	"Ich habe mein Passwort vergessen";
#elif L==2
	"I forgot my password";
#elif L==3
	"He olvidado mi contrase&ntilde;a";
#elif L==4
	"J'ai oubli&eacute; mon mot de passe";
#elif L==5
	"He olvidado mi contrase&ntilde;a";	// Okoteve traduccin
#elif L==6
	"Ho dimenticato la mia password";
#elif L==7
	"Nie pami&eogon;tam has&lstrok;a";
#elif L==8
	"Esqueci minha senha";
#endif

const char *Txt_I_have_read_this_information =
#if   L==0
	"He le&iacute;do esta informaci&oacute;n";	// Necessita traduccio
#elif L==1
	"Ich habe diese Information lesen";
#elif L==2
	"I have read this information";
#elif L==3
	"He le&iacute;do esta informaci&oacute;n";
#elif L==4
	"J'ai lu cette information";
#elif L==5
	"He le&iacute;do esta informaci&oacute;n";	// Okoteve traduccin
#elif L==6
	"Ho letto questa informazione";
#elif L==7
	"Czytalem te informacje";
#elif L==8
	"Eu li esta informa&ccedil;&atilde;o";
#endif

const char *Txt_I_understand_that_this_action_may_have_serious_consequences_and_can_not_be_undone =
#if   L==0
	"Entenc que aquesta acci&oacute; pot tenir conseq&uuml;&egrave;ncies greus"
	" i que no es pot desfer.";
#elif L==1
	"Ich verstehe, dass diese Aktion kann schwerwiegende Folgen haben"
	" und kann nicht r&uuml;ckg&auml;ngig gemacht werden.";
#elif L==2
	"I understand that this action may have serious consequences"
	" and can not be undone.";
#elif L==3
	"Entiendo que esta acci&oacute;n puede tener consecuencias graves"
	" y que no se puede deshacer.";
#elif L==4
	"Je comprends que cette action peut avoir des cons&eacute;quences graves"
	" et ne peut &ecirc;tre annul&eacute;e.";
#elif L==5
	"Entiendo que esta acci&oacute;n puede tener consecuencias graves"
	" y que no se puede deshacer.";		// Okoteve traduccin
#elif L==6
	"Mi rendo conto che questa azione pu&ograve; avere gravi conseguenze"
	" e non pu&ograve; essere annullata.";
#elif L==7
	"Rozumiem, &zdot;e dzia&lstrok;anie to mo&zdot;e mie&cacute; powa&zdot;ne konsekwencje"
	" i nie mo&zdot;na cofn&aogon;&cacute;.";
#elif L==8
	"Eu entendo que esta a&ccedil;&atilde;o pode ter consequ&ecirc;ncias graves"
	" e n&atilde;o pode ser desfeita.";
#endif

const char *Txt_Icons =
#if   L==0
	"Icones";
#elif L==1
	"Symbole";
#elif L==2
	"Icons";
#elif L==3
	"Iconos";
#elif L==4
	"Ic&ocirc;nes";
#elif L==5
	"Iconos";	// Okoteve traduccin
#elif L==6
	"Icone";
#elif L==7
	"Ikony";
#elif L==8
	"&Iacute;cones";
#endif

const char *Txt_ID =
#if   L==0
	"ID (DNI/c&egrave;dula)";
#elif L==1
	"Ausweis-Nr.";
#elif L==2
	"ID (identity number/passport)";
#elif L==3
	"ID (DNI/c&eacute;dula)";
#elif L==4
	"Num. d'identit&eacute;";
#elif L==5
	"ID (DNI/c&eacute;dula)";	// Okoteve traduccin
#elif L==6
	"Numero carta d'identit&agrave;";
#elif L==7
	"Dow&oacute;d osobisty";
#elif L==8
	"N&ordm; identif.";
#endif

const char *Txt_ID_X_confirmed =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"ID (DNI/c&eacute;dula) %s confirmado";		// Necessita traduccio
#elif L==1
	"ID %s confirmed";				// Need bersetzung
#elif L==2
	"ID %s confirmed";
#elif L==3
	"ID (DNI/c&eacute;dula) %s confirmado";
#elif L==4
	"ID %s confirmed";				// Besoin de traduction
#elif L==5
	"ID (DNI/c&eacute;dula) %s confirmado";		// Okoteve traduccin
#elif L==6
	"ID %s confirmed";				// Bisogno di traduzione
#elif L==7
	"ID %s confirmed";				// Potrzebujesz tlumaczenie
#elif L==8
	"ID %s confirmed";				// Necessita de traduo
#endif

const char *Txt_ID_X_removed =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"ID (DNI/c&eacute;dula) <strong>%s</strong> eliminat.";
#elif L==1
	"ID <strong>%s</strong> entfernt.";
#elif L==2
	"ID <strong>%s</strong> removed.";
#elif L==3
	"ID (DNI/c&eacute;dula) <strong>%s</strong> eliminado.";
#elif L==4
	"ID <strong>%s</strong> supprim&eacute;.";
#elif L==5
	"ID (DNI/c&eacute;dula) <strong>%s</strong> eliminado.";	// Okoteve traduccin
#elif L==6
	"ID <strong>%s</strong> rimosso.";
#elif L==7
	"ID <strong>%s</strong> usuniete.";
#elif L==8
	"E-mail <strong>%s</strong> removido.";
#endif

const char *Txt_Identified_views =
#if   L==0
	"Accessos identificats";
#elif L==1
	"Identifizierte Ansichten";
#elif L==2
	"Identified views";
#elif L==3
	"Accesos identificados";
#elif L==4
	"Vues identifi&eacute;es";
#elif L==5
	"Accesos identificados";	// Okoteve traduccin
#elif L==6
	"Visualizzazioni identificati";
#elif L==7
	"Zidentyfikowane ods&lstrok;on";
#elif L==8
	"Acessos identificados";
#endif

const char *Txt_If_there_are_multiple_versions_of_the_ID_ =
#if   L==0
	"Si hi ha diverses versions de la ID"
	" (per ex. DNI amb o sense lletra al final),"
	" indiqueu totes les versions.";
#elif L==1
	"Wenn es mehrere Versionen der ID"
	" (zB. mit oder ohne Buchstabe am Ende),"
	" schreiben alle Versionen.";
#elif L==2
	"If there are multiple versions of the ID"
	" (eg. with or without letter at the end),"
	" write out all versions.";
#elif L==3
	"Si existen varias versiones del ID"
	" (por ej. DNI con o sin letra al final),"
	" indique todas las versiones.";
#elif L==4
	"S'il existe plusieurs versions du ID"
	" (par example avec ou sans lettre &agrave; la fin),"
	" &eacute;crire toutes les versions.";
#elif L==5
	"Si existen varias versiones del ID"
	" (por ej. DNI con o sin letra al final),"
	" indique todas las versiones.";	// Okoteve traduccin
#elif L==6
	"Se ci sono pi&ugrave; versioni del ID"
	" (ad es. con o senza lettera alla fine),"
	" scrivere tutte le versioni.";
#elif L==7
	"Je&sacute;li istnieje wiele wersji identyfikatora"
	" (np. Z lub bez litery na ko&nacute;cu),"
	" wypisa&cacute; wszystkie wersje.";
#elif L==8
	"Se houver v&aacute;rias vers&otilde;es do ID"
	" (eg., com ou sem letra no final),"
	" escreva todas as vers&otilde;es.";
#endif

const char *Txt_If_this_is_a_new_user_in_X_you_should_indicate_her_his_ID =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"Si se trata de un usuario nuevo en %s,"
	" debe indicar el ID (DNI/c&eacute;dula).";	// Necessita traduccio
#elif L==1
	"If this is a new user in %s,"
	" you should indicate her/his ID.";		// bersetzung notwendig
#elif L==2
	"If this is a new user in %s,"
	" you should indicate her/his ID.";
#elif L==3
	"Si se trata de un usuario nuevo en %s,"
	" debe indicar el ID (DNI/c&eacute;dula).";
#elif L==4
	"If this is a new user in %s,"
	" you should indicate her/his ID.";		// Besoin de traduction
#elif L==5
	"Si se trata de un usuario nuevo en %s,"
	" debe indicar el ID (DNI/c&eacute;dula).";	// Okoteve traduccin
#elif L==6
	"If this is a new user in %s,"
	" you should indicate her/his ID.";		// Bisogno di traduzione
#elif L==7
	"If this is a new user in %s,"
	" you should indicate her/his ID.";		// Potrzebujesz tlumaczenie
#elif L==8
	"If this is a new user in %s,"
	" you should indicate her/his ID.";		// Necessita de traduo
#endif

const char *Txt_If_you_have_an_Android_device_try_SWADroid =
#if   L==0
	"Si teniu un dispositiu Android, provi SWADroid";
#elif L==1
	"Wenn Sie ein Android-Ger&auml;t haben, versuchen SWADroid";
#elif L==2
	"If you have an Android device, try SWADroid";
#elif L==3
	"Si tiene un dispositivo Android, pruebe SWADroid";
#elif L==4
	"Si vous avez un appareil Android, essayer SWADroid";
#elif L==5
	"Si tiene un dispositivo Android, pruebe SWADroid";	// Okoteve traduccin
#elif L==6
	"Se avete un dispositivo Android, provate SWADroid";
#elif L==7
	"Je&sacute;li masz telefon z systemem Android, spr&oacute;buj SWADroid";
#elif L==8
	"Se voc&ecirc; tiver um dispositivo Android, tente SWADroid";
#endif

const char *Txt_If_you_have_forgotten_your_password_ =
#if   L==0
	"Si usted ha olvidado su contrase&ntilde;a,"
	" se le enviar&aacute; una nueva a la direcci&oacute;n de correo electr&oacute;nico que escribi&oacute; en su ficha personal.";	// Necessita traduccio
#elif L==1
	"Wenn Sie Ihr Passwort vergessen haben,"
	" eine neue wird die E-Mail von Ihnen angegebene Adresse in Ihren Profil gesendet werden vergessen.";
#elif L==2
	"If you have forgotten your password,"
	" a new one will be sent to the e-mail address you entered in your record.";
#elif L==3
	"Si usted ha olvidado su contrase&ntilde;a,"
	" se le enviar&aacute; una nueva a la direcci&oacute;n de correo electr&oacute;nico que escribi&oacute; en su ficha personal.";
#elif L==4
	"Si vous avez oubli&eacute; votre mot de passe,"
	" une nouvelle sera envoy&eacute; &agrave; l'adresse &eacute;lectronique que vous avez entr&eacute; dans votre carte.";
#elif L==5
	"Si usted ha olvidado su contrase&ntilde;a,"
	" se le enviar&aacute; una nueva a la direcci&oacute;n de correo electr&oacute;nico que escribi&oacute; en su ficha personal.";	// Okoteve traduccin
#elif L==6
	"Se hai dimenticato la tua password,"
	" una nuova verr&agrave; inviata all'indirizzo e-mail che hai inserito nella tua scheda.";
#elif L==7
	"Jesli nie pamietasz hasla,"
	" nowe zostanie wyslane na adres e-mail, kt&oacute;ry wszedl w rekordzie.";
#elif L==8
	"Se voc&ecirc; tiver esquecido sua senha,"
	" uma nova ser&aacute; enviada para o e-mail que voc&ecirc; inseriu em seu cart&atilde;o.";
#endif

const char *Txt_If_you_have_written_your_ID_nickname_or_email_correctly_ =
#if   L==0
	"Si usted ha escrito correctamente su @apodo, correo o ID (DNI/c&eacute;dula),"
	" y la direcci&oacute;n de correo electr&oacute;nico en su ficha es correcta,"
	" recibir&aacute; un correo electr&oacute;nico con una nueva contrase&ntilde;a.";	// Necessita traduccio
#elif L==1
	"Wenn Sie Ihre @Benutzername, E-mail oder Ausweis-Nr.,"
	" korrekt geschrieben wurden, und die E-Mail-Adresse auf Ihren Profil korrekt ist,"
	" erhalten Sie eine E-Mail mit einem neuen Passwort.";
#elif L==2
	"If you have written your @nick, e-mail or ID correctly,"
	" and the e-mail address on your record is correct,"
	" you will receive an e-mail with a new password.";
#elif L==3
	"Si usted ha escrito correctamente su @apodo, correo o ID (DNI/c&eacute;dula),"
	" y la direcci&oacute;n de correo electr&oacute;nico en su ficha es correcta,"
	" recibir&aacute; un correo electr&oacute;nico con una nueva contrase&ntilde;a.";
#elif L==4
	"Si vous avez &eacute;crit votre @surnom, courrier ou num&eacute;ro d'identit&eacute; correctement,"
	" ainsi que l'adresse e-mail sur votre carte est correct,"
	" vous recevrez un e-mail avec un nouveau mot de passe.";
#elif L==5
	"Si usted ha escrito correctamente su @apodo, correo o ID (DNI/c&eacute;dula),"
	" y la direcci&oacute;n de correo electr&oacute;nico en su ficha es correcta,"
	" recibir&aacute; un correo electr&oacute;nico con una nueva contrase&ntilde;a.";	// Okoteve traduccin
#elif L==6
	"Se hai scritto il tuo nome utente, e-mail o numero di carta d'identit&agrave; correttamente,"
	" e l'indirizzo e-mail della tua scheda &egrave; corretto,"
	" riceverai una e-mail con una nuova password.";
#elif L==7
	"Jesli napisales swoja nazwe lub nick poprawnie,"
	" a adres e-mail na zapis jest poprawny,"
	" otrzymasz maila z nowym haslem.";
#elif L==8
	"Se voc&ecirc; ter escrito o suo alcunha, e-mail ou n&ordm; identif. corretamente,"
	" bem como o endere&ccedil;o de e-mail em seu cart&atilde;o est&aacute; correto,"
	" voc&ecirc; receber&aacute; um e-mail com uma nova senha.";
#endif

const char *Txt_If_you_send_this_message_you_will_appear_as_its_author =
#if   L==0
	"si env&iacute;a este mensaje, usted aparecer&aacute; como autor";	// Necessita traduccio
#elif L==1
	"Wenn Senden diese Nachricht, Sie wird als dessen Urheber";
#elif L==2
	"if you send this message, you will appear as its author";
#elif L==3
	"si env&iacute;a este mensaje, usted aparecer&aacute; como autor";
#elif L==4
	"si vous envoyez ce message, vous appara&icirc;trez comme son auteur";
#elif L==5
	"si env&iacute;a este mensaje, usted aparecer&aacute; como autor";	// Okoteve traduccin
#elif L==6
	"se invii questo messaggio, apparirai come suo autore";
#elif L==7
	"jesli wyslesz te wiadomosc, pojawi sie jako jego autor";
#elif L==8
	"se voc&ecirc; enviar esta mensagem, voc&ecirc; vai aparecer como o seu autor";
#endif

const char *Txt_Image =
#if   L==0
	"Imatge";
#elif L==1
	"Abbild";
#elif L==2
	"Image";
#elif L==3
	"Imagen";
#elif L==4
	"Image";
#elif L==5
	"Ta'&atilde;nga";
#elif L==6
	"Immagine";
#elif L==7
	"Obraz";
#elif L==8
	"Imagem";
#endif

const char *Txt_Import_questions =
#if   L==0
	"Importar preguntas";	// Necessita traduccio
#elif L==1
	"Import questions";	// Need bersetzung
#elif L==2
	"Import questions";
#elif L==3
	"Importar preguntas";
#elif L==4
	"Import questions";	// Besoin de traduction
#elif L==5
	"Importar preguntas";	// Okoteve traduccin
#elif L==6
	"Import questions";	// Bisogno di traduzione
#elif L==7
	"Import questions";	// Potrzebujesz tlumaczenie
#elif L==8
	"Import questions";	// Necessita de traduo
#endif

const char *Txt_Import_questions_from_XML_file =
#if   L==0
	"Importar preguntas de un archivo XML";	// Necessita traduccio
#elif L==1
	"Import questions from XML file";	// Need bersetzung
#elif L==2
	"Import questions from XML file";
#elif L==3
	"Importar preguntas de un archivo XML";
#elif L==4
	"Import questions from XML file";	// Besoin de traduction
#elif L==5
	"Importar preguntas de un archivo XML";	// Okoteve traduccin
#elif L==6
	"Import questions from XML file";	// Bisogno di traduzione
#elif L==7
	"Import questions from XML file";	// Potrzebujesz tlumaczenie
#elif L==8
	"Import questions from XML file";	// Necessita de traduo
#endif

const char *Txt_Imported_questions =
#if   L==0
	"Preguntas importadas";	// Necessita traduccio
#elif L==1
	"Imported questions";	// Need bersetzung
#elif L==2
	"Imported questions";
#elif L==3
	"Preguntas importadas";
#elif L==4
	"Imported questions";	// Besoin de traduction
#elif L==5
	"Preguntas importadas";	// Okoteve traduccin
#elif L==6
	"Imported questions";	// Bisogno di traduzione
#elif L==7
	"Imported questions";	// Potrzebujesz tlumaczenie
#elif L==8
	"Imported questions";	// Necessita de traduo
#endif

const char *Txt_In_a_type_of_group_with_single_enrollment_students_can_not_be_registered_in_more_than_one_group =
#if   L==0
	"En un tipo de grupo de adscripci&oacute;n &uacute;nica"
	" no es posible apuntar estudiantes a m&aacute;s de un grupo.";	// Necessita traduccio
#elif L==1
	"In a type of group with single enrollment,"
	" students can not be registered in more than one group.";	// Need bersetzung
#elif L==2
	"In a type of group with single enrollment,"
	" tudents can not be registered in more than one group.";
#elif L==3
	"En un tipo de grupo de adscripci&oacute;n &uacute;nica"
	" no es posible apuntar estudiantes a m&aacute;s de un grupo.";
#elif L==4
	"In a type of group with single enrollment,"
	" students can not be registered in more than one group.";	// Besoin de traduction
#elif L==5
	"En un tipo de grupo de adscripci&oacute;n &uacute;nica"
	" no es posible apuntar estudiantes a m&aacute;s de un grupo.";	// Okoteve traduccin
#elif L==6
	"In un tipo di gruppo a singola registrazione,"
	" non puoi registrare studenti in pi&ugrave; di un gruppo.";
#elif L==7
	"W rodzaju grupy z jednej rejestracji,"
	" nie mozesz rejestrowac uczni&oacute;w w wiecej niz jednej grupy.";
#elif L==8
	"In a type of group with single enrollment,"
	" students can not be registered in more than one group.";	// Necessita de traduo
#endif

const char *Txt_In_the_year_X_of_the_degree_Y_already_existed_a_course_with_the_name_Z =	// Warning: it is very important to include three %s in the following sentences
#if   L==0
	"En el curso <strong>%s</strong> de la titulaci&oacute;n <strong>%s</strong>"
	" ya exist&iacute;a una asignatura con el nombre <strong>%s</strong>.";	// Necessita traduccio
#elif L==1
	"In the year <strong>%s</strong> of the degree <strong>%s</strong>"
	" already existed a course with the name <strong>%s</strong>.";		// Need bersetzung
#elif L==2
	"In the year <strong>%s</strong> of the degree <strong>%s</strong>"
	" already existed a course with the name <strong>%s</strong>.";
#elif L==3
	"En el curso <strong>%s</strong> de la titulaci&oacute;n <strong>%s</strong>"
	" ya exist&iacute;a una asignatura con el nombre <strong>%s</strong>.";
#elif L==4
	"In the year <strong>%s</strong> of the degree <strong>%s</strong>"
	" already existed a course with the name <strong>%s</strong>.";		// Besoin de traduction
#elif L==5
	"En el curso <strong>%s</strong> de la titulaci&oacute;n <strong>%s</strong>"
	" ya exist&iacute;a una asignatura con el nombre <strong>%s</strong>.";	// Okoteve traduccin
#elif L==6
	"Nell'anno <strong>%s</strong> di laurea <strong>%s</strong>"
	" gi&agrave; esiste un corso con il nome <strong>%s</strong>.";
#elif L==7
	"W roku <strong>%s</strong> stopnia <strong>%s</strong>"
	" istnialy juz kurs&oacute;w z nazwa <strong>%s</strong>.";
#elif L==8
	"In the year <strong>%s</strong> of the degree <strong>%s</strong>"
	" already existed a course with the name <strong>%s</strong>.";		// Necessita de traduo
#endif

const char *Txt_Incomplete_timetable_for_lack_of_space =
#if   L==0
	"Horari incomplet per falta d'espai";
#elif L==1
	"Unvollst&auml;ndige Studenplan f&uuml;r Platzmangel";
#elif L==2
	"Incomplete timetable for lack of space";
#elif L==3
	"Horario incompleto por falta de espacio";
#elif L==4
	"Horaire incompl&egrave;te par manque de place";
#elif L==5
	"Horario incompleto por falta de espacio";	// Okoteve traduccin
#elif L==6
	"Orario incompleto per mancanza di spazio";
#elif L==7
	"Niekompletne harmonogram brak miejsca";
#elif L==8
	"Hor&aacute;rio incompleto por falta de espa&ccedil;o";
#endif

const char *Txt_Increase_level_of_X =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"Aumentar nivel de %s";		// Necessita traduccio
#elif L==1
	"Increase level of %s";		// Need bersetzung
#elif L==2
	"Increase level of %s";
#elif L==3
	"Aumentar nivel de %s";
#elif L==4
	"Increase level of %s";		// Besoin de traduction
#elif L==5
	"Aumentar nivel de %s";		// Okoteve traduccin
#elif L==6
	"Incrementare livello di %s";
#elif L==7
	"Wzrost poziomu %s";
#elif L==8
	"Increase level of %s";		// Necessita de traduo
#endif

const char *Txt_Indicators =
#if   L==0
	"Indicadors";
#elif L==1
	"Indikatoren";
#elif L==2
	"Indicators";
#elif L==3
	"Indicadores";
#elif L==4
	"Indicateurs";
#elif L==5
	"Indicadores";	// Okoteve traduccin
#elif L==6
	"Indicatori";
#elif L==7
	"Wska&zacute;niki";
#elif L==8
	"Indocadores";
#endif

const char *Txt_Indicators_of_courses =
#if   L==0
	"Indicadors d'assignatures";
#elif L==1
	"Anzeigen von Kursen";
#elif L==2
	"Indicators of courses";
#elif L==3
	"Indicadores de asignaturas";
#elif L==4
	"Indicateurs de mati&egrave;res";
#elif L==5
	"Indicadores de asignaturas";	// Okoteve traduccin
#elif L==6
	"Indicatori di corsi";
#elif L==7
	"Wska&zacute;niki kurs&oacute;w";
#elif L==8
	"Indicadores de disciplinas";
#endif

const char *Txt_INFO_TITLE[Inf_NUM_INFO_TYPES] =
   {
   // Inf_INTRODUCTION
#if   L==0
	 "Informaci&oacute;"
#elif L==1
	 "Information"
#elif L==2
	 "Information"
#elif L==3
	 "Informaci&oacute;n"
#elif L==4
	 "Information"
#elif L==5
	 "Informaci&oacute;n"	// Okoteve traduccin
#elif L==6
	 "Informazione"
#elif L==7
	 "Informacja"
#elif L==8
	 "Informa&ccedil;&atilde;o"
#endif
	 ,
	 // Inf_TEACHING_GUIDE
#if   L==0
	 "Guia docent"
#elif L==1
	 "Hinweise zur Kurs"
#elif L==2
	 "Teaching guide"
#elif L==3
	 "Gu&iacute;a docente"
#elif L==4
	 "Guide p&eacute;dagogique"
#elif L==5
	 "Gu&iacute;a docente"	// Okoteve traduccin
#elif L==6
	 "Guida didattica"
#elif L==7
	 "Nauczanie przypomnienie"
#elif L==8
	 "Guia de ensino"
#endif
	 ,
	 // Inf_LECTURES
#if   L==0
	 "Programa teoria"
#elif L==1
	 "Studienplan (Vorlesungen)"
#elif L==2
	 "Syllabus (lectures)"
#elif L==3
	 "Programa teor&iacute;a"
#elif L==4
	 "Plan th&eacute;orie"
#elif L==5
	 "Programa teor&iacute;a"	// Okoteve traduccin
#elif L==6
	 "Programma teoria"
#elif L==7
	 "Syllabus (wyklady)"
#elif L==8
	 "Programa teoria"
#endif
	 ,
	 // Inf_PRACTICALS
#if   L==0
	 "Programa pr&agrave;ctiques"
#elif L==1
	 "Studienplan (&Uuml;bungen)"
#elif L==2
	 "Syllabus (practicals)"
#elif L==3
	 "Programa pr&aacute;cticas"
#elif L==4
	 "Plan pratique"
#elif L==5
	 "Programa pr&aacute;cticas"	// Okoteve traduccin
#elif L==6
	 "Programma pratica"
#elif L==7
	 "Syllabus (practicals)"
#elif L==8
	 "Programa pr&aacute;tica"
#endif
	 ,
	 // Inf_BIBLIOGRAPHY
#if   L==0
	 "Bibliografia"
#elif L==1
	 "Literatur"
#elif L==2
	 "Bibliography"
#elif L==3
	 "Bibliograf&iacute;a"
#elif L==4
	 "Bibliographie"
#elif L==5
	 "Arandukapurupyre"
#elif L==6
	 "Bibliografia"
#elif L==7
	 "Bibliografia"
#elif L==8
	 "Bibliografia"
#endif
	 ,
	 // Inf_FAQ
#if   L==0
	 "FAQ"
#elif L==1
	 "FAQ"
#elif L==2
	 "FAQ"
#elif L==3
	 "FAQ"
#elif L==4
	 "FAQ"
#elif L==5
	 "FAQ"	// Okoteve traduccin
#elif L==6
	 "FAQ"
#elif L==7
	 "FAQ"
#elif L==8
	 "FAQ"
#endif
	 ,
	 // Inf_LINKS
#if   L==0
	 "Enlla&ccedil;os"
#elif L==1
	 "Links"
#elif L==2
	 "Links"
#elif L==3
	 "Enlaces"
#elif L==4
	 "Liens"
#elif L==5
	 "Ambu&eacute;re joaju"
#elif L==6
	 "Link"
#elif L==7
	 "Linki"
#elif L==8
	 "Liga&ccedil;&otilde;es"
#endif
	 ,
	 // Inf_ASSESSMENT
#if   L==0
	 "Sistema d'avaluaci&oacute;"
#elif L==1
	 "Bewertungssystem"
#elif L==2
	 "Assessment system"
#elif L==3
	 "Sistema de evaluaci&oacute;n"
#elif L==4
	 "Syst&egrave;me d'&eacute;valuation"
#elif L==5
	 "Sistema de evaluaci&oacute;n"	// Okoteve traduccin
#elif L==6
	 "Sistema di valutazione"
#elif L==7
	 "System oceniania"
#elif L==8
	 "Sistema de avalia&ccedil;&atilde;o"
#endif
	};

const char *Txt_INFO_SRC_FULL_TEXT[Inf_NUM_INFO_SOURCES] =
   {
#if   L==0
	 "No ofrecer ninguna informaci&oacute;n"	// Necessita traduccio
#elif L==1
	 "Geben keine Auskunft"
#elif L==2
	 "Do not give any information"
#elif L==3
	 "No ofrecer ninguna informaci&oacute;n"
#elif L==4
	 "Ne pas donner d'information"
#elif L==5
	 "No ofrecer ninguna informaci&oacute;n"	// Okoteve traduccin
#elif L==6
	 "Non dare alcuna informazione"
#elif L==7
	 "Nie podawaj zadnych informacji"
#elif L==8
	 "N&atilde;o d&ecirc; nenhuma informa&ccedil;&atilde;o"
#endif
	 ,
#if   L==0
	 "Editor interno"	// Necessita traduccio
#elif L==1
	 "Interner Editor"
#elif L==2
	 "Internal editor"
#elif L==3
	 "Editor interno"
#elif L==4
	 "&Eacute;diteur interne"
#elif L==5
	 "Editor interno"	// Okoteve traduccin
#elif L==6
	 "Editor interno"
#elif L==7
	 "Wewnetrzny edytor"
#elif L==8
	 "Interno editor"
#endif
	 ,
#if   L==0
	 "Text sense format"
#elif L==1
	 "Plain Text"
#elif L==2
	 "Plain text"
#elif L==3
	 "Texto sin formato"
#elif L==4
	 "Texte brut"
#elif L==5
	 "Texto sin formato"	// Okoteve traduccin
#elif L==6
	 "Testo normale"
#elif L==7
	 "Zwykly tekst"
#elif L==8
	 "Texto plano"
#endif
	 ,
#if   L==0
	 "Text enriquit"
#elif L==1
	 "Rich Text"
#elif L==2
	 "Rich text"
#elif L==3
	 "Texto enriquecido"
#elif L==4
	 "Texte enrichi"
#elif L==5
	 "Texto enriquecido"	// Okoteve traduccin
#elif L==6
	 "Testo formattato"
#elif L==7
	 "Rich text"		// Potrzebujesz tlumaczenie
#elif L==8
	 "Rich text"		// Necessita de traduo
#endif
	 ,
#if   L==0
	 "P&aacute;gina web alojada aqu&iacute;"	// Necessita traduccio
#elif L==1
	 "Webseite gehostet hier"
#elif L==2
	 "Web page hosted here"
#elif L==3
	 "P&aacute;gina web alojada aqu&iacute;"
#elif L==4
	 "Page web h&eacute;berg&eacute;e ici"
#elif L==5
	 "P&aacute;gina web alojada aqu&iacute;"	// Okoteve traduccin
#elif L==6
	 "Pagina web ospitata qui"
#elif L==7
	 "Strona internetowa hostowana tutaj"
#elif L==8
	 "P&aacute;gina da Web hospedada aqui"
#endif
	 ,
#if   L==0
	 "P&aacute;gina web alojada en otro servidor"	// Necessita traduccio
#elif L==1
	 "Webseite auf einem anderen Server gehostet"
#elif L==2
	 "Web page hosted on another server"
#elif L==3
	 "P&aacute;gina web alojada en otro servidor"
#elif L==4
	 "Page Web h&eacute;berg&eacute;e sur un autre serveur"
#elif L==5
	 "P&aacute;gina web alojada en otro servidor"	// Okoteve traduccin
#elif L==6
	 "Pagina Web ospitata su un altro server"
#elif L==7
	 "Strona internetowa umieszczona na innym serwerze"
#elif L==8
	 "P&aacute;gina da Web hospedada em outro servidor"
#endif
	};

const char *Txt_INFO_SRC_HELP[Inf_NUM_INFO_SOURCES] =
	 {
	 NULL,
#if   L==0
	   "editor integrado, s&oacute;lo para los temarios de teor&iacute;a y pr&aacute;cticas"// Necessita traduccio
#elif L==1
	   "Interner Editor, nur f&uuml;r Studienplan (Vorlesung und &Uuml;bungen)"
#elif L==2
	   "internal editor, only for syllabus (lectures and practicals)"
#elif L==3
	   "editor integrado, s&oacute;lo para los temarios de teor&iacute;a y pr&aacute;cticas"
#elif L==4
	   "&eacute;diteur interne, que pour les plans (th&eacute;orie et pratique)"
#elif L==5
	   "editor integrado, s&oacute;lo para los temarios de teor&iacute;a y pr&aacute;cticas"// Okoteve traduccin
#elif L==6
	   "editor interno, solo per programmi di teoria e di pratica"
#elif L==7
	   "Wewnetrzny edytor, tylko program (wyklady i practicals)"
#elif L==8
	   "interno editor, apenas para programas de teoria y pr&aacute;tica"
#endif
	   ,
#if   L==0
	   "editor de texto plano"	// Necessita traduccio
#elif L==1
	   "Plain Text Editor"
#elif L==2
	   "plain text editor"
#elif L==3
	   "editor de texto plano"
#elif L==4
	   "&eacute;diteur de texte simple"
#elif L==5
	   "editor de texto plano"	// Okoteve traduccin
#elif L==6
	   "editor di testo normale"
#elif L==7
	   "edytor tekstu"
#elif L==8
	   "editor de texto simples"
#endif
	   ,
#if   L==0
	   "editor de text enriquit"
#elif L==1
	   "Rich Text Editor"
#elif L==2
	   "rich text editor"
#elif L==3
	   "editor de texto enriquecido"
#elif L==4
	   "&eacute;diteur de texte enrichi"
#elif L==5
	   "editor de texto enriquecido"	// Okoteve traduccin
#elif L==6
	   "editor di testo formattato"
#elif L==7
	   "rich text editor"			// Potrzebujesz tlumaczenie
#elif L==8
	   "rich text editor"			// Necessita de traduo
#endif
	   ,
#if   L==0
	   "env&iacute;e un archivo HTML, o bien un archivo ZIP que contenga un archivo index.html/index.htm y archivos con im&aacute;genes"// Necessita traduccio
#elif L==1
	   "senden Sie eine HTML-Datei, oder als ZIP-Datei mit einer Datei index.html / index.htm und Bild-Dateien"
#elif L==2
	   "send an HTML file, or a ZIP file containing a file index.html / index.htm and image files"
#elif L==3
	   "env&iacute;e un archivo HTML, o bien un archivo ZIP que contenga un archivo index.html/index.htm y archivos con im&aacute;genes"
#elif L==4
	   "envoyer un fichier HTML, ou un fichier ZIP contenant un fichier index.html / index.htm et les fichiers d'images"
#elif L==5
	   "env&iacute;e un archivo HTML, o bien un archivo ZIP que contenga un archivo index.html/index.htm y archivos con im&aacute;genes"// Okoteve traduccin
#elif L==6
	   "inviare un file HTML, o un file ZIP contenente un file index.html / index.htm e file di immagine"
#elif L==7
	   "wyslac plik HTML lub plik ZIP zawierajacy plik index.html / index.htm i pliki graficzne"
#elif L==8
	   "enviar um arquivo HTML, ou um arquivo ZIP que cont&eacute;m um arquivo index.html / index.htm e limas da imagem"
#endif
	   ,
#if   L==0
	   "escriba la direcci&oacute;n web de una p&aacute;gina que se encuentre alojada en otro servidor"	// Necessita traduccio
#elif L==1
	   "Geben Sie die Adresse einer Web-Seite geleitet werden, der sich auf einem anderen Server"
#elif L==2
	   "enter the address of a web page that is hosted on another server"
#elif L==3
	   "escriba la direcci&oacute;n web de una p&aacute;gina que se encuentre alojada en otro servidor"
#elif L==4
	   "entrer l'adresse d'une page Web qui est h&eacute;berg&eacute;e sur un autre serveur"
#elif L==5
	   "escriba la direcci&oacute;n web de una p&aacute;gina que se encuentre alojada en otro servidor"	// Okoteve traduccin
#elif L==6
	   "immettere l'indirizzo di una pagina web che &egrave; ospitata su un altro server"
#elif L==7
	   "wprowadzic adres strony internetowej, kt&oacute;ra znajduje sie na innym serwerze"
#elif L==8
	   "introduza o endere&ccedil;o de uma p&aacute;gina Web que est&aacute; hospedada em outro servidor"
#endif
	};

const char *Txt_INFO_SRC_SHORT_TEXT[Inf_NUM_INFO_SOURCES] =
   {
#if   L==0
	 "Ninguna informaci&oacute;n"	// Necessita traduccio
#elif L==1
	 "Keine Angaben"
#elif L==2
	 "No information"
#elif L==3
	 "Ninguna informaci&oacute;n"
#elif L==4
	 "Aucune information"
#elif L==5
	 "Ninguna informaci&oacute;n"	// Okoteve traduccin
#elif L==6
	 "Nessuna informazione"
#elif L==7
	 "Brak informacji"
#elif L==8
	 "Sem informa&ccedil;&atilde;o"
#endif
	 ,
#if   L==0
	 "Editor interno"	// Necessita traduccio
#elif L==1
	 "Interner Editor"
#elif L==2
	 "Internal editor"
#elif L==3
	 "Editor interno"
#elif L==4
	 "&Eacute;diteur interne"
#elif L==5
	 "Editor interno"	// Okoteve traduccin
#elif L==6
	 "Editor interno"
#elif L==7
	 "Wewnetrzny edytor"
#elif L==8
	 "Interno editor"
#endif
	 ,
#if   L==0
	 "Text sense format"
#elif L==1
	 "Plain Text"
#elif L==2
	 "Plain text"
#elif L==3
	 "Texto sin formato"
#elif L==4
	 "Plain text"
#elif L==5
	 "Texto sin formato"	// Okoteve traduccin
#elif L==6
	 "Testo normale"
#elif L==7
	 "Zwykly tekst"
#elif L==8
	 "Texto plano"
#endif
	 ,
#if   L==0
	 "Text enriquit"
#elif L==1
	 "Rich Text"
#elif L==2
	 "Rich text"
#elif L==3
	 "Texto enriquecido"
#elif L==4
	 "Texte enrichi"
#elif L==5
	 "Texto enriquecido"	// Okoteve traduccin
#elif L==6
	 "Testo formattato"
#elif L==7
	 "Rich text"		// Potrzebujesz tlumaczenie
#elif L==8
	 "Rich text"		// Necessita de traduo
#endif
	 ,
#if   L==0
	 "P&aacute;gina interna"	// Necessita traduccio
#elif L==1
	 "Interne Webseite"
#elif L==2
	 "Internal page"
#elif L==3
	 "P&aacute;gina interna"
#elif L==4
	 "Page interne"
#elif L==5
	 "P&aacute;gina interna"	// Okoteve traduccin
#elif L==6
	 "Pagina interna"
#elif L==7
	 "Strona wewnetrzna"
#elif L==8
	 "Interna p&aacute;gina"
#endif
	 ,
#if   L==0
	 "P&aacute;gina externa"	// Necessita traduccio
#elif L==1
	 "Externe Webseite"
#elif L==2
	 "External page"
#elif L==3
	 "P&aacute;gina externa"
#elif L==4
	 "Page externe"
#elif L==5
	 "P&aacute;gina externa"	// Okoteve traduccin
#elif L==6
	 "Pagina esterna"
#elif L==7
	 "Strona zewnetrzna"
#elif L==8
	 "Externa p&aacute;gina"
#endif
	};

const char *Txt_Institution =
#if   L==0
	"Instituci&oacute;";
#elif L==1
	"Hochschule";
#elif L==2
	"Institution";
#elif L==3
	"Instituci&oacute;n";
#elif L==4
	"&Eacute;tablissement";
#elif L==5
	"Mbo'ehao";
#elif L==6
	"Istituzione";
#elif L==7
	"Instytucji";
#elif L==8
	"Institu&ccedil;&atilde;o";
#endif

const char *Txt_institution =
#if   L==0
	"instituci&oacute;";
#elif L==1
	"Hochschule";
#elif L==2
	"institution";
#elif L==3
	"instituci&oacute;n";
#elif L==4
	"&eacute;tablissement";
#elif L==5
	"mbo'ehao";
#elif L==6
	"istituzione";
#elif L==7
	"instytucji";
#elif L==8
	"institu&ccedil;&atilde;o";
#endif

const char *Txt_Institution_centre_and_department =
#if   L==0
	"Instituci&oacute;, centre i departament";
#elif L==1
	"Hochschule, Lehrinstitut und Abteilung";
#elif L==2
	"Institution, centre and department";
#elif L==3
	"Instituci&oacute;n, centro y departamento";
#elif L==4
	"&Eacute;tablissement, centre et d&eacute;partement";
#elif L==5
	"Mbo'ehao, mbo'ehao, peh&#7869;";
#elif L==6
	"Istituzione, centro e dipartimento";
#elif L==7
	"Instytucji, centrum i wydzia&lstrok;";
#elif L==8
	"Institu&ccedil;&atilde;o, centro e departamento";
#endif

const char *Txt_Institution_X_removed =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"Instituci&oacute;n <strong>%s</strong> eliminada.";	// Necessita traduccio
#elif L==1
	"Hochschule <strong>%s</strong> entfernt.";
#elif L==2
	"Institution <strong>%s</strong> removed.";
#elif L==3
	"Instituci&oacute;n <strong>%s</strong> eliminada.";
#elif L==4
	"Institution <strong>%s</strong> removed.";		// Besoin de traduction
#elif L==5
	"Instituci&oacute;n <strong>%s</strong> eliminada.";	// Okoteve traduccin
#elif L==6
	"Istituzione <strong>%s</strong> rimossa.";
#elif L==7
	"Instytucja <strong>%s</strong> usunieta.";
#elif L==8
	"Institution <strong>%s</strong> removed.";		// Necessita de traduo
#endif


const char *Txt_INSTITUTION_STATUS[Ins_NUM_STATUS_TXT] =
   {
#if   L==0	// Ins_STATUS_UNKNOWN
	 "Desconegudo"
#elif L==1
	 "Unbekannt"
#elif L==2
	 "Unknown"
#elif L==3
	 "Desconocido"
#elif L==4
	 "Inconnu"
#elif L==5
	 "Desconocido"			// Okoteve traduccin
#elif L==6
	 "Sconosciuto"
#elif L==7
	 "Nieznany"
#elif L==8
	 "Desconhecido"
#endif
	 ,
#if   L==0	// Ins_STATUS_ACTIVE
	 "Activa"
#elif L==1
	 "Aktive"
#elif L==2
	 "Active"
#elif L==3
	 "Activa"
#elif L==4
	 "Actif"
#elif L==5
	 "Activa"	// Okoteve traduccin
#elif L==6
	 "Attiva"
#elif L==7
	 "Aktywne"
#elif L==8
	 "Ativa"
#endif
	 ,
#if   L==0	// Ins_STATUS_PENDING
	 "Pendent"
#elif L==1
	 "Offen"
#elif L==2
	 "Pending"
#elif L==3
	 "Pendiente"
#elif L==4
	 "En attente"
#elif L==5
	 "Pendiente"	// Okoteve traduccin
#elif L==6
	 "In attesa"
#elif L==7
	 "Do czasu"
#elif L==8
	 "Pendente"
#endif
	 ,
#if   L==0	// Ins_STATUS_REMOVED
	 "Eliminada"
#elif L==1
	 "Entfernt"
#elif L==2
	 "Removed"
#elif L==3
	 "Eliminada"
#elif L==4
	 "Supprim&eacute;"
#elif L==5
	 "Eliminada"	// Okoteve traduccin
#elif L==6
	 "Rimossa"
#elif L==7
	 "Usuniete"
#elif L==8
	 "Suprimida"
#endif
	};

const char *Txt_Institutional_code =
#if   L==0
	"Codi institucional";
#elif L==1
	"Institutionelle Code";
#elif L==2
	"Institutional code";
#elif L==3
	"C&oacute;digo institucional";
#elif L==4
	"Code institutionnel";
#elif L==5
	"C&oacute;digo institucional";	// Okoteve traduccin
#elif L==6
	"Codice istituzionale";
#elif L==7
	"Kod instytucjonalna";
#elif L==8
	"C&oacute;digo institutional";
#endif

const char *Txt_Institutional_BR_code =
#if   L==0
	"Codi<br />instituc.";
#elif L==1
	"Institutionelle<br />Code";
#elif L==2
	"Institut.<br />code";
#elif L==3
	"C&oacute;digo<br />instituc.";
#elif L==4
	"Code<br />institut.";
#elif L==5
	"C&oacute;digo<br />instituc.";	// Okoteve traduccin
#elif L==6
	"Codice<br />istituz.";
#elif L==7
	"Kod<br />instytuc.";
#elif L==8
	"C&oacute;digo<br />instituc.";
#endif

const char *Txt_Institutions =
#if   L==0
	"Institucions";
#elif L==1
	"Hochschulen";
#elif L==2
	"Institutions";
#elif L==3
	"Instituciones";
#elif L==4
	"&Eacute;tablissements";
#elif L==5
	"Mbo'ehao";
#elif L==6
	"Istituzioni";
#elif L==7
	"Instytucje";
#elif L==8
	"Institu&ccedil;&otilde;es";
#endif

const char *Txt_Institutions_ABBREVIATION =
#if   L==0
	"Instit.";
#elif L==1
	"Hochschulen";
#elif L==2
	"Instit.";
#elif L==3
	"Instit.";
#elif L==4
	"Instit.";
#elif L==5
	"Instit.";	// Okoteve traduccin
#elif L==6
	"Istit.";
#elif L==7
	"Instyt.";
#elif L==8
	"Instit.";
#endif

const char *Txt_institutions =
#if   L==0
	"institucions";
#elif L==1
	"Hochschulen";
#elif L==2
	"institutions";
#elif L==3
	"instituciones";
#elif L==4
	"&eacute;tablissements";
#elif L==5
	"mbo'ehao";
#elif L==6
	"istituzioni";
#elif L==7
	"instytucje";
#elif L==8
	"institu&ccedil;&otilde;es";
#endif

const char *Txt_INSTITUTIONS_HELP_ORDER[2] =
   {
#if   L==0
	 "Ordenar por instituci&oacute;n"	// Necessita traduccio
#elif L==1
	 "Nach Hochschule sortieren"
#elif L==2
	 "Sort by institution"
#elif L==3
	 "Ordenar por instituci&oacute;n"
#elif L==4
	 "Trier par &eacute;tablissement"
#elif L==5
	 "Ordenar por instituci&oacute;n"	// Okoteve traduccin
#elif L==6
	 "Ordina per istituzione"
#elif L==7
	 "Sortuj wedlug instytucji"
#elif L==8
	 "Classificar por institui&ccedil;&atilde;o"
#endif
	 ,
#if   L==0
	 "Ordenar por n&ordm; de usuarios"	// Necessita traduccio
#elif L==1
	 "Nach Anzahl der Benutzer sortieren"
#elif L==2
	 "Sort by no. of users"
#elif L==3
	 "Ordenar por n&ordm; de usuarios"
#elif L==4
	 "Trier par nombre d'utilisateurs"
#elif L==5
	 "Ordenar por n&ordm; de usuarios"	// Okoteve traduccin
#elif L==6
	 "Ordina per numero di utenti"
#elif L==7
	 "Sortowanie wedlug liczby uzytkownikow"
#elif L==8
	 "Classificar por n&ordm; de utilizadores"
#endif
	};

const char *Txt_Institutions_of_COUNTRY_X =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"Institucions de %s";
#elif L==1
	"Hochschulen von %s";
#elif L==2
	"Institutions of %s";
#elif L==3
	"Instituciones de %s";
#elif L==4
	"&Eacute;tablissements de %s";
#elif L==5
	"Instituciones de %s";	// Okoteve traduccin
#elif L==6
	"Istituzioni di %s";
#elif L==7
	"Instytucje %s";
#elif L==8
	"Institu&ccedil;&otilde;es de %s";
#endif

const char *Txt_INSTITUTIONS_ORDER[2] =
   {
#if   L==0
	 "Instituci&oacute;"
#elif L==1
	 "Hochschule"
#elif L==2
	 "Institution"
#elif L==3
	 "Instituci&oacute;n"
#elif L==4
	 "&Eacute;tablissement"
#elif L==5
	 "Mbo'ehao"
#elif L==6
	 "Istituzione"
#elif L==7
	 "Instytucji"
#elif L==8
	 "Institui&ccedil;&atilde;o"
#endif
	 ,
#if   L==0
	 "Usuaris"
#elif L==1
	 "Benutzer"
#elif L==2
	 "Users"
#elif L==3
	 "Usuarios"
#elif L==4
	 "Utilisateurs"
#elif L==5
	 "Puruh&aacute;ra"
#elif L==6
	 "Utenti"
#elif L==7
	 "U&zdot;ytkownicy"
#elif L==8
	 "Utilizadores"
#endif
	};

const char *Txt_Institutions_with_pending_centres =
#if   L==0
	"Institucions amb centres pendents d'activar";
#elif L==1
	"Hochschulen mit anh&auml;ngig Lehrinstitute";
#elif L==2
	"Institutions with pending centres";
#elif L==3
	"Instituciones con centros pendientes de activar";
#elif L==4
	"&Eacute;tablissements avec des centres en attente";
#elif L==5
	"Instituciones con centros pendientes de activar";	// Okoteve traduccin
#elif L==6
	"Istituzioni con centri in sospeso";
#elif L==7
	"Instytucji z centra oczekuj&aogon;cy";
#elif L==8
	"Institu&ccedil;&otilde;es com centros pendentes";
#endif

const char *Txt_Integer_number =
#if   L==0
	"Nombre enter";
#elif L==1
	"Ganze Zahl";
#elif L==2
	"Integer number";
#elif L==3
	"N&ordm; entero";
#elif L==4
	"Nombre entier";
#elif L==5
	"N&ordm; entero";	// Okoteve traduccin
#elif L==6
	"Numero intero";
#elif L==7
	"Liczba calkowita";
#elif L==8
	"N&ordm; inteiro";
#endif

const char *Txt_Internal_code =
#if   L==0
	"Codi intern";
#elif L==1
	"Interner Code";
#elif L==2
	"Internal code";
#elif L==3
	"C&oacute;digo interno";
#elif L==4
	"Code interne";
#elif L==5
	"C&oacute;digo interno";	// Okoteve traduccin
#elif L==6
	"Codice interno";
#elif L==7
	"Kod wewn&eogon;trzny";
#elif L==8
	"C&oacute;digo interno";
#endif

const char *Txt_IP =		// Internet Protocol address
#if   L==0
	"IP";
#elif L==1
	"IP";
#elif L==2
	"IP";
#elif L==3
	"IP";
#elif L==4
	"IP";
#elif L==5
	"IP";	// Okoteve traduccin
#elif L==6
	"IP";
#elif L==7
	"IP";
#elif L==8
	"IP";
#endif

const char *Txt_It_is_mandatory_to_choose_a_group =
#if   L==0
	"Elegir grupo es obligatorio";		// Necessita traduccio
#elif L==1
	"Es muss eine Gruppe ausgew&auml;hlt werden";
#elif L==2
	"It is mandatory to choose a group";
#elif L==3
	"Elegir grupo es obligatorio";
#elif L==4
	"Il est obligatoire de choisir un groupe";
#elif L==5
	"Elegir grupo es obligatorio";		// Okoteve traduccin
#elif L==6
	"E 'obbligatorio scegliere un gruppo";
#elif L==7
	"Jest to obowiazkowe, aby wybrac grupe";
#elif L==8
	"Escolher um grupo &eacute; obrigat&oacute;rio";
#endif

const char *Txt_It_is_optional_to_choose_a_group =
#if   L==0
	"Elegir grupo es voluntario";		// Necessita traduccio
#elif L==1
	"Die Ausw&auml;hl einer Gruppe ist freiwillig";
#elif L==2
	"It is optional to choose a group";
#elif L==3
	"Elegir grupo es voluntario";
#elif L==4
	"Il est volontaire de choisir un groupe";
#elif L==5
	"Elegir grupo es voluntario";		// Okoteve traduccin
#elif L==6
	"E' facoltativo scegliere un gruppo";
#elif L==7
	"Opcjonalnie mozna wybrac grupe";
#elif L==8
	"Escolher um grupo &eacute; volunt&aacute;rio";
#endif

const char *Txt_Language =
#if   L==0
	"Idioma";
#elif L==1
	"Sprache";
#elif L==2
	"Language";
#elif L==3
	"Idioma";
#elif L==4
	"Langue";
#elif L==5
	"&Ntilde;e'&#7869;";
#elif L==6
	"Lingua";
#elif L==7
	"J&eogon;zyk";
#elif L==8
	"Lingua";
#endif

const char *Txt_Last_clicks_in_real_time =
#if   L==0
	"&Uacute;ltims clics en temps real";
#elif L==1
	"Letzte Klicks in Echtzeit";
#elif L==2
	"Last clicks in real time";
#elif L==3
	"&Uacute;ltimos clics en tiempo real";
#elif L==4
	"Derni&egrave;re clics en temps r&eacute;el";
#elif L==5
	"&Uacute;ltimos clics en tiempo real";	// Okoteve traduccin
#elif L==6
	"Ultimi clic in tempo reale";
#elif L==7
	"Ostatnie dwa klikni&eogon;cia w czasie rzeczywistym";
#elif L==8
	"&Uacute;ltimos cliques em tempo real";
#endif

const char *Txt_Last_BR_year =	// "Last academic year", the opposite to "First academic year"
#if   L==0
	"&Uacute;lt..<br />curso";	// Necessita traduccio
#elif L==1
	"Last<br />year";		// Need bersetzung
#elif L==2
	"Last<br />year";
#elif L==3
	"&Uacute;lt..<br />curso";
#elif L==4
	"Last<br />year";		// Besoin de traduction
#elif L==5
	"&Uacute;lt..<br />curso";	// Okoteve traduccin
#elif L==6
	"Ultimo<br />anno";
#elif L==7
	"Ostatni<br />rok";
#elif L==8
	"Last<br />year";		// Necessita de traduo
#endif

const char *Txt_Layout =
#if   L==0
	"Disseny";
#elif L==1
	"Layout";
#elif L==2
	"Layout";
#elif L==3
	"Dise&ntilde;o";
#elif L==4
	"Disposition";
#elif L==5
	"Ta'&atilde;ngahai";
#elif L==6
	"Layout";
#elif L==7
	"Uk&lstrok;ad";
#elif L==8
	"Layout";
#endif

const char *Txt_LAYOUT_NAMES[Lay_NUM_LAYOUTS] =
   {
#if   L==0
	 "Pantalla grande"	// Necessita traduccio
#elif L==1
	 "Desktop"		// Need bersetzung
#elif L==2
	 "Desktop"
#elif L==3
	 "Pantalla grande"
#elif L==4
	 "Desktop"		// Besoin de traduction
#elif L==5
	 "Pantalla grande"	// Okoteve traduccin
#elif L==6
	 "Desktop"
#elif L==7
	 "Pulpit"
#elif L==8
	 "Desktop"		// Necessita de traduo
#endif
	 ,
#if   L==0
	 "M&oacute;vil"		// Necessita traduccio
#elif L==1
	 "Mobile"		// Need bersetzung
#elif L==2
	 "Mobile"
#elif L==3
	 "M&oacute;vil"
#elif L==4
	 "Mobile"		// Besoin de traduction
#elif L==5
	 "M&oacute;vil"		// Okoteve traduccin
#elif L==6
	 "Cellulare"
#elif L==7
	 "Komorka"
#elif L==8
	 "Mobile"		// Necessita de traduo
#endif
	};

const char *Txt_LAYOUT_SIDE_COLUMNS[4] =
   {
#if   L==0
	 "S&oacute;lo zona central"	// Necessita traduccio
#elif L==1
	 "Nur mittleren Bereich anzeigen"
#elif L==2
	 "Display central area only"
#elif L==3
	 "S&oacute;lo zona central"
#elif L==4
	 "Display central area only"	// Besoin de traduction
#elif L==5
	 "S&oacute;lo zona central"	// Okoteve traduccin
#elif L==6
	 "Mostra solo la parte centrale"
#elif L==7
	 "Wyswietl centralnej czesci tylko"
#elif L==8
	 "Display central area only"	// Necessita de traduo
#endif
	 ,
#if   L==0
	 "Columna derecha visible"	// Necessita traduccio
#elif L==1
	 "Rechte Spalte anzeigen"
#elif L==2
	 "Display right column"
#elif L==3
	 "Columna derecha visible"
#elif L==4
	 "Display right column"		// Besoin de traduction
#elif L==5
	 "Columna derecha visible"	// Okoteve traduccin
#elif L==6
	 "Mostra la colonna destra"
#elif L==7
	 "Wyswietl prawa kolumne"
#elif L==8
	 "Display right column"		// Necessita de traduo
#endif
	 ,
#if   L==0
	 "Columna izquierda visible"	// Necessita traduccio
#elif L==1
	 "Linke Spalte anzeigen"
#elif L==2
	 "Display left column"
#elif L==3
	 "Columna izquierda visible"
#elif L==4
	 "Display left column"		// Besoin de traduction
#elif L==5
	 "Columna izquierda visible"	// Okoteve traduccin
#elif L==6
	 "Mostra la colonna sinistra"
#elif L==7
	 "Wyswietl lewa kolumne"
#elif L==8
	 "Display left column"		// Necessita de traduo
#endif
	 ,
#if   L==0
	 "Columnas izquierda y derecha visibles"	// Necessita traduccio
#elif L==1
	 "Linke und rechte Spalte anzeigen"
#elif L==2
	 "Display left and right columns"
#elif L==3
	 "Columnas izquierda y derecha visibles"
#elif L==4
	 "Display left and right columns"		// Besoin de traduction
#elif L==5
	 "Columnas izquierda y derecha visibles"	// Okoteve traduccin
#elif L==6
	 "Mostra la colonna sinistra e destra"
#elif L==7
	 "Wyswietl lewa i prawa kolumne"
#elif L==8
	 "Display left and right columns"		// Necessita de traduo
#endif
	};

const char *Txt_level =
#if   L==0
	"nivel";	// Necessita traduccio
#elif L==1
	"Ebene";
#elif L==2
	"level";
#elif L==3
	"nivel";
#elif L==4
	"niveau";
#elif L==5
	"nivel";	// Okoteve traduccin
#elif L==6
	"livello";
#elif L==7
	"poziom";
#elif L==8
	"n&iacute;vel";
#endif

const char *Txt_Levels =
#if   L==0
	"Niveles";	// Necessita traduccio
#elif L==1
	"Ebenen";
#elif L==2
	"Levels";
#elif L==3
	"Niveles";
#elif L==4
	"Niveaux";
#elif L==5
	"Niveles";	// Okoteve traduccin
#elif L==6
	"Livelli";
#elif L==7
	"Poziomy";
#elif L==8
	"N&iacute;veis";
#endif

const char *Txt_levels =
#if   L==0
	"niveles";	// Necessita traduccio
#elif L==1
	"Ebenen";
#elif L==2
	"levels";
#elif L==3
	"niveles";
#elif L==4
	"niveaux";
#elif L==5
	"niveles";	// Okoteve traduccin
#elif L==6
	"livelli";
#elif L==7
	"poziomy";
#elif L==8
	"n&iacute;veis";
#endif

const char *Txt_License =
#if   L==0
	"Llic&egrave;ncia";
#elif L==1
	"Lizenz";
#elif L==2
	"License";
#elif L==3
	"Licencia";
#elif L==4
	"Licence";
#elif L==5
	"Licencia";	// Okoteve traduccin
#elif L==6
	"Licenza";
#elif L==7
	"Pozwolenie";
#elif L==8
	"Licen&ccedil;a";
#endif

const char *Txt_LICENSES[Brw_NUM_LICENSES] =
	 {
#if   L==0
	   "desconeguda / altra"
#elif L==1
	   "Unbekannte / sonstige"
#elif L==2
	   "unknown / other"
#elif L==3
	   "desconocida / otra"
#elif L==4
	   "inconnue / autre"
#elif L==5
	   "desconocida / otra"			// Okoteve traduccin
#elif L==6
	   "sconosciuta / altri"
#elif L==7
	   "nieznana / inny"
#elif L==8
	   "desconhecida / outra"
#endif
	   ,
#if   L==0
	   "Tots els drets reservats"			// All Rights Reserved
#elif L==1
	   "Alle Rechte vorbehalten"			// All Rights Reserved
#elif L==2
	   "All rights reserved"			// All Rights Reserved
#elif L==3
	   "Todos los derechos reservados"		// All Rights Reserved
#elif L==4
	   "Tous droits r&eacute;serv&eacute;s"		// All Rights Reserved
#elif L==5
	   "Todos los derechos reservados"		// All Rights Reserved	// Okoteve traduccin
#elif L==6
	   "Tutti i diritti riservati"			// All Rights Reserved
#elif L==7
	   "Wszelkie prawa zastrze&zdot;one"		// All Rights Reserved
#elif L==8
	   "Todos os direitos reservados"		// All Rights Reserved
#endif
	   ,
#if L==0
	   "CC Reconeixement"				// CC Attribution License
#elif L==1
	   "CC Namensnennung"				// CC Attribution License
#elif L==2
	   "CC Attribution"				// CC Attribution License
#elif L==3
	   "CC Reconocimiento"				// CC Attribution License
#elif L==4
	   "CC Attribution"				// CC Attribution License
#elif L==5
	   "CC Reconocimiento"				// CC Attribution License	// Okoteve traduccin
#elif L==6
	   "CC Attribuzione"				// CC Attribution License
#elif L==7
	   "CC Uznanie autorstwa"			// CC Attribution License
#elif L==8
	   "CC Atribui&ccedil;&atilde;o"		// CC Attribution License
#endif
	   ,
#if L==0
	   "CC Reconeixement - Compartir igual"				// CC Attribution-ShareAlike License
#elif L==1
	   "CC Namensnennung - Weitergabe unter gleichen Bedingungen"	// CC Attribution-ShareAlike License
#elif L==2
	   "CC Attribution - Share Alike"				// CC Attribution-ShareAlike License
#elif L==3
	   "CC Reconocimiento - Compartir bajo la misma licencia"	// CC Attribution-ShareAlike License
#elif L==4
	   "CC Attribution - Partage &agrave; l'Identique"		// CC Attribution-ShareAlike License
#elif L==5
	   "CC Reconocimiento - Compartir bajo la misma licencia"	// CC Attribution-ShareAlike License	// Okoteve traduccin
#elif L==6
	   "CC Attribuzione - Condividi allo stesso modo"		// CC Attribution-ShareAlike License
#elif L==7
	   "CC Uznanie autorstwa - Na tych samych warunkach"		// CC Attribution-ShareAlike License
#elif L==8
	   "CC Atribui&ccedil;&atilde;o - Compartilha Igual"		// CC Attribution-ShareAlike License
#endif
	   ,
#if L==0
	   "CC Reconeixement - Sense obra derivada"				// CC Attribution-NoDerivs License
#elif L==1
	   "CC Namensnennung - Keine Bearbeitung"				// CC Attribution-NoDerivs License
#elif L==2
	   "CC Attribution - No Derivative Works"				// CC Attribution-NoDerivs License
#elif L==3
	   "CC Reconocimiento - Sin obras derivadas"				// CC Attribution-NoDerivs License
#elif L==4
	   "CC Attribution - Pas de travaux d&eacute;riv&eacute;s"		// CC Attribution-NoDerivs License
#elif L==5
	   "CC Reconocimiento - Sin obras derivadas"				// CC Attribution-NoDerivs License	// Okoteve traduccin
#elif L==6
	   "CC Attribuzione - Non opere derivate"				// CC Attribution-NoDerivs License
#elif L==7
	   "CC Uznanie autorstwa - Bez utwor&oacute;w zale&zdot;nych"		// CC Attribution-NoDerivs License
#elif L==8
	   "CC Atribui&ccedil;&atilde;o - Trabalhos Derivados Pro&iacute;bidos"	// CC Attribution-NoDerivs License
#endif
	   ,
#if L==0
	   "CC Reconeixement - No comercial"			// CC Attribution-NonCommercial License
#elif L==1
	   "CC Namensnennung - Keine kommerzielle Nutzung"	// CC Attribution-NonCommercial License
#elif L==2
	   "CC Attribution - Noncommercial"			// CC Attribution-NonCommercial License
#elif L==3
	   "CC Reconocimiento - No comercial"			// CC Attribution-NonCommercial License
#elif L==4
	   "CC Attribution - Pas d'Utilisation Commerciale"	// CC Attribution-NonCommercial License
#elif L==5
	   "CC Reconocimiento - No comercial"			// CC Attribution-NonCommercial License	// Okoteve traduccin
#elif L==6
	   "CC Attribuzione - Non commerciale"			// CC Attribution-NonCommercial License
#elif L==7
	   "CC Uznanie autorstwa - U&zdot;ycie niekomercyjne"	// CC Attribution-NonCommercial License
#elif L==8
	   "CC Atribui&ccedil;&atilde;o - N&atilde;oComercial"	// CC Attribution-NonCommercial License
#endif
	   ,
#if L==0
	   "CC Reconeixement - No comercial - Compartir igual"						// CC Attribution-NonCommercial-ShareAlike License
#elif L==1
	   "CC Namensnennung - Keine kommerzielle Nutzung - Weitergabe unter gleichen Bedingungen"	// CC Attribution-NonCommercial-ShareAlike License
#elif L==2
	   "CC Attribution - Noncommercial - Share Alike"						// CC Attribution-NonCommercial-ShareAlike License
#elif L==3
	   "CC Reconocimiento - No comercial - Compartir bajo la misma licencia"			// CC Attribution-NonCommercial-ShareAlike License
#elif L==4
	   "CC Attribution - Pas d'Utilisation Commerciale - Partage &agrave; l'Identique"		// CC Attribution-NonCommercial-ShareAlike License
#elif L==5
	   "CC Reconocimiento - No comercial - Compartir bajo la misma licencia"			// CC Attribution-NonCommercial-ShareAlike License	// Okoteve traduccin
#elif L==6
	   "CC Attribuzione - Non commerciale - Condividi allo stesso modo"				// CC Attribution-NonCommercial-ShareAlike License
#elif L==7
	   "CC Uznanie autorstwa - U&zdot;ycie niekomercyjne - Na tych samych warunkach"		// CC Attribution-NonCommercial-ShareAlike License
#elif L==8
	   "CC Atribui&ccedil;&atilde;o - N&atilde;oComercial - Compartilha Igual"			// CC Attribution-NonCommercial-ShareAlike License
#endif
	   ,
#if L==0
	   "CC Reconeixement - No comercial - Sense obra derivada"					// CC Attribution-NonCommercial-NoDerivs License
#elif L==1
	   "CC Namensnennung - Keine kommerzielle Nutzung - Keine Bearbeitung"				// CC Attribution-NonCommercial-NoDerivs License
#elif L==2
	   "CC Attribution - Noncommercial - No Derivative Works"					// CC Attribution-NonCommercial-NoDerivs License
#elif L==3
	   "CC Reconocimiento - No comercial - Sin obras derivadas"					// CC Attribution-NonCommercial-NoDerivs License
#elif L==4
	   "CC Attribution - Pas dUtilisation Commerciale - Pas de travaux d&eacute;riv&eacute;s"	// CC Attribution-NonCommercial-NoDerivs License
#elif L==5
	   "CC Reconocimiento - No comercial - Sin obras derivadas"					// CC Attribution-NonCommercial-NoDerivs License	// Okoteve traduccin
#elif L==6
	   "CC Attribuzione - Non commerciale - Non opere derivate"					// CC Attribution-NonCommercial-NoDerivs License
#elif L==7
	   "CC Uznanie autorstwa - U&zdot;ycie niekomercyjne - Bez utwor&oacute;w zale&zdot;nych"	// CC Attribution-NonCommercial-NoDerivs License
#elif L==8
	   "CC Atribui&ccedil;&atilde;o - N&atilde;oComercial - Trabalhos Derivados Pro&iacute;bidos"	// CC Attribution-NonCommercial-NoDerivs License
#endif
	};

const char *Txt_Link =
#if   L==0
	"Enlla&ccedil;";
#elif L==1
	"Link";
#elif L==2
	"Link";
#elif L==3
	"Enlace";
#elif L==4
	"Lien";
#elif L==5
	"Enlace";	// Okoteve traduccin
#elif L==6
	"Link";
#elif L==7
	"Link";
#elif L==8
	"Liga&ccedil;&atilde;o.";
#endif

const char *Txt_link =
#if   L==0
	"enlla&ccedil;";
#elif L==1
	"Link";
#elif L==2
	"link";
#elif L==3
	"enlace";
#elif L==4
	"lien";
#elif L==5
	"enlace";	// Okoteve traduccin
#elif L==6
	"link";
#elif L==7
	"link";
#elif L==8
	"liga&ccedil;&atilde;o.";
#endif

const char *Txt_Link_X_removed =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"Enlla&ccedil; <strong>%s</strong> eliminat.";
#elif L==1
	"Link <strong>%s</strong> entfernt.";
#elif L==2
	"Link <strong>%s</strong> removed.";
#elif L==3
	"Enlace <strong>%s</strong> eliminado.";
#elif L==4
	"Lien <strong>%s</strong> supprim&eacute;.";
#elif L==5
	"Enlace <strong>%s</strong> eliminado.";	// Okoteve traduccin
#elif L==6
	"Link <strong>%s</strong> rimosso.";
#elif L==7
	"Link <strong>%s</strong> usuniety.";
#elif L==8
	"Liga&ccedil;&atilde;o <strong>%s</strong> removida.";
#endif

const char *Txt_Link_to_announcement_of_exam =
#if   L==0
	"Enlla&ccedil; a convocat&ograve;ria d'examen";
#elif L==1
	"Link zu Aufruf f&uuml;r Pr&uuml;fung";
#elif L==2
	"Link to announcement of exam";
#elif L==3
	"Enlace a convocatoria de examen";
#elif L==4
	"Lien vers convocation &agrave; un examen";
#elif L==5
	"Enlace a convocatoria de examen";	// Okoteve traduccin
#elif L==6
	"Link alla sessione d'esame";
#elif L==7
	"Link do ogloszenie egzaminu";
#elif L==8
	"Link para chamada para exame";
#endif

const char *Txt_List_of_detailed_clicks_in_the_course_X =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"Listado de clics detallados en la asignatura <strong>%s</strong>";	// Necessita traduccio
#elif L==1
	"List of detailed clicks in the course <strong>%s</strong>";		// Need bersetzung
#elif L==2
	"List of detailed clicks in the course <strong>%s</strong>";
#elif L==3
	"Listado de clics detallados en la asignatura <strong>%s</strong>";
#elif L==4
	"List of detailed clicks in the course <strong>%s</strong>";		// Besoin de traduction
#elif L==5
	"Listado de clics detallados en la asignatura <strong>%s</strong>";	// Okoteve traduccin
#elif L==6
	"Lista di scatti dettagliati nel corso <strong>%s</strong>";
#elif L==7
	"Lista szczeg&oacute;lowa klikniec w trakcie <strong>%s</strong>";
#elif L==8
	"List of detailed clicks in the course <strong>%s</strong>";		// Necessita de traduo
#endif

const char *Txt_List_of_nicks_emails_or_IDs =
#if   L==0
	"Lista de @apodos, correos o ID (DNI/c&eacute;dulas)";	// Necessita traduccio
#elif L==1
	"Liste von @Benutzernamen, E-mails oder Ausweis-Nr.";
#elif L==2
	"List of @nicks, e-mails or IDs";
#elif L==3
	"Lista de @apodos, correos o ID (DNI/c&eacute;dulas)";
#elif L==4
	"Liste de num&eacute;ros d'identit&eacute;";
#elif L==5
	"Lista de @apodos, correos o ID (DNI/c&eacute;dulas)";	// Okoteve traduccin
#elif L==6
	"Lista di @nomi-utenti, e-mail o numero carte d'identit&agrave;";
#elif L==7
	"List of @nicks, e-mails or IDs";			// Potrzebujesz tlumaczenie
#elif L==8
	"Lista de @alcunhas, e-mails ou n&ordm;s. identif.";
#endif

const char *Txt_Links =
#if   L==0
	"Enlla&ccedil;os";
#elif L==1
	"Links";
#elif L==2
	"Links";
#elif L==3
	"Enlaces";
#elif L==4
	"Liens";
#elif L==5
	"Enlaces";	// Okoteve traduccin
#elif L==6
	"Link";
#elif L==7
	"Linki";
#elif L==8
	"Liga&ccedil;&otilde;es";
#endif

const char *Txt_Links_copied =
#if   L==0
	"Enlla&ccedil;os copiats";
#elif L==1
	"Links kopiert";
#elif L==2
	"Links copied";
#elif L==3
	"Enlaces copiados";
#elif L==4
	"Fichiers copi&eacute;s";
#elif L==5
	"Enlaces copiados";	// Okoteve traduccin
#elif L==6
	"Link copiati";
#elif L==7
	"Linki kopiowane";
#elif L==8
	"Liga&ccedil;&otilde;es copiadas";
#endif


const char *Txt_Local_address =
#if   L==0
	"Domicilio local";	// Necessita traduccio
#elif L==1
	"Lokale Adresse";
#elif L==2
	"Local address";
#elif L==3
	"Domicilio local";
#elif L==4
	"Adresse locale";
#elif L==5
	"Domicilio local";	// Okoteve traduccin
#elif L==6
	"Indirizzo locale";
#elif L==7
	"Adres lokalny";
#elif L==8
	"Local Address";
#endif

const char *Txt_Location =
#if   L==0
	"Lloc";
#elif L==1
	"Standort";
#elif L==2
	"Location";
#elif L==3
	"Sitio";
#elif L==4
	"Emplacement";
#elif L==5
	"Sitio";	// Okoteve traduccin
#elif L==6
	"Locazione";
#elif L==7
	"Lokalizacja";
#elif L==8
	"Site";
#endif

const char *Txt_LOG_More_info =
#if   L==0
	"Observaciones";	// Necessita traduccio
#elif L==1
	"Weitere Informationen";
#elif L==2
	"More info";
#elif L==3
	"Observaciones";
#elif L==4
	"Plus d'informations";
#elif L==5
	"Observaciones";	// Okoteve traduccin
#elif L==6
	"Maggiori informazioni";
#elif L==7
	"Wiecej informacji";
#elif L==8
	"Mais informa&ccedil;&atilde;o";
#endif

const char *Txt_Log_in =
#if   L==0
	"Iniciar sessi&oacute;";
#elif L==1
	"Anmelden";
#elif L==2
	"Log in";
#elif L==3
	"Iniciar sesi&oacute;n";
#elif L==4
	"Entrer";
#elif L==5
	"Tera&ntilde;emboguapy";
#elif L==6
	"Inizia sessione";
#elif L==7
	"Zaloguj si&eogon;";
#elif L==8
	"Entrar";
#endif

const char *Txt_Log_out =
#if   L==0
	"Tancar sessi&oacute;";
#elif L==1
	"Abmelden";
#elif L==2
	"Log out";
#elif L==3
	"Cerrar sesi&oacute;n";
#elif L==4
	"Fermer session";
#elif L==5
	"S&#7869;";
#elif L==6
	"Chiudi sessione";
#elif L==7
	"Wyloguj si&eogon;";
#elif L==8
	"Fechar sess&atilde;o";
#endif

const char *Txt_logged[Usr_NUM_SEXS] =
   {
#if   L==0	// Usr_SEX_UNKNOWN
	 "identificado/a"	// Necessita traduccio
#elif L==1
	 "angemeldet"
#elif L==2
	 "logged"
#elif L==3
	 "identificado/a"
#elif L==4
	 "connect&eacute;/e"
#elif L==5
	 "identificado/a"	// Okoteve traduccin
#elif L==6
	 "loggato/a"
#elif L==7
	 "zalogowany"
#elif L==8
	 "logado/a"
#endif
	 ,
#if   L==0	// Usr_SEX_FEMALE
	 "identificada"	// Necessita traduccio
#elif L==1
	 "angemeldet"
#elif L==2
	 "logged"
#elif L==3
	 "identificada"
#elif L==4
	 "connect&eacute;e"
#elif L==5
	 "identificada"	// Okoteve traduccin
#elif L==6
	 "loggata"
#elif L==7
	 "zalogowana"
#elif L==8
	 "logada"
#endif
	 ,
#if   L==0	// Usr_SEX_MALE
	 "identificado"	// Necessita traduccio
#elif L==1
	 "angemeldet"
#elif L==2
	 "logged"
#elif L==3
	 "identificado"
#elif L==4
	 "connect&eacute;"
#elif L==5
	 "identificado"	// Okoteve traduccin
#elif L==6
	 "loggato"
#elif L==7
	 "zalogowany"
#elif L==8
	 "logado"
#endif
	 ,
#if   L==0	// Usr_SEX_ALL
	 "identificado/a"	// Necessita traduccio
#elif L==1
	 "angemeldet"
#elif L==2
	 "logged"
#elif L==3
	 "identificado/a"
#elif L==4
	 "connect&eacute;/e"
#elif L==5
	 "identificado/a"	// Okoteve traduccin
#elif L==6
	 "loggato/a"
#elif L==7
	 "zalogowani"
#elif L==8
	 "logado/a"
#endif
	};

const char *Txt_Logo =
#if   L==0
	"Logo";	// Necessita traduccio
#elif L==1
	"Logo";
#elif L==2
	"Logo";
#elif L==3
	"Logo";
#elif L==4
	"Logo";
#elif L==5
	"Logo";	// Okoteve traduccin
#elif L==6
	"Logo";
#elif L==7
	"Logo";
#elif L==8
	"Logotipo";
#endif

const char *Txt_Mail_domain_X_removed =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"Dominio de correo <strong>%s</strong> eliminado.";	// Necessita traduccio
#elif L==1
	"Mail-Dom&auml;ne <strong>%s</strong> entfernt.";
#elif L==2
	"Mail domain <strong>%s</strong> removed.";
#elif L==3
	"Dominio de correo <strong>%s</strong> eliminado.";
#elif L==4
	"Mail domain <strong>%s</strong> removed.";		// Besoin de traduction
#elif L==5
	"Dominio de correo <strong>%s</strong> eliminado.";	// Okoteve traduccin
#elif L==6
	"Campo mail <strong>%s</strong> rimosso.";
#elif L==7
	"Domeny poczty <strong>%s</strong> usuniete.";
#elif L==8
	"Mail domain <strong>%s</strong> removed.";		// Necessita de traduo
#endif

const char *Txt_Mail_domains_allowed_for_notifications =
#if   L==0
	"Dominios de correo permitidos para notificaciones";	// Necessita traduccio
#elif L==1
	"Mail-Dom&auml;nen, an die Benachrichtigungen gesendet werden k&ouml;nnen";
#elif L==2
	"Mail domains allowed for notifications";
#elif L==3
	"Dominios de correo permitidos para notificaciones";
#elif L==4
	"Mail domains allowed for notifications";		// Besoin de traduction
#elif L==5
	"Dominios de correo permitidos para notificaciones";	// Okoteve traduccin
#elif L==6
	"Campo mail permesso per notifiche";
#elif L==7
	"Mail domen dozwolone powiadomien";
#elif L==8
	"Mail domains allowed for notifications";		// Necessita de traduo
#endif

const char *Txt_MAIL_DOMAIN_HELP_ORDER[3] =
   {
#if   L==0
	 "Ordenar por dominio de correo"	// Necessita traduccio
#elif L==1
	 "Nach Mail-Dom&auml;ne sortieren"
#elif L==2
	 "Sort by mail domain"
#elif L==3
	 "Ordenar por dominio de correo"
#elif L==4
	 "Trier par domaine de messagerie"
#elif L==5
	 "Ordenar por dominio de correo"	// Okoteve traduccin
#elif L==6
	 "Ordina per campo mail"
#elif L==7
	 "Sortuj wedlug domeny poczty"
#elif L==8
	 "Classificar por dom&iacute;nio de email"
#endif
	 ,
#if   L==0
	 "Ordenar por informaci&oacute;n"	// Necessita traduccio
#elif L==1
	 "Nach Information sortieren"
#elif L==2
	 "Sort by information"
#elif L==3
	 "Ordenar por informaci&oacute;n"
#elif L==4
	 "Trier par information"
#elif L==5
	 "Ordenar por informaci&oacute;n"	// Okoteve traduccin
#elif L==6
	 "Ordina per informazione"
#elif L==7
	 "Sortuj wedlug informacji"
#elif L==8
	 "Classificar por informa&ccedil;&atilde;o"
#endif
	 ,
#if   L==0
	 "Ordenar por n&ordm; de usuarios"	// Necessita traduccio
#elif L==1
	 "Nach Anzahl der Benutzer sortieren"
#elif L==2
	 "Sort by no. of users"
#elif L==3
	 "Ordenar por n&ordm; de usuarios"
#elif L==4
	 "Trier par nombre d'utilisateurs"
#elif L==5
	 "Ordenar por n&ordm; de usuarios"	// Okoteve traduccin
#elif L==6
	 "Ordina per numero di utenti"
#elif L==7
	 "Sortuj wedlug numerow uzytkownikow"
#elif L==8
	 "Classificar por n&ordm; de utilizadores"
#endif
	};

const char *Txt_MAIL_DOMAIN_ORDER[3] =
   {
#if   L==0
	 "Domini de correu"
#elif L==1
	 "Mail-Dom&auml;ne"
#elif L==2
	 "Mail domain"
#elif L==3
	 "Dominio de correo"
#elif L==4
	 "Domaine de messagerie"
#elif L==5
	 "Dominio de correo"	// Okoteve traduccin
#elif L==6
	 "Campo mail"
#elif L==7
	 "Domeny poczty"
#elif L==8
	 "Dom&iacute;nio de email"
#endif
	 ,
#if   L==0
	 "Informaci&oacute;"
#elif L==1
	 "Information"
#elif L==2
	 "Information"
#elif L==3
	 "Informaci&oacute;n"
#elif L==4
	 "Information"
#elif L==5
	 "Informaci&oacute;n"	// Okoteve traduccin
#elif L==6
	 "Informazione"
#elif L==7
	 "Informacja"
#elif L==8
	 "Informa&ccedil;&atilde;o"
#endif
	 ,
#if   L==0
	 "Usuaris"
#elif L==1
	 "Benutzer"
#elif L==2
	 "Users"
#elif L==3
	 "Usuarios"
#elif L==4
	 "Utilisateurs"
#elif L==5
	 "Usuarios"	// Okoteve traduccin
#elif L==6
	 "Utenti"
#elif L==7
	 "Uzytkownikow"
#elif L==8
	 "Utilizadores"
#endif
	};

const char *Txt_Mandatory_enrollment =		// (to a type of group)
#if   L==0
	"&iquest;Adscripci&oacute;n obligatoria?";	// Necessita traduccio
#elif L==1
	"Obligatorische beitreten?";
#elif L==2
	"Mandatory enrollment?";
#elif L==3
	"&iquest;Adscripci&oacute;n obligatoria?";
#elif L==4
	"Inscription obligatoire?";
#elif L==5
	"&iquest;Adscripci&oacute;n obligatoria?";	// Okoteve traduccin
#elif L==6
	"Iscrizione obbligatoria?";
#elif L==7
	"Obowiazkowa rejestracja?";
#elif L==8
	"Inscri&ccedil;&atilde;o obrigat&oacute;ria?";
#endif

const char *Txt_Mark_all_notifications_as_read =
#if   L==0
	"Marca totes les notificacions com llegides";
#elif L==1
	"Markieren Sie alle Anmeldungen als gelesen";
#elif L==2
	"Mark all notifications as read";
#elif L==3
	"Marcar todas las notificaciones como le&iacute;das";
#elif L==4
	"Marquer tous les notifications comme lus";
#elif L==5
	"Marcar todas las notificaciones como le&iacute;das";	// Okoteve traduccin
#elif L==6
	"Segnare tutte le notifiche come gi&grave; letti";
#elif L==7
	"Mark all notifications as read";			// Potrzebujesz tlumaczenie
#elif L==8
	"Marcar todas as notifica&ccedil;&otilde;es como lidas";
#endif

const char *Txt_Marks_management_zone =
#if   L==0
	"Zona d'administraci&oacute; de qualificacions";
#elif L==1
	"Bewertungs-Verwaltung";
#elif L==2
	"Marks management area";
#elif L==3
	"Zona de administraci&oacute;n de calificaciones";
#elif L==4
	"Zone de gestion de notes";
#elif L==5
	"Zona de administraci&oacute;n de calificaciones";	// Okoteve traduccin
#elif L==6
	"Area di gestione dei risultati";
#elif L==7
	"Obszar zarzadzania znak&oacute;w";
#elif L==8
	"Zona de gest&atilde;o de notas";
#endif

const char *Txt_marks_management_zone =
#if   L==0
	"zona de administraci&oacute;n de calificaciones";	// Necessita traduccio
#elif L==1
	"Bewertungs-Verwaltung";
#elif L==2
	"marks management area";
#elif L==3
	"zona de administraci&oacute;n de calificaciones";
#elif L==4
	"zone de gestion de notes";
#elif L==5
	"zona de administraci&oacute;n de calificaciones";	// Okoteve traduccin
#elif L==6
	"area di gestione dei risultati";
#elif L==7
	"obszar zarzadzania znak&oacute;w";
#elif L==8
	"zona de gest&atilde;o de notas";
#endif

const char *Txt_Marks_zone =
#if   L==0
	"Zona de qualificacions";
#elif L==1
	"Bewertungs-Bereich";
#elif L==2
	"Marks area";
#elif L==3
	"Zona de calificaciones";
#elif L==4
	"Zone de notes";
#elif L==5
	"Zona de calificaciones";	// Okoteve traduccin
#elif L==6
	"Area risultati";
#elif L==7
	"Powierzchnia znak&oacute;w";
#elif L==8
	"Zona de notas";
#endif

const char *Txt_Materials =
#if   L==0
	"Materiales";	// Necessita traduccio
#elif L==1
	"Material";
#elif L==2
	"Materials";
#elif L==3
	"Materiales";
#elif L==4
	"Mat&eacute;riaux";
#elif L==5
	"Materiales";	// Okoteve traduccin
#elif L==6
	"Materiale";
#elif L==7
	"Materialy";
#elif L==8
	"Materiais";
#endif

const char *Txt_Max_levels =
#if   L==0
	"M&aacute;x. niveles";	// Necessita traduccio
#elif L==1
	"Max. Ebenen";
#elif L==2
	"Max. levels";
#elif L==3
	"M&aacute;x. niveles";
#elif L==4
	"Max. niveaux";
#elif L==5
	"M&aacute;x. niveles";	// Okoteve traduccin
#elif L==6
	"Max. livelli";
#elif L==7
	"Max. poziom&oacute;w";
#elif L==8
	"M&aacute;x. n&iacute;veis";
#endif

const char *Txt_Max_BR_students =
#if   L==0
	"M&agrave;xim<br />d'estud.";
#elif L==1
	"Max.<br />Stud.";
#elif L==2
	"Max.<br />stud.";
#elif L==3
	"M&aacute;x.<br />estud.";
#elif L==4
	"Max.<br />&eacute;tud.";
#elif L==5
	"M&aacute;x.<br />estud.";	// Okoteve traduccin
#elif L==6
	"Max.<br />stud.";
#elif L==7
	"Max.<br />stud.";
#elif L==8
	"M&aacute;x.<br />estud.";
#endif

const char *Txt_maximum =
#if   L==0
	"m&aacute;ximo";	// Necessita traduccio
#elif L==1
	"Maximum";
#elif L==2
	"maximum";
#elif L==3
	"m&aacute;ximo";
#elif L==4
	"maximum";
#elif L==5
	"m&aacute;ximo";	// Okoteve traduccin
#elif L==6
	"massimo";
#elif L==7
	"maksimum";
#elif L==8
	"m&aacute;ximo";
#endif

const char *Txt_Menu =
#if   L==0
	"Men&uacute;";
#elif L==1
	"Men&uuml;";
#elif L==2
	"Menu";
#elif L==3
	"Men&uacute;";
#elif L==4
	"Menu";
#elif L==5
	"Men&uacute;";	// Okoteve traduccin
#elif L==6
	"Menu";
#elif L==7
	"Menu";
#elif L==8
	"Menu";
#endif

const char *Txt_MENU_NAMES[Mnu_NUM_MENUS] =
   {
#if   L==0
	 "Horitzontal"
#elif L==1
	 "Horizontal"
#elif L==2
	 "Horizontal"
#elif L==3
	 "Horizontal"
#elif L==4
	 "Horizontal"
#elif L==5
	 "Horizontal"	// Okoteve traduccin
#elif L==6
	 "Orizzontale"
#elif L==7
	 "Poziomy"
#elif L==8
	 "Horizontal"
#endif
	 ,
#if   L==0
	 "Vertical"
#elif L==1
	 "Vertikal"
#elif L==2
	 "Vertical"
#elif L==3
	 "Vertical"
#elif L==4
	 "Vertical"
#elif L==5
	 "Vertical"		// Okoteve traduccin
#elif L==6
	 "Verticale"
#elif L==7
	 "Pionowy"
#elif L==8
	 "Vertical"
#endif
	};

const char *Txt_MENU_TITLE[Tab_NUM_TABS][Act_MAX_OPTIONS_IN_MENU_PER_TAB] =
{
	{
	// TabUnk ****************************************************
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	},
	{
	// TabSys *****************************************************
	// ActSysReqSch
#if   L==0
	"Cercar"
#elif L==1
	"Suchen"
#elif L==2
	"Search"
#elif L==3
	"Buscar"
#elif L==4
	"Chercher"
#elif L==5
	"Heka"
#elif L==6
	"Cercare"
#elif L==7
	"Szukaj"
#elif L==8
	"Procurar"
#endif
	,
	// ActSeeCty
#if   L==0
	"Pa&iuml;sos"
#elif L==1
	"L&auml;nder"
#elif L==2
	"Countries"
#elif L==3
	"Pa&iacute;ses"
#elif L==4
	"Pays"
#elif L==5
	"Tet&atilde;"
#elif L==6
	"Paesi"
#elif L==7
	"Kraje"
#elif L==8
	"Pa&iacute;ses"
#endif
	,
	// ActSeePen
#if   L==0
	"Pendents"
#elif L==1
	"Anh&auml;ngig"
#elif L==2
	"Pending"
#elif L==3
	"Pendientes"
#elif L==4
	"En attente"
#elif L==5
	"Pendientes"	// Okoteve traduccin
#elif L==6
	"In sospeso"
#elif L==7
	"Oczekuj&aogon;cy"
#elif L==8
	"Pendentes"
#endif
	,
	// ActReqRemOldCrs
#if   L==0
	"Antics"
#elif L==1
	"Fr&uuml;here"
#elif L==2
	"Old"
#elif L==3
	"Antiguas"
#elif L==4
	"Anciens"
#elif L==5
	"Antiguas"	// Okoteve traduccin
#elif L==6
	"Vecchi"
#elif L==7
	"Starych"
#elif L==8
	"Antigas"
#endif
	,
	// ActSeeDegTyp
#if   L==0
	"Estudis"
#elif L==1
	"Studium"
#elif L==2
	"Studies"
#elif L==3
	"Estudios"
#elif L==4
	"&Eacute;tudes"
#elif L==5
	"Estudios"	// Okoteve traduccin
#elif L==6
	"Studi"
#elif L==7
	"Studia"
#elif L==8
	"Estudos"
#endif
	,
	// ActSeeMai
#if   L==0
	"Dominis"
#elif L==1
	"Dom&auml;nen"
#elif L==2
	"Domains"
#elif L==3
	"Dominios"
#elif L==4
	"Domaines"
#elif L==5
	"Dominios"	// Okoteve traduccin
#elif L==6
	"Campi"
#elif L==7
	"Domen"
#elif L==8
	"Dom&iacute;nios"
#endif
	,
	// ActSeeBan
#if   L==0
	"Banners"
#elif L==1
	"Werbebanners"
#elif L==2
	"Banners"
#elif L==3
	"Banners"
#elif L==4
	"Banni&eacute;res"
#elif L==5
	"Banners"
#elif L==6
	"Banners"
#elif L==7
	"Banners"
#elif L==8
	"Banners"
#endif
	,
	// ActSeeLnk
#if   L==0
	"Enlla&ccedil;os"
#elif L==1
	"Links"
#elif L==2
	"Links"
#elif L==3
	"Enlaces"
#elif L==4
	"Liens"
#elif L==5
	"Ambu&eacute;re"
#elif L==6
	"Link"
#elif L==7
	"Linki"
#elif L==8
	"Liga&ccedil;&otilde;es"
#endif
	,
	// ActLstPlg
#if   L==0
	"Complements"
#elif L==1
	"Plugins"
#elif L==2
	"Plugins"
#elif L==3
	"Complementos"
#elif L==4
	"Plugins"
#elif L==5
	"Complementos"	// Okoteve traduccin
#elif L==6
	"Plugin"
#elif L==7
	"Plugin&oacute;w"
#elif L==8
	"Plugins"
#endif
	,
	// ActSetUp
#if   L==0
	"Instal lar"
#elif L==1
	"Einrichten"
#elif L==2
	"Set up"
#elif L==3
	"Instalar"
#elif L==4
	"Installer"
#elif L==5
	"Instalar"	// Okoteve traduccin
#elif L==6
	"Setup"
#elif L==7
	"Set up"
#elif L==8
	"Instalar"
#endif
	,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	},
	{
	// TabCty *****************************************************
	// ActCtyReqSch
#if   L==0
	"Cercar"
#elif L==1
	"Suchen"
#elif L==2
	"Search"
#elif L==3
	"Buscar"
#elif L==4
	"Chercher"
#elif L==5
	"Heka"
#elif L==6
	"Cercare"
#elif L==7
	"Szukaj"
#elif L==8
	"Procurar"
#endif
	,
	// ActSeeCtyInf
#if   L==0
	"Informaci&oacute;"
#elif L==1
	"Information"
#elif L==2
	"Information"
#elif L==3
	"Informaci&oacute;n"
#elif L==4
	"Information"
#elif L==5
	"Informaci&oacute;n"	// Okoteve traduccin
#elif L==6
	"Informazione"
#elif L==7
	"Informacja"
#elif L==8
	"Informa&ccedil;&atilde;o"
#endif
	,
	// ActSeeIns
#if   L==0
	"Institucions"
#elif L==1
	"Hochschulen"
#elif L==2
	"Institutions"
#elif L==3
	"Instituciones"
#elif L==4
	"&Eacute;tablissements"
#elif L==5
	"Mbo'ehao"
#elif L==6
	"Istituzioni"
#elif L==7
	"Instytucje"
#elif L==8
	"Institu&ccedil;&otilde;es"
#endif
	,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	},
	{
	// TabIns *****************************************************
	// ActInsReqSch
#if   L==0
	"Cercar"
#elif L==1
	"Suchen"
#elif L==2
	"Search"
#elif L==3
	"Buscar"
#elif L==4
	"Chercher"
#elif L==5
	"Heka"
#elif L==6
	"Cercare"
#elif L==7
	"Szukaj"
#elif L==8
	"Procurar"
#endif
	,
	// ActSeeInsInf
#if   L==0
	"Informaci&oacute;"
#elif L==1
	"Information"
#elif L==2
	"Information"
#elif L==3
	"Informaci&oacute;n"
#elif L==4
	"Information"
#elif L==5
	"Informaci&oacute;n"	// Okoteve traduccin
#elif L==6
	"Informazione"
#elif L==7
	"Informacja"
#elif L==8
	"Informa&ccedil;&atilde;o"
#endif
	,
	// ActSeeCtr
#if   L==0
	"Centres"
#elif L==1
	"Lehrinstitute"
#elif L==2
	"Centres"
#elif L==3
	"Centros"
#elif L==4
	"Centres"
#elif L==5
	"Mbo'ehao"
#elif L==6
	"Centri"
#elif L==7
	"Centra"
#elif L==8
	"Centros"
#endif
	,
	// ActSeeDpt
#if   L==0
	"Departaments"
#elif L==1
	"Abteilungen"
#elif L==2
	"Departments"
#elif L==3
	"Departamentos"
#elif L==4
	"D&eacute;partements"
#elif L==5
	"Peh&#7869;"
#elif L==6
	"Dipartimenti"
#elif L==7
	"Wydzia&lstrok;y"
#elif L==8
	"Departamentos"
#endif
	,
	// ActSeePlc
#if   L==0
	"Llocs"
#elif L==1
	"Standorte"
#elif L==2
	"Places"
#elif L==3
	"Lugares"
#elif L==4
	"Emplacements"
#elif L==5
	"Tenda"
#elif L==6
	"Localit&agrave;"
#elif L==7
	"Miejsca"
#elif L==8
	"Localiza&ccedil;oes"
#endif
	,
	// ActSeeHld
#if   L==0
	"Festius"
#elif L==1
	"Feiertage"
#elif L==2
	"Holidays"
#elif L==3
	"Festivos"
#elif L==4
	"F&eacute;ri&eacute;s"
#elif L==5
	"Arete"
#elif L==6
	"Vacanze"
#elif L==7
	"Noclegi"
#elif L==8
	"F&eacute;rias"
#endif
	,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	},
	{
	// TabCtr *****************************************************
	// ActCtrReqSch
#if   L==0
	"Cercar"
#elif L==1
	"Suchen"
#elif L==2
	"Search"
#elif L==3
	"Buscar"
#elif L==4
	"Chercher"
#elif L==5
	"Heka"
#elif L==6
	"Cercare"
#elif L==7
	"Szukaj"
#elif L==8
	"Procurar"
#endif
	,
	// ActSeeCtrInf
#if   L==0
	"Informaci&oacute;"
#elif L==1
	"Information"
#elif L==2
	"Information"
#elif L==3
	"Informaci&oacute;n"
#elif L==4
	"Information"
#elif L==5
	"Informaci&oacute;n"	// Okoteve traduccin
#elif L==6
	"Informazione"
#elif L==7
	"Informacja"
#elif L==8
	"Informa&ccedil;&atilde;o"
#endif
	,
	// ActSeeDeg
#if   L==0
	"Titulacions"
#elif L==1
	"Studiengang"
#elif L==2
	"Degrees"
#elif L==3
	"Titulaciones"
#elif L==4
	"&Eacute;tudes"
#elif L==5
	"Arandur&atilde;"
#elif L==6
	"Lauree"
#elif L==7
	"Stopnie"
#elif L==8
	"Titula&ccedil;&atilde;os"
#endif
	,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	},
	{
	// TabDeg *****************************************************
	// ActDegReqSch
#if   L==0
	"Cercar"
#elif L==1
	"Suchen"
#elif L==2
	"Search"
#elif L==3
	"Buscar"
#elif L==4
	"Chercher"
#elif L==5
	"Heka"
#elif L==6
	"Cercare"
#elif L==7
	"Szukaj"
#elif L==8
	"Procurar"
#endif
	,
	// ActSeeDegInf
#if   L==0
	"Informaci&oacute;"
#elif L==1
	"Information"
#elif L==2
	"Information"
#elif L==3
	"Informaci&oacute;n"
#elif L==4
	"Information"
#elif L==5
	"Informaci&oacute;n"	// Okoteve traduccin
#elif L==6
	"Informazione"
#elif L==7
	"Informacja"
#elif L==8
	"Informa&ccedil;&atilde;o"
#endif
	,
	// ActSeeCrs
#if   L==0
	"Assignatures"
#elif L==1
	"Kursen"
#elif L==2
	"Courses"
#elif L==3
	"Asignaturas"
#elif L==4
	"Mati&egrave;res"
#elif L==5
	"Mbo'esyry"
#elif L==6
	"Corsi"
#elif L==7
	"Kurs&oacute;w"
#elif L==8
	"Disciplinas"
#endif
	,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	},
	{
	// TabCrs *****************************************************
	// ActCtrReqSch
#if   L==0
	"Cercar"
#elif L==1
	"Suchen"
#elif L==2
	"Search"
#elif L==3
	"Buscar"
#elif L==4
	"Chercher"
#elif L==5
	"Heka"
#elif L==6
	"Cercare"
#elif L==7
	"Szukaj"
#elif L==8
	"Procurar"
#endif
	,
	// ActSeeCrsInf
#if   L==0
	"Informaci&oacute;"
#elif L==1
	"Information"
#elif L==2
	"Information"
#elif L==3
	"Informaci&oacute;n"
#elif L==4
	"Information"
#elif L==5
	"Informaci&oacute;n"	// Okoteve traduccin
#elif L==6
	"Informazione"
#elif L==7
	"Informacja"
#elif L==8
	"Informa&ccedil;&atilde;o"
#endif
	,
	// ActSeeTchGui
#if   L==0
	"Guia"
#elif L==1
	"Hinweise"
#elif L==2
	"Guide"
#elif L==3
	"Gu&iacute;a"
#elif L==4
	"Guide"
#elif L==5
	"Gu&iacute;a"	// Okoteve traduccin
#elif L==6
	"Guida"
#elif L==7
	"Podr&eogon;cznik"
#elif L==8
	"Guia"
#endif
	,
	// ActSeeSyl
#if   L==0
	"Programa"
#elif L==1
	"Studienplan"
#elif L==2
	"Syllabus"
#elif L==3
	"Programa"
#elif L==4
	"Plan"
#elif L==5
	"Programa"	// Okoteve traduccin
#elif L==6
	"Programma"
#elif L==7
	"Syllabus"
#elif L==8
	"Programa"
#endif
	,
	// ActSeeDoc
#if   L==0
	"Documents"
#elif L==1
	"Dokumente"
#elif L==2
	"Documents"
#elif L==3
	"Documentos"
#elif L==4
	"Documents"
#elif L==5
	"Kuatia"
#elif L==6
	"Documenti"
#elif L==7
	"Dokument&oacute;w"
#elif L==8
	"Documentos"
#endif
	,
	// ActAdmCom
#if   L==0
	"Compartits"
#elif L==1
	"Freigegebene"
#elif L==2
	"Shared"
#elif L==3
	"Compartidos"
#elif L==4
	"Partag&eacute;s"
#elif L==5
	"Compartidos"	// Okoteve traduccin
#elif L==6
	"Condivisi"
#elif L==7
	"Udost&eogon;pnione"
#elif L==8
	"Compartilhados"
#endif
	,
	// ActSeeCrsTimTbl
#if   L==0
	"Horari"
#elif L==1
	"Stundenplan"
#elif L==2
	"Timetable"
#elif L==3
	"Horario"
#elif L==4
	"Horaire"
#elif L==5
	"Horario"	// Okoteve traduccin
#elif L==6
	"Orario"
#elif L==7
	"Plan"
#elif L==8
	"Hor&aacute;rio"
#endif
	,
	// ActSeeBib
#if   L==0
	"Bibliografia"
#elif L==1
	"Literatur"
#elif L==2
	"Bibliography"
#elif L==3
	"Bibliograf&iacute;a"
#elif L==4
	"Bibliographie"
#elif L==5
	"Arandukapurupyre"
#elif L==6
	"Bibliografia"
#elif L==7
	"Bibliografia"
#elif L==8
	"Bibliografia"
#endif
	,
	// ActSeeFAQ
#if   L==0
	"FAQ"	// Necessita traduccio
#elif L==1
	"FAQ"
#elif L==2
	"FAQ"
#elif L==3
	"FAQ"
#elif L==4
	"FAQ"
#elif L==5
	"FAQ"	// Okoteve traduccin
#elif L==6
	"FAQ"
#elif L==7
	"FAQ"
#elif L==8
	"FAQ"
#endif
	,
	// ActSeeCrsLnk
#if   L==0
	"Enlla&ccedil;os"
#elif L==1
	"Links"
#elif L==2
	"Links"
#elif L==3
	"Enlaces"
#elif L==4
	"Liens"
#elif L==5
	"Ambu&eacute;re"
#elif L==6
	"Link"
#elif L==7
	"Linki"
#elif L==8
	"Liga&ccedil;&otilde;es"
#endif
	,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	},
	{
	// TabAss *****************************************************
	// ActSeeAss
#if   L==0
	"Sistema"
#elif L==1
	"System"
#elif L==2
	"System"
#elif L==3
	"Sistema"
#elif L==4
	"Syst&egrave;me"
#elif L==5
	"Sistema"	// Okoteve traduccin
#elif L==6
	"Sistema"
#elif L==7
	"System"
#elif L==8
	"Sistema"
#endif
	,
	// ActSeeAsg
#if   L==0
	"Activitats"
#elif L==1
	"Aufgaben"
#elif L==2
	"Assignments"
#elif L==3
	"Actividades"
#elif L==4
	"Activit&eacute;s"
#elif L==5
	"Tar&eacute;a"
#elif L==6
	"Attivit&agrave;"
#elif L==7
	"Zadania"
#elif L==8
	"Atividades"
#endif
	,
	// ActAdmAsgWrkUsr
#if   L==0
	"Treballs"
#elif L==1
	"Hausarbeiten"
#elif L==2
	"Homework"
#elif L==3
	"Trabajos"
#elif L==4
	"Travaux"
#elif L==5
	"Tembiapo"
#elif L==6
	"Lavori"
#elif L==7
	"Zadanie"
#elif L==8
	"Trabalhos"
#endif
	,
	// ActReqAsgWrkCrs
#if   L==0
	"Treballs"
#elif L==1
	"Hausarbeiten"
#elif L==2
	"Homework"
#elif L==3
	"Trabajos"
#elif L==4
	"Travaux"
#elif L==5
	"Tembiapo"
#elif L==6
	"Lavori"
#elif L==7
	"Zadanie"
#elif L==8
	"Trabalhos"
#endif
	,
	// ActReqTst
#if   L==0
	"Tests"
#elif L==1
	"Tests"
#elif L==2
	"Tests"
#elif L==3
	"Tests"
#elif L==4
	"Tests"
#elif L==5
	"Tests"	// Okoteve traduccin
#elif L==6
	"Test"
#elif L==7
	"Test&oacute;w"
#elif L==8
	"Testes"
#endif
	,
	// ActSeeCal
#if   L==0
	"Calendari"
#elif L==1
	"Kalender"
#elif L==2
	"Calendar"
#elif L==3
	"Calendario"
#elif L==4
	"Calendrier"
#elif L==5
	"Arapapaha"
#elif L==6
	"Calendario"
#elif L==7
	"Kalendarz"
#elif L==8
	"Calend&aacute;rio"
#endif
	,
	// ActSeeExaAnn
#if   L==0
	"Convocat&ograve;ries"
#elif L==1
	"Pr&uuml;fungsaufrufe"
#elif L==2
	"Announcements"
#elif L==3
	"Convocatorias"
#elif L==4
	"Convocations"
#elif L==5
	"Convocatorias"	// Okoteve traduccin
#elif L==6
	"Appelli"
#elif L==7
	"Ogloszenia"
#elif L==8
	"Chamadas"
#endif
	,
	// ActSeeMrk
#if   L==0
	"Qualificacions"
#elif L==1
	"Bewertungen"
#elif L==2
	"Marks"
#elif L==3
	"Calificaciones"
#elif L==4
	"Notes"
#elif L==5
	"Calificaciones"	// Okoteve traduccin
#elif L==6
	"Risultati"
#elif L==7
	"Marks"
#elif L==8
	"Notas"
#endif
	,
	// ActSeeRecCrs
#if   L==0
	"Fitxa"
#elif L==1
	"Profil"
#elif L==2
	"Record"
#elif L==3
	"Ficha"
#elif L==4
	"Carte"
#elif L==5
	"Ficha"	// Okoteve traduccin
#elif L==6
	"Scheda"
#elif L==7
	"Rekord"
#elif L==8
	"Cart&atilde;o"
#endif
	,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	},
	{
	// TabUsr *****************************************************
	// ActReqSelGrp
#if   L==0
	"Grups"
#elif L==1
	"Gruppen"
#elif L==2
	"Groups"
#elif L==3
	"Grupos"
#elif L==4
	"Groupes"
#elif L==5
	"Aty"
#elif L==6
	"Gruppi"
#elif L==7
	"Grupy"
#elif L==8
	"Grupos"
#endif
	,
	// ActLstInv
#if   L==0
	"Convidats"
#elif L==1
	"Eingeladen"
#elif L==2
	"Guests"
#elif L==3
	"Invitados"
#elif L==4
	"Invit&eacute;s"
#elif L==5
	"Invitados"	// Okoteve traduccin
#elif L==6
	"Ospiti"
#elif L==7
	"Zaproszenie"
#elif L==8
	"Convidados"
#endif
	,
	// ActLstStd
#if   L==0
	"Estudiants"
#elif L==1
	"Studenten"
#elif L==2
	"Students"
#elif L==3
	"Estudiantes"
#elif L==4
	"&Eacute;tudiants"
#elif L==5
	"Estudiantes"	// Okoteve traduccin
#elif L==6
	"Studenti"
#elif L==7
	"Student&oacute;w"
#elif L==8
	"Estudantes"
#endif
	,
	// ActSeeAtt
#if   L==0
	"Assist&egrave;ncia"
#elif L==1
	"Anwesenheit"
#elif L==2
	"Attendance"
#elif L==3
	"Asistencia"
#elif L==4
	"Pr&eacute;sence"
#elif L==5
	"Asistencia"	// Okoteve traduccin
#elif L==6
	"Frequenza"
#elif L==7
	"Frekwencji"
#elif L==8
	"Presen&ccedil;a"
#endif
	,
	// ActLstTch
#if   L==0
	"Professors"
#elif L==1
	"Lehrkraft"
#elif L==2
	"Teachers"
#elif L==3
	"Profesores"
#elif L==4
	"Enseignants"
#elif L==5
	"Mbo'eh&aacute;ra"
#elif L==6
	"Professori"
#elif L==7
	"Nauczyciele"
#elif L==8
	"Professores"
#endif
	,
	// ActLstAdm
#if   L==0
	"Administrad."	// Administradors
#elif L==1
	"Verwalter"
#elif L==2
	"Administrat."	// Administrators
#elif L==3
	"Administrad."	// Administradores
#elif L==4
	"Administrat."	// Administrateurs
#elif L==5
	"Administrad."	// Administradores	// Okoteve traduccin
#elif L==6
	"Amministrat."	// Amministratori
#elif L==7
	"Administrat."	// Administratorzy
#elif L==8
	"Administrad."	// Administradores
#endif
	,
	// ActReqSignUp
#if   L==0
	"Inscriure"
#elif L==1
	"Anmeldung"
#elif L==2
	"Sign up"
#elif L==3
	"Inscribirme"
#elif L==4
	"Signer"
#elif L==5
	"Inscribirme"	// Okoteve traduccin
#elif L==6
	"Iscrivi"
#elif L==7
	"Zapisa&cacute;"
#elif L==8
	"Inscrever"
#endif
	,
	// ActSeeSignUpReq
#if   L==0
	"Peticions"
#elif L==1
	"Ersuchen"
#elif L==2
	"Requests"
#elif L==3
	"Solicitudes"
#elif L==4
	"Demandes"
#elif L==5
	"Solicitudes"// Okoteve traduccin
#elif L==6
	"Richieste"
#elif L==7
	"Upraszanie"
#elif L==8
	"Solicita&ccedil;&otilde;es"
#endif
	,
	// ActReqMdfSevUsr
#if   L==0
	"Administrar"
#elif L==1
	"Verwalten"
#elif L==2
	"Administer"
#elif L==3
	"Administrar"
#elif L==4
	"G&eacute;rer"
#elif L==5
	"Administrar"	// Okoteve traduccin
#elif L==6
	"Gestire"
#elif L==7
	"Zarz&aogon;dzaj"
#elif L==8
	"Gerenciar"
#endif
	,
	// ActLstCon
#if   L==0
	"Connectats"
#elif L==1
	"Angemeldet"
#elif L==2
	"Connected"
#elif L==3
	"Conectados"
#elif L==4
	"Connect&eacute;s"
#elif L==5
	"Conectados"	// Okoteve traduccin
#elif L==6
	"Connessi"
#elif L==7
	"Online"
#elif L==8
	"Conectados"
#endif
	,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	},
	{
	// TabMsg *****************************************************
	// ActSeeNtf
#if   L==0
	"Notificacions"
#elif L==1
	"Anmeldungen"
#elif L==2
	"Notifications"
#elif L==3
	"Notificaciones"
#elif L==4
	"Notifications"
#elif L==5
	"Momarandu"
#elif L==6
	"Notifiche"
#elif L==7
	"Powiadomienia"
#elif L==8
	"Notifica&ccedil;&otilde;es"
#endif
	,
	// ActSeeAnn
#if   L==0
	"Anuncis"
#elif L==1
	"Bekanntmachungen"
#elif L==2
	"Announcements"
#elif L==3
	"Anuncios"
#elif L==4
	"Annonces"
#elif L==5
	"Anuncios"	// Okoteve traduccin
#elif L==6
	"Annunci"
#elif L==7
	"Og&lstrok;oszenia"
#elif L==8
	"An&uacute;ncios"
#endif
	,
	// ActSeeNot
#if   L==0
	"Avisos"
#elif L==1
	"Ank&uuml;ndigungen"
#elif L==2
	"Notices"
#elif L==3
	"Avisos"
#elif L==4
	"Avis"
#elif L==5
	"Marandu"
#elif L==6
	"Avvisi"
#elif L==7
	"Ogloszenia"
#elif L==8
	"Avisos"
#endif
	,
	// ActSeeFor
#if   L==0
	"F&ograve;rums"
#elif L==1
	"Foren"
#elif L==2
	"Forums"
#elif L==3
	"Foros"
#elif L==4
	"Forums"
#elif L==5
	"Foros"	// Okoteve traduccin
#elif L==6
	"Forum"
#elif L==7
	"Forum"
#elif L==8
	"F&oacute;runs"
#endif
	,
	// ActSeeChtRms
#if   L==0
	"Xat"
#elif L==1
	"Chat"
#elif L==2
	"Chat"
#elif L==3
	"Chat"
#elif L==4
	"Chat"
#elif L==5
	"&Ntilde;omongeta"
#elif L==6
	"Chat"
#elif L==7
	"Chat"
#elif L==8
	"Chat"
#endif
	,
	// ActReqMsgUsr
#if   L==0
	"Redactar"
#elif L==1
	"Schreiben"
#elif L==2
	"Write"
#elif L==3
	"Redactar"
#elif L==4
	"Ecrire"
#elif L==5
	"Redactar"	// Okoteve traduccin
#elif L==6
	"Scrivi"
#elif L==7
	"Napisz"
#elif L==8
	"Escrever"
#endif
	,
	// ActSeeRcvMsg
#if   L==0
	"Rebuts"
#elif L==1
	"Empfangene"
#elif L==2
	"Received"
#elif L==3
	"Recibidos"
#elif L==4
	"Re&ccedil;us"
#elif L==5
	"Recibidos"	// Okoteve traduccin
#elif L==6
	"Ricevuti"
#elif L==7
	"Odebrane"
#elif L==8
	"Recebidas"
#endif
	,
	// ActSeeSntMsg
#if   L==0
	"Enviats"
#elif L==1
	"Gesendete"
#elif L==2
	"Sent"
#elif L==3
	"Enviados"
#elif L==4
	"Envoy&eacute;s"
#elif L==5
	"Enviados"	// Okoteve traduccin
#elif L==6
	"Inviati"
#elif L==7
	"Wysylane"
#elif L==8
	"Enviadas"
#endif
	,
	// ActMaiStd
#if   L==0
	"Correu"
#elif L==1
	"E-Mail"
#elif L==2
	"E-mail"
#elif L==3
	"Correo"
#elif L==4
	"Courrier"
#elif L==5
	"Correo"	// Okoteve traduccin
#elif L==6
	"E-mail"
#elif L==7
	"E-mail"
#elif L==8
	"E-mail"
#endif
	,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	},
	{
	// TabSta **********************************************************************
	// ActSeeAllSvy
#if   L==0
	"Enquestes"
#elif L==1
	"Umfragen"
#elif L==2
	"Surveys"
#elif L==3
	"Encuestas"
#elif L==4
	"Sondages"
#elif L==5
	"Encuestas"	// Okoteve traduccin
#elif L==6
	"Sondaggi"
#elif L==7
	"Ankiety"
#elif L==8
	"Inqu&eacute;ritos"
#endif
	,
	// ActReqUseGbl
#if   L==0
	"Xifres"
#elif L==1
	"Zahlen"
#elif L==2
	"Figures"
#elif L==3
	"Cifras"
#elif L==4
	"Chiffres"
#elif L==5
	"Cifras"	// Okoteve traduccin
#elif L==6
	"Cifre"
#elif L==7
	"Cyfry"
#elif L==8
	"N&uacute;meros"
#endif
	,
	// ActSeePhoDeg
#if   L==0
	"Titulacions"
#elif L==1
	"Studiengang"
#elif L==2
	"Degrees"
#elif L==3
	"Titulaciones"
#elif L==4
	"&Eacute;tudes"
#elif L==5
	"Arandur&atilde;"
#elif L==6
	"Lauree"
#elif L==7
	"Stopnie"
#elif L==8
	"Titula&ccedil;&atilde;os"
#endif
	,
	// ActReqStaCrs
#if   L==0
	"Indicadors"
#elif L==1
	"Anzeigen"
#elif L==2
	"Indicators"
#elif L==3
	"Indicadores"
#elif L==4
	"Indicateurs"
#elif L==5
	"Indicadores"	// Okoteve traduccin
#elif L==6
	"Indicatori"
#elif L==7
	"Wska&zacute;niki"
#elif L==8
	"Indicadores"
#endif
	,
	// ActReqAccGbl
#if   L==0
	"Accessos"
#elif L==1
	"Anmeldungen"
#elif L==2
	"Logins"
#elif L==3
	"Accesos"
#elif L==4
	"Visites"
#elif L==5
	"Accesos"	// Okoteve traduccin
#elif L==6
	"Accessi"
#elif L==7
	"Logowanie"
#elif L==8
	"Visitas"
#endif
	,
	// ActLstClk
#if   L==0
	"Clics"
#elif L==1
	"Klicks"
#elif L==2
	"Clicks"
#elif L==3
	"Clics"
#elif L==4
	"Clics"
#elif L==5
	"Clics"	// Okoteve traduccin
#elif L==6
	"Clic"
#elif L==7
	"Klikni&eogon;&cacute;"
#elif L==8
	"Cliques"
#endif
	,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	},
	{
	// TabPrf *****************************************************
	// ActFrmLogIn
#if   L==0
	"Sessi&oacute;"
#elif L==1
	"Sitzung"
#elif L==2
	"Session"
#elif L==3
	"Sesi&oacute;n"
#elif L==4
	"Session"
#elif L==5
	"Sesi&oacute;n"	// Okoteve traduccin
#elif L==6
	"Sessione"
#elif L==7
	"Sesja"
#elif L==8
	"Sess&atilde;o"
#endif
	,
	// ActMyCrs
#if   L==0
	"Assignatures"
#elif L==1
	"Kursen"
#elif L==2
	"Courses"
#elif L==3
	"Asignaturas"
#elif L==4
	"Mati&egrave;res"
#elif L==5
	"Mbo'esyry"
#elif L==6
	"Corsi"
#elif L==7
	"Kurs&oacute;w"
#elif L==8
	"Disciplinas"
#endif
	,
	// ActSeeMyTimTbl
#if   L==0
	"Horari"
#elif L==1
	"Stundenplan"
#elif L==2
	"Timetable"
#elif L==3
	"Horario"
#elif L==4
	"Horaire"
#elif L==5
	"Horario"	// Okoteve traduccin
#elif L==6
	"Orario"
#elif L==7
	"Plan"
#elif L==8
	"Hor&aacute;rio"
#endif
	,
	// ActFrmUsrAcc
#if   L==0
	"Compte"
#elif L==1
	"Benutzerkonto"
#elif L==2
	"Account"
#elif L==3
	"Cuenta"
#elif L==4
	"Compte"
#elif L==5
	"Cuenta"	// Okoteve traduccin
#elif L==6
	"Account"
#elif L==7
	"Konto"
#elif L==8
	"Conta"
#endif
	,
	// ActReqEdiRecCom
#if   L==0
	"Fitxa"
#elif L==1
	"Profil"
#elif L==2
	"Record"
#elif L==3
	"Ficha"
#elif L==4
	"Carte"
#elif L==5
	"Ficha"	// Okoteve traduccin
#elif L==6
	"Scheda"
#elif L==7
	"Karta"
#elif L==8
	"Cart&atilde;o"
#endif
	,
	// ActEdiPrf
#if   L==0
	"Prefer&egrave;ncies"
#elif L==1
	"Einstellungen"
#elif L==2
	"Preferences"
#elif L==3
	"Preferencias"
#elif L==4
	"Pr&eacute;f&eacute;rences"
#elif L==5
	"Mbohoryha"
#elif L==6
	"Preferenze"
#elif L==7
	"Preferencje"
#elif L==8
	"Prefer&ecirc;ncias"
#endif
	,
	// ActAdmBrf
#if   L==0
	"Malet&iacute;"
#elif L==1
	"Aktentasche"
#elif L==2
	"Briefcase"
#elif L==3
	"Malet&iacute;n"
#elif L==4
	"Portedocuments"
#elif L==5
	"Malet&iacute;n"	// Okoteve traduccin
#elif L==6
	"Cartella"
#elif L==7
	"Teczka"
#elif L==8
	"Pasta"
#endif
	,
	// ActMFUAct
#if   L==0
	"Accions"
#elif L==1
	"Aktionen"
#elif L==2
	"Actions"
#elif L==3
	"Acciones"
#elif L==4
	"Actions"
#elif L==5
	"Acciones"	// Okoteve traduccin
#elif L==6
	"Azioni"
#elif L==7
	"Akcje"
#elif L==8
	"A&ccedil;&otilde;es"
#endif
	,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	}
};

const char *Txt_MENU_SUBTITLE[Tab_NUM_TABS][Act_MAX_OPTIONS_IN_MENU_PER_TAB] =
{
	{
	// TabUnk ****************************************************
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	},
	{
	// TabSys *****************************************************
	// ActSysReqSch
#if   L==0
	"Cerca d'assignatures, professors i documents"
#elif L==1
	"Suche Kursen, Lehrer und Dokumente"
#elif L==2
	"Search subjects, teachers and documents"
#elif L==3
	"B&uacute;squeda de asignaturas, profesores y documentos"
#elif L==4
	"Rechercher des mati&egrave;res, des enseignants et des documents"
#elif L==5
	"B&uacute;squeda de asignaturas, profesores y documentos"// Okoteve traduccin
#elif L==6
	"Cercare corsi, professori e documenti"
#elif L==7
	"Szukaj przedmiot&oacute;w, nauczycieli i dokumenty"
#elif L==8
	"Procurar disciplinas, professores e documentos"
#endif
	,
	// ActSeeCty
#if   L==0
	"Llista de pa&iuml;sos i del nombre d'usuaris de cada pa&iacute;s"
#elif L==1
	"L&auml;nder"
#elif L==2
	"Countries"
#elif L==3
	"Lista de pa&iacute;ses y del n&uacute;mero de usuarios de cada pais"
#elif L==4
	"Pays"
#elif L==5
	"Lista de pa&iacute;ses y del n&uacute;mero de usuarios de cada pais"// Okoteve traduccin
#elif L==6
	"Paesi"
#elif L==7
	"Kraje"
#elif L==8
	"Pa&iacute;ses"
#endif
	,
	// ActSeePen
#if   L==0
	"Institutions, centres, titulacions i assignatures pendents d'activar"
#elif L==1
	"Hochschulen, Lehrinstitute, Studiengang und anh&auml;ngig Kursen"
#elif L==2
	"Institutions, centres, degrees and pending courses"
#elif L==3
	"Instituciones, centros, titulaciones y asignaturas pendientes de activar"
#elif L==4
	"&Eacute;tablissements, centres, &eacute;tudes et mati&egrave;res en attente"
#elif L==5
	"Instituciones, centros, titulaciones y asignaturas pendientes de activar"	// Okoteve traduccin
#elif L==6
	"Istituzioni, centri, lauree e corsi in sospeso"
#elif L==7
	"Instytucje, centra, stopnie i kurs&otilde;w oczekuj&aogon;cy"
#elif L==8
	"Institu&ccedil;&otilde;es, centros, titula&ccedil;&otilde;es e disciplinas pendentes"
#endif
	,
	// ActReqRemOldCrs
#if   L==0
	"Eliminar assignatures antics"
#elif L==1
	"Fr&uuml;here Kursen entfernen"
#elif L==2
	"Eliminate old courses"
#elif L==3
	"Eliminar asignaturas antiguas"
#elif L==4
	"&Eacute;liminer anciens mati&egrave;res"
#elif L==5
	"Eliminar asignaturas antiguas"	// Okoteve traduccin
#elif L==6
	"Rimuovere vecchi corsi"
#elif L==7
	"Wyeleminuj starych kurs&oacute;w"
#elif L==8
	"Eliminar antigas disciplinas"
#endif
	,
	// ActSeeDegTyp
#if   L==0
	"Tipus de titulaci&oacute;"
#elif L==1
	"Abschlussarten"
#elif L==2
	"Degree types"
#elif L==3
	"Tipos de titulaci&oacute;n"
#elif L==4
	"Types d'&eacute;tude"
#elif L==5
	"Arandur&atilde; l&aacute;ja"
#elif L==6
	"Tipi di laurea"
#elif L==7
	"Typy stopnia"
#elif L==8
	"Tipos de titula&ccedil;&atilde;o"
#endif
	,
	// ActSeeMai
#if   L==0
	"Llista de dominis de correu permesos"
	" per a l'enviament de notificacions"
#elif L==1
	"Mail-Dom&auml;nen f&uuml;r Benachrichtigungen"
#elif L==2
	"Mail domains for notifications"
#elif L==3
	"Lista de dominios de correo permitidos"
	" para el env&iacute;o de notificaciones"
#elif L==4
	"Domaines de messagerie pour notifications"
#elif L==5
	"Lista de dominios de correo permitidos"
	" para el env&iacute;o de notificaciones"// Okoteve traduccin
#elif L==6
	"Campi mail per invio di notifiche"
#elif L==7
	"Domen Mail powiadomien"
#elif L==8
	"Dom&iacute;nios de email"
	  " para notifica&ccedil;&otilde;es"
#endif
	,
	// ActSeeBan
#if   L==0
	"Llista de banners publicitaris"
#elif L==1
	"Werbebanners Liste"
#elif L==2
	"Banners list"
#elif L==3
	"Lista de banners publicitarios"
#elif L==4
	"Liste des banni&eacute;res publicitaires"
#elif L==5
	"Lista de banners publicitarios"// Okoteve traduccin
#elif L==6
	"Lista di banners"
#elif L==7
	"Lista banery"
#elif L==8
	"Lista de banners publicit&aacute;rios"
#endif
	,
	// ActSeeLnk
#if   L==0
	"Llista d'enlla&ccedil;os institucionals"
#elif L==1
	"Offizielle Links Liste"
#elif L==2
	"Institutional links list"
#elif L==3
	"Lista de enlaces institucionales"
#elif L==4
	"Liens institutionnels"
#elif L==5
	"Lista de enlaces institucionales"// Okoteve traduccin
#elif L==6
	"Link istituzionali"
#elif L==7
	"Instytucjonalne linki"
#elif L==8
	"Liga&ccedil;&otilde;es institucionais"
#endif
	,
	// ActLstPlg
#if   L==0
	"Llista de complements (versi&oacute; beta)"
#elif L==1
	"Plugin-Liste"
#elif L==2
	"List of plugins"
#elif L==3
	"Lista de complementos (versi&oacute;n beta)"
#elif L==4
	"Liste de plugins"
#elif L==5
	"Lista de complementos (versi&oacute;n beta)"// Okoteve traduccin
#elif L==6
	"Lista di plugin"
#elif L==7
	"Lista plugin&oacute;w"
#elif L==8
	"Lista de plugins"
#endif
	,
	// ActSetUp
#if   L==0
	"Instal laci&oacute; de la plataforma (en proves)"
#elif L==1
	"Einrichten"
#elif L==2
	"Set up"
#elif L==3
	"Instalaci&oacute;n de la plataforma (en pruebas)"
#elif L==4
	"Installer"
#elif L==5
	"Instalaci&oacute;n de la plataforma (en pruebas)"// Okoteve traduccin
#elif L==6
	"Setup"
#elif L==7
	"Set up"
#elif L==8
	"Instalar"
#endif
	,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	},
	{
	// TabCty *****************************************************
	// ActCtyReqSch
#if   L==0
	"Cerca d'assignatures, professors i documents"
#elif L==1
	"Suche Kursen, Lehrer und Dokumente"
#elif L==2
	"Search subjects, teachers and documents"
#elif L==3
	"B&uacute;squeda de asignaturas, profesores y documentos"
#elif L==4
	"Rechercher des mati&egrave;res, des enseignants et des documents"
#elif L==5
	"B&uacute;squeda de asignaturas, profesores y documentos"// Okoteve traduccin
#elif L==6
	"Cercare corsi, professori e documenti"
#elif L==7
	"Szukaj przedmiot&oacute;w, nauczycieli i dokumenty"
#elif L==8
	"Procurar disciplinas, professores e documentos"
#endif
	,
	// ActSeeCtyInf
#if   L==0
	"Informaci&oacute; d'aquest pa&iacute;s"
#elif L==1
	"Information dieses Land"
#elif L==2
	"Information on this country"
#elif L==3
	"Informaci&oacute;n sobre este pa&iacute;s"
#elif L==4
	"Information de cet pays"
#elif L==5
	"Informaci&oacute;n sobre este pa&iacute;s"// Okoteve traduccin
#elif L==6
	"Informazione di questo paese"
#elif L==7
	"Informacja na ten kraj"
#elif L==8
	"Informa&ccedil;&atilde;o deste pa&iacute;s"
#endif
	,
	// ActSeeIns
#if   L==0
	"Llista d'institucions"
#elif L==1
	"Hochschulen"
#elif L==2
	"List of institutions of the current country"
#elif L==3
	"Lista de instituciones (universidades, institutos...) de este pa&iacute;s"
#elif L==4
	"&Eacute;tablissement"
#elif L==5
	"Lista de instituciones (universidades, institutos...) de este pa&iacute;s"// Okoteve traduccin
#elif L==6
	"Istituzioni"
#elif L==7
	"Instytucje"
#elif L==8
	"Institu&ccedil;&otilde;es"
#endif
	,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	},
	{
	// TabIns *****************************************************
	// ActInsReqSch
#if   L==0
	"Cerca d'assignatures, professors i documents"
#elif L==1
	"Suche Kursen, Lehrer und Dokumente"
#elif L==2
	"Search subjects, teachers and documents"
#elif L==3
	"B&uacute;squeda de asignaturas, profesores y documentos"
#elif L==4
	"Rechercher des mati&egrave;res, des enseignants et des documents"
#elif L==5
	"B&uacute;squeda de asignaturas, profesores y documentos"// Okoteve traduccin
#elif L==6
	"Cercare corsi, professori e documenti"
#elif L==7
	"Szukaj przedmiot&oacute;w, nauczycieli i dokumenty"
#elif L==8
	"Procurar disciplinas, professores e documentos"
#endif
	,
	// ActSeeInsInf
#if   L==0
	"Informaci&oacute; d'aquesta instituci&oacute;"
#elif L==1
	"Information dieses Hochschule"
#elif L==2
	"Information on this institution"
#elif L==3
	"Informaci&oacute;n sobre esta instituci&oacute;n"
#elif L==4
	"Information de cet &eacute;tablissement"
#elif L==5
	"Informaci&oacute;n sobre esta instituci&oacute;n"// Okoteve traduccin
#elif L==6
	"Informazione di questa istituzione"
#elif L==7
	"Informacja na ten instytucja"
#elif L==8
	"Informa&ccedil;&atilde;o desta institu&ccedil;&atilde;o"
#endif
	,
	// ActSeeCtr
#if   L==0
	"Llista de centres"
#elif L==1
	"Lehrinstitute"
#elif L==2
	"List of centres of the current institution"
#elif L==3
	"Lista de centros (facultades, escuelas...) de esta instituci&oacute;n"
#elif L==4
	"Centres"
#elif L==5
	"Lista de centros (facultades, escuelas...) de esta instituci&oacute;n"	// Okoteve traduccin
#elif L==6
	"Centri"
#elif L==7
	"Centra"
#elif L==8
	"Centros"
#endif
	,
	// ActSeeDpt
#if   L==0
	"Llista de tots els departaments d'aquesta instituci&oacute;"
#elif L==1
	"Abteilungen"
#elif L==2
	"List all departments of this institution"
#elif L==3
	"Lista de todos los departamentos de esta instituci&oacute;n"
#elif L==4
	"D&eacute;partements"
#elif L==5
	"Lista de todos los departamentos de esta instituci&oacute;n"// Okoteve traduccin
#elif L==6
	"Dipartimenti"
#elif L==7
	"Wydzia&lstrok;y"
#elif L==8
	"Departamentos"
#endif
	,
	// ActSeePlc
#if   L==0
	"Llista d'empla&ccedil;aments d'aquesta instituci&oacute;"
#elif L==1
	"Standorte"
#elif L==2
	"List of places of this institution"
#elif L==3
	"Lista de emplazamientos de esta instituci&oacute;n"
#elif L==4
	"Emplacements"
#elif L==5
	"Lista de emplazamientos de esta instituci&oacute;n"// Okoteve traduccin
#elif L==6
	"Localit&agrave;"
#elif L==7
	"Miejsca"
#elif L==8
	"Localiza&ccedil;oes"
#endif
	,
	// ActSeeHld
#if   L==0
	"Dies festius i per&iacute;odes no lectius d'aquesta instituci&oacute;"
#elif L==1
	"Urlaub und vorlesungsfreie Zeit"
#elif L==2
	"Holidays and non school days of this institution"
#elif L==3
	"D&iacute;as festivos y per&iacute;odos no lectivos de esta instituci&oacute;n"
#elif L==4
	"Jours f&eacute;ri&eacute;s et sans &eacute;cole"
#elif L==5
	"D&iacute;as festivos y per&iacute;odos no lectivos de esta instituci&oacute;n"	// Okoteve traduccin
#elif L==6
	"Festivit&agrave; e giorni senza lezione"
#elif L==7
	"Noclegi i innych dni zajec szkolnych"
#elif L==8
	"Feriados e dias sem escola"
#endif
	,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	},
	{
	// TabCtr *****************************************************
	// ActCtrReqSch
#if   L==0
	"Cerca d'assignatures, professors i documents"
#elif L==1
	"Suche Kursen, Lehrer und Dokumente"
#elif L==2
	"Search subjects, teachers and documents"
#elif L==3
	"B&uacute;squeda de asignaturas, profesores y documentos"
#elif L==4
	"Rechercher des mati&egrave;res, des enseignants et des documents"
#elif L==5
	"B&uacute;squeda de asignaturas, profesores y documentos"// Okoteve traduccin
#elif L==6
	"Cercare corsi, professori e documenti"
#elif L==7
	"Szukaj przedmiot&oacute;w, nauczycieli i dokumenty"
#elif L==8
	"Procurar disciplinas, professores e documentos"
#endif
	,
	// ActSeeCtrInf
#if   L==0
	"Informaci&oacute; d'aquest centre"
#elif L==1
	"Information dieses Lehrinstitut"
#elif L==2
	"Information on this centre"
#elif L==3
	"Informaci&oacute;n sobre este centro"
#elif L==4
	"Information de cet centre"
#elif L==5
	"Informaci&oacute;n sobre este centro"// Okoteve traduccin
#elif L==6
	"Informazione di questo centro"
#elif L==7
	"Informacja na ten centrum"
#elif L==8
	"Informa&ccedil;&atilde;o deste centro"
#endif
	,
	// ActSeeDeg
#if   L==0
	"Titulacions d'aquest centre"
#elif L==1
	"Studiengang"
#elif L==2
	"List of degrees of the current centre"
#elif L==3
	"Lista de titulaciones (carreras, m&aacute;steres...) de este centro"
#elif L==4
	"&Eacute;tudes"
#elif L==5
	"Lista de titulaciones (carreras, m&aacute;steres...) de este centro"// Okoteve traduccin
#elif L==6
	"Lauree"
#elif L==7
	"Stopnie"
#elif L==8
	"Titula&ccedil;&atilde;os"
#endif
	,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	},
	{
	// TabDeg *****************************************************
	// ActDegReqSch
#if   L==0
	"Cerca d'assignatures, professors i documents"
#elif L==1
	"Suche Kursen, Lehrer und Dokumente"
#elif L==2
	"Search subjects, teachers and documents"
#elif L==3
	"B&uacute;squeda de asignaturas, profesores y documentos"
#elif L==4
	"Rechercher des mati&egrave;res, des enseignants et des documents"
#elif L==5
	"B&uacute;squeda de asignaturas, profesores y documentos"// Okoteve traduccin
#elif L==6
	"Cercare corsi, professori e documenti"
#elif L==7
	"Szukaj przedmiot&oacute;w, nauczycieli i dokumenty"
#elif L==8
	"Procurar disciplinas, professores e documentos"
#endif
	,
	// ActSeeDegInf
#if   L==0
	"Informaci&oacute; d'aquesta titulaci&oacute;"
#elif L==1
	"Information dieses Studiengang"
#elif L==2
	"Information on this degree"
#elif L==3
	"Informaci&oacute;n sobre esta titulaci&oacute;n"
#elif L==4
	"Information de cet &eacute;tude"
#elif L==5
	"Informaci&oacute;n sobre esta titulaci&oacute;n"// Okoteve traduccin
#elif L==6
	"Informazione di questa laurea"
#elif L==7
	"Informacja na ten stopie&nacute;"
#elif L==8
	"Informa&ccedil;&atilde;o desta titula&ccedil;&atilde;o"
#endif
	,
	// ActSeeCrs
#if   L==0
	"Assignatures d'aquesta titulaci&oacute;"
#elif L==1
	"Kursen in diesem Studiengang"
#elif L==2
	"List of ourses of the current degree"
#elif L==3
	"Lista de asignaturas de esta titulaci&oacute;n"
#elif L==4
	"Mati&egrave;res dans cet &eacute;tude"
#elif L==5
	"Lista de asignaturas de esta titulaci&oacute;n"// Okoteve traduccin
#elif L==6
	"Corsi di questa laurea"
#elif L==7
	"Kurs&oacute;w w tym stopie&nacute;"
#elif L==8
	"Disciplinas nesta titula&ccedil;&atilde;o"
#endif
	,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	},
	{
	// TabCrs *****************************************************
	// ActCrsReqSch
#if   L==0
	"Cerca de professors i documents"
#elif L==1
	"Suche Lehrer und Dokumente"
#elif L==2
	"Search teachers and documents"
#elif L==3
	"B&uacute;squeda de profesores y documentos"
#elif L==4
	"Rechercher des enseignants et des documents"
#elif L==5
	"B&uacute;squeda de profesores y documentos"// Okoteve traduccin
#elif L==6
	"Cercare professori e documenti"
#elif L==7
	"Szukaj nauczycieli i dokumenty"
#elif L==8
	"Procurar professores e documentos"
#endif
	,
	// ActSeeCrsInf
#if   L==0
	"Informaci&oacute; d'aquesta assignatura"
#elif L==1
	"Information dieses Kurs"
#elif L==2
	"Information on this course"
#elif L==3
	"Informaci&oacute;n sobre esta asignatura"
#elif L==4
	"Information de cette mati&egrave;re"
#elif L==5
	"Informaci&oacute;n sobre esta asignatura"// Okoteve traduccin
#elif L==6
	"Informazione di questo corso"
#elif L==7
	"Informacja na ten kurs"
#elif L==8
	"Informa&ccedil;&atilde;o desta disciplina"
#endif
	,
	// ActSeeTchGui
#if   L==0
	"Guia docent de l'assignatura"
#elif L==1
	"Hinweise zur Kurs"
#elif L==2
	"Teaching guide"
#elif L==3
	"Gu&iacute;a docente de la asignatura"
#elif L==4
	"Guide p&eacute;dagogique de la mati&egrave;re"
#elif L==5
	"Gu&iacute;a docente de la asignatura"// Okoteve traduccin
#elif L==6
	"Guida didattica del corso"
#elif L==7
	"Nauczanie przypomnienie"
#elif L==8
	"Guia de ensino"
#endif
	,
	// ActSeeSyl
#if   L==0
	"Programa de l'assignatura"
#elif L==1
	"Studienplan"
#elif L==2
	"Syllabus"
#elif L==3
	"Programa de la asignatura"
#elif L==4
	"Plan"
#elif L==5
	"Programa de la asignatura"// Okoteve traduccin
#elif L==6
	"Programma"
#elif L==7
	"Syllabus"
#elif L==8
	"Programa"
#endif
	,
	// ActSeeDoc
#if   L==0
	"Documents de l'assignatura o d'un grup enviats per professors"
#elif L==1
	"Dokumente"
#elif L==2
	"Documents of the course or a group uploaded by teachers"
#elif L==3
	"Documentos de la asignatura o de un grupo subidos por profesores"
#elif L==4
	"Documents"
#elif L==5
	"Documentos de la asignatura o de un grupo subidos por profesores"// Okoteve traduccin
#elif L==6
	"Area documenti"
#elif L==7
	"Obszar Dokument&oacute;w"
#elif L==8
	"Documentos"
#endif
	,
	// ActAdmCom
#if   L==0
	"Arxius compartits per professors i estudiants"
	" de l'assignatura o d'un grup"
#elif L==1
	"Freigegebene Dateien"
#elif L==2
	"Files shared by teachers and students"
	" of the course or group"
#elif L==3
	"Archivos compartidos por profesores y estudiantes"
	" de la asignatura o de un grupo"
#elif L==4
	"Fichiers partag&eacute;s"
#elif L==5
	"Archivos compartidos por profesores y estudiantes"
	" de la asignatura o de un grupo"// Okoteve traduccin
#elif L==6
	"File condivisi"
#elif L==7
	"Pliki udost&eogon;pnione"
#elif L==8
	"Arquivos compartilhados"
#endif
	,
	// ActSeeCrsTimTbl
#if   L==0
	"Horari de classes de l'assignatura"
#elif L==1
	"Stundenplan"
#elif L==2
	"Class timetable"
#elif L==3
	"Horario de clases de la asignatura"
#elif L==4
	"Horaire"
#elif L==5
	"Horario de clases de la asignatura"	// Okoteve traduccin
#elif L==6
	"Orario del corso"
#elif L==7
	"Harmonogram klasy"
#elif L==8
	"Hor&aacute;rio"
#endif
	,
	// ActSeeBib
#if   L==0
	"Bibliografia relacionada amb els continguts de l'assignatura"
#elif L==1
	"Literatur"
#elif L==2
	"Bibliography"
#elif L==3
	"Bibliograf&iacute;a relacionada con los contenidos de la asignatura"
#elif L==4
	"Bibliographie"
#elif L==5
	"Bibliograf&iacute;a relacionada con los contenidos de la asignatura"// Okoteve traduccin
#elif L==6
	"Bibliografia"
#elif L==7
	"Bibliografia"	// Potrzebujesz tlumaczenie
#elif L==8
	"Bibliografia"
#endif
	,
	// ActSeeFAQ
#if   L==0
	"FAQ (preguntes freq&uuml;entment formulades i les seves respostes)"
#elif L==1
	"FAQ"
#elif L==2
	"FAQ (Frequently Asked Questions)"
#elif L==3
	"FAQ (preguntas frecuentemente formuladas y sus respuestas)"
#elif L==4
	"FAQ"
#elif L==5
	"FAQ (preguntas frecuentemente formuladas y sus respuestas)"// Okoteve traduccin
#elif L==6
	"FAQ"
#elif L==7
	"FAQ (Frequently Asked Questions)"
#elif L==8
	"FAQ"
#endif
	,
	// ActSeeCrsLnk
#if   L==0
	"Enlla&ccedil;os relacionats amb els continguts de l'assignatura"
#elif L==1
	"Links"
#elif L==2
	"Links"
#elif L==3
	"Enlaces relacionados con los contenidos de la asignatura"
#elif L==4
	"Liens"
#elif L==5
	"Enlaces relacionados con los contenidos de la asignatura"// Okoteve traduccin
#elif L==6
	"Link"
#elif L==7
	"Linki"
#elif L==8
	"Liga&ccedil;&otilde;es"
#endif
	,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	},
	{
	// TabAss *****************************************************
	// ActSeeAss
#if   L==0
	"Sistema o criteris d'avaluaci&oacute;"
	" de l'estudiant en aquesta assignatura"
#elif L==1
	"Bewertungssystem"
#elif L==2
	"Assessment system"
#elif L==3
	"Sistema o criterios de evaluaci&oacute;n"
	" del estudiante en esta asignatura"
#elif L==4
	"Syst&egrave;me d'&eacute;valuation"
#elif L==5
	"Sistema o criterios de evaluaci&oacute;n"
	" del estudiante en esta asignatura"// Okoteve traduccin
#elif L==6
	"Sistema di valutazione"
#elif L==7
	"System oceny"
#elif L==8
	"Sistema de avalia&ccedil;&atilde;o"
#endif
	,
	// ActSeeAsg
#if   L==0
	"Activitats obligat&ograve;ries o recomanades en aquesta assignatura"
#elif L==1
	"Aufgaben"
#elif L==2
	"Assignments"
#elif L==3
	"Actividades obligatorias o recomendadas en esta asignatura"
#elif L==4
	"Activit&eacute;s"
#elif L==5
	"Actividades obligatorias o recomendadas en esta asignatura"// Okoteve traduccin
#elif L==6
	"Attivit&agrave;"
#elif L==7
	"Zadania"
#elif L==8
	"Atividades"
#endif
	,
	// ActAdmAsgWrkUsr
#if   L==0
	"Els meus documents d'activitats i altres treballs en aquesta assignatura"
#elif L==1
	"Eingereichte Hausarbeiten"
#elif L==2
	"My homework"
#elif L==3
	"Mis documentos de actividades y otros trabajos en esta asignatura"
#elif L==4
	"Mes travaux"
#elif L==5
	"Mis documentos de actividades y otros trabajos en esta asignatura"// Okoteve traduccin
#elif L==6
	"Miei lavori"
#elif L==7
	"Moje zadanie domowe"
#elif L==8
	"Os meus trabalhos"
#endif
	,
	// ActReqAsgWrkCrs
#if   L==0
	"Activitats i altres treballs enviats pels usuaris d'aquesta assignatura"
#elif L==1
	"Hausarbeiten der Benutzer"
#elif L==2
	"Users' homework"
#elif L==3
	"Actividades y otros trabajos enviados por los usuarios de esta asignatura"
#elif L==4
	"Travails d'utilisateurs"
#elif L==5
	"Actividades y otros trabajos enviados por los usuarios de esta asignatura"// Okoteve traduccin
#elif L==6
	"Lavori di utenti"
#elif L==7
	"Uzytkownik&oacute;w domowych"
#elif L==8
	"Trabalhos de utilizadores"
#endif
	,
	// ActReqTst
#if   L==0
	"Tests sobre els continguts d'aquesta assignatura"
#elif L==1
	"Tests"
#elif L==2
	"Tests on the contents of this course"
#elif L==3
	"Tests sobre los contenidos de esta asignatura"
#elif L==4
	"Tests sur le contenu de cette mati&egrave;re"
#elif L==5
	"Tests sobre los contenidos de esta asignatura"// Okoteve traduccin
#elif L==6
	"Test sui contenuti del corso"
#elif L==7
	"Test&oacute;w"
#elif L==8
	"Testes"
#endif
	,
	// ActSeeCal
#if   L==0
	"Calendari acad&egrave;mic (els dies festius"
	" corresponen al lloc del centre seleccionada)"
#elif L==1
	"Kalender"
#elif L==2
	"Academic calendar"
#elif L==3
	"Calendario acad&eacute;mico (los d&iacute;as festivos"
	" corresponden al lugar del centro seleccionado)"
#elif L==4
	"Calendrier"
#elif L==5
	"Calendario acad&eacute;mico (los d&iacute;as festivos"
	" corresponden al lugar del centro seleccionado)"// Okoteve traduccin
#elif L==6
	"Calendario del corso"
#elif L==7
	"Kalendarz akademicki"
#elif L==8
	"Calend&aacute;rio"
#endif
	,
	// ActSeeExaAnn
#if   L==0
	"Llistat de convocat&ograve;ries d'exmens d'aquesta assignatura"
#elif L==1
	"Pr&uuml;fungsaufruf"
#elif L==2
	"Announcements of exams"
#elif L==3
	"Listado de convocatorias de ex&aacute;menes de esta asignatura"
#elif L==4
	"Convocations &agrave; examens"
#elif L==5
	"Listado de convocatorias de ex&aacute;menes de esta asignatura"// Okoteve traduccin
#elif L==6
	"Appelli d'esame"
#elif L==7
	"Ogloszenia egzamin&oacute;w"
#elif L==8
	"Chamadas para exames"
#endif
	,
	// ActSeeMrk
#if   L==0
	"Consulta individualitzada de qualificacions"
	" (cada estudiant nom&eacute;s veu les seves qualificacions,"
	" el professor veu la qualificaci&oacute; d'un estudiant seleccionat a l'atzar)"
#elif L==1
	"Bewertungen"
#elif L==2
	"Marks"
#elif L==3
	"Consulta individualizada de calificaciones"
	" (cada estudiante s&oacute;lo ve sus calificaciones,"
	" el profesor ve la calificaci&oacute;n de un estudiante seleccionado al azar)"
#elif L==4
	"Notes"
#elif L==5
	"Consulta individualizada de calificaciones"
	" (cada estudiante s&oacute;lo ve sus calificaciones,"
	" el profesor ve la calificaci&oacute;n de un estudiante seleccionado al azar)"	// Okoteve traduccin
#elif L==6
	"Risultati"
#elif L==7
	"Marks"
#elif L==8
	"Notas"
#endif
	,
	// ActSeeRecCrs
#if   L==0
	"La meva fitxa personalitzada per a aquesta assignatura"
#elif L==1
	"Profil bei Kurs"
#elif L==2
	"My record for this course"
#elif L==3
	"Mi ficha personalizada para esta asignatura"
#elif L==4
	"Ma carte dans mati&egrave;re"
#elif L==5
	"Mi ficha personalizada para esta asignatura"// Okoteve traduccin
#elif L==6
	"La mia scheda per questo corso"
#elif L==7
	"M&oacute;j rekord na ten kurs"
#elif L==8
	"Meu cart&atilde;o em disciplina"
#endif
	,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	},
	{
	// TabUsr *****************************************************
	// ActReqSelGrp
#if   L==0
	"Grups en aquesta assignatura"
#elif L==1
	"Gruppen"
#elif L==2
	"Groups"
#elif L==3
	"Grupos en esta asignatura"
#elif L==4
	"Groupes"
#elif L==5
	"Grupos en esta asignatura"	// Okoteve traduccin
#elif L==6
	"Gruppi"
#elif L==7
	"Grupy"
#elif L==8
	"Grupos"
#endif
	,
	// ActLstInv
#if   L==0
	"Llistat i fitxes dels convidats"
#elif L==1
	"Eingeladenverzeichnis und profile"
#elif L==2
	"Guests' list and records"
#elif L==3
	"Listado y fichas de los invitados"
#elif L==4
	"Liste et cartes d'invit&eacute;s"
#elif L==5
	"Listado y fichas de los invitados"// Okoteve traduccin
#elif L==6
	"Elenco e schede ospiti"
#elif L==7
	"Listy i rekordy zaproszenie"
#elif L==8
	"Lista e cart&otilde;es convidados"
#endif
	,
	// ActLstStd
#if   L==0
	"Llistat i fitxes dels estudiants"
#elif L==1
	"Studentenverzeichnis und profile"
#elif L==2
	"Students' list and records"
#elif L==3
	"Listado y fichas de los estudiantes"
#elif L==4
	"Liste et cartes d'&eacute;tudiants"
#elif L==5
	"Listado y fichas de los estudiantes"// Okoteve traduccin
#elif L==6
	"Elenco e schede studenti"
#elif L==7
	"Listy i rekordy student&oacute;w"
#elif L==8
	"Lista e cart&otilde;es estudantes"
#endif
	,
	// ActSeeAtt
#if   L==0
	"Control d'assist&egrave;ncia dels estudiants"
	" a les classes d'aquesta assignatura"
#elif L==1
	"Kontrolle der Studierenden Anwesenheit"
#elif L==2
	"Control of students' attendance"
	" to classes in this course"
#elif L==3
	"Control de asistencia de los estudiantes"
	" a las clases de esta asignatura"
#elif L==4
	"Contr&ocirc;le de pr&eacute;sence des &eacute;tudiants"
#elif L==5
	"Control de asistencia de los estudiantes"
	" a las clases de esta asignatura"// Okoteve traduccin
#elif L==6
	"Controllo di frequenza degli studenti"
#elif L==7
	"Kontrola frekwencji"
#elif L==8
	"Lista de presen&ccedil;a"
#endif
	,
	// ActLstTch
#if   L==0
	"Llistat i fitxes de professors"
#elif L==1
	"Lehrkr&auml;fteverzeichnis und profile"
#elif L==2
	"Teachers' list and records"
#elif L==3
	"Listado y fichas de profesores"
#elif L==4
	"Liste et cartes d'enseignants"
#elif L==5
	"Listado y fichas de profesores"// Okoteve traduccin
#elif L==6
	"Elenco e schede professori"
#elif L==7
	"Lista i rekordy nauczycieli"
#elif L==8
	"Lista e cart&otilde;es professores"
#endif
	,
	// ActLstAdm
#if   L==0
	"Llista d'administradors de titulacions"
	" a les quals podeu consultar"
	" per crear assignatures o altres gestions"
#elif L==1
	"Liste der Verwalter"
#elif L==2
	"Administrators"
#elif L==3
	"Lista de administradores de titulaciones"
	" a los que puede consultar"
	" para crear asignaturas u otras gestiones"
#elif L==4
	"Liste administrateurs"
#elif L==5
	"Lista de administradores de titulaciones"
	" a los que puede consultar"
	" para crear asignaturas u otras gestiones"// Okoteve traduccin
#elif L==6
	"Lista degli amministratori"
#elif L==7
	"Administratorzy"
#elif L==8
	"Lista administradores"
#endif
	,
	// ActReqSignUp
#if   L==0
	"Sollicitar inscripci&oacute;"
#elif L==1
	"Registrierung beantragen"
#elif L==2
	"Sign up in this course"
#elif L==3
	"Solicitar mi inscripci&oacute;n en esta asignatura"
#elif L==4
	"Demander inscription"
#elif L==5
	"Solicitar mi inscripci&oacute;n en esta asignatura"// Okoteve traduccin
#elif L==6
	"Richiedere iscrizione"
#elif L==7
	"Wniosku rejestracj&eogon;"
#elif L==8
	"Solicitar inscri&ccedil;&atilde;o"
#endif
	,
	// ActSeeSignUpReq
#if   L==0
	"Peticions d'inscripci&oacute;"
#elif L==1
	"Registrierungsanforderungen"
#elif L==2
	"Requests for enrollment in this course pending verification"
#elif L==3
	"Peticiones de inscripci&oacute;n pendientes de verificar"
#elif L==4
	"Demandes d'inscription"
#elif L==5
	"Peticiones de inscripci&oacute;n pendientes de verificar"// Okoteve traduccin
#elif L==6
	"Richieste di iscrizione"
#elif L==7
	"&Zdot;&aogon;dania rejestrowania"
#elif L==8
	"Solicita&ccedil;&otilde;es de inscri&ccedil;&atilde;o"
#endif
	,
	// ActReqMdfSevUsr
#if   L==0
	"Inscripci&oacute; / eliminaci&oacute;n de usuaris"
#elif L==1
	"Anmeldung / Entfernen von Benutzern"
#elif L==2
	"Enrollment / removing of users"
#elif L==3
	"Inscripci&oacute;n / eliminaci&oacute;n de usuarios"
#elif L==4
	"Inscription / suppression de utilisateurs"
#elif L==5
	"Inscripci&oacute;n / eliminaci&oacute;n de usuario"// Okoteve traduccin
#elif L==6
	"Iscrizione / rimozione di utenti"
#elif L==7
	"Rekrutacja / usuni&eogon;cie u&zdot;ytkownik&oacute;w"
#elif L==8
	"Inscri&ccedil;&atilde;o / remo&ccedil;&atilde;o de utilizadores"
#endif
	,
	// ActLstCon
#if   L==0
	"Llista d'usuaris connectats en aquest moment"
#elif L==1
	"Angemeldet"
#elif L==2
	"Connected"
#elif L==3
	"Lista de usuarios conectados en este momento"
#elif L==4
	"Connect&eacute;s"
#elif L==5
	"Lista de usuarios conectados en este momento"// Okoteve traduccin
#elif L==6
	"Connessi"
#elif L==7
	"Online"
#elif L==8
	"Conectados"
#endif
	,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	},
	{
	// TabMsg *****************************************************
	// ActSeeNtf
#if   L==0
	"Esdeveniments recents que han estat o seran notificats"
#elif L==1
	"Die j&uuml;ngsten Ereignisse wurden oder werden benachrichtigt"
#elif L==2
	"Recent events that have been or will be notified"
#elif L==3
	"Eventos recientes que han sido o ser&aacute;n notificados"
#elif L==4
	"Les &eacute;v&eacute;nements r&eacute;cents qui ont &eacute;t&eacute; ou seront avis&eacute;s"
#elif L==5
	"Eventos recientes que han sido o ser&aacute;n notificados"// Okoteve traduccin
#elif L==6
	"Recenti eventi che sono stati o saranno notificati"
#elif L==7
	"Ostatnie wydarzenia, kt&oacute;re zostaly lub zostana powiadomieni"
#elif L==8
	"Os recentes acontecimentos que foram ou ser&atilde;o notificados"
#endif
	,
	// ActSeeAnn
#if   L==0
	"Anuncis globals mostrat en iniciar sessi&oacute;"
#elif L==1
	"Globale Bekanntmachungen bei der Anmeldung sichtbar"
#elif L==2
	"Global announcements visible at login"
#elif L==3
	"Anuncios globales visibles al iniciar sesi&oacute;n"
#elif L==4
	"Annonces globales visibles lors de la connexion"
#elif L==5
	"Anuncios globales visibles al iniciar sesi&oacute;n"	// Okoteve traduccin
#elif L==6
	"Annunci globali visibili al login"
#elif L==7
	"Globalne og&lstrok;oszenia widoczne przy logowaniu"
#elif L==8
	"An&uacute;ncios globais vis&iacute;veis no in&iacute;cio da sess&atilde;o"
#endif
	,
	// ActSeeNot
#if   L==0
	"Breus avisos relacionats amb aquesta assignatura,"
	" visibles per qualsevol visitant, encara que no estigui identificat"
#elif L==1
	"Ank&uuml;ndigungen"
#elif L==2
	"Notices"
#elif L==3
	"Breves avisos relacionados con esta asignatura,"
	" visibles por cualquier visitante, aunque no est&eacute; identificado"
#elif L==4
	"Avis"
#elif L==5
	"Breves avisos relacionados con esta asignatura,"
	" visibles por cualquier visitante, aunque no est&eacute; identificado"	// Okoteve traduccin
#elif L==6
	"Avvisi"
#elif L==7
	"Ogloszenia"
#elif L==8
	"Avisos"
#endif
	,
	// ActSeeFor
#if   L==0
	"F&ograve;rums de discussi&oacute;, distribu&iuml;ts"
	" segons la tem&agrave;tica (escolliu el f&ograve;rum adequat)"
#elif L==1
	"Foren"
#elif L==2
	"Forums"
#elif L==3
	"Foros de discusi&oacute;n, distribuidos"
	" seg&uacute;n la tem&aacute;tica (elija el foro adecuado)"
#elif L==4
	"Forums"
#elif L==5
	"Foros de discusi&oacute;n, distribuidos"
	" seg&uacute;n la tem&aacute;tica (elija el foro adecuado)"// Okoteve traduccin
#elif L==6
	"Forum"
#elif L==7
	"Forum"
#elif L==8
	"F&oacute;runs"
#endif
	,
	// ActSeeChtRms
#if   L==0
	"Conversa en temps real amb altres usuaris"
#elif L==1
	"Chat"
#elif L==2
	"Chat"
#elif L==3
	"Conversaci&oacute;n en tiempo real con otros usuarios"
#elif L==4
	"Chat"
#elif L==5
	"Conversaci&oacute;n en tiempo real con otros usuarios"	// Okoteve traduccin
#elif L==6
	"Chat"
#elif L==7
	"Chat"
#elif L==8
	"Chat"
#endif
	,
	// ActReqMsgUsr
#if   L==0
	"Enviament d'un nou missatge privat a altres usuaris"
#elif L==1
	"Neue Nachricht"
#elif L==2
	"New message"
#elif L==3
	"Env&iacute;o de un nuevo mensaje privado a otros usuarios"
#elif L==4
	"Nouveau message"
#elif L==5
	"Env&iacute;o de un nuevo mensaje privado a otros usuarios"// Okoteve traduccin
#elif L==6
	"Nuovo messaggio"
#elif L==7
	"Nowe wiadomosci"
#elif L==8
	"Nova mensagem"
#endif
	,
	// ActSeeRcvMsg
#if   L==0
	"Missatges privats rebuts d'altres usuaris"
#elif L==1
	"Empfangene Nachrichten"
#elif L==2
	"Messages received"
#elif L==3
	"Mensajes privados recibidos de otros usuarios"
#elif L==4
	"Messages re&ccedil;us"
#elif L==5
	"Mensajes privados recibidos de otros usuarios"	// Okoteve traduccin
#elif L==6
	"Messaggi ricevuti"
#elif L==7
	"Wiadomo&sacute;ci odebrane"
#elif L==8
	"Mensagens recebidas"
#endif
	,
	// ActSeeSntMsg
#if   L==0
	"Missatges privats enviats per mi a altres usuaris"
#elif L==1
	"Gesendete Nachrichten"
#elif L==2
	"Messages sent"
#elif L==3
	"Mensajes privados enviados por m&iacute; a otros usuarios"
#elif L==4
	"Messages envoy&eacute;s"
#elif L==5
	"Mensajes privados enviados por m&iacute; a otros usuarios"// Okoteve traduccin
#elif L==6
	"Messaggi inviati"
#elif L==7
	"Wiadomo&sacute;ci odebrane"
#elif L==8
	"Mensagens enviadas"
#endif
	,
	// ActMaiStd
#if   L==0
	"Creaci&oacute; d'un missatge de correu electr&ograve;nic"
	" per enviar-lo a estudiants d'aquesta assignatura"
	" utilitzant el programa client de correu predeterminat en el meu sistema"
#elif L==1
	"E-Mail"
#elif L==2
	"E-mail"
#elif L==3
	"Creaci&oacute;n de un mensaje de correo electr&oacute;nico"
	" para enviarlo a estudiantes de esta asignatura"
	" usando el programa cliente de correo predeterminado en mi sistema"
#elif L==4
	"Courrier &eacute;lectronique"
#elif L==5
	"Creaci&oacute;n de un mensaje de correo electr&oacute;nico"
	" para enviarlo a estudiantes de esta asignatura"
	" usando el programa cliente de correo predeterminado en mi sistema"// Okoteve traduccin
#elif L==6
	"E-mail"
#elif L==7
	"E-mail"
#elif L==8
	"E-mail"
#endif
	,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	},
	{
	// TabSta *****************************************************
	// ActSeeAllSvy
#if   L==0
	"Enquestes an&ograve;nimes"
#elif L==1
	"Umfragen"
#elif L==2
	"Surveys"
#elif L==3
	"Encuestas an&oacute;nimas"
#elif L==4
	"Sondages"
#elif L==5
	"Encuestas an&oacute;nimas"	// Okoteve traduccin
#elif L==6
	"Sondaggi"
#elif L==7
	"Ankiety"
#elif L==8
	"Inqu&eacute;ritos"
#endif
	,
	// ActReqUseGbl
#if   L==0
	"Grans nombres sobre l'&uacute;s actual de tota la plataforma,"
	" o d'aquesta titulaci&oacute; o assignatura"
#elif L==1
	"Nutzung von SWAD"
#elif L==2
	"Use of SWAD"
#elif L==3
	"Grandes n&uacute;meros sobre el uso actual de toda la plataforma,"
	" o de esta titulaci&oacute;n o asignatura"
#elif L==4
	"Utilisation de SWAD"
#elif L==5
	"Grandes n&uacute;meros sobre el uso actual de toda la plataforma,"
	" o de esta titulaci&oacute;n o asignatura"// Okoteve traduccin
#elif L==6
	"Uso di SWAD"
#elif L==7
	"Korzystanie z SWAD"
#elif L==8
	"Utiliza&ccedil;&atilde;o de SWAD"
#endif
	,
	// ActSeePhoDeg
#if   L==0
	"Estad&iacute;stiques del nombre d'estudiants en cada titulaci&oacute;"
	" representades per les fotos mitjana dels estudiants de cada titulaci&oacute;"
#elif L==1
	"Mischbilder der Studieng&auml;nge"
#elif L==2
	"Photos of degrees"
#elif L==3
	"Estad&iacute;sticas del n&uacute;mero de estudiantes en cada titulaci&oacute;n"
	" representadas por las fotos promedio de los estudiantes de cada titulaci&oacute;n"
#elif L==4
	"Photographie &eacute;tudes"
#elif L==5
	"Estad&iacute;sticas del n&uacute;mero de estudiantes en cada titulaci&oacute;n"
	" representadas por las fotos promedio de los estudiantes de cada titulaci&oacute;n"// Okoteve traduccin
#elif L==6
	"Foto di lauree"
#elif L==7
	"Zdjecia stopni"
#elif L==8
	"Fotografia titula&ccedil;oes"
#endif
	,
	// ActReqStaCrs
#if   L==0
	"Indicadors d'assignatures per al contracte-programa"
#elif L==1
	"Anzeigen von Kursen f&uuml;r den Vertrag Programm"
#elif L==2
	"Indicators of courses for the contract program"
#elif L==3
	"Indicadores de asignaturas para el contrato-programa"
#elif L==4
	"Indicateurs de mati&egrave;res pour le contrat-programme"
#elif L==5
	"Indicadores de asignaturas para el contrato-programa"// Okoteve traduccin
#elif L==6
	"Indicatori di corsi per il contratto programma"
#elif L==7
	"Wska&zacute;niki kurs&oacute;w dla umowy programu"
#elif L==8
	"Indicadores de disciplinas para o contrato programa"
#endif
	,
	// ActReqAccGbl
#if   L==0
	"Estad&iacute;stiques d'accesso a la plataforma"
#elif L==1
	"Zugriffsstatistik auf die Plattform"
#elif L==2
	"Access statistics to the platform"
#elif L==3
	"Estad&iacute;sticas de acceso a la plataforma"
#elif L==4
	"Statistiques de connexion &agrave; la plate-forme"
#elif L==5
	"Estad&iacute;sticas de acceso a la plataforma"	// Okoteve traduccin
#elif L==6
	"Statistiche di accesso alla piattaforma"
#elif L==7
	"Statystyki dost&eogon;p do platformy"
#elif L==8
	"Estat&iacute;sticas de acesso &agrave; plataforma"
#endif
	,
	// ActLstClk
#if   L==0
	"&Uacute;ltims clics en temps real"
#elif L==1
	"Letzte Klicks in Echtzeit"
#elif L==2
	"Last clicks in real time"
#elif L==3
	"&Uacute;ltimos clics en tiempo real"
#elif L==4
	"Derni&egrave;re clics en temps r&eacute;el"
#elif L==5
	"&Uacute;ltimos clics en tiempo real"// Okoteve traduccin
#elif L==6
	"Ultimi clic in tempo reale"
#elif L==7
	"Ostatnie dwa klikni&eogon;cia w czasie rzeczywistym"
#elif L==8
	"&Uacute;ltimos cliques em tempo real"
#endif
	,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	},
	{
	// TabPrf *****************************************************
	// ActFrmLogIn
#if   L==0
	"Rol en aquesta sessi&oacute;"
#elif L==1
	"Rolle in dieser Sitzung"
#elif L==2
	"Role in this session"
#elif L==3
	"Rol en esta sesi&oacute;n"
#elif L==4
	"R&ocirc;le dans cette session"
#elif L==5
	"Rol en esta sesi&oacute;n"	// Okoteve traduccin
#elif L==6
	"Ruolo in questa sessione"
#elif L==7
	"Rola w tej sesji"
#elif L==8
	"Role nesta sess&atilde;o"
#endif
	,
	// ActMyCrs
#if   L==0
	"Assignatures a les quals pertanyo"
#elif L==1
	"Kursen der ich angeh&ouml;re"
#elif L==2
	"Courses to which I belong"
#elif L==3
	"Asignaturas a las que pertenezco"
#elif L==4
	"Mati&egrave;res &agrave; laquelle je appartiens"
#elif L==5
	"Asignaturas a las que pertenezco"	// Okoteve traduccin
#elif L==6
	"Corsi a cui appartengo"
#elif L==7
	"Kursy do kt&oacute;rego nale&zdot;&eogon;"
#elif L==8
	"Disciplinas a que perten&ccedil;o"
#endif
	,
	// ActSeeMyTimTbl
#if   L==0
	"Horari conjunt de totes les meves assignatures a la plataforma"
#elif L==1
	"Stundenplan"
#elif L==2
	"My timetable"
#elif L==3
	"Horario conjunto de todas mis asignaturas en la plataforma"
#elif L==4
	"Horaire mes mati&egrave;res"
#elif L==5
	"Horario conjunto de todas mis asignaturas en la plataforma"// Okoteve traduccin
#elif L==6
	"Orario dei miei corsi"
#elif L==7
	"M&oacute;j plan lekcji"
#elif L==8
	"Hor&aacute;rio minha disciplinas"
#endif
	,
	// ActFrmUsrAcc
#if   L==0
	"Creaci&oacute; o edici&oacute; del meu compte d'usuari"
#elif L==1
	"Erstellen oder Bearbeiten meinem Benutzerkonto"
#elif L==2
	"Create or edit my user account"
#elif L==3
	"Creaci&oacute;n o edici&oacute;n de mi cuenta de usuario"
#elif L==4
	"Cr&eacute;er ou modifier mon compte d'utilisateur"
#elif L==5
	"Creaci&oacute;n o edici&oacute;n de mi cuenta de usuario"// Okoteve traduccin
#elif L==6
	"Creare o modificare il mio account utente"
#elif L==7
	"Utworzy&cacute; lub edytowa&cacute; konto u&zdot;ytkownika"
#elif L==8
	"Criar ou editar minha conta de usu&aacute;rio"
#endif
	,
	// ActReqEdiRecCom
#if   L==0
	"Dades personals, comuns per a totes les assignatures"
#elif L==1
	"Profil"
#elif L==2
	"Record"
#elif L==3
	"Datos personales, comunes para todas las asignaturas"
#elif L==4
	"Carte"
#elif L==5
	"Datos personales, comunes para todas las asignaturas"// Okoteve traduccin
#elif L==6
	"Scheda"
#elif L==7
	"Karta"
#elif L==8
	"Cart&atilde;o"
#endif
	,
	// ActEdiPrf
#if   L==0
	"Les meves prefer&egrave;ncies (idioma, colors, notificacions ...)"
#elif L==1
	"Einstellungen"
#elif L==2
	"Preferences"
#elif L==3
	"Mis preferencias (idioma, colores, notificaciones...)"
#elif L==4
	"Pr&eacute;f&eacute;rences"
#elif L==5
	"Mis preferencias (idioma, colores, notificaciones...)"	// Okoteve traduccin
#elif L==6
	"Preferenze"
#elif L==7
	"Preferencje"
#elif L==8
	"Prefer&ecirc;ncias"
#endif
	,
	// ActAdmBrf
#if   L==0
	"La meva zona privada d'emmagatzematge de fitxers"
#elif L==1
	"Aktentasche"
#elif L==2
	"Virtual pendrive"
#elif L==3
	"Mi zona privada de almacenamiento de archivos"
#elif L==4
	"Portedocuments"
#elif L==5
	"Mi zona privada de almacenamiento de archivos"	// Okoteve traduccin
#elif L==6
	"Penna USB"
#elif L==7
	"Pendrive wirtualne"
#elif L==8
	"Pasta"
#endif
	,
	// ActMFUAct
#if   L==0
	"Acc&eacute;s directe a les meves accions m&eacute;s freq&uuml;ents"
#elif L==1
	"Direkter Zugriff auf meine h&auml;ufige Aktionen"
#elif L==2
	"Direct access to my most frequent actions"
#elif L==3
	"Acceso directo a mis acciones m&aacute;s frecuentes"
#elif L==4
	"Acc&egrave;s direct &agrave; mes actions les plus fr&eacute;quentes"
#elif L==5
	"Acceso directo a mis acciones m&aacute;s frecuentes"// Okoteve traduccin
#elif L==6
	"Accesso diretto alle mie azioni pi&ugrave; frequenti"
#elif L==7
	"Bezpo&sacute;redni dost&eogon;p do najcz&eogon;stszych dzia&lstrok;a&nacute;"
#elif L==8
	"Acesso direto das minhas a&ccedil;&otilde;es mais freq&uuml;entes"
#endif
	,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	}
};

const char *Txt_message =
#if   L==0
	"missatge";
#elif L==1
	"Nachricht";
#elif L==2
	"message";
#elif L==3
	"mensaje";
#elif L==4
	"message";
#elif L==5
	"marandu";
#elif L==6
	"messaggio";
#elif L==7
	"wiadomosc";
#elif L==8
	"mensagem";
#endif

const char *Txt_Message_deleted =
#if   L==0
	"Mensaje eliminado.";		// Necessita traduccio
#elif L==1
	"Nachricht entfernt.";
#elif L==2
	"Message deleted.";
#elif L==3
	"Mensaje eliminado.";
#elif L==4
	"Message deleted.";		// Besoin de traduction
#elif L==5
	"Mensaje eliminado.";		// Okoteve traduccin
#elif L==6
	"Messaggio rimosso.";
#elif L==7
	"Wiadomo&sacute;&cacute; usunieta.";
#elif L==8
	"Message deleted.";		// Necessita de traduo
#endif

const char *Txt_message_not_sent_to_X =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"Missatge no enviat a %s.";
#elif L==1
	"Nachricht nicht gesendet, um %s.";
#elif L==2
	"Message not sent to %s.";
#elif L==3
	"Mensaje no enviado a %s.";
#elif L==4
	"Message non envoy&eacute; &agrave; %s.";
#elif L==5
	"Mensaje no enviado a %s.";	// Okoteve traduccin
#elif L==6
	"Messaggio non inviato a %s.";
#elif L==7
	"Wiadomo&sacute;&cacute; nie zostala wyslana do %s.";
#elif L==8
	"Mensagem n&atilde;o enviada para %s.";
#endif

const char *Txt_message_received_from_A_USER =
#if   L==0
	"mensaje recibido de";		// Necessita traduccio
#elif L==1
	"empfangene Nachricht aus";
#elif L==2
	"message received from";
#elif L==3
	"mensaje recibido de";
#elif L==4
	"message re&ccedil;u de";
#elif L==5
	"mensaje recibido de";		// Okoteve traduccin
#elif L==6
	"messaggio ricevuto da";
#elif L==7
	"wiadomosci otrzymane od";
#elif L==8
	"mensagem recebida de";
#endif

const char *Txt_message_sent_to =
#if   L==0
	"mensaje enviado a";	// Necessita traduccio
#elif L==1
	"Nachricht gesendet an";
#elif L==2
	"message sent to";
#elif L==3
	"mensaje enviado a";
#elif L==4
	"message envoy&eacute; &agrave;";
#elif L==5
	"mensaje enviado a";	// Okoteve traduccin
#elif L==6
	"messaggio inviato a";
#elif L==7
	"Wiadomo&sacute;&cacute; wyslana do";
#elif L==8
	"mensagem enviada para";
#endif

const char *Txt_message_sent_to_X_not_notified_by_e_mail =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"Mensaje enviado a %s (no notificado por e-mail).";	// Necessita traduccio
#elif L==1
	"Message sent to %s (not notified by e-mail).";	// Need bersetzung
#elif L==2
	"Message sent to %s (not notified by e-mail).";
#elif L==3
	"Mensaje enviado a %s (no notificado por e-mail).";
#elif L==4
	"Message sent to %s (not notified by e-mail).";	// Besoin de traduction
#elif L==5
	"Mensaje enviado a %s (no notificado por e-mail).";	// Okoteve traduccin
#elif L==6
	"Messaggio inviato a %s (non notificato per e-mail).";
#elif L==7
	"Wiadomo&sacute;&cacute; wyslana do %s (nie zgloszonych przez e-mail).";	//
#elif L==8
	"Message sent to %s (not notified by e-mail).";	// Necessita de traduo
#endif

const char *Txt_message_sent_to_X_notified_by_e_mail =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"Mensaje enviado a %s (notificado por e-mail).";	// Necessita traduccio
#elif L==1
	"Message sent to %s (notified by e-mail).";	// Need bersetzung
#elif L==2
	"Message sent to %s (notified by e-mail).";
#elif L==3
	"Mensaje enviado a %s (notificado por e-mail).";
#elif L==4
	"Message sent to %s (notified by e-mail).";	// Besoin de traduction
#elif L==5
	"Mensaje enviado a %s (notificado por e-mail).";	// Okoteve traduccin
#elif L==6
	"Messaggio inviato a %s (notificato per e-mail).";
#elif L==7
	"Wiadomo&sacute;&cacute; wyslana do %s (zgloszone przez e-mail).";
#elif L==8
	"Message sent to %s (notified by e-mail).";	// Necessita de traduo
#endif

const char *Txt_Messages =
#if   L==0
	"Missatges";
#elif L==1
	"Nachrichten";
#elif L==2
	"Messages";
#elif L==3
	"Mensajes";
#elif L==4
	"Messages";
#elif L==5
	"Marandu";
#elif L==6
	"Messaggi";
#elif L==7
	"Wiadomo&sacute;ci";
#elif L==8
	"Mensagens";
#endif

const char *Txt_messages =
#if   L==0
	"missatges";
#elif L==1
	"Nachrichten";
#elif L==2
	"messages";
#elif L==3
	"mensajes";
#elif L==4
	"messages";
#elif L==5
	"marandu";
#elif L==6
	"messaggi";
#elif L==7
	"wiadomo&sacute;ci";
#elif L==8
	"mensagens";
#endif

const char *Txt_X_messages_have_been_deleted =	// Warning: it is very important to include %u in the following sentences
#if   L==0
	"Se han eliminado %u mensajes.";	// Necessita traduccio
#elif L==1
	"%u messages have been deleted.";	// Need bersetzung
#elif L==2
	"%u messages have been deleted.";
#elif L==3
	"Se han eliminado %u mensajes.";
#elif L==4
	"%u messages have been deleted.";	// Besoin de traduction
#elif L==5
	"Se han eliminado %u mensajes.";	// Okoteve traduccin
#elif L==6
	"%u messaggi sono stati rimossi.";
#elif L==7
	"wiadomosci% u zostaly usuniete.";
#elif L==8
	"%u messages have been deleted.";	// Necessita de traduo
#endif

const char *Txt_Messages_of_THE_USER_X_have_been_deleted =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"Se han eliminado los mensajes de <strong>%s</strong>.";	// Necessita traduccio
#elif L==1
	"Messages of <strong>%s</strong> have been deleted.";	// Need bersetzung
#elif L==2
	"Messages of <strong>%s</strong> have been deleted.";
#elif L==3
	"Se han eliminado los mensajes de <strong>%s</strong>.";
#elif L==4
	"Messages of <strong>%s</strong> have been deleted.";	// Besoin de traduction
#elif L==5
	"Se han eliminado los mensajes de <strong>%s</strong>.";	// Okoteve traduccin
#elif L==6
	"Messaggi de <strong>%s</strong> sono stati rimossi.";
#elif L==7
	"Wiadomo&sacute;ci od <strong>%s</strong> zostaly usuniete.";
#elif L==8
	"Messages of <strong>%s</strong> have been deleted.";	// Necessita de traduo
#endif

const char *Txt_Messages_received =
#if   L==0
	"Missatges rebuts";
#elif L==1
	"Empfangene Nachrichten";
#elif L==2
	"Messages received";
#elif L==3
	"Mensajes recibidos";
#elif L==4
	"Messages re&ccedil;us";
#elif L==5
	"Mensajes recibidos";	// Okoteve traduccin
#elif L==6
	"Messaggi ricevuti";
#elif L==7
	"Wiadomo&sacute;ci odebrane";
#elif L==8
	"Mensagens recebidas";
#endif

const char *Txt_Messages_received_from_A_COURSE =
#if   L==0
	"Missatges rebuts des de";
#elif L==1
	"Empfangene Nachrichten aus";
#elif L==2
	"Messages received from";
#elif L==3
	"Mensajes recibidos desde";
#elif L==4
	"Messages re&ccedil;us de";
#elif L==5
	"Mensajes recibidos desde";	// Okoteve traduccin
#elif L==6
	"Messaggi ricevuti da";
#elif L==7
	"Wiadomo&sacute;ci odebrane";
#elif L==8
	"Mensagens recebidas de";
#endif

const char *Txt_messages_received_from_A_USER =
#if   L==0
	"mensajes recibidos de";	// Necessita traduccio
#elif L==1
	"empfangene Nachrichten aus";
#elif L==2
	"messages received from";
#elif L==3
	"mensajes recibidos de";
#elif L==4
	"messages re&ccedil;us de";
#elif L==5
	"mensajes recibidos de";	// Okoteve traduccin
#elif L==6
	"messaggi ricevuti da";
#elif L==7
	"wiadomosci odebrane";
#elif L==8
	"mensagens recebidas de";
#endif

const char *Txt_Messages_sent =
#if   L==0
	"Missatges enviats";
#elif L==1
	"Gesendete Nachrichten";
#elif L==2
	"Messages sent";
#elif L==3
	"Mensajes enviados";
#elif L==4
	"Messages envoy&eacute;s";
#elif L==5
	"Mensajes enviados";	// Okoteve traduccin
#elif L==6
	"Messaggi inviati";
#elif L==7
	"Wiadomo&sacute;ci wysylane";
#elif L==8
	"Mensagens enviadas";
#endif

const char *Txt_Messages_sent_by_teachers =
#if   L==0
	"Mensajes enviados por profesores";	// Necessita traduccio
#elif L==1
	"Gesendete Nachrichten, die von Lehrkr&auml;ften";
#elif L==2
	"Messages sent by teachers";
#elif L==3
	"Mensajes enviados por profesores";
#elif L==4
	"Messages envoy&eacute;s par enseignants";
#elif L==5
	"Mensajes enviados por profesores";	// Okoteve traduccin
#elif L==6
	"Messaggi inviati dai professori";
#elif L==7
	"Wiadomo&sacute;ci wysylane przez nauczycieli";
#elif L==8
	"Mensagens enviadas por professores";
#endif

const char *Txt_Messages_sent_from_A_COURSE =
#if   L==0
	"Missatges enviats des de";
#elif L==1
	"Gesendete Nachrichten aus";
#elif L==2
	"Messages sent from";
#elif L==3
	"Mensajes enviados desde";
#elif L==4
	"Messages envoy&eacute;s de";
#elif L==5
	"Mensajes enviados desde";	// Okoteve traduccin
#elif L==6
	"Messaggi inviati da";
#elif L==7
	"Wiadomo&sacute;ci wyslane z";
#elif L==8
	"Mensagens enviadas de";
#endif

const char *Txt_messages_sent_to_A_USER =
#if   L==0
	"mensajes enviados a";	// Necessita traduccio
#elif L==1
	"gesendete Nachrichten an";
#elif L==2
	"messages sent to";
#elif L==3
	"mensajes enviados a";
#elif L==4
	"messages envoy&eacute;s &agrave;";
#elif L==5
	"mensajes enviados a";	// Okoteve traduccin
#elif L==6
	"messaggi inviati a";
#elif L==7
	"wiadomosci wysylanych do";
#elif L==8
	"mensagens enviadas para";
#endif

const char *Txt_minimum =
#if   L==0
	"m&iacute;nimo";	// Necessita traduccio
#elif L==1
	"Mindest";
#elif L==2
	"minimum";
#elif L==3
	"m&iacute;nimo";
#elif L==4
	"minimum";
#elif L==5
	"m&iacute;nimo";	// Okoteve traduccin
#elif L==6
	"minimo";
#elif L==7
	"minimum";
#elif L==8
	"m&iacute;nimo";
#endif

const char *Txt_Minimum_time_seconds_per_question_between_two_tests =
#if   L==0
	"Tiempo m&iacute;nimo (segundos)<br />por pregunta<br />entre dos tests";	// Necessita traduccio
#elif L==1
	"Minimale Zeit (Sekunden)<br />pro Frage<br />zwischen zwei Tests";
#elif L==2
	"Minimum time (seconds)<br />per question<br />between two tests";
#elif L==3
	"Tiempo m&iacute;nimo (segundos)<br />por pregunta<br />entre dos tests";
#elif L==4
	"Minimum de temps (en secondes)<br />par question<br />entre deux tests";
#elif L==5
	"Tiempo m&iacute;nimo (segundos)<br />por pregunta<br />entre dos tests";	// Okoteve traduccin
#elif L==6
	"Minimo tempo (in secondi)<br />per domanda<br />tra due test";
#elif L==7
	"Minimalny czas (w sekundach)<br />na pytanie miedzy<br />dwoma badaniami";
#elif L==8
	"Tempo m&iacute;nimo (segundos)<br />por quest&atilde;o<br />entre dois testes";
#endif

const char *Txt_minute =
#if   L==0
	"minuto";	// Necessita traduccio
#elif L==1
	"Minute";
#elif L==2
	"minute";
#elif L==3
	"minuto";
#elif L==4
	"minute";
#elif L==5
	"aravo'i";
#elif L==6
	"minuto";
#elif L==7
	"minut&eogon;";
#elif L==8
	"minuto";
#endif

const char *Txt_minutes =
#if   L==0
	"minutos";	// Necessita traduccio
#elif L==1
	"Minute";
#elif L==2
	"minutes";
#elif L==3
	"minutos";
#elif L==4
	"minutes";
#elif L==5
	"aravo'i";
#elif L==6
	"minuti";
#elif L==7
	"minuty";
#elif L==8
	"minutos";
#endif

const char *Txt_Modify_assignment =
#if   L==0
	"Modificar actividad";	// Necessita traduccio
#elif L==1
	"&Auml;ndern Aufgabe";
#elif L==2
	"Modify assignment";
#elif L==3
	"Modificar actividad";
#elif L==4
	"Modifier activit&eacute;";
#elif L==5
	"Modificar actividad";	// Okoteve traduccin
#elif L==6
	"Modifica attivit&agrave;";
#elif L==7
	"Modyfikacja zadania";
#elif L==8
	"Modificar atividade";
#endif

const char *Txt_Modify_event =
#if   L==0
	"Modificar evento";	// Necessita traduccio
#elif L==1
	"&Auml;ndern Ereignis";
#elif L==2
	"Modify event";
#elif L==3
	"Modificar evento";
#elif L==4
	"Modifier &eacute;v&eacute;nement";
#elif L==5
	"Modificar evento";	// Okoteve traduccin
#elif L==6
	"Modifica evento";
#elif L==7
	"Modyfikacja wydarzenie";
#elif L==8
	"Modificar evento";
#endif

const char *Txt_Modify_me_in_the_course_X =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"Modificarme en l'assignatura <strong>%s</strong>";
#elif L==1
	"&Auml;ndern Sie mir im Kurs <strong>%s</strong>";
#elif L==2
	"Modify me in the course <strong>%s</strong>";
#elif L==3
	"Modificarme en la asignatura <strong>%s</strong>";
#elif L==4
	"Modifier-moi dans la mati&egrave;re <strong>%s</strong>";
#elif L==5
	"Modificarme en la asignatura <strong>%s</strong>";	// Okoteve traduccin
#elif L==6
	"Modificami utente nel corso <strong>%s</strong>";
#elif L==7
	"Modyfikacja mnie w kurs <strong>%s</strong>";
#elif L==8
	"Modificar me na disciplina <strong>%s</strong>";
#endif

const char *Txt_Modify_user_in_the_course_X =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"Modificar usuari en l'assignatura <strong>%s</strong>";
#elif L==1
	"&Auml;ndern Benutzer im Kurs <strong>%s</strong>";
#elif L==2
	"Modify user in the course <strong>%s</strong>";
#elif L==3
	"Modificar usuario en la asignatura <strong>%s</strong>";
#elif L==4
	"Modifier utilisateur dans la mati&egrave;re <strong>%s</strong>";
#elif L==5
	"Modificar usuario en la asignatura <strong>%s</strong>";	// Okoteve traduccin
#elif L==6
	"Modifica utente nel corso <strong>%s</strong>";
#elif L==7
	"Modyfikacja u&zdot;ytkownika w kurs <strong>%s</strong>";
#elif L==8
	"Modificar utilizador na disciplina <strong>%s</strong>";
#endif

const char *Txt_Modify_survey =
#if   L==0
	"Modifier sondage";	// Necessita traduccio
#elif L==1
	"&Auml;ndern Umfrage";
#elif L==2
	"Modify survey";
#elif L==3
	"Modificar encuesta";
#elif L==4
	"Modifier sondage";
#elif L==5
	"Modificar encuesta";	// Okoteve traduccin
#elif L==6
	"Modifica sondaggio";
#elif L==7
	"Badanie Modyfikuj";
#elif L==8
	"Modificar inqu&eacute;rito";
#endif

const char *Txt_Month =
#if   L==0
	"Mes";		// Necessita traduccio
#elif L==1
	"Monat";
#elif L==2
	"Month";
#elif L==3
	"Mes";
#elif L==4
	"Mois";
#elif L==5
	"Jasy";
#elif L==6
	"Mese";
#elif L==7
	"Miesiac";
#elif L==8
	"M&ecirc;s";
#endif

const char *Txt_MONTHS_CAPS[12] =
   {
#if   L==0
	 "GENER"
#elif L==1
	 "JANUAR"
#elif L==2
	 "JANUARY"
#elif L==3
	 "ENERO"
#elif L==4
	 "JANVIER"
#elif L==5
	 "JASYTE&#296;"
#elif L==6
	 "GENNAIO"
#elif L==7
	 "STYCZNIA"
#elif L==8
	 "JANEIRO"
#endif
	 ,
#if   L==0
	 "FEBRER"
#elif L==1
	 "FEBRUAR"
#elif L==2
	 "FEBRUARY"
#elif L==3
	 "FEBRERO"
#elif L==4
	 "F&Eacute;VRIER"
#elif L==5
	 "JASYK&Otilde;I"
#elif L==6
	 "FEBBRAIO"
#elif L==7
	 "LUTEGO"
#elif L==8
	 "FEVEREIRO"
#endif
	 ,
#if   L==0
	 "MAR&Ccedil;"
#elif L==1
	 "M&Auml;RZ"
#elif L==2
	 "MARCH"
#elif L==3
	 "MARZO"
#elif L==4
	 "MARS"
#elif L==5
	 "JASYAPY"
#elif L==6
	 "MARZO"
#elif L==7
	 "MARCA"
#elif L==8
	 "MAR&Ccedil;O"
#endif
	 ,
#if   L==0
	 "ABRIL"
#elif L==1
	 "APRIL"
#elif L==2
	 "APRIL"
#elif L==3
	 "ABRIL"
#elif L==4
	 "AVRIL"
#elif L==5
	 "JASYRUNDY"
#elif L==6
	 "APRILE"
#elif L==7
	 "KWIETNIA"
#elif L==8
	 "ABRIL"
#endif
	 ,
#if   L==0
	 "MAIG"
#elif L==1
	 "MAI"
#elif L==2
	 "MAY"
#elif L==3
	 "MAYO"
#elif L==4
	 "MAI"
#elif L==5
	 "JASYPO"
#elif L==6
	 "MAGGIO"
#elif L==7
	 "MAJA"
#elif L==8
	 "MAIO"
#endif
	 ,
#if   L==0
	 "JUNY"
#elif L==1
	 "JUNI"
#elif L==2
	 "JUNE"
#elif L==3
	 "JUNIO"
#elif L==4
	 "JUIN"
#elif L==5
	 "JASYPOTE&#296;"
#elif L==6
	 "GIUGNO"
#elif L==7
	 "CZERWCA"
#elif L==8
	 "JUNHO"
#endif
	 ,
#if   L==0
	 "JULIOL"
#elif L==1
	 "JULI"
#elif L==2
	 "JULY"
#elif L==3
	 "JULIO"
#elif L==4
	 "JUILLET"
#elif L==5
	 "JASYPOK&Otilde;I"
#elif L==6
	 "LUGLIO"
#elif L==7
	 "LIPCA"
#elif L==8
	 "JULHO"
#endif
	 ,
#if   L==0
	 "AGOST"
#elif L==1
	 "AUGUST"
#elif L==2
	 "AUGUST"
#elif L==3
	 "AGOSTO"
#elif L==4
	 "AO&Ucirc;T"
#elif L==5
	 "JASYPOAPY"
#elif L==6
	 "AGOSTO"
#elif L==7
	 "SIERPNIA"
#elif L==8
	 "AGOSTO"
#endif
	 ,
#if   L==0
	 "SETEMBRE"
#elif L==1
	 "SEPTEMBER"
#elif L==2
	 "SEPTEMBER"
#elif L==3
	 "SEPTIEMBRE"
#elif L==4
	 "SEPTEMBRE"
#elif L==5
	 "JASYPORUNDY"
#elif L==6
	 "SETTEMBRE"
#elif L==7
	 "WRZE&Sacute;NIA"
#elif L==8
	 "SETEMBRO"
#endif
	 ,
#if   L==0
	 "OCTUBRE"
#elif L==1
	 "OKTOBER"
#elif L==2
	 "OCTOBER"
#elif L==3
	 "OCTUBRE"
#elif L==4
	 "OCTOBRE"
#elif L==5
	 "JASYPA"
#elif L==6
	 "OTTOBRE"
#elif L==7
	 "PA&Zacute;DZIERNIKA"
#elif L==8
	 "OUTUBRO"
#endif
	 ,
#if   L==0
	 "NOVEMBRE"
#elif L==1
	 "NOVEMBER"
#elif L==2
	 "NOVEMBER"
#elif L==3
	 "NOVIEMBRE"
#elif L==4
	 "NOVEMBRE"
#elif L==5
	 "JASYPATE&#296;"
#elif L==6
	 "NOVEMBRE"
#elif L==7
	 "LISTOPADA"
#elif L==8
	 "NOVEMBRO"
#endif
	 ,
#if   L==0
	 "DESEMBRE"
#elif L==1
	 "DEZEMBER"
#elif L==2
	 "DECEMBER"
#elif L==3
	 "DICIEMBRE"
#elif L==4
	 "D&Eacute;CEMBRE"
#elif L==5
	 "JASYPAK&Otilde;I"
#elif L==6
	 "DICEMBRE"
#elif L==7
	 "GRUDNIA"
#elif L==8
	 "DEZEMBRO"
#endif
	};

const char *Txt_MONTHS_SMALL[12] =
   {
#if   L==0
	 "gener"
#elif L==1
	 "Januar"
#elif L==2
	 "January"
#elif L==3
	 "enero"
#elif L==4
	 "janvier"
#elif L==5
	 "jasyte&#297;"
#elif L==6
	 "gennaio"
#elif L==7
	 "Stycznia"
#elif L==8
	 "janeiro"
#endif
	 ,
#if   L==0
	 "febrer"
#elif L==1
	 "Februar"
#elif L==2
	 "February"
#elif L==3
	 "febrero"
#elif L==4
	 "f&eacute;vrier"
#elif L==5
	 "jasyk&otilde;i"
#elif L==6
	 "febbraio"
#elif L==7
	 "Lutego"
#elif L==8
	 "fevereiro"
#endif
	 ,
#if   L==0
	 "mar&ccedil;"
#elif L==1
	 "M;&auml;rz"
#elif L==2
	 "March"
#elif L==3
	 "marzo"
#elif L==4
	 "mars"
#elif L==5
	 "jasyapy"
#elif L==6
	 "marzo"
#elif L==7
	 "Marca"
#elif L==8
	 "mar&ccedil;o"
#endif
	 ,
#if   L==0
	 "abril"
#elif L==1
	 "April"
#elif L==2
	 "April"
#elif L==3
	 "abril"
#elif L==4
	 "avril"
#elif L==5
	 "jasyrundy"
#elif L==6
	 "aprile"
#elif L==7
	 "Kwietnia"
#elif L==8
	 "abril"
#endif
	 ,
#if   L==0
	 "maig"
#elif L==1
	 "Mai"
#elif L==2
	 "May"
#elif L==3
	 "mayo"
#elif L==4
	 "mai"
#elif L==5
	 "jasypo"
#elif L==6
	 "maggio"
#elif L==7
	 "Maja"
#elif L==8
	 "maio"
#endif
	 ,
#if   L==0
	 "juny"
#elif L==1
	 "Juni"
#elif L==2
	 "June"
#elif L==3
	 "junio"
#elif L==4
	 "juin"
#elif L==5
	 "jasypote&#297;"
#elif L==6
	 "giugno"
#elif L==7
	 "Czerwca"
#elif L==8
	 "junho"
#endif
	 ,
#if   L==0
	 "juliol"
#elif L==1
	 "Juli"
#elif L==2
	 "July"
#elif L==3
	 "julio"
#elif L==4
	 "juillet"
#elif L==5
	 "jasypok&otilde;i"
#elif L==6
	 "luglio"
#elif L==7
	 "Lipca"
#elif L==8
	 "julho"
#endif
	 ,
#if   L==0
	 "agost"
#elif L==1
	 "August"
#elif L==2
	 "August"
#elif L==3
	 "agosto"
#elif L==4
	 "ao&ucirc;t"
#elif L==5
	 "jasypoapy"
#elif L==6
	 "agosto"
#elif L==7
	 "Sierpnia"
#elif L==8
	 "agosto"
#endif
	 ,
#if   L==0
	 "setembre"
#elif L==1
	 "September"
#elif L==2
	 "September"
#elif L==3
	 "septiembre"
#elif L==4
	 "septembre"
#elif L==5
	 "jasyporundy"
#elif L==6
	 "settembre"
#elif L==7
	 "Wrze&sacute;nia"
#elif L==8
	 "setembro"
#endif
	 ,
#if   L==0
	 "octubre"
#elif L==1
	 "Oktober"
#elif L==2
	 "October"
#elif L==3
	 "octubre"
#elif L==4
	 "octobre"
#elif L==5
	 "jasypa"
#elif L==6
	 "ottobre"
#elif L==7
	 "Pa&zacute;dziernika"
#elif L==8
	 "outubro"
#endif
	 ,
#if   L==0
	 "novembre"
#elif L==1
	 "November"
#elif L==2
	 "November"
#elif L==3
	 "noviembre"
#elif L==4
	 "novembre"
#elif L==5
	 "jasypate&#297;"
#elif L==6
	 "novembre"
#elif L==7
	 "Listopada"
#elif L==8
	 "novembro"
#endif
	 ,
#if   L==0
	 "desembre"
#elif L==1
	 "Dezember"
#elif L==2
	 "December"
#elif L==3
	 "diciembre"
#elif L==4
	 "d&eacute;cembre"
#elif L==5
	 "jasypak&otilde;i"
#elif L==6
	 "dicembre"
#elif L==7
	 "Grudnia"
#elif L==8
	 "dezembro"
#endif
	};

const char *Txt_MONTHS_SMALL_SHORT[12] =
   {
#if   L==0
	 "gen"
#elif L==1
	 "Jan"
#elif L==2
	 "Jan"
#elif L==3
	 "ene"
#elif L==4
	 "jan"
#elif L==5
	 "ene"	// Okoteve traduccin
#elif L==6
	 "gen"
#elif L==7
	 "Sty"
#elif L==8
	 "jan"
#endif
	 ,
#if   L==0
	 "feb"
#elif L==1
	 "Feb"
#elif L==2
	 "Feb"
#elif L==3
	 "feb"
#elif L==4
	 "f&eacute;v"
#elif L==5
	 "feb"	// Okoteve traduccin
#elif L==6
	 "feb"
#elif L==7
	 "Lut"
#elif L==8
	 "fev"
#endif
	 ,
#if   L==0
	 "mar"
#elif L==1
	 "M;&auml;r"
#elif L==2
	 "Mar"
#elif L==3
	 "mar"
#elif L==4
	 "mar"
#elif L==5
	 "mar"	// Okoteve traduccin
#elif L==6
	 "mar"
#elif L==7
	 "Mar"
#elif L==8
	 "mar"
#endif
	 ,
#if   L==0
	 "abr"
#elif L==1
	 "Apr"
#elif L==2
	 "Apr"
#elif L==3
	 "abr"
#elif L==4
	 "avr"
#elif L==5
	 "abr"	// Okoteve traduccin
#elif L==6
	 "apr"
#elif L==7
	 "Kwi"
#elif L==8
	 "abr"
#endif
	 ,
#if   L==0
	 "mai"
#elif L==1
	 "Mai"
#elif L==2
	 "May"
#elif L==3
	 "may"
#elif L==4
	 "mai"
#elif L==5
	 "may"	// Okoteve traduccin
#elif L==6
	 "mag"
#elif L==7
	 "Maj"
#elif L==8
	 "mai"
#endif
	 ,
#if   L==0
	 "jun"
#elif L==1
	 "Jun"
#elif L==2
	 "Jun"
#elif L==3
	 "jun"
#elif L==4
	 "jui"
#elif L==5
	 "jun"	// Okoteve traduccin
#elif L==6
	 "giu"
#elif L==7
	 "Cze"
#elif L==8
	 "jun"
#endif
	 ,
#if   L==0
	 "jul"
#elif L==1
	 "Jul"
#elif L==2
	 "Jul"
#elif L==3
	 "jul"
#elif L==4
	 "jui"
#elif L==5
	 "jul"	// Okoteve traduccin
#elif L==6
	 "lug"
#elif L==7
	 "Lip"
#elif L==8
	 "jul"
#endif
	 ,
#if   L==0
	 "ago"
#elif L==1
	 "Aug"
#elif L==2
	 "Aug"
#elif L==3
	 "ago"
#elif L==4
	 "ao&ucirc;"
#elif L==5
	 "ago"	// Okoteve traduccin
#elif L==6
	 "ago"
#elif L==7
	 "Sie"
#elif L==8
	 "ago"
#endif
	 ,
#if   L==0
	 "set"
#elif L==1
	 "Sep"
#elif L==2
	 "Sep"
#elif L==3
	 "sep"
#elif L==4
	 "sep"
#elif L==5
	 "sep"	// Okoteve traduccin
#elif L==6
	 "set"
#elif L==7
	 "Wrz"
#elif L==8
	 "set"
#endif
	 ,
#if   L==0
	 "oct"
#elif L==1
	 "Okt"
#elif L==2
	 "Oct"
#elif L==3
	 "oct"
#elif L==4
	 "oct"
#elif L==5
	 "oct"	// Okoteve traduccin
#elif L==6
	 "ott"
#elif L==7
	 "Pa&zacute;"
#elif L==8
	 "out"
#endif
	 ,
#if   L==0
	 "nov"
#elif L==1
	 "Nov"
#elif L==2
	 "Nov"
#elif L==3
	 "nov"
#elif L==4
	 "nov"
#elif L==5
	 "nov"	// Okoteve traduccin
#elif L==6
	 "nov"
#elif L==7
	 "Lis"
#elif L==8
	 "nov"
#endif
	 ,
#if   L==0
	 "des"
#elif L==1
	 "Dez"
#elif L==2
	 "Dec"
#elif L==3
	 "dic"
#elif L==4
	 "d&eacute;c"
#elif L==5
	 "dic"	// Okoteve traduccin
#elif L==6
	 "dic"
#elif L==7
	 "Gru"
#elif L==8
	 "dez"
#endif
	};

const char *Txt_Move_up_X =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"Subir %s";		// Necessita traduccio
#elif L==1
	"Move up %s";		// Need bersetzung
#elif L==2
	"Move up %s";
#elif L==3
	"Subir %s";
#elif L==4
	"Move up %s";		// Besoin de traduction
#elif L==5
	"Subir %s";		// Okoteve traduccin
#elif L==6
	"Muovi su %s";
#elif L==7
	"Przesun w g&oacute;re %s";
#elif L==8
	"Move up %s";		// Necessita de traduo
#endif

const char *Txt_Move_up_X_and_its_subsections =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"Subir %s y sus subapartados";			// Necessita traduccio
#elif L==1
	"Move up %s and its subsections";		// Need bersetzung
#elif L==2
	"Move up %s and its subsections";
#elif L==3
	"Subir %s y sus subapartados";
#elif L==4
	"Move up %s and its subsections";		// Besoin de traduction
#elif L==5
	"Subir %s y sus subapartados";			// Okoteve traduccin
#elif L==6
	"Muovi su %s e le sue sottosezioni";
#elif L==7
	"Przenies sie %s, a jego podrozdzialy";
#elif L==8
	"Move up %s and its subsections";		// Necessita de traduo
#endif

const char *Txt_Move_down_X =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"Bajar %s";		// Necessita traduccio
#elif L==1
	"Move down %s";		// Need bersetzung
#elif L==2
	"Move down %s";
#elif L==3
	"Bajar %s";
#elif L==4
	"Move down %s";		// Besoin de traduction
#elif L==5
	"Bajar %s";		// Okoteve traduccin
#elif L==6
	"Muovi gi&ugrave; %s";
#elif L==7
	"Opuszczac %s";
#elif L==8
	"Move down %s";		// Necessita de traduo
#endif

const char *Txt_Move_down_X_and_its_subsections =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"Bajar %s y sus subapartados";			// Necessita traduccio
#elif L==1
	"Move down %s and its subsections";		// Need bersetzung
#elif L==2
	"Move down %s and its subsections";
#elif L==3
	"Bajar %s y sus subapartados";
#elif L==4
	"Move down %s and its subsections";		// Besoin de traduction
#elif L==5
	"Bajar %s y sus subapartados";			// Okoteve traduccin
#elif L==6
	"Muovi gi&ugrave; %s e le sue sottosezioni";
#elif L==7
	"Przesun w d&oacute;l %s, a jego podrozdzialy";
#elif L==8
	"Move down %s and its subsections";		// Necessita de traduo
#endif

const char *Txt_Move_thread =
#if   L==0
	"Mover discusi&oacute;n";	// Necessita traduccio
#elif L==1
	"Verschieben Thread";
#elif L==2
	"Move thread";
#elif L==3
	"Mover discusi&oacute;n";
#elif L==4
	"D&eacute;placer fil";
#elif L==5
	"Mover discusi&oacute;n";	// Okoteve traduccin
#elif L==6
	"Sposta discussione";
#elif L==7
	"Przenies watku";
#elif L==8
	"Mover thread";
#endif

const char *Txt_MSG_containing_the_text =
#if   L==0
	"que contenga el texto";	// Necessita traduccio
#elif L==1
	"mit dem Text";
#elif L==2
	"containing the text";
#elif L==3
	"que contenga el texto";
#elif L==4
	"contenant le texte";
#elif L==5
	"que contenga el texto";	// Okoteve traduccin
#elif L==6
	"contenente il testo";
#elif L==7
	"zawierajacy tekst";
#elif L==8
	"contendo o texto";
#endif

const char *Txt_MSG_Deleted_without_opening =
#if   L==0
	"Eliminado sin abrir";		// Necessita traduccio
#elif L==1
	"Gel&ouml;scht ohne &Ouml;ffnen";
#elif L==2
	"Deleted without opening";
#elif L==3
	"Eliminado sin abrir";
#elif L==4
	"Supprim&eacute; sans ouvrir";
#elif L==5
	"Eliminado sin abrir";		// Okoteve traduccin
#elif L==6
	"Rimosso senza aprire";
#elif L==7
	"Usuniete bez otwierania";
#elif L==8
	"Suprimida sem abrir";
#endif

const char *Txt_MSG_From =
#if   L==0
	"De";		// Necessita traduccio
#elif L==1
	"Von";
#elif L==2
	"From";
#elif L==3
	"De";
#elif L==4
	"De";
#elif L==5
	"De";		// Okoteve traduccin
#elif L==6
	"Da";
#elif L==7
	"z";
#elif L==8
	"De";
#endif

const char *Txt_MSG_from =
#if   L==0
	"from";		// Necessita traduccio
#elif L==1
	"von";
#elif L==2
	"from";
#elif L==3
	"de";
#elif L==4
	"de";
#elif L==5
	"de";	// Okoteve traduccin
#elif L==6
	"da";
#elif L==7
	"z";
#elif L==8
	"de";
#endif

const char *Txt_MSG_Message =
#if   L==0
	"Missatge";
#elif L==1
	"Nachricht";
#elif L==2
	"Message";
#elif L==3
	"Mensaje";
#elif L==4
	"Message";
#elif L==5
	"Marandu";
#elif L==6
	"Messaggio";
#elif L==7
	"Wiadomo&sacute;&cacute;";
#elif L==8
	"Mensagem";
#endif

const char *Txt_MSG_New =
#if   L==0
	"Nuevo";	// Necessita traduccio
#elif L==1
	"Neue";
#elif L==2
	"New";
#elif L==3
	"Nuevo";
#elif L==4
	"Nouveau";
#elif L==5
	"Nuevo";	// Okoteve traduccin
#elif L==6
	"Nuovo";
#elif L==7
	"nowe";
#elif L==8
	"Nova";
#endif

const char *Txt_MSG_Not_replied =
#if   L==0
	"No respondido";	// Necessita traduccio
#elif L==1
	"Nicht geantwortet";
#elif L==2
	"Not replied";
#elif L==3
	"No respondido";
#elif L==4
	"Pas r&eacute;pondu";
#elif L==5
	"No respondido";	// Okoteve traduccin
#elif L==6
	"Non risposto";
#elif L==7
	"Nie odpowiedzial";
#elif L==8
	"N&atilde;o respondida";
#endif

const char *Txt_MSG_Open =
#if   L==0
	"Abierto";	// Necessita traduccio
#elif L==1
	"Ge&ouml;ffneten";
#elif L==2
	"Open";
#elif L==3
	"Abierto";
#elif L==4
	"Ouvert";
#elif L==5
	"Abierto";	// Okoteve traduccin
#elif L==6
	"Aperto";
#elif L==7
	"otwarte";
#elif L==8
	"Aberta";
#endif

const char *Txt_MSG_Open_and_deleted =
#if   L==0
	"Abierto y eliminado";	// Necessita traduccio
#elif L==1
	"Ge&ouml;ffneten und gel&ouml;scht";
#elif L==2
	"Open and deleted";
#elif L==3
	"Abierto y eliminado";
#elif L==4
	"Ouvert et supprim&eacute;";
#elif L==5
	"Abierto y eliminado";	// Okoteve traduccin
#elif L==6
	"Aperto e rimosso";
#elif L==7
	"Open i usuniete";
#elif L==8
	"Aberta e suprimida";
#endif

const char *Txt_MSG_Replied =
#if   L==0
	"Respondido";	// Necessita traduccio
#elif L==1
	"Geantwortet";
#elif L==2
	"Replied";
#elif L==3
	"Respondido";
#elif L==4
	"R&eacute;pondu";
#elif L==5
	"Respondido";	// Okoteve traduccin
#elif L==6
	"Risposto";
#elif L==7
	"Odpowiedziano";
#elif L==8
	"Respondida";
#endif

const char *Txt_MSG_Sent =
#if   L==0
	"Enviado";	// Necessita traduccio
#elif L==1
	"Gesendet";
#elif L==2
	"Sent";
#elif L==3
	"Enviado";
#elif L==4
	"Envoy&eacute;";
#elif L==5
	"Enviado";	// Okoteve traduccin
#elif L==6
	"Inviato";
#elif L==7
	"Wyslane";
#elif L==8
	"Enviada";
#endif

const char *Txt_MSG_Sent_and_deleted =
#if   L==0
	"Enviado y eliminado";	// Necessita traduccio
#elif L==1
	"Gesendet und gel&ouml;scht";
#elif L==2
	"Sent and deleted";
#elif L==3
	"Enviado y eliminado";
#elif L==4
	"Envoy&eacute; et supprim&eacute;";
#elif L==5
	"Enviado y eliminado";	// Okoteve traduccin
#elif L==6
	"Inviato e rimosso";
#elif L==7
	"Wyslane i usuniete";
#elif L==8
	"Enviada e suprimida";
#endif

const char *Txt_MSG_Subject =
#if   L==0
	"Assumpte";
#elif L==1
	"Betreff";
#elif L==2
	"Subject";
#elif L==3
	"Asunto";
#elif L==4
	"Objet";
#elif L==5
	"Asunto";	// Okoteve traduccin
#elif L==6
	"Oggetto";
#elif L==7
	"Z zastrzezeniem";
#elif L==8
	"Assunto";
#endif

const char *Txt_MSG_To =
#if   L==0
	"Per";
#elif L==1
	"An";
#elif L==2
	"To";
#elif L==3
	"Para";
#elif L==4
	"Pour";
#elif L==5
	"Para";	// Okoteve traduccin
#elif L==6
	"A";
#elif L==7
	"Aby";
#elif L==8
	"Para";
#endif

const char *Txt_MSG_Unopened =
#if   L==0
	"Sin abrir";	// Necessita traduccio
#elif L==1
	"Unge&ouml;ffnet";
#elif L==2
	"Unopened";
#elif L==3
	"Sin abrir";
#elif L==4
	"Non ouvert";
#elif L==5
	"Sin abrir";	// Okoteve traduccin
#elif L==6
	"Non aperto";
#elif L==7
	"Otwarte";
#elif L==8
	"Fechada";
#endif

const char *Txt_MSGS_containing_the_text =
#if   L==0
	"que contengan el texto";	// Necessita traduccio
#elif L==1
	"mit dem Text";
#elif L==2
	"containing the text";
#elif L==3
	"que contengan el texto";
#elif L==4
	"contenant le texte";
#elif L==5
	"que contengan el texto";	// Okoteve traduccin
#elif L==6
	"contenenti il testo";
#elif L==7
	"zawierajacy tekst";
#elif L==8
	"contendo o texto";
#endif

const char *Txt_MSGS_Deleted =
#if   L==0
	"Eliminados";	// Necessita traduccio
#elif L==1
	"Gel&ouml;scht";
#elif L==2
	"Deleted";
#elif L==3
	"Eliminados";
#elif L==4
	"Supprim&eacute;s";
#elif L==5
	"Eliminados";	// Okoteve traduccin
#elif L==6
	"Rimossi";
#elif L==7
	"Usuniete";
#elif L==8
	"Suprimidas";
#endif

const char *Txt_MSGS_Not_deleted =
#if   L==0
	"No eliminados";	// Necessita traduccio
#elif L==1
	"Nicht Gel&ouml;scht";
#elif L==2
	"Not deleted";
#elif L==3
	"No eliminados";
#elif L==4
	"Non supprim&eacute;s";
#elif L==5
	"No eliminados";	// Okoteve traduccin
#elif L==6
	"Non rimossi";
#elif L==7
	"Nie usuniete";
#elif L==8
	"N&atilde;o suprimidas";
#endif

const char *Txt_MSGS_Received =
#if   L==0
	"Rebuts";
#elif L==1
	"Empfangene";
#elif L==2
	"Received";
#elif L==3
	"Recibidos";
#elif L==4
	"Re&ccedil;us";
#elif L==5
	"Recibidos";	// Okoteve traduccin
#elif L==6
	"Ricevuti";
#elif L==7
	"Odebrane";
#elif L==8
	"Recebidas";
#endif

const char *Txt_MSGS_Sent =
#if   L==0
	"Enviats";
#elif L==1
	"Gesendete";
#elif L==2
	"Sent";
#elif L==3
	"Enviados";
#elif L==4
	"Envoy&eacute;s";
#elif L==5
	"Enviados";	// Okoteve traduccin
#elif L==6
	"Inviati";
#elif L==7
	"Wyslane";
#elif L==8
	"Enviadas";
#endif

const char *Txt_Multiple_enrollment =		// (to a type of group)
#if   L==0
	"&iquest;Adscripci&oacute;n m&uacute;ltiple?";	// Necessita traduccio
#elif L==1
	"Mehrere beitreten?";
#elif L==2
	"Multiple enrollment?";
#elif L==3
	"&iquest;Adscripci&oacute;n m&uacute;ltiple?";
#elif L==4
	"Inscription multiple?";
#elif L==5
	"&iquest;Adscripci&oacute;n m&uacute;ltiple?";	// Okoteve traduccin
#elif L==6
	"Registrazione multipla?";
#elif L==7
	"Rejestracja wielu element&oacute;w?";
#elif L==8
	"Inscri&ccedil;&atilde;o m&uacute;ltiplas?";
#endif

const char *Txt_My_courses =
#if   L==0
	"Els meus assignatures";
#elif L==1
	"Meine Kursen";
#elif L==2
	"My courses";
#elif L==3
	"Mis asignaturas";
#elif L==4
	"Mes mati&egrave;res";
#elif L==5
	"Che mbo'esyry";
#elif L==6
	"Miei corsi";
#elif L==7
	"Moje kurs&oacute;w";
#elif L==8
	"As minha disciplinas";
#endif

const char *Txt_My_documents =
#if   L==0
	"Els meus documents";
#elif L==1
	"Meine Dokumente";
#elif L==2
	"My documents";
#elif L==3
	"Mis documentos";
#elif L==4
	"Mes documents";
#elif L==5
	"Mis documentos";	// Okoteve traduccin
#elif L==6
	"I miei documenti";
#elif L==7
	"Moje dokumenty";
#elif L==8
	"Meus documentos";
#endif

const char *Txt_My_groups =
#if   L==0
	"Meus grups";
#elif L==1
	"Meine Gruppen";
#elif L==2
	"My groups";
#elif L==3
	"Mis grupos";
#elif L==4
	"Mes groupes";
#elif L==5
	"Mis grupos";	// Okoteve traduccin
#elif L==6
	"Miei gruppi";
#elif L==7
	"Moje grupy";
#elif L==8
	"Meus grupos";
#endif

const char *Txt_My_views =
#if   L==0
	"Els meus accessos";
#elif L==1
	"Meine Ansichten";
#elif L==2
	"My views";
#elif L==3
	"Mis accesos";
#elif L==4
	"Mes vues";
#elif L==5
	"Mis accesos";	// Okoteve traduccin
#elif L==6
	"Miei visualizzazioni";
#elif L==7
	"Moje ods&lstrok;on";
#elif L==8
	"Meus acessos";
#endif

const char *Txt_Name =
#if   L==0
	"Nom";
#elif L==1
	"Name";
#elif L==2
	"Name";
#elif L==3
	"Nombre";
#elif L==4
	"Nom";
#elif L==5
	"T&eacute;ra";
#elif L==6
	"Nome";
#elif L==7
	"Nazwa";
#elif L==8
	"Nome";
#endif

const char *Txt_New_account_created =
#if   L==0
	"Nou compte creat.";
#elif L==1
	"Neues Konto erstellt.";
#elif L==2
	"New account created.";
#elif L==3
	"Nueva cuenta creada.";
#elif L==4
	"Nouveau compte cr&eacute;&eacute;.";
#elif L==5
	"Nueva cuenta creada.";	// Okoteve traduccin
#elif L==6
	"Nuovo account creato.";
#elif L==7
	"Nowe konto utworzone.";
#elif L==8
	"Nova conta criada.";
#endif

const char *Txt_New_assignment =
#if   L==0
	"Nova activitat";
#elif L==1
	"Neue Aufgabe";
#elif L==2
	"New assignment";
#elif L==3
	"Nueva actividad";
#elif L==4
	"Nouvelle activit&eacute;";
#elif L==5
	"Nueva actividad";	// Okoteve traduccin
#elif L==6
	"Nuova attivit&agrave;";
#elif L==7
	"Nowe zadania";
#elif L==8
	"Nova atividade";
#endif

const char *Txt_New_announcement =
#if   L==0
	"Nou anunci";
#elif L==1
	"Neue Bekanntmachung";
#elif L==2
	"New announcement";
#elif L==3
	"Nuevo anuncio";
#elif L==4
	"Nouvel annonce";
#elif L==5
	"Nuevo anuncio";	// Okoteve traduccin
#elif L==6
	"Nuovo annuncio";
#elif L==7
	"Nowe og&lstrok;oszenia";
#elif L==8
	"Novo an&uacute;ncio";
#endif

const char *Txt_New_announcement_of_exam =
#if   L==0
	"Nova convocat&ograve;ria d'examen";
#elif L==1
	"Neuer Aufruf f&uuml;r Pr&uuml;fung";
#elif L==2
	"New announcement of exam";
#elif L==3
	"Nueva convocatoria de examen";
#elif L==4
	"Nouvelle convocation &agrave; un examen";
#elif L==5
	"Nueva convocatoria de examen";	// Okoteve traduccin
#elif L==6
	"Nuova sessione d'esame";
#elif L==7
	"Nowe ogloszenie egzaminu";
#elif L==8
	"Nova chamada para exame";
#endif

const char *Txt_New_banner =
#if   L==0
	"Nuevo banner";		// Necessita traduccio
#elif L==1
	"Neue Werbebanner";
#elif L==2
	"New banner";
#elif L==3
	"Nuevo banner";
#elif L==4
	"Nouvelle banni&eacute;re";
#elif L==5
	"Nuevo banner";		// Okoteve traduccin
#elif L==6
	"Nuovo banner";
#elif L==7
	"Nowy banner";
#elif L==8
	"Novo banner";
#endif

const char *Txt_New_centre_of_INSTITUTION_X =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"Nou centre de %s";
#elif L==1
	"Neues Lehrinstitut von %s";
#elif L==2
	"New centre of %s";
#elif L==3
	"Nuevo centro de %s";
#elif L==4
	"Nouveau centre de %s";
#elif L==5
	"Nuevo centro de %s";	// Okoteve traduccin
#elif L==6
	"Nuovo centro di %s";
#elif L==7
	"Nowe centrum %s";
#elif L==8
	"Novo centro de %s";
#endif

const char *Txt_New_country =
#if   L==0
	"Nuevo pa&iacute;s";	// Necessita traduccio
#elif L==1
	"Neues Land";
#elif L==2
	"New country";
#elif L==3
	"Nuevo pa&iacute;s";
#elif L==4
	"Nouveau pays";
#elif L==5
	"Nuevo pa&iacute;s";	// Okoteve traduccin
#elif L==6
	"Nuovo paese";
#elif L==7
	"Nowy kraj";
#elif L==8
	"Novo pa&iacute;s";
#endif

const char *Txt_New_course_of_DEGREE_X =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"Nova assignatura de %s";
#elif L==1
	"Neue Kurs der %s";
#elif L==2
	"New course of %s";
#elif L==3
	"Nueva asignatura de %s";
#elif L==4
	"Nouvelle mati&egrave;re de %s";
#elif L==5
	"Nueva asignatura de %s";	// Okoteve traduccin
#elif L==6
	"Nuovo corso di %s";
#elif L==7
	"Nowy kurs %s";
#elif L==8
	"Nova disciplina de %s";
#endif

const char *Txt_New_degree_of_CENTRE_X =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"Nova titulaci&oacute; de %s";
#elif L==1
	"Neuer Studiengang von %s";
#elif L==2
	"New degree of %s";
#elif L==3
	"Nueva titulaci&oacute;n de %s";
#elif L==4
	"Nouvel &eacute;tude de %s";
#elif L==5
	"Nueva titulaci&oacute;n de %s";	// Okoteve traduccin
#elif L==6
	"Nuova laurea di %s";
#elif L==7
	"Nowy stopien %s";
#elif L==8
	"Nova titula&ccedil;&atilde;o de %s";
#endif

const char *Txt_New_department =
#if   L==0
	"Nuevo departamento";	// Necessita traduccio
#elif L==1
	"Neue Abteilung";
#elif L==2
	"New department";
#elif L==3
	"Nuevo departamento";
#elif L==4
	"Nouveau d&eacute;partement";
#elif L==5
	"Nuevo departamento";	// Okoteve traduccin
#elif L==6
	"Nuovo dipartimento";
#elif L==7
	"Nowy dzia&lstrok;";
#elif L==8
	"Novo departamento";
#endif

const char *Txt_New_email =
#if   L==0
	"Nou correu electr&grave;nic";
#elif L==1
	"Neues E-Mail";
#elif L==2
	"New e-mail";
#elif L==3
	"Nuevo correo electr&oacute;nico";
#elif L==4
	"Nouveau courrier &eacute;lectronique";
#elif L==5
	"Nuevo correo electr&oacute;nico";	// Okoteve traduccin
#elif L==6
	"Nuovo e-mail";
#elif L==7
	"Nowy e-mail";
#elif L==8
	"Novo e-mail";
#endif

const char *Txt_New_event =
#if   L==0
	"Nou esdeveniment";
#elif L==1
	"Neue Ereignis";
#elif L==2
	"New event";
#elif L==3
	"Nuevo evento";
#elif L==4
	"Nouveau &eacute;v&eacute;nement";
#elif L==5
	"Nuevo evento";	// Okoteve traduccin
#elif L==6
	"Nuovo evento";
#elif L==7
	"Nowe wydarzenie";
#elif L==8
	"Novo evento";
#endif

const char *Txt_New_FILE_OR_FOLDER =
#if   L==0
	"Nou";
#elif L==1
	"Neue";
#elif L==2
	"New";
#elif L==3
	"Nuevo";
#elif L==4
	"Nouveau";
#elif L==5
	"Nuevo";	// Okoteve traduccin
#elif L==6
	"Nuovo";
#elif L==7
	"Nowe";
#elif L==8
	"Novo";
#endif

const char *Txt_New_group =
#if   L==0
	"Nuevo grupo";	// Necessita traduccio
#elif L==1
	"Neue Gruppe";
#elif L==2
	"New group";
#elif L==3
	"Nuevo grupo";
#elif L==4
	"Nouveau groupe";
#elif L==5
	"Nuevo grupo";	// Okoteve traduccin
#elif L==6
	"Nuovo gruppo";
#elif L==7
	"Nowa grupa";
#elif L==8
	"Novo grupo";
#endif

const char *Txt_New_holiday =
#if   L==0
	"Nueva festividad";	// Necessita traduccio
#elif L==1
	"Neue Feiertag";
#elif L==2
	"New holiday";
#elif L==3
	"Nueva festividad";
#elif L==4
	"Nouvelle f&ecirc;te";
#elif L==5
	"Nueva festividad";	// Okoteve traduccin
#elif L==6
	"Nuova festivit&agrave;";
#elif L==7
	"Nowe swieto";
#elif L==8
	"Nova f&eacute;ria";
#endif

const char *Txt_New_institution_of_COUNTRY_X =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"Nova instituci&oacute;n de %s";
#elif L==1
	"Neue Hochschule von %s";
#elif L==2
	"New institution of %s";
#elif L==3
	"Nueva instituci&oacute;n de %s";
#elif L==4
	"Nouvel &eacute;tablissement de %s";
#elif L==5
	"Nueva instituci&oacute;n de %s";	// Okoteve traduccin
#elif L==6
	"Nuova istituzione di %s";
#elif L==7
	"Nowa instytucja %s";
#elif L==8
	"Nova institu&ccedil;&atilde;o de %s";
#endif

const char *Txt_New_link =
#if   L==0
	"Nuevo enlace";		// Necessita traduccio
#elif L==1
	"Neue Link";
#elif L==2
	"New link";
#elif L==3
	"Nuevo enlace";
#elif L==4
	"Nouveau lien";
#elif L==5
	"Nuevo enlace";		// Okoteve traduccin
#elif L==6
	"Nuovo link";
#elif L==7
	"Nowy link";
#elif L==8
	"Nova liga&ccedil;&atilde;o";
#endif

const char *Txt_New_mail_domain =
#if   L==0
	"Nuevo dominio de correo";	// Necessita traduccio
#elif L==1
	"Neue Mail-Dom&auml;ne";
#elif L==2
	"New mail domain";
#elif L==3
	"Nuevo dominio de correo";
#elif L==4
	"Nouveau domaine de messagerie";
#elif L==5
	"Nuevo dominio de correo";	// Okoteve traduccin
#elif L==6
	"Nuovo campo mail";
#elif L==7
	"Nowe domeny mail";
#elif L==8
	"Novo dom&iacute;nio de email";
#endif

const char *Txt_New_message =
#if   L==0
	"Nuevo mensaje";	// Necessita traduccio
#elif L==1
	"Neue Nachricht";
#elif L==2
	"New message";
#elif L==3
	"Nuevo mensaje";
#elif L==4
	"Nouveau message";
#elif L==5
	"Nuevo mensaje";	// Okoteve traduccin
#elif L==6
	"Nuovo messaggio";
#elif L==7
	"Nowa wiadomosc";
#elif L==8
	"Nova mensagem";
#endif

const char *Txt_New_nickname =
#if   L==0
	"Nou sobrenom";
#elif L==1
	"Neues Benutzername";
#elif L==2
	"New nickname";
#elif L==3
	"Nuevo apodo";
#elif L==4
	"Nouveau surnom";
#elif L==5
	"Nuevo apodo";	// Okoteve traduccin
#elif L==6
	"Nuovo nome utente";
#elif L==7
	"Nowy nick";
#elif L==8
	"Nova alcunha";
#endif

const char *Txt_New_notice =
#if   L==0
	"Nou av&iacute;s";
#elif L==1
	"Neue Ank&uuml;ndigung";
#elif L==2
	"New notice";
#elif L==3
	"Nuevo aviso";
#elif L==4
	"Nouvel avis";
#elif L==5
	"Nuevo aviso";	// Okoteve traduccin
#elif L==6
	"Nuovo avviso";
#elif L==7
	"Nowe obwieszczenie";
#elif L==8
	"Novo aviso";
#endif

const char *Txt_New_on_PLATFORM_Sign_up =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"No t&eacute; compte a %s? Registra't ara";
#elif L==1
	"Neu bei %s? Registriere Dich!";
#elif L==2
	"New on %s? Sign up";
#elif L==3
	"&iquest;No tiene cuenta en %s? Reg&iacute;strese";
#elif L==4
	"Nouveau sur %s? Inscrivez-vous";
#elif L==5
	"&iquest;No tiene cuenta en %s? Reg&iacute;strese";	// Okoteve traduccin
#elif L==6
	"Nuovo su %s? Iscriviti";
#elif L==7
	"Pierwszy raz na %s? Zarejestruj si&eogon;";
#elif L==8
	"Novo no %s? Inscreva-se";
#endif

const char *Txt_New_password =
#if   L==0
	"Nova contrasenya";
#elif L==1
	"Neues Passwort";
#elif L==2
	"New password";
#elif L==3
	"Nueva contrase&ntilde;a";
#elif L==4
	"Nouveau mot de passe";
#elif L==5
	"Nueva contrase&ntilde;a";	// Okoteve traduccin
#elif L==6
	"Nuova password";
#elif L==7
	"Nowe has&lstrok;o";
#elif L==8
	"Nova senha";
#endif

const char *Txt_New_place =
#if   L==0
	"Nuevo lugar";	// Necessita traduccio
#elif L==1
	"Neue Standort";
#elif L==2
	"New place";
#elif L==3
	"Nuevo lugar";
#elif L==4
	"Nouvel emplacement";
#elif L==5
	"Nuevo lugar";	// Okoteve traduccin
#elif L==6
	"Nuova localit&grave;";
#elif L==7
	"Nowe miejsce";
#elif L==8
	"Nova localiza&ccedil;&atilde;o";
#endif

const char *Txt_New_plugin =
#if   L==0
	"Nuevo complemento";	// Necessita traduccio
#elif L==1
	"Neue Plugin";
#elif L==2
	"New plugin";
#elif L==3
	"Nuevo complemento";
#elif L==4
	"Nouveau plugin";
#elif L==5
	"Nuevo complemento";	// Okoteve traduccin
#elif L==6
	"Nuovo plugin";
#elif L==7
	"Nowy plugin";
#elif L==8
	"Novo plugin";
#endif

const char *Txt_New_question =
#if   L==0
	"Pregunta nova";
#elif L==1
	"Neuer Frage";
#elif L==2
	"New question";
#elif L==3
	"Pregunta nueva";
#elif L==4
	"Nouvelle question";
#elif L==5
	"Pregunta nueva";	// Okoteve traduccin
#elif L==6
	"Nuova domanda";
#elif L==7
	"Nowe pytanie";
#elif L==8
	"Nova quest&atilde;o";
#endif

const char *Txt_New_record_field =
#if   L==0
	"Nuevo campo de ficha";		// Necessita traduccio
#elif L==1
	"Neues Datenfeld";
#elif L==2
	"New record field";
#elif L==3
	"Nuevo campo de ficha";
#elif L==4
	"Nouveau champ de carte";
#elif L==5
	"Nuevo campo de ficha";		// Okoteve traduccin
#elif L==6
	"Nuovo campo di scheda";
#elif L==7
	"Nowy rekord pola";
#elif L==8
	"Novo campo de cart&atilde;o";
#endif

const char *Txt_New_survey =
#if   L==0
	"Nova enquesta";
#elif L==1
	"Neuer Umfrage";
#elif L==2
	"New survey";
#elif L==3
	"Nueva encuesta";
#elif L==4
	"Nouveau sondage";
#elif L==5
	"Nueva encuesta";	// Okoteve traduccin
#elif L==6
	"Nuovo sondaggio";
#elif L==7
	"Nowe badania";
#elif L==8
	"Novo inqu&eacute;rito";
#endif

const char *Txt_new_tag =		// Tag used in test
#if   L==0
	"nuevo descriptor";	// Necessita traduccio
#elif L==1
	"neuen Tag";
#elif L==2
	"new tag";
#elif L==3
	"nuevo descriptor";
#elif L==4
	"nouveau descripteur";
#elif L==5
	"nuevo descriptor";	// Okoteve traduccin
#elif L==6
	"nuova etichetta";
#elif L==7
	"nowy tag";
#elif L==8
	"novo descritor";
#endif

const char *Txt_New_thread =
#if   L==0
	"Nueva discusi&oacute;n";	// Necessita traduccio
#elif L==1
	"Neuen Thread";
#elif L==2
	"New thread";
#elif L==3
	"Nueva discusi&oacute;n";
#elif L==4
	"Nouveau fil";
#elif L==5
	"Nueva discusi&oacute;n";	// Okoteve traduccin
#elif L==6
	"Nuova discussione";
#elif L==7
	"Nowy temat";
#elif L==8
	"Novo thread";
#endif

const char *Txt_New_type_of_degree =
#if   L==0
	"Nuevo tipo de titulaci&oacute;n";	// Necessita traduccio
#elif L==1
	"Neue Abschlussart";
#elif L==2
	"New type of degree";
#elif L==3
	"Nuevo tipo de titulaci&oacute;n";
#elif L==4
	"Nouveau type d'&eacute;tude";
#elif L==5
	"Nuevo tipo de titulaci&oacute;n";	// Okoteve traduccin
#elif L==6
	"Nuovo tipo di laurea";
#elif L==7
	"Nowy rodzaj stopnia";
#elif L==8
	"Novo tipo de titula&ccedil;&atilde;o";
#endif

const char *Txt_New_type_of_group =
#if   L==0
	"Nuevo tipo de grupo";		// Necessita traduccio
#elif L==1
	"Neue Art der Gruppe";
#elif L==2
	"New type of group";
#elif L==3
	"Nuevo tipo de grupo";
#elif L==4
	"Nouveau type de groupe";
#elif L==5
	"Nuevo tipo de grupo";		// Okoteve traduccin
#elif L==6
	"Nuovo tipo di gruppo";
#elif L==7
	"Nowy typ grupy";
#elif L==8
	"Novo tipo de grupo";
#endif

const char *Txt_NEW_YEAR_GREETING =	// Warning: it is very important to include %u in the following sentences
#if   L==0
	"&iexcl;Feliz %u!";	// Necessita traduccio
#elif L==1
	"Happy %u!";
#elif L==2
	"Happy %u!";
#elif L==3
	"&iexcl;Feliz %u!";
#elif L==4
	"Happy %u!";
#elif L==5
	"&iexcl;Feliz %u!";	// Okoteve traduccin
#elif L==6
	"Felice %u!";
#elif L==7
	"Happy %u!";
#elif L==8
	"Feliz %u!";
#endif

const char *Txt_nick_email_or_ID =
#if   L==0
	"@sobrenom, correu o ID (DNI/c&egrave;dula)";
#elif L==1
	"@Benutzername, E-mail oder Ausweis-Nr.";
#elif L==2
	"@nick, e-mail or ID";
#elif L==3
	"@apodo, correo o ID (DNI/c&eacute;dula)";
#elif L==4
	"@surnom, courrier ou num&eacute;ro d'identit&eacute;";
#elif L==5
	"@apodo, correo o ID (DNI/c&eacute;dula)";	// Okoteve traduccin
#elif L==6
	"@nome-utente, e-mail o numero carta d'identit&agrave;";
#elif L==7
	"@pseudonim, e-mail lub ID";
#elif L==8
	"@alcunha, e-mail ou n&ordm; identif.";
#endif

const char *Txt_nicks_emails_or_IDs_separated_by_commas =
#if   L==0
	"@apodos, correos o ID (DNI/c&eacute;dulas) separados por comas";	// Necessita traduccio
#elif L==1
	"@Benutzernamen, E-mails oder Ausweis-Nr. durch Kommas getrennt";
#elif L==2
	"@nicks, e-mails or IDs separated by commas";
#elif L==3
	"@apodos, correos o ID (DNI/c&eacute;dulas) separados por comas";
#elif L==4
	"@surnoms, courriers ou num&eacute;ros d'identit&eacute; s&eacute;par&eacute;s par des virgules";
#elif L==5
	"@apodos, correos o ID (DNI/c&eacute;dulas) separados por comas";	// Okoteve traduccin
#elif L==6
	"@nomi-utenti, e-mail o numero carte d'identit&agrave; separati da virgole";
#elif L==7
	"@nicks, e-mails or IDs separated by commas";	// Potrzebujesz tlumaczenie
#elif L==8
	"@alcunhas, e-mails ou n&ordm;s. identif. separados por v&iacute;rgulas";
#endif

const char *Txt_Nickname =
#if   L==0
	"Sobrenom";
#elif L==1
	"Benutzername";
#elif L==2
	"Nickname";
#elif L==3
	"Apodo";
#elif L==4
	"Surnom";
#elif L==5
	"Apodo";	// Okoteve traduccin
#elif L==6
	"Nome utente";
#elif L==7
	"Pseudonim";
#elif L==8
	"Alcunha";
#endif

const char *Txt_Nickname_X_removed =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"Sobrenom <strong>@%s</strong> eliminat.";
#elif L==1
	"Benutzername <strong>@%s</strong> entfernt.";
#elif L==2
	"Nickname <strong>@%s</strong> removed.";
#elif L==3
	"Apodo <strong>@%s</strong> eliminado.";
#elif L==4
	"Surnom <strong>@%s</strong> supprim&eacute;.";
#elif L==5
	"Apodo <strong>@%s</strong> eliminado.";	// Okoteve traduccin
#elif L==6
	"Nome utente <strong>@%s</strong> rimosso.";
#elif L==7
	"Pseudonim <strong>@%s</strong> usuniete.";
#elif L==8
	"Alcunha <strong>@%s</strong> removido.";
#endif

const char *Txt_NO =
#if   L==0
	"NO";
#elif L==1
	"NEIN";
#elif L==2
	"NO";
#elif L==3
	"NO";
#elif L==4
	"NON";
#elif L==5
	"NAH&aACUTE;NIRI";
#elif L==6
	"NO";
#elif L==7
	"NO";
#elif L==8
	"N&Atilde;O";
#endif

const char *Txt_No =
#if   L==0
	"No";
#elif L==1
	"Nein";
#elif L==2
	"No";
#elif L==3
	"No";
#elif L==4
	"Non";
#elif L==5
	"Nah&aacute;niri";
#elif L==6
	"No";
#elif L==7
	"No";
#elif L==8
	"N&atilde;o";
#endif

const char *Txt_No_announcements_of_exams_of_X =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"No hay convocatorias de examen de <strong>%s</strong>.";	// Necessita traduccio
#elif L==1
	"Keine Aufrufe f&uuml;r Pr&uuml;fung von <strong>%s</strong>.";
#elif L==2
	"No announcements of exams of <strong>%s</strong>.";
#elif L==3
	"No hay convocatorias de examen de <strong>%s</strong>.";
#elif L==4
	"Aucune convocation &agrave; un examen de <strong>%s</strong>.";
#elif L==5
	"No hay convocatorias de examen de <strong>%s</strong>.";	// Okoteve traduccin
#elif L==6
	"Nessun appello d'esame di <strong>%s</strong>.";
#elif L==7
	"Brak ogloszen egzamin&oacute;w <strong>%s</strong>.";
#elif L==8
	"N&atilde;o chamadas para exame de <strong>%s</strong>.";
#endif

const char *Txt_no_course_of_origin =	// Means: "message sent from any course"
#if   L==0
	"sin asignatura de origen";	// Necessita traduccio
#elif L==1
	"no course of origin";		// Need bersetzung
#elif L==2
	"no course of origin";
#elif L==3
	"sin asignatura de origen";
#elif L==4
	"no course of origin";		// Besoin de traduction
#elif L==5
	"sin asignatura de origen";	// Okoteve traduccin
#elif L==6
	"senza corso di origine";
#elif L==7
	"Oczywiscie nie pochodzenia";
#elif L==8
	"no course of origin";		// Necessita de traduo
#endif

const char *Txt_No_COURSE_SELECTED =
#if   L==0
	"Cap assignatura";
#elif L==1
	"Keine Kurs";
#elif L==2
	"No course";
#elif L==3
	"Ninguna asignatura";
#elif L==4
	"Aucune mati&egrave;re";
#elif L==5
	"Ninguna asignatura";	// Okoteve traduccin
#elif L==6
	"Nessun corso";
#elif L==7
	"No oczywiscie";
#elif L==8
	"Nenhuma disciplina";
#endif

const char *Txt_No_centres_have_been_created_in_this_institution =
#if   L==0
	"No se han creado centros en esta instituci&oacute;n.";	// Necessita traduccio
#elif L==1
	"No centres have been created in this institution.";	// Need bersetzung
#elif L==2
	"No centres have been created in this institution.";
#elif L==3
	"No se han creado centros en esta instituci&oacute;n.";
#elif L==4
	"No centres have been created in this institution.";	// Besoin de traduction
#elif L==5
	"No se han creado centros en esta instituci&oacute;n.";	// Okoteve traduccin
#elif L==6
	"No centres have been created in this institution.";	// Bisogno di traduzione
#elif L==7
	"No centres have been created in this institution.";	// Potrzebujesz tlumaczenie
#elif L==8
	"No centres have been created in this institution.";	// Necessita de traduo
#endif

const char *Txt_No_countries_have_been_created =
#if   L==0
	"No se han creado pa&iacute;ses.";	// Necessita traduccio
#elif L==1
	"No countries have been created.";	// Need bersetzung
#elif L==2
	"No countries have been created.";
#elif L==3
	"No se han creado pa&iacute;ses.";
#elif L==4
	"No countries have been created.";	// Besoin de traduction
#elif L==5
	"No se han creado pa&iacute;ses.";	// Okoteve traduccin
#elif L==6
	"No countries have been created.";	// Bisogno di traduzione
#elif L==7
	"No countries have been created.";	// Potrzebujesz tlumaczenie
#elif L==8
	"No countries have been created.";	// Necessita de traduo
#endif

const char *Txt_No_courses_have_been_created_in_this_degree =
#if   L==0
	"No se han creado asignaturas en esta titulaci&oacute;n.";	// Necessita traduccio
#elif L==1
	"No courses have been created in this degree.";			// Need bersetzung
#elif L==2
	"No courses have been created in this degree.";
#elif L==3
	"No se han creado asignaturas en esta titulaci&oacute;n.";
#elif L==4
	"No courses have been created in this degree.";			// Besoin de traduction
#elif L==5
	"No se han creado asignaturas en esta titulaci&oacute;n.";	// Okoteve traduccin
#elif L==6
	"No courses have been created in this degree.";			// Bisogno di traduzione
#elif L==7
	"No courses have been created in this degree.";			// Potrzebujesz tlumaczenie
#elif L==8
	"No courses have been created in this degree.";			// Necessita de traduo
#endif

const char *Txt_No_degrees_have_been_created_in_this_centre =
#if   L==0
	"No se han creado titulaciones en este centro.";	// Necessita traduccio
#elif L==1
	"No degrees have been created in this centre.";		// Need bersetzung
#elif L==2
	"No degrees have been created in this centre.";
#elif L==3
	"No se han creado titulaciones en este centro.";
#elif L==4
	"No degrees have been created in this centre.";		// Besoin de traduction
#elif L==5
	"No se han creado titulaciones en este centro.";	// Okoteve traduccin
#elif L==6
	"No degrees have been created in this centre.";		// Bisogno di traduzione
#elif L==7
	"No degrees have been created in this centre.";		// Potrzebujesz tlumaczenie
#elif L==8
	"No degrees have been created in this centre.";		// Necessita de traduo
#endif

const char *Txt_No_enrollment_requests =
#if   L==0
	"No hay peticiones de inscripci&oacute;n.";	// Necessita traduccio
#elif L==1
	"No enrollment requests.";			// Need bersetzung
#elif L==2
	"No enrollment requests.";
#elif L==3
	"No hay peticiones de inscripci&oacute;n.";
#elif L==4
	"No enrollment requests.";			// Besoin de traduction
#elif L==5
	"No hay peticiones de inscripci&oacute;n.";	// Okoteve traduccin
#elif L==6
	"No enrollment requests.";			// Bisogno di traduzione
#elif L==7
	"No enrollment requests.";			// Potrzebujesz tlumaczenie
#elif L==8
	"No enrollment requests.";			// Necessita de traduo
#endif

const char *Txt_No_file_index_html_index_htm_found_within_the_ZIP_file =
#if   L==0
	"No se ha encontrado un archivo <em>index.html</em> / <em>index.htm</em>"
	" dentro del archivo <em>ZIP</em>.";		// Necessita traduccio
#elif L==1
	"No file <em>index.html</em> / <em>index.htm</em> found"
	" within the <em>ZIP</em> file.";		// Need bersetzung
#elif L==2
	"No file <em>index.html</em> / <em>index.htm</em> found"
	" within the <em>ZIP</em> file.";
#elif L==3
	"No se ha encontrado un archivo <em>index.html</em> / <em>index.htm</em>"
	" dentro del archivo <em>ZIP</em>.";
#elif L==4
	"No file <em>index.html</em> / <em>index.htm</em> found"
	" within the <em>ZIP</em> file.";		// Besoin de traduction
#elif L==5
	"No se ha encontrado un archivo <em>index.html</em> / <em>index.htm</em>"
	" dentro del archivo <em>ZIP</em>.";		// Okoteve traduccin
#elif L==6
	"Nessun documento <em>index.html</em> / <em>index.htm</em> trovato"
	" nel file <em>ZIP</em>.";
#elif L==7
	"Nr pliku <em>indeks.html</em> / <em>indeks.htm</em> znalezionych"
	" w <em>ZIP</em> plik.";
#elif L==8
	"No file <em>index.html</em> / <em>index.htm</em> found"
	" within the <em>ZIP</em> file.";		// Necessita de traduo
#endif

const char *Txt_No_groups_have_been_created_in_the_course_X =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"No se han creado grupos en la asignatura <strong>%s</strong>.";	// Necessita traduccio
#elif L==1
	"No groups have been created in the course <strong>%s</strong>.";	// Need bersetzung
#elif L==2
	"No groups have been created in the course <strong>%s</strong>.";
#elif L==3
	"No se han creado grupos en la asignatura <strong>%s</strong>.";
#elif L==4
	"No groups have been created in the course <strong>%s</strong>.";	// Besoin de traduction
#elif L==5
	"No se han creado grupos en la asignatura <strong>%s</strong>.";	// Okoteve traduccin
#elif L==6
	"Nessun gruppo &egrave; stato creato nel corso <strong>%s</strong>.";
#elif L==7
	"Nr grupy zostaly utworzone w ramach <strong>%s</strong>.";
#elif L==8
	"No groups have been created in the course <strong>%s</strong>.";	// Necessita de traduo
#endif

const char *Txt_No_groups_have_been_created_in_the_course_X_Therefore_ =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"No se han creado grupos en la asignatura <strong>%s</strong>."
	" Por tanto, los usuarios especificados ser&aacute;n inscritos/eliminados en/de la asignatura.";	// Necessita traduccio
#elif L==1
	"No groups have been created in the course <strong>%s</strong>."
	"Therefore, the specified users will be enrolled/removed in/from the course.";				// Need bersetzung
#elif L==2
	"No groups have been created in the course <strong>%s</strong>."
	"Therefore, the specified users will be enrolled/removed in/from the course.";
#elif L==3
	"No se han creado grupos en la asignatura <strong>%s</strong>."
	" Por tanto, los usuarios especificados ser&aacute;n inscritos/eliminados en/de la asignatura.";
#elif L==4
	"No groups have been created in the course <strong>%s</strong>."
	"Therefore, the specified users will be enrolled/removed in/from the course.";				// Besoin de traduction
#elif L==5
	"No se han creado grupos en la asignatura <strong>%s</strong>."
	" Por tanto, los usuarios especificados ser&aacute;n inscritos/eliminados en/de la asignatura.";	// Okoteve traduccin
#elif L==6
	"Nessun gruppo &egrave; stato creato nel corso <strong>%s</strong>."
	"Dunque, gli utenti specificati saranno registrati/rimossi in/dal corso.";
#elif L==7
	"Nr grupy zostaly utworzone w ramach <strong>%s</strong>."
	"Dlatego okreslonych uzytkownik&oacute;w zostanie zarejestrowany / usuniete w / z kursu.";
#elif L==8
	"No groups have been created in the course <strong>%s</strong>."
	"Therefore, the specified users will be enrolled/removed in/from the course.";				// Necessita de traduo
#endif

const char *Txt_No_INDEX =	// Short version of "Number" (as an index)...
#if   L==0
	"N&ordm;";	// Necessita traduccio
#elif L==1
	"Anzahl";
#elif L==2
	"#";
#elif L==3
	"N&ordm;";
#elif L==4
	"Nombre";
#elif L==5
	"Papanga";
#elif L==6
	"Numero";
#elif L==7
	"Numer";
#elif L==8
	"N&ordm;";
#endif

const char *Txt_No_information_available =
#if   L==0
	"No hay informaci&oacute;n disponible.";	// Necessita traduccio
#elif L==1
	"Keine Informationen verf&uuml;gbar.";
#elif L==2
	"No information available.";
#elif L==3
	"No hay informaci&oacute;n disponible.";
#elif L==4
	"Aucune information disponible.";
#elif L==5
	"No hay informaci&oacute;n disponible.";	// Okoteve traduccin
#elif L==6
	"Nessuna informazione disponibile.";
#elif L==7
	"Brak informacji.";
#elif L==8
	"N&atilde;o h&aacute; informa&ccedil;&atilde;o dispon&iacute;vel.";
#endif

const char *Txt_No_institutions_have_been_created_in_this_country =
#if   L==0
	"No se han creado instituciones en este pa&iacute;s.";	// Necessita traduccio
#elif L==1
	"No institutions have been created in this country.";	// Need bersetzung
#elif L==2
	"No institutions have been created in this country.";
#elif L==3
	"No se han creado instituciones en este pa&iacute;s.";
#elif L==4
	"No institutions have been created in this country.";	// Besoin de traduction
#elif L==5
	"No se han creado instituciones en este pa&iacute;s.";	// Okoteve traduccin
#elif L==6
	"No institutions have been created in this country.";	// Bisogno di traduzione
#elif L==7
	"No institutions have been created in this country.";	// Potrzebujesz tlumaczenie
#elif L==8
	"No institutions have been created in this country.";	// Necessita de traduo
#endif

const char *Txt_No_BR_msgs =
#if   L==0
	"N&ordm;<br />mens.";	// Necessita traduccio
#elif L==1
	"Anzahl<br />Nachr.";
#elif L==2
	"No.<br />msgs.";
#elif L==3
	"N&ordm;<br />mens.";
#elif L==4
	"Nombre<br />msgs.";
#elif L==5
	"N&ordm;<br />mens.";	// Okoteve traduccin
#elif L==6
	"Num.<br />msg.";
#elif L==7
	"Numer<br />msgs.";
#elif L==8
	"N&ordm;<br />mens.";
#endif

const char *Txt_No_new_posts =
#if   L==0
	"No hay mensajes nuevos";	// Necessita traduccio
#elif L==1
	"Keine neuen Nachrichten";
#elif L==2
	"No new posts";
#elif L==3
	"No hay mensajes nuevos";
#elif L==4
	"Pas de nouveaux messages";
#elif L==5
	"No hay mensajes nuevos";	// Okoteve traduccin
#elif L==6
	"Nessun nuovo post";
#elif L==7
	"Brak nowych post&oacute;w";
#elif L==8
	"Sem posts novos";
#endif

const char *Txt_No_notices =
#if   L==0
	"No hay avisos.";	// Necessita traduccio
#elif L==1
	"Keine Ank&uuml;ndigungen.";
#elif L==2
	"No notices.";
#elif L==3
	"No hay avisos.";
#elif L==4
	"Il n'y a pas d'avis.";
#elif L==5
	"No hay avisos.";	// Okoteve traduccin
#elif L==6
	"Non ci sono avvisi.";
#elif L==7
	"Brak ogloszen.";
#elif L==8
	"N&atilde;o h&aacute; avisos.";
#endif

const char *Txt_No_notifications_for_you =
#if   L==0
	"No tiene notificaciones.";	// Necessita traduccio
#elif L==1
	"Keine Anmeldungen f&uuml;r Sie.";
#elif L==2
	"No notifications for you.";
#elif L==3
	"No tiene notificaciones.";
#elif L==4
	"Il n'y a pas de notifications pour vous.";
#elif L==5
	"No tiene notificaciones.";	// Okoteve traduccin
#elif L==6
	"Non ci sono notifiche per te.";
#elif L==7
	"Brak powiadomienia dla Ciebie.";
#elif L==8
	"N&atilde;o h&aacute; notifica&ccedil;&otilde;es para voc&ecirc;.";
#endif

const char *Txt_No_of_files =
#if   L==0
	"Nombre d'arxius";
#elif L==1
	"Anzahl der Dateien";
#elif L==2
	"Number of files";
#elif L==3
	"N&ordm; de archivos";
#elif L==4
	"Nombre de fichiers";
#elif L==5
	"N&ordm; de archivos";	// Okoteve traduccin
#elif L==6
	"Numero di file";
#elif L==7
	"Liczba pliki";
#elif L==8
	"N&ordm; de arquivos";
#endif

const char *Txt_No_of_files_in_common_zones =
#if   L==0
	"Nombre d'arxius compartits";
#elif L==1
	"Anzahl der freigegebene Dateien";
#elif L==2
	"Number of shared files";
#elif L==3
	"N&ordm; de archivos compartidos";
#elif L==4
	"Nombre de fichiers partag&eacute;s";
#elif L==5
	"N&ordm; de archivos compartidos";		// Okoteve traduccin
#elif L==6
	"Numero di file condivisi";
#elif L==7
	"Liczba pliki udost&eogon;pnione";
#elif L==8
	"N&ordm; de arquivos compartilhados";
#endif

const char *Txt_No_of_files_in_documents_zones =
#if   L==0
	"N&ordm; de archivos en zonas de documentos";	// Necessita traduccio
#elif L==1
	"Anzahl der Dateien im Dokumente-Bereich";
#elif L==2
	"Number of files in documents area";
#elif L==3
	"N&ordm; de archivos en zonas de documentos";
#elif L==4
	"Nombre de fichiers dans les zones de documents";
#elif L==5
	"N&ordm; de archivos en zonas de documentos";	// Okoteve traduccin
#elif L==6
	"Numero di file nell'area documenti";
#elif L==7
	"Number of files in documents area";	// Potrzebujesz tlumaczenie
#elif L==8
	"N&ordm; de arquivos em zonas de documentos";
#endif

const char *Txt_No_of_folders =
#if   L==0
	"Nombre de carpetes";
#elif L==1
	"Anzahl der Verzeichnisse";
#elif L==2
	"No. of folders";
#elif L==3
	"N&ordm; de carpetas";
#elif L==4
	"Nombre de r&eacute;pertoires";
#elif L==5
	"N&ordm; de carpetas";	// Okoteve traduccin
#elif L==6
	"Numero di cartelle";
#elif L==7
	"Liczba folder&oacute;w";
#elif L==8
	"N&ordm; de diret&oacute;rios";
#endif

const char *Txt_No_of_forums =
#if   L==0
	"N&ordm; de foros";	// Necessita traduccio
#elif L==1
	"Anzahl der Foren";
#elif L==2
	"No. of forums";
#elif L==3
	"N&ordm; de foros";
#elif L==4
	"Nombre de forums";
#elif L==5
	"N&ordm; de foros";	// Okoteve traduccin
#elif L==6
	"Numero di forum";
#elif L==7
	"Liczba forum";
#elif L==8
	"N&ordm; de f&oacute;runs";
#endif

const char *Txt_No_of_BR_groups =
#if   L==0
	"N&ordm; de<br />grupos";	// Necessita traduccio
#elif L==1
	"Anzahl der<br />Gruppen";
#elif L==2
	"No. of<br />groups";
#elif L==3
	"N&ordm; de<br />grupos";
#elif L==4
	"Nombre de<br />groupes";
#elif L==5
	"N&ordm; de<br />grupos";	// Okoteve traduccin
#elif L==6
	"Numero di<br />gruppi";
#elif L==7
	"Liczba z<br />grupy";
#elif L==8
	"N&ordm; de<br />grupos";
#endif

const char *Txt_No_of_indicators =
#if   L==0
	"N&ordm; de indicadores";	// Necessita traduccio
#elif L==1
	"Anzahl der Indikatoren";
#elif L==2
	"No. of indicators";
#elif L==3
	"N&ordm; de indicadores";
#elif L==4
	"Nombre d'indicateurs";
#elif L==5
	"N&ordm; de indicadores";	// Okoteve traduccin
#elif L==6
	"Numero di indicatori";
#elif L==7
	"Liczba z wskazniki";
#elif L==8
	"N&ordm; de indocadores";
#endif

const char *Txt_No_of_BR_lines =
#if   L==0
	"N&ordm; de<br />l&iacute;neas";	// Necessita traduccio
#elif L==1
	"Anzahl der<br />Zeilen";
#elif L==2
	"No. of<br />lines";
#elif L==3
	"N&ordm; de<br />l&iacute;neas";
#elif L==4
	"Nombre de<br />lignes";
#elif L==5
	"N&ordm; de<br />l&iacute;neas";	// Okoteve traduccin
#elif L==6
	"Numero di<br />linee";
#elif L==7
	"Liczba z<br />linie";
#elif L==8
	"N&ordm; de<br />linhas";
#endif

const char *Txt_No_of_messages =
#if   L==0
	"N&ordm; de mensajes";	// Necessita traduccio
#elif L==1
	"Anzahl der Nachrichten";
#elif L==2
	"No. of messages";
#elif L==3
	"N&ordm; de mensajes";
#elif L==4
	"Nombre de messages";
#elif L==5
	"N&ordm; de mensajes";	// Okoteve traduccin
#elif L==6
	"Numero di messaggi";
#elif L==7
	"Liczba wiadomosci";
#elif L==8
	"N&ordm; de mensagens";
#endif

const char *Txt_No_of_messages_BR_per_forum =
#if   L==0
	"N&ordm; de mensajes<br />por foro";	// Necessita traduccio
#elif L==1
	"Anzahl der Nachrichten<br />pro Forum";
#elif L==2
	"No. of messages<br />per forum";
#elif L==3
	"N&ordm; de mensajes<br />por foro";
#elif L==4
	"Nombre de messages<br />par forum";
#elif L==5
	"N&ordm; de mensajes<br />por foro";	// Okoteve traduccin
#elif L==6
	"Numero di messaggi<br />per forum";
#elif L==7
	"Liczba wiadomosci<br />per forum";
#elif L==8
	"N&ordm; de mensagens<br />por f&oacute;rum";
#endif

const char *Txt_No_of_messages_BR_per_thread =
#if   L==0
	"N&ordm; de mensajes<br />por discusi&oacute;n";	// Necessita traduccio
#elif L==1
	"Anzahl der Nachrichten<br />pro Thread";
#elif L==2
	"No. of messages<br />per thread";
#elif L==3
	"N&ordm; de mensajes<br />por discusi&oacute;n";
#elif L==4
	"Nombre de messages<br />par fil";
#elif L==5
	"N&ordm; de mensajes<br />por discusi&oacute;n";	// Okoteve traduccin
#elif L==6
	"Numero di messaggi<br />per discussione";
#elif L==7
	"Liczba wiadomosci<br />per watku";
#elif L==8
	"N&ordm; de mensagens<br />por thread";
#endif

const char *Txt_No_of_private_files =
#if   L==0
	"N&ordm; de archivos privados";	// Necessita traduccio
#elif L==1
	"Anzahl der private Dateien";
#elif L==2
	"Number of private files";
#elif L==3
	"N&ordm; de archivos privados";
#elif L==4
	"Nombre de fichiers priv&eacute;s";
#elif L==5
	"N&ordm; de archivos privados";	// Okoteve traduccin
#elif L==6
	"Numero di file privati";
#elif L==7
	"Liczba prywatne pliki";
#elif L==8
	"N&ordm; de arquivos privados";
#endif

const char *Txt_No_of_public_files =
#if   L==0
	"N&ordm; de archivos p&uacute;blicos";	// Necessita traduccio
#elif L==1
	"Anzahl der &ouml;ffentliche Dateien";
#elif L==2
	"Number of public files";
#elif L==3
	"N&ordm; de archivos p&uacute;blicos";
#elif L==4
	"Nombre de fichiers publics";
#elif L==5
	"N&ordm; de archivos p&uacute;blicos";	// Okoteve traduccin
#elif L==6
	"Numero di file pubblici";
#elif L==7
	"Liczba publicznych pliki";
#elif L==8
	"N&ordm; de arquivos p&uacute;blicos";
#endif

const char *Txt_No_of_questions =
#if   L==0
	"Nombre de preguntes";
#elif L==1
	"Anzahl der Fragen";
#elif L==2
	"No. of questions";
#elif L==3
	"N&ordm; de preguntas";
#elif L==4
	"Nombre de questions";
#elif L==5
	"N&ordm; de preguntas";	// Okoteve traduccin
#elif L==6
	"Numero di domande";
#elif L==7
	"Liczba pyta&nacute;";
#elif L==8
	"N&ordm; de quest&otilde;es";
#endif

const char *Txt_No_of_threads =
#if   L==0
	"N&ordm; de discusiones";	// Necessita traduccio
#elif L==1
	"Anzahl der Threads";
#elif L==2
	"No. of threads";
#elif L==3
	"N&ordm; de discusiones";
#elif L==4
	"Nombre de fils";
#elif L==5
	"N&ordm; de discusiones";	// Okoteve traduccin
#elif L==6
	"Numero di discussioni";
#elif L==7
	"Liczba watk&oacute;w";
#elif L==8
	"N&ordm; de threads";
#endif

const char *Txt_No_of_threads_BR_per_forum =
#if   L==0
	"N&ordm; de discusiones<br />por foro";	// Necessita traduccio
#elif L==1
	"Anzahl der Threads<br />pro Forum";
#elif L==2
	"No. of threads<br />per forum";
#elif L==3
	"N&ordm; de discusiones<br />por foro";
#elif L==4
	"Nombre de fils<br />par forum";
#elif L==5
	"N&ordm; de discusiones<br />por foro";	// Okoteve traduccin
#elif L==6
	"Numero di discussioni<br />per forum";
#elif L==7
	"Liczba watk&oacute;w<br />per forum";
#elif L==8
	"N&ordm; de threads<br />por f&oacute;rum";
#endif

const char *Txt_Number_of_BR_times_that_BR_questions_BR_have_been_BR_responded =
#if   L==0
	"N&ordm; de<br />veces<br />que se ha<br />respondido";	// Necessita traduccio
#elif L==1
	"Number of<br />times that<br />questions<br />have been<br />responded";	// Need bersetzung
#elif L==2
	"Number of<br />times that<br />questions<br />have been<br />responded";
#elif L==3
	"N&ordm; de<br />veces<br />que se ha<br />respondido";
#elif L==4
	"Number of<br />times that<br />questions<br />have been<br />responded";	// Besoin de traduction
#elif L==5
	"N&ordm; de<br />veces<br />que se ha<br />respondido";	// Okoteve traduccin
#elif L==6
	"Number of<br />times that<br />questions<br />have been<br />responded";	// Bisogno di traduzione
#elif L==7
	"Number of<br />times that<br />questions<br />have been<br />responded";	// Potrzebujesz tlumaczenie
#elif L==8
	"Number of<br />times that<br />questions<br />have been<br />responded";	// Necessita de traduo
#endif

const char *Txt_No_of_users =
#if   L==0
	"Nombre d'usuaris";	// Necessita traduccio
#elif L==1
	"Anzahl der Benutzer";
#elif L==2
	"No. of users";
#elif L==3
	"N&ordm; de usuarios";
#elif L==4
	"Nombre d'utilisateurs";
#elif L==5
	"N&ordm; de usuarios";	// Okoteve traduccin
#elif L==6
	"Numero di utenti";
#elif L==7
	"Liczba uzytkownik&oacute;w";
#elif L==8
	"N&ordm; de utilizadores";
#endif

const char *Txt_No_of_users_who_will_be_notified_by_e_mail =
#if   L==0
	"N&ordm; de usuarios que ser&aacute;n notificados por correo electr&oacute;nico";	// Necessita traduccio
#elif L==1
	"Anzahl der Benutzer, um benachrichtigt zu werden per E-Mail";
#elif L==2
	"No. of users who will be notified by e-mail";
#elif L==3
	"N&ordm; de usuarios que ser&aacute;n notificados por correo electr&oacute;nico";
#elif L==4
	"Nombre d'utilisateurs qui seront notifi&eacute;s par e-mail";
#elif L==5
	"N&ordm; de usuarios que ser&aacute;n notificados por correo electr&oacute;nico";	// Okoteve traduccin
#elif L==6
	"Numero di utenti che saranno notificati via e-mail";
#elif L==7
	"Liczba uzytkownik&oacute;w, kt&oacute;rzy zostana powiadomieni przez e-mail";
#elif L==8
	"N&ordm; de utilizadores que ser&atilde;o notificados por e-mail";
#endif

const char *Txt_No_questions_found_matching_your_search_criteria =
#if   L==0
	"No hay preguntas con el criterio de b&uacute;squeda seleccionado.";	// Necessita traduccio
#elif L==1
	"Keine Testfragen gefunden, die mit Ihrer Suchanfrage entsprechen.";
#elif L==2
	"No questions found matching your search criteria.";
#elif L==3
	"No hay preguntas con el criterio de b&uacute;squeda seleccionado.";
#elif L==4
	"Aucune question trouv&eacute;e correspondant &agrave; vos crit&egrave;res de recherche.";
#elif L==5
	"No hay preguntas con el criterio de b&uacute;squeda seleccionado.";	// Okoteve traduccin
#elif L==6
	"Non ci sono domande trovate con il criterio di ricerca selezionato.";
#elif L==7
	"Brak pyta&nacute; spelniajacych zadane kryteria wyszukiwania.";
#elif L==8
	"N&atilde;o quest&otilde;es com os seus crit&eacute;rios de pesquisa selecionados.";
#endif

const char *Txt_No_results =
#if   L==0
	"No hi ha resultats.";
#elif L==1
	"Keine Ergebnisse.";
#elif L==2
	"No results.";
#elif L==3
	"No hay resultados.";
#elif L==4
	"Pas de r&eacute;sultats.";
#elif L==5
	"No hay resultados.";		// Okoteve traduccin
#elif L==6
	"Nessun risultato.";
#elif L==7
	"Brak wynik&oacute;w.";
#elif L==8
	"Nenhum resultado.";
#endif

const char *Txt_no_subject =
#if   L==0
	"sin asunto";	// Necessita traduccio
#elif L==1
	"kein Betreff";
#elif L==2
	"no subject";
#elif L==3
	"sin asunto";
#elif L==4
	"pas de sujet";
#elif L==5
	"sin asunto";	// Okoteve traduccin
#elif L==6
	"nessun oggetto";
#elif L==7
	"bez tematu";
#elif L==8
	"sem assunto";
#endif

const char *Txt_no_tags =		// Tags used in test
#if   L==0
	"sin descriptores";	// Necessita traduccio
#elif L==1
	"keine Tags";
#elif L==2
	"no tags";
#elif L==3
	"sin descriptores";
#elif L==4
	"pas de descripteurs";
#elif L==5
	"sin descriptores";	// Okoteve traduccin
#elif L==6
	"senza etichette";
#elif L==7
	"zadnych tag&oacute;w";
#elif L==8
	"sem descritores";
#endif

const char *Txt_No_teachers_or_students_found =
#if   L==0
	"No hay profesores ni estudiantes.";	// Necessita traduccio
#elif L==1
	"Keine Lehrer oder Studenten gefunden.";
#elif L==2
	"No teachers or students found.";
#elif L==3
	"No hay profesores ni estudiantes.";
#elif L==4
	"Aucun enseignant ou &eacute;tudiant trouv&eacute;.";
#elif L==5
	"No hay profesores ni estudiantes.";	// Okoteve traduccin
#elif L==6
	"Nessun professore o studente trovato.";
#elif L==7
	"Nie znaleziono nauczycieli i uczni&oacute;w.";
#elif L==8
	"Nenhum professor ou estudante encontrado.";
#endif

const char *Txt_No_test_questions_in_X =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"No hay preguntas de test en <strong>%s</strong>.";	// Necessita traduccio
#elif L==1
	"Keine Testfragen im <strong>%s</strong>.";
#elif L==2
	"No test questions in <strong>%s</strong>.";
#elif L==3
	"No hay preguntas de test en <strong>%s</strong>.";
#elif L==4
	"Il n'y a pas de questions de test dans <strong>%s</strong>.";
#elif L==5
	"No hay preguntas de test en <strong>%s</strong>.";	// Okoteve traduccin
#elif L==6
	"Non ci sono domande di test in <strong>%s</strong>.";
#elif L==7
	"Brak pyta&nacute; testowych <strong>%s</strong>.";
#elif L==8
	"N&atilde;o existem quest&otilde;es de test em <strong>%s</strong>.";
#endif

const char *Txt_No_user_has_been_eliminated =
#if   L==0
	"No se ha eliminado ning&uacute;n usuario.";	// Necessita traduccio
#elif L==1
	"No user has been eliminated.";			// Need bersetzung
#elif L==2
	"No user has been eliminated.";
#elif L==3
	"No se ha eliminado ning&uacute;n usuario.";
#elif L==4
	"No user has been eliminated.";			// Besoin de traduction
#elif L==5
	"No se ha eliminado ning&uacute;n usuario.";	// Okoteve traduccin
#elif L==6
	"Nessun utente &egrave; stato rimosso.";
#elif L==7
	"Zaden uzytkownik nie zostal wyeliminowany.";
#elif L==8
	"No user has been eliminated.";			// Necessita de traduo
#endif

const char *Txt_No_user_has_been_enrolled =
#if   L==0
	"No se ha inscrito ning&uacute;n usuario.";	// Necessita traduccio
#elif L==1
	"No user has been enrolled.";			// Need bersetzung
#elif L==2
	"No user has been enrolled.";
#elif L==3
	"No se ha inscrito ning&uacute;n usuario.";
#elif L==4
	"No user has been enrolled.";			// Besoin de traduction
#elif L==5
	"No se ha inscrito ning&uacute;n usuario.";	// Okoteve traduccin
#elif L==6
	"Nessun utente &egrave; stato registrato.";
#elif L==7
	"Zaden uzytkownik nie zostal zarejestrowany.";
#elif L==8
	"No user has been enrolled.";			// Necessita de traduo
#endif

const char *Txt_No_user_has_been_removed =
#if   L==0
	"No se ha eliminado ning&uacute;n usuario.";	// Necessita traduccio
#elif L==1
	"No user has been removed.";			// Need bersetzung
#elif L==2
	"No user has been removed.";
#elif L==3
	"No se ha eliminado ning&uacute;n usuario.";
#elif L==4
	"No user has been removed.";			// Besoin de traduction
#elif L==5
	"No se ha eliminado ning&uacute;n usuario.";	// Okoteve traduccin
#elif L==6
	"Nessun utente &egrave; stato rimosso.";
#elif L==7
	"Zaden uzytkownik nie zostal usuniety.";
#elif L==8
	"No user has been removed.";			// Necessita de traduo
#endif

const char *Txt_No_users_found[Rol_NUM_ROLES] =
   {
#if   L==0	// Rol_ROLE_UNKNOWN
	 "No hay usuarios."	// Necessita traduccio
#elif L==1
	 "Keine Benutzer gefunden."
#elif L==2
	 "No users found."
#elif L==3
	 "No hay usuarios."
#elif L==4
	 "Aucun utilisateur trouv&eacute;."
#elif L==5
	 "No hay usuarios."	// Okoteve traduccin
#elif L==6
	 "Nessun utente trovato."
#elif L==7
	 "Nie znaleziono uzytkownik&oacute;w."
#elif L==8
	 "Nenhum utilizador encontrado."
#endif
	 ,
#if   L==0	// Rol_ROLE_GUEST
	 "No hi ha convidats."
#elif L==1
	 "Keine Eingeladen gefunden."
#elif L==2
	 "No guests found."
#elif L==3
	 "No hay invitados."
#elif L==4
	 "Aucun invit&eacute; trouv&eacute;."
#elif L==5
	 "No hay invitados."	// Okoteve traduccin
#elif L==6
	 "Nessun ospite trovato."
#elif L==7
	 "Nie znaleziono zaproszeni."
#elif L==8
	 "Nenhum convidado encontrado."
#endif
	 ,
#if   L==0	// Rol_ROLE_VISITOR
	 "No hay visitantes."	// Necessita traduccio
#elif L==1
	 "Keine Besucher gefunden."
#elif L==2
	 "No visitors found."
#elif L==3
	 "No hay visitantes."
#elif L==4
	 "Aucun visiteur trouv&eacute;."
#elif L==5
	 "No hay visitantes."	// Okoteve traduccin
#elif L==6
	 "Nessun visitatore trovato."
#elif L==7
	 "No visitors found."	// Potrzebujesz tlumaczenie
#elif L==8
	 "Nenhum visitante encontrado."
#endif
	 ,
#if   L==0	// Rol_ROLE_STUDENT
	 "No hay estudiantes."	// Necessita traduccio
#elif L==1
	 "Keine Studenten gefunden."
#elif L==2
	 "No students found."
#elif L==3
	 "No hay estudiantes."
#elif L==4
	 "Aucun &eacute;tudiant trouv&eacute;."
#elif L==5
	 "No hay estudiantes."	// Okoteve traduccin
#elif L==6
	 "Non ci sono studenti."
#elif L==7
	 "No students found."	// Potrzebujesz tlumaczenie
#elif L==8
	 "Nenhum estudante encontrado."
#endif
	 ,
#if   L==0	// Rol_ROLE_TEACHER
	 "No hay profesores."	// Necessita traduccio
#elif L==1
	 "Keine Lehrkr&auml;ften gefunden."
#elif L==2
	 "No teachers found."
#elif L==3
	 "No hay profesores."
#elif L==4
	 "Aucun enseignant trouv&eacute;."
#elif L==5
	 "No hay profesores."	// Okoteve traduccin
#elif L==6
	 "Non ci sono professori."
#elif L==7
	 "Nie znaleziono nauczycieli."
#elif L==8
	 "Nenhum professor encontrado."
#endif
	 ,
#if   L==0	// Rol_ROLE_DEG_ADMIN
	 "No hay administradores."	// Necessita traduccio
#elif L==1
	 "Keine Administratoren gefunden."
#elif L==2
	 "No administrators found."
#elif L==3
	 "No hay administradores."
#elif L==4
	 "Aucun administrateur trouv&eacute;."
#elif L==5
	 "No hay administradores."	// Okoteve traduccin
#elif L==6
	 "Non ci sono amministratori."
#elif L==7
	 "Nie znaleziono administrator&oacute;w."
#elif L==8
	 "Nenhum administrador encontrado."
#endif
	 ,
#if   L==0	// Rol_ROLE_CTR_ADMIN
	 "No hay administradores."	// Necessita traduccio
#elif L==1
	 "Keine Administratoren gefunden."
#elif L==2
	 "No administrators found."
#elif L==3
	 "No hay administradores."
#elif L==4
	 "Aucun administrateur trouv&eacute;."
#elif L==5
	 "No hay administradores."	// Okoteve traduccin
#elif L==6
	 "Non ci sono amministratori."
#elif L==7
	 "Nie znaleziono administrator&oacute;w."
#elif L==8
	 "Nenhum administrador encontrado."
#endif
	 ,
#if   L==0	// Rol_ROLE_INS_ADMIN
	 "No hay administradores."	// Necessita traduccio
#elif L==1
	 "Keine Administratoren gefunden."
#elif L==2
	 "No administrators found."
#elif L==3
	 "No hay administradores."
#elif L==4
	 "Aucun administrateur trouv&eacute;."
#elif L==5
	 "No hay administradores."	// Okoteve traduccin
#elif L==6
	 "Non ci sono amministratori."
#elif L==7
	 "Nie znaleziono administrator&oacute;w."
#elif L==8
	 "Nenhum administrador encontrado."
#endif
	 ,
#if   L==0	// Rol_ROLE_SUPERUSER
	 "No hay superusuarios."	// Necessita traduccio
#elif L==1
	 "Keine Superuser gefunden."
#elif L==2
	 "No superusers found."
#elif L==3
	 "No hay superusuarios."
#elif L==4
	 "Aucun superutilisateur trouv&eacute;."
#elif L==5
	 "No hay superusuarios."	// Okoteve traduccin
#elif L==6
	 "Non ci sono superutenti."
#elif L==7
	 "Nie znaleziono superu&zdot;ytkownika."
#elif L==8
	 "Nenhum superutilizador encontrado."
#endif
	};

const char *Txt_nobody_else_can_access_this_content =
#if   L==0
	"ning&uacute; m&eacute;s pot accedir a aquest contingut";
#elif L==1
	"Niemand sonst kann auf diese Inhalte";
#elif L==2
	"nobody else can access this content";
#elif L==3
	"nadie m&aacute;s puede acceder a este contenido";
#elif L==4
	"personne d'autre ne peut acc&eacute;der &agrave; ce contenu";
#elif L==5
	"nadie m&aacute;s puede acceder a este contenido";	// Okoteve traduccin
#elif L==6
	"nessun altro pu&ograve; accedere a questo contenuto";
#elif L==7
	"nikt inny nie moze uzyskac dostepu do tej zawartosci";
#elif L==8
	"ningu&eacute;m mais pode acessar este conte&uacute;do";
#endif

const char *Txt_non_blank_QUESTIONS =
#if   L==0
	"contestades";
#elif L==1
	"Non-blank";
#elif L==2
	"non-blank";
#elif L==3
	"contestadas";
#elif L==4
	"non vide";
#elif L==5
	"contestadas";	// Okoteve traduccin
#elif L==6
	"risposte";
#elif L==7
	"niepustych";
#elif L==8
	"respondidas";
#endif

const char *Txt_Non_blank_BR_questions =
#if   L==0
	"Preguntes<br />contestades";
#elif L==1
	"Non-blank<br />Fragen";
#elif L==2
	"Non-blank<br />questions";
#elif L==3
	"Preguntas<br />contestadas";
#elif L==4
	"Questions<br />non vide";
#elif L==5
	"Preguntas<br />contestadas";	// Okoteve traduccin
#elif L==6
	"Domande<br />risposte";
#elif L==7
	"Niepustych<br />pytania";
#elif L==8
	"Quest&otilde;es<br />respondidas";
#endif

const char *Txt_Nothing_has_been_pasted_because_the_clipboard_is_empty_ =
#if   L==0
	"No se pegado nada porque el portapapeles est&aacute; vac&iacute;o."
	" Trate de copiar y pegar de nuevo.";	// Necessita traduccio
#elif L==1
	"Es konnten keine Daten eingef&uuml;gt werden. Der Zwischenspeicher ist leer."
	"Versuchen Sie den Kopieren-Einf&uuml;gen-Vorgang bitte erneut.";
#elif L==2
	"Nothing has been pasted because the clipboard is empty."
	"Try copy-paste again.";
#elif L==3
	"No se pegado nada porque el portapapeles est&aacute; vac&iacute;o."
	" Trate de copiar y pegar de nuevo.";
#elif L==4
	"Nothing has been pasted because the clipboard is empty."
	"Try copy-paste again.";		// Besoin de traduction
#elif L==5
	"No se pegado nada porque el portapapeles est&aacute; vac&iacute;o."
	" Trate de copiar y pegar de nuevo.";	// Okoteve traduccin
#elif L==6
	"Niente &egrave; stato copiato perch&egrave; la bacheca &egrave; piena."
	"Prova copia-incolla nuovamente.";
#elif L==7
	"Nic nie zostal wklejony bo schowek jest pusty."
	"Spr&oacute;buj ponownie kopiuj-wklej.";
#elif L==8
	"Nothing has been pasted because the clipboard is empty."
	"Try copy-paste again.";		// Necessita de traduo
#endif

const char *Txt_NOTICE_Active_SINGULAR =
#if   L==0
	"Activo";	// Necessita traduccio
#elif L==1
	"Aktive";
#elif L==2
	"Active";
#elif L==3
	"Activo";
#elif L==4
	"Active";
#elif L==5
	"Activo";	// Okoteve traduccin
#elif L==6
	"Attivo";
#elif L==7
	"Active";
#elif L==8
	"Ativo";
#endif

const char *Txt_NOTICE_Active_BR_notices =
#if   L==0
	"Avisos<br />activos";	// Necessita traduccio
#elif L==1
	"Aktive<br />Ank&uuml;ndigungen";
#elif L==2
	"Active<br />notices";
#elif L==3
	"Avisos<br />activos";
#elif L==4
	"Avis<br />actives";
#elif L==5
	"Avisos<br />activos";	// Okoteve traduccin
#elif L==6
	"Avvisi<br />attivi";
#elif L==7
	"Active<br />Ogloszenia";
#elif L==8
	"Avisos<br />ativos";
#endif

const char *Txt_NOTICE_Active_Mark_as_obsolete =
#if   L==0
	"Actiu; marcar com obsolet";
#elif L==1
	"Aktive; marke als veraltet";
#elif L==2
	"Active; mark as obsolete";
#elif L==3
	"Activo; marcar como obsoleto";
#elif L==4
	"Active; marquer comme obsol&egrave;te";
#elif L==5
	"Activo; marcar como obsoleto";	// Okoteve traduccin
#elif L==6
	"Attivo; segnare come obsoleto";
#elif L==7
	"Active; zaznaczy jako przestarzale";
#elif L==8
	"Ativo; marcar como obsoleto";
#endif

const char *Txt_NOTICE_Deleted_BR_notices =
#if   L==0
	"Avisos<br />eliminados";	// Necessita traduccio
#elif L==1
	"Gel&ouml;scht<br />Ank&uuml;ndigungen";
#elif L==2
	"Deleted<br />notices";
#elif L==3
	"Avisos<br />eliminados";
#elif L==4
	"Avis<br />supprim&eacute;s";
#elif L==5
	"Avisos<br />eliminados";	// Okoteve traduccin
#elif L==6
	"Avvisi<br />rimossi";
#elif L==7
	"Usuniete<br />notices";
#elif L==8
	"Avisos<br />suprimidos";
#endif

const char *Txt_NOTICE_Obsolete_SINGULAR =
#if   L==0
	"Obsoleto";	// Necessita traduccio
#elif L==1
	"Veraltet";
#elif L==2
	"Obsolete";
#elif L==3
	"Obsoleto";
#elif L==4
	"Obsol&egrave;te";
#elif L==5
	"Obsoleto";	// Okoteve traduccin
#elif L==6
	"Obsoleto";
#elif L==7
	"Przestarzaly";
#elif L==8
	"Obsoleto";
#endif

const char *Txt_NOTICE_Obsolete_BR_notices =
#if   L==0
	"Avisos<br />obsoletos";	// Necessita traduccio
#elif L==1
	"Veraltet<br />Ank&uuml;ndigungen";
#elif L==2
	"Obsolete<br />notices";
#elif L==3
	"Avisos<br />obsoletos";
#elif L==4
	"Avis<br />obsol&egrave;tes";
#elif L==5
	"Avisos<br />obsoletos";	// Okoteve traduccin
#elif L==6
	"Avvisi<br />obsoleti";
#elif L==7
	"Przestarzaly<br />Ogloszenia";
#elif L==8
	"Avisos<br />obsoletos";
#endif

const char *Txt_NOTICE_Obsolete_Mark_as_active =
#if   L==0
	"Obsolet; marcar com actiu";
#elif L==1
	"Veraltet; marke als aktive";
#elif L==2
	"Obsolete; mark as active";
#elif L==3
	"Obsoleto; marcar como activo";
#elif L==4
	"Obsol&egrave;te; marquer comme active";
#elif L==5
	"Obsoleto; marcar como activo";	// Okoteve traduccin
#elif L==6
	"Obsoleto; segnare come attivo";
#elif L==7
	"Przestarzaly; znak towarowy jako aktywnych";
#elif L==8
	"Obsoleto; marcar como ativo";
#endif

const char *Txt_Notice =
#if   L==0
	"Av&iacute;s";
#elif L==1
	"Ank&uuml;ndigung";
#elif L==2
	"Notice";
#elif L==3
	"Aviso";
#elif L==4
	"Avis";
#elif L==5
	"Marandu";
#elif L==6
	"Avviso";
#elif L==7
	"Informacja";
#elif L==8
	"Aviso";
#endif

const char *Txt_Notice_created =
#if   L==0
	"Av&iacute;s creat.";
#elif L==1
	"Ank&uuml;ndigung erstellt.";
#elif L==2
	"Notice created.";
#elif L==3
	"Aviso creado.";
#elif L==4
	"Avis cr&eacute;&eacute;.";
#elif L==5
	"Aviso creado.";	// Okoteve traduccin
#elif L==6
	"Avviso creato.";
#elif L==7
	"Zawiadomienie stworzony.";
#elif L==8
	"Aviso criado.";
#endif

const char *Txt_NOTIF_new_SINGULAR =
#if   L==0
	"nueva";	// Necessita traduccio
#elif L==1
	"neue";
#elif L==2
	"new";
#elif L==3
	"nueva";
#elif L==4
	"nouvelle";
#elif L==5
	"nueva";	// Okoteve traduccin
#elif L==6
	"nuova";
#elif L==7
	"nowy";
#elif L==8
	"nova";
#endif

const char *Txt_NOTIF_new_PLURAL =
#if   L==0
	"nuevas";	// Necessita traduccio
#elif L==1
	"neue";
#elif L==2
	"new";
#elif L==3
	"nuevas";
#elif L==4
	"nouvelles";
#elif L==5
	"nuevas";	// Okoteve traduccin
#elif L==6
	"nuove";
#elif L==7
	"nowy";
#elif L==8
	"novas";
#endif

const char *Txt_NOTIFICATION_STATUS[Ntf_NUM_STATUS_TXT] =
   {
#if   L==0	// Ntf_STATUS_NO_EMAIL
	 "No"	// Necessita traduccio
#elif L==1
	 "Nein"
#elif L==2
	 "No"
#elif L==3
	 "No"
#elif L==4
	 "Non"
#elif L==5
	 "Nah&aacute;niri"
#elif L==6
	 "No"
#elif L==7
	 "No"
#elif L==8
	 "N&atilde;o"
#endif
	 ,
#if   L==0	// Ntf_STATUS_EMAIL_PENDING
	 "Pendiente"	// Necessita traduccio
#elif L==1
	 "Offen"
#elif L==2
	 "Pending"
#elif L==3
	 "Pendiente"
#elif L==4
	 "En attente"
#elif L==5
	 "Pendiente"	// Okoteve traduccin
#elif L==6
	 "In attesa"
#elif L==7
	 "Do czasu"
#elif L==8
	 "Pendente"
#endif
	 ,
#if   L==0	// Ntf_STATUS_EMAIL_CANCELLED
	 "Cancelado"	// Necessita traduccio
#elif L==1
	 "Abgebrochen"
#elif L==2
	 "Cancelled"
#elif L==3
	 "Cancelado"
#elif L==4
	 "Annul&eacute;"
#elif L==5
	 "Cancelado"	// Okoteve traduccin
#elif L==6
	 "Rimosso"
#elif L==7
	 "Odwolany"
#elif L==8
	 "Cancelado"
#endif
	 ,
#if   L==0	// Ntf_STATUS_EMAIL_SENT
	 "Enviado"	// Necessita traduccio
#elif L==1
	 "Gesendet"
#elif L==2
	 "Sent"
#elif L==3
	 "Enviado"
#elif L==4
	 "Envoy&eacute;e"
#elif L==5
	 "Enviado"	// Okoteve traduccin
#elif L==6
	 "Inviata"
#elif L==7
	 "Wyslane"
#elif L==8
	 "Enviada"
#endif
	};

const char *Txt_notification =
#if   L==0
	"notificaci&oacute;";
#elif L==1
	"Anmeldung";
#elif L==2
	"notification";
#elif L==3
	"notificaci&oacute;n";
#elif L==4
	"notification";
#elif L==5
	"momarandu";
#elif L==6
	"notifica";
#elif L==7
	"powiadomienie";
#elif L==8
	"notifica&ccedil;&atilde;o";
#endif

const char *Txt_Notifications =
#if   L==0
	"Notificacions";
#elif L==1
	"Anmeldungen";
#elif L==2
	"Notifications";
#elif L==3
	"Notificaciones";
#elif L==4
	"Notifications";
#elif L==5
	"Momarandu";
#elif L==6
	"Notifiche";
#elif L==7
	"Powiadomienia";
#elif L==8
	"Notifica&ccedil;&otilde;es";
#endif

const char *Txt_notifications =
#if   L==0
	"notificacions";
#elif L==1
	"Anmeldungen";
#elif L==2
	"notifications";
#elif L==3
	"notificaciones";
#elif L==4
	"notifications";
#elif L==5
	"momarandu";
#elif L==6
	"notifiche";
#elif L==7
	"powiadomienia";
#elif L==8
	"notifica&ccedil;&otilde;es";
#endif

const char *Txt_NOTIFY_EVENTS_PLURAL[Ntf_NUM_NOTIFY_EVENTS] =
   {
#if   L==0	// Ntf_EVENT_UNKNOWN
	 "Esdeveniments desconeguts"
#elif L==1
	 "Unbekannte Ereignisse"
#elif L==2
	 "Unknown events"
#elif L==3
	 "Eventos desconocidos"
#elif L==4
	 "&Eacute;v&eacute;nements inconnus"
#elif L==5
	 "Eventos desconocidos"	// Okoteve traduccin
#elif L==6
	 "Eventi sconosciuto"
#elif L==7
	 "Brak wydarzen"
#elif L==8
	 "Eventos desconhecidos"
#endif
	 ,
#if   L==0	// Ntf_EVENT_DOCUMENT_FILE
	 "Nous documents"
#elif L==1
	 "Neue Dokumentdateien"
#elif L==2
	 "New document files"
#elif L==3
	 "Nuevos documentos"
#elif L==4
	 "Nouveaux fichiers de documents"
#elif L==5
	 "Nuevos documentos"	// Okoteve traduccin
#elif L==6
	 "Nuovi documenti"
#elif L==7
	 "Nowy plik dokumentu"
#elif L==8
	 "Novos arquivos de documento"
#endif
	 ,
#if   L==0	// Ntf_EVENT_SHARED_FILE
	 "Nous arxiu compartit"
#elif L==1
	 "New freigegebene Dateien"
#elif L==2
	 "New shared files"
#elif L==3
	 "Nuevos archivos compartidos"
#elif L==4
	 "Nouveaux fichiers partag&eacute;s"
#elif L==5
	 "Nuevos archivos compartidos"	// Okoteve traduccin
#elif L==6
	 "Nuovi file condiviso"
#elif L==7
	 "Nowy Udost&eogon;pniony plik"
#elif L==8
	 "Novos arquivos compartilhado"
#endif
	 ,
#if   L==0	// Ntf_EVENT_ASSIGNMENT
	 "Noves activitats"
#elif L==1
	 "Neue Aufgaben"
#elif L==2
	 "New assignments"
#elif L==3
	 "Nuevas actividades"
#elif L==4
	 "Nouvelles activit&eacute;s"
#elif L==5
	 "Nuevas actividades"	// Okoteve traduccin
#elif L==6
	 "Nuove attivit&agrave;"
#elif L==7
	 "Nowe zadania"
#elif L==8
	 "Novas atividades"
#endif
	 ,
#if   L==0	// Ntf_EVENT_EXAM_ANNOUNCEMENT
	 "Noves convocat&ograve;ries d'examen"
#elif L==1
	 "Neue Aufrufe f&uuml;r Pr&uuml;fung"
#elif L==2
	 "New announcements of exam"
#elif L==3
	 "Nuevas convocatorias de examen"
#elif L==4
	 "Nouvelles convocations &agrave; un examen"
#elif L==5
	 "Nuevas convocatorias de examen"	// Okoteve traduccin
#elif L==6
	 "Nuovo appello d'esame"
#elif L==7
	 "Nowe anonse egzamin"
#elif L==8
	 "Novas chamadas para exame"
#endif
	 ,
#if   L==0	// Ntf_EVENT_MARKS_FILE
	 "Nous fitxers de qualificacions"
#elif L==1
	 "Neue Dateien mit Bewertung"
#elif L==2
	 "New files with marks"
#elif L==3
	 "Nuevos archivos de calificaciones"
#elif L==4
	 "Nouveaux fichiers avec des notes"
#elif L==5
	 "Nuevos archivos de calificaciones"	// Okoteve traduccin
#elif L==6
	 "Nuovi file con risultati"
#elif L==7
	 "Nowe pliki ze znakami"
#elif L==8
	 "Novos arquivos de notas"
#endif
	 ,
#if   L==0	// Ntf_EVENT_ENROLLMENT
	 "Novas inscripci&oacute;s"
#elif L==1
	 "New Einschreibungen"
#elif L==2
	 "New enrollments"
#elif L==3
	 "Nuevas inscripciones"
#elif L==4
	 "Nouvelles inscriptions"
#elif L==5
	 "Nuevas inscripciones"	// Okoteve traduccin
#elif L==6
	 "Nuova iscrizione"
#elif L==7
	 "Nowe zapisy"
#elif L==8
	 "Novas inscri&ccedil;&otilde;es"
#endif
	 ,
#if   L==0	// Ntf_EVENT_ENROLLMENT_REQUEST
	 "Novas petici&oacute;s d'inscripci&oacute;"
#elif L==1
	 "New Registrierungsanforderungen"
#elif L==2
	 "New enrollment requests"
#elif L==3
	 "Nuevas peticiones de inscripci&oacute;n"
#elif L==4
	 "Nouvelles demandes d'inscription"
#elif L==5
	 "Nuevas peticiones de inscripci&oacute;n"	// Okoteve traduccin
#elif L==6
	 "Nuova richiesta di iscrizione"
#elif L==7
	 "Nowe pro&sacute;by o rejestracji"
#elif L==8
	 "Novos pedidos de inscri&ccedil;&atilde;o"
#endif
	 ,
#if   L==0	// Ntf_EVENT_NOTICE
	 "Nous avisos"
#elif L==1
	 "Neue Ank&uuml;ndigungen"
#elif L==2
	 "New notices"
#elif L==3
	 "Nuevos avisos"
#elif L==4
	 "Nouveaux avis"
#elif L==5
	 "Nuevos avisos"	// Okoteve traduccin
#elif L==6
	 "Nuovi avvisi"
#elif L==7
	 "Nowe ogloszenia"
#elif L==8
	 "Novos avisos"
#endif
	 ,
#if   L==0	// Ntf_EVENT_FORUM_POST_COURSE
	 "Novos missatges en f&ograve;rums"
#elif L==1
	 "Neue Beitr&auml;ge in den Foren"
#elif L==2
	 "New posts in course forums"
#elif L==3
	 "Nuevos mensajes en foros de asignaturas"
#elif L==4
	 "Nouveaux messages dans les forums"
#elif L==5
	 "Nuevos mensajes en foros de asignaturas"	// Okoteve traduccin
#elif L==6
	 "Nuovi post nei forum"
#elif L==7
	 "Nowe post&oacute;w na forum"
#elif L==8
	 "Novos posts em f&oacute;runs"
#endif
	 ,
#if   L==0	// Ntf_EVENT_FORUM_REPLY
	 "Noves respostes als meus missatges en f&ograve;rums"
#elif L==1
	 "Neue Antworten auf meine Beitr&auml;ge in den Foren"
#elif L==2
	 "New responses to my posts in forums"
#elif L==3
	 "Nuevas respuestas a mis mensajes en foros"
#elif L==4
	 "Nouvelles r&eacute;ponses &agrave; mes messages dans les forums"
#elif L==5
	 "Nuevas respuestas a mis mensajes en foros"	// Okoteve traduccin
#elif L==6
	 "Nuove risposte ai miei post nei forum"
#elif L==7
	 "Nowe podejscie do moich post&oacute;w na forum"
#elif L==8
	 "Novas respostas aos meus posts em f&oacute;runs"
#endif
	 ,
#if   L==0	// Ntf_EVENT_MESSAGE
	 "Nous missatges"
#elif L==1
	 "Neue Nachrichten"
#elif L==2
	 "New messages"
#elif L==3
	 "Nuevos mensajes"
#elif L==4
	 "Nouveaux messages"
#elif L==5
	 "Nuevos mensajes"	// Okoteve traduccin
#elif L==6
	 "Nuovi messaggi"
#elif L==7
	 "Nowa wiadomosc"
#elif L==8
	 "Novas mensagens"
#endif
	 ,
#if   L==0	// Ntf_EVENT_SURVEY
	 "Noves enquestes"
#elif L==1
	 "Neue Umfragen"
#elif L==2
	 "New surveys"
#elif L==3
	 "Nuevas encuestas"
#elif L==4
	 "Nouveaux sondages"
#elif L==5
	 "Nuevas encuestas"	// Okoteve traduccin
#elif L==6
	 "Nuove sondaggi"
#elif L==7
	 "Nowe badania"
#elif L==8
	 "Novos inqu&eacute;ritos"
#endif
	};

const char *Txt_NOTIFY_EVENTS_SINGULAR[Ntf_NUM_NOTIFY_EVENTS] =
   {
#if   L==0	// Ntf_EVENT_UNKNOWN
	 "Esdeveniment desconegut"
#elif L==1
	 "Unbekannt Ereignis"
#elif L==2
	 "Unknown event"
#elif L==3
	 "Evento desconocido"
#elif L==4
	 "&Eacute;v&eacute;nement inconnu"
#elif L==5
	 "Evento desconocido"	// Okoteve traduccin
#elif L==6
	 "Evento sconosciuto"
#elif L==7
	 "Nieznane zdarzenia"
#elif L==8
	 "Evento desconhecido"
#endif
	 ,
#if   L==0	// Ntf_EVENT_DOCUMENT_FILE
	 "Document"
#elif L==1
	 "Dokumentdatei"
#elif L==2
	 "Document file"
#elif L==3
	 "Documento"
#elif L==4
	 "Fichier de document"
#elif L==5
	 "Documento"	// Okoteve traduccin
#elif L==6
	 "Documento"
#elif L==7
	 "Plik dokumentu"
#elif L==8
	 "Arquivo de documento"
#endif
	 ,
#if   L==0	// Ntf_EVENT_SHARED_FILE
	 "Arxiu compartit"
#elif L==1
	 "Freigegebene Datei"
#elif L==2
	 "Shared file"
#elif L==3
	 "Archivo compartido"
#elif L==4
	 "Fichier partag&eacute;"
#elif L==5
	 "Archivo compartido"	// Okoteve traduccin
#elif L==6
	 "File condiviso"
#elif L==7
	 "Udost&eogon;pniony plik"
#elif L==8
	 "Arquivo compartilhado"
#endif
	 ,
#if   L==0	// Ntf_EVENT_ASSIGNMENT
	 "Activitat"
#elif L==1
	 "Aufgabe"
#elif L==2
	 "Assignment"
#elif L==3
	 "Actividad"
#elif L==4
	 "Activit&eacute;"
#elif L==5
	 "Tar&eacute;a"
#elif L==6
	 "Attivit&agrave;"
#elif L==7
	 "Przyporzadkowanie"
#elif L==8
	 "Atividade"
#endif
	 ,
#if   L==0	// Ntf_EVENT_EXAM_ANNOUNCEMENT
	 "Convocat&ograve;ria d'examen"
#elif L==1
	 "Aufrufe fr Pr&uuml;fung"
#elif L==2
	 "Announcement of exam"
#elif L==3
	 "Convocatoria de examen"
#elif L==4
	 "Convocation &agrave; un examen"
#elif L==5
	 "Convocatoria de examen"	// Okoteve traduccin
#elif L==6
	 "Sessione d'esame"
#elif L==7
	 "Ogloszenie egzamin"
#elif L==8
	 "Chamada para exame"
#endif
	 ,
#if   L==0	// Ntf_EVENT_MARKS_FILE
	 "Fitxer de qualificacions"
#elif L==1
	 "Datei mit Bewertung"
#elif L==2
	 "File with marks"
#elif L==3
	 "Archivo de calificaciones"
#elif L==4
	 "Fichier avec des notes"
#elif L==5
	 "Archivo de calificaciones"	// Okoteve traduccin
#elif L==6
	 "File con risultati"
#elif L==7
	 "Plik ze znakami"
#elif L==8
	 "Arquivo de notas"
#endif
	 ,
#if   L==0	// Ntf_EVENT_ENROLLMENT
	 "Inscripci&oacute;"
#elif L==1
	 "Einschreibung"
#elif L==2
	 "Enrollment"
#elif L==3
	 "Inscripci&oacute;n"
#elif L==4
	 "Inscription"
#elif L==5
	 "Inscripci&oacute;n"	// Okoteve traduccin
#elif L==6
	 "Iscrizione"
#elif L==7
	 "Zapisy"
#elif L==8
	 "Inscri&ccedil;&atilde;o"
#endif
	 ,
#if   L==0	// Ntf_EVENT_ENROLLMENT_REQUEST
	 "Petici&oacute; d'inscripci&oacute;"
#elif L==1
	 "Registrierungsanforderung"
#elif L==2
	 "Enrollment request"
#elif L==3
	 "Petici&oacute;n de inscripci&oacute;n"
#elif L==4
	 "Demande d'inscription"
#elif L==5
	 "Petici&oacute;n de inscripci&oacute;n"	// Okoteve traduccin
#elif L==6
	 "Richiesta di iscrizione"
#elif L==7
	 "&Zdot;&aogon;danie rejestracji"
#elif L==8
	 "Pedido de inscri&ccedil;&atilde;o"
#endif
	 ,
#if   L==0	// Ntf_EVENT_NOTICE
	 "Av&iacute;s"
#elif L==1
	 "Ank&uuml;ndigungen"
#elif L==2
	 "Notice"
#elif L==3
	 "Aviso"
#elif L==4
	 "Avis"
#elif L==5
	 "Marandu"
#elif L==6
	 "Avviso"
#elif L==7
	 "Informacja"
#elif L==8
	 "An&uacute;ncio"
#endif
	 ,
#if   L==0	// Ntf_EVENT_FORUM_POST_COURSE
	 "Missatge en un f&ograve;rum"
#elif L==1
	 "Beitr&auml;ge in einem Forum"
#elif L==2
	 "Post in a forum"
#elif L==3
	 "Mensaje en un foro"
#elif L==4
	 "Message dans un forum"
#elif L==5
	 "Mensaje en un foro"	// Okoteve traduccin
#elif L==6
	 "Post in un forum"
#elif L==7
	 "Post na forum"
#elif L==8
	 "Post em um f&oacute;rum"
#endif
	 ,
#if   L==0	// Ntf_EVENT_FORUM_REPLY
	 "Resposta en un f&ograve;rum"
#elif L==1
	 "Antwort in einem Forum"
#elif L==2
	 "Response in a forum"
#elif L==3
	 "Respuesta en un foro"
#elif L==4
	 "R&eacute;ponse dans un forum"
#elif L==5
	 "Respuesta en un foro"	// Okoteve traduccin
#elif L==6
	 "Risposta in un forum"
#elif L==7
	 "Odpowiedz na forum"
#elif L==8
	 "Resposta em um f&oacute;rum"
#endif
	 ,
#if   L==0	// Ntf_EVENT_MESSAGE
	 "Missatge"
#elif L==1
	 "Nachricht"
#elif L==2
	 "Message"
#elif L==3
	 "Mensaje"
#elif L==4
	 "Message"
#elif L==5
	 "Marandu"
#elif L==6
	 "Messaggio"
#elif L==7
	 "Wiadomo&sacute;&cacute;"
#elif L==8
	 "Mensagem"
#endif
	 ,
#if   L==0	// Ntf_EVENT_SURVEY
	 "Enquesta"
#elif L==1
	 "Umfrage"
#elif L==2
	 "Survey"
#elif L==3
	 "Encuesta"
#elif L==4
	 "Sondage"
#elif L==5
	 "Encuesta"	// Okoteve traduccin
#elif L==6
	 "Sondaggio"
#elif L==7
	 "Badania"
#elif L==8
	 "Inqu&eacute;rito"
#endif
	};

const char *Txt_Notify_me_BR_by_e_mail =
#if   L==0
	"Avisar-me<br />per correu";
#elif L==1
	"Benachrichtigt mich<br />per E-Mail";
#elif L==2
	"Notify me<br />by e-mail";
#elif L==3
	"Avisarme<br />por correo";
#elif L==4
	"Avertissez-moi<br />par e-mail";
#elif L==5
	"Avisarme<br />por correo";		// Okoteve traduccin
#elif L==6
	"Notificatami<br />via e-mail";
#elif L==7
	"Informuj<br />mnie e-mailem";
#elif L==8
	"Notifique-me<br />por e-mail";
#endif

const char *Txt_Now_each_student_can_belong_to_multiple_groups_of_type_X =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"Ahora cada estudiante puede pertenecer a varios grupos de tipo <strong>%s</strong>.";	// Necessita traduccio
#elif L==1
	"Now each student can belong to multiple groups of type <strong>%s</strong>.";
#elif L==2
	"Now each student can belong to multiple groups of type <strong>%s</strong>.";
#elif L==3
	"Ahora cada estudiante puede pertenecer a varios grupos de tipo <strong>%s</strong>.";
#elif L==4
	"Now each student can belong to multiple groups of type <strong>%s</strong>.";
#elif L==5
	"Ahora cada estudiante puede pertenecer a varios grupos de tipo <strong>%s</strong>.";	// Okoteve traduccin
#elif L==6
	"Ora ogni studente pu&ograve; appartenere a gruppi multipli di tipo <strong>%s</strong>.";
#elif L==7
	"Teraz kazdy student moze nalezec do wielu grup typu <strong>%s</strong>.";
#elif L==8
	"Now each student can belong to multiple groups of type <strong>%s</strong>.";
#endif

const char *Txt_Now_each_student_can_only_belong_to_a_group_of_type_X =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"Ahora cada estudiante s&oacute;lo puede pertenecer a un grupo de tipo <strong>%s</strong>.";	// Necessita traduccio
#elif L==1
	"Now each student can only belong to a group of type <strong>%s</strong>.";
#elif L==2
	"Now each student can only belong to a group of type <strong>%s</strong>.";
#elif L==3
	"Ahora cada estudiante s&oacute;lo puede pertenecer a un grupo de tipo <strong>%s</strong>.";
#elif L==4
	"Now each student can only belong to a group of type <strong>%s</strong>.";
#elif L==5
	"Ahora cada estudiante s&oacute;lo puede pertenecer a un grupo de tipo <strong>%s</strong>.";	// Okoteve traduccin
#elif L==6
	"Ora ogni studente pu&ograve; solo appartenere a un gruppo di tipo <strong>%s</strong>.";
#elif L==7
	"Teraz kazdy student moze nalezec tylko do grupy typu <strong>%s</strong>.";
#elif L==8
	"Now each student can only belong to a group of type <strong>%s</strong>.";
#endif

const char *Txt_Number_of_BR_assignments =
#if   L==0
	"Nombre<br />d'activitats";
#elif L==1
	"Anzahl der<br />Aufgaben";
#elif L==2
	"Number of<br />assignments";
#elif L==3
	"N&ordm; de<br />actividades";
#elif L==4
	"Nombre<br />d'activit&eacute;s";
#elif L==5
	"N&ordm; de<br />actividades";	// Okoteve traduccin
#elif L==6
	"Numero di<br />attivit&agrave;";
#elif L==7
	"Number of<br />assignments";	// Potrzebujesz tlumaczenie
#elif L==8
	"N&ordm; de<br />atividades";
#endif

const char *Txt_Number_of_courses =
#if   L==0
	"Nombre d'assignat.";
#elif L==1
	"Anzahl der Kursen";
#elif L==2
	"Number of courses";
#elif L==3
	"N&ordm; de asignat.";
#elif L==4
	"Nombre de mati&egrave;res";
#elif L==5
	"N&ordm; de asignat.";	// Okoteve traduccin
#elif L==6
	"Numero di corsi";
#elif L==7
	"Liczba kurs&oacute;w";
#elif L==8
	"N&ordm; de disciplinas";
#endif

const char *Txt_Number_of_BR_courses_with_BR_assignments =
#if   L==0
	"Nombre<br />d'assignatures<br />amb activitats";
#elif L==1
	"Anzahl der<br />Kursen<br />mit Aufgaben";
#elif L==2
	"Number of<br />courses with<br />assignments";
#elif L==3
	"N&ordm; de<br />asignaturas<br />con actividades";
#elif L==4
	"Nombre de<br />mati&egrave;res<br />avec activit&eacute;s";
#elif L==5
	"N&ordm; de<br />asignaturas<br />con actividades";	// Okoteve traduccin
#elif L==6
	"Numero di<br />corsi con<br />attivit&agrave;";
#elif L==7
	"Liczba<br />kurs&oacute;w z<br />zadania";
#elif L==8
	"N&ordm; de<br />disciplinas<br />com atividades";
#endif

const char *Txt_Number_of_BR_courses_with_BR_exportable_BR_test_BR_questions =
#if   L==0
	"Nombre<br />d'assignatures<br />amb preguntes<br />de test<br />exportables";
#elif L==1
	"Anzahl der<br />Kursen<br />mit exportierbar<br />Testfragen";
#elif L==2
	"Number of<br />courses with<br />exportable<br />test<br />questions";
#elif L==3
	"N&ordm; de<br />asignaturas<br />con preguntas<br />de test<br />exportables";
#elif L==4
	"Nombre de<br />mati&egrave;res<br />avec questions<br />de test<br />exportables";
#elif L==5
	"N&ordm; de<br />asignaturas<br />con preguntas<br />de test<br />exportables";	// Okoteve traduccin
#elif L==6
	"Numero di<br />corsi con<br />domande<br />de test<br />esportabili";
#elif L==7
	"Liczba<br />kurs&oacute;w<br />z eksportowane<br />pyta&nacute;<br />testowych";
#elif L==8
	"N&ordm; de<br />disciplinas<br />com quest&otilde;es<br />de test<br />export&aacute;veis";
#endif

const char *Txt_Number_of_BR_courses_BR_with_test_BR_questions =
#if   L==0
	"Nombre<br />d'assignatures<br />amb preguntes<br />de test";
#elif L==1
	"Anzahl der<br />Kursen<br />mit Testfragen";
#elif L==2
	"Number of<br />courses<br />with test<br />questions";
#elif L==3
	"N&ordm; de<br />asignaturas<br />con preguntas<br />de test";
#elif L==4
	"Nombre de<br />mati&egrave;res<br />avec questions<br />de test";
#elif L==5
	"N&ordm; de<br />asignaturas<br />con preguntas<br />de test";	// Okoteve traduccin
#elif L==6
	"Numero di<br />corsi con<br />domande<br />de test";
#elif L==7
	"Liczba<br />kurs&oacute;w<br />z pyta&nacute;<br />testowych";
#elif L==8
	"N&ordm; de<br />disciplinas<br />com quest&otilde;es<br />de test";
#endif

const char *Txt_Number_of_BR_courses_with_BR_surveys =
#if   L==0
	"Nombre<br />d'assignatures<br />amb enquestes";
#elif L==1
	"Anzahl der<br />Kursen<br />mit Umfragen";
#elif L==2
	"Number of<br />courses with<br />surveys";
#elif L==3
	"N&ordm; de<br />asignaturas<br />con encuestas";
#elif L==4
	"Nombre de<br />mati&egrave;res<br />avec sondages";
#elif L==5
	"N&ordm; de<br />asignaturas<br />con encuestas";	// Okoteve traduccin
#elif L==6
	"Numero di<br />corsi con<br />sondaggi";
#elif L==7
	"Liczba<br />kurs&oacute;w z<br />badania";
#elif L==8
	"N&ordm; de<br />disciplinas<br />com inqu&eacute;ritos";
#endif

const char *Txt_Number_of_BR_events =
#if   L==0
	"N&ordm; de<br />eventos";	// Necessita traduccio
#elif L==1
	"Anzahl der<br />Ereignisse";
#elif L==2
	"Number of<br />events";
#elif L==3
	"N&ordm; de<br />eventos";
#elif L==4
	"Nombre<br />d'&eacute;v&eacute;nements";
#elif L==5
	"N&ordm; de<br />eventos";	// Okoteve traduccin
#elif L==6
	"Numero di<br />eventi";
#elif L==7
	"Liczba of<br />zdarzenia";
#elif L==8
	"N&uacute;mero de<br />eventos";
#endif

const char *Txt_Number_of_BR_e_mails =
#if   L==0
	"N&ordm; de<br />correos";	// Necessita traduccio
#elif L==1
	"Anzahl der<br />E-Mails";
#elif L==2
	"Number of<br />e-mails";
#elif L==3
	"N&ordm; de<br />correos";
#elif L==4
	"Nombre<br />d'e-mails";
#elif L==5
	"N&ordm; de<br />correos";	// Okoteve traduccin
#elif L==6
	"Numero di<br />e-mail";
#elif L==7
	"Liczba<br />e-mail";
#elif L==8
	"N&uacute;mero de<br />e-mails";
#endif

const char *Txt_Number_of_BR_notifications =
#if   L==0
	"Nombre de<br />notificacions";
#elif L==1
	"Anzahl der<br />Anmeldungen";
#elif L==2
	"Number of<br />notifications";
#elif L==3
	"N&ordm; de<br />notificaciones";
#elif L==4
	"Nombre de<br />notifications";
#elif L==5
	"N&ordm; de<br />notificaciones";	// Okoteve traduccin
#elif L==6
	"Numero di<br />notifiche";
#elif L==7
	"Liczba<br />zgloszen";
#elif L==8
	"N&uacute;mero de<br />notifica&ccedil;&otilde;es";
#endif

const char *Txt_Number_of_students =
#if   L==0
	"Nombre d'estudiants";
#elif L==1
	"Anzahl der Studenten";
#elif L==2
	"Number of students";
#elif L==3
	"N&ordm; de estudiantes";
#elif L==4
	"Nombre d'&eacute;tudiants";
#elif L==5
	"N&ordm; de estudiantes";	// Okoteve traduccin
#elif L==6
	"Numero di studenti";
#elif L==7
	"Liczba studenci";
#elif L==8
	"N&ordm; de estudantes";
#endif

const char *Txt_Number_of_BR_surveys =
#if   L==0
	"Nombre<br />d'enquestes";
#elif L==1
	"Anzahl der<br />Umfragen";
#elif L==2
	"Number of<br />surveys";
#elif L==3
	"N&ordm; de<br />encuestas";
#elif L==4
	"Nombre de<br />sondages";
#elif L==5
	"N&ordm; de<br />encuestas";	// Okoteve traduccin
#elif L==6
	"Numero di<br />sondaggi";
#elif L==7
	"Liczba<br />badania";
#elif L==8
	"N&ordm; de<br />inqu&eacute;ritos";
#endif

const char *Txt_Number_BR_of_test_BR_questions =
#if   L==0
	"Nombre de<br />preguntes<br />de test";
#elif L==1
	"Anzahl der<br />Testfragen";
#elif L==2
	"Number<br />of test<br />questions";
#elif L==3
	"N&ordm; de<br />preguntas<br />de test";
#elif L==4
	"Nombre de<br />questions<br />de test";
#elif L==5
	"N&ordm; de<br />preguntas<br />de test";	// Okoteve traduccin
#elif L==6
	"Numero di<br />domande<br />de test";
#elif L==7
	"Liczba<br />pyta&nacute;<br />testowych";
#elif L==8
	"N&ordm; de<br />quest&otilde;es<br />de test";
#endif

const char *Txt_Numeric_BR_code_BR_ISO_3166_1 =
#if   L==0
	"C&oacute;d.<br />num&eacute;rico<br />ISO 3166-1";	// Necessita traduccio
#elif L==1
	"Numerischen<br />Code<br />ISO 3166-1";
#elif L==2
	"Numeric<br />code<br />ISO 3166-1";
#elif L==3
	"C&oacute;d.<br />num&eacute;rico<br />ISO 3166-1";
#elif L==4
	"Code<br />num&eacute;rique<br />ISO 3166-1";
#elif L==5
	"C&oacute;d.<br />num&eacute;rico<br />ISO 3166-1";	// Okoteve traduccin
#elif L==6
	"Codice<br />numerico<br />ISO 3166-1";
#elif L==7
	"Nnumerycznych<br />kod<br />ISO 3166-1";
#elif L==8
	"C&oacute;d.<br />num&eacute;rico<br />ISO 3166-1";
#endif

const char *Txt_of_PART_OF_A_TOTAL =	// Example: transferred 2 of 3 files
#if   L==0
	"de";	// Necessita traduccio
#elif L==1
	"von";
#elif L==2
	"of";
#elif L==3
	"de";
#elif L==4
	"de";
#elif L==5
	"de";	// Okoteve traduccin
#elif L==6
	"di";
#elif L==7
	"z";
#elif L==8
	"de";
#endif

const char *Txt_Office =
#if   L==0
	"Despatx";
#elif L==1
	"Raum";
#elif L==2
	"Office";
#elif L==3
	"Despacho";
#elif L==4
	"Bureau";
#elif L==5
	"Mba'apoha";
#elif L==6
	"Ufficio";
#elif L==7
	"Biuro";
#elif L==8
	"Gabinete";
#endif

const char *Txt_Official_students =
#if   L==0
	"Estudiants oficials";
#elif L==1
	"Offizielle Teilnehmer";
#elif L==2
	"Official students";
#elif L==3
	"Estudiantes oficiales";
#elif L==4
	"&Eacute;tudiants officiels";
#elif L==5
	"Estudiantes oficiales";	// Okoteve traduccin
#elif L==6
	"Studenti ufficiali";
#elif L==7
	"Dziennik student&oacute;w";
#elif L==8
	"Estudantes oficiais";
#endif

const char *Txt_one_day =
#if   L==0
	"un d&iacute;a";	// Necessita traduccio
#elif L==1
	"ein Tag";
#elif L==2
	"one day";
#elif L==3
	"un d&iacute;a";
#elif L==4
	"un jour";
#elif L==5
	"un d&iacute;a";	// Okoteve traduccin
#elif L==6
	"un giorno";
#elif L==7
	"jeden dzien";
#elif L==8
	"um dia";
#endif

const char *Txt_One_message_has_been_deleted =
#if   L==0
	"Se ha eliminado un mensaje.";			// Necessita traduccio
#elif L==1
	"One message has been deleted.";		// Need bersetzung
#elif L==2
	"One message has been deleted.";
#elif L==3
	"Se ha eliminado un mensaje.";
#elif L==4
	"One message has been deleted.";		// Besoin de traduction
#elif L==5
	"Se ha eliminado un mensaje.";			// Okoteve traduccin
#elif L==6
	"Un messaggio &egrave; stato rimosso.";
#elif L==7
	"Jedna wiadomosc zostala usunieta.";
#elif L==8
	"One message has been deleted.";		// Necessita de traduo
#endif

const char *Txt_One_user_has_been_eliminated =
#if   L==0
	"Se ha eliminado un usuario.";			// Necessita traduccio
#elif L==1
	"One user has been eliminated.";		// Need bersetzung
#elif L==2
	"One user has been eliminated.";
#elif L==3
	"Se ha eliminado un usuario.";
#elif L==4
	"One user has been eliminated.";		// Besoin de traduction
#elif L==5
	"Se ha eliminado un usuario.";			// Okoteve traduccin
#elif L==6
	"Un utente &egrave; stato rimosso.";
#elif L==7
	"Jeden uzytkownik zostal usuniety.";
#elif L==8
	"One user has been eliminated.";		// Necessita de traduo
#endif

const char *Txt_One_user_has_been_enrolled =
#if   L==0
	"Se ha inscrito un usuario.";		// Necessita traduccio
#elif L==1
	"One user has been enrolled.";		// Need bersetzung
#elif L==2
	"One user has been enrolled.";
#elif L==3
	"Se ha inscrito un usuario.";
#elif L==4
	"One user has been enrolled.";		// Besoin de traduction
#elif L==5
	"Se ha inscrito un usuario.";		// Okoteve traduccin
#elif L==6
	"Un utente &egrave; stato registrato.";
#elif L==7
	"Jeden uzytkownik zostal zarejestrowany.";
#elif L==8
	"One user has been enrolled.";		// Necessita de traduo
#endif

const char *Txt_One_user_has_been_removed =
#if   L==0
	"Se ha eliminado un usuario.";		// Necessita traduccio
#elif L==1
	"One user has been removed.";		// Need bersetzung
#elif L==2
	"One user has been removed.";
#elif L==3
	"Se ha eliminado un usuario.";
#elif L==4
	"One user has been removed.";		// Besoin de traduction
#elif L==5
	"Se ha eliminado un usuario.";		// Okoteve traduccin
#elif L==6
	"Un utente &egrave; stato rimosso.";
#elif L==7
	"Jeden z uzytkownik&oacute;w zostaly usuniete.";
#elif L==8
	"One user has been removed.";		// Necessita de traduo
#endif

const char *Txt_Online_tutoring =
#if   L==0
	"Tutor&iacute;as virtuales";	// Necessita traduccio
#elif L==1
	"Virtual Anleitungen";
#elif L==2
	"Online tutoring";
#elif L==3
	"Tutor&iacute;as virtuales";
#elif L==4
	"Tutorat en ligne";
#elif L==5
	"Tutor&iacute;as virtuales";	// Okoteve traduccin
#elif L==6
	"Tutoraggio on-line";
#elif L==7
	"Korepetycje online";
#elif L==8
	"Tutoria on-line";
#endif

const char *Txt_only_if_the_scope_is_X =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"nom&eacute;s si l'&agrave;mbit &eacute;s %s";
#elif L==1
	"nur dann, wenn der Geltungsbereich ist %s";
#elif L==2
	"only if the scope is %s";
#elif L==3
	"s&oacute;lo si el &aacute;mbito es %s";
#elif L==4
	"seulement si le port&eacute;e est %s";
#elif L==5
	"s&oacute;lo si el &aacute;mbito es %s";	// Okoteve traduccin
#elif L==6
	"solo se l'ambito &egrave; %s";
#elif L==7
	"tylko wtedy, gdy zakres jest %s";
#elif L==8
	"somente se o &acirc;mbito &eacute; %s";
#endif

const char *Txt_only_students =
#if   L==0
	", s&oacute;lo estudiantes";	// Necessita traduccio
#elif L==1
	", nur Studenten";
#elif L==2
	", only students";
#elif L==3
	", s&oacute;lo estudiantes";
#elif L==4
	", seulement &eacute;tudiants";
#elif L==5
	", s&oacute;lo estudiantes";	// Okoteve traduccin
#elif L==6
	", solo studenti";
#elif L==7
	", tylko studenci";
#elif L==8
	", apenas estudantes";
#endif

const char *Txt_only_teachers =
#if   L==0
	", professors";
#elif L==1
	", Lehrkr&auml;fte";
#elif L==2
	", teachers";
#elif L==3
	", profesores";
#elif L==4
	", enseignants";
#elif L==5
	", profesores";	// Okoteve traduccin
#elif L==6
	", professori";
#elif L==7
	", nauczycieli";
#elif L==8
	", professores";
#endif

const char *Txt_Open_documents =
#if   L==0
	"Documents oberts";
#elif L==1
	"Ge&ouml;ffnete Dokumente";
#elif L==2
	"Open documents";
#elif L==3
	"Documentos abiertos";
#elif L==4
	"Documents ouverts";
#elif L==5
	"Documentos abiertos";	// Okoteve traduccin
#elif L==6
	"Documenti aperti";
#elif L==7
	"Otwarte dokumenty";
#elif L==8
	"Documentos abertos";
#endif

const char *Txt_Open_survey =
#if   L==0
	"Enquesta oberta";
#elif L==1
	"Ge&ouml;ffneten Umfrage";
#elif L==2
	"Open survey";
#elif L==3
	"Encuesta abierta";
#elif L==4
	"Sondage ouvert";
#elif L==5
	"Encuesta abierta";	// Okoteve traduccin
#elif L==6
	"Sondaggio aperto";
#elif L==7
	"otwarte badania";
#elif L==8
	"Inqu&eacute;rito aberta";
#endif

const char *Txt_Opening_of_groups =
#if   L==0
	"Obertura de grups";
#elif L==1
	"Opening of groups";		// Need bersetzung
#elif L==2
	"Opening of groups";
#elif L==3
	"Apertura de grupos";
#elif L==4
	"Opening of groups";		// Besoin de traduction
#elif L==5
	"Apertura de grupos";		// Okoteve traduccin
#elif L==6
	"Apertura di gruppi";
#elif L==7
	"Opening of groups";		// Potrzebujesz tlumaczenie
#elif L==8
	"Opening of groups";		// Necessita de traduo
#endif

const char *Txt_Opt_BR_year =		// Year for optional courses
#if   L==0
	"&iquest;Cur.<br />opt.?";	// Necessita traduccio
#elif L==1
	"Opt.<br />year?";		// Need bersetzung
#elif L==2
	"Opt.<br />year?";
#elif L==3
	"&iquest;Cur.<br />opt.?";
#elif L==4
	"Opt.<br />year?";		// Besoin de traduction
#elif L==5
	"&iquest;Cur.<br />opt.?";	// Okoteve traduccin
#elif L==6
	"Opz.<br />anno?";
#elif L==7
	"Opt.<br />rok?";
#elif L==8
	"Opt.<br />year?";		// Necessita de traduo
#endif

const char *Txt_Option_a_Import_students_from_the_official_lists =
#if   L==0
	"Opci&oacute;n a): Obtenga estudiantes de las listas oficiales";	// Necessita traduccio
#elif L==1
	"Option a): Import students from the official lists";			// Need bersetzung
#elif L==2
	"Option a): Import students from the official lists";
#elif L==3
	"Opci&oacute;n a): Obtenga estudiantes de las listas oficiales";
#elif L==4
	"Option a): Import students from the official lists";			// Besoin de traduction
#elif L==5
	"Opci&oacute;n a): Obtenga estudiantes de las listas oficiales";	// Okoteve traduccin
#elif L==6
	"Opzione a): Importa studenti dalle liste ufficiali";
#elif L==7
	"Opcja a): Student&oacute;w Import z oficjalnej listy";
#elif L==8
	"Option a): Import students from the official lists";			// Necessita de traduo
#endif

const char *Txt_Option_b_Type_or_paste_a_list_of_users =
#if   L==0
	"Opci&oacute;n b): Escriba o pegue una lista de usuarios";	// Necessita traduccio
#elif L==1
	"Option b): Type or paste a list of users";						// Need bersetzung
#elif L==2
	"Option b): Type or paste a list of users";
#elif L==3
	"Opci&oacute;n b): Escriba o pegue una lista de usuarios";
#elif L==4
	"Option b): Type or paste a list of users";						// Besoin de traduction
#elif L==5
	"Opci&oacute;n b): Escriba o pegue una lista de usuarios";	// Okoteve traduccin
#elif L==6
	"Opzione b): Scrivi o incolla una lista di utenti";
#elif L==7
	"Opcja b): Wpisz lub wklej liste uzytkownik&oacute;w";
#elif L==8
	"Option b): Type or paste a list of users";						// Necessita de traduo
#endif

const char *Txt_Option_under_development =
#if   L==0
	"Opci&oacute; en desenvolupament.";
#elif L==1
	"Option in der Entwicklung.";
#elif L==2
	"Option under development.";
#elif L==3
	"Opci&oacute;n en desarrollo.";
#elif L==4
	"Option en cours de d&eacute;veloppement.";
#elif L==5
	"Opci&oacute;n en desarrollo.";			// Okoteve traduccin
#elif L==6
	"Opzione in fase di sviluppo.";
#elif L==7
	"Opcja w fazie rozwoju.";
#elif L==8
	"Op&ccedil;&atilde;o em desenvolvimento.";
#endif

const char *Txt_or_you_can_create_a_new_link_inside_the_folder_X =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"...o pot crear un novo enlla&ccedil;"
	" dins de la carpeta <strong>%s</strong>:";
#elif L==1
	"...Oder Sie k&ouml;nnen ein neues Link"
	" im Verzeichnis <strong>%s</strong> anlegen:";
#elif L==2
	"...or you can create a new link"
	" inside the folder <strong>%s</strong>:";
#elif L==3
	"...o puede crear un nuevo enlace"
	" dentro de la carpeta <strong>%s</strong>:";
#elif L==4
	"...ou vous pouvez cr&eacute;er un nouveau lien"
	" dans le r&eacute;pertoire <strong>%s</strong>:";
#elif L==5
	"...o puede crear un nuevo enlace"
	" dentro de la carpeta <strong>%s</strong>:";	// Okoteve traduccin
#elif L==6
	"...oppure &egrave; possibile creare una nuova cartella"
	" all'interno della cartella <strong>%s</strong>:";
#elif L==7
	"...czy mo&zdot;esz utworzy&cacute; nowy link"
	" wewn&aogon;trz folderu <strong>%s</strong>:";
#elif L==8
	"...ou voc&ecirc; pode criar uma nova liga&ccedil;&atilde;o"
	" dentro do diret&oacute;rio <strong>%s</strong>:";
#endif

const char *Txt_or_you_can_make_a_file_copy_to_the_folder_X =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"...o pot realitzar una c&ograve;pia d'arxius"
	" a la carpeta <strong>%s</strong>:";
#elif L==1
	"...Oder Sie k&ouml;nnen die Kopie einer Datei"
	" in das Verzeichnis <strong>%s</strong>:";
#elif L==2
	"...or you can make a file copy"
	" to the folder <strong>%s</strong>:";
#elif L==3
	"...o puede realizar una copia de archivos"
	" a la carpeta <strong>%s</strong>:";
#elif L==4
	"...ou vous pouvez faire une copie de fichiers"
	" vers le r&eacute;pertoire <strong>%s</strong>:";
#elif L==5
	"...o puede realizar una copia de archivos"
	" a la carpeta <strong>%s</strong>:";		// Okoteve traduccin
#elif L==6
	"...oppure &egrave; possibile effettuare una copia dei file"
	" nella cartella <strong>%s</strong>:";
#elif L==7
	"...czy mozna zrobic kopie pliku"
	" do folderu <strong>%s</strong>:";
#elif L==8
	"...ou voc&ecirc; pode fazer uma c&oacute;pia de arquivo"
	" na diret&oacute;rio <strong>%s</strong>:";
#endif

const char *Txt_or_you_can_upload_a_new_file_to_the_folder_X =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"...o puede subir un nuevo archivo"
	" a la carpeta <strong>%s</strong>:";	// Necessita traduccio
#elif L==1
	"...oder senden Sie eine neue Datei"
	" in das Verzeichnis <strong>%s</strong>:";
#elif L==2
	"...or you can upload a new file"
	" to the folder <strong>%s</strong>:";
#elif L==3
	"...o puede subir un nuevo archivo"
	" a la carpeta <strong>%s</strong>:";
#elif L==4
	"...ou vous pouvez envoyer un nouveau fichier"
	" dans le r&eacute;pertoire <strong>%s</strong>:";
#elif L==5
	"...o puede subir un nuevo archivo"
	" a la carpeta <strong>%s</strong>:";	// Okoteve traduccin
#elif L==6
	"...oppure si pu&ograve; inviare un nuovo file"
	" nella cartella <strong>%s</strong>:";
#elif L==7
	"...lub wyslac nowy plik"
	" do folderu <strong>%s</strong>:";
#elif L==8
	"...ou voc&ecirc; pode enviar um novo arquivo"
	" para o diret&oacute;rio <strong>%s</strong>:";
#endif

const char *Txt_or_you_can_upload_new_files_to_the_folder_X =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"...o puede subir nuevos archivos"
	" a la carpeta <strong>%s</strong>:";	// Necessita traduccio
#elif L==1
	"...oder senden Sie neue Dateien"
	" in das Verzeichnis <strong>%s</strong>:";
#elif L==2
	"...or you can upload new files"
	" to the folder <strong>%s</strong>:";
#elif L==3
	"...o puede subir nuevos archivos"
	" a la carpeta <strong>%s</strong>:";
#elif L==4
	"...ou vous pouvez envoyer des nouveaux fichiers"
	" dans le r&eacute;pertoire <strong>%s</strong>:";
#elif L==5
	"...o puede subir nuevos archivos"
	" a la carpeta <strong>%s</strong>:";	// Okoteve traduccin
#elif L==6
	"...oppure si pu&ograve; inviare nuovi file"
	" nella cartella <strong>%s</strong>:";
#elif L==7
	"...lub wyslac nowy plik"
	" do folderu <strong>%s</strong>:";
#elif L==8
	"...ou voc&ecirc; pode enviar novos arquivos"
	" para o diret&oacute;rio <strong>%s</strong>:";
#endif

const char *Txt_Original_message =
#if   L==0
	"Mensaje original";	// Necessita traduccio
#elif L==1
	"Original-Nachricht";
#elif L==2
	"Original message";
#elif L==3
	"Mensaje original";
#elif L==4
	"Message d'origine";
#elif L==5
	"Mensaje original";	// Okoteve traduccin
#elif L==6
	"Messaggio originale";
#elif L==7
	"Oryginalna wiadomo&sacute;&cacute;";
#elif L==8
	"Mensagem original";
#endif

const char *Txt_Other_countries =
#if   L==0
	"Altres pa&iuml;sos";
#elif L==1
	"Andere L&auml;nder";
#elif L==2
	"Other countries";
#elif L==3
	"Otros pa&iacute;ses";
#elif L==4
	"Autres pays";
#elif L==5
	"Otros pa&iacute;ses";	// Okoteve traduccin
#elif L==6
	"Altri paesi";
#elif L==7
	"Inne kraje";
#elif L==8
	"Outros pa&iacute;ses";
#endif

const char *Txt_Other_departments =
#if   L==0
	"Altres departaments";
#elif L==1
	"Andere Abteilung";
#elif L==2
	"Other departments";
#elif L==3
	"Otros departamentos";
#elif L==4
	"Autres d&eacute;partements";
#elif L==5
	"Otros departamentos";	// Okoteve traduccin
#elif L==6
	"Altri dipartimenti";
#elif L==7
	"Inne wydzialy";
#elif L==8
	"Outros departamentos";
#endif

const char *Txt_Other_emails =
#if   L==0
	"Altres correus";
#elif L==1
	"Andere E-Mails";
#elif L==2
	"Other e-mails";
#elif L==3
	"Otros correos";
#elif L==4
	"Autres e-mails";
#elif L==5
	"Otros correos";	// Okoteve traduccin
#elif L==6
	"Altre e-mail";
#elif L==7
	"Inne e-mail";
#elif L==8
	"Outros e-mails";
#endif

const char *Txt_Other_nicknames =
#if   L==0
	"Altres sobrenoms";
#elif L==1
	"Andere Benutzername";
#elif L==2
	"Other nicknames";
#elif L==3
	"Otros apodos";
#elif L==4
	"Autres surnoms";
#elif L==5
	"Otros apodos";	// Okoteve traduccin
#elif L==6
	"Altri nomi utenti";
#elif L==7
	"Inne przydomki";
#elif L==8
	"Outras alcunha";
#endif

const char *Txt_Other_places =
#if   L==0
	"Altres llocs";
#elif L==1
	"Andere Standorte";
#elif L==2
	"Other places";
#elif L==3
	"Otros lugares";
#elif L==4
	"Autres emplacements";
#elif L==5
	"Otros lugares";	// Okoteve traduccin
#elif L==6
	"Altre localit&agrave;";
#elif L==7
	"Inne miejsca";
#elif L==8
	"Outras localiza&ccedil;oes";
#endif

const char *Txt_Other_recipients =
#if   L==0
	"Otros destinatarios";	// Necessita traduccio
#elif L==1
	"Andere Empf&auml;nger";
#elif L==2
	"Other recipients";
#elif L==3
	"Otros destinatarios";
#elif L==4
	"Autres destinataires";
#elif L==5
	"Otros destinatarios";	// Okoteve traduccin
#elif L==6
	"Altri destinatari";
#elif L==7
	"Inne odbiorc&oacute;w";
#elif L==8
	"Outros destinat&aacute;rios";
#endif

const char *Txt_out_of_PART_OF_A_SCORE =	// Example: score is 2 out of 10
#if   L==0
	"sobre";	// Necessita traduccio
#elif L==1
	"von";
#elif L==2
	"out of";
#elif L==3
	"sobre";
#elif L==4
	"sur";
#elif L==5
	"sobre";	// Okoteve traduccin
#elif L==6
	"su";
#elif L==7
	"obecnie";
#elif L==8
	"out of";
#endif

const char *Txt_Page =
#if   L==0
	"P&agrave;g.";
#elif L==1
	"Page";
#elif L==2
	"Page";
#elif L==3
	"P&aacute;g.";
#elif L==4
	"Page";
#elif L==5
	"Kuatiarogue";
#elif L==6
	"Pag.";
#elif L==7
	"Strona";
#elif L==8
	"P&aacute;g.";
#endif

const char *Txt_page =
#if   L==0
	"p&agrave;g.";
#elif L==1
	"Page";
#elif L==2
	"page";
#elif L==3
	"p&aacute;g.";
#elif L==4
	"page";
#elif L==5
	"kuatiarogue";
#elif L==6
	"pag.";
#elif L==7
	"strona";
#elif L==8
	"p&aacute;g.";
#endif

const char *Txt_PAGE1_Page_generated_in =
#if   L==0
	"P&agrave;gina generada en";
#elif L==1
	"Page generated in";
#elif L==2
	"Page generated in";
#elif L==3
	"P&aacute;gina generada en";
#elif L==4
	"Page g&eacute;n&eacute;r&eacute;e en";
#elif L==5
	"P&aacute;gina generada en";	// Okoteve traduccin
#elif L==6
	"Pagina generata in";
#elif L==7
	"Strona wygenerowana w";
#elif L==8
	"P&aacute;gina gerada em";
#endif

const char *Txt_PAGE2_and_sent_in =
#if   L==0
	"i enviat en";
#elif L==1
	"gesendet und in";
#elif L==2
	"and sent in";
#elif L==3
	"y enviada en";
#elif L==4
	"et envoy&eacute;e en";
#elif L==5
	"y enviada en";	// Okoteve traduccin
#elif L==6
	"e inviata in";
#elif L==7
	"i wysylane w";
#elif L==8
	"e enviada em";
#endif

const char *Txt_PAGES_Previous =
#if   L==0
	"Anteriores";	// Necessita traduccio
#elif L==1
	"Zur&uuml;ck";
#elif L==2
	"Previous";
#elif L==3
	"Anteriores";
#elif L==4
	"Pr&eacute;c&eacute;dentes";
#elif L==5
	"Anteriores";	// Okoteve traduccin
#elif L==6
	"Precedenti";
#elif L==7
	"Poprzedni";
#elif L==8
	"Anteriores";
#endif

const char *Txt_PAGES_Next =
#if   L==0
	"Siguientes";	// Necessita traduccio
#elif L==1
	"N&auml;chste";
#elif L==2
	"Next";
#elif L==3
	"Siguientes";
#elif L==4
	"Suivantes";
#elif L==5
	"Siguientes";	// Okoteve traduccin
#elif L==6
	"Seguenti";
#elif L==7
	"Nastepny";
#elif L==8
	"Seguintes";
#endif

const char *Txt_Password =
#if   L==0
	"Contrasenya";
#elif L==1
	"Passwort";
#elif L==2
	"Password";
#elif L==3
	"Contrase&ntilde;a";
#elif L==4
	"Mot de passe";
#elif L==5
	"Tera&ntilde;emi";
#elif L==6
	"Password";
#elif L==7
	"Has&lstrok;o";
#elif L==8
	"Senha";
#endif

const char *Txt_Paste =
#if   L==0
	"Pegar";	// Necessita traduccio
#elif L==1
	"Einf&uuml;gen";
#elif L==2
	"Paste";
#elif L==3
	"Pegar";
#elif L==4
	"Coller";
#elif L==5
	"Pegar";	// Okoteve traduccin
#elif L==6
	"Incolla";
#elif L==7
	"Wklej";
#elif L==8
	"Colar";
#endif

const char *Txt_Paste_in_X =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"Pegar en %s";	// Necessita traduccio
#elif L==1
	"Einf&uuml;gen an %s";
#elif L==2
	"Paste in %s";
#elif L==3
	"Pegar en %s";
#elif L==4
	"Coller dans %s";
#elif L==5
	"Pegar en %s";	// Okoteve traduccin
#elif L==6
	"Incolla in %s";
#elif L==7
	"Wklej w %s";
#elif L==8
	"Colar em %s";
#endif

const char *Txt_Paste_thread =
#if   L==0
	"Pegar discusi&oacute;n";	// Necessita traduccio
#elif L==1
	"Einf&uuml;gen Thread";
#elif L==2
	"Paste thread";
#elif L==3
	"Pegar discusi&oacute;n";
#elif L==4
	"Coller fil";
#elif L==5
	"Pegar discusi&oacute;n";	// Okoteve traduccin
#elif L==6
	"Incolla discussione";
#elif L==7
	"Wklej watku";
#elif L==8
	"Colar thread";
#endif

const char *Txt_PERCENT_of_users =
#if   L==0
	"% de usuarios";	// Necessita traduccio
#elif L==1
	"% der Benutzer";
#elif L==2
	"% of users";
#elif L==3
	"% de usuarios";
#elif L==4
	"% d'utilisateurs";
#elif L==5
	"% de usuarios";	// Okoteve traduccin
#elif L==6
	"% di utenti";
#elif L==7
	"% uzytkownik&oacute;w";
#elif L==8
	"% de utilizadores";
#endif

const char *Txt_Phone =
#if   L==0
	"Tel&egrave;fon";
#elif L==1
	"Telefon";
#elif L==2
	"Phone";
#elif L==3
	"Tel&eacute;fono";
#elif L==4
	"T&eacute;l&eacute;phone";
#elif L==5
	"Pumbyry";
#elif L==6
	"Telefono";
#elif L==7
	"Telefon";
#elif L==8
	"Telefone";
#endif

const char *Txt_Photo =
#if   L==0
	"Foto";
#elif L==1
	"Foto";
#elif L==2
	"Photo";
#elif L==3
	"Foto";
#elif L==4
	"Photo";
#elif L==5
	"Ta'anga";
#elif L==6
	"Foto";
#elif L==7
	"Zdj&eogon;cie";
#elif L==8
	"Foto";
#endif

const char *Txt_Photo_has_been_updated =
#if   L==0
	"La fotograf&iacute;a se ha actualizado.";	// Necessita traduccio
#elif L==1
	"Photo has been updated.";			// Need bersetzung
#elif L==2
	"Photo has been updated.";
#elif L==3
	"La fotograf&iacute;a se ha actualizado.";
#elif L==4
	"Photo has been updated.";			// Besoin de traduction
#elif L==5
	"La fotograf&iacute;a se ha actualizado.";	// Okoteve traduccin
#elif L==6
	"La foto &egrave; stata aggiornata.";
#elif L==7
	"Zdjecie zostalo zaktualizowane.";
#elif L==8
	"Photo has been updated.";			// Necessita de traduo
#endif

const char *Txt_Photo_of_THE_USER_X_has_been_removed =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"Se ha eliminado la foto de <strong>%s</strong>.";	// Necessita traduccio
#elif L==1
	"Photo of <strong>%s</strong> has been removed.";	// Need bersetzung
#elif L==2
	"Photo of <strong>%s</strong> has been removed.";
#elif L==3
	"Se ha eliminado la foto de <strong>%s</strong>.";
#elif L==4
	"Photo of <strong>%s</strong> has been removed.";	// Besoin de traduction
#elif L==5
	"Se ha eliminado la foto de <strong>%s</strong>.";	// Okoteve traduccin
#elif L==6
	"La foto de <strong>%s</strong> &egrave; stata rimossa.";
#elif L==7
	"Zdjecie <strong>%s</strong> zostalo usuniete.";
#elif L==8
	"Photo of <strong>%s</strong> has been removed.";	// Necessita de traduo
#endif

const char *Txt_PHOTO_PROCESSING_CAPTIONS[3] =
   {
#if   L==0
	 "Rostro seleccionado"		// Necessita traduccio
#elif L==1
	 "Face selected"			// Need bersetzung
#elif L==2
	 "Face selected"
#elif L==3
	 "Rostro seleccionado"
#elif L==4
	 "Face selected"			// Besoin de traduction
#elif L==5
	 "Rostro seleccionado"		// Okoteve traduccin
#elif L==6
	 "Viso selezionato"
#elif L==7
	 "Twarze wybranych"
#elif L==8
	 "Face selected"			// Necessita de traduo
#endif
	 ,
#if   L==0
	 "Contraste y saturaci&oacute;n mejorados"	// Necessita traduccio
#elif L==1
	 "Contrast and saturation enhanced"		// Need bersetzung
#elif L==2
	 "Contrast and saturation enhanced"
#elif L==3
	 "Contraste y saturaci&oacute;n mejorados"
#elif L==4
	 "Contrast and saturation enhanced"		// Besoin de traduction
#elif L==5
	 "Contraste y saturaci&oacute;n mejorados"	// Okoteve traduccin
#elif L==6
	 "Contrasto e saturazione migliorata"
#elif L==7
	 "Zwiekszenie kontrastu i nasycenia"
#elif L==8
	 "Contrast and saturation enhanced"		// Necessita de traduo
#endif
	 ,
#if   L==0
	 "Balance de blancos realizado"		// Necessita traduccio
#elif L==1
	 "White balance made"			// Need bersetzung
#elif L==2
	 "White balance made"
#elif L==3
	 "Balance de blancos realizado"
#elif L==4
	 "White balance made"			// Besoin de traduction
#elif L==5
	 "Balance de blancos realizado"		// Okoteve traduccin
#elif L==6
	 "Bilanciamento di bianco realizzato"
#elif L==7
	 "Balans bieli sie"
#elif L==8
	 "White balance made"			// Necessita de traduo
#endif
	};

const char *Txt_Photo_removed =
#if   L==0
	"Fotograf&iacute;a eliminada.";	// Necessita traduccio
#elif L==1
	"Foto entfernt.";
#elif L==2
	"Photo removed.";
#elif L==3
	"Fotograf&iacute;a eliminada.";
#elif L==4
	"Photo removed.";		// Besoin de traduction
#elif L==5
	"Fotograf&iacute;a eliminada.";	// Okoteve traduccin
#elif L==6
	"Foto rimossa.";
#elif L==7
	"Zdjecie usuniete.";
#elif L==8
	"Photo removed.";		// Necessita de traduo
#endif

const char *Txt_photos =
#if   L==0
	"fotos";
#elif L==1
	"Fotos";
#elif L==2
	"photos";
#elif L==3
	"fotos";
#elif L==4
	"photos";
#elif L==5
	"fotos";	// Okoteve traduccin
#elif L==6
	"foto";
#elif L==7
	"zdjecia";
#elif L==8
	"fotos";
#endif

const char *Txt_Place =		// A particular spot or area
#if   L==0
	"Lloc";
#elif L==1
	"Standort";
#elif L==2
	"Place";
#elif L==3
	"Lugar";
#elif L==4
	"Emplacement";
#elif L==5
	"Tenda";
#elif L==6
	"Luogo";
#elif L==7
	"Miejsce";
#elif L==8
	"Localiza&ccedil;&atilde;o";
#endif

const char *Txt_Place_of_origin =
#if   L==0
	"Lugar de origen";	// Necessita traduccio
#elif L==1
	"Herkunftsort";
#elif L==2
	"Place of origin";
#elif L==3
	"Lugar de origen";
#elif L==4
	"Lieu d'origine";
#elif L==5
	"Lugar de origen";	// Okoteve traduccin
#elif L==6
	"Luogo d'origine";
#elif L==7
	"Miejsce pochodzenia";
#elif L==8
	"Lugar de origem";
#endif

const char *Txt_Place_X_removed =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"Lugar <strong>%s</strong> eliminado.";		// Necessita traduccio
#elif L==1
	"Standort <strong>%s</strong> entfernt.";
#elif L==2
	"Place <strong>%s</strong> removed.";
#elif L==3
	"Lugar <strong>%s</strong> eliminado.";
#elif L==4
	"Emplacement <strong>%s</strong> supprim&eacute;.";
#elif L==5
	"Lugar <strong>%s</strong> eliminado.";		// Okoteve traduccin
#elif L==6
	"Luogo <strong>%s</strong> rimosso.";
#elif L==7
	"Miejsce <strong>%s</strong> usuniete.";
#elif L==8
	"Place <strong>%s</strong> removed.";		// Necessita de traduo
#endif

const char *Txt_Place_unspecified =
#if   L==0
	"Lloc sense especificar";
#elif L==1
	"Standort nicht spezifiziert";
#elif L==2
	"Place unspecified";
#elif L==3
	"Lugar sin especificar";
#elif L==4
	"Emplacement non sp&eacute;cifi&eacute;";
#elif L==5
	"Lugar sin especificar";	// Okoteve traduccin
#elif L==6
	"Luogo non specificato";
#elif L==7
	"Miejsce nieokreslone";
#elif L==8
	"Localiza&ccedil;&atilde;o indeterminada";
#endif

const char *Txt_Places =
#if   L==0
	"Llocs";
#elif L==1
	"Standorte";
#elif L==2
	"Places";
#elif L==3
	"Lugares";
#elif L==4
	"Emplacements";
#elif L==5
	"Tenda";
#elif L==6
	"Localit&grave;";
#elif L==7
	"Miejsca";
#elif L==8
	"Localiza&ccedil;oes";
#endif

const char *Txt_PLACES_HELP_ORDER[2] =
   {
#if   L==0
	 "Ordenar por lugar"	// Necessita traduccio
#elif L==1
	 "Nach Standort sortieren"
#elif L==2
	 "Sort by place"
#elif L==3
	 "Ordenar por lugar"
#elif L==4
	 "Trier par emplacement"
#elif L==5
	 "Ordenar por lugar"	// Okoteve traduccin
#elif L==6
	 "Ordina per localit&grave;"
#elif L==7
	 "Sortuj wedlug miejsca"
#elif L==8
	 "Classificar por localiza&ccedil;&atilde;o"
#endif
	 ,
#if   L==0
	 "Ordenar por n&ordm; de titulaciones"	// Necessita traduccio
#elif L==1
	 "Nach Anzahl der Studieng&auml;nge sortieren"
#elif L==2
	 "Sort by no. of degrees"
#elif L==3
	 "Ordenar por n&ordm; de titulaciones"
#elif L==4
	 "Trier par nombre d'&eacute;tudes"
#elif L==5
	 "Ordenar por n&ordm; de titulaciones"	// Okoteve traduccin
#elif L==6
	 "Ordina per numero di lauree"
#elif L==7
	 "Sortuj wedlug stopni"
#elif L==8
	 "Classificar por n&ordm; de titula&ccedil;oes"
#endif
	};

const char *Txt_PLACES_ORDER[2] =
   {
#if   L==0
	 "Lloc"
#elif L==1
	 "Standort"
#elif L==2
	 "Place"
#elif L==3
	 "Lugar"
#elif L==4
	 "Emplacement"
#elif L==5
	 "Tenda"
#elif L==6
	 "Luogo"
#elif L==7
	 "Miejsce"
#elif L==8
	 "Localiza&ccedil;&atilde;o"
#endif
	 ,
#if   L==0
	 "Titulacions"
#elif L==1
	 "Studieng&auml;nge"
#elif L==2
	 "Degrees"
#elif L==3
	 "Titulaciones"
#elif L==4
	 "&Eacute;tudes"
#elif L==5
	 "Titulaciones"	// Okoteve traduccin
#elif L==6
	 "Lauree"
#elif L==7
	 "Stopni"
#elif L==8
	 "Titula&ccedil;oes"
#endif
	};

const char *Txt_Please_specify_if_you_allow_access_to_test_questions_from_mobile_applications =
#if   L==0
	"Por favor, especifique si permite el acceso"
	" a las preguntas de test desde aplicaciones m&oacute;viles.";	// Necessita traduccio
#elif L==1
	"Please specify if you allow access"
	" to test questions from mobile applications.";	// Need bersetzung
#elif L==2
	"Please specify if you allow access"
	" to test questions from mobile applications.";
#elif L==3
	"Por favor, especifique si permite el acceso"
	" a las preguntas de test desde aplicaciones m&oacute;viles.";
#elif L==4
	"Please specify if you allow access"
	" to test questions from mobile applications.";	// Besoin de traduction
#elif L==5
	"Por favor, especifique si permite el acceso"
	" a las preguntas de test desde aplicaciones m&oacute;viles.";	// Okoteve traduccin
#elif L==6
	"Please specify if you allow access"
	" to test questions from mobile applications.";	// Bisogno di traduzione
#elif L==7
	"Please specify if you allow access"
	" to test questions from mobile applications.";	// Potrzebujesz tlumaczenie
#elif L==8
	"Please specify if you allow access"
	" to test questions from mobile applications.";	// Necessita de traduo
#endif

const char *Txt_Plugin =
#if   L==0
	"Complement";
#elif L==1
	"Plugin";
#elif L==2
	"Plugin";
#elif L==3
	"Complemento";
#elif L==4
	"Plugin";
#elif L==5
	"Complemento";	// Okoteve traduccin
#elif L==6
	"Plugin";
#elif L==7
	"Wtyczka";
#elif L==8
	"Plugin";
#endif

const char *Txt_Plugin_X_removed =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"Complemento <strong>%s</strong> eliminado.";	// Necessita traduccio
#elif L==1
	"Plugin <strong>%s</strong> entfernt.";
#elif L==2
	"Plugin <strong>%s</strong> removed.";
#elif L==3
	"Complemento <strong>%s</strong> eliminado.";
#elif L==4
	"Plugin <strong>%s</strong> removed.";		// Besoin de traduction
#elif L==5
	"Complemento <strong>%s</strong> eliminado.";	// Okoteve traduccin
#elif L==6
	"Plugin <strong>%s</strong> rimosso.";
#elif L==7
	"Wtyczka <strong>%s</strong> usunieta.";
#elif L==8
	"Plugin <strong>%s</strong> removed.";		// Necessita de traduo
#endif

const char *Txt_Plugins =
#if   L==0
	"Complements";
#elif L==1
	"Plugins";
#elif L==2
	"Plugins";
#elif L==3
	"Complementos";
#elif L==4
	"Plugins";
#elif L==5
	"Complementos";		// Okoteve traduccin
#elif L==6
	"Plugin";
#elif L==7
	"wtyczki";
#elif L==8
	"Plugins";
#endif

const char *Txt_Post_X_allowed =	// Warning: it is very important to include %u in the following sentences
#if   L==0
	"Mensaje %u permitido.";	// Necessita traduccio
#elif L==1
	"Post %u allowed.";		// Need bersetzung
#elif L==2
	"Post %u allowed.";
#elif L==3
	"Mensaje %u permitido.";
#elif L==4
	"Post %u allowed.";		// Besoin de traduction
#elif L==5
	"Mensaje %u permitido.";	// Okoteve traduccin
#elif L==6
	"Post %u autorizzato.";
#elif L==7
	"Post %u dozwolone.";
#elif L==8
	"Post %u allowed.";		// Necessita de traduo
#endif

const char *Txt_Post_X_allowed_Click_to_ban_it =	// Warning: it is very important to include %u in the following sentences
#if   L==0
	"Mensaje %u permitido. Pulse para censurarlo.";	// Necessita traduccio
#elif L==1
	"Post %u allowed. Click to ban it.";		// Need bersetzung
#elif L==2
	"Post %u allowed. Click to ban it.";
#elif L==3
	"Mensaje %u permitido. Pulse para censurarlo.";
#elif L==4
	"Post %u allowed. Click to ban it.";		// Besoin de traduction
#elif L==5
	"Mensaje %u permitido. Pulse para censurarlo.";	// Okoteve traduccin
#elif L==6
	"Post %u autorizzato. Clicca per censurarlo.";
#elif L==7
	"Post% u dozwolone. Kliknij, aby to zakazac.";
#elif L==8
	"Post %u allowed. Click to ban it.";		// Necessita de traduo
#endif

const char *Txt_Post_and_thread_removed =
#if   L==0
	"Mensaje y discusi&oacute;n eliminados.";	// Necessita traduccio
#elif L==1
	"Nachricht und Faden entfernt.";
#elif L==2
	"Post and thread removed.";
#elif L==3
	"Mensaje y discusi&oacute;n eliminados.";
#elif L==4
	"Post and thread removed.";			// Besoin de traduction
#elif L==5
	"Mensaje y discusi&oacute;n eliminados.";	// Okoteve traduccin
#elif L==6
	"Post e discussione rimossi.";
#elif L==7
	"Post watku usuniety.";
#elif L==8
	"Post and thread removed.";			// Necessita de traduo
#endif

const char *Txt_Post_banned =
#if   L==0
	"Mensaje no permitido";		// Necessita traduccio
#elif L==1
	"Post banned.";			// Need bersetzung
#elif L==2
	"Post banned.";
#elif L==3
	"Mensaje no permitido";
#elif L==4
	"Post banned.";			// Besoin de traduction
#elif L==5
	"Mensaje no permitido";		// Okoteve traduccin
#elif L==6
	"Post censurato.";
#elif L==7
	"Post zakazany.";
#elif L==8
	"Post banned.";			// Necessita de traduo
#endif

const char *Txt_Post_X_banned =	// Warning: it is very important to include %u in the following sentences
#if   L==0
	"Mensaje %u no permitido";	// Necessita traduccio
#elif L==1
	"Post %u banned.";		// Need bersetzung
#elif L==2
	"Post %u banned.";
#elif L==3
	"Mensaje %u no permitido";
#elif L==4
	"Post %u banned.";		// Besoin de traduction
#elif L==5
	"Mensaje %u no permitido";	// Okoteve traduccin
#elif L==6
	"Post %u censurato.";
#elif L==7
	"Post %u zakazany.";
#elif L==8
	"Post %u banned.";		// Necessita de traduo
#endif

const char *Txt_Post_X_banned_Click_to_unban_it =	// Warning: it is very important to include %u in the following sentences
#if   L==0
	"Mensaje %u no permitido. Pulse para permitirlo.";	// Necessita traduccio
#elif L==1
	"Post %u banned. Click to unban it.";		// Need bersetzung
#elif L==2
	"Post %u banned. Click to unban it.";
#elif L==3
	"Mensaje %u no permitido. Pulse para permitirlo.";
#elif L==4
	"Post %u banned. Click to unban it.";		// Besoin de traduction
#elif L==5
	"Mensaje %u no permitido. Pulse para permitirlo.";	// Okoteve traduccin
#elif L==6
	"Post %u censurato. Clicca per autorizzarlo.";
#elif L==7
	"Post %u zakazany.Kliknij, aby to odbanowac.";
#elif L==8
	"Post %u banned. Click to unban it.";		// Necessita de traduo
#endif

const char *Txt_Post_sent =
#if   L==0
	"Mensaje enviado.";	// Necessita traduccio
#elif L==1
	"Post sent.";		// Need bersetzung
#elif L==2
	"Post sent.";
#elif L==3
	"Mensaje enviado.";
#elif L==4
	"Post sent.";		// Besoin de traduction
#elif L==5
	"Mensaje enviado.";	// Okoteve traduccin
#elif L==6
	"Post inviato.";
#elif L==7
	"Post wyslany.";
#elif L==8
	"Post sent.";		// Necessita de traduo
#endif

const char *Txt_Post_removed =
#if   L==0
	"Mensaje eliminado.";		// Necessita traduccio
#elif L==1
	"Nachricht entfernt.";
#elif L==2
	"Post removed.";
#elif L==3
	"Mensaje eliminado.";
#elif L==4
	"Post removed.";		// Besoin de traduction
#elif L==5
	"Mensaje eliminado.";		// Okoteve traduccin
#elif L==6
	"Post rimosso.";
#elif L==7
	"Post usuniety.";
#elif L==8
	"Post removed.";		// Necessita de traduo
#endif

const char *Txt_Post_unbanned =
#if   L==0
	"Mensaje permitido";		// Necessita traduccio
#elif L==1
	"Post unbanned.";		// Need bersetzung
#elif L==2
	"Post unbanned";
#elif L==3
	"Mensaje permitido";
#elif L==4
	"Post unbanned.";		// Besoin de traduction
#elif L==5
	"Mensaje permitido";		// Okoteve traduccin
#elif L==6
	"Post autorizzato.";
#elif L==7
	"Post odbanowany";
#elif L==8
	"Post unbanned.";		// Necessita de traduo
#endif

const char *Txt_Preferences =
#if   L==0
	"Prefer&egrave;ncies";
#elif L==1
	"Einstellungen";
#elif L==2
	"Preferences";
#elif L==3
	"Preferencias";
#elif L==4
	"Pr&eacute;f&eacute;rences";
#elif L==5
	"Preferencias";	// Okoteve traduccin
#elif L==6
	"Preferenze";
#elif L==7
	"Preferencje";
#elif L==8
	"Prefer&ecirc;ncias";
#endif

const char *Txt_Present =
#if   L==0
	"Present";
#elif L==1
	"Vorhanden";
#elif L==2
	"Present";
#elif L==3
	"Presente";
#elif L==4
	"Pr&eacute;sent";
#elif L==5
	"Presente";	// Okoteve traduccin
#elif L==6
	"Presente";
#elif L==7
	"Obecny";
#elif L==8
	"Presente";
#endif

const char *Txt_Presents =
#if   L==0
	"Presents";
#elif L==1
	"Vorhanden";
#elif L==2
	"Presents";
#elif L==3
	"Presentes";
#elif L==4
	"Pr&eacute;sents";
#elif L==5
	"Presentes";	// Okoteve traduccin
#elif L==6
	"Presenti";
#elif L==7
	"Obecny";
#elif L==8
	"Presentes";
#endif

const char *Txt_Print_view =
#if   L==0
	"Vista d'impressi&oacute;";
#elif L==1
	"Druckansicht";
#elif L==2
	"Print view";
#elif L==3
	"Vista de impresi&oacute;n";
#elif L==4
	"Vue d'impression";
#elif L==5
	"Vista de impresi&oacute;n";	// Okoteve traduccin
#elif L==6
	"Anteprima di stampa";
#elif L==7
	"Widok do druku";
#elif L==8
	"Impress&atilde;o vista";
#endif

const char *Txt_Private_available_to_certain_users_identified =
#if   L==0
	"Privat, disponible per a certs usuaris identificats";
#elif L==1
	"Private, verf&uuml;gbar auf bestimmte Benutzer identifiziert";
#elif L==2
	"Private, available to certain users identified";
#elif L==3
	"Privado, disponible para ciertos usuarios identificados";
#elif L==4
	"Priv&eacute;, &agrave; la disposition de certains utilisateurs identifi&eacute;s";
#elif L==5
	"Privado, disponible para ciertos usuarios identificados";	// Okoteve traduccin
#elif L==6
	"Privato, a disposizione degli utenti individuati alcuni";
#elif L==7
	"Prywatne, dost&eogon;pne dla okre&sacute;lonych u&zdot;ytkownik&oacute;w zidentyfikowanych";
#elif L==8
	"Privado, dispon&iacute;vel para alguns usu&aacute;rios identificados";
#endif

const char *Txt_Private_Photo =
#if   L==0
	"Foto privada";
#elif L==1
	"Private Foto";
#elif L==2
	"Private photo";
#elif L==3
	"Foto privada";
#elif L==4
	"Photo priv&eacute;e";
#elif L==5
	"Foto privada";	// Okoteve traduccin
#elif L==6
	"Foto privata";
#elif L==7
	"Prywatne zdjecia";
#elif L==8
	"Foto privada";
#endif

const char *Txt_Private_storage_zone =
#if   L==0
	"Zona d'emmagatzematge privada";
#elif L==1
	"Privater Speicherbereich";
#elif L==2
	"Private storage area";
#elif L==3
	"Zona de almacenamiento privada";
#elif L==4
	"Zone de stockage priv&eacute;e";
#elif L==5
	"Zona de almacenamiento privada";	// Okoteve traduccin
#elif L==6
	"Area di archiviazione privata";
#elif L==7
	"Prywatny magazyn";
#elif L==8
	"Zona privada de armazenamento";
#endif

const char *Txt_private_storage_zone =
#if   L==0
	"zona de almacenamiento privada";	// Necessita traduccio
#elif L==1
	"privater Speicherbereich";
#elif L==2
	"private storage area";
#elif L==3
	"zona de almacenamiento privada";
#elif L==4
	"zone de stockage priv&eacute;e";
#elif L==5
	"zona de almacenamiento privada";	// Okoteve traduccin
#elif L==6
	"area di archiviazione privata";
#elif L==7
	"obszar prywatnego przechowywania";
#elif L==8
	"zona privada de armazenamento";
#endif

const char *Txt_post =
#if   L==0
	"missatge";
#elif L==1
	"Nachricht";
#elif L==2
	"post";
#elif L==3
	"mensaje";
#elif L==4
	"message";
#elif L==5
	"marandu";
#elif L==6
	"post";
#elif L==7
	"post";
#elif L==8
	"post";
#endif

const char *Txt_posts =
#if   L==0
	"missatges";
#elif L==1
	"Nachrichten";
#elif L==2
	"posts";
#elif L==3
	"mensajes";
#elif L==4
	"messages";
#elif L==5
	"marandu";
#elif L==6
	"post";
#elif L==7
	"posty";
#elif L==8
	"posts";
#endif

const char *Txt_Public_photo =
#if   L==0
	"Foto p&uacute;blica";
#elif L==1
	"&Ouml;ffentliches Foto";
#elif L==2
	"Public photo";
#elif L==3
	"Foto p&uacute;blica";
#elif L==4
	"Photo public";
#elif L==5
	"Foto p&uacute;blica";	// Okoteve traduccin
#elif L==6
	"Foto pubblica";
#elif L==7
	"Zdj&eogon;cie publiczne";
#elif L==8
	"Foto p&uacute;blica";
#endif

const char *Txt_Public_open_educational_resource_OER_for_everyone =
#if   L==0
	"P&uacute;blic, recurs educatiu obert (OER) per a tothom";
#elif L==1
	"&Ouml;ffentliche, Open Educational Resource (OER) f&uuml;r jedermann";
#elif L==2
	"Public, open educational resource for everyone";
#elif L==3
	"P&uacute;blico, recurso educativo abierto (OER) para todo el mundo";
#elif L==4
	"Public, ressource &eacute;ducatif libre (REL) pour tout le monde";
#elif L==5
	"P&uacute;blico, recurso educativo abierto (OER) para todo el mundo";	// Okoteve traduccin
#elif L==6
	"Pubblico, risorse didattici aperto (OER) per tutti";
#elif L==7
	"Publiczne, otwarte zasoby edukacyjne (OER) dla ka&zdot;dego";
#elif L==8
	"P&uacute;blico, recurso educacional aberto (OER) para todos";
#endif

const char *Txt_Public_views =
#if   L==0
	"Accessos p&uacute;blics";
#elif L==1
	"&Ouml;ffentliche Ansichten";
#elif L==2
	"Public views";
#elif L==3
	"Accesos p&uacute;blicos";
#elif L==4
	"Vues publics";
#elif L==5
	"Accesos p&uacute;blicos";	// Okoteve traduccin
#elif L==6
	"Visualizzazioni pubblici";
#elif L==7
	"Publiczne ods&lstrok;on";
#elif L==8
	"Acessos p&uacute;blicos";
#endif

const char *Txt_QR_code =
#if   L==0
	"Codi QR";
#elif L==1
	"QR-Code";
#elif L==2
	"QR code";
#elif L==3
	"C&oacute;digo QR";
#elif L==4
	"QR Code";
#elif L==5
	"C&oacute;digo QR";	// Okoteve traduccin
#elif L==6
	"Codice QR";
#elif L==7
	"QR Code";
#elif L==8
	"C&oacute;digo QR";
#endif

const char *Txt_Question =
#if   L==0
	"Pregunta";	// Necessita traduccio
#elif L==1
	"Frage";
#elif L==2
	"Question";
#elif L==3
	"Pregunta";
#elif L==4
	"Question";
#elif L==5
	"Pregunta";	// Okoteve traduccin
#elif L==6
	"Domanda";
#elif L==7
	"Pytanie";
#elif L==8
	"Quest&atilde;o";
#endif

const char *Txt_Question_code_X =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"Pregunta amb codi %ld";
#elif L==1
	"Frage Code %ld";
#elif L==2
	"Code %ld";
#elif L==3
	"Pregunta con c&oacute;digo %ld";
#elif L==4
	"Code de question %ld";
#elif L==5
	"Pregunta con c&oacute;digo %ld";	// Okoteve traduccin
#elif L==6
	"Codice di domanda %ld";
#elif L==7
	"Kodeks pytanie %ld";
#elif L==8
	"Quest&atilde;o com c&oacute;digo %ld";
#endif

const char *Txt_Question_modified =
#if   L==0
	"Pregunta modificada.";
#elif L==1
	"Frage modifiziert.";
#elif L==2
	"Question modified.";
#elif L==3
	"Pregunta modificada.";
#elif L==4
	"Question modifi&eacute;e.";
#elif L==5
	"Pregunta modificada.";		// Okoteve traduccin
#elif L==6
	"Domanda modificata.";
#elif L==7
	"Pytanie zmodyfikowane.";
#elif L==8
	"Quest&atilde;o modificada.";
#endif

const char *Txt_Question_removed =
#if   L==0
	"Pregunta eliminada.";
#elif L==1
	"Frage entfernt.";
#elif L==2
	"Question removed.";
#elif L==3
	"Pregunta eliminada.";
#elif L==4
	"Question supprim&eacute;e.";
#elif L==5
	"Pregunta eliminada.";		// Okoteve traduccin
#elif L==6
	"Domanda rimossa.";
#elif L==7
	"Pytanie usuniete.";
#elif L==8
	"Quest&atilde;o removioda.";
#endif

const char *Txt_Questions =
#if   L==0
	"Preguntes";
#elif L==1
	"Fragen";
#elif L==2
	"Questions";
#elif L==3
	"Preguntas";
#elif L==4
	"Questions";
#elif L==5
	"Preguntas";	// Okoteve traduccin
#elif L==6
	"Domande";
#elif L==7
	"Pyta&nacute;";
#elif L==8
	"Quest&otilde;es";
#endif

const char *Txt_Questions_and_problems =
#if   L==0
	"Consultes i problemes";
#elif L==1
	"Fragen und Probleme";
#elif L==2
	"Questions and problems";
#elif L==3
	"Consultas y problemas";
#elif L==4
	"Questions et probl&egrave;mes";
#elif L==5
	"Consultas y problemas";		// Okoteve traduccin
#elif L==6
	"Domande e problemi";
#elif L==7
	"Pytania i problemy";
#elif L==8
	"Quest&otilde;es e problemas";
#endif

const char *Txt_Quota_exceeded =
#if   L==0
	"&iexcl;Cuota excedida!";	// Necessita traduccio
#elif L==1
	"Kontingent &uuml;berschritten!";
#elif L==2
	"Quota exceeded!";
#elif L==3
	"&iexcl;Cuota excedida!";
#elif L==4
	"Quota d&eacute;pass&eacute;!";
#elif L==5
	"&iexcl;Cuota excedida!";	// Okoteve traduccin
#elif L==6
	"Quota superata!";
#elif L==7
	"Przekroczono przydzial!";
#elif L==8
	"Quota excedida!";
#endif

const char *Txt_ReaBRders =
#if   L==0
	"Lec-<br />tores";	// Necessita traduccio
#elif L==1
	"Leser";
#elif L==2
	"Rea-<br />ders";
#elif L==3
	"Lec-<br />tores";
#elif L==4
	"Lec-<br />teurs";
#elif L==5
	"Lec-<br />tores";	// Okoteve traduccin
#elif L==6
	"Le-<br />ttori";
#elif L==7
	"Czytel-<br />nik&oacute;w";
#elif L==8
	"Lei-<br />tores";
#endif

const char *Txt_Real_number_between_A_and_B_1 =
#if   L==0
	"Nombre real entre";
#elif L==1
	"Reelle Zahl zwischen";
#elif L==2
	"Real number between";
#elif L==3
	"N&ordm; real entre";
#elif L==4
	"Nombre r&eacute;el entre";
#elif L==5
	"N&ordm; real entre";	// Okoteve traduccin
#elif L==6
	"Numero reale compreso tra";
#elif L==7
	"Prawdziwa liczbe miedzy";
#elif L==8
	"N&ordm; real entre";
#endif

const char *Txt_Real_number_between_A_and_B_2 =
#if   L==0
	"y";	// Necessita traduccio
#elif L==1
	"und";
#elif L==2
	"and";
#elif L==3
	"y";
#elif L==4
	"et";
#elif L==5
	"y";	// Okoteve traduccin
#elif L==6
	"e";
#elif L==7
	"i";
#elif L==8
	"e";
#endif

const char *Txt_Record_card_of_THE_USER_X_has_been_removed =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"Se ha eliminado la ficha de <strong>%s</strong>.";	// Necessita traduccio
#elif L==1
	"Record card of <strong>%s</strong> has been removed.";	// Need bersetzung
#elif L==2
	"Record card of <strong>%s</strong> has been removed.";
#elif L==3
	"Se ha eliminado la ficha de <strong>%s</strong>.";
#elif L==4
	"Record card of <strong>%s</strong> has been removed.";	// Besoin de traduction
#elif L==5
	"Se ha eliminado la ficha de <strong>%s</strong>.";	// Okoteve traduccin
#elif L==6
	"La scheda de <strong>%s</strong> &egrave; stata rimossa.";
#elif L==7
	"Karta Rekord <strong>%s</strong> zostala usunieta.";
#elif L==8
	"Record card of <strong>%s</strong> has been removed.";	// Necessita de traduo
#endif

const char *Txt_record_cards_per_page =
#if   L==0
	"fichas por p&aacute;gina";		// Necessita traduccio
#elif L==1
	"record cards per page";		// Need bersetzung
#elif L==2
	"record cards per page";
#elif L==3
	"fichas por p&aacute;gina";
#elif L==4
	"record cards per page";		// Besoin de traduction
#elif L==5
	"fichas por p&aacute;gina";		// Okoteve traduccin
#elif L==6
	"scheda per pagina";
#elif L==7
	"karty zapisu na stronie";
#elif L==8
	"record cards per page";		// Necessita de traduo
#endif

const char *Txt_Record_field_X_removed =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"Campo de ficha <strong>%s</strong> eliminado.";	// Necessita traduccio
#elif L==1
	"Datenfeld <strong>%s</strong> entfernt.";
#elif L==2
	"Record field <strong>%s</strong> removed.";
#elif L==3
	"Campo de ficha <strong>%s</strong> eliminado.";
#elif L==4
	"Record field <strong>%s</strong> removed.";	// Besoin de traduction
#elif L==5
	"Campo de ficha <strong>%s</strong> eliminado.";	// Okoteve traduccin
#elif L==6
	"Campo scheda <strong>%s</strong> rimosso.";
#elif L==7
	"Pole rekordu <strong>%s</strong> usuniete.";
#elif L==8
	"Record field <strong>%s</strong> removed.";	// Necessita de traduo
#endif

const char *Txt_RECORD_FIELD_VISIBILITY_MENU[Rec_NUM_TYPES_VISIBILITY] =
   {
#if   L==0
	 "Oculto"	// Necessita traduccio
#elif L==1
	 "Versteckt"
#elif L==2
	 "Hidden"
#elif L==3
	 "Oculto"
#elif L==4
	 "Cach&eacute;"
#elif L==5
	 "Oculto"	// Okoteve traduccin
#elif L==6
	 "Nascosto"
#elif L==7
	 "Ukryty"
#elif L==8
	 "Oculto"
#endif
	 ,
#if   L==0
	 "Visible pero no editable"	// Necessita traduccio
#elif L==1
	 "Sichtbar, aber nicht bearbeitet werden"
#elif L==2
	 "Visible but not editable"
#elif L==3
	 "Visible pero no editable"
#elif L==4
	 "Visible mais non modifiable"
#elif L==5
	 "Visible pero no editable"	// Okoteve traduccin
#elif L==6
	 "Visibile ma non modificabile"
#elif L==7
	 "Widoczne, ale nie mozna edytowa&cacute;"
#elif L==8
	 "Vis&iacute;vel, mas n&atilde;o edit&aacute;vel"
#endif
	 ,
#if   L==0
	 "Visible y editable"	// Necessita traduccio
#elif L==1
	 "Sichtbar und bearbeitet werde"
#elif L==2
	 "Visible and editable"
#elif L==3
	 "Visible y editable"
#elif L==4
	 "Visible et modifiable"
#elif L==5
	 "Visible y editable"	// Okoteve traduccin
#elif L==6
	 "Visibile e modificabile"
#elif L==7
	 "Widzialne i mozna edytowa&cacute;"
#elif L==8
	 "Vis&iacute;vel e edit&aacute;vel"
#endif
	};

const char *Txt_RECORD_FIELD_VISIBILITY_MSG[Rec_NUM_TYPES_VISIBILITY] =	// Warning: it is very important to include %s in the following sentences
	 {
#if   L==0
	   "A partir de ahora los estudiantes"
	   " no podr&aacute;n ver el campo <strong>%s</strong> de las fichas."// Necessita traduccio
#elif L==1
	   "Von nun an k&ouml;nnen die Studenten"
	   " nicht das Feld <strong>%s</strong> der Profile."
#elif L==2
	   "From now on students"
	   " can not see the field <strong>%s</strong> of the records."
#elif L==3
	   "A partir de ahora los estudiantes"
	   " no podr&aacute;n ver el campo <strong>%s</strong> de las fichas."
#elif L==4
	   "A partir de maintenant, les &eacute;tudiants"
	   " ne peuvent pas voir le champ <strong>%s</strong> de les cartes."
#elif L==5
	   "A partir de ahora los estudiantes"
	   " no podr&aacute;n ver el campo <strong>%s</strong> de las fichas."// Okoteve traduccin
#elif L==6
	   "Da ora in poi gli studenti"
	   " non possono visualizzare il campo <strong>%s</strong> delle schede."
#elif L==7
	   "Od teraz studenci"
	   " nie widac pola <strong>%s</strong> zapis&oacute;w."
#elif L==8
	   "De agora em diante,"
		 " os alunos n&atilde;o podem ver o campo <strong>%s</strong> dos cart&otilde;es."
#endif
	   ,
#if   L==0
	   "A partir de ahora los estudiantes"
	   " podr&aacute;n ver pero no editar el campo <strong>%s</strong> de las fichas."// Necessita traduccio
#elif L==1
	   "Von nun an k&ouml;nnen sich Studenten,"
	   " aber nicht bearbeiten das Feld <strong>%s</strong> der Profile."
#elif L==2
	   "From now on students"
	   " can view but not edit the field <strong>%s</strong> of the records."
#elif L==3
	   "A partir de ahora los estudiantes"
	   " podr&aacute;n ver pero no editar el campo <strong>%s</strong> de las fichas."
#elif L==4
	   "A partir de maintenant, les &eacute;tudiants"
	   " peuvent consulter mais pas modifier le champ <strong>%s</strong> de les cartes."
#elif L==5
	   "A partir de ahora los estudiantes"
	   " podr&aacute;n ver pero no editar el campo <strong>%s</strong> de las fichas."// Okoteve traduccin
#elif L==6
	   "Da ora in poi gli studenti"
	   " possono visualizzare ma non modificare il campo <strong>%s</strong> delle schede."
#elif L==7
	   "Od teraz studenci"
	   " moga przegladac, ale nie edytowa&cacute; pole <strong>%s</strong> zapis&oacute;w."
#elif L==8
	   "De agora em diante,"
		 " os alunos podem visualizar, mas n&atilde;o editar o campo <strong>%s</strong> dos cart&otilde;es."
#endif
	   ,
#if   L==0
	   "A partir de ahora los estudiantes"
	   " podr&aacute;n ver y editar el campo <strong>%s</strong> de las fichas."// Necessita traduccio
#elif L==1
	   "Von nun an k&ouml;nnen die Studenten"
	   " anzuzeigen und zu bearbeiten das Feld <strong>%s</strong> der Profile."
#elif L==2
	   "From now on students"
	   " can view and edit the field <strong>%s</strong> of the records."
#elif L==3
	   "A partir de ahora los estudiantes"
	   " podr&aacute;n ver y editar el campo <strong>%s</strong> de las fichas."
#elif L==4
	   "A partir de maintenant,"
	   " les &eacute;tudiants peuvent consulter et modifier le champ <strong>%s</strong> de les cartes."
#elif L==5
	   "A partir de ahora los estudiantes"
	   " podr&aacute;n ver y editar el campo <strong>%s</strong> de las fichas."// Okoteve traduccin
#elif L==6
	   "Da ora in poi"
	   " gli studenti possono visualizzare e modificare il campo <strong>%s</strong> delle schede."
#elif L==7
	   "Od teraz studenci"
	   " moga ogladac i edytowa&cacute; pola <strong>%s</strong> zapis&oacute;w."
#elif L==8
	   "De agora em diante,"
		 " os alunos podem visualizar e editar o campo <strong>%s</strong> dos cart&otilde;es."
#endif
	};

const char *Txt_RECORD_FIELD_VISIBILITY_RECORD[Rec_NUM_TYPES_VISIBILITY] =
   {
#if   L==0
	 "oculto"	// Necessita traduccio
#elif L==1
	 "versteckt"
#elif L==2
	 "hidden"
#elif L==3
	 "oculto"
#elif L==4
	 "cach&eacute;"
#elif L==5
	 "oculto"	// Okoteve traduccin
#elif L==6
	 "nascosto"
#elif L==7
	 "ukryty"
#elif L==8
	 "oculto"
#endif
	 ,
#if   L==0
	 "visible"	// Necessita traduccio
#elif L==1
	 "sichtbar"
#elif L==2
	 "visible"
#elif L==3
	 "visible"
#elif L==4
	 "visible"
#elif L==5
	 "visible"	// Okoteve traduccin
#elif L==6
	 "visibile"
#elif L==7
	 "widoczny"
#elif L==8
	 "vis&iacute;vel"
#endif
	 ,
#if   L==0
	 "editable"	// Necessita traduccio
#elif L==1
	 "bearbeitet werde"
#elif L==2
	 "editable"
#elif L==3
	 "editable"
#elif L==4
	 "modifiable"
#elif L==5
	 "editable"	// Okoteve traduccin
#elif L==6
	 "modificabile"
#elif L==7
	 "edytowalne"
#elif L==8
	 "edit&aacute;vel"
#endif
	};

const char *Txt_Record_fields_in_X =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"Camps de fitxes en <strong>%s</strong>:";
#elif L==1
	"Record fields in <strong>%s</strong>:";		// Need bersetzung
#elif L==2
	"Record fields in <strong>%s</strong>:";
#elif L==3
	"Campos de fichas en <strong>%s</strong>:";
#elif L==4
	"Record fields in <strong>%s</strong>:";		// Besoin de traduction
#elif L==5
	"Campos de fichas en <strong>%s</strong>:";	// Okoteve traduccin
#elif L==6
	"Campo schede in <strong>%s</strong>:";
#elif L==7
	"Rekord w dziedzinie <strong>%s</strong>:";
#elif L==8
	"Record fields in <strong>%s</strong>:";		// Necessita de traduo
#endif

const char *Txt_Recipients =
#if   L==0
	"Destinatarios";	// Necessita traduccio
#elif L==1
	"Empf&auml;nger";
#elif L==2
	"Recipients";
#elif L==3
	"Destinatarios";
#elif L==4
	"Destinataires";
#elif L==5
	"Destinatarios";	// Okoteve traduccin
#elif L==6
	"Destinatari";
#elif L==7
	"Odbiorcy";
#elif L==8
	"Destinat&aacute;rios";
#endif

const char *Txt_Register =
#if   L==0
	"Inscriure";
#elif L==1
	"Anmeldung";
#elif L==2
	"Register";
#elif L==3
	"Inscribir";
#elif L==4
	"Inscrire";
#elif L==5
	"Inscribir";	// Okoteve traduccin
#elif L==6
	"Registrare";
#elif L==7
	"Zarejestruj sie";
#elif L==8
	"Registrar";
#endif

const char *Txt_Register_students =
#if   L==0
	"Inscriure estudiants";
#elif L==1
	"Anmeldung Studenten";
#elif L==2
	"Register students";
#elif L==3
	"Inscribir estudiantes";
#elif L==4
	"Inscrire &eacute;tudiants";
#elif L==5
	"Inscribir estudiantes";	// Okoteve traduccin
#elif L==6
	"Registrare studenti";
#elif L==7
	"Rejestracja student&oacute;w";
#elif L==8
	"Registrar estudantes";
#endif

const char *Txt_Register_the_users_indicated_in_step_1 =
#if   L==0
	"A&ntilde;adir (inscribir los usuarios indicados en el paso 1)";	// Necessita traduccio
#elif L==1
	"Anmeldung Sie die Benutzer, die auf der Liste";
#elif L==2
	"Add (register the users indicated in step 1)";
#elif L==3
	"A&ntilde;adir (inscribir los usuarios indicados en el paso 1)";
#elif L==4
	"Inscrire les utilisateurs qui sont sur la liste";
#elif L==5
	"A&ntilde;adir (inscribir los usuarios indicados en el paso 1)";	// Okoteve traduccin
#elif L==6
	"Registrare gli utenti che sono in lista";
#elif L==7
	"Rejestracja uzytkownik&oacute;w, kt&oacute;rzy sa na liscie";
#elif L==8
	"Registrar os utilizadores que est&atilde;o na lista";
#endif

const char *Txt_Register_the_users_indicated_in_step_1_and_remove_the_users_not_indicated =
#if   L==0
	"Actualizar (inscribir los usuarios indicados en el paso 1"
	" y eliminar los <strong>no</strong> indicados)";	// Necessita traduccio
#elif L==1
	"Entfernen Sie die Benutzer, die <strong>nicht</strong> auf der Liste,"
	" und Anmeldung Sie die Benutzer, die auf der Liste";
#elif L==2
	"Update (register the users indicated in step 1"
	" and remove the users <strong>not</strong> indicated)";
#elif L==3
	"Actualizar (inscribir los usuarios indicados en el paso 1"
	" y eliminar los <strong>no</strong> indicados)";
#elif L==4
	"Enlever les utilisateurs qui <strong>ne</strong> sont <strong>pas</strong> sur la liste,"
	" et inscrire les utilisateurs qui sont sur la liste";
#elif L==5
	"Actualizar (inscribir los usuarios indicados en el paso 1"
	" y eliminar los <strong>no</strong> indicados)";	// Okoteve traduccin
#elif L==6
	"Rimuovere gli utenti che <strong>non</strong> sono sulla lista,"
	" e registrare gli utenti che invece lo sono";
#elif L==7
	"Usun u&zdot;ytkownikami, kt&oacute;rzy sa <strong>nie</strong> na liscie,"
	"i  zarejestrowac uzytkownik&oacute;w, kt&oacute;rzy sa na liscie";
#elif L==8
	"Tirar os utilizadores que <strong>n&atilde;o</strong> est&atilde;o na lista,"
	" e registrar os utilizadores que est&atilde;o na lista";
#endif

const char *Txt_Register_user_IN_A_COURSE_OR_DEGREE =
#if   L==0
	"Inscriure usuari";
#elif L==1
	"Anmeldung Benutzer";
#elif L==2
	"Register user";
#elif L==3
	"Inscribir usuario";
#elif L==4
	"Inscrire utilisateur";
#elif L==5
	"Inscribir usuario";	// Okoteve traduccin
#elif L==6
	"Registrare utente";
#elif L==7
	"Rejestracja u&zdot;ytkownika";
#elif L==8
	"Registrar utilizador";
#endif

const char *Txt_Register_me_in_the_course_X =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"Inscriureme en l'assignatura <strong>%s</strong>";
#elif L==1
	"Anmeldung Sie mir im Kurs <strong>%s</strong>";
#elif L==2
	"Register me in the course <strong>%s</strong>";
#elif L==3
	"Inscribirme en la asignatura <strong>%s</strong>";
#elif L==4
	"Inscrire-moi dans la mati&egrave;re <strong>%s</strong>";
#elif L==5
	"Inscribirme en la asignatura <strong>%s</strong>";	// Okoteve traduccin
#elif L==6
	"Registrami utente nel corso <strong>%s</strong>";
#elif L==7
	"Rejestracja mnie w kurs <strong>%s</strong>";
#elif L==8
	"Registrar me na disciplina <strong>%s</strong>";
#endif

const char *Txt_Register_user_in_the_course_X =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"Inscriure usuari en l'assignatura <strong>%s</strong>";
#elif L==1
	"Anmeldung Benutzer im Kurs <strong>%s</strong>";
#elif L==2
	"Register user in the course <strong>%s</strong>";
#elif L==3
	"Inscribir usuario en la asignatura <strong>%s</strong>";
#elif L==4
	"Inscrire utilisateur dans la mati&egrave;re <strong>%s</strong>";
#elif L==5
	"Inscribir usuario en la asignatura <strong>%s</strong>";	// Okoteve traduccin
#elif L==6
	"Registrare utente nel corso <strong>%s</strong>";
#elif L==7
	"Rejestracja u&zdot;ytkownika w kurs <strong>%s</strong>";
#elif L==8
	"Registrar utilizador na disciplina <strong>%s</strong>";
#endif

const char *Txt_Register_user_as_an_administrator_of_the_degree_X =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"Inscriure usuari com a administrador"
	" de la titulaci&oacute; <strong>%s</strong>";
#elif L==1
	"Anmeldung Benutzer als Administrator"
	" des Studiengang <strong>%s</strong>";
#elif L==2
	"Register user as an administrator"
	" of the degree <strong>%s</strong>";
#elif L==3
	"Inscribir usuario como administrador"
	" de la titulaci&oacute;n <strong>%s</strong>";
#elif L==4
	"Inscrire utilisateur en tant qu'administrateur"
	" du &eacute;tude <strong>%s</strong>";
#elif L==5
	"Inscribir usuario como administrador"
	" de la titulaci&oacute;n <strong>%s</strong>";	// Okoteve traduccin
#elif L==6
	"Registrare utente come amministratore"
	" della laurea <strong>%s</strong>";
#elif L==7
	"Rejestracja u&zdot;ytkownika jako administrator"
	" stopnia <strong>%s</strong>";
#elif L==8
	"Registrar utilizador como administrador"
	" da titula&ccedil;&atilde;o <strong>%s</strong>";
#endif

const char *Txt_Enrollment_confirmed =
#if   L==0
	"Inscripci&oacute;n confirmada";	// Necessita traduccio
#elif L==1
	"Enrollment confirmed";		// Need bersetzung
#elif L==2
	"Enrollment confirmed";
#elif L==3
	"Inscripci&oacute;n confirmada";
#elif L==4
	"Enrollment confirmed";		// Besoin de traduction
#elif L==5
	"Inscripci&oacute;n confirmada";	// Okoteve traduccin
#elif L==6
	"Registrazione confermata";
#elif L==7
	"Rejestracja potwierdzona";
#elif L==8
	"Enrollment confirmed";		// Necessita de traduo
#endif

const char *Txt_Enrollment_not_confirmed =
#if   L==0
	"Inscripci&oacute;n pendiente de confirmaci&oacute;n";	// Necessita traduccio
#elif L==1
	"Enrollment not confirmed";			// Need bersetzung
#elif L==2
	"Enrollment not confirmed";
#elif L==3
	"Inscripci&oacute;n pendiente de confirmaci&oacute;n";
#elif L==4
	"Enrollment not confirmed";			// Besoin de traduction
#elif L==5
	"Inscripci&oacute;n pendiente de confirmaci&oacute;n";	// Okoteve traduccin
#elif L==6
	"Registrazione non confermata";
#elif L==7
	"Rejestracja nie potwierdzona";
#elif L==8
	"Enrollment not confirmed";			// Necessita de traduo
#endif

const char *Txt_Reject =
#if   L==0
	"Rebutjar";
#elif L==1
	"Ablehnen";
#elif L==2
	"Reject";
#elif L==3
	"Rechazar";
#elif L==4
	"Rejeter";
#elif L==5
	"Rechazar";	// Okoteve traduccin
#elif L==6
	"Rifiutare";
#elif L==7
	"Odrzuca&cacute;";
#elif L==8
	"Rejeitar";
#endif

const char *Txt_Remove =
#if   L==0
	"Eliminar";
#elif L==1
	"Entfernen";
#elif L==2
	"Remove";
#elif L==3
	"Eliminar";
#elif L==4
	"Supprimer";
#elif L==5
	"Mbogue";
#elif L==6
	"Rimuovere";
#elif L==7
	"Usu&nacute;";
#elif L==8
	"Remover";
#endif

const char *Txt_Remove_account =
#if   L==0
	"Eliminar compte";
#elif L==1
	"Konto entfernen";
#elif L==2
	"Remove account";
#elif L==3
	"Eliminar cuenta";
#elif L==4
	"Supprimer compte";
#elif L==5
	"Eliminar cuenta";	// Okoteve traduccin
#elif L==6
	"Rimuovere account";
#elif L==7
	"Usu&nacute; konto";
#elif L==8
	"Remover conta";
#endif

const char *Txt_Remove_all_students =
#if   L==0
	"Eliminar tots estudiants";
#elif L==1
	"Alle Studenten entfernen";
#elif L==2
	"Remove all students";
#elif L==3
	"Eliminar todos los estudiantes";
#elif L==4
	"Enlever tous &eacute;tudiants";
#elif L==5
	"Eliminar todos los estudiantes";	// Okoteve traduccin
#elif L==6
	"Rimuovere tutti gli stud.";
#elif L==7
	"Usu&nacute; wszystkich student&oacute;w";
#elif L==8
	"Tirar todos estudantes";
#endif

const char *Txt_Remove_announcement_of_exam =
#if   L==0
	"Eliminar convocat&ograve;ria d'examen";
#elif L==1
	"Entfernen Aufruf f&uuml;r Pr&uuml;fung";
#elif L==2
	"Remove announcement of exam";
#elif L==3
	"Eliminar convocatoria de examen";
#elif L==4
	"Supprimer convocation &agrave; l'examen";
#elif L==5
	"Eliminar convocatoria de examen";	// Okoteve traduccin
#elif L==6
	"Rimuovere sessione d'esame";
#elif L==7
	"Usu&nacute; ogloszenie egzaminu";
#elif L==8
	"Remover chamada para o exame";
#endif

const char *Txt_Remove_assignment =
#if   L==0
	"Eliminar activitat";
#elif L==1
	"Aufgabe entfernen";
#elif L==2
	"Remove assignment";
#elif L==3
	"Eliminar actividad";
#elif L==4
	"Supprimer activit&eacute;";
#elif L==5
	"Eliminar actividad";	// Okoteve traduccin
#elif L==6
	"Rimuovere attivit&agrave;";
#elif L==7
	"Usu&nacute; przypisanie";
#elif L==8
	"Remover atividade";
#endif

const char *Txt_Remove_banner =
#if   L==0
	"Eliminar enlace";	// Necessita traduccio
#elif L==1
	"Entfernen Werbebanner";
#elif L==2
	"Remove banner";
#elif L==3
	"Eliminar banner";
#elif L==4
	"Supprimer banni&eacute;re";
#elif L==5
	"Eliminar banner";	// Okoteve traduccin
#elif L==6
	"Rimuovere banner";
#elif L==7
	"Usu&nacute; banner";
#elif L==8
	"Remover banner";
#endif

const char *Txt_Remove_centre =
#if   L==0
	"Eliminar centro";	// Necessita traduccio
#elif L==1
	"Lehrinstitut entfernen";
#elif L==2
	"Remove centre";
#elif L==3
	"Eliminar centro";
#elif L==4
	"Supprimer centre";
#elif L==5
	"Eliminar centro";	// Okoteve traduccin
#elif L==6
	"Rimuovere centro";
#elif L==7
	"Usu&nacute; centrum";
#elif L==8
	"Remover centro";
#endif

const char *Txt_Remove_country =
#if   L==0
	"Eliminar pa&iacute;s";	// Necessita traduccio
#elif L==1
	"Land entfernen";
#elif L==2
	"Remove country";
#elif L==3
	"Eliminar pa&iacute;s";
#elif L==4
	"Supprimer pays";
#elif L==5
	"Eliminar pa&iacute;s";	// Okoteve traduccin
#elif L==6
	"Rimuovere paese";
#elif L==7
	"Usu&nacute; kraj";
#elif L==8
	"Remover pa&iacute;s";
#endif

const char *Txt_Remove_course =
#if   L==0
	"Eliminar asignatura";	// Necessita traduccio
#elif L==1
	"Kurs entfernen";
#elif L==2
	"Remove course";
#elif L==3
	"Eliminar asignatura";
#elif L==4
	"Supprimer mati&egrave;re";
#elif L==5
	"Eliminar asignatura";	// Okoteve traduccin
#elif L==6
	"Rimuovere corso";
#elif L==7
	"Usu&nacute; kurs";
#elif L==8
	"Remover disciplina";
#endif

const char *Txt_Remove_degree =
#if   L==0
	"Eliminar titulaci&oacute;n";	// Necessita traduccio
#elif L==1
	"Studiengang Entfernen";
#elif L==2
	"Remove degree";
#elif L==3
	"Eliminar titulaci&oacute;n";
#elif L==4
	"Supprimer &eacute;tude";
#elif L==5
	"Eliminar titulaci&oacute;n";	// Okoteve traduccin
#elif L==6
	"Rimuovere laurea";
#elif L==7
	"Usu&nacute; stopnia";
#elif L==8
	"Remover titula&ccedil;&atilde;o";
#endif

const char *Txt_Remove_department =
#if   L==0
	"Eliminar departamento";	// Necessita traduccio
#elif L==1
	"Abteilung entfernen";
#elif L==2
	"Remove department";
#elif L==3
	"Eliminar departamento";
#elif L==4
	"Supprimer d&eacute;partement";
#elif L==5
	"Eliminar departamento";	// Okoteve traduccin
#elif L==6
	"Rimuovere dipartimento";
#elif L==7
	"Usu&nacute; dzia&lstrok;";
#elif L==8
	"Remover departamento";
#endif

const char *Txt_Remove_email_X =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"Eliminar correu electr&ograve;nic %s";
#elif L==1
	"Entfernen E-Mail %s";
#elif L==2
	"Remove e-mail %s";
#elif L==3
	"Eliminar correo electr&oacute;nico %s";
#elif L==4
	"Supprimer courrier &eacute;lectronique %s";
#elif L==5
	"Eliminar correo electr&oacute;nico %s";	// Okoteve traduccin
#elif L==6
	"Rimuovere e-mail %s";
#elif L==7
	"Usu&nacute; e-mail %s";
#elif L==8
	"Remover e-mail %s";
#endif

const char *Txt_Remove_event =
#if   L==0
	"Eliminar evento";	// Necessita traduccio
#elif L==1
	"Ereignis Entfernen";
#elif L==2
	"Remove event";
#elif L==3
	"Eliminar evento";
#elif L==4
	"Supprimer &eacute;v&eacute;nement";
#elif L==5
	"Eliminar evento";	// Okoteve traduccin
#elif L==6
	"Rimuovere evento";
#elif L==7
	"Usu&nacute; wydarzenie";
#elif L==8
	"Remover evento";
#endif

const char *Txt_Remove_file =
#if   L==0
	"Eliminar fitxer";
#elif L==1
	"Datei Entfernen";
#elif L==2
	"Remove file";
#elif L==3
	"Eliminar archivo";
#elif L==4
	"Supprimer fichier";
#elif L==5
	"Eliminar archivo";	// Okoteve traduccin
#elif L==6
	"Rimuovere file";
#elif L==7
	"Usu&nacute; plik";
#elif L==8
	"Remover arquivo";
#endif

const char *Txt_Remove_FILE_OR_LINK_X =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"Eliminar %s";
#elif L==1
	"Datei %s";
#elif L==2
	"Remove %s";
#elif L==3
	"Eliminar %s";
#elif L==4
	"Supprimer %s";
#elif L==5
	"Eliminar %s";	// Okoteve traduccin
#elif L==6
	"Rimuovere %s";
#elif L==7
	"Usu&nacute; %s";
#elif L==8
	"Remover %s";
#endif

const char *Txt_Remove_folder_X = // Warning: it is very important to include %s in the following sentences
#if   L==0
	"Eliminar carpeta %s";
#elif L==1
	"Verzeichnis entfernen %s";
#elif L==2
	"Remove folder %s";
#elif L==3
	"Eliminar carpeta %s";
#elif L==4
	"Supprimer r&eacute;pertoire %s";
#elif L==5
	"Eliminar carpeta %s";	// Okoteve traduccin
#elif L==6
	"Rimuovere cartella %s";
#elif L==7
	"Usu&nacute; folder %s";
#elif L==8
	"Remover diret&oacute;rio %s";
#endif

const char *Txt_Remove_folder =
#if   L==0
	"Eliminar carpeta";
#elif L==1
	"Verzeichnis entfernen";
#elif L==2
	"Remove folder";
#elif L==3
	"Eliminar carpeta";
#elif L==4
	"Supprimer r&eacute;pertoire";
#elif L==5
	"Eliminar carpeta";	// Okoteve traduccin
#elif L==6
	"Rimuovere cartella";
#elif L==7
	"Usu&nacute; folder";
#elif L==8
	"Remover diret&oacute;rio";
#endif

const char *Txt_Remove_group =
#if   L==0
	"Eliminar grupo";	// Necessita traduccio
#elif L==1
	"Entfernen Gruppe";
#elif L==2
	"Remove group";
#elif L==3
	"Eliminar grupo";
#elif L==4
	"Supprimer groupe";
#elif L==5
	"Eliminar grupo";	// Okoteve traduccin
#elif L==6
	"Rimuovere gruppo";
#elif L==7
	"Usu&nacute; gruper";
#elif L==8
	"Remover grupo";
#endif

const char *Txt_Remove_holiday =
#if   L==0
	"Eliminar festividad";	// Necessita traduccio
#elif L==1
	"Entfernen Feiertag";
#elif L==2
	"Remove holiday";
#elif L==3
	"Eliminar festividad";
#elif L==4
	"Supprimer f&ecirc;te";
#elif L==5
	"Eliminar festividad";	// Okoteve traduccin
#elif L==6
	"Rimuovere festivit&agrave;";
#elif L==7
	"Usu&nacute; wakacje";
#elif L==8
	"Remover f&eacute;ria";
#endif

const char *Txt_Remove_ID_X =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"Eliminar ID (DNI/c&egrave;dula/passaport) %s";
#elif L==1
	"Entfernen Ausweis-Nr. %s";
#elif L==2
	"Remove ID %s";
#elif L==3
	"Eliminar ID (DNI/c&eacute;dula) %s";
#elif L==4
	"Supprimer num&eacute;ro d'identit&eacute; %s";
#elif L==5
	"Eliminar ID (DNI/c&eacute;dula) %s";	// Okoteve traduccin
#elif L==6
	"Rimuovere carta d'identit&agrave; %s";
#elif L==7
	"Remove ID %s";				// Potrzebujesz tlumaczenie
#elif L==8
	"Remover n&ordm; de identif. %s";
#endif

const char *Txt_Remove_institution =
#if   L==0
	"Eliminar instituci&oacute;n";	// Necessita traduccio
#elif L==1
	"Hochschule entfernen";
#elif L==2
	"Remove institution";
#elif L==3
	"Eliminar instituci&oacute;n";
#elif L==4
	"Supprimer &eacute;tablissement";
#elif L==5
	"Eliminar instituci&oacute;n";	// Okoteve traduccin
#elif L==6
	"Rimuovere istituzione";
#elif L==7
	"Usu&nacute; instytucje";
#elif L==8
	"Remover institui&ccedil;&atilde;o";
#endif

const char *Txt_Remove_link =
#if   L==0
	"Eliminar enlla&ccedil;";
#elif L==1
	"Entfernen Link";
#elif L==2
	"Remove link";
#elif L==3
	"Eliminar enlace";
#elif L==4
	"Supprimer lien";
#elif L==5
	"Eliminar enlace";	// Okoteve traduccin
#elif L==6
	"Rimuovere link";
#elif L==7
	"Usu&nacute; link";
#elif L==8
	"Remover liga&ccedil;&atilde;o";
#endif

const char *Txt_Remove_mail_domain =
#if   L==0
	"Eliminar dominio de correo";	// Necessita traduccio
#elif L==1
	"Entfernen Mail-Dom&auml;ne";
#elif L==2
	"Remove mail domain";
#elif L==3
	"Eliminar dominio de correo";
#elif L==4
	"Supprimer domaine de messagerie";
#elif L==5
	"Eliminar dominio de correo";	// Okoteve traduccin
#elif L==6
	"Rimuovere campo mail";
#elif L==7
	"Usu&nacute; domene mail";
#elif L==8
	"Remover dom&iacute;nio de email";
#endif

const char *Txt_Remove_nickname_X =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"Eliminar sobrenom @%s";
#elif L==1
	"Entfernen Benutzername @%s";
#elif L==2
	"Remove nickname @%s";
#elif L==3
	"Eliminar apodo @%s";
#elif L==4
	"Supprimer surnom @%s";
#elif L==5
	"Eliminar apodo @%s";	// Okoteve traduccin
#elif L==6
	"Rimuovere nome utente @%s";
#elif L==7
	"Usu&nacute; pseudonim @%s";
#elif L==8
	"Remover alcunha @%s";
#endif

const char *Txt_Remove_me_as_an_administrator =
#if   L==0
	"Eliminarme com a administrador";
#elif L==1
	"Entfernen Sie mir als Administrator";
#elif L==2
	"Remove me as an administrator";
#elif L==3
	"Eliminarme como administrador";
#elif L==4
	"Enlever-moi en tant qu'administrateur";
#elif L==5
	"Eliminarme como administrador";	// Okoteve traduccin
#elif L==6
	"Rimuovimi come amministratore";
#elif L==7
	"Usu&nacute; mnie jako administrator";
#elif L==8
	"Tirar me como administrador";
#endif

const char *Txt_Remove_me_as_an_administrator_of_the_degree_X =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"Eliminarme com a administrador"
	" de la titulaci&oacute; <strong>%s</strong>";
#elif L==1
	"Entfernen Sie mir als Administrator"
	" des Studiengang <strong>%s</strong>";
#elif L==2
	"Remove me as an administrator"
	" of the degree <strong>%s</strong>";
#elif L==3
	"Eliminarme como administrador"
	" de la titulaci&oacute;n <strong>%s</strong>";
#elif L==4
	"Enlever-moi en tant qu'administrateur"
	" du &eacute;tude <strong>%s</strong>";
#elif L==5
	"Eliminarme como administrador"
	" de la titulaci&oacute;n <strong>%s</strong>";	// Okoteve traduccin
#elif L==6
	"Rimuovimi come amministratore"
	" della laurea <strong>%s</strong>";
#elif L==7
	"Usu&nacute; mnie jako administrator"
	" stopnia <strong>%s</strong>";
#elif L==8
	"Tirar me como administrador"
	" da titula&ccedil;&atilde;o <strong>%s</strong>";
#endif

const char *Txt_Remove_me_from_this_course =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"Eliminarme d'aquesta assignatura";
#elif L==1
	"Entfernen Sie diesen Kurs";
#elif L==2
	"Remove me from this course";
#elif L==3
	"Eliminarme de esta asignatura";
#elif L==4
	"Enlever-moi de cette mati&egrave;re";
#elif L==5
	"Eliminarme de esta asignatura";	// Okoteve traduccin
#elif L==6
	"Rimuovimi da questo corso";
#elif L==7
	"Usu&ntilde; z kursu";
#elif L==8
	"Tirar me desta disciplina";
#endif

const char *Txt_Remove_me_from_the_course_X =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"Eliminarme de l'assignatura <strong>%s</strong>";
#elif L==1
	"Entfernen Sie mir aus dem Kurs <strong>%s</strong>";
#elif L==2
	"Remove me from the course <strong>%s</strong>";
#elif L==3
	"Eliminarme de la asignatura <strong>%s</strong>";
#elif L==4
	"Enlever-moi de la mati&egrave;re <strong>%s</strong>";
#elif L==5
	"Eliminarme de la asignatura <strong>%s</strong>";	// Okoteve traduccin
#elif L==6
	"Rimuovimi dal corso <strong>%s</strong>";
#elif L==7
	"Usu&nacute; mnie w kurs <strong>%s</strong>";
#elif L==8
	"Tirar me da disciplina <strong>%s</strong>";
#endif

const char *Txt_Remove_notice =
#if   L==0
	"Eliminar aviso";	// Necessita traduccio
#elif L==1
	"Entfernen Ank&uuml;ndigung";
#elif L==2
	"Remove notice";
#elif L==3
	"Eliminar aviso";
#elif L==4
	"Supprimer avis";
#elif L==5
	"Eliminar aviso";	// Okoteve traduccin
#elif L==6
	"Rimuovere avviso";
#elif L==7
	"Usu&nacute; ogloszenie";
#elif L==8
	"Remover aviso";
#endif

const char *Txt_Remove_old_users =
#if   L==0
	"Eliminar usuaris antics";
#elif L==1
	"Fr&uuml;here Benutzer entfernen";
#elif L==2
	"Eliminate old users";
#elif L==3
	"Eliminar usuarios antiguos";
#elif L==4
	"&Eacute;liminer anciens utilisateurs";
#elif L==5
	"Eliminar usuarios antiguos";	// Okoteve traduccin
#elif L==6
	"Rimuovere vecchi utenti";
#elif L==7
	"Usu&nacute; starych uzytkownik&oacute;w";
#elif L==8
	"Eliminar antigos utilizadores";
#endif

const char *Txt_Remove_photo =
#if   L==0
	"Eliminar foto";
#elif L==1
	"Entfernen Foto";
#elif L==2
	"Remove photo";
#elif L==3
	"Eliminar foto";
#elif L==4
	"Supprimer photo";
#elif L==5
	"Eliminar foto";	// Okoteve traduccin
#elif L==6
	"Rimuovere foto";
#elif L==7
	"Usu&nacute; zdjecie";
#elif L==8
	"Remover foto";
#endif

const char *Txt_Remove_place =
#if   L==0
	"Eliminar lugar";	// Necessita traduccio
#elif L==1
	"Entfernen Standort";
#elif L==2
	"Remove place";
#elif L==3
	"Eliminar lugar";
#elif L==4
	"Supprimer emplacement";
#elif L==5
	"Eliminar lugar";	// Okoteve traduccin
#elif L==6
	"Rimuovere localit&grave;";
#elif L==7
	"Usu&nacute; miejsce";
#elif L==8
	"Remover localiza&ccedil;&atilde;o";
#endif

const char *Txt_Remove_plugin =
#if   L==0
	"Eliminar complemento";	// Necessita traduccio
#elif L==1
	"Entfernen Plugin";
#elif L==2
	"Remove plugin";
#elif L==3
	"Eliminar complemento";
#elif L==4
	"Supprimer plugin";
#elif L==5
	"Eliminar complemento";	// Okoteve traduccin
#elif L==6
	"Rimuovere plugin";
#elif L==7
	"Usu&nacute; plugin";
#elif L==8
	"Remover plugin";
#endif

const char *Txt_Remove_post =
#if   L==0
	"Eliminar mensaje";	// Necessita traduccio
#elif L==1
	"Entfernen Beitr&auml;g";
#elif L==2
	"Remove post";
#elif L==3
	"Eliminar mensaje";
#elif L==4
	"Supprimer message";
#elif L==5
	"Eliminar mensaje";	// Okoteve traduccin
#elif L==6
	"Rimuovere post";
#elif L==7
	"Usu&nacute; post";
#elif L==8
	"Remover post";
#endif

const char *Txt_Remove_question =
#if   L==0
	"Eliminar pregunta";	// Necessita traduccio
#elif L==1
	"Entfernen Frage";
#elif L==2
	"Remove question";
#elif L==3
	"Eliminar pregunta";
#elif L==4
	"Supprimer question";
#elif L==5
	"Eliminar pregunta";	// Okoteve traduccin
#elif L==6
	"Rimuovere domanda";
#elif L==7
	"Usu&nacute; pytanie";
#elif L==8
	"Remover quest&atilde;o";
#endif

const char *Txt_Remove_record_field =
#if   L==0
	"Eliminar campo de ficha";	// Necessita traduccio
#elif L==1
	"Entfernen Datenfeld";
#elif L==2
	"Remove record field";
#elif L==3
	"Eliminar campo de ficha";
#elif L==4
	"Supprimer champ de carte";
#elif L==5
	"Eliminar campo de ficha";	// Okoteve traduccin
#elif L==6
	"Rimuovere campo di scheda";
#elif L==7
	"Usu&nacute; pola rekordu";
#elif L==8
	"Remover campo de cart&atilde;o";
#endif

const char *Txt_Remove_students =
#if   L==0
	"Eliminar estudiants";
#elif L==1
	"Entfernen Studenten";
#elif L==2
	"Remove students";
#elif L==3
	"Eliminar estudiantes";
#elif L==4
	"Enlever &eacute;tudiants";
#elif L==5
	"Eliminar estudiantes";	// Okoteve traduccin
#elif L==6
	"Rimuovere studenti";
#elif L==7
	"Usu&nacute; student&oacute;w";
#elif L==8
	"Tirar estudantes";
#endif

const char *Txt_Remove_survey =
#if   L==0
	"Eliminar encuesta";	// Necessita traduccio
#elif L==1
	"Entfernen Umfrage";
#elif L==2
	"Remove survey";
#elif L==3
	"Eliminar encuesta";
#elif L==4
	"Supprimer sondage";
#elif L==5
	"Eliminar encuesta";	// Okoteve traduccin
#elif L==6
	"Rimuovere sondaggio";
#elif L==7
	"Usu&nacute; badania";
#elif L==8
	"Remover inqu&eacute;rito";
#endif

const char *Txt_Remove_the_MESSAGE =
#if   L==0
	"Eliminar el";	// Necessita traduccio
#elif L==1
	"Entfernen Sie die";
#elif L==2
	"Remove the";
#elif L==3
	"Eliminar el";
#elif L==4
	"Supprimer le";
#elif L==5
	"Eliminar el";	// Okoteve traduccin
#elif L==6
	"Rimuovere il";
#elif L==7
	"Usu&nacute;";
#elif L==8
	"Remover a";
#endif

const char *Txt_Remove_thread =
#if   L==0
	"Eliminar discusi&oacute;n";	// Necessita traduccio
#elif L==1
	"Entfernen Thread";
#elif L==2
	"Remove thread";
#elif L==3
	"Eliminar discusi&oacute;n";
#elif L==4
	"Supprimer fil";
#elif L==5
	"Eliminar discusi&oacute;n";	// Okoteve traduccin
#elif L==6
	"Rimuovere discussione";
#elif L==7
	"Usu&nacute; watkek";
#elif L==8
	"Remover thread";
#endif

const char *Txt_Remove_type_of_degree =
#if   L==0
	"Eliminar tipo de titulaci&oacute;n";	// Necessita traduccio
#elif L==1
	"Entfernen Abschlussart";
#elif L==2
	"Remove type of degree";
#elif L==3
	"Eliminar tipo de titulaci&oacute;n";
#elif L==4
	"Supprimer type d'&eacute;tude";
#elif L==5
	"Eliminar tipo de titulaci&oacute;n";	// Okoteve traduccin
#elif L==6
	"Rimuovere tipo di laurea";
#elif L==7
	"Usu&nacute; typu stopien";
#elif L==8
	"Remover tipo de titula&ccedil;&atilde;";
#endif

const char *Txt_Remove_type_of_group =
#if   L==0
	"Eliminar tipo de grupo";	// Necessita traduccio
#elif L==1
	"Entfernen Art der Gruppe";
#elif L==2
	"Remove type of group";
#elif L==3
	"Eliminar tipo de grupo";
#elif L==4
	"Supprimer type de groupe";
#elif L==5
	"Eliminar tipo de grupo";	// Okoteve traduccin
#elif L==6
	"Rimuovere tipo di gruppo";
#elif L==7
	"Usu&nacute; typ grupy";
#elif L==8
	"Remover tipo de grupo";
#endif

const char *Txt_Remove_the_users_not_indicated_in_step_1 =
#if   L==0
	"Eliminar los usuarios <strong>no</strong> indicados en el paso 1";	// Necessita traduccio
#elif L==1
	"Entfernen Sie die Benutzer, die <strong>nicht</strong> auf der Liste";
#elif L==2
	"Remove the users <strong>not</strong> indicated in step 1";
#elif L==3
	"Eliminar los usuarios <strong>no</strong> indicados en el paso 1";
#elif L==4
	"Enlever les utilisateurs qui <strong>ne</strong> sont <strong>pas</strong> sur la liste";
#elif L==5
	"Eliminar los usuarios <strong>no</strong> indicados en el paso 1";	// Okoteve traduccin
#elif L==6
	"Rimuovere gli utenti che <strong>non</strong> sono sulla lista";
#elif L==7
	"Usu&nacute; u&zdot;ytkownikami, kt&oacute;rzy sa <strong>nie</strong> na liscie";
#elif L==8
	"Tirar os utilizadores que <strong>n&atilde;o</strong> est&atilde;o na lista";
#endif

const char *Txt_Remove_the_users_indicated_in_step_1 =
#if   L==0
	"Eliminar los usuarios indicados en el paso 1";	// Necessita traduccio
#elif L==1
	"Entfernen Sie die Benutzer, die auf der Liste";
#elif L==2
	"Remove the users indicated in step 1";
#elif L==3
	"Eliminar los usuarios indicados en el paso 1";
#elif L==4
	"Enlever les utilisateurs qui sont sur la liste";
#elif L==5
	"Eliminar los usuarios indicados en el paso 1";	// Okoteve traduccin
#elif L==6
	"Rimuovere gli utenti che sono in lista";
#elif L==7
	"Usu&nacute; u&zdot;ytkownikami, kt&oacute;rzy sa na liscie";
#elif L==8
	"Tirar os utilizadores que est&atilde;o na lista";
#endif

const char *Txt_Remove_the_X_MESSAGES =
#if   L==0
	"Eliminar los";	// Necessita traduccio
#elif L==1
	"Entfernen Sie die";
#elif L==2
	"Remove the";
#elif L==3
	"Eliminar los";
#elif L==4
	"Supprimer les";
#elif L==5
	"Eliminar los";	// Okoteve traduccin
#elif L==6
	"Rimuovere il";
#elif L==7
	"Usu&nacute;";
#elif L==8
	"Remover as";
#endif

const char *Txt_Remove_user_from_this_course =
#if   L==0
	"Eliminar usuari d'aquesta assignatura";
#elif L==1
	"Entfernen Benutzer Sie diesen Kurs";
#elif L==2
	"Remove user from this course";
#elif L==3
	"Eliminar usuario de esta asignatura";
#elif L==4
	"Enlever utilisateur de cette mati&egrave;re";
#elif L==5
	"Eliminar usuario de esta asignatura";	// Okoteve traduccin
#elif L==6
	"Rimuovi utente da questo corso";
#elif L==7
	"Usu&ntilde; u&zdot;ytkownika z kursu";
#elif L==8
	"Tirar utilizador desta disciplina";
#endif

const char *Txt_Remove_user_as_an_administrator =
#if   L==0
	"Eliminar usuari com a administrador";
#elif L==1
	"Entfernen Benutzer als Administrator";
#elif L==2
	"Remove user as an administrator";
#elif L==3
	"Eliminar usuario como administrador";
#elif L==4
	"Enlever utilisateur en tant qu'administrateur";
#elif L==5
	"Eliminar usuario como administrador";	// Okoteve traduccin
#elif L==6
	"Rimuovi utente come amministratore";
#elif L==7
	"Usu&nacute; u&zdot;ytkownika jako administrator";
#elif L==8
	"Tirar utilizador como administrador";
#endif

const char *Txt_Remove_user_as_an_administrator_of_the_degree_X =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"Eliminar usuari com a administrador"
	" de la titulaci&oacute; <strong>%s</strong>";
#elif L==1
	"Entfernen Benutzer als Administrator"
	" des Studiengang <strong>%s</strong>";
#elif L==2
	"Remove user as an administrator"
	" of the degree <strong>%s</strong>";
#elif L==3
	"Eliminar usuario como administrador"
	" de la titulaci&oacute;n <strong>%s</strong>";
#elif L==4
	"Enlever utilisateur en tant qu'administrateur"
	" du &eacute;tude <strong>%s</strong>";
#elif L==5
	"Eliminar usuario como administrador"
	" de la titulaci&oacute;n <strong>%s</strong>";	// Okoteve traduccin
#elif L==6
	"Rimuovi utente come amministratore"
	" della laurea <strong>%s</strong>";
#elif L==7
	"Usu&nacute; u&zdot;ytkownika jako administrator"
	" stopnia <strong>%s</strong>";
#elif L==8
	"Tirar utilizador como administrador"
	" da titula&ccedil;&atilde;o <strong>%s</strong>";
#endif

const char *Txt_Remove_user_from_the_course_X =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"Eliminar usuari de l'assignatura <strong>%s</strong>";
#elif L==1
	"Entfernen Benutzer aus dem Kurs <strong>%s</strong>";
#elif L==2
	"Remove user from the course <strong>%s</strong>";
#elif L==3
	"Eliminar usuario de la asignatura <strong>%s</strong>";
#elif L==4
	"Enlever utilisateur de la mati&egrave;re <strong>%s</strong>";
#elif L==5
	"Eliminar usuario de la asignatura <strong>%s</strong>";	// Okoteve traduccin
#elif L==6
	"Rimuovi utente dal corso <strong>%s</strong>";
#elif L==7
	"Usu&nacute; u&zdot;ytkownika w kurs <strong>%s</strong>";
#elif L==8
	"Tirar utilizador da disciplina <strong>%s</strong>";
#endif

const char *Txt_Reply =
#if   L==0
	"Responder";	// Necessita traduccio
#elif L==1
	"Antworten";
#elif L==2
	"Reply";
#elif L==3
	"Responder";
#elif L==4
	"R&eacute;pondre";
#elif L==5
	"Responder";	// Okoteve traduccin
#elif L==6
	"Rispondere";
#elif L==7
	"Odpowiedz";
#elif L==8
	"Responder";
#endif

const char *Txt_Reply_again =
#if   L==0
	"Responder otra vez";		// Necessita traduccio
#elif L==1
	"Antworten wieder";
#elif L==2
	"Reply again";
#elif L==3
	"Responder otra vez";
#elif L==4
	"R&eacute;pondre &agrave; nouveau";
#elif L==5
	"Responder otra vez";		// Okoteve traduccin
#elif L==6
	"Rispondere nuovamente";
#elif L==7
	"Odpowiedz ponownie";
#elif L==8
	"Responder novamente";
#endif

const char *Txt_Requester =
#if   L==0
	"Sollicitant";
#elif L==1
	"Requester";
#elif L==2
	"Requester";
#elif L==3
	"Solicitante";
#elif L==4
	"Demandeur";
#elif L==5
	"Solicitante";	// Okoteve traduccin
#elif L==6
	"Richiedente";
#elif L==7
	"&Zdot;&aogon;daj&aogon;cego";
#elif L==8
	"Requerente";
#endif

const char *Txt_Reset =
#if   L==0
	"Poner a cero";	// Necessita traduccio
#elif L==1
	"Reset";
#elif L==2
	"Reset";
#elif L==3
	"Poner a cero";
#elif L==4
	"Reset";
#elif L==5
	"Poner a cero";	// Okoteve traduccin
#elif L==6
	"Resettare";
#elif L==7
	"Reset";
#elif L==8
	"Reiniciar";
#endif

const char *Txt_Reset_survey =
#if   L==0
	"Poner encuesta a cero";	// Necessita traduccio
#elif L==1
	"Reset Umfrage";
#elif L==2
	"Reset survey";
#elif L==3
	"Poner encuesta a cero";
#elif L==4
	"Reset sondage";
#elif L==5
	"Poner encuesta a cero";	// Okoteve traduccin
#elif L==6
	"Resettare sondaggio";
#elif L==7
	"Reset badania";
#elif L==8
	"Reiniciar inqu&eacute;rito";
#endif

const char *Txt_results_per_page =
#if   L==0
	"resultados por p&aacute;gina";	// Necessita traduccio
#elif L==1
	"results per page";		// Need bersetzung
#elif L==2
	"results per page";
#elif L==3
	"resultados por p&aacute;gina";
#elif L==4
	"results per page";		// Besoin de traduction
#elif L==5
	"resultados por p&aacute;gina";	// Okoteve traduccin
#elif L==6
	"risultati per pagina";
#elif L==7
	"wynik&oacute;w na stronie";
#elif L==8
	"results per page";		// Necessita de traduo
#endif

const char *Txt_Results_tests =
#if   L==0
	"Resultats tests";
#elif L==1
	"Ergebnisse Tests";
#elif L==2
	"Results tests";
#elif L==3
	"Resultados tests";
#elif L==4
	"R&eacute;sultats tests";
#elif L==5
	"Resultados tests";			// Okoteve traduccin
#elif L==6
	"Risultati test";
#elif L==7
	"Wyniki test&oacute;w";
#elif L==8
	"Resultados testes";
#endif

const char *Txt_Retype_new_password =
#if   L==0
	"Reescriviu la nova contrasenya";
#elif L==1
	"Wiederholung neues Passwort";
#elif L==2
	"Retype new password";
#elif L==3
	"Reescriba la nueva contrase&ntilde;a";
#elif L==4
	"Confirmez votre nouveau mot de passe";
#elif L==5
	"Reescriba la nueva contrase&ntilde;a";	// Okoteve traduccin
#elif L==6
	"Riscrivi la nuova password";
#elif L==7
	"Powt&oacute;rz nowe has&lstrok;o";
#elif L==8
	"Redigite a nova senha";
#endif

const char *Txt_Retype_password =
#if   L==0
	"Reescriviu la contrasenya";
#elif L==1
	"Wiederholung Passwort";
#elif L==2
	"Retype password";
#elif L==3
	"Reescriba la contrase&ntilde;a";
#elif L==4
	"Confirmez votre mot de passe";
#elif L==5
	"Reescriba la contrase&ntilde;a";	// Okoteve traduccin
#elif L==6
	"Riscrivi la password";
#elif L==7
	"Powt&oacute;rz has&lstrok;o";
#elif L==8
	"Redigite a senha";
#endif

const char *Txt_Role =
#if   L==0
	"Rol";
#elif L==1
	"Rolle";
#elif L==2
	"Role";
#elif L==3
	"Rol";
#elif L==4
	"R&ocirc;le";
#elif L==5
	"Rol";	// Okoteve traduccin
#elif L==6
	"Ruolo";
#elif L==7
	"Rol&eogon;";
#elif L==8
	"Papel";
#endif

const char *Txt_ROLE_STATS[Sta_NUM_ROLES_STAT] =
   {	// Sta_IDENTIFIED_USRS
#if   L==0
	"identificats"
#elif L==1
	"Identifiziert"
#elif L==2
	"identified"
#elif L==3
	"identificados"
#elif L==4
	"identifi&eacute;s"
#elif L==5
	"identificados"	// Okoteve traduccin
#elif L==6
	"identificati"
#elif L==7
	"zidentyfikowane"
#elif L==8
	"identificados"
#endif
	,	// Sta_ALL_USRS
#if   L==0
	"identificats o desconeguts"
#elif L==1
	"Identifiziert oder Unbekannte"
#elif L==2
	"identified or unknown"
#elif L==3
	"identificados o desconocidos"
#elif L==4
	"identifi&eacute;s ou inconnus"
#elif L==5
	"identificados o desconocidos"	// Okoteve traduccin
#elif L==6
	"identificati o sconosciuti"
#elif L==7
	"zidentyfikowanej lub nieznane"
#elif L==8
	"identificados ou desconhecidos"
#endif
	,	// Sta_INS_ADMINS
#if   L==0
	"administradors d'institucions"
#elif L==1
	"Administratoren von Hochschulen"
#elif L==2
	"administrators of institutions"
#elif L==3
	"administradores de instituciones"
#elif L==4
	"administrateurs d'&eacute;tablissements"
#elif L==5
	"administradores de instituciones"	// Okoteve traduccin
#elif L==6
	"amministratori di istituzioni"
#elif L==7
	"Administratorzy instytucje"
#elif L==8
	"administradores de institu&ccedil;&otilde;es"
#endif
	,	// Sta_CTR_ADMINS
#if   L==0
	"administradors de centres"
#elif L==1
	"Administratoren von Lehrinstitute"
#elif L==2
	"administrators of centres"
#elif L==3
	"administradores de centros"
#elif L==4
	"administrateurs de centres"
#elif L==5
	"administradores de centros"	// Okoteve traduccin
#elif L==6
	"amministratori di centri"
#elif L==7
	"Administratorzy centra"
#elif L==8
	"administradores de centros"
#endif
	,	// Sta_DEG_ADMINS
#if   L==0
	"administradors de titulacions"
#elif L==1
	"Administratoren von Studieng&auml;nge"
#elif L==2
	"administrators of degrees"
#elif L==3
	"administradores de titulaciones"
#elif L==4
	"administrateurs de degr&eacute;s"
#elif L==5
	"administradores de titulaciones"	// Okoteve traduccin
#elif L==6
	"amministratori di gradi"
#elif L==7
	"Administratorzy stopni"
#elif L==8
	"administradores de graus"
#endif
	,	// Sta_TEACHERS
#if   L==0
	"professors"
#elif L==1
	"Lehrkr&auml;fte"
#elif L==2
	"teachers"
#elif L==3
	"profesores"
#elif L==4
	"enseignants"
#elif L==5
	"mbo'eh&aacute;ra"
#elif L==6
	"professori"
#elif L==7
	"nauczycieli"
#elif L==8
	"professores"
#endif
	,	// Sta_STUDENTS
#if   L==0
	"estudiants"
#elif L==1
	"Studenten"
#elif L==2
	"students"
#elif L==3
	"estudiantes"
#elif L==4
	"&eacute;tudiants"
#elif L==5
	"temimbo'e"
#elif L==6
	"studenti"
#elif L==7
	"studenci"
#elif L==8
	"estudantes"
#endif
	,	// Sta_VISITORS
#if   L==0
	"visitants"
#elif L==1
	"Besucher"
#elif L==2
	"visitors"
#elif L==3
	"visitantes"
#elif L==4
	"visiteurs"
#elif L==5
	"visitantes"	// Okoteve traduccin
#elif L==6
	"visitatori"
#elif L==7
	"odwiedzaj&aogon;cy"
#elif L==8
	"visitantes"
#endif
	,	// Sta_GUESTS
#if   L==0
	"convidats (sense assignatures)"
#elif L==1
	"Eingeladen (ohne Kursen)"
#elif L==2
	"guests (without courses)"
#elif L==3
	"invitados (sin asignaturas)"
#elif L==4
	"invit&eacute;s (sans mati&egrave;res)"
#elif L==5
	"invitados (sin asignaturas)"	// Okoteve traduccin
#elif L==6
	"ospiti (senza corsi)"
#elif L==7
	"zaproszeny (bez kurs&oacute;w)"
#elif L==8
	"convidados (sem disciplinas)"
#endif
	,	// Sta_UNKNOWN_USRS
#if   L==0
	"desconeguts"
#elif L==1
	"Unbekannte"
#elif L==2
	"unknown"
#elif L==3
	"desconocidos"
#elif L==4
	"inconnus"
#elif L==5
	"desconocidos"	// Okoteve traduccin
#elif L==6
	"sconosciuti"
#elif L==7
	"nieznany"
#elif L==8
	"desconhecidos"
#endif
	,	// Sta_ME
#if   L==0
	"jo"
#elif L==1
	"Mich"
#elif L==2
	"me"
#elif L==3
	"yo"
#elif L==4
	"moi"
#elif L==5
	"che"
#elif L==6
	"me"
#elif L==7
	"ja"
#elif L==8
	"eu"
#endif
	};

const char *Txt_ROLES_SINGULAR_Abc[Rol_NUM_ROLES][Usr_NUM_SEXS] =
   {
	{
#if   L==0	// Rol_ROLE_UNKNOWN - Usr_SEX_UNKNOWN
	"Desconocido/a"	// Necessita traduccio
#elif L==1
	"Unbekannter/e"
#elif L==2
	"Unknown"
#elif L==3
	"Desconocido/a"
#elif L==4
	"Inconnu/e"
#elif L==5
	"Desconocido/a"	// Okoteve traduccin
#elif L==6
	"Sconosciuto/a"
#elif L==7
	"Nieznany"
#elif L==8
	"Desconhecido/a"
#endif
	,
#if   L==0	// Rol_ROLE_UNKNOWN - Usr_SEX_FEMALE
	"Desconocida"	// Necessita traduccio
#elif L==1
	"Unbekannte"
#elif L==2
	"Unknown"
#elif L==3
	"Desconocida"
#elif L==4
	"Inconnue"
#elif L==5
	"Desconocida"	// Okoteve traduccin
#elif L==6
	"Sconosciuta"
#elif L==7
	"Nieznana"
#elif L==8
	"Desconhecida"
#endif
	,
#if   L==0	// Rol_ROLE_UNKNOWN - Usr_SEX_MALE
	"Desconocido"	// Necessita traduccio
#elif L==1
	"Unbekannter"
#elif L==2
	"Unknown"
#elif L==3
	"Desconocido"
#elif L==4
	"Inconnu"
#elif L==5
	"Desconocido"	// Okoteve traduccin
#elif L==6
	"Sconosciuto"
#elif L==7
	"Nieznany"
#elif L==8
	"Desconhecido"
#endif
	,
#if   L==0	// Rol_ROLE_UNKNOWN - Usr_SEX_ALL
	"Desconocido/a"	// Necessita traduccio
#elif L==1
	"Unbekannter/e"
#elif L==2
	"Unknown"
#elif L==3
	"Desconocido/a"
#elif L==4
	"Inconnu/e"
#elif L==5
	"Desconocido/a"	// Okoteve traduccin
#elif L==6
	"Sconosciuto/a"
#elif L==7
	"Nieznani"
#elif L==8
	"Desconhecido/a"
#endif
	 },
	 {
#if   L==0	// Rol_ROLE_GUEST - Usr_SEX_UNKNOWN
	   "Invitado/a"	// Necessita traduccio
#elif L==1
	   "Gast"
#elif L==2
	   "Guest"
#elif L==3
	   "Invitado/a"
#elif L==4
	   "Invit&eacute;/e"
#elif L==5
	   "Invitado/a"	// Okoteve traduccin
#elif L==6
	   "Ospite"
#elif L==7
	   "Go&sacute;&cacute;"
#elif L==8
	   "Convidado/a"
#endif
	   ,
#if   L==0	// Rol_ROLE_GUEST - Usr_SEX_FEMALE
	   "Invitada"	// Necessita traduccio
#elif L==1
	   "Gast"
#elif L==2
	   "Guest"
#elif L==3
	   "Invitada"
#elif L==4
	   "Invit&eacute;e"
#elif L==5
	   "Invitada"	// Okoteve traduccin
#elif L==6
	   "Ospite"
#elif L==7
	   "Opinia"
#elif L==8
	   "Convidada"
#endif
	   ,
#if   L==0	// Rol_ROLE_GUEST - Usr_SEX_MALE
	   "Invitado"	// Necessita traduccio
#elif L==1
	   "Gast"
#elif L==2
	   "Guest"
#elif L==3
	   "Invitado"
#elif L==4
	   "Invit&eacute;"
#elif L==5
	   "Invitado"	// Okoteve traduccin
#elif L==6
	   "Ospite"
#elif L==7
	   "Opinia"
#elif L==8
	   "Convidado"
#endif
	   ,
#if   L==0	// Rol_ROLE_GUEST - Usr_SEX_ALL
	   "Invitado/a"	// Necessita traduccio
#elif L==1
	   "Gast"
#elif L==2
	   "Guest"
#elif L==3
	   "Invitado/a"
#elif L==4
	   "Invit&eacute;/e"
#elif L==5
	   "Invitado/a"	// Okoteve traduccin
#elif L==6
	   "Ospite"
#elif L==7
	   "Opinie"
#elif L==8
	   "Convidado/a"
#endif
	 },
	 {
#if   L==0	// Rol_ROLE_VISITOR - Usr_SEX_UNKNOWN
	   "Visitant"
#elif L==1
	   "Besucher"
#elif L==2
	   "Visitor"
#elif L==3
	   "Visitante"
#elif L==4
	   "Visiteur"
#elif L==5
	   "Visitante"	// Okoteve traduccin
#elif L==6
	   "Visitatore"
#elif L==7
	   "Go&sacute;&cacute;"
#elif L==8
	   "Visitante"
#endif
	   ,
#if   L==0	// Rol_ROLE_VISITOR - Usr_SEX_FEMALE
	   "Visitant"
#elif L==1
	   "Besucher"
#elif L==2
	   "Visitor"
#elif L==3
	   "Visitante"
#elif L==4
	   "Visiteur"
#elif L==5
	   "Visitante"	// Okoteve traduccin
#elif L==6
	   "Visitatore"
#elif L==7
	   "Go&sacute;&cacute;"
#elif L==8
	   "Visitante"
#endif
	   ,
#if   L==0	// Rol_ROLE_VISITOR - Usr_SEX_MALE
	   "Visitant"
#elif L==1
	   "Besucher"
#elif L==2
	   "Visitor"
#elif L==3
	   "Visitante"
#elif L==4
	   "Visiteur"
#elif L==5
	   "Visitante"	// Okoteve traduccin
#elif L==6
	   "Visitatore"
#elif L==7
	   "Go&sacute;&cacute;"
#elif L==8
	   "Visitante"
#endif
	   ,
#if   L==0	// Rol_ROLE_VISITOR - Usr_SEX_ALL
	   "Visitant"
#elif L==1
	   "Besucher"
#elif L==2
	   "Visitor"
#elif L==3
	   "Visitante"
#elif L==4
	   "Visiteur"
#elif L==5
	   "Visitante"	// Okoteve traduccin
#elif L==6
	   "Visitatore"
#elif L==7
	   "Go&sacute;&cacute;"
#elif L==8
	   "Visitante"
#endif
	 },
	 {
#if   L==0	// Rol_ROLE_STUDENT - Usr_SEX_UNKNOWN
	   "Estudiant"
#elif L==1
	   "Student"
#elif L==2
	   "Student"
#elif L==3
	   "Estudiante"
#elif L==4
	   "&Eacute;tudiant/e"
#elif L==5
	   "Temimbo'e"
#elif L==6
	   "Studente"
#elif L==7
	   "Student"
#elif L==8
	   "Estudante"
#endif
	   ,
#if   L==0	// Rol_ROLE_STUDENT - Usr_SEX_FEMALE
	   "Estudiant"
#elif L==1
	   "Student"
#elif L==2
	   "Student"
#elif L==3
	   "Estudiante"
#elif L==4
	   "&Eacute;tudiante"
#elif L==5
	   "Temimbo'e"
#elif L==6
	   "Studentessa"
#elif L==7
	   "Studentka"
#elif L==8
	   "Estudante"
#endif
	   ,
#if   L==0	// Rol_ROLE_STUDENT - Usr_SEX_MALE
	   "Estudiant"
#elif L==1
	   "Student"
#elif L==2
	   "Student"
#elif L==3
	   "Estudiante"
#elif L==4
	   "&Eacute;tudiant"
#elif L==5
	   "Temimbo'e"
#elif L==6
	   "Studente"
#elif L==7
	   "Student"
#elif L==8
	   "Estudante"
#endif
	   ,
#if   L==0	// Rol_ROLE_STUDENT - Usr_SEX_ALL
	   "Estudiant"
#elif L==1
	   "Student"
#elif L==2
	   "Student"
#elif L==3
	   "Estudiante"
#elif L==4
	   "&Eacute;tudiant/e"
#elif L==5
	   "Temimbo'e"
#elif L==6
	   "Studente"
#elif L==7
	   "Studenci"
#elif L==8
	   "Estudante"
#endif
	 },
	 {
#if   L==0	// Rol_ROLE_TEACHER - Usr_SEX_UNKNOWN
	   "Professor/a"
#elif L==1
	   "Lehrkraft"
#elif L==2
	   "Teacher"
#elif L==3
	   "Profesor/a"
#elif L==4
	   "Enseignant/e"
#elif L==5
	   "Mbo'eh&aacute;ra"
#elif L==6
	   "Professore/ssa"
#elif L==7
	   "Nauczyciel"
#elif L==8
	   "Professor/a"
#endif
	   ,
#if   L==0	// Rol_ROLE_TEACHER - Usr_SEX_FEMALE
	   "Professora"
#elif L==1
	   "Lehrkraft"
#elif L==2
	   "Teacher"
#elif L==3
	   "Profesora"
#elif L==4
	   "Enseignante"
#elif L==5
	   "Mbo'eh&aacute;ra"
#elif L==6
	   "Professoressa"
#elif L==7
	   "Nauczycielka"
#elif L==8
	   "Professora"
#endif
	   ,
#if   L==0	// Rol_ROLE_TEACHER - Usr_SEX_MALE
	   "Professor"
#elif L==1
	   "Lehrkraft"
#elif L==2
	   "Teacher"
#elif L==3
	   "Profesor"
#elif L==4
	   "Enseignant"
#elif L==5
	   "Mbo'eh&aacute;ra"
#elif L==6
	   "Professore"
#elif L==7
	   "Nauczyciel"
#elif L==8
	   "Professor"
#endif
	   ,
#if   L==0	// Rol_ROLE_TEACHER - Usr_SEX_ALL
	   "Professor/a"
#elif L==1
	   "Lehrkraft"
#elif L==2
	   "Teacher"
#elif L==3
	   "Profesor/a"
#elif L==4
	   "Enseignant/e"
#elif L==5
	   "Mbo'eh&aacute;ra"
#elif L==6
	   "Professore/ssa"
#elif L==7
	   "Nauczyciele"
#elif L==8
	   "Professor/a"
#endif
	 },
	 {
#if   L==0	// Rol_ROLE_DEG_ADMIN - Usr_SEX_UNKNOWN
	   "Administrador/a"	// Necessita traduccio
#elif L==1
	   "Administrator"
#elif L==2
	   "Administrator"
#elif L==3
	   "Administrador/a"
#elif L==4
	   "Administrateur/se"
#elif L==5
	   "&Ntilde;angarekoh&aacute;ra"
#elif L==6
	   "Amministratore"
#elif L==7
	   "Administrator"
#elif L==8
	   "Administrador/a"
#endif
	   ,
#if   L==0	// Rol_ROLE_DEG_ADMIN - Usr_SEX_FEMALE
	   "Administradora"	// Necessita traduccio
#elif L==1
	   "Administrator"
#elif L==2
	   "Administrator"
#elif L==3
	   "Administradora"
#elif L==4
	   "Administrateuse"
#elif L==5
	   "&Ntilde;angarekoh&aacute;ra"
#elif L==6
	   "Amministratore"
#elif L==7
	   "Administratorka"
#elif L==8
	   "Administradora"
#endif
	   ,
#if   L==0	// Rol_ROLE_DEG_ADMIN - Usr_SEX_MALE
	   "Administrador"	// Necessita traduccio
#elif L==1
	   "Administrator"
#elif L==2
	   "Administrator"
#elif L==3
	   "Administrador"
#elif L==4
	   "Administrateur"
#elif L==5
	   "&Ntilde;angarekoh&aacute;ra"
#elif L==6
	   "Amministratore"
#elif L==7
	   "Administrator"
#elif L==8
	   "Administrador"
#endif
	   ,
#if   L==0	// Rol_ROLE_DEG_ADMIN - Usr_SEX_ALL
	   "Administrador/a"	// Necessita traduccio
#elif L==1
	   "Administrator"
#elif L==2
	   "Administrator"
#elif L==3
	   "Administrador/a"
#elif L==4
	   "Administrateur/se"
#elif L==5
	   "&Ntilde;angarekoh&aacute;ra"
#elif L==6
	   "Amministratore"
#elif L==7
	   "Administratorzy"
#elif L==8
	   "Administrador/a"
#endif
	 },
	 {
#if   L==0	// Rol_ROLE_CTR_ADMIN - Usr_SEX_UNKNOWN
	   "Administrador/a"	// Necessita traduccio
#elif L==1
	   "Administrator"
#elif L==2
	   "Administrator"
#elif L==3
	   "Administrador/a"
#elif L==4
	   "Administrateur/se"
#elif L==5
	   "&Ntilde;angarekoh&aacute;ra"
#elif L==6
	   "Amministratore"
#elif L==7
	   "Administrator"
#elif L==8
	   "Administrador/a"
#endif
	   ,
#if   L==0	// Rol_ROLE_CTR_ADMIN - Usr_SEX_FEMALE
	   "Administradora"	// Necessita traduccio
#elif L==1
	   "Administrator"
#elif L==2
	   "Administrator"
#elif L==3
	   "Administradora"
#elif L==4
	   "Administrateuse"
#elif L==5
	   "&Ntilde;angarekoh&aacute;ra"
#elif L==6
	   "Amministratore"
#elif L==7
	   "Administratorka"
#elif L==8
	   "Administradora"
#endif
	   ,
#if   L==0	// Rol_ROLE_CTR_ADMIN - Usr_SEX_MALE
	   "Administrador"	// Necessita traduccio
#elif L==1
	   "Administrator"
#elif L==2
	   "Administrator"
#elif L==3
	   "Administrador"
#elif L==4
	   "Administrateur"
#elif L==5
	   "&Ntilde;angarekoh&aacute;ra"
#elif L==6
	   "Amministratore"
#elif L==7
	   "Administrator"
#elif L==8
	   "Administrador"
#endif
	   ,
#if   L==0	// Rol_ROLE_CTR_ADMIN - Usr_SEX_ALL
	   "Administrador/a"	// Necessita traduccio
#elif L==1
	   "Administrator"
#elif L==2
	   "Administrator"
#elif L==3
	   "Administrador/a"
#elif L==4
	   "Administrateur/se"
#elif L==5
	   "&Ntilde;angarekoh&aacute;ra"
#elif L==6
	   "Amministratore"
#elif L==7
	   "Administratorzy"
#elif L==8
	   "Administrador/a"
#endif
	 },
	 {
#if   L==0	// Rol_ROLE_INS_ADMIN - Usr_SEX_UNKNOWN
	   "Administrador/a"	// Necessita traduccio
#elif L==1
	   "Administrator"
#elif L==2
	   "Administrator"
#elif L==3
	   "Administrador/a"
#elif L==4
	   "Administrateur/se"
#elif L==5
	   "&Ntilde;angarekoh&aacute;ra"
#elif L==6
	   "Amministratore"
#elif L==7
	   "Administrator"
#elif L==8
	   "Administrador/a"
#endif
	   ,
#if   L==0	// Rol_ROLE_INS_ADMIN - Usr_SEX_FEMALE
	   "Administradora"	// Necessita traduccio
#elif L==1
	   "Administrator"
#elif L==2
	   "Administrator"
#elif L==3
	   "Administradora"
#elif L==4
	   "Administrateuse"
#elif L==5
	   "&Ntilde;angarekoh&aacute;ra"
#elif L==6
	   "Amministratore"
#elif L==7
	   "Administratorka"
#elif L==8
	   "Administradora"
#endif
	   ,
#if   L==0	// Rol_ROLE_INS_ADMIN - Usr_SEX_MALE
	   "Administrador"	// Necessita traduccio
#elif L==1
	   "Administrator"
#elif L==2
	   "Administrator"
#elif L==3
	   "Administrador"
#elif L==4
	   "Administrateur"
#elif L==5
	   "&Ntilde;angarekoh&aacute;ra"
#elif L==6
	   "Amministratore"
#elif L==7
	   "Administrator"
#elif L==8
	   "Administrador"
#endif
	   ,
#if   L==0	// Rol_ROLE_INS_ADMIN - Usr_SEX_ALL
	   "Administrador/a"	// Necessita traduccio
#elif L==1
	   "Administrator"
#elif L==2
	   "Administrator"
#elif L==3
	   "Administrador/a"
#elif L==4
	   "Administrateur/se"
#elif L==5
	   "&Ntilde;angarekoh&aacute;ra"
#elif L==6
	   "Amministratore"
#elif L==7
	   "Administratorzy"
#elif L==8
	   "Administrador/a"
#endif
	 },
	 {
#if   L==0	// Rol_ROLE_SUPERUSER - Usr_SEX_UNKNOWN
	   "Superusuari/&agrave;ria"
#elif L==1
	   "Superuser"
#elif L==2
	   "Superuser"
#elif L==3
	   "Superusuario/a"
#elif L==4
	   "Superutilisateur/se"
#elif L==5
	   "Superusuario/a"	// Okoteve traduccin
#elif L==6
	   "Superutente"
#elif L==7
	   "Superuzytkownik"
#elif L==8
	   "Superutilizador/a"
#endif
	   ,
#if   L==0	// Rol_ROLE_SUPERUSER - Usr_SEX_FEMALE
	   "Superusuaria"	// Necessita traduccio
#elif L==1
	   "Superuser"
#elif L==2
	   "Superuser"
#elif L==3
	   "Superusuaria"
#elif L==4
	   "Superutilisateuse"
#elif L==5
	   "Superusuaria"	// Okoteve traduccin
#elif L==6
	   "Superutente"
#elif L==7
	   "Superuzytkownik"
#elif L==8
	   "Superutilizadora"
#endif
	   ,
#if   L==0	// Rol_ROLE_SUPERUSER - Usr_SEX_MALE
	   "Superusuari"
#elif L==1
	   "Superuser"
#elif L==2
	   "Superuser"
#elif L==3
	   "Superusuario"
#elif L==4
	   "Superutilisateur"
#elif L==5
	   "Superusuario"	// Okoteve traduccin
#elif L==6
	   "Superutente"
#elif L==7
	   "Superuzytkownik"
#elif L==8
	   "Superutilizador"
#endif
	   ,
#if   L==0	// Rol_ROLE_SUPERUSER - Usr_SEX_ALL
	   "Superusuari/&agrave;ria"	// Necessita traduccio
#elif L==1
	   "Superuser"
#elif L==2
	   "Superuser"
#elif L==3
	   "Superusuario/a"
#elif L==4
	   "Superutilisateur/se"
#elif L==5
	   "Superusuario/a"		// Okoteve traduccin
#elif L==6
	   "Superutente"
#elif L==7
	   "Superuzytkownicy"
#elif L==8
	   "Superutilizador/a"
#endif
	 } };

const char *Txt_ROLES_SINGULAR_abc[Rol_NUM_ROLES][Usr_NUM_SEXS] =
   {
	{
#if   L==0	// Rol_ROLE_UNKNOWN - Usr_SEX_UNKNOWN
	"desconocido/a"		// Necessita traduccio
#elif L==1
	"Unbekannter/e"
#elif L==2
	"unknown"
#elif L==3
	"desconocido/a"
#elif L==4
	"inconnu/e"
#elif L==5
	"desconocido/a"		// Okoteve traduccin
#elif L==6
	"sconosciuto/a"
#elif L==7
	"nieznany"
#elif L==8
	"desconhecido/a"
#endif
	,
#if   L==0	// Rol_ROLE_UNKNOWN - Usr_SEX_FEMALE
	"desconocida"	// Necessita traduccio
#elif L==1
	"Unbekannte"
#elif L==2
	"unknown"
#elif L==3
	"desconocida"
#elif L==4
	"inconnue"
#elif L==5
	"desconocida"	// Okoteve traduccin
#elif L==6
	"sconosciuta"
#elif L==7
	"nieznana"
#elif L==8
	"desconhecida"
#endif
	,
#if   L==0	// Rol_ROLE_UNKNOWN - Usr_SEX_MALE
	"desconocido"	// Necessita traduccio
#elif L==1
	"Unbekannter"
#elif L==2
	"unknown"
#elif L==3
	"desconocido"
#elif L==4
	"inconnu"
#elif L==5
	"desconocido"	// Okoteve traduccin
#elif L==6
	"sconosciuto"
#elif L==7
	"nieznany"
#elif L==8
	"desconhecido"
#endif
	,
#if   L==0	// Rol_ROLE_UNKNOWN - Usr_SEX_ALL
	"desconocido/a"		// Necessita traduccio
#elif L==1
	"Unbekannter/e"
#elif L==2
	"unknown"
#elif L==3
	"desconocido/a"
#elif L==4
	"inconnu/e"
#elif L==5
	"desconocido/a"		// Okoteve traduccin
#elif L==6
	"sconosciuto/a"
#elif L==7
	"nieznani"
#elif L==8
	"desconhecido/a"
#endif
	 },
	 {
#if   L==0	// Rol_ROLE_GUEST - Usr_SEX_UNKNOWN
	   "invitado/a"	// Necessita traduccio
#elif L==1
	   "Gast"
#elif L==2
	   "guest"
#elif L==3
	   "invitado/a"
#elif L==4
	   "invit&eacute;/e"
#elif L==5
	   "invitado/a"	// Okoteve traduccin
#elif L==6
	   "ospite"
#elif L==7
	   "go&sacute;&cacute;"
#elif L==8
	   "convidado/a"
#endif
	   ,
#if   L==0	// Rol_ROLE_GUEST - Usr_SEX_FEMALE
	   "invitada"	// Necessita traduccio
#elif L==1
	   "Gast"
#elif L==2
	   "guest"
#elif L==3
	   "invitada"
#elif L==4
	   "invit&eacute;e"
#elif L==5
	   "invitada"	// Okoteve traduccin
#elif L==6
	   "ospite"
#elif L==7
	   "goscinna"
#elif L==8
	   "convidada"
#endif
	   ,
#if   L==0	// Rol_ROLE_GUEST - Usr_SEX_MALE
	   "invitado"	// Necessita traduccio
#elif L==1
	   "Gast"
#elif L==2
	   "guest"
#elif L==3
	   "invitado"
#elif L==4
	   "invit&eacute;"
#elif L==5
	   "invitado"	// Okoteve traduccin
#elif L==6
	   "ospite"
#elif L==7
	   "goscinny"
#elif L==8
	   "convidado"
#endif
	   ,
#if   L==0	// Rol_ROLE_GUEST - Usr_SEX_ALL
	   "invitado/a"	// Necessita traduccio
#elif L==1
	   "Gast"
#elif L==2
	   "guest"
#elif L==3
	   "invitado/a"
#elif L==4
	   "invit&eacute;/e"
#elif L==5
	   "invitado/a"	// Okoteve traduccin
#elif L==6
	   "ospite"
#elif L==7
	   "goscinni"
#elif L==8
	   "convidado/a"
#endif
	 },
	 {
#if   L==0	// Rol_ROLE_VISITOR - Usr_SEX_UNKNOWN
	   "visitant"
#elif L==1
	   "Besucher"
#elif L==2
	   "visitor"
#elif L==3
	   "visitante"
#elif L==4
	   "visiteur"
#elif L==5
	   "visitante"	// Okoteve traduccin
#elif L==6
	   "visitatore"
#elif L==7
	   "go&sacute;&cacute;"
#elif L==8
	   "Visitante"
#endif
	   ,
#if   L==0	// Rol_ROLE_VISITOR - Usr_SEX_FEMALE
	   "visitant"
#elif L==1
	   "Besucher"
#elif L==2
	   "visitor"
#elif L==3
	   "visitante"
#elif L==4
	   "visiteur"
#elif L==5
	   "visitante"	// Okoteve traduccin
#elif L==6
	   "visitatore"
#elif L==7
	   "go&sacute;&cacute;"
#elif L==8
	   "Visitante"
#endif
	   ,
#if   L==0	// Rol_ROLE_VISITOR - Usr_SEX_MALE
	   "visitant"
#elif L==1
	   "Besucher"
#elif L==2
	   "visitor"
#elif L==3
	   "visitante"
#elif L==4
	   "visiteur"
#elif L==5
	   "visitante"	// Okoteve traduccin
#elif L==6
	   "visitatore"
#elif L==7
	   "go&sacute;&cacute;"
#elif L==8
	   "Visitante"
#endif
	   ,
#if   L==0	// Rol_ROLE_VISITOR - Usr_SEX_ALL
	   "visitant"
#elif L==1
	   "Besucher"
#elif L==2
	   "visitor"
#elif L==3
	   "visitante"
#elif L==4
	   "visiteur"
#elif L==5
	   "visitante"	// Okoteve traduccin
#elif L==6
	   "visitatore"
#elif L==7
	   "go&sacute;&cacute;"
#elif L==8
	   "Visitante"
#endif
	 },
	 {
#if   L==0	// Rol_ROLE_STUDENT - Usr_SEX_UNKNOWN
	   "estudiant"
#elif L==1
	   "Student"
#elif L==2
	   "student"
#elif L==3
	   "estudiante"
#elif L==4
	   "&eacute;tudiant/e"
#elif L==5
	   "temimbo'e"
#elif L==6
	   "studente"
#elif L==7
	   "student"
#elif L==8
	   "estudante"
#endif
	   ,
#if   L==0	// Rol_ROLE_STUDENT - Usr_SEX_FEMALE
	   "estudiant"
#elif L==1
	   "Student"
#elif L==2
	   "student"
#elif L==3
	   "estudiante"
#elif L==4
	   "&eacute;tudiante"
#elif L==5
	   "temimbo'e"
#elif L==6
	   "studentessa"
#elif L==7
	   "studentka"
#elif L==8
	   "estudante"
#endif
	   ,
#if   L==0	// Rol_ROLE_STUDENT - Usr_SEX_MALE
	   "estudiant"
#elif L==1
	   "Student"
#elif L==2
	   "student"
#elif L==3
	   "estudiante"
#elif L==4
	   "&eacute;tudiant"
#elif L==5
	   "temimbo'e"
#elif L==6
	   "studente"
#elif L==7
	   "student"
#elif L==8
	   "estudante"
#endif
	   ,
#if   L==0	// Rol_ROLE_STUDENT - Usr_SEX_ALL
	   "estudiant"
#elif L==1
	   "Student"
#elif L==2
	   "student"
#elif L==3
	   "estudiante"
#elif L==4
	   "&eacute;tudiant/e"
#elif L==5
	   "temimbo'e"
#elif L==6
	   "studente/ssa"
#elif L==7
	   "studenci"
#elif L==8
	   "estudante"
#endif
	 },
	 {
#if   L==0	// Rol_ROLE_TEACHER - Usr_SEX_UNKNOWN
	   "professor/a"
#elif L==1
	   "Lehrkraft"
#elif L==2
	   "teacher"
#elif L==3
	   "profesor/a"
#elif L==4
	   "enseignant/e"
#elif L==5
	   "mbo'eh&aacute;ra"
#elif L==6
	   "Professore/ssa"
#elif L==7
	   "nauczyciel"
#elif L==8
	   "professor/a"
#endif
	   ,
#if   L==0	// Rol_ROLE_TEACHER - Usr_SEX_FEMALE
	   "professora"
#elif L==1
	   "Lehrkraft"
#elif L==2
	   "teacher"
#elif L==3
	   "profesora"
#elif L==4
	   "enseignante"
#elif L==5
	   "mbo'eh&aacute;ra"
#elif L==6
	   "Professoressa"
#elif L==7
	   "nauczycielka"
#elif L==8
	   "professora"
#endif
	   ,
#if   L==0	// Rol_ROLE_TEACHER - Usr_SEX_MALE
	   "professor"
#elif L==1
	   "Lehrkraft"
#elif L==2
	   "teacher"
#elif L==3
	   "profesor"
#elif L==4
	   "enseignant"
#elif L==5
	   "mbo'eh&aacute;ra"
#elif L==6
	   "professore"
#elif L==7
	   "nauczyciel"
#elif L==8
	   "professor"
#endif
	   ,
#if   L==0	// Rol_ROLE_TEACHER - Usr_SEX_ALL
	   "professor/a"
#elif L==1
	   "Lehrkraft"
#elif L==2
	   "teacher"
#elif L==3
	   "profesor/a"
#elif L==4
	   "enseignant/e"
#elif L==5
	   "mbo'eh&aacute;ra"
#elif L==6
	   "professore/ssa"
#elif L==7
	   "nauczyciele"
#elif L==8
	   "professor/a"
#endif
	 },
	 {
#if   L==0	// Rol_ROLE_DEG_ADMIN - Usr_SEX_UNKNOWN
	   "administrador/a"	// Necessita traduccio
#elif L==1
	   "Administrator"
#elif L==2
	   "administrator"
#elif L==3
	   "administrador/a"
#elif L==4
	   "administrateur/se"
#elif L==5
	   "&ntilde;angarekoh&aacute;ra"
#elif L==6
	   "amministratore"
#elif L==7
	   "administrator"
#elif L==8
	   "administrador/a"
#endif
	   ,
#if   L==0	// Rol_ROLE_DEG_ADMIN - Usr_SEX_FEMALE
	   "administradora"	// Necessita traduccio
#elif L==1
	   "Administrator"
#elif L==2
	   "administrator"
#elif L==3
	   "administradora"
#elif L==4
	   "administrateuse"
#elif L==5
	   "&ntilde;angarekoh&aacute;ra"
#elif L==6
	   "amministratore"
#elif L==7
	   "administratorka"
#elif L==8
	   "administradora"
#endif
	   ,
#if   L==0	// Rol_ROLE_DEG_ADMIN - Usr_SEX_MALE
	   "administrador"		// Necessita traduccio
#elif L==1
	   "Administrator"
#elif L==2
	   "administrator"
#elif L==3
	   "administrador"
#elif L==4
	   "administrateur"
#elif L==5
	   "&ntilde;angarekoh&aacute;ra"
#elif L==6
	   "amministratore"
#elif L==7
	   "administrator"
#elif L==8
	   "administrador"
#endif
	   ,
#if   L==0	// Rol_ROLE_DEG_ADMIN - Usr_SEX_ALL
	   "administrador/a"
#elif L==1
	   "Administrator"
#elif L==2
	   "administrator"
#elif L==3
	   "administrador/a"
#elif L==4
	   "administrateur/se"
#elif L==5
	   "&ntilde;angarekoh&aacute;ra"
#elif L==6
	   "amministratore"
#elif L==7
	   "administratorzy"
#elif L==8
	   "administrador/a"
#endif
	 },
	 {
#if   L==0	// Rol_ROLE_CTR_ADMIN - Usr_SEX_UNKNOWN
	   "administrador/a"	// Necessita traduccio
#elif L==1
	   "Administrator"
#elif L==2
	   "administrator"
#elif L==3
	   "administrador/a"
#elif L==4
	   "administrateur/se"
#elif L==5
	   "&ntilde;angarekoh&aacute;ra"
#elif L==6
	   "amministratore"
#elif L==7
	   "administrator"
#elif L==8
	   "administrador/a"
#endif
	   ,
#if   L==0	// Rol_ROLE_CTR_ADMIN - Usr_SEX_FEMALE
	   "administradora"	// Necessita traduccio
#elif L==1
	   "Administrator"
#elif L==2
	   "administrator"
#elif L==3
	   "administradora"
#elif L==4
	   "administrateuse"
#elif L==5
	   "&ntilde;angarekoh&aacute;ra"
#elif L==6
	   "amministratore"
#elif L==7
	   "administratorka"
#elif L==8
	   "administradora"
#endif
	   ,
#if   L==0	// Rol_ROLE_CTR_ADMIN - Usr_SEX_MALE
	   "administrador"		// Necessita traduccio
#elif L==1
	   "Administrator"
#elif L==2
	   "administrator"
#elif L==3
	   "administrador"
#elif L==4
	   "administrateur"
#elif L==5
	   "&ntilde;angarekoh&aacute;ra"
#elif L==6
	   "amministratore"
#elif L==7
	   "administrator"
#elif L==8
	   "administrador"
#endif
	   ,
#if   L==0	// Rol_ROLE_CTR_ADMIN - Usr_SEX_ALL
	   "administrador/a"
#elif L==1
	   "Administrator"
#elif L==2
	   "administrator"
#elif L==3
	   "administrador/a"
#elif L==4
	   "administrateur/se"
#elif L==5
	   "&ntilde;angarekoh&aacute;ra"
#elif L==6
	   "amministratore"
#elif L==7
	   "administratorzy"
#elif L==8
	   "administrador/a"
#endif
	 },
	 {
#if   L==0	// Rol_ROLE_INS_ADMIN - Usr_SEX_UNKNOWN
	   "administrador/a"	// Necessita traduccio
#elif L==1
	   "Administrator"
#elif L==2
	   "administrator"
#elif L==3
	   "administrador/a"
#elif L==4
	   "administrateur/se"
#elif L==5
	   "&ntilde;angarekoh&aacute;ra"
#elif L==6
	   "amministratore"
#elif L==7
	   "administrator"
#elif L==8
	   "administrador/a"
#endif
	   ,
#if   L==0	// Rol_ROLE_INS_ADMIN - Usr_SEX_FEMALE
	   "administradora"	// Necessita traduccio
#elif L==1
	   "Administrator"
#elif L==2
	   "administrator"
#elif L==3
	   "administradora"
#elif L==4
	   "administrateuse"
#elif L==5
	   "&ntilde;angarekoh&aacute;ra"
#elif L==6
	   "amministratore"
#elif L==7
	   "administratorka"
#elif L==8
	   "administradora"
#endif
	   ,
#if   L==0	// Rol_ROLE_INS_ADMIN - Usr_SEX_MALE
	   "administrador"		// Necessita traduccio
#elif L==1
	   "Administrator"
#elif L==2
	   "administrator"
#elif L==3
	   "administrador"
#elif L==4
	   "administrateur"
#elif L==5
	   "&ntilde;angarekoh&aacute;ra"
#elif L==6
	   "amministratore"
#elif L==7
	   "administrator"
#elif L==8
	   "administrador"
#endif
	   ,
#if   L==0	// Rol_ROLE_INS_ADMIN - Usr_SEX_ALL
	   "administrador/a"
#elif L==1
	   "Administrator"
#elif L==2
	   "administrator"
#elif L==3
	   "administrador/a"
#elif L==4
	   "administrateur/se"
#elif L==5
	   "&ntilde;angarekoh&aacute;ra"
#elif L==6
	   "amministratore"
#elif L==7
	   "administratorzy"
#elif L==8
	   "administrador/a"
#endif
	 },
	 {
#if   L==0	// Rol_ROLE_SUPERUSER - Usr_SEX_UNKNOWN
	   "superusuari/&agrave;ria"
#elif L==1
	   "Superuser"
#elif L==2
	   "superuser"
#elif L==3
	   "superusuario/a"
#elif L==4
	   "superutilisateur/se"
#elif L==5
	   "superusuario/a"	// Okoteve traduccin
#elif L==6
	   "superutente"
#elif L==7
	   "superuzytkownik"
#elif L==8
	   "superutilizador/a"
#endif
	   ,
#if   L==0	// Rol_ROLE_SUPERUSER - Usr_SEX_FEMALE
	   "superusuaria"		// Necessita traduccio
#elif L==1
	   "Superuser"
#elif L==2
	   "superuser"
#elif L==3
	   "superusuaria"
#elif L==4
	   "superutilisateuse"
#elif L==5
	   "superusuaria"		// Okoteve traduccin
#elif L==6
	   "superutente"
#elif L==7
	   "superuzytkowniczka"
#elif L==8
	   "superutilizadora"
#endif
	   ,
#if   L==0	// Rol_ROLE_SUPERUSER - Usr_SEX_MALE
	   "superusuari"
#elif L==1
	   "Superuser"
#elif L==2
	   "superuser"
#elif L==3
	   "superusuario"
#elif L==4
	   "superutilisateur"
#elif L==5
	   "superusuario"	// Okoteve traduccin
#elif L==6
	   "superutente"
#elif L==7
	   "superu&zdot;ytkownika"
#elif L==8
	   "superutilizador"
#endif
	   ,
#if   L==0	// Rol_ROLE_SUPERUSER - Usr_SEX_ALL
	   "superusuari/&agrave;ria"
#elif L==1
	   "Superuser"
#elif L==2
	   "superuser"
#elif L==3
	   "superusuario/a"
#elif L==4
	   "superutilisateur/se"
#elif L==5
	   "superusuario/a"	// Okoteve traduccin
#elif L==6
	   "superutente"
#elif L==7
	   "superuzytkownicy"
#elif L==8
	   "superutilizador/a"
#endif
	 } };

const char *Txt_ROLES_PLURAL_Abc[Rol_NUM_ROLES][Usr_NUM_SEXS] =
   {
	{
#if   L==0	// Rol_ROLE_UNKNOWN - Usr_SEX_UNKNOWN
	"Desconocidos"	// Necessita traduccio
#elif L==1
	"Unbekannte"
#elif L==2
	"Unknown"
#elif L==3
	"Desconocidos"
#elif L==4
	"Inconnus"
#elif L==5
	"Desconocidos"	// Okoteve traduccin
#elif L==6
	"Sconosciuti"
#elif L==7
	"Nieznany"
#elif L==8
	"Desconhecidos"
#endif
	,
#if   L==0	// Rol_ROLE_UNKNOWN - Usr_SEX_FEMALE
	"Desconocidas"	// Necessita traduccio
#elif L==1
	"Unbekannte"
#elif L==2
	"Unknown"
#elif L==3
	"Desconocidas"
#elif L==4
	"Inconnues"
#elif L==5
	"Desconocidas"	// Okoteve traduccin
#elif L==6
	"Sconosciuti"
#elif L==7
	"Nieznana"
#elif L==8
	"Desconhecidas"
#endif
	,
#if   L==0	// Rol_ROLE_UNKNOWN - Usr_SEX_MALE
	"Desconocidos"	// Necessita traduccio
#elif L==1
	"Unbekannte"
#elif L==2
	"Unknown"
#elif L==3
	"Desconocidos"
#elif L==4
	"Inconnus"
#elif L==5
	"Desconocidos"	// Okoteve traduccin
#elif L==6
	"Sconosciuti"
#elif L==7
	"Nieznany"
#elif L==8
	"Desconhecidos"
#endif
	,
#if   L==0	// Rol_ROLE_UNKNOWN - Usr_SEX_ALL
	"Desconocidos"	// Necessita traduccio
#elif L==1
	"Unbekannte"
#elif L==2
	"Unknown"
#elif L==3
	"Desconocidos"
#elif L==4
	"Inconnus"
#elif L==5
	"Desconocidos"	// Okoteve traduccin
#elif L==6
	"Sconosciuti"
#elif L==7
	"Nieznani"
#elif L==8
	"Desconhecidos"
#endif
	 },
	 {
#if   L==0	// Rol_ROLE_GUEST - Usr_SEX_UNKNOWN
	   "Convidats"
#elif L==1
	   "Eingeladen"
#elif L==2
	   "Guests"
#elif L==3
	   "Invitados"
#elif L==4
	   "Invit&eacute;s"
#elif L==5
	   "Invitados"	// Okoteve traduccin
#elif L==6
	   "Ospiti"
#elif L==7
	   "Zaproszenie"
#elif L==8
	   "Convidados"
#endif
	   ,
#if   L==0	// Rol_ROLE_GUEST - Usr_SEX_FEMALE
	   "Invitadas"	// Necessita traduccio
#elif L==1
	   "Eingeladen"
#elif L==2
	   "Guests"
#elif L==3
	   "Invitadas"
#elif L==4
	   "Invit&eacute;es"
#elif L==5
	   "Invitadas"	// Okoteve traduccin
#elif L==6
	   "Ospiti"
#elif L==7
	   "Zaproszona"
#elif L==8
	   "Convidadas"
#endif
	   ,
#if   L==0	// Rol_ROLE_GUEST - Usr_SEX_MALE
	   "Convidats"
#elif L==1
	   "Eingeladen"
#elif L==2
	   "Guests"
#elif L==3
	   "Invitados"
#elif L==4
	   "Invit&eacute;s"
#elif L==5
	   "Invitados"	// Okoteve traduccin
#elif L==6
	   "Ospiti"
#elif L==7
	   "Zaproszony"
#elif L==8
	   "Convidados"
#endif
	   ,
#if   L==0	// Rol_ROLE_GUEST - Usr_SEX_ALL
	   "Convidats"
#elif L==1
	   "Eingeladen"
#elif L==2
	   "Guests"
#elif L==3
	   "Invitados"
#elif L==4
	   "Invit&eacute;s"
#elif L==5
	   "Invitados"	// Okoteve traduccin
#elif L==6
	   "Ospiti"
#elif L==7
	   "Zaproszeni"
#elif L==8
	   "Convidados"
#endif
	 },
	 {
#if   L==0	// Rol_ROLE_VISITOR - Usr_SEX_UNKNOWN
	   "Visitants"
#elif L==1
	   "Besucher"
#elif L==2
	   "Visitors"
#elif L==3
	   "Visitantes"
#elif L==4
	   "Visiteurs"
#elif L==5
	   "Visitantes"	// Okoteve traduccin
#elif L==6
	   "Visitatori"
#elif L==7
	   "Odwiedzaj&aogon;cy"
#elif L==8
	   "Visitantes"
#endif
	   ,
#if   L==0	// Rol_ROLE_VISITOR - Usr_SEX_FEMALE
	   "Visitants"
#elif L==1
	   "Besucher"
#elif L==2
	   "Visitors"
#elif L==3
	   "Visitantes"
#elif L==4
	   "Visiteurs"
#elif L==5
	   "Visitantes"	// Okoteve traduccin
#elif L==6
	   "Visitatori"
#elif L==7
	   "Odwiedzaj&aogon;cy"
#elif L==8
	   "Visitantes"
#endif
	   ,
#if   L==0	// Rol_ROLE_VISITOR - Usr_SEX_MALE
	   "Visitants"
#elif L==1
	   "Besucher"
#elif L==2
	   "Visitors"
#elif L==3
	   "Visitantes"
#elif L==4
	   "Visiteurs"
#elif L==5
	   "Visitantes"	// Okoteve traduccin
#elif L==6
	   "Visitatori"
#elif L==7
	   "Odwiedzaj&aogon;cy"
#elif L==8
	   "Visitantes"
#endif
	   ,
#if   L==0	// Rol_ROLE_VISITOR - Usr_SEX_ALL
	   "Visitants"
#elif L==1
	   "Besucher"
#elif L==2
	   "Visitors"
#elif L==3
	   "Visitantes"
#elif L==4
	   "Visiteurs"
#elif L==5
	   "Visitantes"	// Okoteve traduccin
#elif L==6
	   "Visitatori"
#elif L==7
	   "Odwiedzaj&aogon;cy"
#elif L==8
	   "Visitantes"
#endif
	 },
	 {
#if   L==0	// Rol_ROLE_STUDENT - Usr_SEX_UNKNOWN
	   "Estudiants"
#elif L==1
	   "Studenten"
#elif L==2
	   "Students"
#elif L==3
	   "Estudiantes"
#elif L==4
	   "&Eacute;tudiants"
#elif L==5
	   "Temimbo'e"
#elif L==6
	   "Studenti"
#elif L==7
	   "Studenci"
#elif L==8
	   "Estudantes"
#endif
	   ,
#if   L==0	// Rol_ROLE_STUDENT - Usr_SEX_FEMALE
	   "Estudiants"
#elif L==1
	   "Studenten"
#elif L==2
	   "Students"
#elif L==3
	   "Estudiantes"
#elif L==4
	   "&Eacute;tudiantes"
#elif L==5
	   "Temimbo'e"
#elif L==6
	   "Studentesse"
#elif L==7
	   "Studentki"
#elif L==8
	   "Estudantes"
#endif
	   ,
#if   L==0	// Rol_ROLE_STUDENT - Usr_SEX_MALE
	   "Estudiants"
#elif L==1
	   "Studenten"
#elif L==2
	   "Students"
#elif L==3
	   "Estudiantes"
#elif L==4
	   "&Eacute;tudiants"
#elif L==5
	   "Temimbo'e"
#elif L==6
	   "Studenti"
#elif L==7
	   "Studenci"
#elif L==8
	   "Estudantes"
#endif
	   ,
#if   L==0	// Rol_ROLE_STUDENT - Usr_SEX_ALL
	   "Estudiants"
#elif L==1
	   "Studenten"
#elif L==2
	   "Students"
#elif L==3
	   "Estudiantes"
#elif L==4
	   "&Eacute;tudiants"
#elif L==5
	   "Temimbo'e"
#elif L==6
	   "Studenti"
#elif L==7
	   "Studenci"
#elif L==8
	   "Estudantes"
#endif
	 },
	 {
#if   L==0	// Rol_ROLE_TEACHER - Usr_SEX_UNKNOWN
	   "Professors"
#elif L==1
	   "Lehrkraft"
#elif L==2
	   "Teachers"
#elif L==3
	   "Profesores"
#elif L==4
	   "Enseignants"
#elif L==5
	   "Mbo'eh&aacute;ra"
#elif L==6
	   "Professori"
#elif L==7
	   "Nauczyciele"
#elif L==8
	   "Professores"
#endif
	   ,
#if   L==0	// Rol_ROLE_TEACHER - Usr_SEX_FEMALE
	   "Professores"
#elif L==1
	   "Lehrkraft"
#elif L==2
	   "Teachers"
#elif L==3
	   "Profesoras"
#elif L==4
	   "Enseignantes"
#elif L==5
	   "Mbo'eh&aacute;ra"
#elif L==6
	   "Professoresse"
#elif L==7
	   "Nauczycielki"
#elif L==8
	   "Professoras"
#endif
	   ,
#if   L==0	// Rol_ROLE_TEACHER - Usr_SEX_MALE
	   "Professors"
#elif L==1
	   "Lehrkraft"
#elif L==2
	   "Teachers"
#elif L==3
	   "Profesores"
#elif L==4
	   "Enseignants"
#elif L==5
	   "Mbo'eh&aacute;ra"
#elif L==6
	   "Professori"
#elif L==7
	   "Nauczyciele"
#elif L==8
	   "Professores"
#endif
	   ,
#if   L==0	// Rol_ROLE_TEACHER - Usr_SEX_ALL
	   "Professors"
#elif L==1
	   "Lehrkraft"
#elif L==2
	   "Teachers"
#elif L==3
	   "Profesores"
#elif L==4
	   "Enseignants"
#elif L==5
	   "Mbo'eh&aacute;ra"
#elif L==6
	   "Professori"
#elif L==7
	   "Nauczyciele"
#elif L==8
	   "Professores"
#endif
	 },
	 {
#if   L==0	// Rol_ROLE_CTR_ADMIN - Usr_SEX_UNKNOWN
	   "Administradores"	// Necessita traduccio
#elif L==1
	   "Administratoren"
#elif L==2
	   "Administrators"
#elif L==3
	   "Administradores"
#elif L==4
	   "Administrateurs"
#elif L==5
	   "&Ntilde;angarekoh&aacute;ra"
#elif L==6
	   "Amministratori"
#elif L==7
	   "Administratorzy"
#elif L==8
	   "Administradores"
#endif
	   ,
#if   L==0	// Rol_ROLE_CTR_ADMIN - Usr_SEX_FEMALE
	   "Administradoras"	// Necessita traduccio
#elif L==1
	   "Administratoren"
#elif L==2
	   "Administrators"
#elif L==3
	   "Administradoras"
#elif L==4
	   "Administrateuses"
#elif L==5
	   "&Ntilde;angarekoh&aacute;ra"
#elif L==6
	   "Amministratori"
#elif L==7
	   "Administratorki"
#elif L==8
	   "Administradoras"
#endif
	   ,
#if   L==0	// Rol_ROLE_CTR_ADMIN - Usr_SEX_MALE
	   "Administradores"	// Necessita traduccio
#elif L==1
	   "Administratoren"
#elif L==2
	   "Administrators"
#elif L==3
	   "Administradores"
#elif L==4
	   "Administrateurs"
#elif L==5
	   "&Ntilde;angarekoh&aacute;ra"
#elif L==6
	   "Amministratori"
#elif L==7
	   "Administratorzy"
#elif L==8
	   "Administradores"
#endif
	   ,
#if   L==0	// Rol_ROLE_CTR_ADMIN - Usr_SEX_ALL
	   "Administradores"	// Necessita traduccio
#elif L==1
	   "Administratoren"
#elif L==2
	   "Administrators"
#elif L==3
	   "Administradores"
#elif L==4
	   "Administrateurs"
#elif L==5
	   "&Ntilde;angarekoh&aacute;ra"
#elif L==6
	   "Amministratori"
#elif L==7
	   "Administratorzy"
#elif L==8
	   "Administradores"
#endif
	 },
	 {
#if   L==0	// Rol_ROLE_INS_ADMIN - Usr_SEX_UNKNOWN
	   "Administradores"	// Necessita traduccio
#elif L==1
	   "Administratoren"
#elif L==2
	   "Administrators"
#elif L==3
	   "Administradores"
#elif L==4
	   "Administrateurs"
#elif L==5
	   "&Ntilde;angarekoh&aacute;ra"
#elif L==6
	   "Amministratori"
#elif L==7
	   "Administratorzy"
#elif L==8
	   "Administradores"
#endif
	   ,
#if   L==0	// Rol_ROLE_INS_ADMIN - Usr_SEX_FEMALE
	   "Administradoras"	// Necessita traduccio
#elif L==1
	   "Administratoren"
#elif L==2
	   "Administrators"
#elif L==3
	   "Administradoras"
#elif L==4
	   "Administrateuses"
#elif L==5
	   "&Ntilde;angarekoh&aacute;ra"
#elif L==6
	   "Amministratori"
#elif L==7
	   "Administratorki"
#elif L==8
	   "Administradoras"
#endif
	   ,
#if   L==0	// Rol_ROLE_INS_ADMIN - Usr_SEX_MALE
	   "Administradores"	// Necessita traduccio
#elif L==1
	   "Administratoren"
#elif L==2
	   "Administrators"
#elif L==3
	   "Administradores"
#elif L==4
	   "Administrateurs"
#elif L==5
	   "&Ntilde;angarekoh&aacute;ra"
#elif L==6
	   "Amministratori"
#elif L==7
	   "Administratorzy"
#elif L==8
	   "Administradores"
#endif
	   ,
#if   L==0	// Rol_ROLE_INS_ADMIN - Usr_SEX_ALL
	   "Administradores"	// Necessita traduccio
#elif L==1
	   "Administratoren"
#elif L==2
	   "Administrators"
#elif L==3
	   "Administradores"
#elif L==4
	   "Administrateurs"
#elif L==5
	   "&Ntilde;angarekoh&aacute;ra"
#elif L==6
	   "Amministratori"
#elif L==7
	   "Administratorzy"
#elif L==8
	   "Administradores"
#endif
	 },
	 {
#if   L==0	// Rol_ROLE_DEG_ADMIN - Usr_SEX_UNKNOWN
	   "Administradores"	// Necessita traduccio
#elif L==1
	   "Administratoren"
#elif L==2
	   "Administrators"
#elif L==3
	   "Administradores"
#elif L==4
	   "Administrateurs"
#elif L==5
	   "&Ntilde;angarekoh&aacute;ra"
#elif L==6
	   "Amministratori"
#elif L==7
	   "Administratorzy"
#elif L==8
	   "Administradores"
#endif
	   ,
#if   L==0	// Rol_ROLE_DEG_ADMIN - Usr_SEX_FEMALE
	   "Administradoras"	// Necessita traduccio
#elif L==1
	   "Administratoren"
#elif L==2
	   "Administrators"
#elif L==3
	   "Administradoras"
#elif L==4
	   "Administrateuses"
#elif L==5
	   "&Ntilde;angarekoh&aacute;ra"
#elif L==6
	   "Amministratori"
#elif L==7
	   "Administratorki"
#elif L==8
	   "Administradoras"
#endif
	   ,
#if   L==0	// Rol_ROLE_DEG_ADMIN - Usr_SEX_MALE
	   "Administradores"	// Necessita traduccio
#elif L==1
	   "Administratoren"
#elif L==2
	   "Administrators"
#elif L==3
	   "Administradores"
#elif L==4
	   "Administrateurs"
#elif L==5
	   "&Ntilde;angarekoh&aacute;ra"
#elif L==6
	   "Amministratori"
#elif L==7
	   "Administratorzy"
#elif L==8
	   "Administradores"
#endif
	   ,
#if   L==0	// Rol_ROLE_DEG_ADMIN - Usr_SEX_ALL
	   "Administradores"	// Necessita traduccio
#elif L==1
	   "Administratoren"
#elif L==2
	   "Administrators"
#elif L==3
	   "Administradores"
#elif L==4
	   "Administrateurs"
#elif L==5
	   "&Ntilde;angarekoh&aacute;ra"
#elif L==6
	   "Amministratori"
#elif L==7
	   "Administratorzy"
#elif L==8
	   "Administradores"
#endif
	 },
	 {
#if   L==0	// Rol_ROLE_SUPERUSER - Usr_SEX_UNKNOWN
	   "Superusuaris"
#elif L==1
	   "Superuser"
#elif L==2
	   "Superusers"
#elif L==3
	   "Superusuarios"
#elif L==4
	   "Superutilisateurs"
#elif L==5
	   "Superusuarios"	// Okoteve traduccin
#elif L==6
	   "Superutenti"
#elif L==7
	   "Superu&zdot;ytkownika"
#elif L==8
	   "Superutilizadores"
#endif
	   ,
#if   L==0	// Rol_ROLE_SUPERUSER - Usr_SEX_FEMALE
	   "Superusuarias"	// Necessita traduccio
#elif L==1
	   "Superuser"
#elif L==2
	   "Superusers"
#elif L==3
	   "Superusuarias"
#elif L==4
	   "Superutilisateuses"
#elif L==5
	   "Superusuarias"	// Okoteve traduccin
#elif L==6
	   "Superutenti"
#elif L==7
	   "Superuzytkowniczka"
#elif L==8
	   "Superutilizadoras"
#endif
	   ,
#if   L==0	// Rol_ROLE_SUPERUSER - Usr_SEX_MALE
	   "Superusuaris"
#elif L==1
	   "Superuser"
#elif L==2
	   "Superusers"
#elif L==3
	   "Superusuarios"
#elif L==4
	   "Superutilisateurs"
#elif L==5
	   "Superusuarios"	// Okoteve traduccin
#elif L==6
	   "Superutenti"
#elif L==7
	   "Superuzytkownik"
#elif L==8
	   "Superutilizadores"
#endif
	   ,
#if   L==0	// Rol_ROLE_SUPERUSER - Usr_SEX_ALL
	   "Superusuaris"
#elif L==1
	   "Superuser"
#elif L==2
	   "Superusers"
#elif L==3
	   "Superusuarios"
#elif L==4
	   "Superutilisateurs"
#elif L==5
	   "Superusuarios"	// Okoteve traduccin
#elif L==6
	   "Superutenti"
#elif L==7
	   "Superuzytkownicy"
#elif L==8
	   "Superutilizadores"
#endif
	 } };

const char *Txt_ROLES_PLURAL_abc[Rol_NUM_ROLES][Usr_NUM_SEXS] =
   {
	{
#if   L==0	// Rol_ROLE_UNKNOWN - Usr_SEX_UNKNOWN
	"desconocidos"	// Necessita traduccio
#elif L==1
	"Unbekannte"
#elif L==2
	"unknown"
#elif L==3
	"desconocidos"
#elif L==4
	"inconnus"
#elif L==5
	"desconocidos"	// Okoteve traduccin
#elif L==6
	"sconosciuti"
#elif L==7
	"nieznany"
#elif L==8
	"desconhecidos"
#endif
	,
#if   L==0	// Rol_ROLE_UNKNOWN - Usr_SEX_FEMALE
	"desconocidas"	// Necessita traduccio
#elif L==1
	"Unbekannte"
#elif L==2
	"unknown"
#elif L==3
	"desconocidas"
#elif L==4
	"inconnues"
#elif L==5
	"desconocidas"	// Okoteve traduccin
#elif L==6
	"sconosciuti"
#elif L==7
	"nieznana"
#elif L==8
	"desconhecidas"
#endif
	,
#if   L==0	// Rol_ROLE_UNKNOWN - Usr_SEX_MALE
	"desconocidos"	// Necessita traduccio
#elif L==1
	"Unbekannte"
#elif L==2
	"unknown"
#elif L==3
	"desconocidos"
#elif L==4
	"inconnus"
#elif L==5
	"desconocidos"	// Okoteve traduccin
#elif L==6
	"sconosciuti"
#elif L==7
	"nieznany"
#elif L==8
	"desconhecidos"
#endif
	,
#if   L==0	// Rol_ROLE_UNKNOWN - Usr_SEX_ALL
	"desconocidos"	// Necessita traduccio
#elif L==1
	"Unbekannte"
#elif L==2
	"unknown"
#elif L==3
	"desconocidos"
#elif L==4
	"inconnus"
#elif L==5
	"desconocidos"	// Okoteve traduccin
#elif L==6
	"sconosciuti"
#elif L==7
	"nieznani"
#elif L==8
	"desconhecidos"
#endif
	 },
	 {
#if   L==0	// Rol_ROLE_GUEST - Usr_SEX_UNKNOWN
	   "convidats"
#elif L==1
	   "Eingeladen"
#elif L==2
	   "guests"
#elif L==3
	   "invitados"
#elif L==4
	   "invit&eacute;s"
#elif L==5
	   "invitados"	// Okoteve traduccin
#elif L==6
	   "ospiti"
#elif L==7
	   "zaproszenie"
#elif L==8
	   "convidados"
#endif
	   ,
#if   L==0	// Rol_ROLE_GUEST - Usr_SEX_FEMALE
	   "invitadas"	// Necessita traduccio
#elif L==1
	   "Eingeladen"
#elif L==2
	   "guests"
#elif L==3
	   "invitadas"
#elif L==4
	   "invit&eacute;es"
#elif L==5
	   "invitadas"	// Okoteve traduccin
#elif L==6
	   "ospiti"
#elif L==7
	   "zaproszona"
#elif L==8
	   "convidadas"
#endif
	   ,
#if   L==0	// Rol_ROLE_GUEST - Usr_SEX_MALE
	   "convidats"
#elif L==1
	   "Eingeladen"
#elif L==2
	   "guests"
#elif L==3
	   "invitados"
#elif L==4
	   "invit&eacute;s"
#elif L==5
	   "invitados"	// Okoteve traduccin
#elif L==6
	   "ospiti"
#elif L==7
	   "zaproszony"
#elif L==8
	   "convidados"
#endif
	   ,
#if   L==0	// Rol_ROLE_GUEST - Usr_SEX_ALL
	   "convidats"
#elif L==1
	   "Eingeladen"
#elif L==2
	   "guests"
#elif L==3
	   "invitados"
#elif L==4
	   "invit&eacute;s"
#elif L==5
	   "invitados"	// Okoteve traduccin
#elif L==6
	   "ospiti"
#elif L==7
	   "zaproszeni"
#elif L==8
	   "convidados"
#endif
	 },
	 {
#if   L==0	// Rol_ROLE_VISITOR - Usr_SEX_UNKNOWN
	   "visitants"
#elif L==1
	   "Besucher"
#elif L==2
	   "visitors"
#elif L==3
	   "visitantes"
#elif L==4
	   "visiteurs"
#elif L==5
	   "visitantes"	// Okoteve traduccin
#elif L==6
	   "visitatori"
#elif L==7
	   "odwiedzaj&aogon;cy"
#elif L==8
	   "visitantes"
#endif
	   ,
#if   L==0	// Rol_ROLE_VISITOR - Usr_SEX_FEMALE
	   "visitants"
#elif L==1
	   "Besucher"
#elif L==2
	   "visitors"
#elif L==3
	   "visitantes"
#elif L==4
	   "visiteurs"
#elif L==5
	   "visitantes"	// Okoteve traduccin
#elif L==6
	   "visitatori"
#elif L==7
	   "odwiedzaj&aogon;cy"
#elif L==8
	   "visitantes"
#endif
	   ,
#if   L==0	// Rol_ROLE_VISITOR - Usr_SEX_MALE
	   "visitants"
#elif L==1
	   "Besucher"
#elif L==2
	   "visitors"
#elif L==3
	   "visitantes"
#elif L==4
	   "visiteurs"
#elif L==5
	   "visitantes"	// Okoteve traduccin
#elif L==6
	   "visitatori"
#elif L==7
	   "odwiedzaj&aogon;cy"
#elif L==8
	   "visitantes"
#endif
	   ,
#if   L==0	// Rol_ROLE_VISITOR - Usr_SEX_ALL
	   "visitants"
#elif L==1
	   "Besucher"
#elif L==2
	   "visitors"
#elif L==3
	   "visitantes"
#elif L==4
	   "visiteurs"
#elif L==5
	   "visitantes"	// Okoteve traduccin
#elif L==6
	   "visitatori"
#elif L==7
	   "odwiedzaj&aogon;cy"
#elif L==8
	   "visitantes"
#endif
	 },
	 {
#if   L==0	// Rol_ROLE_STUDENT - Usr_SEX_UNKNOWN
	   "estudiants"
#elif L==1
	   "Studenten"
#elif L==2
	   "students"
#elif L==3
	   "estudiantes"
#elif L==4
	   "&eacute;tudiants"
#elif L==5
	   "temimbo'e"
#elif L==6
	   "studenti"
#elif L==7
	   "studenci"
#elif L==8
	   "estudantes"
#endif
	   ,
#if   L==0	// Rol_ROLE_STUDENT - Usr_SEX_FEMALE
	   "estudiants"
#elif L==1
	   "Studenten"
#elif L==2
	   "students"
#elif L==3
	   "estudiantes"
#elif L==4
	   "&eacute;tudiantes"
#elif L==5
	   "temimbo'e"
#elif L==6
	   "studentesse"
#elif L==7
	   "studenka"
#elif L==8
	   "estudantes"
#endif
	   ,
#if   L==0	// Rol_ROLE_STUDENT - Usr_SEX_MALE
	   "estudiants"
#elif L==1
	   "Studenten"
#elif L==2
	   "students"
#elif L==3
	   "estudiantes"
#elif L==4
	   "&eacute;tudiants"
#elif L==5
	   "temimbo'e"
#elif L==6
	   "studenti"
#elif L==7
	   "studentci"
#elif L==8
	   "estudantes"
#endif
	   ,
#if   L==0	// Rol_ROLE_STUDENT - Usr_SEX_ALL
	   "estudiants"
#elif L==1
	   "Studenten"
#elif L==2
	   "students"
#elif L==3
	   "estudiantes"
#elif L==4
	   "&eacute;tudiants"
#elif L==5
	   "temimbo'e"
#elif L==6
	   "studenti"
#elif L==7
	   "studentci"
#elif L==8
	   "estudantes"
#endif
	 },
	 {
#if   L==0	// Rol_ROLE_TEACHER - Usr_SEX_UNKNOWN
	   "professors"
#elif L==1
	   "Lehrkr&auml;fte"
#elif L==2
	   "teachers"
#elif L==3
	   "profesores"
#elif L==4
	   "enseignants"
#elif L==5
	   "mbo'eh&aacute;ra"
#elif L==6
	   "professori"
#elif L==7
	   "nauczyciele"
#elif L==8
	   "professores"
#endif
	   ,
#if   L==0	// Rol_ROLE_TEACHER - Usr_SEX_FEMALE
	   "professores"
#elif L==1
	   "Lehrkr&auml;fte"
#elif L==2
	   "teachers"
#elif L==3
	   "profesoras"
#elif L==4
	   "enseignantes"
#elif L==5
	   "mbo'eh&aacute;ra"
#elif L==6
	   "professoresse"
#elif L==7
	   "nauczycielki"
#elif L==8
	   "professoras"
#endif
	   ,
#if   L==0	// Rol_ROLE_TEACHER - Usr_SEX_MALE
	   "professors"
#elif L==1
	   "Lehrkr&auml;fte"
#elif L==2
	   "teachers"
#elif L==3
	   "profesores"
#elif L==4
	   "enseignants"
#elif L==5
	   "mbo'eh&aacute;ra"
#elif L==6
	   "professori"
#elif L==7
	   "nauczyciele"
#elif L==8
	   "professores"
#endif
	   ,
#if   L==0	// Rol_ROLE_TEACHER - Usr_SEX_ALL
	   "professors"
#elif L==1
	   "Lehrkr&auml;fte"
#elif L==2
	   "teachers"
#elif L==3
	   "profesores"
#elif L==4
	   "enseignants"
#elif L==5
	   "mbo'eh&aacute;ra"
#elif L==6
	   "professori"
#elif L==7
	   "nauczyciele"
#elif L==8
	   "professores"
#endif
	 },
	 {
#if   L==0	// Rol_ROLE_DEG_ADMIN - Usr_SEX_UNKNOWN
	   "administradores"	// Necessita traduccio
#elif L==1
	   "Administratoren"
#elif L==2
	   "administrators"
#elif L==3
	   "administradores"
#elif L==4
	   "administrateurs"
#elif L==5
	   "&ntilde;angarekoh&aacute;ra"
#elif L==6
	   "amministratori"
#elif L==7
	   "administratorzy"
#elif L==8
	   "administradores"
#endif
	   ,
#if   L==0	// Rol_ROLE_DEG_ADMIN - Usr_SEX_FEMALE
	   "administradoras"	// Necessita traduccio
#elif L==1
	   "Administratoren"
#elif L==2
	   "administrators"
#elif L==3
	   "administradoras"
#elif L==4
	   "administrateuses"
#elif L==5
	   "&ntilde;angarekoh&aacute;ra"
#elif L==6
	   "amministratori"
#elif L==7
	   "administratorki"
#elif L==8
	   "administradoras"
#endif
	   ,
#if   L==0	// Rol_ROLE_DEG_ADMIN - Usr_SEX_MALE
	   "administradores"	// Necessita traduccio
#elif L==1
	   "Administratoren"
#elif L==2
	   "administrators"
#elif L==3
	   "administradores"
#elif L==4
	   "administrateurs"
#elif L==5
	   "&ntilde;angarekoh&aacute;ra"
#elif L==6
	   "amministratori"
#elif L==7
	   "administratorzy"
#elif L==8
	   "administradores"
#endif
	   ,
#if   L==0	// Rol_ROLE_DEG_ADMIN - Usr_SEX_ALL
	   "administradores"	// Necessita traduccio
#elif L==1
	   "Administratoren"
#elif L==2
	   "administrators"
#elif L==3
	   "administradores"
#elif L==4
	   "administrateurs"
#elif L==5
	   "&ntilde;angarekoh&aacute;ra"
#elif L==6
	   "amministratori"
#elif L==7
	   "administratorzy"
#elif L==8
	   "administradores"
#endif
	 },
	 {
#if   L==0	// Rol_ROLE_CTR_ADMIN - Usr_SEX_UNKNOWN
	   "administradores"	// Necessita traduccio
#elif L==1
	   "Administratoren"
#elif L==2
	   "administrators"
#elif L==3
	   "administradores"
#elif L==4
	   "administrateurs"
#elif L==5
	   "&ntilde;angarekoh&aacute;ra"
#elif L==6
	   "amministratori"
#elif L==7
	   "administratorzy"
#elif L==8
	   "administradores"
#endif
	   ,
#if   L==0	// Rol_ROLE_CTR_ADMIN - Usr_SEX_FEMALE
	   "administradoras"	// Necessita traduccio
#elif L==1
	   "Administratoren"
#elif L==2
	   "administrators"
#elif L==3
	   "administradoras"
#elif L==4
	   "administrateuses"
#elif L==5
	   "&ntilde;angarekoh&aacute;ra"
#elif L==6
	   "amministratori"
#elif L==7
	   "administratorki"
#elif L==8
	   "administradoras"
#endif
	   ,
#if   L==0	// Rol_ROLE_CTR_ADMIN - Usr_SEX_MALE
	   "administradores"	// Necessita traduccio
#elif L==1
	   "Administratoren"
#elif L==2
	   "administrators"
#elif L==3
	   "administradores"
#elif L==4
	   "administrateurs"
#elif L==5
	   "&ntilde;angarekoh&aacute;ra"
#elif L==6
	   "amministratori"
#elif L==7
	   "administratorzy"
#elif L==8
	   "administradores"
#endif
	   ,
#if   L==0	// Rol_ROLE_CTR_ADMIN - Usr_SEX_ALL
	   "administradores"	// Necessita traduccio
#elif L==1
	   "Administratoren"
#elif L==2
	   "administrators"
#elif L==3
	   "administradores"
#elif L==4
	   "administrateurs"
#elif L==5
	   "&ntilde;angarekoh&aacute;ra"
#elif L==6
	   "amministratori"
#elif L==7
	   "administratorzy"
#elif L==8
	   "administradores"
#endif
	 },
	 {
#if   L==0	// Rol_ROLE_INS_ADMIN - Usr_SEX_UNKNOWN
	   "administradores"	// Necessita traduccio
#elif L==1
	   "Administratoren"
#elif L==2
	   "administrators"
#elif L==3
	   "administradores"
#elif L==4
	   "administrateurs"
#elif L==5
	   "&ntilde;angarekoh&aacute;ra"
#elif L==6
	   "amministratori"
#elif L==7
	   "administratorzy"
#elif L==8
	   "administradores"
#endif
	   ,
#if   L==0	// Rol_ROLE_INS_ADMIN - Usr_SEX_FEMALE
	   "administradoras"	// Necessita traduccio
#elif L==1
	   "Administratoren"
#elif L==2
	   "administrators"
#elif L==3
	   "administradoras"
#elif L==4
	   "administrateuses"
#elif L==5
	   "&ntilde;angarekoh&aacute;ra"
#elif L==6
	   "amministratori"
#elif L==7
	   "administratorki"
#elif L==8
	   "administradoras"
#endif
	   ,
#if   L==0	// Rol_ROLE_INS_ADMIN - Usr_SEX_MALE
	   "administradores"	// Necessita traduccio
#elif L==1
	   "Administratoren"
#elif L==2
	   "administrators"
#elif L==3
	   "administradores"
#elif L==4
	   "administrateurs"
#elif L==5
	   "&ntilde;angarekoh&aacute;ra"
#elif L==6
	   "amministratori"
#elif L==7
	   "administratorzy"
#elif L==8
	   "administradores"
#endif
	   ,
#if   L==0	// Rol_ROLE_INS_ADMIN - Usr_SEX_ALL
	   "administradores"	// Necessita traduccio
#elif L==1
	   "Administratoren"
#elif L==2
	   "administrators"
#elif L==3
	   "administradores"
#elif L==4
	   "administrateurs"
#elif L==5
	   "&ntilde;angarekoh&aacute;ra"
#elif L==6
	   "amministratori"
#elif L==7
	   "administratorzy"
#elif L==8
	   "administradores"
#endif
	 },
	 {
#if   L==0	// Rol_ROLE_SUPERUSER - Usr_SEX_UNKNOWN
	   "superusuaris"
#elif L==1
	   "Superuser"
#elif L==2
	   "superusers"
#elif L==3
	   "superusuarios"
#elif L==4
	   "superutilisateurs"
#elif L==5
	   "superusuarios"		// Okoteve traduccin
#elif L==6
	   "superutenti"
#elif L==7
	   "superusers"			// Potrzebujesz tlumaczenie
#elif L==8
	   "superutilizadores"
#endif
	   ,
#if   L==0	// Rol_ROLE_SUPERUSER - Usr_SEX_FEMALE
	   "superusuarias"		// Necessita traduccio
#elif L==1
	   "Superuser"
#elif L==2
	   "superusers"
#elif L==3
	   "superusuarias"
#elif L==4
	   "superutilisateuses"
#elif L==5
	   "superusuarias"		// Okoteve traduccin
#elif L==6
	   "superutenti"
#elif L==7
	   "superusers"			// Potrzebujesz tlumaczenie
#elif L==8
	   "superutilizadoras"
#endif
	   ,
#if   L==0	// Rol_ROLE_SUPERUSER - Usr_SEX_MALE
	   "superusuaris"
#elif L==1
	   "Superuser"
#elif L==2
	   "superusers"
#elif L==3
	   "superusuarios"
#elif L==4
	   "superutilisateurs"
#elif L==5
	   "superusuarios"		// Okoteve traduccin
#elif L==6
	   "superutenti"
#elif L==7
	   "superusers"			// Potrzebujesz tlumaczenie
#elif L==8
	   "superutilizadores"
#endif
	   ,
#if   L==0	// Rol_ROLE_SUPERUSER - Usr_SEX_ALL
	   "superusuaris"
#elif L==1
	   "Superuser"
#elif L==2
	   "superusers"
#elif L==3
	   "superusuarios"
#elif L==4
	   "superutilisateurs"
#elif L==5
	   "superusuarios"		// Okoteve traduccin
#elif L==6
	   "superutenti"
#elif L==7
	   "superusers"			// Potrzebujesz tlumaczenie
#elif L==8
	   "superutilizadores"
#endif
	 } };

const char *Txt_Rooms_with_users =
#if   L==0
	"Salas con usuarios";		// Necessita traduccio
#elif L==1
	"Rooms with users";		// Need bersetzung
#elif L==2
	"Rooms with users";
#elif L==3
	"Salas con usuarios";
#elif L==4
	"Rooms with users";		// Besoin de traduction
#elif L==5
	"Salas con usuarios";		// Okoteve traduccin
#elif L==6
	"Rooms con utenti";
#elif L==7
	"Rooms with users";		// Potrzebujesz tlumaczenie
#elif L==8
	"Rooms with users";		// Necessita de traduo
#endif

const char *Txt_ROOT_FOLDER_EXTERNAL_NAMES[Brw_NUM_TYPES_FILE_BROWSER] =
   {
   // Brw_FILE_BRW_UNKNOWN
	 "",
	 // Brw_FILE_BRW_SEE_DOCUMENTS_CRS
#if   L==0
	 "documents"
#elif L==1
	 "Dokumente"
#elif L==2
	 "documents"
#elif L==3
	 "documentos"
#elif L==4
	 "documents"
#elif L==5
	 "kuatia"
#elif L==6
	 "documenti"
#elif L==7
	 "documents"		// Potrzebujesz tlumaczenie
#elif L==8
	 "documentos"
#endif
	 ,
	 // Brw_FILE_BRW_SEE_MARKS_CRS
#if   L==0
	 "qualificacions"
#elif L==1
	 "Bewertungen"
#elif L==2
	 "marks"
#elif L==3
	 "calificaciones"
#elif L==4
	 "notes"
#elif L==5
	 "calificaciones"	// Okoteve traduccin
#elif L==6
	 "risultati"
#elif L==7
	 "marks"		// Potrzebujesz tlumaczenie
#elif L==8
	 "notas"
#endif
	 ,
	 // Brw_FILE_BRW_ADMIN_DOCUMENTS_CRS
#if   L==0
	 "documents"
#elif L==1
	 "Dokumente"
#elif L==2
	 "documents"
#elif L==3
	 "documentos"
#elif L==4
	 "documents"
#elif L==5
	 "kuatia"
#elif L==6
	 "documenti"
#elif L==7
	 "documents"		// Potrzebujesz tlumaczenie
#elif L==8
	 "documentos"
#endif
	 ,
	 // Brw_FILE_BRW_COMMON_CRS
#if   L==0
	 "compartits"
#elif L==1
	 "Freigegebene"
#elif L==2
	 "shared"
#elif L==3
	 "compartidos"
#elif L==4
	 "partages"
#elif L==5
	 "compartidos"			// Okoteve traduccin
#elif L==6
	 "condivisi"
#elif L==7
	 "shared"			// Potrzebujesz tlumaczenie
#elif L==8
	 "comum"
#endif
	 ,
	 // Brw_FILE_BRW_COMMON_GRP
#if   L==0
	 "compartits"
#elif L==1
	 "Freigegebene"
#elif L==2
	 "shared"
#elif L==3
	 "compartidos"
#elif L==4
	 "partages"
#elif L==5
	 "compartidos"			// Okoteve traduccin
#elif L==6
	 "condivisi"
#elif L==7
	 "shared"			// Potrzebujesz tlumaczenie
#elif L==8
	 "comum"
#endif
	 ,
	 // Brw_FILE_BRW_WORKS_USR
#if   L==0
	 "treballs"
#elif L==1
	 "Hausarbeiten"
#elif L==2
	 "homework"
#elif L==3
	 "trabajos"
#elif L==4
	 "travaux"
#elif L==5
	 "tembiapo"
#elif L==6
	 "lavori"
#elif L==7
	 "homework"		// Potrzebujesz tlumaczenie
#elif L==8
	 "trabalhos"
#endif
	 ,
	 // Brw_FILE_BRW_WORKS_CRS
#if   L==0
	 "treballs"
#elif L==1
	 "Hausarbeiten"
#elif L==2
	 "homework"
#elif L==3
	 "trabajos"
#elif L==4
	 "travaux"
#elif L==5
	 "tembiapo"
#elif L==6
	 "lavori"
#elif L==7
	 "homework"		// Potrzebujesz tlumaczenie
#elif L==8
	 "trabalhos"
#endif
	 ,
	 // Brw_FILE_BRW_ADMIN_MARKS_CRS
#if   L==0
	 "qualificacions"
#elif L==1
	 "Bewertungen"
#elif L==2
	 "marks"
#elif L==3
	 "calificaciones"
#elif L==4
	 "notes"
#elif L==5
	 "calificaciones"	// Okoteve traduccin
#elif L==6
	 "risultati"
#elif L==7
	 "marks"		// Potrzebujesz tlumaczenie
#elif L==8
	 "notas"
#endif
	 ,
	 // Brw_FILE_BRW_BRIEFCASE_USR
#if   L==0
	 "maleti"
#elif L==1
	 "Aktentasche"
#elif L==2
	 "briefcase"
#elif L==3
	 "maletin"
#elif L==4
	 "porte-documents"
#elif L==5
	 "maletin"		// Okoteve traduccin
#elif L==6
	 "cartella"
#elif L==7
	 "briefcase"		// Potrzebujesz tlumaczenie
#elif L==8
	 "pasta"
#endif
	 ,
	 // Brw_FILE_BRW_SEE_DOCUMENTS_GRP
#if   L==0
	 "documents"
#elif L==1
	 "Dokumente"
#elif L==2
	 "documents"
#elif L==3
	 "documentos"
#elif L==4
	 "documents"
#elif L==5
	 "kuatia"
#elif L==6
	 "documenti"
#elif L==7
	 "documents"		// Potrzebujesz tlumaczenie
#elif L==8
	 "documentos"
#endif
	 ,
	 // Brw_FILE_BRW_ADMIN_DOCUMENTS_GRP
#if   L==0
	 "documents"
#elif L==1
	 "Dokumente"
#elif L==2
	 "documents"
#elif L==3
	 "documentos"
#elif L==4
	 "documents"
#elif L==5
	 "kuatia"
#elif L==6
	 "documenti"
#elif L==7
	 "documents"		// Potrzebujesz tlumaczenie
#elif L==8
	 "documentos"
#endif
	 ,
	 // Brw_FILE_BRW_SEE_MARKS_GRP
#if   L==0
	 "qualificacions"	// Necessita traduccio
#elif L==1
	 "Bewertungen"
#elif L==2
	 "marks"
#elif L==3
	 "calificaciones"
#elif L==4
	 "notes"
#elif L==5
	 "calificaciones"	// Okoteve traduccin
#elif L==6
	 "risultati"
#elif L==7
	 "marks"		// Potrzebujesz tlumaczenie
#elif L==8
	 "notas"
#endif
	 ,
	 // Brw_FILE_BRW_ADMIN_MARKS_GRP
#if   L==0
	 "qualificacions"
#elif L==1
	 "Bewertungen"
#elif L==2
	 "marks"
#elif L==3
	 "calificaciones"
#elif L==4
	 "notes"
#elif L==5
	 "calificaciones"	// Okoteve traduccin
#elif L==6
	 "risultati"
#elif L==7
	 "marks"		// Potrzebujesz tlumaczenie
#elif L==8
	 "notas"
#endif
	 ,
	 // Brw_FILE_BRW_ASSIGNMENTS_USR
#if   L==0
	 "activitats"
#elif L==1
	 "Aufgaben"
#elif L==2
	 "assignments"
#elif L==3
	 "actividades"
#elif L==4
	 "activites"
#elif L==5
	 "tar&eacute;a"
#elif L==6
	 "attivit&agrave;"
#elif L==7
	 "assignments"		// Potrzebujesz tlumaczenie
#elif L==8
	 "atividades"
#endif
	 ,
	 // Brw_FILE_BRW_ASSIGNMENTS_CRS
#if   L==0
	 "activitats"
#elif L==1
	 "Aufgaben"
#elif L==2
	 "assignments"
#elif L==3
	 "actividades"
#elif L==4
	 "activites"
#elif L==5
	 "tar&eacute;a"
#elif L==6
	 "attivit&agrave;"
#elif L==7
	 "assignments"		// Potrzebujesz tlumaczenie
#elif L==8
	 "atividades"
#endif
	};

const char *Txt_Save =
#if   L==0
	"Guardar";
#elif L==1
	"Speichern";
#elif L==2
	"Save";
#elif L==3
	"Guardar";
#elif L==4
	"Enregistrer";
#elif L==5
	"&Ntilde;ongatu";
#elif L==6
	"Salvare";
#elif L==7
	"Zapisa&cacute;";
#elif L==8
	"Salvar";
#endif

const char *Txt_Save_changes =
#if   L==0
	"Desar canvis";
#elif L==1
	"&Auml;nderungen speichern";
#elif L==2
	"Save changes";
#elif L==3
	"Guardar cambios";
#elif L==4
	"Enregistrer les modifications";
#elif L==5
	"&Ntilde;ongatu";
#elif L==6
	"Salva modifiche";
#elif L==7
	"Zapisz zmiany";
#elif L==8
	"Salvar altera&ccedil;&otilde;es";
#endif

const char *Txt_Save_file_properties =
#if   L==0
	"Guardar les propietats del fitxer";
#elif L==1
	"Speichern Dateieigenschaften";
#elif L==2
	"Save file properties";
#elif L==3
	"Guardar propiedades del archivo";
#elif L==4
	"Enregistrer les propri&eacute;t&eacute;s du fichier";
#elif L==5
	"Guardar propiedades del archivo";
#elif L==6
	"Salva le propriet&agrave; dei file";
#elif L==7
	"Zapisz w&lstrok;a&sacute;ciwo&sacute;ci pliku";
#elif L==8
	"Salvar as propriedades do arquivo";
#endif

const char *Txt_Scope =
#if   L==0
	"&Agrave;mbit";
#elif L==1
	"Geltungsbereich";
#elif L==2
	"Scope";
#elif L==3
	"&Aacute;mbito";
#elif L==4
	"Port&eacute;e";
#elif L==5
	"&Aacute;mbito";	// Okoteve traduccin
#elif L==6
	"Ambito";
#elif L==7
	"Scope";		// Potrzebujesz tlumaczenie
#elif L==8
	"&Acirc;mbito";
#endif

const char *Txt_Score =
#if   L==0
	"Puntuaci&oacute;n";	// Necessita traduccio
#elif L==1
	"Ergebnis";
#elif L==2
	"Score";
#elif L==3
	"Puntuaci&oacute;n";
#elif L==4
	"Note";
#elif L==5
	"Puntuaci&oacute;n";	// Okoteve traduccin
#elif L==6
	"Punteggio";
#elif L==7
	"Score";		// Potrzebujesz tlumaczenie
#elif L==8
	"Pontua&ccedil;&atilde;o";
#endif

const char *Txt_Search =
#if   L==0
	"Cercar";
#elif L==1
	"Suchen";
#elif L==2
	"Search";
#elif L==3
	"Buscar";
#elif L==4
	"Chercher";
#elif L==5
	"Heka";
#elif L==6
	"Cercare";
#elif L==7
	"Szukaj";
#elif L==8
	"Procurar";
#endif

const char *Txt_Search_courses =
#if   L==0
	"Cercar assignatures";
#elif L==1
	"Suchen Kursen";
#elif L==2
	"Search courses";
#elif L==3
	"Buscar asignaturas";
#elif L==4
	"Chercher mati&egrave;res";
#elif L==5
	"Heka mbo'esyry";
#elif L==6
	"Cercare corsi";
#elif L==7
	"Szukaj kurs&oacute;w";
#elif L==8
	"Procurar disciplinas";
#endif

const char *Txt_See_exam =
#if   L==0
	"Veure ex&agrave;men";
#elif L==1
	"Siehe Pr&uuml;fung";
#elif L==2
	"See exam";
#elif L==3
	"Ver examen";
#elif L==4
	"Voir examen";
#elif L==5
	"Ver examen";	// Okoteve traduccin
#elif L==6
	"Vedi esame";
#elif L==7
	"Wiadomo&sacute;ci egzamin";
#elif L==8
	"Veja exame";
#endif

const char *Txt_See_exams =
#if   L==0
	"Veure ex&agrave;mens";
#elif L==1
	"Siehe Pr&uuml;fungen";
#elif L==2
	"See exams";
#elif L==3
	"Ver ex&aacute;menes";
#elif L==4
	"Voir examens";
#elif L==5
	"Ver ex&aacute;menes";	// Okoteve traduccin
#elif L==6
	"Vedi esami";
#elif L==7
	"Wiadomo&sacute;ci egzaminy";
#elif L==8
	"Veja exames";
#endif

const char *Txt_See_full_notice =
#if   L==0
	"Veure avs complet";
#elif L==1
	"Siehe voll Ank&uuml;ndigung";
#elif L==2
	"See full notice";
#elif L==3
	"Ver aviso completo";
#elif L==4
	"Voir l'avis complet";
#elif L==5
	"Ver aviso completo";	// Okoteve traduccin
#elif L==6
	"Vedi avviso completo ";
#elif L==7
	"Wiadomo&sacute;ci ca&lstrok;y obwieszczenie";
#elif L==8
	"Veja aviso completo";
#endif

const char *Txt_See_message =
#if   L==0
	"Veure missatge";
#elif L==1
	"Siehe Nachricht";
#elif L==2
	"See message";
#elif L==3
	"Ver mensaje";
#elif L==4
	"Voir message";
#elif L==5
	"Ver mensaje";	// Okoteve traduccin
#elif L==6
	"Vedi messaggio";
#elif L==7
	"See message";	// Potrzebujesz tlumaczenie
#elif L==8
	"Veja mensagem";
#endif

const char *Txt_See_messages_received =
#if   L==0
	"Veure missatges rebuts";
#elif L==1
	"Siehe Empfangene Nachrichten";
#elif L==2
	"See messages received";
#elif L==3
	"Ver mensajes recibidos";
#elif L==4
	"Voir messages re&ccedil;us";
#elif L==5
	"Ver mensajes recibidos";	// Okoteve traduccin
#elif L==6
	"Vedi messaggi ricevuti";
#elif L==7
	"Wiadomo&sacute;ci odebrane";
#elif L==8
	"Veja mensagens recebidas";
#endif

const char *Txt_See_notifications =
#if   L==0
	"Ver notificaciones";	// Necessita traduccio
#elif L==1
	"Siehe Anmeldungen";
#elif L==2
	"See notifications";
#elif L==3
	"Ver notificaciones";
#elif L==4
	"Voir notifications";
#elif L==5
	"Ver notificaciones";	// Okoteve traduccin
#elif L==6
	"Vedi notifiche";
#elif L==7
	"See notifications";	// Potrzebujesz tlumaczenie
#elif L==8
	"Veja notifica&ccedil;&otilde;es";
#endif

const char *Txt_See_page_X_of_Y =	// Warning: it is very important to include two %u in the following sentences
#if   L==0
	"Ver p&aacute;gina %u de %u";	// Necessita traduccio
#elif L==1
	"Siehe Seite %u von %u";
#elif L==2
	"See page %u of %u";
#elif L==3
	"Ver p&aacute;gina %u de %u";
#elif L==4
	"Voir page %u de %u";
#elif L==5
	"Ver p&aacute;gina %u de %u";	// Okoteve traduccin
#elif L==6
	"Vedi pagina %u di %u";
#elif L==7
	"See page %u of %u";		// Potrzebujesz tlumaczenie
#elif L==8
	"Veja p&aacute;gina %u de %u";
#endif

const char *Txt_Select_one_or_more_files_from_your_computer_or_drag_and_drop_here =
#if   L==0
	"Seleccioneu un o m&eacute;s fitxers del seu ordinador<br />"
	"o arrossegueu i deixeu-los anar aqu&iacute;";
#elif L==1
	"W&auml;hlen Sie eine oder mehrere Dateien von Ihrem Computer<br />"
	"oder per Drag & Drop hier";
#elif L==2
	"Select one or more files from your computer<br />"
	"or drag and drop here";
#elif L==3
	"Seleccione uno o m&aacute;s archivos de su ordenador<br />"
	"o arr&aacute;strelos y su&eacute;ltelos aqu&iacute;";
#elif L==4
	"S&eacute;lectionnez un ou plusieurs fichiers &agrave; partir de votre ordinateur<br />"
	"ou glisser-d&eacute;poser ici";
#elif L==5
	"Seleccione uno o m&aacute;s archivos de su ordenador<br />"
	"o arr&aacute;strelos y su&eacute;ltelos aqu&iacute;";	// Okoteve traduccin
#elif L==6
	"Selezionare uno o pi&ugrave; file dal tuo computer<br />"
	"o trascinare e rilasciare qui";
#elif L==7
	"Wybierz jeden lub wi&eogon;cej plik&oacute;w z komputera<br />"
	"lub przeci&aogon;gnij i upu&sacute;&cacute; tutaj";
#elif L==8
	"Selecione um ou mais arquivos de seu computador<br />"
	"ou arrastar e soltar aqui";
#endif

const char *Txt_Select_the_groups_in_from_which_you_want_to_register_remove_users_ =
#if   L==0
	"Seleccione los grupos en/de los que quiere inscribir/eliminar usuarios."
	" S&oacute;lo puede seleccionar aquellos grupos a los que usted pertenezca.<br />"
	"Si selecciona grupos, los usuarios ser&aacute;n inscritos en la asignatura y en los grupos seleccionados,"
	" o bien eliminados de los grupos seleccionados pero no de la asignatura.<br />"
	"Si no selecciona ning&uacute;n grupo, los usuarios ser&aacute;n inscritos en la asignatura pero no en grupos concretos,"
	" o bien eliminados de la asignatura y de todos los grupos.";	// Necessita traduccio
#elif L==1
	"Select the groups in/from which you want to register/remove users."
	"You can only select those groups to which you belong.<br />"
	"If you select any groups, users will be enrolled in the course and the selected groups,"
	" or removed from the selected groups but not from the course.<br />"
	"If you do not select any groups, users will be enrolled in the course but not in specific groups,"
	" or removed from the course and from all groups.";	// Need bersetzung
#elif L==2
	"Select the groups in/from which you want to register/remove users."
	"You can only select those groups to which you belong.<br />"
	"If you select any groups, users will be enrolled in the course and the selected groups,"
	" or removed from the selected groups but not from the course.<br />"
	"If you do not select any groups, users will be enrolled in the course but not in specific groups,"
	" or removed from the course and from all groups.";
#elif L==3
	"Seleccione los grupos en/de los que quiere inscribir/eliminar usuarios."
	" S&oacute;lo puede seleccionar aquellos grupos a los que usted pertenezca.<br />"
	"Si selecciona grupos, los usuarios ser&aacute;n inscritos en la asignatura y en los grupos seleccionados,"
	" o bien eliminados de los grupos seleccionados pero no de la asignatura.<br />"
	"Si no selecciona ning&uacute;n grupo, los usuarios ser&aacute;n inscritos en la asignatura pero no en grupos concretos,"
	" o bien eliminados de la asignatura y de todos los grupos.";
#elif L==4
	"Select the groups in/from which you want to register/remove users."
	"You can only select those groups to which you belong.<br />"
	"If you select any groups, users will be enrolled in the course and the selected groups,"
	" or removed from the selected groups but not from the course.<br />"
	"If you do not select any groups, users will be enrolled in the course but not in specific groups,"
	" or removed from the course and from all groups.";	// Besoin de traduction
#elif L==5
	"Seleccione los grupos en/de los que quiere inscribir/eliminar usuarios."
	" S&oacute;lo puede seleccionar aquellos grupos a los que usted pertenezca.<br />"
	"Si selecciona grupos, los usuarios ser&aacute;n inscritos en la asignatura y en los grupos seleccionados,"
	" o bien eliminados de los grupos seleccionados pero no de la asignatura.<br />"
	"Si no selecciona ning&uacute;n grupo, los usuarios ser&aacute;n inscritos en la asignatura pero no en grupos concretos,"
	" o bien eliminados de la asignatura y de todos los grupos.";	// Okoteve traduccin
#elif L==6
	"Seleziona i gruppi nel/dal quale vuoi registrare/rimuovere utenti."
	"Puoi solo selezionare quei gruppi ai quali appartieni.<br />"
	"Se selezioni qualche gruppo, gli utenti saranno registrati nel corso e nel gruppo selezionato,"
	" o rimossi dai gruppi selezionati ma non dal corso.<br />"
	"Se non selezioni alcun gruppo, gli utenti saranno registrati nel corso ma non in gruppi specifici,"
	" o rimossi dal corso e da tutti i gruppi.";
#elif L==7
	"Select the groups in/from which you want to register/remove users."
	"You can only select those groups to which you belong.<br />"
	"If you select any groups, users will be enrolled in the course and the selected groups,"
	" or removed from the selected groups but not from the course.<br />"
	"If you do not select any groups, users will be enrolled in the course but not in specific groups,"
	" or removed from the course and from all groups.";	// Potrzebujesz tlumaczenie
#elif L==8
	"Select the groups in/from which you want to register/remove users."
	"You can only select those groups to which you belong.<br />"
	"If you select any groups, users will be enrolled in the course and the selected groups,"
	" or removed from the selected groups but not from the course.<br />"
	"If you do not select any groups, users will be enrolled in the course but not in specific groups,"
	" or removed from the course and from all groups.";	// Necessita de traduo
#endif

const char *Txt_Select_the_groups_of_students_you_want_to_register_in_remove_from_this_course =
#if   L==0
	"Seleccione los grupos de estudiantes que desee inscribir en / eliminar de esta asignatura.";	// Necessita traduccio
#elif L==1
	"Select the groups of students you want to register in / remove from this course.";		// Need bersetzung
#elif L==2
	"Select the groups of students you want to register in / remove from this course.";
#elif L==3
	"Seleccione los grupos de estudiantes que desee inscribir en / eliminar de esta asignatura.";
#elif L==4
	"Select the groups of students you want to register in / remove from this course.";		// Besoin de traduction
#elif L==5
	"Seleccione los grupos de estudiantes que desee inscribir en / eliminar de esta asignatura.";	// Okoteve traduccin
#elif L==6
	"Seleziona i gruppi di studenti che vuoi registrare in / rimuovere da questo corso.";
#elif L==7
	"Select the groups of students you want to register in / remove from this course.";		// Potrzebujesz tlumaczenie
#elif L==8
	"Select the groups of students you want to register in / remove from this course.";		// Necessita de traduo
#endif

const char *Txt_Semester =
#if   L==0
	"Quadrimestre / Semestre";
#elif L==1
	"Semester";
#elif L==2
	"Semester";
#elif L==3
	"Cuatrimestre / Semestre";
#elif L==4
	"Semestre";
#elif L==5
	"Cuatrimestre / Semestre";	// Okoteve traduccin
#elif L==6
	"Semestre";
#elif L==7
	"Semestr";
#elif L==8
	"Semestre";
#endif

const char *Txt_Semester_ABBREVIATION =
#if   L==0
	"Quadr./<br />Semes.";
#elif L==1
	"Semester";
#elif L==2
	"Semester";
#elif L==3
	"Cuatr./<br/>Semes.";
#elif L==4
	"Semestre";
#elif L==5
	"Cuatr./<br/>Semes.";	// Okoteve traduccin
#elif L==6
	"Semestre";
#elif L==7
	"Semestr";
#elif L==8
	"Semestre";
#endif

const char *Txt_SEMESTER_OF_YEAR[1 + 2] =
   {
#if   L==0
	 "Anual"	// Necessita traduccio
#elif L==1
	 "Yearlong"	// Need bersetzung
#elif L==2
	 "Yearlong"
#elif L==3
	 "Anual"
#elif L==4
	 "Yearlong"	// Besoin de traduction
#elif L==5
	 "Anual"	// Okoteve traduccin
#elif L==6
	 "Annuale"
#elif L==7
	 "Yearlong"	// Potrzebujesz tlumaczenie
#elif L==8
	 "Yearlong"	// Necessita de traduo
#endif
	 ,
#if   L==0
	 "1&ordm;"	// Necessita traduccio
#elif L==1
	 "1st"		// Need bersetzung
#elif L==2
	 "1st"
#elif L==3
	 "1&ordm;"
#elif L==4
	 "1st"		// Besoin de traduction
#elif L==5
	 "1&ordm;"	// Okoteve traduccin
#elif L==6
	 "Primo"
#elif L==7
	 "1st"		// Potrzebujesz tlumaczenie
#elif L==8
	 "1st"		// Necessita de traduo
#endif
	 ,
#if   L==0
	 "2&ordm;"	// Necessita traduccio
#elif L==1
	 "2nd"		// Need bersetzung
#elif L==2
	 "2nd"
#elif L==3
	 "2&ordm;"
#elif L==4
	 "2nd"		// Besoin de traduction
#elif L==5
	 "2&ordm;"	// Okoteve traduccin
#elif L==6
	 "Secondo"
#elif L==7
	 "2nd"		// Potrzebujesz tlumaczenie
#elif L==8
	 "2nd"		// Necessita de traduo
#endif
	};

const char *Txt_Send =
#if   L==0
	"Enviar";
#elif L==1
	"Senden";
#elif L==2
	"Send";
#elif L==3
	"Enviar";
#elif L==4
	"Envoyer";
#elif L==5
	"Enviar";	// Okoteve traduccin
#elif L==6
	"Invia";
#elif L==7
	"Send";		// Potrzebujesz tlumaczenie
#elif L==8
	"Enviar";
#endif

const char *Txt_Send_announcement_of_exam =
#if   L==0
	"Enviar convocatoria de examen";	// Necessita traduccio
#elif L==1
	"Senden Aufruf f&uuml;r Pr&uuml;fung";
#elif L==2
	"Send announcement of exam";
#elif L==3
	"Enviar convocatoria de examen";
#elif L==4
	"Envoyer convocation &agrave; l'examen";
#elif L==5
	"Enviar convocatoria de examen";	// Okoteve traduccin
#elif L==6
	"Invia sessione d'esame";
#elif L==7
	"Send announcement of exam";		// Potrzebujesz tlumaczenie
#elif L==8
	"Enviar chamada para o exame";
#endif

const char *Txt_Send_message =
#if   L==0
	"Enviar mensaje";	// Necessita traduccio
#elif L==1
	"Senden Nachricht";
#elif L==2
	"Send message";
#elif L==3
	"Enviar mensaje";
#elif L==4
	"Envoyer message";
#elif L==5
	"Enviar mensaje";	// Okoteve traduccin
#elif L==6
	"Invia messaggio";
#elif L==7
	"Send message";		// Potrzebujesz tlumaczenie
#elif L==8
	"Enviar mensagem";
#endif

const char *Txt_Send_survey =
#if   L==0
	"Enviar encuesta";	// Necessita traduccio
#elif L==1
	"Senden Umfrage";
#elif L==2
	"Send survey";
#elif L==3
	"Enviar encuesta";
#elif L==4
	"Envoyer sondage";
#elif L==5
	"Enviar encuesta";	// Okoteve traduccin
#elif L==6
	"Invia sondaggio";
#elif L==7
	"Send survey";		// Potrzebujesz tlumaczenie
#elif L==8
	"Enviar inqu&eacute;rito";
#endif

const char *Txt_Send_URL =
#if   L==0
	"Enviar URL";		// Necessita traduccio
#elif L==1
	"Senden URL";
#elif L==2
	"Send URL";
#elif L==3
	"Enviar URL";
#elif L==4
	"Envoyer URL";
#elif L==5
	"Enviar URL";		// Okoteve traduccin
#elif L==6
	"Invia URL";
#elif L==7
	"Send URL";		// Potrzebujesz tlumaczenie
#elif L==8
	"Enviar URL";
#endif

const char *Txt_Sender_banned_click_to_unban_him =
#if   L==0
	"Remitente bloqueado/a; pulse para desbloquearlo/a";	// Necessita traduccio
#elif L==1
	"Sender banned; click to unban her/him";		// Need bersetzung
#elif L==2
	"Sender banned; click to unban her/him";
#elif L==3
	"Remitente bloqueado/a; pulse para desbloquearlo/a";
#elif L==4
	"Sender banned; click to unban her/him";		// Besoin de traduction
#elif L==5
	"Remitente bloqueado/a; pulse para desbloquearlo/a";	// Okoteve traduccin
#elif L==6
	"Sender banned; click to unban her/him";		// Bisogno di traduzione
#elif L==7
	"Sender banned; click to unban her/him";		// Potrzebujesz tlumaczenie
#elif L==8
	"Sender banned; click to unban her/him";		// Necessita de traduo
#endif

const char *Txt_Sender_permitted_click_to_ban_him =
#if   L==0
	"Remitente permitido/a; pulse para bloquearlo/a";	// Necessita traduccio
#elif L==1
	"Sender permitted; click to ban her/him";		// Need bersetzung
#elif L==2
	"Sender permitted; click to ban her/him";
#elif L==3
	"Remitente permitido/a; pulse para bloquearlo/a";
#elif L==4
	"Sender permitted; click to ban her/him";		// Besoin de traduction
#elif L==5
	"Remitente permitido/a; pulse para bloquearlo/a";	// Okoteve traduccin
#elif L==6
	"Sender permitted; click to ban her/him";		// Bisogno di traduzione
#elif L==7
	"Sender permitted; click to ban her/him";		// Potrzebujesz tlumaczenie
#elif L==8
	"Sender permitted; click to ban her/him";		// Necessita de traduo
#endif

const char *Txt_session =
#if   L==0
	"sesi&oacute;n";	// Necessita traduccio
#elif L==1
	"Sitzung";
#elif L==2
	"session";
#elif L==3
	"sesi&oacute;n";
#elif L==4
	"session";
#elif L==5
	"sesi&oacute;n";	// Okoteve traduccin
#elif L==6
	"sessione";
#elif L==7
	"sesja";
#elif L==8
	"sess&atilde;o";
#endif

const char *Txt_sessions =
#if   L==0
	"sessions";
#elif L==1
	"Sitzungen";
#elif L==2
	"sessions";
#elif L==3
	"sesiones";
#elif L==4
	"sessions";
#elif L==5
	"sesiones";	// Okoteve traduccin
#elif L==6
	"sessioni";
#elif L==7
	"sessions";	// Potrzebujesz tlumaczenie
#elif L==8
	"sess&otilde;es";
#endif

const char *Txt_Sex =
#if   L==0
	"Sexe";
#elif L==1
	"Sex";
#elif L==2
	"Sex";
#elif L==3
	"Sexo";
#elif L==4
	"Sexe";
#elif L==5
	"Me&ntilde;a";
#elif L==6
	"Sesso";
#elif L==7
	"Ple&cacute;";
#elif L==8
	"Sexo";
#endif

const char *Txt_SEX_PLURAL_Abc[Usr_NUM_SEXS] =
   {
#if   L==0	// Usr_SEX_UNKNOWN
	 "Desconocidos"	// Necessita traduccio
#elif L==1
	 "Unbekannt"
#elif L==2
	 "Unknown"
#elif L==3
	 "Desconocidos"
#elif L==4
	 "Inconnus"
#elif L==5
	 "Desconocidos"	// Okoteve traduccin
#elif L==6
	 "Sconosciuti"
#elif L==7
	 "Nieznany"
#elif L==8
	 "Desconhecidos"
#endif
	 ,
#if   L==0	// Usr_SEX_FEMALE
	 "Dones"	// Necessita traduccio
#elif L==1
	 "Frauen"
#elif L==2
	 "Women"
#elif L==3
	 "Mujeres"
#elif L==4
	 "Femmes"
#elif L==5
	 "Mujeres"	// Okoteve traduccin
#elif L==6
	 "Donne"
#elif L==7
	 "Kobiety"
#elif L==8
	 "Mulheres"
#endif
	 ,
#if   L==0	// Usr_SEX_MALE
	 "Homes"
#elif L==1
	 "M&auml;nner"
#elif L==2
	 "Men"
#elif L==3
	 "Hombres"
#elif L==4
	 "Hommes"
#elif L==5
	 "Hombres"	// Okoteve traduccin
#elif L==6
	 "Uomini"
#elif L==7
	 "Mezczyzni"
#elif L==8
	 "Homens"
#endif
	 ,
#if   L==0	// Usr_SEX_ALL
	 "Todos"	// Necessita traduccio
#elif L==1
	 "Alle"
#elif L==2
	 "All"
#elif L==3
	 "Todos"
#elif L==4
	 "Tous"
#elif L==5
	 "Todos"	// Okoteve traduccin
#elif L==6
	 "Tutti"
#elif L==7
	 "Wszyscy"
#elif L==8
	 "Todos"
#endif
	};

const char *Txt_SEX_PLURAL_abc[Usr_NUM_SEXS] =
   {
#if   L==0	// Usr_SEX_UNKNOWN
	 "desconocidos"	// Necessita traduccio
#elif L==1
	 "unbekannt"
#elif L==2
	 "unknown"
#elif L==3
	 "desconocidos"
#elif L==4
	 "inconnus"
#elif L==5
	 "desconocidos"	// Okoteve traduccin
#elif L==6
	 "sconosciuti"
#elif L==7
	 "nieznany"
#elif L==8
	 "desconhecidos"
#endif
	 ,
#if   L==0	// Usr_SEX_FEMALE
	 "dones"
#elif L==1
	 "Frauen"
#elif L==2
	 "women"
#elif L==3
	 "mujeres"
#elif L==4
	 "femmes"
#elif L==5
	 "mujeres"	// Okoteve traduccin
#elif L==6
	 "donne"
#elif L==7
	 "kobiety"
#elif L==8
	 "mulheres"
#endif
	 ,
#if   L==0	// Usr_SEX_MALE
	 "homes"
#elif L==1
	 "M&auml;nner"
#elif L==2
	 "men"
#elif L==3
	 "hombres"
#elif L==4
	 "hommes"
#elif L==5
	 "hombres"	// Okoteve traduccin
#elif L==6
	 "uomini"
#elif L==7
	 "mezczyzni"
#elif L==8
	 "homens"
#endif
	 ,
#if   L==0	// Usr_SEX_ALL
	 "todos"	// Necessita traduccio
#elif L==1
	 "alle"
#elif L==2
	 "all"
#elif L==3
	 "todos"
#elif L==4
	 "tous"
#elif L==5
	 "todos"	// Okoteve traduccin
#elif L==6
	 "tutti"
#elif L==7
	 "wszyscy"
#elif L==8
	 "todos"
#endif
	};

const char *Txt_SEX_SINGULAR_Abc[Usr_NUM_SEXS] =
   {
#if   L==0	// Usr_SEX_UNKNOWN
	 "Desconocido"	// Necessita traduccio
#elif L==1
	 "Unbekannt"
#elif L==2
	 "Unknown"
#elif L==3
	 "Desconocido"
#elif L==4
	 "Inconnu"
#elif L==5
	 "Desconocido"	// Okoteve traduccin
#elif L==6
	 "Sconosciuto"
#elif L==7
	 "Nieznany"
#elif L==8
	 "Desconhecido"
#endif
	 ,
#if   L==0	// Usr_SEX_FEMALE
	 "Dona"
#elif L==1
	 "Frau"
#elif L==2
	 "Woman"
#elif L==3
	 "Mujer"
#elif L==4
	 "Femme"
#elif L==5
	 "Ku&ntilde;a"
#elif L==6
	 "Donna"
#elif L==7
	 "Kobieta"
#elif L==8
	 "Mulher"
#endif
	 ,
#if   L==0	// Usr_SEX_MALE
	 "Home"
#elif L==1
	 "Mann"
#elif L==2
	 "Man"
#elif L==3
	 "Hombre"
#elif L==4
	 "Homme"
#elif L==5
	 "Kuimba'e"
#elif L==6
	 "Uomo"
#elif L==7
	 "M&eogon;&zdot;czyzna"
#elif L==8
	 "Homem"
#endif
	 ,
#if   L==0	// Usr_SEX_ALL
	 "Todos"		// Necessita traduccio
#elif L==1
	 "Alle"
#elif L==2
	 "All"
#elif L==3
	 "Todos"
#elif L==4
	 "Tous"
#elif L==5
	 "Todos"		// Okoteve traduccin
#elif L==6
	 "Tutti"
#elif L==7
	 "Wwszyscy"
#elif L==8
	 "Todos"
#endif
	};

const char *Txt_Shared_files_area =
#if   L==0
	"Zona d'arxius compartits";
#elif L==1
	"Freigegebene Dateien-Bereich";
#elif L==2
	"Shared files area";
#elif L==3
	"Zona de archivos compartidos";
#elif L==4
	"Zone de fichiers partag&eacute;s";
#elif L==5
	"Zona de archivos compartidos";	// Okoteve traduccin
#elif L==6
	"Area comune";
#elif L==7
	"Wsp&oacute;lnej dla";
#elif L==8
	"Zona comum";
#endif

const char *Txt_Short_Name =
#if   L==0
	"Nombre breve";		// Necessita traduccio
#elif L==1
	"Kurzname";
#elif L==2
	"Short name";
#elif L==3
	"Nombre breve";
#elif L==4
	"Nom abr&eacute;g&eacute;";
#elif L==5
	"Nombre breve";		// Okoteve traduccin
#elif L==6
	"Nome breve";
#elif L==7
	"Short name";		// Potrzebujesz tlumaczenie
#elif L==8
	"Nome abreviado";
#endif

const char *Txt_Short_BR_Name =
#if   L==0
	"Nombre<br />breve";	// Necessita traduccio
#elif L==1
	"Kurzname";
#elif L==2
	"Short<br />name";
#elif L==3
	"Nombre<br />breve";
#elif L==4
	"Nom<br />abr&eacute;g&eacute;";
#elif L==5
	"Nombre<br />breve";	// Okoteve traduccin
#elif L==6
	"Nome<br />breve";
#elif L==7
	"Short<br />name";	// Potrzebujesz tlumaczenie
#elif L==8
	"Nome<br />abreviado";
#endif

const char *Txt_Shortcut_to_this_course =
#if   L==0
	"Acceso directo a la asignatura";	// Necessita traduccio
#elif L==1
	"Shortcut to this course";		// Need bersetzung
#elif L==2
	"Shortcut to this course";
#elif L==3
	"Acceso directo a la asignatura";
#elif L==4
	"Shortcut to this course";		// Besoin de traduction
#elif L==5
	"Acceso directo a la asignatura";	// Okoteve traduccin
#elif L==6
	"Accesso diretto al corso";
#elif L==7
	"Shortcut to this course";		// Potrzebujesz tlumaczenie
#elif L==8
	"Shortcut to this course";		// Necessita de traduo
#endif

const char *Txt_Shortcut_to_this_degree =
#if   L==0
	"Acceso directo a la titulaci&oacute;n";	// Necessita traduccio
#elif L==1
	"Shortcut to this degree";			// Need bersetzung
#elif L==2
	"Shortcut to this degree";
#elif L==3
	"Acceso directo a la titulaci&oacute;n";
#elif L==4
	"Shortcut to this degree";			// Besoin de traduction
#elif L==5
	"Acceso directo a la titulaci&oacute;n";	// Okoteve traduccin
#elif L==6
	"Accesso diretto alla laurea";
#elif L==7
	"Shortcut to this degree";			// Potrzebujesz tlumaczenie
#elif L==8
	"Shortcut to this degree";			// Necessita de traduo
#endif

const char *Txt_Shortcut_to_this_centre =
#if   L==0
	"Acceso directo al centro";		// Necessita traduccio
#elif L==1
	"Shortcut to this centre";		// Need bersetzung
#elif L==2
	"Shortcut to this centre";
#elif L==3
	"Acceso directo al centro";
#elif L==4
	"Shortcut to this centre";		// Besoin de traduction
#elif L==5
	"Acceso directo al centro";		// Okoteve traduccin
#elif L==6
	"Accesso diretto al centro";
#elif L==7
	"Shortcut to this centre";		// Potrzebujesz tlumaczenie
#elif L==8
	"Shortcut to this centre";		// Necessita de traduo
#endif

const char *Txt_Shortcut_to_this_country =
#if   L==0
	"Acceso directo al pa&iacute;s";	// Necessita traduccio
#elif L==1
	"Shortcut to this country";		// Need bersetzung
#elif L==2
	"Shortcut to this country";
#elif L==3
	"Acceso directo al pa&iacute;s";
#elif L==4
	"Shortcut to this country";		// Besoin de traduction
#elif L==5
	"Acceso directo al pa&iacute;s";	// Okoteve traduccin
#elif L==6
	"Accesso diretto al paese";
#elif L==7
	"Shortcut to this country";		// Potrzebujesz tlumaczenie
#elif L==8
	"Shortcut to this country";		// Necessita de traduo
#endif

const char *Txt_Shortcut_to_this_institution =
#if   L==0
	"Acceso directo a la instituci&oacute;n";	// Necessita traduccio
#elif L==1
	"Shortcut to this institution";			// Need bersetzung
#elif L==2
	"Shortcut to this institution";
#elif L==3
	"Acceso directo a la instituci&oacute;n";
#elif L==4
	"Shortcut to this institution";			// Besoin de traduction
#elif L==5
	"Acceso directo a la instituci&oacute;n";	// Okoteve traduccin
#elif L==6
	"Accesso diretto a l'istituzione";
#elif L==7
	"Shortcut to this institution";			// Potrzebujesz tlumaczenie
#elif L==8
	"Shortcut to this institution";			// Necessita de traduo
#endif

const char *Txt_Show =
#if   L==0
	"Mostra";
#elif L==1
	"Zeige";
#elif L==2
	"Show";
#elif L==3
	"Mostrar";
#elif L==4
	"Voir";
#elif L==5
	"Mostrar";	// Okoteve traduccin
#elif L==6
	"Visualizza";
#elif L==7
	"Pokaza&cacute;";
#elif L==8
	"Mostrar";
#endif

const char *Txt_Show_FOLDER_FILE_OR_LINK_X =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"Mostra %s";
#elif L==1
	"Zeige %s";
#elif L==2
	"Show %s";
#elif L==3
	"Mostrar %s";
#elif L==4
	"Voir %s";
#elif L==5
	"Mostrar %s";	// Okoteve traduccin
#elif L==6
	"Visualizza %s";
#elif L==7
	"Pokaza&cacute; %s";
#elif L==8
	"Mostrar %s";
#endif

const char *Txt_Show_all_data =
#if   L==0
	"Mostrar totes les dades";
#elif L==1
	"Alle Daten";
#elif L==2
	"Show all data";
#elif L==3
	"Mostrar todos los datos";
#elif L==4
	"Voir toutes les donn&eacute;es";
#elif L==5
	"Mostrar todos los datos";	// Okoteve traduccin
#elif L==6
	"Visualizza tutti i dati";
#elif L==7
	"Show all data";		// Potrzebujesz tlumaczenie
#elif L==8
	"Mostrar todos os dados";
#endif

const char *Txt_Show_all_files =
#if   L==0
	"Mostrar tots els fitxers";
#elif L==1
	"Alle Dateien anzeigen";
#elif L==2
	"Show all files";
#elif L==3
	"Mostrar todos los archivos";
#elif L==4
	"Afficher tous les fichiers";
#elif L==5
	"Mostrar todos los archivos";	// Okoteve traduccin
#elif L==6
	"Mostra tutti i file";
#elif L==7
	"Poka&zdot; wszystkie pliki";
#elif L==8
	"Mostrar todos os arquivos";
#endif

const char *Txt_Show_all_notifications =
#if   L==0
	"Mostrar tots les notificacions";
#elif L==1
	"Alle Anmeldungen anzeigen";
#elif L==2
	"Show all notifications";
#elif L==3
	"Mostrar todas las notificaciones";
#elif L==4
	"Afficher tous les notifications";
#elif L==5
	"Mostrar todas las notificaciones";	// Okoteve traduccin
#elif L==6
	"Mostra tutti le notifiche";
#elif L==7
	"Poka&zdot; wszystkie powiadomienia";
#elif L==8
	"Mostrar todas as notifica&ccedil;&otilde;es";
#endif

const char *Txt_Show_anyway =
#if   L==0
	"Mostra de totes maneres";
#elif L==1
	"Zeigesowieso";
#elif L==2
	"Show anyway";
#elif L==3
	"Mostrar de todos modos";
#elif L==4
	"Voir de toute faon";
#elif L==5
	"Mostrar de todos modos";	// Okoteve traduccin
#elif L==6
	"Visualizza comunque";
#elif L==7
	"Show anyway";			// Potrzebujesz tlumaczenie
#elif L==8
	"Mostrar de qualquer maneira";
#endif

const char *Txt_Show_calendar =
#if   L==0
	"Mostra calendari";
#elif L==1
	"Kalender anzeigen";
#elif L==2
	"Show calendar";
#elif L==3
	"Ver calendario";
#elif L==4
	"Afficher calendrier";
#elif L==5
	"Ver calendario";	// Okoteve traduccin
#elif L==6
	"Mostra calendario";
#elif L==7
	"Poka&zdot; kalendarz";
#elif L==8
	"Mostrar calend&aacute;rio";
#endif

const char *Txt_Show_left_column =
#if   L==0
	"Mostrar columna izquierda";	// Necessita traduccio
#elif L==1
	"Links Spalte anzeigen";
#elif L==2
	"Show left column";
#elif L==3
	"Mostrar columna izquierda";
#elif L==4
	"Afficher la colonne de gauche";
#elif L==5
	"Mostrar columna izquierda";	// Okoteve traduccin
#elif L==6
	"Mostra colonna di sinistra";
#elif L==7
	"Show left column";		// Potrzebujesz tlumaczenie
#elif L==8
	"Mostrar coluna da esquerda";
#endif

const char *Txt_Show_list =
#if   L==0
	"Mostra la llista";
#elif L==1
	"Liste anzeigen";
#elif L==2
	"Show list";
#elif L==3
	"Ver lista";
#elif L==4
	"Afficher la liste";
#elif L==5
	"Ver lista";	// Okoteve traduccin
#elif L==6
	"Mostra elenco";
#elif L==7
	"Poka&zdot; list&eogon;";
#elif L==8
	"Mostrar lista";
#endif

const char *Txt_Show_office_hours =
#if   L==0
	"Mostra horaris tutories";
#elif L==1
	"Gesch&auml;ftszeiten anzeigen";
#elif L==2
	"Show office hours";
#elif L==3
	"Ver horarios tutor&iacute;as";
#elif L==4
	"Voir horaires tuteur";
#elif L==5
	"Ver horarios tutor&iacute;as";	// Okoteve traduccin
#elif L==6
	"Mostra orari tutoraggi";
#elif L==7
	"Poka&zdot; godzin pracy biura";
#elif L==8
	"Mostrar hor&aacute;rios tutor";
#endif

const char *Txt_Show_only_unread_messages =
#if   L==0
	"Mostrar nom&eacute;s els missatges no llegits";
#elif L==1
	"Zeige nur ungelesene Nachrichten";
#elif L==2
	"Show only unread messages";
#elif L==3
	"Mostrar &uacute;nicamente los mensajes no le&iacute;dos";
#elif L==4
	"Afficher uniquement les messages non lus";
#elif L==5
	"Mostrar &uacute;nicamente los mensajes no le&iacute;dos";	// Okoteve traduccin
#elif L==6
	"Mostra solo i messaggi non letti";
#elif L==7
	"Show only unread messages";					// Potrzebujesz tlumaczenie
#elif L==8
	"Mostrar apenas mensagens n&atilde;o lidas";
#endif

const char *Txt_Show_right_column =
#if   L==0
	"Mostrar columna derecha";		// Necessita traduccio
#elif L==1
	"Rechte Spalte anzeigen";
#elif L==2
	"Show right column";
#elif L==3
	"Mostrar columna derecha";
#elif L==4
	"Afficher la colonne de droite";
#elif L==5
	"Mostrar columna derecha";		// Okoteve traduccin
#elif L==6
	"Mostra colonna di destra";
#elif L==7
	"Show right column";			// Potrzebujesz tlumaczenie
#elif L==8
	"Mostrar coluna da direita";
#endif

const char *Txt_Show_WHICH_groups[2] =
   {
#if   L==0
	 "Nom&eacute;s els meus grups"
#elif L==1
	 "Nur meine Gruppen"
#elif L==2
	 "Only my groups"
#elif L==3
	 "S&oacute;lo mis grupos"
#elif L==4
	 "Uniquement mes groupes"
#elif L==5
	 "S&oacute;lo mis grupos"	// Okoteve traduccin
#elif L==6
	 "Solo i miei gruppi"
#elif L==7
	 "Only my groups"			// Potrzebujesz tlumaczenie
#elif L==8
	 "Apenas os meus grupos"
#endif
	 ,
#if   L==0
	 "Tots els grups"
#elif L==1
	 "Alle Gruppen"
#elif L==2
	 "All groups"
#elif L==3
	 "Todos los grupos"
#elif L==4
	 "Tous les groupes"
#elif L==5
	 "Todos los grupos"	// Okoteve traduccin
#elif L==6
	 "Tutti i gruppi"
#elif L==7
	 "All groups"		// Potrzebujesz tlumaczenie
#elif L==8
	 "Todos os grupos"
#endif
	};

const char *Txt_Show_more_details =
#if   L==0
	"Mostra m&eacute;s detalls";
#elif L==1
	"Mehr Details";
#elif L==2
	"Show more details";
#elif L==3
	"Mostrar m&aacute;s detalles";
#elif L==4
	"Montrer plus de d&eacute;tails";
#elif L==5
	"Mostrar m&aacute;s detalles";	// Okoteve traduccin
#elif L==6
	"Mostra pi&ugrave; dettagli";
#elif L==7
	"Poka&zdot; wi&eogon;cej szczeg&oacute;&lstrok;&oacute;w";
#elif L==8
	"Mostrar mais detalhes";
#endif

const char *Txt_Show_next_X_clicks =	// Warning: it is very important to include %lu in the following sentences
#if   L==0
	"Mostrar %lu clics posteriores";	// Necessita traduccio
#elif L==1
	"Show next %lu clicks";			// Need bersetzung
#elif L==2
	"Show next %lu clicks";
#elif L==3
	"Mostrar %lu clics posteriores";
#elif L==4
	"Show next %lu clicks";			// Besoin de traduction
#elif L==5
	"Mostrar %lu clics posteriores";	// Okoteve traduccin
#elif L==6
	"Mostra %lu scatti seguenti";
#elif L==7
	"Show next %lu clicks";			// Potrzebujesz tlumaczenie
#elif L==8
	"Show next %lu clicks";			// Necessita de traduo
#endif

const char *Txt_Show_previous_X_clicks =	// Warning: it is very important to include %lu in the following sentences
#if   L==0
	"Mostrar %lu clics anteriores";		// Necessita traduccio
#elif L==1
	"Show previous %lu clicks";		// Need bersetzung
#elif L==2
	"Show previous %lu clicks";
#elif L==3
	"Mostrar %lu clics anteriores";
#elif L==4
	"Show previous %lu clicks";		// Besoin de traduction
#elif L==5
	"Mostrar %lu clics anteriores";		// Okoteve traduccin
#elif L==6
	"Mostra %lu scatti precedenti";
#elif L==7
	"Show previous %lu clicks";		// Potrzebujesz tlumaczenie
#elif L==8
	"Show previous %lu clicks";		// Necessita de traduo
#endif

const char *Txt_Show_questions =
#if   L==0
	"Mostrar preguntes";
#elif L==1
	"Zeige Testfragen";
#elif L==2
	"Show questions";
#elif L==3
	"Mostrar preguntas";
#elif L==4
	"Voir les questions";
#elif L==5
	"Mostrar preguntas";	// Okoteve traduccin
#elif L==6
	"Visualizza le domande";
#elif L==7
	"Show questions";	// Potrzebujesz tlumaczenie
#elif L==8
	"Mostrar quest&otilde;es";
#endif

const char *Txt_Show_records =
#if   L==0
	"Mostrar fichas";	// Necessita traduccio
#elif L==1
	"Zeige Profile";
#elif L==2
	"Show records";
#elif L==3
	"Mostrar fichas";
#elif L==4
	"Voir les cartes";
#elif L==5
	"Mostrar fichas";	// Okoteve traduccin
#elif L==6
	"Visualizza le schede";
#elif L==7
	"Show records";		// Potrzebujesz tlumaczenie
#elif L==8
	"Mostrar cart&otilde;es";
#endif

const char *Txt_Show_statistic =
#if   L==0
	"Mostrar estad&iacute;stica";
#elif L==1
	"Zeige Statistik";
#elif L==2
	"Show statistic";
#elif L==3
	"Mostrar estad&iacute;stica";
#elif L==4
	"Voir statistique";
#elif L==5
	"Mostrar estad&iacute;stica";	// Okoteve traduccin
#elif L==6
	"Visualizza statistica";
#elif L==7
	"Show statistic";		// Potrzebujesz tlumaczenie
#elif L==8
	"Mostrar estat&iacute;stica";
#endif

const char *Txt_Show_timetable =
#if   L==0
	"Veure horari";
#elif L==1
	"Zeige Stundenplan";
#elif L==2
	"Show timetable";
#elif L==3
	"Ver horario";
#elif L==4
	"Voir horaire";
#elif L==5
	"Ver horario";	// Okoteve traduccin
#elif L==6
	"Vedi orario";
#elif L==7
	"Wiadomo&sacute;ci plan lekcji";
#elif L==8
	"Veja hor&aacute;rio";
#endif

const char *Txt_Show_visits =
#if   L==0
	"Veure accessos";
#elif L==1
	"Zeige Anmeldungen";
#elif L==2
	"Show visits";
#elif L==3
	"Ver accesos";
#elif L==4
	"Voir visites";
#elif L==5
	"Ver accesos";		// Okoteve traduccin
#elif L==6
	"Visualizza visite";
#elif L==7
	"Show visits";		// Potrzebujesz tlumaczenie
#elif L==8
	"Mostrar visitas";
#endif

const char *Txt_Shuffle =
#if   L==0
	"Barrejar";
#elif L==1
	"Shuffle";
#elif L==2
	"Shuffle";
#elif L==3
	"Barajar";
#elif L==4
	"M&eacute;langer";
#elif L==5
	"Barajar";	// Okoteve traduccin
#elif L==6
	"Mischiare";
#elif L==7
	"Shuffle";	// Potrzebujesz tlumaczenie
#elif L==8
	"Baralhar";
#endif

const char *Txt_Sign_up =
#if   L==0
	"Sollicitar inscripci&oacute;";
#elif L==1
	"Registrierung beantragen";
#elif L==2
	"Sign up";
#elif L==3
	"Solicitar inscripci&oacute;n";
#elif L==4
	"Demander inscription";
#elif L==5
	"Solicitar inscripci&oacute;n";	// Okoteve traduccin
#elif L==6
	"Richiedere iscrizione";
#elif L==7
	"Wniosku rejestracj&eogon;";
#elif L==8
	"Solicitar inscri&ccedil;&atilde;o";
#endif

const char *Txt_Size =		// ...of a file zone
#if   L==0
	"Mida";
#elif L==1
	"Gr&ouml;&szlig;e";
#elif L==2
	"Size";
#elif L==3
	"Tama&ntilde;o";
#elif L==4
	"Taille";
#elif L==5
	"Tama&ntilde;o";	// Okoteve traduccin
#elif L==6
	"Dimensione";
#elif L==7
	"Size";			// Potrzebujesz tlumaczenie
#elif L==8
	"Tamanho";
#endif

const char *Txt_Size_of_photos =
#if   L==0
	"Mida de les fotos";
#elif L==1
	"Gr&ouml;&szlig;e des Fotos";
#elif L==2
	"Size of photos";
#elif L==3
	"Tama&ntilde;o de las fotos";
#elif L==4
	"Taille des photos";
#elif L==5
	"Tama&ntilde;o de las fotos";	// Okoteve traduccin
#elif L==6
	"Dimensioni delle foto";
#elif L==7
	"Rozmiar zdj&eogon;&cacute;";
#elif L==8
	"Tamanho das fotos";
#endif

const char *Txt_Skype =
#if   L==0
	"Skype&trade;";
#elif L==1
	"Skype&trade;";
#elif L==2
	"Skype&trade;";
#elif L==3
	"Skype&trade;";
#elif L==4
	"Skype&trade;";
#elif L==5
	"Skype&trade;";	// Okoteve traduccin
#elif L==6
	"Skype&trade;";
#elif L==7
	"Skype&trade;";
#elif L==8
	"Skype&trade;";
#endif

const char *Txt_Sort_degrees_by =
#if   L==0
	"Ordenar titulacions per";
#elif L==1
	"Sortieren Studieng&auml;nge nach";
#elif L==2
	"Sort degrees by";
#elif L==3
	"Ordenar titulaciones por";
#elif L==4
	"Trier &eacute;tudes par";
#elif L==5
	"Ordenar titulaciones por";	// Okoteve traduccin
#elif L==6
	"Ordina laurea per";
#elif L==7
	"Sortuj stopni wed&lstrok;ug";
#elif L==8
	"Classificar titula&ccedil;oes por";
#endif

const char *Txt_Source_of_information =
#if   L==0
	"Font d'informaci&oacute;";
#elif L==1
	"Quelle von Informationen";
#elif L==2
	"Source of information";
#elif L==3
	"Fuente de informaci&oacute;n";
#elif L==4
	"Source d'information";
#elif L==5
	"Fuente de informaci&oacute;n";	// Okoteve traduccin
#elif L==6
	"Fonte delle informazioni";
#elif L==7
	"&Zacute;r&oacute;d&lstrok;o informacji";
#elif L==8
	"Fonte de informa&ccedil;&atilde;o";
#endif

const char *Txt_Start_date =
#if   L==0
	"Data inicial";
#elif L==1
	"Startdatum";
#elif L==2
	"Start date";
#elif L==3
	"Fecha inicial";
#elif L==4
	"Date initiale";
#elif L==5
	"Fecha inicial";	// Okoteve traduccin
#elif L==6
	"Data iniziale";
#elif L==7
	"Data rozpocz&eogon;cia";
#elif L==8
	"Data de in&iacute;cio";
#endif

const char *Txt_STAT_CLICK_STAT_TYPES[Sta_NUM_TYPES_CLICK_STATS] =
   {
#if   L==0
	 "Mostra clics detallats"
#elif L==1
	 "Zeige detaillierte Klicks"
#elif L==2
	 "Show detailed clicks"
#elif L==3
	 "Mostrar clics detallados"
#elif L==4
	 "Afficher clics d&eacute;taill&eacute;es"
#elif L==5
	 "Mostrar clics detallados"	// Okoteve traduccin
#elif L==6
	 "Visualizza dettagli di scatto"
#elif L==7
	 "Poka&zdot; szczeg&oacute;&lstrok;owe klikni&eogon;&cacute;"
#elif L==8
	 "Mostrar detalhadamente cliques"
#endif
	 ,
#if   L==0
	 "usuario"	// Necessita traduccio
#elif L==1
	 "Benutzer"
#elif L==2
	 "user"
#elif L==3
	 "usuario"
#elif L==4
	 "utilisateur"
#elif L==5
	 "puruh&aacute;ra"
#elif L==6
	 "utente"
#elif L==7
	 "u&zdot;ytkownik"
#elif L==8
	 "utilizador"
#endif
	 ,
#if   L==0
	 "dia"
#elif L==1
	 "Tag"
#elif L==2
	 "day"
#elif L==3
	 "d&iacute;a"
#elif L==4
	 "jour"
#elif L==5
	 "d&iacute;a"	// Okoteve traduccin
#elif L==6
	 "giorno"
#elif L==7
	 "dzie&nacute;"
#elif L==8
	 "dia"
#endif
	 ,
#if   L==0
	 "dia i hora"
#elif L==1
	 "Tag und Stunde"
#elif L==2
	 "day and hour"
#elif L==3
	 "d&iacute;a y hora"
#elif L==4
	 "jour et heure"
#elif L==5
	 "d&iacute;a y hora"	// Okoteve traduccin
#elif L==6
	 "giorno e ora"
#elif L==7
	 "dzie&nacute; i godzin&eogon;"
#elif L==8
	 "dia e hora"
#endif
	 ,
#if   L==0
	 "setmana"
#elif L==1
	 "Woche"
#elif L==2
	 "week"
#elif L==3
	 "semana"
#elif L==4
	 "semaine"
#elif L==5
	 "arapok&otilde;indy"
#elif L==6
	 "settimana"
#elif L==7
	 "tydzie&nacute;"
#elif L==8
	 "semana"
#endif
	 ,
#if   L==0
	 "mes"
#elif L==1
	 "Monat"
#elif L==2
	 "month"
#elif L==3
	 "mes"
#elif L==4
	 "mois"
#elif L==5
	 "jasy"
#elif L==6
	 "mese"
#elif L==7
	 "miesi&aogon;c"
#elif L==8
	 "m&ecirc;s"
#endif
	 ,
#if   L==0
	 "hora"
#elif L==1
	 "Stunde"
#elif L==2
	 "hour"
#elif L==3
	 "hora"
#elif L==4
	 "heure"
#elif L==5
	 "aravo"
#elif L==6
	 "ora"
#elif L==7
	 "godzina"
#elif L==8
	 "hora"
#endif
	 ,
#if   L==0
	 "minut"
#elif L==1
	 "Minute"
#elif L==2
	 "minute"
#elif L==3
	 "minuto"
#elif L==4
	 "minute"
#elif L==5
	 "aravo'i"
#elif L==6
	 "minuto"
#elif L==7
	 "minut&eogon;"
#elif L==8
	 "minuto"
#endif
	 ,
#if   L==0
	 "acci&oacute;"
#elif L==1
	 "Handlung"
#elif L==2
	 "action"
#elif L==3
	 "acci&oacute;n"
#elif L==4
	 "action"
#elif L==5
	 "tembiapo"
#elif L==6
	 "azione"
#elif L==7
	 "dzia&lstrok;anie"
#elif L==8
	 "a&ccedil;&atilde;o"
#endif
	 ,
#if   L==0
	 "dia"
#elif L==1
	 "Tag"
#elif L==2
	 "day"
#elif L==3
	 "d&iacute;a"
#elif L==4
	 "jour"
#elif L==5
	 "d&iacute;a"	// Okoteve traduccin
#elif L==6
	 "giorno"
#elif L==7
	 "dzie&nacute;"
#elif L==8
	 "dia"
#endif
	 ,
#if   L==0
	 "dia i hora"
#elif L==1
	 "Tag und Stunde"
#elif L==2
	 "day and hour"
#elif L==3
	 "d&iacute;a y hora"
#elif L==4
	 "jour et heure"
#elif L==5
	 "d&iacute;a y hora"	// Okoteve traduccin
#elif L==6
	 "giorno e ora"
#elif L==7
	 "dzie&nacute; i godzin&eogon;"
#elif L==8
	 "dia e hora"
#endif
	 ,
#if   L==0
	 "setmana"
#elif L==1
	 "Woche"
#elif L==2
	 "week"
#elif L==3
	 "semana"
#elif L==4
	 "semaine"
#elif L==5
	 "arapok&otilde;indy"
#elif L==6
	 "settimana"
#elif L==7
	 "tydzie&nacute;"
#elif L==8
	 "semana"
#endif
	 ,
#if   L==0
	 "mes"
#elif L==1
	 "Monat"
#elif L==2
	 "month"
#elif L==3
	 "mes"
#elif L==4
	 "mois"
#elif L==5
	 "jasy"
#elif L==6
	 "mese"
#elif L==7
	 "miesi&aogon;c"
#elif L==8
	 "m&ecirc;s"
#endif
	 ,
#if   L==0
	 "hora"
#elif L==1
	 "Stunde"
#elif L==2
	 "hour"
#elif L==3
	 "hora"
#elif L==4
	 "heure"
#elif L==5
	 "aravo"
#elif L==6
	 "ora"
#elif L==7
	 "godzina"
#elif L==8
	 "hora"
#endif
	 ,
#if   L==0
	 "minut"
#elif L==1
	 "Minute"
#elif L==2
	 "minute"
#elif L==3
	 "minuto"
#elif L==4
	 "minute"
#elif L==5
	 "aravo'i"
#elif L==6
	 "minuto"
#elif L==7
	 "minut&eogon;"
#elif L==8
	 "minuto"
#endif
	 ,
#if   L==0
	 "acci&oacute;"
#elif L==1
	 "Handlung"
#elif L==2
	 "action"
#elif L==3
	 "acci&oacute;n"
#elif L==4
	 "action"
#elif L==5
	 "tembiapo"
#elif L==6
	 "azione"
#elif L==7
	 "dzia&lstrok;anie"
#elif L==8
	 "a&ccedil;&atilde;o"
#endif
	 ,
#if   L==0
	 "complement"
#elif L==1
	 "Plugin"
#elif L==2
	 "plugin"
#elif L==3
	 "complemento"
#elif L==4
	 "plugin"
#elif L==5
	 "complemento"	// Okoteve traduccin
#elif L==6
	 "plugin"
#elif L==7
	 "wtyczka"
#elif L==8
	 "plugin"
#endif
	 ,
#if   L==0
	 "funci&oacute;"
#elif L==1
	 "Funktion"
#elif L==2
	 "function"
#elif L==3
	 "funci&oacute;n"
#elif L==4
	 "fonction"
#elif L==5
	 "funci&oacute;n"	// Okoteve traduccin
#elif L==6
	 "funzione"
#elif L==7
	 "funkcja"
#elif L==8
	 "fun&ccedil;&atilde;o"
#endif
	 ,
#if   L==0
	 "banner"
#elif L==1
	 "Werbebanner"
#elif L==2
	 "banner"
#elif L==3
	 "banner"
#elif L==4
	 "banni&eacute;re"
#elif L==5
	 "banner"	// Okoteve traduccin
#elif L==6
	 "banner"
#elif L==7
	 "banner"
#elif L==8
	 "banner"
#endif
	 ,
#if   L==0
	 "titulaci&oacute;"
#elif L==1
	 "Studiengang"
#elif L==2
	 "degree"
#elif L==3
	 "titulaci&oacute;n"
#elif L==4
	 "&eacute;tude"
#elif L==5
	 "arandur&atilde;"
#elif L==6
	 "laurea"
#elif L==7
	 "stopie&nacute;"
#elif L==8
	 "titula&ccedil;&atilde;o"
#endif
	 ,
#if   L==0
	 "assignatura"
#elif L==1
	 "Kurs"
#elif L==2
	 "course"
#elif L==3
	 "asignatura"
#elif L==4
	 "mati&egrave;re"
#elif L==5
	 "mbo'esyry"
#elif L==6
	 "corso"
#elif L==7
	 "kurs"
#elif L==8
	 "disciplina"
#endif
	};

const char *Txt_STAT_COLOR_TYPES[Sta_NUM_COLOR_TYPES] =
   {
#if   L==0
	 "color"
#elif L==1
	 "Farbe"
#elif L==2
	 "color"
#elif L==3
	 "color"
#elif L==4
	 "couleur"
#elif L==5
	 "color"	// Okoteve traduccin
#elif L==6
	 "colore"
#elif L==7
	 "kolor"
#elif L==8
	 "cor"
#endif
	 ,
#if   L==0
	 "de negre a blanc"
#elif L==1
	 "schwarz zu wei&szlig;"
#elif L==2
	 "black to white"
#elif L==3
	 "de negro a blanco"
#elif L==4
	 "du noir au blanc"
#elif L==5
	 "de negro a blanco"	// Okoteve traduccin
#elif L==6
	 "dal nero al bianco"
#elif L==7
	 "czarny na bia&lstrok;y"
#elif L==8
	 "do preto ao branco"
#endif
	 ,
#if   L==0
	 "de blanc a negre"
#elif L==1
	 "wei&szlig; zu schwarz"
#elif L==2
	 "white to black"
#elif L==3
	 "de blanco a negro"
#elif L==4
	 "du blanc au noir"
#elif L==5
	 "de blanco a negro"	// Okoteve traduccin
#elif L==6
	 "dal bianco al nero"
#elif L==7
	 "bia&lstrok;y do czarny"
#elif L==8
	 "do branco ao preto"
#endif
	};

const char *Txt_STAT_COURSE_FILE_ZONES[Sta_NUM_STAT_CRS_FILE_ZONES] =	// Use &nbsp; instead of space
	 {
#if   L==0	// Brw_FILE_BRW_ADMIN_DOCUMENTS_CRS
	   "Documents&nbsp;(ass.)"
#elif L==1
	   "Dokumente&nbsp;(Ver.)"
#elif L==2
	   "Documents&nbsp;(crs.)"
#elif L==3
	   "Documentos&nbsp;(asg.)"
#elif L==4
	   "Documents&nbsp;(mat.)"
#elif L==5
	   "Documentos&nbsp;(asg.)"		// Okoteve traduccin
#elif L==6
	   "Documenti&nbsp;(crs.)"
#elif L==7
	   "Dokumenty&nbsp;(kurs)"
#elif L==8
	   "Documentos&nbsp;(crs.)"
#endif
	   ,
#if   L==0	// Brw_FILE_BRW_ADMIN_DOCUMENTS_GRP
	   "Documents&nbsp;(grp.)"
#elif L==1
	   "Dokumente&nbsp;(Grp.)"
#elif L==2
	   "Documents&nbsp;(grp.)"
#elif L==3
	   "Documentos&nbsp;(grp.)"
#elif L==4
	   "Documents&nbsp;(grp.)"
#elif L==5
	   "Documentos&nbsp;(grp.)"	// Okoteve traduccin
#elif L==6
	   "Documenti&nbsp;(grp.)"
#elif L==7
	   "Dokumenty&nbsp;(grp.)"
#elif L==8
	   "Documentos&nbsp;(grp.)"
#endif
	   ,
#if   L==0	// Brw_FILE_BRW_COMMON_CRS
	   "Compartits&nbsp;(asg.)"
#elif L==1
	   "Freigegebene&nbsp;(Ver.)"
#elif L==2
	   "Shared&nbsp;(crs.)"
#elif L==3
	   "Compartidos&nbsp;(asg.)"
#elif L==4
	   "Partag&eacute;s&nbsp;(mat.)"
#elif L==5
	   "Compartidos&nbsp;(asg.)"	// Okoteve traduccin
#elif L==6
	   "Condivisi&nbsp;(crs.)"
#elif L==7
	   "Wsp&oacute;lna&nbsp;(kurs)"
#elif L==8
	   "Compartilhados&nbsp;(crs.)"
#endif
	   ,
#if   L==0	// Brw_FILE_BRW_COMMON_GRP
	   "Compartits&nbsp;(grp.)"
#elif L==1
	   "Freigegebene&nbsp;(Grp.)"
#elif L==2
	   "Shared&nbsp;(grp.)"
#elif L==3
	   "Compartidos&nbsp;(grp.)"
#elif L==4
	   "Partag&eacute;s&nbsp;(grp.)"
#elif L==5
	   "Compartidos&nbsp;(grp.)"	// Okoteve traduccin
#elif L==6
	   "Condivisi&nbsp;(grp.)"
#elif L==7
	   "Wsp&oacute;lna&nbsp;(grp.)"
#elif L==8
	   "Compartilhados&nbsp;(grp.)"
#endif
	   ,
#if   L==0	// Brw_FILE_BRW_ADMIN_MARKS_CRS
	   "Qualific.&nbsp;(ass.)"
#elif L==1
	   "Bewertungen&nbsp;(Ver.)"
#elif L==2
	   "Marks&nbsp;(crs.)"
#elif L==3
	   "Calific.&nbsp;(asg.)"
#elif L==4
	   "Notes&nbsp;(mat.)"
#elif L==5
	   "Calific.&nbsp;(asg.)"		// Okoteve traduccin
#elif L==6
	   "Risultati&nbsp;(crs.)"
#elif L==7
	   "Marks&nbsp;(crs.)"			// Potrzebujesz tlumaczenie
#elif L==8
	   "Notas&nbsp;(crs.)"
#endif
	   ,
#if   L==0	// Brw_FILE_BRW_ADMIN_MARKS_GRP
	   "Calific.&nbsp;(grp.)"		// Necessita traduccio
#elif L==1
	   "Bewertungen&nbsp;(Grp.)"
#elif L==2
	   "Marks&nbsp;(grp.)"
#elif L==3
	   "Calific.&nbsp;(grp.)"
#elif L==4
	   "Notes&nbsp;(grp.)"
#elif L==5
	   "Calific.&nbsp;(grp.)"		// Okoteve traduccin
#elif L==6
	   "Risultati&nbsp;(grp.)"
#elif L==7
	   "Marks&nbsp;(grp.)"			// Potrzebujesz tlumaczenie
#elif L==8
	   "Notas&nbsp;(grp.)"
#endif
	   ,
#if   L==0	// Brw_FILE_BRW_ASSIGNMENTS_USR
	   "Activitats"
#elif L==1
	   "Aufgaben"
#elif L==2
	   "Assignments"
#elif L==3
	   "Actividades"
#elif L==4
	   "Activit&eacute;s"
#elif L==5
	   "Tar&eacute;a"
#elif L==6
	   "Attivit&agrave;"
#elif L==7
	   "Zadania"
#elif L==8
	   "Atividades"
#endif
	   ,
#if   L==0	// Brw_FILE_BRW_WORKS_USR
	   "Treballs"
#elif L==1
	   "Hausarbeiten"
#elif L==2
	   "Homework"
#elif L==3
	   "Trabajos"
#elif L==4
	   "Travails"
#elif L==5
	   "Tembiapo"
#elif L==6
	   "Lavori"
#elif L==7
	   "Praca domowa"
#elif L==8
	   "Trabalhos"
#endif
	   ,
#if   L==0	// Brw_FILE_BRW_UNKNOWN
	   "Total"
#elif L==1
	   "Gesamtzahl"
#elif L==2
	   "Total"
#elif L==3
	   "Total"
#elif L==4
	   "Total"
#elif L==5
	   "Total"		// Okoteve traduccin
#elif L==6
	   "Totale"
#elif L==7
	   "Ca&lstrok;kowity"
#elif L==8
	   "Total"
#endif
	};

const char *Txt_STAT_DEGREE_PHOTO_SIZE[Pho_NUM_HOW_COMPUTE_PHOTO_SIZES] =
   {
#if   L==0
	 "proporcional al n&ordm; de estudiantes"	// Necessita traduccio
#elif L==1
	 "proportional to the number of students"	// Need bersetzung
#elif L==2
	 "proportional to the number of students"
#elif L==3
	 "proporcional al n&ordm; de estudiantes"
#elif L==4
	 "proportional to the number of students"	// Besoin de traduction
#elif L==5
	 "proporcional al n&ordm; de estudiantes"	// Okoteve traduccin
#elif L==6
	 "proporzionale al numero di studenti"
#elif L==7
	 "proportional to the number of students"	// Potrzebujesz tlumaczenie
#elif L==8
	 "proportional to the number of students"	// Necessita de traduo
#endif
	 ,
#if   L==0
	 "proporcional al n&ordm; de estudiantes con fotograf&iacute;a"	// Necessita traduccio
#elif L==1
	 "proportional to the number of students with photo"		// Need bersetzung
#elif L==2
	 "proportional to the number of students with photo"
#elif L==3
	 "proporcional al n&ordm; de estudiantes con fotograf&iacute;a"
#elif L==4
	 "proportional to the number of students with photo"		// Besoin de traduction
#elif L==5
	 "proporcional al n&ordm; de estudiantes con fotograf&iacute;a"	// Okoteve traduccin
#elif L==6
	 "proporzionale al numero di studenti con foto"
#elif L==7
	 "proportional to the number of students with photo"		// Potrzebujesz tlumaczenie
#elif L==8
	 "proportional to the number of students with photo"		// Necessita de traduo
#endif
	 ,
#if   L==0
	 "proporcional al porcentaje de estudiantes con fotograf&iacute;a"	// Necessita traduccio
#elif L==1
	 "proportional to the percentage of students with photo"		// Need bersetzung
#elif L==2
	 "proportional to the percentage of students with photo"
#elif L==3
	 "proporcional al porcentaje de estudiantes con fotograf&iacute;a"
#elif L==4
	 "proportional to the percentage of students with photo"		// Besoin de traduction
#elif L==5
	 "proporcional al porcentaje de estudiantes con fotograf&iacute;a"	// Okoteve traduccin
#elif L==6
	 "proporzionale alla percentuale di studenti con foto"
#elif L==7
	 "proportional to the percentage of students with photo"		// Potrzebujesz tlumaczenie
#elif L==8
	 "proportional to the percentage of students with photo"		// Necessita de traduo
#endif
	 ,
#if   L==0
	 "igual para todas las titulaciones"	// Necessita traduccio
#elif L==1
	 "same for all degrees"			// Need bersetzung
#elif L==2
	 "same for all degrees"
#elif L==3
	 "igual para todas las titulaciones"
#elif L==4
	 "same for all degrees"			// Besoin de traduction
#elif L==5
	 "igual para todas las titulaciones"	// Okoteve traduccin
#elif L==6
	 "uguale per tutte le lauree"
#elif L==7
	 "same for all degrees"			// Potrzebujesz tlumaczenie
#elif L==8
	 "same for all degrees"			// Necessita de traduo
#endif
	};

const char *Txt_STAT_DEGREE_PHOTO_ORDER[Pho_NUM_HOW_ORDER_DEGREES] =
   {
#if   L==0
	 "nombre d'estudiants"
#elif L==1
	 "Anzahl der Studenten"
#elif L==2
	 "number of students"
#elif L==3
	 "n&ordm; de estudiantes"
#elif L==4
	 "nombre d'&eacute;tudiants"
#elif L==5
	 "n&ordm; de estudiantes"	// Okoteve traduccin
#elif L==6
	 "numero di studenti"
#elif L==7
	 "liczba student&oacute;w"
#elif L==8
	 "n&uacute;mero de estudantes"
#endif
	 ,
#if   L==0
	 "nombre d'estudiants amb fotografia"
#elif L==1
	 "Anzahl der Studenten mit Foto"
#elif L==2
	 "number of students with photo"
#elif L==3
	 "n&ordm; de estudiantes con fotograf&iacute;a"
#elif L==4
	 "nombre d'&eacute;tudiants avec photo"
#elif L==5
	 "n&ordm; de estudiantes con fotograf&iacute;a"	// Okoteve traduccin
#elif L==6
	 "numero di studenti con foto"
#elif L==7
	 "liczba student&oacute;w ze zdj&eogon;ciem"
#elif L==8
	 "n&uacute;mero de estudantes com foto"
#endif
	 ,
#if   L==0
	 "porcentaje de estudiantes con fotograf&iacute;a"	// Necessita traduccio
#elif L==1
	 "percentage of students with photo"			// Need bersetzung
#elif L==2
	 "percentage of students with photo"
#elif L==3
	 "porcentaje de estudiantes con fotograf&iacute;a"
#elif L==4
	 "percentage of students with photo"			// Besoin de traduction
#elif L==5
	 "porcentaje de estudiantes con fotograf&iacute;a"	// Okoteve traduccin
#elif L==6
	 "percentuale di studenti con foto"
#elif L==7
	 "percentage of students with photo"			// Potrzebujesz tlumaczenie
#elif L==8
	 "percentage of students with photo"			// Necessita de traduo
#endif
	 ,
#if   L==0
	 "nombre de titulaci&oacute;n"	// Necessita traduccio
#elif L==1
	 "degree name"			// Need bersetzung
#elif L==2
	 "degree name"
#elif L==3
	 "nombre de titulaci&oacute;n"
#elif L==4
	 "degree name"			// Besoin de traduction
#elif L==5
	 "nombre de titulaci&oacute;n"	// Okoteve traduccin
#elif L==6
	 "nome laurea"
#elif L==7
	 "degree name"			// Potrzebujesz tlumaczenie
#elif L==8
	 "degree name"			// Necessita de traduo
#endif
	};

const char *Txt_STAT_TYPE_COUNT_CAPS[Sta_NUM_STAT_COUNT_TYPES] =
   {
#if   L==0
	 "N&ordm; total de clics"	// Necessita traduccio
#elif L==1
	 "Gesamtzahl der Klicks"
#elif L==2
	 "Total number of clicks"
#elif L==3
	 "N&ordm; total de clics"
#elif L==4
	 "Nombre total de clics"
#elif L==5
	 "N&ordm; total de clics"	// Okoteve traduccin
#elif L==6
	 "Numero totale di scatti"
#elif L==7
	 "Total number of clicks"	// Potrzebujesz tlumaczenie
#elif L==8
	 "N&uacute;mero total de cliques"
#endif
	 ,
#if   L==0
	 "N&ordm; de usuarios distintos"		// Necessita traduccio
#elif L==1
	 "Anzahl der unterschiedlichen Benutzer"
#elif L==2
	 "Number of distinct users"
#elif L==3
	 "N&ordm; de usuarios distintos"
#elif L==4
	 "Nombre des diff&eacute;rents utilisateurs"
#elif L==5
	 "N&ordm; de usuarios distintos"		// Okoteve traduccin
#elif L==6
	 "Numero di utenti distinti"
#elif L==7
	 "Number of distinct users"			// Potrzebujesz tlumaczenie
#elif L==8
	 "N&uacute;mero de utilizadores diferentes"
#endif
	 ,
#if   L==0
	 "N&ordm; de clics por usuario"		// Necessita traduccio
#elif L==1
	 "Anzahl der Klicks pro Benutzer"
#elif L==2
	 "Number of clicks per user"
#elif L==3
	 "N&ordm; de clics por usuario"
#elif L==4
	 "Nombre de clics par utilisateur"
#elif L==5
	 "N&ordm; de clics por usuario"		// Okoteve traduccin
#elif L==6
	 "Numero di scatti per utente"
#elif L==7
	 "Number of clicks per user"		// Potrzebujesz tlumaczenie
#elif L==8
	 "n&uacute;mero de cliques por utilizador"
#endif
	 ,
#if   L==0
	 "Tiempo de generaci&oacute;n de p&aacute;gina"	// Necessita traduccio
#elif L==1
	 "Zeit zur Erzeugung der Seite"
#elif L==2
	 "Time to generate page"
#elif L==3
	 "Tiempo de generaci&oacute;n de p&aacute;gina"
#elif L==4
	 "Temps pour g&eacute;n&eacute;rer la page"
#elif L==5
	 "Tiempo de generaci&oacute;n de p&aacute;gina"	// Okoteve traduccin
#elif L==6
	 "Tempo per generare pagina"
#elif L==7
	 "Time to generate page"			// Potrzebujesz tlumaczenie
#elif L==8
	 "Tempo para gerar p&aacute;gina"
#endif
	 ,
#if   L==0
	 "Tiempo de env&iacute;o de p&aacute;gina"	// Necessita traduccio
#elif L==1
	 "Zeit, um die Seite zu senden"
#elif L==2
	 "Time to send page"
#elif L==3
	 "Tiempo de env&iacute;o de p&aacute;gina"
#elif L==4
	 "Temps pour envoyer la page"
#elif L==5
	 "Tiempo de env&iacute;o de p&aacute;gina"	// Okoteve traduccin
#elif L==6
	 "Tempo d'invio della pagina"
#elif L==7
	 "Time to send page"				// Potrzebujesz tlumaczenie
#elif L==8
	 "Tempo para enviar a p&aacute;gina"
#endif
	};

const char *Txt_STAT_TYPE_COUNT_SMALL[Sta_NUM_STAT_COUNT_TYPES] =
   {
#if   L==0
	 "n&ordm; total de clics"	// Necessita traduccio
#elif L==1
	 "Gesamtzahl der Klicks"
#elif L==2
	 "total number of clicks"
#elif L==3
	 "n&ordm; total de clics"
#elif L==4
	 "nombre total de clics"
#elif L==5
	 "n&ordm; total de clics"	// Okoteve traduccin
#elif L==6
	 "numero totale di scatti"
#elif L==7
	 "total number of clicks"	// Potrzebujesz tlumaczenie
#elif L==8
	 "n&uacute;mero total de cliques"
#endif
	 ,
#if   L==0
	 "n&ordm; de usuarios distintos"		// Necessita traduccio
#elif L==1
	 "Anzahl der unterschiedliche Benutzer"
#elif L==2
	 "number of distinct users"
#elif L==3
	 "n&ordm; de usuarios distintos"
#elif L==4
	 "nombre des diff&eacute;rents utilisateurs"
#elif L==5
	 "n&ordm; de usuarios distintos"		// Okoteve traduccin
#elif L==6
	 "numero di utenti distinti"
#elif L==7
	 "number of distinct users"			// Potrzebujesz tlumaczenie
#elif L==8
	 "n&uacute;mero de utilizadores diferentes"
#endif
	 ,
#if   L==0
	 "n&ordm; de clics por usuario"		// Necessita traduccio
#elif L==1
	 "Anzahl der Klicks pro Benutzer"
#elif L==2
	 "number of clicks per user"
#elif L==3
	 "n&ordm; de clics por usuario"
#elif L==4
	 "nombre de clics par utilisateur"
#elif L==5
	 "n&ordm; de clics por usuario"		// Okoteve traduccin
#elif L==6
	 "numero di scatti per utente"
#elif L==7
	 "number of clicks per user"		// Potrzebujesz tlumaczenie
#elif L==8
	 "n&uacute;mero de cliques por utilizador"
#endif
	 ,
#if   L==0
	 "tiempo de generaci&oacute;n de p&aacute;gina"	// Necessita traduccio
#elif L==1
	 "Zeit zur Erzeugung der Seite"
#elif L==2
	 "time to generate page"
#elif L==3
	 "tiempo de generaci&oacute;n de p&aacute;gina"
#elif L==4
	 "temps pour g&eacute;n&eacute;rer la page"
#elif L==5
	 "tiempo de generaci&oacute;n de p&aacute;gina"	// Okoteve traduccin
#elif L==6
	 "tempo per generare pagina"
#elif L==7
	 "time to generate page"			// Potrzebujesz tlumaczenie
#elif L==8
	 "tempo para gerar p&aacute;gina"
#endif
	 ,
#if   L==0
	 "tiempo de env&iacute;o de p&aacute;gina"	// Necessita traduccio
#elif L==1
	 "Zeit, um Seite zu senden"
#elif L==2
	 "time to send page"
#elif L==3
	 "tiempo de env&iacute;o de p&aacute;gina"
#elif L==4
	 "temps pour envoyer la page"
#elif L==5
	 "tiempo de env&iacute;o de p&aacute;gina"	// Okoteve traduccin
#elif L==6
	 "tempo d'invio di una pagina"
#elif L==7
	 "time to send page"				// Potrzebujesz tlumaczenie
#elif L==8
	 "Tempo para enviar a p&aacute;gina"
#endif
	};

const char *Txt_STAT_USE_STAT_TYPES[Sta_NUM_TYPES_USE_STATS] =
   {
#if   L==0
	 "Titulaciones y asignaturas"	// Necessita traducci
#elif L==1
	 "Studieng&auml;nge und Kursen"
#elif L==2
	 "Degrees and courses"
#elif L==3
	 "Titulaciones y asignaturas"
#elif L==4
	 "&Eacute;tudes et mati&egrave;res"
#elif L==5
	 "Titulaciones y asignaturas"	// Okoteve traduccin
#elif L==6
	 "Lauree e corsi"
#elif L==7
	 "Degrees and courses"		// Potrzebujesz tlumaczenie
#elif L==8
	 "Titula&ccedil;oes e disciplinas"
#endif
	 ,
#if   L==0
	 "Usuaris"
#elif L==1
	 "Benutzer"
#elif L==2
	 "Users"
#elif L==3
	 "Usuarios"
#elif L==4
	 "Utilisateurs"
#elif L==5
	 "Puruh&aacute;ra"
#elif L==6
	 "Utenti"
#elif L==7
	 "U&zdot;ytkownicy"
#elif L==8
	 "Utilizadores"
#endif
	 ,
#if   L==0
	 "Webs / xarxes socials"
#elif L==1
	 "Webs / Social Networks"
#elif L==2
	 "Webs / social networks"
#elif L==3
	 "Webs / redes sociales"
#elif L==4
	 "Webs / r&eacute;seaux sociaux"
#elif L==5
	 "Webs / redes sociales"	// Okoteve traduccin
#elif L==6
	 "Webs / reti sociali"
#elif L==7
	 "Webs / sieci spo&lstrok;eczne"
#elif L==8
	 "Webs / redes sociais"
#endif
	 ,
#if   L==0
	 "Carpetes i fitxers"
#elif L==1
	 "Verzeichnisse und Dateien"
#elif L==2
	 "Folders and files"
#elif L==3
	 "Carpetas y archivos"
#elif L==4
	 "R&eacute;pertoires et fichiers"
#elif L==5
	 "Carpetas y archivos"		// Okoteve traduccin
#elif L==6
	 "Cartelle e file"
#elif L==7
	 "Folders and files"		// Potrzebujesz tlumaczenie
#elif L==8
	 "Diret&oacute;rios e arquivos"
#endif
	 ,
#if   L==0
	 "Recursos Educatius Oberts (OER)"
#elif L==1
	 "Open Educational Resources (OER)"
#elif L==2
	 "Open Educational Resources (OER)"
#elif L==3
	 "Recursos Educativos Abiertos (OER)"
#elif L==4
	 "Ressources &Eacute;ducatives Libres (REL)"
#elif L==5
	 "Recursos Educativos Abiertos (OER)"	// Okoteve traduccin
#elif L==6
	 "Risorse Didattiche Aperte (OER)"
#elif L==7
	 "Otwarte Zasoby Edukacyjne (OER)"
#elif L==8
	 "Recursos Educacionais Abertos (OER)"
#endif
	 ,
#if   L==0
	 "Activitats"
#elif L==1
	 "Aufgaben"
#elif L==2
	 "Assignments"
#elif L==3
	 "Actividades"
#elif L==4
	 "Activit&eacute;s"
#elif L==5
	 "Tar&eacute;a"
#elif L==6
	 "Attivit&agrave;"
#elif L==7
	 "Assignments"	// Potrzebujesz tlumaczenie
#elif L==8
	 "Atividades"
#endif
	 ,
#if   L==0
	 "Tests"
#elif L==1
	 "Tests"
#elif L==2
	 "Tests"
#elif L==3
	 "Tests"
#elif L==4
	 "Tests"
#elif L==5
	 "Tests"		// Okoteve traduccin
#elif L==6
	 "Test"
#elif L==7
	 "Tests"		// Potrzebujesz tlumaczenie
#elif L==8
	 "Testes"
#endif
	 ,
#if   L==0
	 "Notificacions"
#elif L==1
	 "Anmeldungen"
#elif L==2
	 "Notifications"
#elif L==3
	 "Notificaciones"
#elif L==4
	 "Notifications"
#elif L==5
	 "Momarandu"
#elif L==6
	 "Notifiche"
#elif L==7
	 "Powiadomienia"
#elif L==8
	 "Notifica&ccedil;&otilde;es"
#endif
	 ,
#if   L==0
	 "Avisos"
#elif L==1
	 "Ank&uuml;ndigungen"
#elif L==2
	 "Notices"
#elif L==3
	 "Avisos"
#elif L==4
	 "Avis"
#elif L==5
	 "Marandu"
#elif L==6
	 "Avvisi"
#elif L==7
	 "Notices"	// Potrzebujesz tlumaczenie
#elif L==8
	 "Avisos"
#endif
	 ,
#if   L==0
	 "Missatges"
#elif L==1
	 "Nachrichten"
#elif L==2
	 "Messages"
#elif L==3
	 "Mensajes"
#elif L==4
	 "Messages"
#elif L==5
	 "Marandu"
#elif L==6
	 "Messaggi"
#elif L==7
	 "Wiadomo&sacute;ci"
#elif L==8
	 "Mensagens"
#endif
	 ,
#if   L==0
	 "F&ograve;rums"
#elif L==1
	 "Foren"
#elif L==2
	 "Forums"
#elif L==3
	 "Foros"
#elif L==4
	 "Forums"
#elif L==5
	 "Foros"		// Okoteve traduccin
#elif L==6
	 "Forum"
#elif L==7
	 "Forums"		// Potrzebujesz tlumaczenie
#elif L==8
	 "F&oacute;runs"
#endif
	 ,
#if   L==0
	 "Enquestes"
#elif L==1
	 "Umfragen"
#elif L==2
	 "Surveys"
#elif L==3
	 "Encuestas"
#elif L==4
	 "Sondages"
#elif L==5
	 "Encuestas"	// Okoteve traduccin
#elif L==6
	 "Sondaggi"
#elif L==7
	 "Badania"
#elif L==8
	 "Inqu&eacute;ritos"
#endif
	 ,
#if   L==0
	 "Idioma"
#elif L==1
	 "Sprache"
#elif L==2
	 "Language"
#elif L==3
	 "Idioma"
#elif L==4
	 "Langue"
#elif L==5
	 "&Ntilde;e'&#7869;"
#elif L==6
	 "Lingua"
#elif L==7
	 "Language"	// Potrzebujesz tlumaczenie
#elif L==8
	 "Lingua"
#endif
	 ,
#if   L==0
	 "Disseny"
#elif L==1
	 "layout"
#elif L==2
	 "Layout"
#elif L==3
	 "Dise&ntilde;o"
#elif L==4
	 "Disposition"
#elif L==5
	 "Ta'&atilde;ngahai"
#elif L==6
	 "Layout"
#elif L==7
	 "Uk&lstrok;ad"
#elif L==8
	 "Layout"
#endif
	 ,
#if   L==0
	 "Tema (colors)"
#elif L==1
	 "Skin"
#elif L==2
	 "Theme"
#elif L==3
	 "Tema (colores)"
#elif L==4
	 "Th&egrave;me"
#elif L==5
	 "Tema (colores)"	// Okoteve traduccin
#elif L==6
	 "Tema"
#elif L==7
	 "Temat"
#elif L==8
	 "Skin"
#endif
	 ,
#if   L==0
	 "Icones"
#elif L==1
	 "Symbole"
#elif L==2
	 "Icons"
#elif L==3
	 "Iconos"
#elif L==4
	 "Ic&ocirc;nes"
#elif L==5
	 "Iconos"	// Okoteve traduccin
#elif L==6
	 "Icone"
#elif L==7
	 "Ikony"
#elif L==8
	 "&Iacute;cones"
#endif
	 ,
#if   L==0
	"Men&uacute;"
#elif L==1
	"Men&uuml;"
#elif L==2
	"Menu"
#elif L==3
	"Men&uacute;"
#elif L==4
	"Menu"
#elif L==5
	"Men&uacute;"	// Okoteve traduccin
#elif L==6
	"Menu"
#elif L==7
	"Menu"
#elif L==8
	"Menu"
#endif
	 ,
#if   L==0
	 "Columnes"
#elif L==1
	 "Spalten"
#elif L==2
	 "Columns"
#elif L==3
	 "Columnas"
#elif L==4
	 "Colonnes"
#elif L==5
	 "Columnas"	// Okoteve traduccin
#elif L==6
	 "Colonne"
#elif L==7
	 "Kolumny"
#elif L==8
	 "Colunas"
#endif
	};

const char *Txt_Statistic =
#if   L==0
	 "Estad&iacute;stique";
#elif L==1
	 "Statistik";
#elif L==2
	 "Statistic";
#elif L==3
	 "Estad&iacute;stica";
#elif L==4
	 "Statistique";
#elif L==5
	 "Papy'aporeko";
#elif L==6
	 "Statistica";
#elif L==7
	 "Statystyki";
#elif L==8
	 "Estat&iacute;sticas";
#endif

const char *Txt_Statistics_of_all_visits =
#if   L==0
	"Estad&iacute;stica de todos los accesos";	// Necessita traduccio
#elif L==1
	"Statistics of all visits";			// Need bersetzung
#elif L==2
	"Statistics of all visits";
#elif L==3
	"Estad&iacute;stica de todos los accesos";
#elif L==4
	"Statistics of all visits";			// Besoin de traduction
#elif L==5
	"Estad&iacute;stica de todos los accesos";	// Okoteve traduccin
#elif L==6
	"Statistiche di tutte gli accessi";
#elif L==7
	"Statistics of all visits";			// Potrzebujesz tlumaczenie
#elif L==8
	"Statistics of all visits";			// Necessita de traduo
#endif

const char *Txt_Statistics_of_visits_to_the_centre_X =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"Estad&iacute;stica de accesos al centro <strong>%s</strong>";	// Necessita traduccio
#elif L==1
	"Statistics of visits to the centre <strong>%s</strong>";	// Need bersetzung
#elif L==2
	"Statistics of visits to the centre <strong>%s</strong>";
#elif L==3
	"Estad&iacute;stica de accesos al centro <strong>%s</strong>";
#elif L==4
	"Statistics of visits to the centre <strong>%s</strong>";	// Besoin de traduction
#elif L==5
	"Estad&iacute;stica de accesos al centro <strong>%s</strong>";	// Okoteve traduccin
#elif L==6
	"Statistiche degli accessi al centro <strong>%s</strong>";
#elif L==7
	"Statistics of visits to the centre <strong>%s</strong>";	// Potrzebujesz tlumaczenie
#elif L==8
	"Statistics of visits to the centre <strong>%s</strong>";	// Necessita de traduo
#endif

const char *Txt_Statistics_of_visits_to_the_course_X =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"Estad&iacute;stica d'accessos a l'assignatura <strong>%s</strong>";
#elif L==1
	"Statistics of visits to the course <strong>%s</strong>";		// Need bersetzung
#elif L==2
	"Statistics of visits to the course <strong>%s</strong>";
#elif L==3
	"Estad&iacute;stica de accesos a la asignatura <strong>%s</strong>";
#elif L==4
	"Statistics of visits to the course <strong>%s</strong>";		// Besoin de traduction
#elif L==5
	"Estad&iacute;stica de accesos a la asignatura <strong>%s</strong>";	// Okoteve traduccin
#elif L==6
	"Statistiche degli accessi al corso <strong>%s</strong>";
#elif L==7
	"Statistics of visits to the course <strong>%s</strong>";		// Potrzebujesz tlumaczenie
#elif L==8
	"Statistics of visits to the course <strong>%s</strong>";		// Necessita de traduo
#endif

const char *Txt_Statistics_of_visits_to_the_degree_X =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"Estad&iacute;stica de accesos a la titulaci&oacute;n <strong>%s</strong>";	// Necessita traduccio
#elif L==1
	"Statistics of visits to the degree <strong>%s</strong>";			// Need bersetzung
#elif L==2
	"Statistics of visits to the degree <strong>%s</strong>";
#elif L==3
	"Estad&iacute;stica de accesos a la titulaci&oacute;n <strong>%s</strong>";
#elif L==4
	"Statistics of visits to the degree <strong>%s</strong>";			// Besoin de traduction
#elif L==5
	"Estad&iacute;stica de accesos a la titulaci&oacute;n <strong>%s</strong>";	// Okoteve traduccin
#elif L==6
	"Statistiche degli accessi alla laurea <strong>%s</strong>";
#elif L==7
	"Statistics of visits to the degree <strong>%s</strong>";			// Potrzebujesz tlumaczenie
#elif L==8
	"Statistics of visits to the degree <strong>%s</strong>";			// Necessita de traduo
#endif

const char *Txt_Statistics_of_visits_to_the_institution_X =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"Estad&iacute;stica de accesos a la instituci&oacute;n <strong>%s</strong>";	// Necessita traduccio
#elif L==1
	"Statistics of visits to the institution <strong>%s</strong>";			// Need bersetzung
#elif L==2
	"Statistics of visits to the institution <strong>%s</strong>";
#elif L==3
	"Estad&iacute;stica de accesos a la instituci&oacute;n <strong>%s</strong>";
#elif L==4
	"Statistics of visits to the institution <strong>%s</strong>";			// Besoin de traduction
#elif L==5
	"Estad&iacute;stica de accesos a la instituci&oacute;n <strong>%s</strong>";	// Okoteve traduccin
#elif L==6
	"Statistiche degli accessi alla istituzione <strong>%s</strong>";
#elif L==7
	"Statistics of visits to the institution <strong>%s</strong>";			// Potrzebujesz tlumaczenie
#elif L==8
	"Statistics of visits to the institution <strong>%s</strong>";			// Necessita de traduo
#endif

const char *Txt_Status =
#if   L==0
	"Estado";	// Necessita traduccio
#elif L==1
	"Status";
#elif L==2
	"Status";
#elif L==3
	"Estado";
#elif L==4
	"Statut";
#elif L==5
	"O&#297; h&aacute;icha";
#elif L==6
	"Stato";
#elif L==7
	"Status";	// Potrzebujesz tlumaczenie
#elif L==8
	"Status";
#endif

const char *Txt_Stay_connected_with_SWADroid =
#if   L==0
	"Seguiu connectat/da amb <strong>SWADroid</strong>.";
#elif L==1
	"Stay connected with <strong>SWADroid</strong>.";		// Need bersetzung
#elif L==2
	"Stay connected with <strong>SWADroid</strong>.";
#elif L==3
	"Siga conectado/a con <strong>SWADroid</strong>.";
#elif L==4
	"Stay connected with <strong>SWADroid</strong>.";		// Besoin de traduction
#elif L==5
	"Siga conectado/a con <strong>SWADroid</strong>.";		// Okoteve traduccin
#elif L==6
	"Resta in contatto con <strong>SWADroid</strong>.";
#elif L==7
	"Stay connected with <strong>SWADroid</strong>.";		// Potrzebujesz tlumaczenie
#elif L==8
	"Stay connected with <strong>SWADroid</strong>.";		// Necessita de traduo
#endif

const char *Txt_Stem =		// Stem of a test question
#if   L==0
	"Enunciat";
#elif L==1
	"Stamm";
#elif L==2
	"Stem";
#elif L==3
	"Enunciado";
#elif L==4
	"Question";
#elif L==5
	"Enunciado";	// Okoteve traduccin
#elif L==6
	"Domanda";
#elif L==7
	"Stem";		// Potrzebujesz tlumaczenie
#elif L==8
	"Quest&atilde;o";
#endif

const char *Txt_Step_1_Provide_a_list_of_users =
#if   L==0
	"Paso 1: Proporcione una lista de usuarios";	// Necessita traduccio
#elif L==1
	"Step 1: Provide a list of users";		// Need bersetzung
#elif L==2
	"Step 1: Provide a list of users";
#elif L==3
	"Paso 1: Proporcione una lista de usuarios";
#elif L==4
	"Step 1: Provide a list of users";		// Besoin de traduction
#elif L==5
	"Paso 1: Proporcione una lista de usuarios";	// Okoteve traduccin
#elif L==6
	"Passo 1: Fornisci una lista di utenti";
#elif L==7
	"Step 1: Provide a list of users";		// Potrzebujesz tlumaczenie
#elif L==8
	"Step 1: Provide a list of users";		// Necessita de traduo
#endif

const char *Txt_Step_2_Select_the_type_of_user_to_register_remove =
#if   L==0
	"Paso 2: Seleccione el tipo de usuario a inscribir / eliminar";	// Necessita traduccio
#elif L==1
	"Step 2: Select the type of user to register / remove";		// Need bersetzung
#elif L==2
	"Step 2: Select the type of user to register / remove";
#elif L==3
	"Paso 2: Seleccione el tipo de usuario a inscribir / eliminar";
#elif L==4
	"Step 2: Select the type of user to register / remove";		// Besoin de traduction
#elif L==5
	"Paso 2: Seleccione el tipo de usuario a inscribir / eliminar";	// Okoteve traduccin
#elif L==6
	"Passo 2: Seleziona il tipo di utente da registrare / rimuovere";
#elif L==7
	"Step 2: Select the type of user to register / remove";		// Potrzebujesz tlumaczenie
#elif L==8
	"Step 2: Select the type of user to register / remove";		// Necessita de traduo
#endif

const char *Txt_Step_3_Select_the_desired_action =
#if   L==0
	"Paso 3: Seleccione la acci&oacute;n que desea realizar";	// Necessita traduccio
#elif L==1
	"Step 3: Select the desired action";				// Need bersetzung
#elif L==2
	"Step 3: Select the desired action";
#elif L==3
	"Paso 3: Seleccione la acci&oacute;n que desea realizar";
#elif L==4
	"Step 3: Select the desired action";				// Besoin de traduction
#elif L==5
	"Paso 3: Seleccione la acci&oacute;n que desea realizar";	// Okoteve traduccin
#elif L==6
	"Passo 3: Seleziona l'azione desiderata";
#elif L==7
	"Step 3: Select the desired action";				// Potrzebujesz tlumaczenie
#elif L==8
	"Step 3: Select the desired action";				// Necessita de traduo
#endif

const char *Txt_Step_4_Optionally_select_groups =
#if   L==0
	"Paso 4: Opcionalmente, seleccione grupos";	// Necessita traduccio
#elif L==1
	"Step 4: Optionally, select groups";		// Need bersetzung
#elif L==2
	"Step 4: Optionally, select groups";
#elif L==3
	"Paso 4: Opcionalmente, seleccione grupos";
#elif L==4
	"Step 4: Optionally, select groups";		// Besoin de traduction
#elif L==5
	"Paso 4: Opcionalmente, seleccione grupos";	// Okoteve traduccin
#elif L==6
	"Passo 4: A scelta, seleziona i gruppi";
#elif L==7
	"Step 4: Optionally, select groups";		// Potrzebujesz tlumaczenie
#elif L==8
	"Step 4: Optionally, select groups";		// Necessita de traduo
#endif

const char *Txt_Step_5_Confirm_the_enrollment_removing =
#if   L==0
	"Paso 5: Confirme la inscripci&oacute;n / eliminaci&oacute;n";	// Necessita traduccio
#elif L==1
	"Step 5: Confirm the enrollment / removing";			// Need bersetzung
#elif L==2
	"Step 5: Confirm the enrollment / removing";
#elif L==3
	"Paso 5: Confirme la inscripci&oacute;n / eliminaci&oacute;n";
#elif L==4
	"Step 5: Confirm the enrollment / removing";			// Besoin de traduction
#elif L==5
	"Paso 5: Confirme la inscripci&oacute;n / eliminaci&oacute;n";	// Okoteve traduccin
#elif L==6
	"Passo 5: Conferma la registrazione / rimozione";
#elif L==7
	"Step 5: Confirm the enrollment / removing";			// Potrzebujesz tlumaczenie
#elif L==8
	"Step 5: Confirm the enrollment / removing";			// Necessita de traduo
#endif

const char *Txt_Student_comment =
#if   L==0
	"Comentari de l'estudiant";
#elif L==1
	"Student-Kommentar";
#elif L==2
	"Student's comment";
#elif L==3
	"Comentario del estudiante";
#elif L==4
	"Commentaire de l'&eacute;tudiant";
#elif L==5
	"Comentario del estudiante";	// Okoteve traduccin
#elif L==6
	"Commento di student";
#elif L==7
	"Komentarz studenta";
#elif L==8
	"Coment&aacute;rio do estudante";
#endif

const char *Txt_Students_ABBREVIATION =	// Abbreviation of "Students"
#if   L==0
	"Estud.";
#elif L==1
	"Stud.";
#elif L==2
	"Stud.";
#elif L==3
	"Estud.";
#elif L==4
	"&Eacute;tud.";
#elif L==5
	"Temi.";
#elif L==6
	"Stud.";
#elif L==7
	"Stud.";	// Potrzebujesz tlumaczenie
#elif L==8
	"Estu.";
#endif

const char *Txt_students_ABBREVIATION =	// Abbreviation of "students"
#if   L==0
	"estud.";
#elif L==1
	"Stud.";
#elif L==2
	"stud.";
#elif L==3
	"estud.";
#elif L==4
	"&eacute;tud.";
#elif L==5
	"Temi.";
#elif L==6
	"stud.";
#elif L==7
	"stud.";	// Potrzebujesz tlumaczenie
#elif L==8
	"estu.";
#endif

const char *Txt_Students_authentication =
#if   L==0
	"Autenticaci&oacute;<br />d'estudiants";
#elif L==1
	"Studenten-<br />Authentifizierung";
#elif L==2
	"Students'<br />authentication";
#elif L==3
	"Autenticaci&oacute;n<br />de estudiantes";
#elif L==4
	"Authentification des &eacute;tudiants";
#elif L==5
	"Autenticaci&oacute;n<br />de estudiantes";	// Okoteve traduccin
#elif L==6
	"Autenticazione degli studenti";
#elif L==7
	"Uwierzytelnianie student&oacute;w";
#elif L==8
	"Autentica&ccedil;&atilde;o de estudantes";
#endif

const char *Txt_STUDENTS_may_enter_directly =
#if   L==0
	"poden entrar directament";
#elif L==1
	"k&ouml;nnen direkt eingeben";
#elif L==2
	"may enter directly";
#elif L==3
	"pueden entrar directamente";
#elif L==4
	"peuvent entrer directement";
#elif L==5
	"pueden entrar directamente";	// Okoteve traduccin
#elif L==6
	"possono entrare direttamente";
#elif L==7
	"moga wej&sacute;&cacute; bezpo&sacute;rednio";
#elif L==8
	"podem entrar diretamente";
#endif

const char *Txt_STUDENTS_must_enter_through_X =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"han d'entrar per %s";
#elif L==1
	"m&uuml;ssen durch %s eingeben";
#elif L==2
	"must enter through %s";
#elif L==3
	"deben entrar por %s";
#elif L==4
	"doivent entrer par %s";
#elif L==5
	"deben entrar por %s";	// Okoteve traduccin
#elif L==6
	"devono entrare attraverso %s";
#elif L==7
	"musz&aogon; wej&sacute;&cacute; przez %s";
#elif L==8
	"devem entrar atrav&eacute;s %s";
#endif

const char *Txt_Student_record_card_in_this_course_has_been_updated =
#if   L==0
	"La ficha del estudiante en la asignatura se ha actualizado.";	// Necessita traduccio
#elif L==1
	"Student record card in this course has been updated.";		// Need bersetzung
#elif L==2
	"Student record card in this course has been updated.";
#elif L==3
	"La ficha del estudiante en la asignatura se ha actualizado.";
#elif L==4
	"Student record card in this course has been updated.";		// Besoin de traduction
#elif L==5
	"La ficha del estudiante en la asignatura se ha actualizado.";	// Okoteve traduccin
#elif L==6
	"La scheda dello/a studente/ssa in questo corso &egrave; stata aggiornata.";
#elif L==7
	"Student record card in this course has been updated.";		// Potrzebujesz tlumaczenie
#elif L==8
	"Student record card in this course has been updated.";		// Necessita de traduo
#endif

const char *Txt_Students_are_no_longer_obliged_to_read_this_information =
#if   L==0
	"Los estudiantes ya no est&aacute;n obligados a leer esta informaci&oacute;n.";	// Necessita traduccio
#elif L==1
	"Students are no longer obliged to read this information.";			// Need bersetzung
#elif L==2
	"Students are no longer obliged to read this information.";
#elif L==3
	"Los estudiantes ya no est&aacute;n obligados a leer esta informaci&oacute;n.";
#elif L==4
	"Students are no longer obliged to read this information.";			// Besoin de traduction
#elif L==5
	"Los estudiantes ya no est&aacute;n obligados a leer esta informaci&oacute;n.";	// Okoteve traduccin
#elif L==6
	"Gli studenti non sono obbligati a leggere questa informazione.";
#elif L==7
	"Students are no longer obliged to read this information.";			// Potrzebujesz tlumaczenie
#elif L==8
	"Students are no longer obliged to read this information.";			// Necessita de traduo
#endif

const char *Txt_Students_now_are_required_to_read_this_information =
#if   L==0
	"Los estudiantes ahora est&aacute;n obligados a leer esta informaci&oacute;n.";	// Necessita traduccio
#elif L==1
	"Students now are required to read this information.";				// Need bersetzung
#elif L==2
	"Students now are required to read this information.";
#elif L==3
	"Los estudiantes ahora est&aacute;n obligados a leer esta informaci&oacute;n.";
#elif L==4
	"Students now are required to read this information.";				// Besoin de traduction
#elif L==5
	"Los estudiantes ahora est&aacute;n obligados a leer esta informaci&oacute;n.";	// Okoteve traduccin
#elif L==6
	"Gli studenti ora sono obbligati a leggere questa informazione.";
#elif L==7
	"Students now are required to read this information.";				// Potrzebujesz tlumaczenie
#elif L==8
	"Students now are required to read this information.";				// Necessita de traduo
#endif

const char *Txt_X_students_who_have_e_mail =	// Warning: it is very important to include %u and %f in the following sentences
#if   L==0
	"%u estudiantes con correo electr&oacute;nico (%.1f%% de %u estudiantes)";	// Necessita traduccio
#elif L==1
	"%u Studenten mit E-Mail (%.1f%% von %u Studenten)";
#elif L==2
	"%u students who have e-mail (%.1f%% of %u students)";
#elif L==3
	"%u estudiantes con correo electr&oacute;nico (%.1f%% de %u estudiantes)";
#elif L==4
	"%u &eacute;tudiants qui ont l'e-mail (%.1f%% de %u &eacute;tudiants)";
#elif L==5
	"%u estudiantes con correo electr&oacute;nico (%.1f%% de %u estudiantes)";	// Okoteve traduccin
#elif L==6
	"%u studenti che hanno e-mail (%.1f%% di %u studenti)";
#elif L==7
	"%u students who have e-mail (%.1f%% of %u students)";				// Potrzebujesz tlumaczenie
#elif L==8
	"%u estudantes que t&ecirc;m e-mail (%.1f%% dos %u estudantes)";
#endif

const char *Txt_Students_who_have_accepted_and_who_have_e_mail =
#if   L==0
	"Estudiantes que han aceptado la inscripci&oacute;n"
	" y que tienen correo electr&oacute;nico";	// Necessita traduccio
#elif L==1
	"Studenten die akzeptiert wurden"
	" und die E-Mail";
#elif L==2
	"Students who have accepted"
	" and who have e-mail";
#elif L==3
	"Estudiantes que han aceptado la inscripci&oacute;n"
	" y que tienen correo electr&oacute;nico";
#elif L==4
	"&Eacute;tudiants qui ont accept&eacute;"
	" et qui ont l'e-mail";
#elif L==5
	"Estudiantes que han aceptado la inscripci&oacute;n"
	" y que tienen correo electr&oacute;nico";	// Okoteve traduccin
#elif L==6
	"Studenti che hanno aderito"
	" e che hanno e-mail";
#elif L==7
	"Students who have accepted"
	" and who have e-mail";				// Potrzebujesz tlumaczenie
#elif L==8
	"Estudantes que aceitaram"
	" e que t&ecirc;m e-mail";
#endif

const char *Txt_X_students_who_have_accepted_and_who_have_e_mail =	// Warning: it is very important to include %u and %f in the following sentences
#if   L==0
	"%u estudiantes que han aceptado la inscripci&oacute;n"
	" y que tienen correo electr&oacute;nico (%.1f%% de %u estudiantes)";	// Necessita traduccio
#elif L==1
	"%u Studenten die akzeptiert wurden"
	" und die E-Mail (%.1f%% von %u Studenten)";
#elif L==2
	"%u students who have accepted"
	" and who have e-mail (%.1f%% of %u students)";
#elif L==3
	"%u estudiantes que han aceptado la inscripci&oacute;n"
	" y que tienen correo electr&oacute;nico (%.1f%% de %u estudiantes)";
#elif L==4
	"%u &eacute;tudiants qui ont accept&eacute;"
	" et qui ont l'e-mail (%.1f%% de %u &eacute;tudiants)";
#elif L==5
	"%u estudiantes que han aceptado la inscripci&oacute;n"
	" y que tienen correo electr&oacute;nico (%.1f%% de %u estudiantes)";	// Okoteve traduccin
#elif L==6
	"%u studenti che hanno aderito"
	" e che hanno e-mail (%.1f%% di %u studenti)";
#elif L==7
	"%u students who have accepted"
	" and who have e-mail (%.1f%% of %u students)";				// Potrzebujesz tlumaczenie
#elif L==8
	"%u estudantes que aceitaram"
	" e que t&ecirc;m e-mail (%.1f%% dos %u estudantes)";
#endif

const char *Txt_students_with_no_group =
#if   L==0
	"estudiantes en ning&uacute;n grupo";	// Necessita traduccio
#elif L==1
	"Studente mit nicht-Gruppe";
#elif L==2
	"students with no group";
#elif L==3
	"estudiantes en ning&uacute;n grupo";
#elif L==4
	"&eacute;tudiants avec aucun groupe";
#elif L==5
	"estudiantes en ning&uacute;n grupo";	// Okoteve traduccin
#elif L==6
	"studenti con nessun gruppo";
#elif L==7
	"students with no group";		// Potrzebujesz tlumaczenie
#elif L==8
	"estudantes com nenhum grupo";
#endif

const char *Txt_Surname_1 =
#if   L==0
	"Primer cognom";
#elif L==1
	"Nachname 1";
#elif L==2
	"Surname 1";
#elif L==3
	"Primer apellido";
#elif L==4
	"Nom de famille 1";
#elif L==5
	"Terajoapy 1";
#elif L==6
	"Primo cognome";
#elif L==7
	"Nazwisko 1";
#elif L==8
	"Primeiro apelido";
#endif

const char *Txt_Surname_2 =
#if   L==0
	"Segon cognom";
#elif L==1
	"Nachname 2";
#elif L==2
	"Surname 2";
#elif L==3
	"Segundo apellido";
#elif L==4
	"Nom de famille 2";
#elif L==5
	"Terajoapy 2";
#elif L==6
	"Secondo cognome";
#elif L==7
	"Nazwisko 2";
#elif L==8
	"Segundo apelido";
#endif

const char *Txt_Survey =
#if   L==0
	"Encuesta";	// Necessita traduccio
#elif L==1
	"Umfrage";
#elif L==2
	"Survey";
#elif L==3
	"Encuesta";
#elif L==4
	"Sondage";
#elif L==5
	"Encuesta";	// Okoteve traduccin
#elif L==6
	"Sondaggio";
#elif L==7
	"Survey";	// Potrzebujesz tlumaczenie
#elif L==8
	"Inqu&eacute;rito";
#endif

const char *Txt_SURVEY_You_belong_to_degree_coruse_or_groups =
#if   L==0
	"Pertenece a titulaci&oacute;n, asignatura o grupo(s)";	// Necessita traduccio
#elif L==1
	"You belong to degree, course or group(s)";		// Need bersetzung
#elif L==2
	"You belong to degree, course or group(s)";
#elif L==3
	"Pertenece a titulaci&oacute;n, asignatura o grupo(s)";
#elif L==4
	"You belong to degree, course or group(s)";		// Besoin de traduction
#elif L==5
	"Pertenece a titulaci&oacute;n, asignatura o grupo(s)";	// Okoteve traduccin
#elif L==6
	"Appartieni alla laurea, corso o gruppo";
#elif L==7
	"You belong to degree, course or group(s)";		// Potrzebujesz tlumaczenie
#elif L==8
	"You belong to degree, course or group(s)";		// Necessita de traduo
#endif

const char *Txt_SURVEY_You_dont_belong_to_degree_coruse_or_groups =
#if   L==0
	"No pertenece a titulaci&oacute;n, asignatura o grupo(s)";	// Necessita traduccio
#elif L==1
	"You don't belong to degree, course or group(s)";		// Need bersetzung
#elif L==2
	"You don't belong to degree, course or group(s)";
#elif L==3
	"No pertenece a titulaci&oacute;n, asignatura o grupo(s)";
#elif L==4
	"You don't belong to degree, course or group(s)";		// Besoin de traduction
#elif L==5
	"No pertenece a titulaci&oacute;n, asignatura o grupo(s)";	// Okoteve traduccin
#elif L==6
	"Non appartieni alla laurea, corso o gruppo(s)";
#elif L==7
	"You don't belong to degree, course or group(s)";		// Potrzebujesz tlumaczenie
#elif L==8
	"You don't belong to degree, course or group(s)";		// Necessita de traduo
#endif

const char *Txt_SURVEY_You_have_already_answered =
#if   L==0
	"Ya ha respondido";		// Necessita traduccio
#elif L==1
	"You have already answered";	// Need bersetzung
#elif L==2
	"You have already answered";
#elif L==3
	"Ya ha respondido";
#elif L==4
	"You have already answered";	// Besoin de traduction
#elif L==5
	"Ya ha respondido";		// Okoteve traduccin
#elif L==6
	"Hai gi&agrave; risposto";
#elif L==7
	"You have already answered";	// Potrzebujesz tlumaczenie
#elif L==8
	"You have already answered";	// Necessita de traduo
#endif

const char *Txt_SURVEY_You_have_not_answered =
#if   L==0
	"No ha respondido";		// Necessita traduccio
#elif L==1
	"You have not answered";	// Need bersetzung
#elif L==2
	"You have not answered";
#elif L==3
	"No ha respondido";
#elif L==4
	"You have not answered";	// Besoin de traduction
#elif L==5
	"No ha respondido";		// Okoteve traduccin
#elif L==6
	"Non hai risposto";
#elif L==7
	"You have not answered";	// Potrzebujesz tlumaczenie
#elif L==8
	"You have not answered";	// Necessita de traduo
#endif

const char *Txt_SURVEY_Type_of_user_allowed =
#if   L==0
	"Tipo de usuario permitido";	// Necessita traduccio
#elif L==1
	"Type of user allowed";		// Need bersetzung
#elif L==2
	"Type of user allowed";
#elif L==3
	"Tipo de usuario permitido";
#elif L==4
	"Type of user allowed";		// Besoin de traduction
#elif L==5
	"Tipo de usuario permitido";	// Okoteve traduccin
#elif L==6
	"Tipo di utente autorizzato";
#elif L==7
	"Type of user allowed";		// Potrzebujesz tlumaczenie
#elif L==8
	"Type of user allowed";		// Necessita de traduo
#endif

const char *Txt_SURVEY_Type_of_user_not_allowed =
#if   L==0
	"Tipo de usuario no permitido";		// Necessita traduccio
#elif L==1
	"Type of user not allowed";		// Need bersetzung
#elif L==2
	"Type of user not allowed";
#elif L==3
	"Tipo de usuario no permitido";
#elif L==4
	"Type of user not allowed";		// Besoin de traduction
#elif L==5
	"Tipo de usuario no permitido";		// Okoteve traduccin
#elif L==6
	"Tipo di utente non autorizzato";
#elif L==7
	"Type of user not allowed";		// Potrzebujesz tlumaczenie
#elif L==8
	"Type of user not allowed";		// Necessita de traduo
#endif

const char *Txt_Survey_X_is_now_hidden =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"La encuesta <strong>%s</strong> ahora est&aacute; oculta.";	// Necessita traduccio
#elif L==1
	"Die Umfrage <strong>%s</strong> ist jetzt ausgeblendet.";
#elif L==2
	"Survey <strong>%s</strong> is now hidden.";
#elif L==3
	"La encuesta <strong>%s</strong> ahora est&aacute; oculta.";
#elif L==4
	"Le sondage <strong>%s</strong> est maintenant cach&eacute;.";
#elif L==5
	"La encuesta <strong>%s</strong> ahora est&aacute; oculta.";	// Okoteve traduccin
#elif L==6
	"Il sondaggio <strong>%s</strong> &egrave; ora nascosto.";
#elif L==7
	"Survey <strong>%s</strong> is now hidden.";			// Potrzebujesz tlumaczenie
#elif L==8
	"O inqu&eacute;rito <strong>%s</strong> &eacute; agora oculta.";
#endif

const char *Txt_Survey_X_is_now_visible =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"La encuesta <strong>%s</strong> ahora est&aacute; visible.";	// Necessita traduccio
#elif L==1
	"Die Umfrage <strong>%s</strong> ist jetzt sichtbar.";
#elif L==2
	"Survey <strong>%s</strong> is now visible.";
#elif L==3
	"La encuesta <strong>%s</strong> ahora est&aacute; visible.";
#elif L==4
	"Le sondage <strong>%s</strong> est maintenant visible.";
#elif L==5
	"La encuesta <strong>%s</strong> ahora est&aacute; visible.";	// Okoteve traduccin
#elif L==6
	"Il sondaggio <strong>%s</strong> &egrave; ora visibile.";
#elif L==7
	"Survey <strong>%s</strong> is now visible.";			// Potrzebujesz tlumaczenie
#elif L==8
	"O inqu&eacute;rito <strong>%s</strong> &eacute; agora vis&iacute;vel.";
#endif

const char *Txt_Survey_questions =
#if   L==0
	"Preguntas de la encuesta";	// Necessita traduccio
#elif L==1
	"Umfrage-Fragen";
#elif L==2
	"Survey questions";
#elif L==3
	"Preguntas de la encuesta";
#elif L==4
	"Questions du sondage";
#elif L==5
	"Preguntas de la encuesta";	// Okoteve traduccin
#elif L==6
	"Domande di sondaggio";
#elif L==7
	"Survey questions";		// Potrzebujesz tlumaczenie
#elif L==8
	"Quest&otilde;es do inqu&eacute;rito";
#endif

const char *Txt_Survey_X_removed =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"Encuesta <strong>%s</strong> eliminada.";	// Necessita traduccio
#elif L==1
	"Umfrage <strong>%s</strong> entfernt.";
#elif L==2
	"Survey <strong>%s</strong> removed.";
#elif L==3
	"Encuesta <strong>%s</strong> eliminada.";
#elif L==4
	"Sondage <strong>%s</strong> supprim&eacute;.";
#elif L==5
	"Encuesta <strong>%s</strong> eliminada.";	// Okoteve traduccin
#elif L==6
	"Sondaggio <strong>%s</strong> rimosso.";
#elif L==7
	"Survey <strong>%s</strong> removed.";		// Potrzebujesz tlumaczenie
#elif L==8
	"Inqu&eacute;rito <strong>%s</strong> removido.";
#endif

const char *Txt_Survey_X_reset =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"Encuesta <strong>%s</strong> puesta a cero.";	// Necessita traduccio
#elif L==1
	"Umfrage <strong>%s</strong> zur&uuml;ckgesetzt.";
#elif L==2
	"Survey <strong>%s</strong> reset.";
#elif L==3
	"Encuesta <strong>%s</strong> puesta a cero.";
#elif L==4
	"Sondage <strong>%s</strong> remis &agrave; z&eacute;ro.";
#elif L==5
	"Encuesta <strong>%s</strong> puesta a cero.";	// Okoteve traduccin
#elif L==6
	"Sondaggio <strong>%s</strong> resettato.";
#elif L==7
	"Survey <strong>%s</strong> reset.";		// Potrzebujesz tlumaczenie
#elif L==8
	"Inqu&eacute;rito <strong>%s</strong> fixado em zero.";
#endif

const char *Txt_SURVEY_STR_ANSWER_TYPES[Svy_NUM_ANS_TYPES] =
   {
#if   L==0
	 "Elecci&oacute; &uacute;nica"
#elif L==1
	 "Unique-Choice"
#elif L==2
	 "Unique choice"
#elif L==3
	 "Elecci&oacute;n &uacute;nica"
#elif L==4
	 "Choix unique"
#elif L==5
	 "Elecci&oacute;n &uacute;nica"		// Okoteve traduccin
#elif L==6
	 "Scelta unica"
#elif L==7
	 "Unique choice"			// Potrzebujesz tlumaczenie
#elif L==8
	 "&Uacute;nica escolha"
#endif
	 ,
#if   L==0
	 "Elecci&oacute; m&uacute;ltiple"
#elif L==1
	 "Multiple-Choice"
#elif L==2
	 "Multiple choice"
#elif L==3
	 "Elecci&oacute;n m&uacute;ltiple"
#elif L==4
	 "Choix multiple"
#elif L==5
	 "Elecci&oacute;n m&uacute;ltiple"	// Okoteve traduccin
#elif L==6
	 "Scelta multipla"
#elif L==7
	 "Multiple choice"			// Potrzebujesz tlumaczenie
#elif L==8
	 "M&uacute;ltipla escolha"
#endif
	};

const char *Txt_Surveys =
#if   L==0
	"Enquestes";
#elif L==1
	"Umfragen";
#elif L==2
	"Surveys";
#elif L==3
	"Encuestas";
#elif L==4
	"Sondages";
#elif L==5
	"Encuestas";	// Okoteve traduccin
#elif L==6
	"Sondaggi";
#elif L==7
	"Ankiety";
#elif L==8
	"Inqu&eacute;ritos";
#endif

const char *Txt_Syllabus =
#if   L==0
	"Programa";
#elif L==1
	"Studienplan";
#elif L==2
	"Syllabus";
#elif L==3
	"Programa";
#elif L==4
	"Plan";
#elif L==5
	"Programa";	// Okoteve traduccin
#elif L==6
	"Programma";
#elif L==7
	"Syllabus";	// Potrzebujesz tlumaczenie
#elif L==8
	"Programa";
#endif

const char *Txt_Syllabus_of_the_course =
#if   L==0
	"Programa de la asignatura";	// Necessita traduccio
#elif L==1
	"Studienplan der Kurs";
#elif L==2
	"Syllabus of the course";
#elif L==3
	"Programa de la asignatura";
#elif L==4
	"Plan de la mati&egrave;re";
#elif L==5
	"Programa de la asignatura";	// Okoteve traduccin
#elif L==6
	"Programma del corso";
#elif L==7
	"Syllabus of the course";	// Potrzebujesz tlumaczenie
#elif L==8
	"Programa da disciplina";
#endif

const char *Txt_SYLLABUS_WHICH_SYLLABUS[Syl_NUM_WHICH_SYLLABUS] =
	 {
	 // Lectures
#if   L==0
	 "Teoria"
#elif L==1
	 "Vorlesung"
#elif L==2
	 "Lectures"
#elif L==3
	 "Teor&iacute;a"
#elif L==4
	 "Th&eacute;orie"
#elif L==5
	 "Teor&iacute;a"		// Okoteve traduccin
#elif L==6
	 "Teoria"
#elif L==7
	 "Wyklady"
#elif L==8
	 "Teoria"
#endif
	 ,
	 // Practicals
#if   L==0
	 "Pr&agrave;ctiques"
#elif L==1
	 "&Uuml;bung"
#elif L==2
	 "Practicals"
#elif L==3
	 "Pr&aacute;cticas"
#elif L==4
	 "Pratique"
#elif L==5
	 "Pr&aacute;cticas"		// Okoteve traduccin
#elif L==6
	 "Pratica"
#elif L==7
	 "Praktyki"
#elif L==8
	 "Pr&aacute;tica"
#endif
	 ,
	 };

const char *Txt_System =
#if   L==0
	"Plataforma";
#elif L==1
	"Plattform";
#elif L==2
	"System";
#elif L==3
	"Plataforma";
#elif L==4
	"Plate-forme";
#elif L==5
	"Ypykatu";
#elif L==6
	"Piattaforma";
#elif L==7
	"System";
#elif L==8
	"Sistema";
#endif

const char *Txt_TABLE_Footer =
#if   L==0
	"Pie";		// Necessita traduccio
#elif L==1
	"Fu&szlig;";
#elif L==2
	"Footer";
#elif L==3
	"Pie";
#elif L==4
	"Pied";
#elif L==5
	"Pie";		// Okoteve traduccin
#elif L==6
	"Piede";
#elif L==7
	"Footer";	// Potrzebujesz tlumaczenie
#elif L==8
	"P&eacute;";
#endif

const char *Txt_TABLE_Header =
#if   L==0
	"Cabecera";	// Necessita traduccio
#elif L==1
	"Header";
#elif L==2
	"Header";
#elif L==3
	"Cabecera";
#elif L==4
	"T&ecirc;te";
#elif L==5
	"Cabecera";	// Okoteve traduccin
#elif L==6
	"Intestazione";
#elif L==7
	"Header";	// Potrzebujesz tlumaczenie
#elif L==8
	"Cabe&ccedil;alho";
#endif

const char *Txt_Table_not_found_in_the_file_of_marks =
#if   L==0
	"No hay ninguna tabla en el archivo de calificaciones.";	// Necessita traduccio
#elif L==1
	"Table not found in the file of marks.";			// Need bersetzung
#elif L==2
	"Table not found in the file of marks.";
#elif L==3
	"No hay ninguna tabla en el archivo de calificaciones.";
#elif L==4
	"Table not found in the file of marks.";			// Besoin de traduction
#elif L==5
	"No hay ninguna tabla en el archivo de calificaciones.";	// Okoteve traduccin
#elif L==6
	"Tabella non trovata nel file dei risultati.";
#elif L==7
	"Table not found in the file of marks.";			// Potrzebujesz tlumaczenie
#elif L==8
	"Table not found in the file of marks.";			// Necessita de traduo
#endif

const char *Txt_TABS_FULL_TXT[Tab_NUM_TABS] =
   {
   // TabUnk
	 NULL,
	 // TabSys
#if   L==0
	 "Plataforma"
#elif L==1
	 "Plattform"
#elif L==2
	 "System"
#elif L==3
	 "Plataforma"
#elif L==4
	 "Plate-forme"
#elif L==5
	 "Ypykatu"
#elif L==6
	 "Piattaforma"
#elif L==7
	 "System"
#elif L==8
	 "Sistema"
#endif
	 ,
	 // TabCty
#if   L==0
	 "Pa&iacute;s"
#elif L==1
	 "Land"
#elif L==2
	 "Country"
#elif L==3
	 "Pa&iacute;s"
#elif L==4
	 "Pays"
#elif L==5
	 "Tet&atilde;"
#elif L==6
	 "Paese"
#elif L==7
	 "Kraj"
#elif L==8
	 "Pa&iacute;s"
#endif
	 ,
	 // TabIns
#if   L==0
	 "Instituci&oacute;"
#elif L==1
	 "Hochschule"
#elif L==2
	 "Institution"
#elif L==3
	 "Instituci&oacute;n"
#elif L==4
	 "&Eacute;tablissement"
#elif L==5
	 "Mbo'ehao"
#elif L==6
	 "Istituzione"
#elif L==7
	 "Instytucji"
#elif L==8
	 "Institu&ccedil;&atilde;o"
#endif
	 ,
	 // TabCtr
#if   L==0
	 "Centre"
#elif L==1
	 "Lehrinstitut"
#elif L==2
	 "Centre"
#elif L==3
	 "Centro"
#elif L==4
	 "Centre"
#elif L==5
	 "Mbo'ehao"
#elif L==6
	 "Centro"
#elif L==7
	 "Centrum"
#elif L==8
	 "Centro"
#endif
	 ,
	 // TabDeg
#if   L==0
	 "Titulaci&oacute;"
#elif L==1
	 "Studiengang"
#elif L==2
	 "Degree"
#elif L==3
	 "Titulaci&oacute;n"
#elif L==4
	 "&Eacute;tude"
#elif L==5
	 "Arandur&atilde;"
#elif L==6
	 "Laurea"
#elif L==7
	 "Stopien"
#elif L==8
	 "Titula&ccedil;&atilde;o"
#endif
	 ,
	 // TabCrs
#if   L==0
	 "Assignatura"
#elif L==1
	 "Kurs"
#elif L==2
	 "Course"
#elif L==3
	 "Asignatura"
#elif L==4
	 "Mati&egrave;re"
#elif L==5
	 "Mbo'esyry"
#elif L==6
	 "Corso"
#elif L==7
	 "Kurs"
#elif L==8
	 "Disciplina"
#endif
	 ,
	 // TabAss
#if   L==0
	 "Avaluaci&oacute;"
#elif L==1
	 "Bewertung"
#elif L==2
	 "Assessment"
#elif L==3
	 "Evaluaci&oacute;n"
#elif L==4
	 "&Eacute;valuation"
#elif L==5
	 "Mbohepy"
#elif L==6
	 "Valutazione"
#elif L==7
	 "Ocena"
#elif L==8
	 "Avalia&ccedil;&atilde;o"
#endif
	 ,
	 // TabUsr
#if   L==0
	 "Usuaris"
#elif L==1
	 "Benutzer"
#elif L==2
	 "Users"
#elif L==3
	 "Usuarios"
#elif L==4
	 "Utilisateurs"
#elif L==5
	 "Puruh&aacute;ra"
#elif L==6
	 "Utenti"
#elif L==7
	 "U&zdot;ytkownicy"
#elif L==8
	 "Utilizadores"
#endif
	 ,
	 // TabMsg
#if   L==0
	 "Missatges"
#elif L==1
	 "Nachrichten"
#elif L==2
	 "Messages"
#elif L==3
	 "Mensajes"
#elif L==4
	 "Messages"
#elif L==5
	 "Marandu"
#elif L==6
	 "Messaggi"
#elif L==7
	 "Wiadomo&sacute;ci"
#elif L==8
	 "Mensagens"
#endif
	 ,
	 // TabSta
#if   L==0
	 "Estad&iacute;stiques"
#elif L==1
	 "Statistik"
#elif L==2
	 "Statistics"
#elif L==3
	 "Estad&iacute;sticas"
#elif L==4
	 "Statistiques"
#elif L==5
	 "Papy'aporeko"
#elif L==6
	 "Statistiche"
#elif L==7
	 "Statystyki"
#elif L==8
	 "Estat&iacute;sticas"
#endif
	 ,
	 // TabPrf
#if   L==0
	 "Perfil"
#elif L==1
	 "Profil"
#elif L==2
	 "Profile"
#elif L==3
	 "Perfil"
#elif L==4
	 "Profil"
#elif L==5
	 "Che"
#elif L==6
	 "Profilo"
#elif L==7
	 "Profil"
#elif L==8
	 "Perfil"
#endif
	};

const char *Txt_TABS_SHORT_TXT[Tab_NUM_TABS] =	// Must be visible in 1024x768 without horizontal scroll bar
	 {
	 // TabUnk
	   NULL,
	   // TabSys
#if   L==0
	   "PLATAFORMA"
#elif L==1
	   "PLATTFORM"
#elif L==2
	   "SYSTEM"
#elif L==3
	   "PLATAFORMA"
#elif L==4
	   "PLATE-FORME"
#elif L==5
	   "YPYKATU"
#elif L==6
	   "PIATTAFORMA"
#elif L==7
	   "SYSTEM"
#elif L==8
	   "SISTEMA"
#endif
	   ,
	   // TabCty
#if   L==0
	   "PA&Iacute;S"
#elif L==1
	   "LAND"
#elif L==2
	   "COUNTRY"
#elif L==3
	   "PA&Iacute;S"
#elif L==4
	   "PAYS"
#elif L==5
	   "TET&Atilde;"
#elif L==6
	   "PAESE"
#elif L==7
	   "KRAJ"
#elif L==8
	   "PA&Iacute;S"
#endif
	   ,
	   // TabIns
#if   L==0
	   "INSTITUCI&Oacute;"
#elif L==1
	   "HOCHSCHULE"
#elif L==2
	   "INSTITUTION"
#elif L==3
	   "INSTITUCI&Oacute;N"
#elif L==4
	   "&Eacute;TABLISSEMENT"
#elif L==5
	   "MBO'EHAO"
#elif L==6
	   "ISTITUZIONE"
#elif L==7
	   "INSTYTUCJI"
#elif L==8
	   "INSTITU&Ccedil;&Atilde;O"
#endif
	   ,
	   // TabCtr
#if   L==0
	   "CENTRE"
#elif L==1
	   "LEHRINSTITUT"
#elif L==2
	   "CENTRE"
#elif L==3
	   "CENTRO"
#elif L==4
	   "CENTRE"
#elif L==5
	   "MBO'EHAO"
#elif L==6
	   "CENTRO"
#elif L==7
	   "CENTRUM"
#elif L==8
	   "CENTRO"
#endif
	   ,
	   // TabDeg
#if   L==0
	   "TITULACI&Oacute;"
#elif L==1
	   "STUDIENGANG"
#elif L==2
	   "DEGREE"
#elif L==3
	   "TITULACI&Oacute;N"
#elif L==4
	   "&Eacute;TUDE"
#elif L==5
	   "ARANDUR&Atilde;"
#elif L==6
	   "LAUREA"
#elif L==7
	   "STOPIEN"
#elif L==8
	   "TITULA&Ccedil;&Atilde;O"
#endif
	   ,
	   // TabCrs
#if   L==0
	   "ASSIGNATURA"
#elif L==1
	   "VERANSTALTUNG"
#elif L==2
	   "COURSE"
#elif L==3
	   "ASIGNATURA"
#elif L==4
	   "MATI&Egrave;RE"
#elif L==5
	   "MBO'ESYRY"
#elif L==6
	   "CORSO"
#elif L==7
	   "KURS"
#elif L==8
	   "DISCIPLINA"
#endif
	   ,
	   // TabAss
#if   L==0
	   "AVALUACI&Oacute;"
#elif L==1
	   "BEWERTUNG"
#elif L==2
	   "ASSESSMENT"
#elif L==3
	   "EVALUACI&Oacute;N"
#elif L==4
	   "&Eacute;VALUATION"
#elif L==5
	   "MBOHEPY"
#elif L==6
	   "VALUTAZIONE"
#elif L==7
	   "OCENA"
#elif L==8
	   "AVALIA&Ccedil;&Atilde;O"
#endif
	   ,
	   // TabUsr
#if   L==0
	   "USUARIS"
#elif L==1
	   "BENUTZER"
#elif L==2
	   "USERS"
#elif L==3
	   "USUARIOS"
#elif L==4
	   "UTILISATEURS"
#elif L==5
	   "PURUH&Aacute;RA"
#elif L==6
	   "UTENTI"
#elif L==7
	   "U&Zdot;YTKOWNICY"
#elif L==8
	   "UTILIZADORES"
#endif
	   ,
	   // TabMsg
#if   L==0
	   "MISSATGES"
#elif L==1
	   "NACHRICHTEN"
#elif L==2
	   "MESSAGES"
#elif L==3
	   "MENSAJES"
#elif L==4
	   "MESSAGES"
#elif L==5
	   "MARANDU"
#elif L==6
	   "MESSAGGI"
#elif L==7
	   "WIADOMO&Sacute;CY"
#elif L==8
	   "MENSAGENS"
#endif
	   ,
	   // TabSta
#if   L==0
	   "ESTAD&Iacute;STIQUES"
#elif L==1
	   "STATISTIK"
#elif L==2
	   "STATS"
#elif L==3
	   "ESTAD&Iacute;STICAS"
#elif L==4
	   "STATISTIQUES"
#elif L==5
	   "PAPY'APOREKO"
#elif L==6
	   "STATISTICHE"
#elif L==7
	   "STATYSTYKI"
#elif L==8
	   "ESTAT&Iacute;STICAS"
#endif
	   ,
	   // TabPrf
#if   L==0
	   "PERFIL"
#elif L==1
	   "PROFIL"
#elif L==2
	   "PROFILE"
#elif L==3
	   "PERFIL"
#elif L==4
	   "PROFIL"
#elif L==5
	   "CHE"
#elif L==6
	   "PROFILO"
#elif L==7
	   "PROFIL"
#elif L==8
	   "PERFIL"
#endif
	};

const char *Txt_Tag_allowed =
#if   L==0
	"Descriptor permitido";	// Necessita traduccio
#elif L==1
	"Tag erlaubt";
#elif L==2
	"Tag allowed";
#elif L==3
	"Descriptor permitido";
#elif L==4
	"Descripteur admis";
#elif L==5
	"Descriptor permitido";	// Okoteve traduccin
#elif L==6
	"Etichetta ammessa";
#elif L==7
	"Tag allowed";		// Potrzebujesz tlumaczenie
#elif L==8
	"Descritor permitido";
#endif

const char *Txt_Tag_X_allowed_Click_to_disable_it =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"Descriptor %s permitido. Pulse para prohibirlo.";	// Necessita traduccio
#elif L==1
	"Tag %s allowed. Click to disable it.";			// Need bersetzung
#elif L==2
	"Tag %s allowed. Click to disable it.";
#elif L==3
	"Descriptor %s permitido. Pulse para prohibirlo.";
#elif L==4
	"Tag %s allowed. Click to disable it.";			// Besoin de traduction
#elif L==5
	"Descriptor %s permitido. Pulse para prohibirlo.";	// Okoteve traduccin
#elif L==6
	"Etichetta %s autorizzata. Clicca per censurarla.";
#elif L==7
	"Tag %s allowed. Click to disable it.";			// Potrzebujesz tlumaczenie
#elif L==8
	"Tag %s allowed. Click to disable it.";			// Necessita de traduo
#endif

const char *Txt_Tag_not_allowed =
#if   L==0
	"Descriptor no permitido";	// Necessita traduccio
#elif L==1
	"Tag nicht erlaubt";
#elif L==2
	"Tag not allowed";
#elif L==3
	"Descriptor no permitido";
#elif L==4
	"Descripteur non admis";
#elif L==5
	"Descriptor no permitido";	// Okoteve traduccin
#elif L==6
	"Etichetta non autorizzata";
#elif L==7
	"Tag not allowed";		// Potrzebujesz tlumaczenie
#elif L==8
	"Descritor n&atilde;o permitido";
#endif

const char *Txt_Tag_X_not_allowed_Click_to_allow_it =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"Descriptor %s no permitido. Pulse para permitirlo.";	// Necessita traduccio
#elif L==1
	"Tag %s not allowed. Click to allow it.";		// Need bersetzung
#elif L==2
	"Tag %s not allowed. Click to allow it.";
#elif L==3
	"Descriptor %s no permitido. Pulse para permitirlo.";
#elif L==4
	"Tag %s not allowed. Click to allow it.";		// Besoin de traduction
#elif L==5
	"Descriptor %s no permitido. Pulse para permitirlo.";	// Okoteve traduccin
#elif L==6
	"Etichetta %s censurata. Clicca per autorizzarla.";
#elif L==7
	"Tag %s not allowed. Click to allow it.";		// Potrzebujesz tlumaczenie
#elif L==8
	"Tag %s not allowed. Click to allow it.";		// Necessita de traduo
#endif

const char *Txt_Tags =		// Tags used in test
#if   L==0
	"Descriptors";
#elif L==1
	"Tags";
#elif L==2
	"Tags";
#elif L==3
	"Descriptores";
#elif L==4
	"Descripteurs";
#elif L==5
	"Descriptores";	// Okoteve traduccin
#elif L==6
	"Etichette";
#elif L==7
	"Tags";		// Potrzebujesz tlumaczenie
#elif L==8
	"Descritores";
#endif

const char *Txt_Teachers_ABBREVIATION =	// Abbreviation of "Teachers"
#if   L==0
	"Prof.";	// Necessita traduccio
#elif L==1
	"Lehr.";
#elif L==2
	"Teach.";
#elif L==3
	"Prof.";
#elif L==4
	"Enseig.";
#elif L==5
	"Prof.";	// Okoteve traduccin
#elif L==6
	"Prof.";
#elif L==7
	"Teach.";	// Potrzebujesz tlumaczenie
#elif L==8
	"Prof.";
#endif

const char *Txt_Test =
#if   L==0
	"Test";
#elif L==1
	"Test";
#elif L==2
	"Test";
#elif L==3
	"Test";
#elif L==4
	"Test";
#elif L==5
	"Test";	// Okoteve traduccin
#elif L==6
	"Test";
#elif L==7
	"Test";
#elif L==8
	"Teste";
#endif

const char *Txt_TEST_Correct_answer =
#if   L==0
	"Resposta correcta";
#elif L==1
	"Correct answer";		// Need bersetzung
#elif L==2
	"Correct answer";
#elif L==3
	"Respuesta correcta";
#elif L==4
	"Correct answer";		// Besoin de traduction
#elif L==5
	"Respuesta correcta";		// Okoteve traduccin
#elif L==6
	"Risposta corretta";
#elif L==7
	"Prawidlowa odpowiedz";
#elif L==8
	"Correct answer";		// Necessita de traduo
#endif

const char *Txt_TEST_User_answer =
#if   L==0
	"Resposta usuari";
#elif L==1
	"User's answer";		// Need bersetzung
#elif L==2
	"User's answer";
#elif L==3
	"Respuesta usuario";
#elif L==4
	"User's answer";		// Besoin de traduction
#elif L==5
	"Respuesta usuario";		// Okoteve traduccin
#elif L==6
	"Risposta utente";
#elif L==7
	"User's answer";		// Potrzebujesz tlumaczenie
#elif L==8
	"User's answer";		// Necessita de traduo
#endif

const char *Txt_Test_No_X_that_you_make_in_this_course =	// Warning: it is very important to include %u in the following sentences
#if   L==0
	"Test n&ordm; %u que realiza usted en esta asignatura";	// Necessita traduccio
#elif L==1
	"Der %u. Test, den Sie in dieser Kurs machen";
#elif L==2
	"Test No. %u that you make in this course";
#elif L==3
	"Test n&ordm; %u que realiza usted en esta asignatura";
#elif L==4
	"Test n&ordm; %u que vous faites dans cette mati&egrave;re";
#elif L==5
	"Test n&ordm; %u que realiza usted en esta asignatura";	// Okoteve traduccin
#elif L==6
	"Test n. %u che fai in questo corso";
#elif L==7
	"Test No. %u that you make in this course";		// Potrzebujesz tlumaczenie
#elif L==8
	"Teste n&ordm; %u que voc&ecirc; faz nesta disciplina";
#endif

const char *Txt_Test_result =
#if   L==0
	"Resultado del test";	// Necessita traduccio
#elif L==1
	"Testergebnis";
#elif L==2
	"Test result";
#elif L==3
	"Resultado del test";
#elif L==4
	"R&eacute;sultat du test";
#elif L==5
	"Resultado del test";	// Okoteve traduccin
#elif L==6
	"Risultato del test";
#elif L==7
	"Test result";		// Potrzebujesz tlumaczenie
#elif L==8
	"Resultado do teste";
#endif

const char *Txt_Thanks_for_answering_the_survey =
#if   L==0
	"Gracias por responder la encuesta.";	// Necessita traduccio
#elif L==1
	"Vielen Dank f&uuml;r die Beantwortung der Umfrage.";
#elif L==2
	"Thanks for answering the survey.";
#elif L==3
	"Gracias por responder la encuesta.";
#elif L==4
	"Merci de r&eacute;pondre au sondage.";
#elif L==5
	"Gracias por responder la encuesta.";	// Okoteve traduccin
#elif L==6
	"Grazie per la risposta al sondaggio.";
#elif L==7
	"Thanks for answering the survey.";	// Potrzebujesz tlumaczenie
#elif L==8
	"Obrigado por responder o inqu&eacute;rito.";
#endif

const char *Txt_The_alphabetical_code_X_already_exists =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"El c&oacute;digo alfab&eacute;tico ISO 3166-1 %s ya existe.";	// Necessita traduccio
#elif L==1
	"The ISO 3166-1 alphabetical code %s already exists.";		// Need bersetzung
#elif L==2
	"The ISO 3166-1 alphabetical code %s already exists.";
#elif L==3
	"El c&oacute;digo alfab&eacute;tico ISO 3166-1 %s ya existe.";
#elif L==4
	"The ISO 3166-1 alphabetical code %s already exists.";		// Besoin de traduction
#elif L==5
	"El c&oacute;digo alfab&eacute;tico ISO 3166-1 %s ya existe.";	// Okoteve traduccin
#elif L==6
	"Il codice alfabetico ISO 3166-1 %s gi&agrave; esiste.";
#elif L==7
	"The ISO 3166-1 alphabetical code %s already exists.";		// Potrzebujesz tlumaczenie
#elif L==8
	"The ISO 3166-1 alphabetical code %s already exists.";		// Necessita de traduo
#endif

const char *Txt_The_alphabetical_code_X_is_not_correct =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"El c&oacute;digo alfab&eacute;tico ISO 3166-1 %s no es correcto.";	// Necessita traduccio
#elif L==1
	"The ISO 3166-1 alphabetical code %s is not correct.";			// Need bersetzung
#elif L==2
	"The ISO 3166-1 alphabetical code %s is not correct.";
#elif L==3
	"El c&oacute;digo alfab&eacute;tico ISO 3166-1 %s no es correcto.";
#elif L==4
	"The ISO 3166-1 alphabetical code %s is not correct.";			// Besoin de traduction
#elif L==5
	"El c&oacute;digo alfab&eacute;tico ISO 3166-1 %s no es correcto.";	// Okoteve traduccin
#elif L==6
	"Il codice alfabetico ISO 3166-1 %s non &egrave; corretto.";
#elif L==7
	"The ISO 3166-1 alphabetical code %s is not correct.";			// Potrzebujesz tlumaczenie
#elif L==8
	"The ISO 3166-1 alphabetical code %s is not correct.";			// Necessita de traduo
#endif

const char *Txt_The_announcement_of_exam_has_been_successfully_updated =
#if   L==0
	"La convocatoria de examen se ha actualizado correctamente.";	// Necessita traduccio
#elif L==1
	"The announcement of exam has been successfully updated.";	// Need bersetzung
#elif L==2
	"The announcement of exam has been successfully updated.";
#elif L==3
	"La convocatoria de examen se ha actualizado correctamente.";
#elif L==4
	"The announcement of exam has been successfully updated.";	// Besoin de traduction
#elif L==5
	"La convocatoria de examen se ha actualizado correctamente.";	// Okoteve traduccin
#elif L==6
	"La sessione d'esame &egrave; stata aggiornata con successo.";
#elif L==7
	"The announcement of exam has been successfully updated.";	// Potrzebujesz tlumaczenie
#elif L==8
	"The announcement of exam has been successfully updated.";	// Necessita de traduo
#endif

const char *Txt_The_answers_of_the_question_with_code_X_will_appear_shuffled =	// Warning: it is very important to include %ld in the following sentences
#if   L==0
	"Las respuestas de la pregunta con c&oacute;digo <strong>%ld</strong>"
	" aparecer&aacute;n barajadas.";	// Necessita traduccio
#elif L==1
	"The answers of the question with code <strong>%ld</strong>"
	" will appear shuffled.";		// Need bersetzung
#elif L==2
	"The answers of the question with code <strong>%ld</strong>"
	" will appear shuffled.";
#elif L==3
	"Las respuestas de la pregunta con c&oacute;digo <strong>%ld</strong>"
	" aparecer&aacute;n barajadas.";
#elif L==4
	"The answers of the question with code <strong>%ld</strong>"
	" will appear shuffled.";		// Besoin de traduction
#elif L==5
	"Las respuestas de la pregunta con c&oacute;digo <strong>%ld</strong>"
	" aparecer&aacute;n barajadas.";	// Okoteve traduccin
#elif L==6
	"Le risposte alla domande con il codice <strong>%ld</strong>"
	" appariranno mischiate.";
#elif L==7
	"The answers of the question with code <strong>%ld</strong>"
	" will appear shuffled.";		// Potrzebujesz tlumaczenie
#elif L==8
	"The answers of the question with code <strong>%ld</strong>"
	" will appear shuffled.";		// Necessita de traduo
#endif

const char *Txt_The_answers_of_the_question_with_code_X_will_appear_without_shuffling =	// Warning: it is very important to include %ld in the following sentences
#if   L==0
	"Las respuestas de la pregunta con c&oacute;digo <strong>%ld</strong>"
	" aparecer&aacute;n sin barajar.";	// Necessita traduccio
#elif L==1
	"The answers of the question with code <strong>%ld</strong>"
	" will appear without shuffling.";	// Need bersetzung
#elif L==2
	"The answers of the question with code <strong>%ld</strong>"
	" will appear without shuffling.";
#elif L==3
	"Las respuestas de la pregunta con c&oacute;digo <strong>%ld</strong>"
	" aparecer&aacute;n sin barajar.";
#elif L==4
	"The answers of the question with code <strong>%ld</strong>"
	" will appear without shuffling.";	// Besoin de traduction
#elif L==5
	"Las respuestas de la pregunta con c&oacute;digo <strong>%ld</strong>"
	" aparecer&aacute;n sin barajar.";	// Okoteve traduccin
#elif L==6
	"Le risposte della domanda con il codice <strong>%ld</strong>"
	" appariranno senza essere mischiate.";
#elif L==7
	"The answers of the question with code <strong>%ld</strong>"
	" will appear without shuffling.";	// Potrzebujesz tlumaczenie
#elif L==8
	"The answers of the question with code <strong>%ld</strong>"
	" will appear without shuffling.";	// Necessita de traduo
#endif

const char *Txt_The_assignment_has_been_modified =
#if   L==0
	"La actividad ha sido modificada.";	// Necessita traduccio
#elif L==1
	"Die Aufgabe wurde ge&auml;ndert.";
#elif L==2
	"The assignment has been modified.";
#elif L==3
	"La actividad ha sido modificada.";
#elif L==4
	"L'activit&eacute; a &eacute;t&eacute; modifi&eacute;e.";
#elif L==5
	"La actividad ha sido modificada.";	// Okoteve traduccin
#elif L==6
	"L'attivit&agrave; &egrave; stata modificata.";
#elif L==7
	"The assignment has been modified.";	// Potrzebujesz tlumaczenie
#elif L==8
	"A atividade foi modificada.";
#endif

const char *Txt_The_banner_X_already_exists =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"El banner <strong>%s</strong> ya existe.";		// Necessita traduccio
#elif L==1
	"Der Werbebanner <strong>%s</strong> wurde bereits eingegeben.";
#elif L==2
	"The banner <strong>%s</strong> already exists.";
#elif L==3
	"El banner <strong>%s</strong> ya existe.";
#elif L==4
	"The banner <strong>%s</strong> already exists.";	// Besoin de traduction
#elif L==5
	"El banner <strong>%s</strong> ya existe.";		// Okoteve traduccin
#elif L==6
	"Il banner <strong>%s</strong> gi&agrave; esiste.";
#elif L==7
	"The banner <strong>%s</strong> already exists.";	// Potrzebujesz tlumaczenie
#elif L==8
	"The banner <strong>%s</strong> already exists.";	// Necessita de traduo
#endif

const char *Txt_The_banner_X_has_been_renamed_as_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==0
	"El banner <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";	// Necessita traduccio
#elif L==1
	"The Werbebanner <strong>%s</strong> has been renamed as <strong>%s</strong>.";	// Need bersetzung
#elif L==2
	"The banner <strong>%s</strong> has been renamed as <strong>%s</strong>.";
#elif L==3
	"El banner <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";
#elif L==4
	"The banner <strong>%s</strong> has been renamed as <strong>%s</strong>.";	// Besoin de traduction
#elif L==5
	"El banner <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";	// Okoteve traduccin
#elif L==6
	"Il banner <strong>%s</strong> &egrave; stato rinominato come <strong>%s</strong>.";
#elif L==7
	"The banner <strong>%s</strong> has been renamed as <strong>%s</strong>.";	// Potrzebujesz tlumaczenie
#elif L==8
	"The banner <strong>%s</strong> has been renamed as <strong>%s</strong>.";	// Necessita de traduo
#endif

const char *Txt_The_banner_X_is_now_hidden =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"El banner <strong>%s</strong> ara est&agrave; ocult.";
#elif L==1
	"Der Werbebanner <strong>%s</strong> ist jetzt ausgeblendet.";	// Need bersetzung
#elif L==2
	"The banner <strong>%s</strong> is now hidden.";
#elif L==3
	"El banner <strong>%s</strong> ahora est&aacute; oculto.";
#elif L==4
	"Le banner <strong>%s</strong> est maintenant cach&eacute;.";
#elif L==5
	"El banner <strong>%s</strong> ahora est&aacute; oculto.";	// Okoteve traduccin
#elif L==6
	"Il banner <strong>%s</strong> &egrave; ora nascosto.";
#elif L==7
	"The banner <strong>%s</strong> jest ukryte.";
#elif L==8
	"O banner <strong>%s</strong> &eacute; agora oculto.";
#endif

const char *Txt_The_banner_X_is_now_visible =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"El banner <strong>%s</strong> ara est&agrave; visible.";
#elif L==1
	"Der Werbebanner <strong>%s</strong> ist jetzt sichtbar.";	// Need bersetzung
#elif L==2
	"The banner <strong>%s</strong> is now visible.";
#elif L==3
	"El banner <strong>%s</strong> ahora est&aacute; visible.";
#elif L==4
	"Le banner <strong>%s</strong> est maintenant visible.";
#elif L==5
	"El banner <strong>%s</strong> ahora est&aacute; visible.";	// Okoteve traduccin
#elif L==6
	"Il banner <strong>%s</strong> &egrave; ora visible.";
#elif L==7
	"The banner <strong>%s</strong> jest teraz widoczny.";
#elif L==8
	"O banner <strong>%s</strong> &eacute; agora visible.";
#endif

const char *Txt_The_centre_of_the_degree_has_changed =
#if   L==0
	"El centro de la titulaci&oacute;n ha cambiado.";	// Necessita traduccio
#elif L==1
	"The centre of the degree has changed.";		// Need bersetzung
#elif L==2
	"The centre of the degree has changed.";
#elif L==3
	"El centro de la titulaci&oacute;n ha cambiado.";
#elif L==4
	"The centre of the degree has changed.";		// Besoin de traduction
#elif L==5
	"El centro de la titulaci&oacute;n ha cambiado.";	// Okoteve traduccin
#elif L==6
	"Il centro della laurea &egrave; cambiato.";
#elif L==7
	"The centre of the degree has changed.";		// Potrzebujesz tlumaczenie
#elif L==8
	"The centre of the degree has changed.";		// Necessita de traduo
#endif

const char *Txt_The_centre_X_already_exists =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"El centro <strong>%s</strong> ya existe.";		// Necessita traduccio
#elif L==1
	"Es wurde bereits ein Lehrinstitut mit dem Namen <strong>%s</strong> eingegeben.";
#elif L==2
	"The centre <strong>%s</strong> already exists.";
#elif L==3
	"El centro <strong>%s</strong> ya existe.";
#elif L==4
	"The centre <strong>%s</strong> already exists.";	// Besoin de traduction
#elif L==5
	"El centro <strong>%s</strong> ya existe.";		// Okoteve traduccin
#elif L==6
	"Il centro <strong>%s</strong> gi&agrave; esiste.";
#elif L==7
	"The centre <strong>%s</strong> already exists.";	// Potrzebujesz tlumaczenie
#elif L==8
	"The centre <strong>%s</strong> already exists.";	// Necessita de traduo
#endif

const char *Txt_The_centre_X_has_been_renamed_as_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==0
	"El centro <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";	// Necessita traduccio
#elif L==1
	"The centre <strong>%s</strong> has been renamed as <strong>%s</strong>.";	// Need bersetzung
#elif L==2
	"The centre <strong>%s</strong> has been renamed as <strong>%s</strong>.";
#elif L==3
	"El centro <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";
#elif L==4
	"The centre <strong>%s</strong> has been renamed as <strong>%s</strong>.";	// Besoin de traduction
#elif L==5
	"El centro <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";	// Okoteve traduccin
#elif L==6
	"Il centro <strong>%s</strong> &egrave; stato rinominato <strong>%s</strong>.";
#elif L==7
	"The centre <strong>%s</strong> has been renamed as <strong>%s</strong>.";	// Potrzebujesz tlumaczenie
#elif L==8
	"The centre <strong>%s</strong> has been renamed as <strong>%s</strong>.";	// Necessita de traduo
#endif

const char *Txt_The_configuration_of_the_course_X_has_been_updated =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"La configuraci&oacute;n de la asignatura <strong>%s</strong> se ha actualizado.";	// Necessita traduccio
#elif L==1
	"The configuration of the course <strong>%s</strong> has been updated.";		// Need bersetzung
#elif L==2
	"The configuration of the course <strong>%s</strong> has been updated.";
#elif L==3
	"La configuraci&oacute;n de la asignatura <strong>%s</strong> se ha actualizado.";
#elif L==4
	"The configuration of the course <strong>%s</strong> has been updated.";		// Besoin de traduction
#elif L==5
	"La configuraci&oacute;n de la asignatura <strong>%s</strong> se ha actualizado.";	// Okoteve traduccin
#elif L==6
	"La configurazione del corso <strong>%s</strong> &egrave; stata aggiornata.";
#elif L==7
	"The configuration of the course <strong>%s</strong> has been updated.";		// Potrzebujesz tlumaczenie
#elif L==8
	"The configuration of the course <strong>%s</strong> has been updated.";		// Necessita de traduo
#endif

const char *Txt_The_contents_of_the_folder_are_too_big =
#if   L==0
	"El contingut de la carpeta &eacute;s massa gran.";
#elif L==1
	"Der Inhalt der Ordner sind zu gro&szlig;.";
#elif L==2
	"The contents of the folder are too big.";
#elif L==3
	"El contenido de la carpeta es demasiado grande.";
#elif L==4
	"Le contenu du dossier est trop gros.";
#elif L==5
	"El contenido de la carpeta es demasiado grande.";	// Okoteve traduccin
#elif L==6
	"Il contenuto della cartella &egrave; troppo grande.";
#elif L==7
	"Zawarto&sacute;&cacute; folderu s&aogon; zbyt du&zdot;e.";
#elif L==8
	"O conte&uacute;do do diret&oacute;rio &eacute; muito grande.";
#endif

const char *Txt_The_copy_has_been_successful =
#if   L==0
	"La copia se ha realizado con &eacute;xito.";	// Necessita traduccio
#elif L==1
	"Die Kopie wurde erfolgreich.";
#elif L==2
	"The copy has been successful.";
#elif L==3
	"La copia se ha realizado con &eacute;xito.";
#elif L==4
	"La copie a &eacute;t&eacute; un succ&egrave;s.";
#elif L==5
	"La copia se ha realizado con &eacute;xito.";	// Okoteve traduccin
#elif L==6
	"La copia &egrave; avvenuta con successo.";
#elif L==7
	"The copy has been successful.";		// Potrzebujesz tlumaczenie
#elif L==8
	"A c&oacute;pia foi feita com sucesso.";
#endif

const char *Txt_The_copy_has_stopped_when_trying_to_paste_the_file_X_because_it_would_exceed_the_disk_quota =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"La copia se ha detenido al intentar pegar el archivo <strong>%s</strong>"
	" porque se exceder&iacute;a la cuota de disco.";	// Necessita traduccio
#elif L==1
	"Der Kopiervorgang wurde aus Platzgr&uuml;nden abgebrochen, als versucht wurde,"
	" die Datei <strong>%s</strong> einzuf&uuml;gen.";
#elif L==2
	"The copy has stopped when trying to paste the file <strong>%s</strong>"
	" because it would exceed the disk quota.";
#elif L==3
	"La copia se ha detenido al intentar pegar el archivo <strong>%s</strong>"
	" porque se exceder&iacute;a la cuota de disco.";
#elif L==4
	"The copy has stopped when trying to paste the file <strong>%s</strong>"
	" because it would exceed the disk quota.";		// Besoin de traduction
#elif L==5
	"La copia se ha detenido al intentar pegar el archivo <strong>%s</strong>"
	" porque se exceder&iacute;a la cuota de disco.";	// Okoteve traduccin
#elif L==6
	"La copia &egrave; terminata quando si &egrave; provato a incollare il file <strong>%s</strong>"
	" perch&egrave; esso superava la quota di gestione del disco.";
#elif L==7
	"The copy has stopped when trying to paste the file <strong>%s</strong>"
	" because it would exceed the disk quota.";		// Potrzebujesz tlumaczenie
#elif L==8
	"The copy has stopped when trying to paste the file <strong>%s</strong>"
	" because it would exceed the disk quota.";		// Necessita de traduo
#endif

const char *Txt_The_copy_has_stopped_when_trying_to_paste_the_folder_X_because_it_would_exceed_the_disk_quota =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"La copia se ha detenido al intentar pegar la carpeta <strong>%s</strong>"
	" porque se exceder&iacute;a la cuota de disco.";	// Necessita traduccio
#elif L==1
	"Der Kopiervorgang wurde aus Platzgr&uuml;nden abgebrochen, als versucht wurde,"
	" das Verzeichnis <strong>%s</strong> einzuf&uuml;gen.";
#elif L==2
	"The copy has stopped when trying to paste the folder <strong>%s</strong>"
	" because it would exceed the disk quota.";
#elif L==3
	"La copia se ha detenido al intentar pegar la carpeta <strong>%s</strong>"
	" porque se exceder&iacute;a la cuota de disco.";
#elif L==4
	"The copy has stopped when trying to paste the folder <strong>%s</strong>"
	" because it would exceed the disk quota.";		// Besoin de traduction
#elif L==5
	"La copia se ha detenido al intentar pegar la carpeta <strong>%s</strong>"
	" porque se exceder&iacute;a la cuota de disco.";	// Okoteve traduccin
#elif L==6
	"La copia &egrave; terminata quando si &egrave; provato a incollare la cartella<strong>%s</strong>"
	" perch&egrave; essa superava la quota di gestione del disco.";
#elif L==7
	"The copy has stopped when trying to paste the folder <strong>%s</strong>"
	" because it would exceed the disk quota.";		// Potrzebujesz tlumaczenie
#elif L==8
	"The copy has stopped when trying to paste the folder <strong>%s</strong>"
	" because it would exceed the disk quota.";		// Necessita de traduo
#endif

const char *Txt_The_copy_has_stopped_when_trying_to_paste_the_link_X_because_it_would_exceed_the_disk_quota =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"La copia se ha detenido al intentar pegar el enlace <strong>%s</strong>"
	" porque se exceder&iacute;a la cuota de disco.";	// Necessita traduccio
#elif L==1
	"Der Kopiervorgang wurde aus Platzgr&uuml;nden abgebrochen, als versucht wurde,"
	" das Link <strong>%s</strong> einzuf&uuml;gen.";
#elif L==2
	"The copy has stopped when trying to paste the link <strong>%s</strong>"
	" because it would exceed the disk quota.";
#elif L==3
	"La copia se ha detenido al intentar pegar el enlace <strong>%s</strong>"
	" porque se exceder&iacute;a la cuota de disco.";
#elif L==4
	"The copy has stopped when trying to paste the link <strong>%s</strong>"
	" because it would exceed the disk quota.";		// Besoin de traduction
#elif L==5
	"La copia se ha detenido al intentar pegar el enlace <strong>%s</strong>"
	" porque se exceder&iacute;a la cuota de disco.";	// Okoteve traduccin
#elif L==6
	"La copia &egrave; terminata quando si &egrave; provato a incollare il link<strong>%s</strong>"
	" perch&egrave; essa superava la quota di gestione del disco.";
#elif L==7
	"The copy has stopped when trying to paste the link <strong>%s</strong>"
	" because it would exceed the disk quota.";		// Potrzebujesz tlumaczenie
#elif L==8
	"The copy has stopped when trying to paste the link <strong>%s</strong>"
	" because it would exceed the disk quota.";		// Necessita de traduo
#endif

const char *Txt_The_copy_has_stopped_when_trying_to_paste_the_file_X_because_it_would_exceed_the_maximum_allowed_number_of_levels =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"La copia se ha detenido al intentar pegar el archivo <strong>%s</strong>"
	" porque se exceder&iacute;a el n&uacute;mero m&aacute;ximo de niveles permitido.";	// Necessita traduccio
#elif L==1
	"Zu viele Ebenen. Der Kopiervorgang wurde abgebrochen, als versucht wurde,"
	" die Datei <strong>%s</strong> einzuf&uuml;gen.";
#elif L==2
	"The copy has stopped when trying to paste the file <strong>%s</strong>"
	" because it would exceed the maximum allowed number of levels.";
#elif L==3
	"La copia se ha detenido al intentar pegar el archivo <strong>%s</strong>"
	" porque se exceder&iacute;a el n&uacute;mero m&aacute;ximo de niveles permitido.";
#elif L==4
	"The copy has stopped when trying to paste the file <strong>%s</strong>"
	" because it would exceed the maximum allowed number of levels.";			// Besoin de traduction
#elif L==5
	"La copia se ha detenido al intentar pegar el archivo <strong>%s</strong>"
	" porque se exceder&iacute;a el n&uacute;mero m&aacute;ximo de niveles permitido.";	// Okoteve traduccin
#elif L==6
	"La copia &egrave; terminata quando si &egrave; provato a incollare il file <strong>%s</strong>"
	" perch&egrave; esso superava il massimo numero di livelli permesso.";
#elif L==7
	"The copy has stopped when trying to paste the file <strong>%s</strong>"
	" because it would exceed the maximum allowed number of levels.";			// Potrzebujesz tlumaczenie
#elif L==8
	"The copy has stopped when trying to paste the file <strong>%s</strong>"
	" because it would exceed the maximum allowed number of levels.";			// Necessita de traduo
#endif

const char *Txt_The_copy_has_stopped_when_trying_to_paste_the_folder_X_because_it_would_exceed_the_maximum_allowed_number_of_levels =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"La copia se ha detenido al intentar pegar el archivo <strong>%s</strong>"
	" porque se exceder&iacute;a el n&uacute;mero m&aacute;ximo de niveles permitido.";	// Necessita traduccio
#elif L==1
	"Zu viele Ebenen. Der Kopiervorgang wurde abgebrochen, als versucht wurde,"
	" die Datei <strong>%s</strong> einzuf&uuml;gen.";
#elif L==2
	"The copy has stopped when trying to paste the file <strong>%s</strong>"
	" because it would exceed the maximum allowed number of levels.";
#elif L==3
	"La copia se ha detenido al intentar pegar el archivo <strong>%s</strong>"
	" porque se exceder&iacute;a el n&uacute;mero m&aacute;ximo de niveles permitido.";
#elif L==4
	"The copy has stopped when trying to paste the file <strong>%s</strong>"
	" because it would exceed the maximum allowed number of levels.";			// Besoin de traduction
#elif L==5
	"La copia se ha detenido al intentar pegar el archivo <strong>%s</strong>"
	" porque se exceder&iacute;a el n&uacute;mero m&aacute;ximo de niveles permitido.";	// Okoteve traduccin
#elif L==6
	"La copia &egrave; terminata quando si &egrave; provato a incollare il file <strong>%s</strong>"
	" perch&egrave; superava il massimo numero di livelli permessi.";
#elif L==7
	"The copy has stopped when trying to paste the file <strong>%s</strong>"
	" because it would exceed the maximum allowed number of levels.";			// Potrzebujesz tlumaczenie
#elif L==8
	"The copy has stopped when trying to paste the file <strong>%s</strong>"
	" because it would exceed the maximum allowed number of levels.";			// Necessita de traduo
#endif

const char *Txt_The_copy_has_stopped_when_trying_to_paste_the_link_X_because_it_would_exceed_the_maximum_allowed_number_of_levels =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"La copia se ha detenido al intentar pegar el enlace <strong>%s</strong>"
	" porque se exceder&iacute;a el n&uacute;mero m&aacute;ximo de niveles permitido.";	// Necessita traduccio
#elif L==1
	"Zu viele Ebenen. Der Kopiervorgang wurde abgebrochen, als versucht wurde,"
	" die Link <strong>%s</strong> einzuf&uuml;gen.";
#elif L==2
	"The copy has stopped when trying to paste the link <strong>%s</strong>"
	" because it would exceed the maximum allowed number of levels.";
#elif L==3
	"La copia se ha detenido al intentar pegar el enlace <strong>%s</strong>"
	" porque se exceder&iacute;a el n&uacute;mero m&aacute;ximo de niveles permitido.";
#elif L==4
	"The copy has stopped when trying to paste the link <strong>%s</strong>"
	" because it would exceed the maximum allowed number of levels.";			// Besoin de traduction
#elif L==5
	"La copia se ha detenido al intentar pegar el enlace <strong>%s</strong>"
	" porque se exceder&iacute;a el n&uacute;mero m&aacute;ximo de niveles permitido.";	// Okoteve traduccin
#elif L==6
	"La copia &egrave; terminata quando si &egrave; provato a incollare il link <strong>%s</strong>"
	" perch&egrave; superava il massimo numero di livelli permessi.";
#elif L==7
	"The copy has stopped when trying to paste the link <strong>%s</strong>"
	" because it would exceed the maximum allowed number of levels.";			// Potrzebujesz tlumaczenie
#elif L==8
	"The copy has stopped when trying to paste the link <strong>%s</strong>"
	" because it would exceed the maximum allowed number of levels.";			// Necessita de traduo
#endif

const char *Txt_The_copy_has_stopped_when_trying_to_paste_the_file_X_because_there_is_already_an_object_with_that_name =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"La copia se ha detenido al intentar pegar el archivo <strong>%s</strong>"
	" porque ya existe otro objeto con ese nombre.";	// Necessita traduccio
#elif L==1
	"Der Kopiervorgang wurde abgebrochen, als versucht wurde, die Datei <strong>%s</strong>"
	" einzuf&uuml;gen. Der Objektname wird bereits verwendet.";
#elif L==2
	"The copy has stopped when trying to paste the file <strong>%s</strong>"
	" because there is already an object with that name.";
#elif L==3
	"La copia se ha detenido al intentar pegar el archivo <strong>%s</strong>"
	" porque ya existe otro objeto con ese nombre.";
#elif L==4
	"The copy has stopped when trying to paste the file <strong>%s</strong>"
	" because there is already an object with that name.";	// Besoin de traduction
#elif L==5
	"La copia se ha detenido al intentar pegar el archivo <strong>%s</strong>"
	" porque ya existe otro objecto con ese nombre.";	// Okoteve traduccin
#elif L==6
	"La copia &egrave; terminata quando si &egrave; provato a incollare il file <strong>%s</strong>"
	" perch&egrave; c'era gi&agrave; un oggetto con quel nome.";
#elif L==7
	"The copy has stopped when trying to paste the file <strong>%s</strong>"
	" because there is already an object with that name.";	// Potrzebujesz tlumaczenie
#elif L==8
	"The copy has stopped when trying to paste the file <strong>%s</strong>"
	" because there is already an object with that name.";	// Necessita de traduo
#endif

const char *Txt_The_copy_has_stopped_when_trying_to_paste_the_link_X_because_there_is_already_an_object_with_that_name =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"La copia se ha detenido al intentar pegar el enlace <strong>%s</strong>"
	" porque ya existe otro objeto con ese nombre.";	// Necessita traduccio
#elif L==1
	"Der Kopiervorgang wurde abgebrochen, als versucht wurde, die Link <strong>%s</strong>"
	" einzuf&uuml;gen. Der Objektname wird bereits verwendet.";
#elif L==2
	"The copy has stopped when trying to paste the link <strong>%s</strong>"
	" because there is already an object with that name.";
#elif L==3
	"La copia se ha detenido al intentar pegar el enlace <strong>%s</strong>"
	" porque ya existe otro objeto con ese nombre.";
#elif L==4
	"The copy has stopped when trying to paste the link <strong>%s</strong>"
	" because there is already an object with that name.";	// Besoin de traduction
#elif L==5
	"La copia se ha detenido al intentar pegar el enlace <strong>%s</strong>"
	" porque ya existe otro objecto con ese nombre.";	// Okoteve traduccin
#elif L==6
	"La copia &egrave; terminata quando si &egrave; provato a incollare il link <strong>%s</strong>"
	" perch&egrave; c'era gi&agrave; un oggetto con quel nome.";
#elif L==7
	"The copy has stopped when trying to paste the link <strong>%s</strong>"
	" because there is already an object with that name.";	// Potrzebujesz tlumaczenie
#elif L==8
	"The copy has stopped when trying to paste the link <strong>%s</strong>"
	" because there is already an object with that name.";	// Necessita de traduo
#endif

const char *Txt_The_copy_has_stopped_when_trying_to_paste_the_file_X_because_you_can_not_paste_a_file_here_of_a_type_other_than_HTML =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"La copia se ha detenido al intentar pegar el archivo <strong>%s</strong>"
	" porque no se puede pegar aqu&iacute; un archivo de un tipo distinto a HTML.";	// Necessita traduccio
#elif L==1
	"Der Kopiervorgang wurde abgebrochen, als versucht wurde, die Datei <strong>%s</strong>"
	" einzuf&uuml;gen. Es d&uuml;rfen nur HTML-Daten eingef&uumlgt werden.";
#elif L==2
	"The copy has stopped when trying to paste the file <strong>%s</strong>"
	" because you can not paste a file here of a type other than HTML.";
#elif L==3
	"La copia se ha detenido al intentar pegar el archivo <strong>%s</strong>"
	" porque no se puede pegar aqu&iacute; un archivo de un tipo distinto a HTML.";
#elif L==4
	"The copy has stopped when trying to paste the file <strong>%s</strong>"
	" because you can not paste a file here of a type other than HTML.";		// Besoin de traduction
#elif L==5
	"La copia se ha detenido al intentar pegar el archivo <strong>%s</strong>"
	" porque no se puede pegar aqu&iacute; un archivo de un tipo distinto a HTML.";	// Okoteve traduccin
#elif L==6
	"La copia &egrave; terminata quando si &egrave; provato a incollare il file <strong>%s</strong>"
	" perch&egrave; non puoi incollare qui un file di un tipo diverso da HTML.";
#elif L==7
	"The copy has stopped when trying to paste the file <strong>%s</strong>"
	" because you can not paste a file here of a type other than HTML.";		// Potrzebujesz tlumaczenie
#elif L==8
	"The copy has stopped when trying to paste the file <strong>%s</strong>"
	" because you can not paste a file here of a type other than HTML.";		// Necessita de traduo
#endif

const char *Txt_The_country_X_already_exists =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"El pa&iacute;s <strong>%s</strong> ya existe.";	// Necessita traduccio
#elif L==1
	"Die Land <strong>%s</strong> wurde bereits eingegeben.";
#elif L==2
	"The country <strong>%s</strong> already exists.";
#elif L==3
	"El pa&iacute;s <strong>%s</strong> ya existe.";
#elif L==4
	"The country <strong>%s</strong> already exists.";	// Besoin de traduction
#elif L==5
	"El pa&iacute;s <strong>%s</strong> ya existe.";	// Okoteve traduccin
#elif L==6
	"Il paese <strong>%s</strong> gi&agrave; esiste.";
#elif L==7
	"The country <strong>%s</strong> already exists.";	// Potrzebujesz tlumaczenie
#elif L==8
	"The country <strong>%s</strong> already exists.";	// Necessita de traduo
#endif

const char *Txt_The_country_X_has_been_renamed_as_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==0
	"El pa&iacute;s <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";	// Necessita traduccio
#elif L==1
	"The country <strong>%s</strong> has been renamed as <strong>%s</strong>.";		// Need bersetzung
#elif L==2
	"The country <strong>%s</strong> has been renamed as <strong>%s</strong>.";
#elif L==3
	"El pa&iacute;s <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";
#elif L==4
	"The country <strong>%s</strong> has been renamed as <strong>%s</strong>.";		// Besoin de traduction
#elif L==5
	"El pa&iacute;s <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";	// Okoteve traduccin
#elif L==6
	"Il paese <strong>%s</strong> &egrave; stato rimosso come <strong>%s</strong>.";
#elif L==7
	"The country <strong>%s</strong> has been renamed as <strong>%s</strong>.";		// Potrzebujesz tlumaczenie
#elif L==8
	"The country <strong>%s</strong> has been renamed as <strong>%s</strong>.";		// Necessita de traduo
#endif

const char *Txt_The_country_of_the_institution_X_has_changed_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==0
	"El pa&iacute;s de la instituci&oacute;n <strong>%s</strong>"
	" ha cambiado a <strong>%s</strong>.";	// Necessita traduccio
#elif L==1
	"The country of the institution <strong>%s</strong>"
	" has changed to <strong>%s</strong>.";
#elif L==2
	"The country of the institution <strong>%s</strong>"
	" has changed to <strong>%s</strong>.";
#elif L==3
	"El pa&iacute;s de la instituci&oacute;n <strong>%s</strong>"
	" ha cambiado a <strong>%s</strong>.";
#elif L==4
	"The country of the institution <strong>%s</strong>"
	" has changed to <strong>%s</strong>.";
#elif L==5
	"El pa&iacute;s de la instituci&oacute;n <strong>%s</strong>"
	" ha cambiado a <strong>%s</strong>.";	// Okoteve traduccin
#elif L==6
	"Il paese dell'istituzione <strong>%s</strong>"
	" &egrave; cambiato in <strong>%s</strong>.";
#elif L==7
	"The country of the institution <strong>%s</strong>"
	" has changed to <strong>%s</strong>.";	// Potrzebujesz tlumaczenie
#elif L==8
	"The country of the institution <strong>%s</strong>"
	" has changed to <strong>%s</strong>.";
#endif

const char *Txt_The_course_X_already_exists =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"La asignatura <strong>%s</strong> ya existe.";		// Necessita traduccio
#elif L==1
	"Die Kurs <strong>%s</strong> wurde bereits eingegeben.";
#elif L==2
	"The course <strong>%s</strong> already exists.";
#elif L==3
	"La asignatura <strong>%s</strong> ya existe.";
#elif L==4
	"The course <strong>%s</strong> already exists.";	// Besoin de traduction
#elif L==5
	"La asignatura <strong>%s</strong> ya existe.";		// Okoteve traduccin
#elif L==6
	"Il corso <strong>%s</strong> gi&agrave; esiste.";
#elif L==7
	"The course <strong>%s</strong> already exists.";	// Potrzebujesz tlumaczenie
#elif L==8
	"The course <strong>%s</strong> already exists.";	// Necessita de traduo
#endif

const char *Txt_The_course_X_already_exists_in_year_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==0
	"La asignatura <strong>%s</strong> ya existe en el curso %s.";	// Necessita traduccio
#elif L==1
	"Die Kurs <strong>%s</strong> wurde bereits in Studienjahr %s eingegeben.";
#elif L==2
	"The course <strong>%s</strong> already exists in year %s.";
#elif L==3
	"La asignatura <strong>%s</strong> ya existe en el curso %s.";
#elif L==4
	"The course <strong>%s</strong> already exists in year %s.";	// Besoin de traduction
#elif L==5
	"La asignatura <strong>%s</strong> ya existe en el curso %s.";	// Okoteve traduccin
#elif L==6
	"Il corso <strong>%s</strong> esiste gi&agrave; nell'anno %s.";
#elif L==7
	"The course <strong>%s</strong> already exists in year %s.";	// Potrzebujesz tlumaczenie
#elif L==8
	"The course <strong>%s</strong> already exists in year %s.";	// Necessita de traduo
#endif

const char *Txt_The_course_X_has_been_moved_to_the_degree_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==0
	"La asignatura <strong>%s</strong> se ha movido"
	" a la titulaci&oacute;n <strong>%s</strong>.";	// Necessita traduccio
#elif L==1
	"The course <strong>%s</strong> has been moved"
	" to the degree <strong>%s</strong>.";		// Need bersetzung
#elif L==2
	"The course <strong>%s</strong> has been moved"
	" to the degree <strong>%s</strong>.";
#elif L==3
	"La asignatura <strong>%s</strong> se ha movido"
	" a la titulaci&oacute;n <strong>%s</strong>.";
#elif L==4
	"The course <strong>%s</strong> has been moved"
	" to the degree <strong>%s</strong>.";		// Besoin de traduction
#elif L==5
	"La asignatura <strong>%s</strong> se ha movido"
	" a la titulaci&oacute;n <strong>%s</strong>.";	// Okoteve traduccin
#elif L==6
	"Il corso <strong>%s</strong> &egrave; stato spostato"
	" dalla laurea <strong>%s</strong>.";
#elif L==7
	"The course <strong>%s</strong> has been moved"
	" to the degree <strong>%s</strong>.";		// Potrzebujesz tlumaczenie
#elif L==8
	"The course <strong>%s</strong> has been moved"
	" to the degree <strong>%s</strong>.";		// Necessita de traduo
#endif

const char *Txt_The_date_of_the_holiday_X_has_changed_to_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==0
	"La fecha de la festividad <strong>%s</strong> ha cambiado a <strong>%s</strong>.";	// Necessita traduccio
#elif L==1
	"The date of the holiday <strong>%s</strong> has changed to <strong>%s</strong>.";	// Need bersetzung
#elif L==2
	"The date of the holiday <strong>%s</strong> has changed to <strong>%s</strong>.";
#elif L==3
	"La fecha de la festividad <strong>%s</strong> ha cambiado a <strong>%s</strong>.";
#elif L==4
	"The date of the holiday <strong>%s</strong> has changed to <strong>%s</strong>.";	// Besoin de traduction
#elif L==5
	"La fecha de la festividad <strong>%s</strong> ha cambiado a <strong>%s</strong>.";	// Okoteve traduccin
#elif L==6
	"La data delle festivit&agrave; <strong>%s</strong> &egrave; cambiata a <strong>%s</strong>.";
#elif L==7
	"The date of the holiday <strong>%s</strong> has changed to <strong>%s</strong>.";	// Potrzebujesz tlumaczenie
#elif L==8
	"The date of the holiday <strong>%s</strong> has changed to <strong>%s</strong>.";	// Necessita de traduo
#endif

const char *Txt_The_date_range_must_be_less_than_or_equal_to_X_days =	// Warning: it is very important to include %u in the following sentences
#if   L==0
	"El rango de fechas debe ser menor o igual a %u d&iacute;as.";	// Necessita traduccio
#elif L==1
	"The date range must be less than or equal to %u days.";	// Need bersetzung
#elif L==2
	"The date range must be less than or equal to %u days.";
#elif L==3
	"El rango de fechas debe ser menor o igual a %u d&iacute;as.";
#elif L==4
	"The date range must be less than or equal to %u days.";	// Besoin de traduction
#elif L==5
	"El rango de fechas debe ser menor o igual a %u d&iacute;as.";	// Okoteve traduccin
#elif L==6
	"Il range della data deve essere minore o uguale a %u giorni.";
#elif L==7
	"The date range must be less than or equal to %u days.";	// Potrzebujesz tlumaczenie
#elif L==8
	"The date range must be less than or equal to %u days.";	// Necessita de traduo
#endif

const char *Txt_The_date_time_of_opening_of_groups_has_changed =
#if   L==0
	"La data / hora d'obertura de grups s'ha modificat.";
#elif L==1
	"The date / time of opening of groups has changed.";			// Need bersetzung
#elif L==2
	"The date / time of opening of groups has changed.";
#elif L==3
	"Los cambios de grupo solicitados se han realizado correctamente.";
#elif L==4
	"The date / time of opening of groups has changed.";			// Besoin de traduction
#elif L==5
	"Los cambios de grupo solicitados se han realizado correctamente.";	// Okoteve traduccin
#elif L==6
	"La data / ora di apertura dei gruppi &egrave; cambiata.";
#elif L==7
	"The date / time of opening of groups has changed.";			// Potrzebujesz tlumaczenie
#elif L==8
	"The date / time of opening of groups has changed.";			// Necessita de traduo
#endif

const char *Txt_The_degree_X_already_exists =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"La titulaci&oacute;n <strong>%s</strong> ya existe.";	// Necessita traduccio
#elif L==1
	"Der Studiengang <strong>%s</strong> wurde bereits eingegeben.";
#elif L==2
	"The degree <strong>%s</strong> already exists.";
#elif L==3
	"La titulaci&oacute;n <strong>%s</strong> ya existe.";
#elif L==4
	"The degree <strong>%s</strong> already exists.";	// Besoin de traduction
#elif L==5
	"La titulaci&oacute;n <strong>%s</strong> ya existe.";	// Okoteve traduccin
#elif L==6
	"La laurea <strong>%s</strong> gi&agrave; esiste.";
#elif L==7
	"The degree <strong>%s</strong> already exists.";	// Potrzebujesz tlumaczenie
#elif L==8
	"The degree <strong>%s</strong> already exists.";	// Necessita de traduo
#endif

const char *Txt_The_degree_has_no_longer_a_year_for_optional_courses =
#if   L==0
	"La titulaci&oacute;n ya no tiene un curso de optativas.";	// Necessita traduccio
#elif L==1
	"The degree has no longer a year for optional courses.";	// Need bersetzung
#elif L==2
	"The degree has no longer a year for optional courses.";
#elif L==3
	"La titulaci&oacute;n ya no tiene un curso de optativas.";
#elif L==4
	"The degree has no longer a year for optional courses.";	// Besoin de traduction
#elif L==5
	"La titulaci&oacute;n ya no tiene un curso de optativas.";	// Okoteve traduccin
#elif L==6
	"La laurea non ha un anno pi&ugrave; lungo per corsi opzionali.";
#elif L==7
	"The degree has no longer a year for optional courses.";	// Potrzebujesz tlumaczenie
#elif L==8
	"The degree has no longer a year for optional courses.";	// Necessita de traduo
#endif

const char *Txt_The_degree_now_has_a_year_for_optional_courses =
#if   L==0
	"La titulaci&oacute;n ahora tiene un curso de optativas.";	// Necessita traduccio
#elif L==1
	"The degree now has a year for optional courses.";		// Need bersetzung
#elif L==2
	"The degree now has a year for optional courses.";
#elif L==3
	"La titulaci&oacute;n ahora tiene un curso de optativas.";
#elif L==4
	"The degree now has a year for optional courses.";		// Besoin de traduction
#elif L==5
	"La titulaci&oacute;n ahora tiene un curso de optativas.";	// Okoteve traduccin
#elif L==6
	"La laurea ora ha un anno per corsi opzionali.";
#elif L==7
	"The degree now has a year for optional courses.";		// Potrzebujesz tlumaczenie
#elif L==8
	"The degree now has a year for optional courses.";		// Necessita de traduo
#endif

const char *Txt_The_department_X_already_exists =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"El departamento <strong>%s</strong> ya existe.";	// Necessita traduccio
#elif L==1
	"Die Abteilung <strong>%s</strong> wurde bereits eingegeben.";
#elif L==2
	"The department <strong>%s</strong> already exists.";
#elif L==3
	"El departamento <strong>%s</strong> ya existe.";
#elif L==4
	"The department <strong>%s</strong> already exists.";	// Besoin de traduction
#elif L==5
	"El departamento <strong>%s</strong> ya existe.";	// Okoteve traduccin
#elif L==6
	"IL dipartimento <strong>%s</strong> esiste gi&agrave;.";
#elif L==7
	"The department <strong>%s</strong> already exists.";	// Potrzebujesz tlumaczenie
#elif L==8
	"The department <strong>%s</strong> already exists.";	// Necessita de traduo
#endif

const char *Txt_The_department_X_has_been_renamed_as_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==0
	"El departamento <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";	// Necessita traduccio
#elif L==1
	"The department <strong>%s</strong> has been renamed as <strong>%s</strong>.";		// Need bersetzung
#elif L==2
	"The department <strong>%s</strong> has been renamed as <strong>%s</strong>.";
#elif L==3
	"El departamento <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";
#elif L==4
	"The department <strong>%s</strong> has been renamed as <strong>%s</strong>.";		// Besoin de traduction
#elif L==5
	"El departamento <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";	// Okoteve traduccin
#elif L==6
	"Il dipartimento <strong>%s</strong> &egrave; stato rinominato come <strong>%s</strong>.";
#elif L==7
	"The department <strong>%s</strong> has been renamed as <strong>%s</strong>.";		// Potrzebujesz tlumaczenie
#elif L==8
	"The department <strong>%s</strong> has been renamed as <strong>%s</strong>.";		// Necessita de traduo
#endif

const char *Txt_The_email_X_has_been_confirmed =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"El correo electr&oacute;nico <strong>%s</strong> ha sido confirmado.";	// Necessita traduccio
#elif L==1
	"The e-mail <strong>%s</strong> has been confirmed.";			// Need bersetzung
#elif L==2
	"The e-mail <strong>%s</strong> has been confirmed.";
#elif L==3
	"El correo electr&oacute;nico <strong>%s</strong> ha sido confirmado.";
#elif L==4
	"The e-mail <strong>%s</strong> has been confirmed.";			// Besoin de traduction
#elif L==5
	"El correo electr&oacute;nico <strong>%s</strong> ha sido confirmado.";	// Okoteve traduccin
#elif L==6
	"The e-mail <strong>%s</strong> has been confirmed.";			// Bisogno di traduzione
#elif L==7
	"The e-mail <strong>%s</strong> has been confirmed.";			// Potrzebujesz tlumaczenie
#elif L==8
	"The e-mail <strong>%s</strong> has been confirmed.";			// Necessita de traduo
#endif

const char *Txt_The_email_address_entered_X_is_not_valid =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"La direcci&oacute;n de correo introducida"
	" <strong>%s</strong> no es v&aacute;lida.";	// Necessita traduccio
#elif L==1
	"The e-mail address entered"
	" <strong>%s</strong> is not valid.";		// Need bersetzung
#elif L==2
	"The e-mail address entered"
	" <strong>%s</strong> is not valid.";
#elif L==3
	"La direcci&oacute;n de correo introducida"
	" <strong>%s</strong> no es v&aacute;lida.";
#elif L==4
	"The e-mail address entered"
	" <strong>%s</strong> is not valid.";		// Besoin de traduction
#elif L==5
	"La direcci&oacute;n de correo introducida"
	" <strong>%s</strong> no es v&aacute;lida.";	// Okoteve traduccin
#elif L==6
	"Il e-mail immesso"
	" <strong>%s</strong> non &egrave; valido.";
#elif L==7
	"The e-mail address entered"
	" <strong>%s</strong> is not valid.";		// Potrzebujesz tlumaczenie
#elif L==8
	"The e-mail address entered"
	" <strong>%s</strong> is not valid.";		// Necessita de traduo
#endif

const char *Txt_The_email_address_X_had_been_registered_by_another_user =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"La direcci&oacute;n de correo <strong>%s</strong>"
	" ya hab&iacute;a sido registrada por otro usuario.";	// Necessita traduccio
#elif L==1
	"The e-mail address <strong>%s</strong>"
	" had been registered by another user.";		// Need bersetzung
#elif L==2
	"The e-mail address <strong>%s</strong>"
	" had been registered by another user.";
#elif L==3
	"La direcci&oacute;n de correo <strong>%s</strong>"
	" ya hab&iacute;a sido registrada por otro usuario.";
#elif L==4
	"The e-mail address <strong>%s</strong>"
	" had been enrolled by another user.";			// Besoin de traduction
#elif L==5
	"La direcci&oacute;n de correo <strong>%s</strong>"
	" ya hab&iacute;a sido registrada por otro usuario.";	// Okoteve traduccin
#elif L==6
	"Il e-mail <strong>%s</strong>"
	" &egrave; stato registrato da un altro utente.";
#elif L==7
	"The e-mail address <strong>%s</strong>"
	" had been registered by another user.";		// Potrzebujesz tlumaczenie
#elif L==8
	"The e-mail address <strong>%s</strong>"
	" had been registered by another user.";		// Necessita de traduo
#endif

const char *Txt_The_email_address_has_not_been_confirmed =
#if   L==0
	"La direcci&oacute;n de correo electr&oacute;nico"
	" no ha podido confirmarse.";			// Necessita traduccio
#elif L==1
	"The email address has not been confirmed.";	// Need bersetzung
#elif L==2
	"The email address has not been confirmed.";
#elif L==3
	"La direcci&oacute;n de correo electr&oacute;nico"
	" no ha podido confirmarse.";
#elif L==4
	"The email address has not been confirmed.";	// Besoin de traduction
#elif L==5
	"La direcci&oacute;n de correo electr&oacute;nico"
	" no ha podido confirmarse.";			// Okoteve traduccin
#elif L==6
	"The email address has not been confirmed.";	// Bisogno di traduzione
#elif L==7
	"The email address has not been confirmed.";	// Potrzebujesz tlumaczenie
#elif L==8
	"The email address has not been confirmed.";	// Necessita de traduo
#endif

const char *Txt_The_email_address_X_matches_the_one_you_had_previously_registered =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"La direcci&oacute;n de correo <strong>%s</strong>"
	" coincide con la que ya ten&iacute;a registrada previamente.";	// Necessita traduccio
#elif L==1
	"The e-mail address <strong>%s</strong>"
	" matches the one you had previously registered.";		// Need bersetzung
#elif L==2
	"The e-mail address <strong>%s</strong>"
	" matches the one you had previously registered.";
#elif L==3
	"La direcci&oacute;n de correo <strong>%s</strong>"
	" coincide con la que ya ten&iacute;a registrada previamente.";
#elif L==4
	"The e-mail address <strong>%s</strong>"
	" matches the one you had previously registered.";		// Besoin de traduction
#elif L==5
	"La direcci&oacute;n de correo <strong>%s</strong>"
	" coincide con la que ya ten&iacute;a registrada previamente.";	// Okoteve traduccin
#elif L==6
	"Il e-mail <strong>%s</strong>"
	" coincide con quellao che hai precedentemente registrato.";
#elif L==7
	"The e-mail address <strong>%s</strong>"
	" matches the one you had previously registered.";		// Potrzebujesz tlumaczenie
#elif L==8
	"The e-mail address <strong>%s</strong>"
	" matches the one you had previously registered.";		// Necessita de traduo
#endif

const char *Txt_The_enrollment_of_students_into_groups_of_type_X_is_now_mandatory =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"La adscripci&oacute;n de estudiantes a grupos del tipo <strong>%s</strong> es ahora obligatoria.";	// Necessita traduccio
#elif L==1
	"The enrollment of students into groups of type <strong>%s</strong> is now mandatory.";			// Need bersetzung
#elif L==2
	"The enrollment of students into groups of type <strong>%s</strong> is now mandatory.";
#elif L==3
	"La adscripci&oacute;n de estudiantes a grupos del tipo <strong>%s</strong> es ahora obligatoria.";
#elif L==4
	"The enrollment of students into groups of type <strong>%s</strong> is now mandatory.";			// Besoin de traduction
#elif L==5
	"La adscripci&oacute;n de estudiantes a grupos del tipo <strong>%s</strong> es ahora obligatoria.";	// Okoteve traduccin
#elif L==6
	"La registrazione di studenti nei gruppi di tipo <strong>%s</strong> &egrave; ora obbligatoria.";
#elif L==7
	"The enrollment of students into groups of type <strong>%s</strong> is now mandatory.";			// Potrzebujesz tlumaczenie
#elif L==8
	"The enrollment of students into groups of type <strong>%s</strong> is now mandatory.";			// Necessita de traduo
#endif

const char *Txt_The_enrollment_of_students_into_groups_of_type_X_is_now_voluntary =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"La adscripci&oacute;n de estudiantes a grupos del tipo <strong>%s</strong> es ahora voluntaria.";	// Necessita traduccio
#elif L==1
	"The enrollment of students into groups of type <strong>%s</strong> is now voluntary.";			// Need bersetzung
#elif L==2
	"The enrollment of students into groups of type <strong>%s</strong> is now voluntary.";
#elif L==3
	"La adscripci&oacute;n de estudiantes a grupos del tipo <strong>%s</strong> es ahora voluntaria.";
#elif L==4
	"The enrollment of students into groups of type <strong>%s</strong> is now voluntary.";			// Besoin de traduction
#elif L==5
	"La adscripci&oacute;n de estudiantes a grupos del tipo <strong>%s</strong> es ahora voluntaria.";	// Okoteve traduccin
#elif L==6
	"La registrazione di studenti nei gruppi di tipo <strong>%s</strong> &egrave; ora volontaria.";
#elif L==7
	"The enrollment of students into groups of type <strong>%s</strong> is now voluntary.";			// Potrzebujesz tlumaczenie
#elif L==8
	"The enrollment of students into groups of type <strong>%s</strong> is now voluntary.";			// Necessita de traduo
#endif

const char *Txt_The_event_has_been_modified =
#if   L==0
	"El evento ha sido modificado.";	// Necessita traduccio
#elif L==1
	"Der Ereignis wurde ge&auml;ndert.";
#elif L==2
	"The event has been modified.";
#elif L==3
	"El evento ha sido modificado.";
#elif L==4
	"L'&eacute;v&eacute;nement a &eacute;t&eacute; modifi&eacute;.";
#elif L==5
	"El evento ha sido modificado.";	// Okoteve traduccin
#elif L==6
	"L'evento &egrave; stata modificato.";
#elif L==7
	"The event has been modified.";		// Potrzebujesz tlumaczenie
#elif L==8
	"O evento foi modificado.";
#endif

const char *Txt_The_file_X_has_been_placed_inside_the_folder_Y =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"El archivo <strong>%s</strong> se ha almacenado"
	" dentro de la carpeta <strong>%s</strong>.";		// Necessita traduccio
#elif L==1
	"Die Datei <strong>%s</strong> wurde"
	" in das Verzeichnis <strong>%s</strong> gesteckt.";
#elif L==2
	"The file <strong>%s</strong> has been placed"
	" inside the folder <strong>%s</strong>.";
#elif L==3
	"El archivo <strong>%s</strong> se ha almacenado"
	" dentro de la carpeta <strong>%s</strong>.";
#elif L==4
	"Le fichier <strong>%s</strong> a &eacute;t&eacute; plac&eacute;"
	" dans le r&eacute;pertoire <strong>%s</strong>.";
#elif L==5
	"El archivo <strong>%s</strong> se ha almacenado"
	" dentro de la carpeta <strong>%s</strong>.";		// Okoteve traduccin
#elif L==6
	"Il file <strong>%s</strong> &egrave; stato messo"
	" all'interno della cartella <strong>%s</strong>.";
#elif L==7
	"The file <strong>%s</strong> has been placed"
	" inside the folder <strong>%s</strong>.";		// Potrzebujesz tlumaczenie
#elif L==8
	"O arquivo <strong>%s</strong> foi colocado"
	" dentro do diret&oacute;rio <strong>%s</strong>.";
#endif

const char *Txt_The_file_is_not_jpg =
#if   L==0
	"El archivo no es <em>jpg</em>.";	// Necessita traduccio
#elif L==1
	"The file is not <em>jpg</em>.";	// Need bersetzung
#elif L==2
	"The file is not <em>jpg</em>.";
#elif L==3
	"El archivo no es <em>jpg</em>.";
#elif L==4
	"The file is not <em>jpg</em>.";	// Besoin de traduction
#elif L==5
	"El archivo no es <em>jpg</em>.";	// Okoteve traduccin
#elif L==6
	"IL file non &egrave; <em>jpg</em>.";
#elif L==7
	"The file is not <em>jpg</em>.";	// Potrzebujesz tlumaczenie
#elif L==8
	"The file is not <em>jpg</em>.";	// Necessita de traduo
#endif

const char *Txt_The_file_is_not_xml =
#if   L==0
	"El archivo no es <em>xml</em>.";	// Necessita traduccio
#elif L==1
	"The file is not <em>xml</em>.";	// Need bersetzung
#elif L==2
	"The file is not <em>xml</em>.";
#elif L==3
	"El archivo no es <em>xml</em>.";
#elif L==4
	"The file is not <em>xml</em>.";	// Besoin de traduction
#elif L==5
	"El archivo no es <em>xml</em>.";	// Okoteve traduccin
#elif L==6
	"IL file non &egrave; <em>xml</em>.";
#elif L==7
	"The file is not <em>xml</em>.";	// Potrzebujesz tlumaczenie
#elif L==8
	"The file is not <em>xml</em>.";	// Necessita de traduo
#endif

const char *Txt_The_file_of_folder_no_longer_exists_or_is_now_hidden =
#if   L==0
	"L'arxiu o carpeta ja no existeix o est&agrave; ara ocult.";
#elif L==1
	"Die Datei oder Ordner existiert nicht mehr oder ist jetzt versteckt.";
#elif L==2
	"The file or folder no longer exists or is now hidden.";
#elif L==3
	"El archivo o carpeta ya no existe o est&aacute; ahora oculto.";
#elif L==4
	"Le fichier ou r&eacute;pertoire n'existe plus ou est maintenant cach&eacute;.";
#elif L==5
	"El archivo o carpeta ya no existe o est&aacute; ahora oculto.";	// Okoteve traduccin
#elif L==6
	"Il file o cartella non esiste pi&ugrave; o &egrave; nascosta.";
#elif L==7
	"Plik lub folder ju&zdot; nie istnieje lub jest ukryte.";
#elif L==8
	"O arquivo ou diret&oacute;rio n&atilde;o existe mais ou est&aacute; escondido.";
#endif

const char *Txt_The_file_type_is_X_and_should_be_HTML_or_ZIP =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"El tipo de archivo es %s y debe ser <em>HTML</em> o <em>ZIP</em>.";	// Necessita traduccio
#elif L==1
	"The file type is %s and should be <em>HTML</em> or <em>ZIP</em>.";	// Need bersetzung
#elif L==2
	"The file type is %s and should be <em>HTML</em> or <em>ZIP</em>.";
#elif L==3
	"El tipo de archivo es %s y debe ser <em>HTML</em> o <em>ZIP</em>.";
#elif L==4
	"The file type is %s and should be <em>HTML</em> or <em>ZIP</em>.";	// Besoin de traduction
#elif L==5
	"El tipo de archivo es %s y debe ser <em>HTML</em> o <em>ZIP</em>.";	// Okoteve traduccin
#elif L==6
	"Il tipo di file &egrave; %s e dovrebbe essere <em>HTML</em> o <em>ZIP</em>.";
#elif L==7
	"The file type is %s and should be <em>HTML</em> or <em>ZIP</em>.";	// Potrzebujesz tlumaczenie
#elif L==8
	"The file type is %s and should be <em>HTML</em> or <em>ZIP</em>.";	// Necessita de traduo
#endif

const char *Txt_The_file_type_should_be_HTML_or_ZIP =
#if   L==0
	"El tipo de archivo debe ser <em>HTML</em> o <em>ZIP</em>.";	// Necessita traduccio
#elif L==1
	"The file type should be <em>HTML</em> or <em>ZIP</em>.";	// Need bersetzung
#elif L==2
	"The file type should be <em>HTML</em> or <em>ZIP</em>.";
#elif L==3
	"El tipo de archivo debe ser <em>HTML</em> o <em>ZIP</em>.";
#elif L==4
	"The file type should be <em>HTML</em> or <em>ZIP</em>.";	// Besoin de traduction
#elif L==5
	"El tipo de archivo debe ser <em>HTML</em> o <em>ZIP</em>.";	// Okoteve traduccin
#elif L==6
	"Il tipo di file dovrebbe essere <em>HTML</em> o <em>ZIP</em>.";
#elif L==7
	"The file type should be <em>HTML</em> or <em>ZIP</em>.";	// Potrzebujesz tlumaczenie
#elif L==8
	"The file type should be <em>HTML</em> or <em>ZIP</em>.";	// Necessita de traduo
#endif

const char *Txt_The_folder_X_has_been_created_inside_the_folder_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==0
	"Se ha creado la carpeta <strong>%s</strong> dentro de la carpeta <strong>%s</strong>.";	// Necessita traduccio
#elif L==1
	"Das Verzeichnis <strong>%s</strong> wurde im Verzeichnis <strong>%s</strong> erstellt.";
#elif L==2
	"The folder <strong>%s</strong> has been created inside the folder <strong>%s</strong>.";
#elif L==3
	"Se ha creado la carpeta <strong>%s</strong> dentro de la carpeta <strong>%s</strong>.";
#elif L==4
	"Le r&eacute;pertoire <strong>%s</strong> a &eacute;t&eacute; cr&eacute;&eacute; dans le r&eacute;pertoire <strong>%s</strong>.";
#elif L==5
	"Se ha creado la carpeta <strong>%s</strong> dentro de la carpeta <strong>%s</strong>.";	// Okoteve traduccin
#elif L==6
	"La cartella <strong>%s</strong> &egrave; stata creata all'interno della cartella <strong>%s</strong>.";
#elif L==7
	"The folder <strong>%s</strong> has been created inside the folder <strong>%s</strong>.";	// Potrzebujesz tlumaczenie
#elif L==8
	"O diret&oacute;rio <strong>%s</strong> foi criado dentro do diret&oacute;rio <strong>%s</strong>.";
#endif

const char *Txt_The_folder_is_empty =
#if   L==0
	"La carpeta est&agrave; buida.";
#elif L==1
	"Der Verzeichnis is leer.";
#elif L==2
	"The folder is empty.";
#elif L==3
	"La carpeta est&aacute; vac&iacute;a.";
#elif L==4
	"Le r&eacute;pertoire est vide.";
#elif L==5
	"La carpeta est&aacute; vac&iacute;a.";	// Okoteve traduccin
#elif L==6
	"La cartella &egrave; vuota.";
#elif L==7
	"Folder jest pusty.";
#elif L==8
	"O diret&oacute;rio est&aacute; vazio.";
#endif

const char *Txt_The_folder_name_X_has_changed_to_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==0
	"Se ha cambiado el nombre de la carpeta <strong>%s</strong> por <strong>%s</strong>.";	// Necessita traduccio
#elif L==1
	"Das Verzeichnis <strong>%s</strong> wurde umbenannt in <strong>%s</strong>.";
#elif L==2
	"The folder name <strong>%s</strong> has changed to <strong>%s</strong>.";
#elif L==3
	"Se ha cambiado el nombre de la carpeta <strong>%s</strong> por <strong>%s</strong>.";
#elif L==4
	"The folder name <strong>%s</strong> has changed to <strong>%s</strong>.";		// Besoin de traduction
#elif L==5
	"Se ha cambiado el nombre de la carpeta <strong>%s</strong> por <strong>%s</strong>.";	// Okoteve traduccin
#elif L==6
	"Il nome della cartella <strong>%s</strong> &egrave; cambiato in <strong>%s</strong>.";
#elif L==7
	"The folder name <strong>%s</strong> has changed to <strong>%s</strong>.";		// Potrzebujesz tlumaczenie
#elif L==8
	"The folder name <strong>%s</strong> has changed to <strong>%s</strong>.";		// Necessita de traduo
#endif

const char *Txt_The_folder_name_X_has_not_changed =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"El nombre de la carpeta <strong>%s</strong> no ha cambiado.";	// Necessita traduccio
#elif L==1
	"Der Name des Verzeichnisses <strong>%s</strong> wurde nicht ge&auml;ndert.";
#elif L==2
	"The folder name <strong>%s</strong> has not changed.";
#elif L==3
	"El nombre de la carpeta <strong>%s</strong> no ha cambiado.";
#elif L==4
	"The folder name <strong>%s</strong> has not changed.";		// Besoin de traduction
#elif L==5
	"El nombre de la carpeta <strong>%s</strong> no ha cambiado.";	// Okoteve traduccin
#elif L==6
	"Il nome della cartella <strong>%s</strong> non &egrave; cambiato.";
#elif L==7
	"The folder name <strong>%s</strong> has not changed.";		// Potrzebujesz tlumaczenie
#elif L==8
	"The folder name <strong>%s</strong> has not changed.";		// Necessita de traduo
#endif

const char *Txt_The_folder_name_X_has_not_changed_because_there_is_already_a_folder_or_a_file_with_the_name_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==0
	"El nombre de la carpeta <strong>%s</strong> no ha cambiado"
	" porque ya existe una carpeta o un archivo con el nombre <strong>%s</strong>.";	// Necessita traduccio
#elif L==1
	"Der Name des Verzeichnisses <strong>%s</strong> wurde nicht ge&auml;ndert,"
	" weil bereits ein Verzeichnis oder eine Datei mit Namen <strong>%s</strong> besteht.";
#elif L==2
	"The folder name <strong>%s</strong> has not changed"
	" because there is already a folder or a file with the name <strong>%s</strong>.";
#elif L==3
	"El nombre de la carpeta <strong>%s</strong> no ha cambiado"
	" porque ya existe una carpeta o un archivo con el nombre <strong>%s</strong>.";
#elif L==4
	"The folder name <strong>%s</strong> has not changed"
	" because there is already a folder or a file with the name <strong>%s</strong>.";	// Besoin de traduction
#elif L==5
	"El nombre de la carpeta <strong>%s</strong> no ha cambiado"
	" porque ya existe una carpeta o un archivo con el nombre <strong>%s</strong>.";	// Okoteve traduccin
#elif L==6
	"Il nome della cartella <strong>%s</strong> non &egrave; cambiato"
	" perch&egrave; c'&egrave; gi&agrave; una cartella o un file con il nome <strong>%s</strong>.";
#elif L==7
	"The folder name <strong>%s</strong> has not changed"
	" because there is already a folder or a file with the name <strong>%s</strong>.";	// Potrzebujesz tlumaczenie
#elif L==8
	"The folder name <strong>%s</strong> has not changed"
	" because there is already a folder or a file with the name <strong>%s</strong>.";	// Necessita de traduo
#endif

const char *Txt_The_graph_shows_the_NUMBER =
#if   L==0
	"La gr&aacute;fica muestra el";	// Necessita traduccio
#elif L==1
	"Die Grafik zeigt die";
#elif L==2
	"The graph shows the";
#elif L==3
	"La gr&aacute;fica muestra el";
#elif L==4
	"Le graphique montre le";
#elif L==5
	"La gr&aacute;fica muestra el";	// Okoteve traduccin
#elif L==6
	"Il grafico mostra il";
#elif L==7
	"The graph shows the";		// Potrzebujesz tlumaczenie
#elif L==8
	"O gr&aacute;fico mostra o";
#endif

const char *Txt_The_group_X_already_exists =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"El grupo <strong>%s</strong> ya existe.";		// Necessita traduccio
#elif L==1
	"Die Gruppe <strong>%s</strong> wurde bereits eingegeben.";
#elif L==2
	"The group <strong>%s</strong> already exists.";
#elif L==3
	"El grupo <strong>%s</strong> ya existe.";
#elif L==4
	"The group <strong>%s</strong> already exists.";	// Besoin de traduction
#elif L==5
	"El grupo <strong>%s</strong> ya existe.";		// Okoteve traduccin
#elif L==6
	"Il gruppo <strong>%s</strong> gi&agrave; esiste.";
#elif L==7
	"The group <strong>%s</strong> already exists.";	// Potrzebujesz tlumaczenie
#elif L==8
	"The group <strong>%s</strong> already exists.";	// Necessita de traduo
#endif

const char *Txt_The_group_X_has_been_renamed_as_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==0
	"El grupo <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";	// Necessita traduccio
#elif L==1
	"The group <strong>%s</strong> has been renamed as <strong>%s</strong>.";	// Need bersetzung
#elif L==2
	"The group <strong>%s</strong> has been renamed as <strong>%s</strong>.";
#elif L==3
	"El grupo <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";
#elif L==4
	"The group <strong>%s</strong> has been renamed as <strong>%s</strong>.";	// Besoin de traduction
#elif L==5
	"El grupo <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";	// Okoteve traduccin
#elif L==6
	"Il gruppo <strong>%s</strong> &egrave; stato rinominato come <strong>%s</strong>.";
#elif L==7
	"The group <strong>%s</strong> has been renamed as <strong>%s</strong>.";	// Potrzebujesz tlumaczenie
#elif L==8
	"The group <strong>%s</strong> has been renamed as <strong>%s</strong>.";	// Necessita de traduo
#endif

const char *Txt_The_group_X_is_now_closed =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"El grupo <strong>%s</strong> est&aacute; ahora cerrado.";	// Necessita traduccio
#elif L==1
	"The group <strong>%s</strong> is now closed.";			// Need bersetzung
#elif L==2
	"The group <strong>%s</strong> is now closed.";
#elif L==3
	"El grupo <strong>%s</strong> est&aacute; ahora cerrado.";
#elif L==4
	"The group <strong>%s</strong> is now closed.";			// Besoin de traduction
#elif L==5
	"El grupo <strong>%s</strong> est&aacute; ahora cerrado.";	// Okoteve traduccin
#elif L==6
	"Il gruppo <strong>%s</strong> &egrave; ora chiuso.";
#elif L==7
	"The group <strong>%s</strong> is now closed.";			// Potrzebujesz tlumaczenie
#elif L==8
	"The group <strong>%s</strong> is now closed.";			// Necessita de traduo
#endif

const char *Txt_The_group_X_is_now_open =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"El grupo <strong>%s</strong> est&aacute; ahora abierto.";	// Necessita traduccio
#elif L==1
	"The group <strong>%s</strong> is now open.";			// Need bersetzung
#elif L==2
	"The group <strong>%s</strong> is now open.";
#elif L==3
	"El grupo <strong>%s</strong> est&aacute; ahora abierto.";
#elif L==4
	"The group <strong>%s</strong> is now open.";			// Besoin de traduction
#elif L==5
	"El grupo <strong>%s</strong> est&aacute; ahora abierto.";	// Okoteve traduccin
#elif L==6
	"Il gruppo <strong>%s</strong> &egrave; ora aperto.";
#elif L==7
	"The group <strong>%s</strong> is now open.";			// Potrzebujesz tlumaczenie
#elif L==8
	"The group <strong>%s</strong> is now open.";			// Necessita de traduo
#endif

const char *Txt_The_group_X_now_has_no_limit_of_students =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"El grupo <strong>%s</strong> no tiene ahora l&iacute;mite de estudiantes.";	// Necessita traduccio
#elif L==1
	"The group <strong>%s</strong> now has no limit of students.";
#elif L==2
	"The group <strong>%s</strong> now has no limit of students.";
#elif L==3
	"El grupo <strong>%s</strong> no tiene ahora l&iacute;mite de estudiantes.";
#elif L==4
	"The group <strong>%s</strong> now has no limit of students.";
#elif L==5
	"El grupo <strong>%s</strong> no tiene ahora l&iacute;mite de estudiantes.";	// Okoteve traduccin
#elif L==6
	"Il gruppo <strong>%s</strong> ora non ha limite di studenti.";
#elif L==7
	"The group <strong>%s</strong> now has no limit of students.";			// Potrzebujesz tlumaczenie
#elif L==8
	"The group <strong>%s</strong> now has no limit of students.";
#endif

const char *Txt_The_groups_will_automatically_open =
#if   L==0
	"Els grups s'obriran autom&agrave;ticament.";
#elif L==1
	"The groups will automatically open.";			// Need bersetzung
#elif L==2
	"The groups will automatically open.";
#elif L==3
	"Los grupos se abrir&aacute;n autom&aacute;ticamente.";
#elif L==4
	"The groups will automatically open.";			// Besoin de traduction
#elif L==5
	"Los grupos se abrir&aacute;n autom&aacute;ticamente.";	// Okoteve traduccin
#elif L==6
	"I gruppi si aprir&agrave; automaticamente.";
#elif L==7
	"The groups will automatically open.";			// Potrzebujesz tlumaczenie
#elif L==8
	"The groups will automatically open.";			// Necessita de traduo
#endif

const char *Txt_The_groups_will_not_automatically_open =
#if   L==0
	"Els grups no s'obriran autom&agrave;ticament.";
#elif L==1
	"The groups will not automatically open.";			// Need bersetzung
#elif L==2
	"The groups will not automatically open.";
#elif L==3
	"Los grupos no se abrir&aacute;n autom&aacute;ticamente.";
#elif L==4
	"The groups will not automatically open.";			// Besoin de traduction
#elif L==5
	"Los grupos no se abrir&aacute;n autom&aacute;ticamente.";	// Okoteve traduccin
#elif L==6
	"I gruppi non si aprir&agrave; automaticamente.";
#elif L==7
	"The groups will not automatically open.";			// Potrzebujesz tlumaczenie
#elif L==8
	"The groups will not automatically open.";			// Necessita de traduo
#endif

const char *Txt_The_HTML_file_has_been_received_successfully =
#if   L==0
	"El archivo <em>HTML</em> se ha recibido correctamente.";	// Necessita traduccio
#elif L==1
	"The <em>HTML</em> file has been received successfully.";	// Need bersetzung
#elif L==2
	"The <em>HTML</em> file has been received successfully.";
#elif L==3
	"El archivo <em>HTML</em> se ha recibido correctamente.";
#elif L==4
	"The <em>HTML</em> file has been received successfully.";	// Besoin de traduction
#elif L==5
	"El archivo <em>HTML</em> se ha recibido correctamente.";	// Okoteve traduccin
#elif L==6
	"Il <em>HTML</em> file &egrave; stato ricevuto con successo.";
#elif L==7
	"The <em>HTML</em> file has been received successfully.";	// Potrzebujesz tlumaczenie
#elif L==8
	"The <em>HTML</em> file has been received successfully.";	// Necessita de traduo
#endif

const char *Txt_The_ID_X_has_been_confirmed =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"El ID (DNI/c&eacute;dula) <strong>%s</strong>"
	" ha sido confirmado.";		// Necessita traduccio
#elif L==1
	"The ID <strong>%s</strong>"
	" has been confirmed.";		// Need bersetzung
#elif L==2
	"The ID <strong>%s</strong>"
	" has been confirmed.";
#elif L==3
	"El ID (DNI/c&eacute;dula) <strong>%s</strong>"
	" ha sido confirmado.";
#elif L==4
	"The ID <strong>%s</strong>"
	" has been confirmed.";		// Besoin de traduction
#elif L==5
	"El ID (DNI/c&eacute;dula) <strong>%s</strong>"
	" ha sido confirmado.";		// Okoteve traduccin
#elif L==6
	"Il ID <strong>%s</strong>"
	" &egrave; stato confermato.";
#elif L==7
	"The ID  ID <strong>%s</strong>"
	" has been confirmed.";		// Potrzebujesz tlumaczenie
#elif L==8
	"The ID  ID <strong>%s</strong>"
	" has been confirmed.";		// Necessita de traduo
#endif

const char *Txt_The_ID_X_has_been_registered_successfully =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"El ID (DNI/c&eacute;dula) <strong>%s</strong>"
	" se ha registrado correctamente.";		// Necessita traduccio
#elif L==1
	"The ID <strong>%s</strong>"
	" has been registered successfully.";		// Need bersetzung
#elif L==2
	"The ID <strong>%s</strong>"
	" has been registered successfully.";
#elif L==3
	"El ID (DNI/c&eacute;dula) <strong>%s</strong>"
	" se ha registrado correctamente.";
#elif L==4
	"The ID <strong>%s</strong>"
	" has been registered successfully.";		// Besoin de traduction
#elif L==5
	"El ID (DNI/c&eacute;dula) <strong>%s</strong>"
	" se ha registrado correctamente.";		// Okoteve traduccin
#elif L==6
	"Il ID <strong>%s</strong>"
	" &egrave; stato registrato con successo.";
#elif L==7
	"The ID  ID <strong>%s</strong>"
	" has been registered successfully.";		// Potrzebujesz tlumaczenie
#elif L==8
	"The ID  ID <strong>%s</strong>"
	" has been registered successfully.";		// Necessita de traduo
#endif

const char *Txt_The_ID_X_is_not_valid =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"El ID (DNI/c&eacute;dula) <strong>%s</strong> no es v&aacute;lido.";	// Necessita traduccio
#elif L==1
	"The ID <strong>%s</strong> is not valid.";				// Need bersetzung
#elif L==2
	"The ID <strong>%s</strong> is not valid.";
#elif L==3
	"El ID (DNI/c&eacute;dula) <strong>%s</strong> no es v&aacute;lido.";
#elif L==4
	"The ID <strong>%s</strong> is not valid.";				// Besoin de traduction
#elif L==5
	"El ID (DNI/c&eacute;dula) <strong>%s</strong> no es v&aacute;lido.";	// Okoteve traduccin
#elif L==6
	"L'ID <strong>%s</strong> non &egrave; valido.";
#elif L==7
	"The ID <strong>%s</strong> is not valid.";				// Potrzebujesz tlumaczenie
#elif L==8
	"The ID <strong>%s</strong> is not valid.";				// Necessita de traduo
#endif

const char *Txt_The_ID_is_used_in_order_to_facilitate_ =
#if   L==0
	"L'ID s'utilitza per tal de facilitar professors i administradors"
	" el proc&eacute;s d'inscripci&oacute; dels usuaris en les assignatures.";
#elif L==1
	"Die ID wird verwendet, um die Lehrer zu erleichtern und Administratoren"
	" den Registrierungsprozess von Benutzern in Kursen.";
#elif L==2
	"The ID is used in order to facilitate teachers and administrators"
	" the enrollment process of users in courses.";
#elif L==3
	"El ID se utiliza con el fin de facilitar a profesores y administradores"
	" el proceso de inscripcin de los usuarios en las asignaturas.";
#elif L==4
	"L'ID est utilis&eacute; afin de faciliter aux enseignants et administrateurs"
	" le processus d'inscription des utilisateurs dans les mati&egrave;res.";
#elif L==5
	"El ID se utiliza con el fin de facilitar a profesores y administradores"
	" el proceso de inscripcin de los usuarios en las asignaturas.";	// Okoteve traduccin
#elif L==6
	"L'ID &egrave; utilizzato al fine di facilitare gli insegnanti e gli amministratori"
	" il processo di registrazione degli utenti in corsi.";
#elif L==7
	"Identyfikator jest u&zdot;ywany w celu u&lstrok;atwienia nauczycielom i administratorom"
	" proces rejestracji u&zdot;ytkownik&oacute;w w kursach.";
#elif L==8
	"O ID &eacute; usado para facilitar a professores e administradores"
	" o processo de inscri&ccedil;&atilde;o de usu&aacute;rios em disciplinas.";
#endif

const char *Txt_The_ID_nickname_or_email_X_is_not_valid =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"El @apodo, correo o ID (DNI/c&eacute;dula) <strong>%s</strong> no es v&aacute;lido.";	// Necessita traduccio
#elif L==1
	"The @nick, e-mail or ID <strong>%s</strong> is not valid.";				// Need bersetzung
#elif L==2
	"The @nick, e-mail or ID <strong>%s</strong> is not valid.";
#elif L==3
	"El @apodo, correo o ID (DNI/c&eacute;dula) <strong>%s</strong> no es v&aacute;lido.";
#elif L==4
	"The @nick, e-mail or ID <strong>%s</strong> is not valid.";				// Besoin de traduction
#elif L==5
	"El @apodo, correo o ID (DNI/c&eacute;dula) <strong>%s</strong> no es v&aacute;lido.";	// Okoteve traduccin
#elif L==6
	"Il @nome-utente, e-mail o ID <strong>%s</strong> non &egrave; valido.";
#elif L==7
	"The @nick, e-mail or ID <strong>%s</strong> is not valid.";				// Potrzebujesz tlumaczenie
#elif L==8
	"The @nick, e-mail or ID <strong>%s</strong> is not valid.";				// Necessita de traduo
#endif

const char *Txt_The_ID_X_matches_one_of_the_existing =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"El ID (DNI/c&eacute;dula) <strong>%s</strong>"
	" coincide con uno de los existentes.";		// Necessita traduccio
#elif L==1
	"The ID <strong>%s</strong>"
	" matches one of the existing.";		// Need bersetzung
#elif L==2
	"The ID <strong>%s</strong>"
	" matches one of the existing.";
#elif L==3
	"El ID (DNI/c&eacute;dula) <strong>%s</strong>"
	" coincide con uno de los existentes.";
#elif L==4
	"The ID <strong>%s</strong>"
	" matches one of the existing.";		// Besoin de traduction
#elif L==5
	"El ID (DNI/c&eacute;dula) <strong>%s</strong>"
	" coincide con uno de los existentes.";		// Okoteve traduccin
#elif L==6
	"The ID <strong>%s</strong>"
	" matches one of the existing.";		// Bisogno di traduzione
#elif L==7
	"The ID <strong>%s</strong>"
	" matches one of the existing.";		// Potrzebujesz tlumaczenie
#elif L==8
	"The ID <strong>%s</strong>"
	" matches one of the existing.";		// Necessita de traduo
#endif

const char *Txt_The_user_does_not_exist_or_password_is_incorrect =
#if   L==0
	"El usuario/a no existe"
	" o bien la contrase&ntilde;a es incorrecta.";	// Necessita traduccio
#elif L==1
	"Die Benutzer ist nicht vorhanden"
	" oder Passwort falsch ist.";
#elif L==2
	"The user does not exist"
	" or password is incorrect.";
#elif L==3
	"El usuario/a no existe"
	" o bien la contrase&ntilde;a es incorrecta.";
#elif L==4
	"L'utilisateur n'existe pas"
	" ou le mot de passe est incorrect.";
#elif L==5
	"El usuario/a no existe"
	" o bien la contrase&ntilde;a es incorrecta.";	// Okoteve traduccin
#elif L==6
	"Il utente non esiste"
	" o la password non &egrave; corretta.";
#elif L==7
	"The user does not exist"
	" or password is incorrect.";			// Potrzebujesz tlumaczenie
#elif L==8
	"O utilizador n&atilde;o existe"
	" ou a senha est&aacute; incorreta.";
#endif

const char *Txt_The_institution_X_already_exists =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"La instituci&oacute;n <strong>%s</strong> ya existe.";	// Necessita traduccio
#elif L==1
	"Die Hochschule <strong>%s</strong> wurde bereits eingegeben.";
#elif L==2
	"The institution <strong>%s</strong> already exists.";
#elif L==3
	"La instituci&oacute;n <strong>%s</strong> ya existe.";
#elif L==4
	"The institution <strong>%s</strong> already exists.";	// Besoin de traduction
#elif L==5
	"La instituci&oacute;n <strong>%s</strong> ya existe.";	// Okoteve traduccin
#elif L==6
	"L'istituzione <strong>%s</strong> gi&agrave; esiste.";
#elif L==7
	"The institution <strong>%s</strong> already exists.";	// Potrzebujesz tlumaczenie
#elif L==8
	"The institution <strong>%s</strong> already exists.";	// Necessita de traduo
#endif

const char *Txt_The_institution_of_the_centre_has_changed =
#if   L==0
	"La instituci&oacute;n del centro ha cambiado.";	// Necessita traduccio
#elif L==1
	"The institution of the centre has changed.";		// Need bersetzung
#elif L==2
	"The institution of the centre has changed.";
#elif L==3
	"La instituci&oacute;n del centro ha cambiado.";
#elif L==4
	"The institution of the centre .";			// Besoin de traduction
#elif L==5
	"La instituci&oacute;n del centro ha cambiado.";	// Okoteve traduccin
#elif L==6
	"L'istituzione del centro &egrave; cambiata.";
#elif L==7
	"The institution of the centre has changed.";		// Potrzebujesz tlumaczenie
#elif L==8
	"The institution of the centre has changed.";		// Necessita de traduo
#endif

const char *Txt_The_institution_of_the_department_has_changed =
#if   L==0
	"La instituci&oacute;n del departamento ha cambiado.";	// Necessita traduccio
#elif L==1
	"The institution of the department has changed.";	// Need bersetzung
#elif L==2
	"The institution of the department has changed.";
#elif L==3
	"La instituci&oacute;n del departamento ha cambiado.";
#elif L==4
	"The institution of the department has changed.";	// Besoin de traduction
#elif L==5
	"La instituci&oacute;n del departamento ha cambiado.";	// Okoteve traduccin
#elif L==6
	"L'istituzione del dipartimento &egrave; cambiata.";
#elif L==7
	"The institution of the department has changed.";	// Potrzebujesz tlumaczenie
#elif L==8
	"The institution of the department has changed.";	// Necessita de traduo
#endif

const char *Txt_The_institutional_code_of_the_course_X_has_changed_to_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==0
	"El c&oacute;digo institucional de la asignatura <strong>%s</strong>"
	" ha cambiado a <strong>%s</strong>.";			// Necessita traduccio
#elif L==1
	"The institutional code of the course <strong>%s</strong>"
	" has changed to <strong>%s</strong> .";		// Need bersetzung
#elif L==2
	"The institutional code of the course <strong>%s</strong>"
	" has changed to <strong>%s</strong>.";
#elif L==3
	"El c&oacute;digo institucional de la asignatura <strong>%s</strong>"
	" ha cambiado a <strong>%s</strong>.";
#elif L==4
	"The institutional code of the course <strong>%s</strong>"
	" has changed to <strong>%s</strong>.";			// Besoin de traduction
#elif L==5
	"El c&oacute;digo institucional de la asignatura <strong>%s</strong>"
	" ha cambiado a <strong>%s</strong>.";			// Okoteve traduccin
#elif L==6
	"Il codice istituzionale del corso <strong>%s</strong>"
	" &egrave; cambiato a <strong>%s</strong>.";
#elif L==7
	"The institutional code of the course <strong>%s</strong>"
	" has changed to <strong>%s</strong>.";			// Potrzebujesz tlumaczenie
#elif L==8
	"The institutional code of the course <strong>%s</strong>"
	" has changed to <strong>%s</strong>.";			// Necessita de traduo
#endif

const char *Txt_The_institutional_code_of_the_course_X_has_not_changed =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"El c&oacute;digo institucional de la asignatura <strong>%s</strong> no ha cambiado.";	// Necessita traduccio
#elif L==1
	"The institutional code of the course <strong>%s</strong> has not changed.";		// Need bersetzung
#elif L==2
	"The institutional code of the course <strong>%s</strong> has not changed.";
#elif L==3
	"El c&oacute;digo institucional de la asignatura <strong>%s</strong> no ha cambiado.";
#elif L==4
	"The institutional code of the course <strong>%s</strong> has not changed.";		// Besoin de traduction
#elif L==5
	"El c&oacute;digo institucional de la asignatura <strong>%s</strong> no ha cambiado.";	// Okoteve traduccin
#elif L==6
	"Il codice istituzionale del corso <strong>%s</strong> non &egrave; cambiato.";
#elif L==7
	"The institutional code of the course <strong>%s</strong> has not changed.";		// Potrzebujesz tlumaczenie
#elif L==8
	"The institutional code of the course <strong>%s</strong> has not changed.";		// Necessita de traduo
#endif

const char *Txt_The_integrated_editor_is_not_yet_available =
#if   L==0
	"El editor integrado a&uacute;n no est&aacute; disponible.";	// Necessita traduccio
#elif L==1
	"Der integrierte Editor ist noch nicht verf&uuml;gbar.";
#elif L==2
	"The integrated editor is not yet available.";
#elif L==3
	"El editor integrado a&uacute;n no est&aacute; disponible.";
#elif L==4
	"The integrated editor is not yet available.";			// Besoin de traduction
#elif L==5
	"El editor integrado a&uacute;n no est&aacute; disponible.";	// Okoteve traduccin
#elif L==6
	"L'editor integrato non &egrave; ancora disponibile.";
#elif L==7
	"The integrated editor is not yet available.";			// Potrzebujesz tlumaczenie
#elif L==8
	"The integrated editor is not yet available.";			// Necessita de traduo
#endif

const char *Txt_The_language_has_changed_to_LANGUAGE =
#if   L==0
	"S'ha canviat l'idioma a catal&agrave;.";
#elif L==1
	"Die Sprache ist nun Deutsch.";
#elif L==2
	"The language has changed to English.";
#elif L==3
	"Se ha cambiado el idioma a espa&ntilde;ol.";
#elif L==4
	"La langue a chang&eacute; au fran&ccedil;ais.";
#elif L==5
	"Pe &ntilde;e'&#7869; ha moambuepapyre pe ava&ntilde;e'&#7869;.";
#elif L==6
	"La lingua &egrave; cambiata a italiano.";
#elif L==7
	"J&eogon;zyk zosta&lstrok; zmieniony na polski.";
#elif L==8
	"O idioma mudou para o portugu&ecirc;s.";
#endif

const char *Txt_The_link_X_already_exists =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"El enlace <strong>%s</strong> ya existe.";	// Necessita traduccio
#elif L==1
	"Der Link <strong>%s</strong> wurde bereits eingegeben.";
#elif L==2
	"The link <strong>%s</strong> already exists.";
#elif L==3
	"El enlace <strong>%s</strong> ya existe.";
#elif L==4
	"The link <strong>%s</strong> already exists.";	// Besoin de traduction
#elif L==5
	"El enlace <strong>%s</strong> ya existe.";	// Okoteve traduccin
#elif L==6
	"Il link <strong>%s</strong> gi&agrave; esiste.";
#elif L==7
	"The link <strong>%s</strong> already exists.";	// Potrzebujesz tlumaczenie
#elif L==8
	"The link <strong>%s</strong> already exists.";	// Necessita de traduo
#endif

const char *Txt_The_link_X_has_been_placed_inside_the_folder_Y =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"El enlace <strong>%s</strong> se ha almacenado"
	" dentro de la carpeta <strong>%s</strong>.";		// Necessita traduccio
#elif L==1
	"Die Link <strong>%s</strong> wurde"
	" in das Verzeichnis <strong>%s</strong> gesteckt.";
#elif L==2
	"The link <strong>%s</strong> has been placed"
	" inside the folder <strong>%s</strong>.";
#elif L==3
	"El enlace <strong>%s</strong> se ha almacenado"
	" dentro de la carpeta <strong>%s</strong>.";
#elif L==4
	"Le lien <strong>%s</strong> a &eacute;t&eacute; plac&eacute;"
	" dans le r&eacute;pertoire <strong>%s</strong>.";
#elif L==5
	"El enlace <strong>%s</strong> se ha almacenado"
	" dentro de la carpeta <strong>%s</strong>.";		// Okoteve traduccin
#elif L==6
	"Il link <strong>%s</strong> &egrave; stato messo"
	" all'interno della cartella <strong>%s</strong>.";
#elif L==7
	"The link <strong>%s</strong> has been placed"
	" inside the folder <strong>%s</strong>.";		// Potrzebujesz tlumaczenie
#elif L==8
	"A liga&ccedil;&atilde;o <strong>%s</strong> foi colocado"
	" dentro do diret&oacute;rio <strong>%s</strong>.";
#endif

const char *Txt_The_link_X_has_been_renamed_as_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==0
	"El enlace <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";	// Necessita traduccio
#elif L==1
	"The link <strong>%s</strong> has been renamed as <strong>%s</strong>.";	// Need bersetzung
#elif L==2
	"The link <strong>%s</strong> has been renamed as <strong>%s</strong>.";
#elif L==3
	"El enlace <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";
#elif L==4
	"The link <strong>%s</strong> has been renamed as <strong>%s</strong>.";	// Besoin de traduction
#elif L==5
	"El enlace <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";	// Okoteve traduccin
#elif L==6
	"Il link <strong>%s</strong> &egrave; stato rinominato come <strong>%s</strong>.";
#elif L==7
	"The link <strong>%s</strong> has been renamed as <strong>%s</strong>.";	// Potrzebujesz tlumaczenie
#elif L==8
	"The link <strong>%s</strong> has been renamed as <strong>%s</strong>.";	// Necessita de traduo
#endif

const char *Txt_The_list_of_X_courses_is_too_large_to_be_displayed =	// Warning: it is very important to include %u in the following sentences
#if   L==0
	"La lista de %u asignaturas es demasiado grande para mostrarla.";	// Necessita traduccio
#elif L==1
	"The list of %u courses is too large to be displayed.";			// Need bersetzung
#elif L==2
	"The list of %u courses is too large to be displayed.";
#elif L==3
	"La lista de %u asignaturas es demasiado grande para mostrarla.";
#elif L==4
	"The list of %u courses is too large to be displayed.";			// Besoin de traduction
#elif L==5
	"La lista de %u asignaturas es demasiado grande para mostrarla.";	// Okoteve traduccin
#elif L==6
	"La lista di %u corsi &egrave; troppo grande per essere mostrata.";
#elif L==7
	"The list of %u courses is too large to be displayed.";			// Potrzebujesz tlumaczenie
#elif L==8
	"The list of %u courses is too large to be displayed.";			// Necessita de traduo
#endif

const char *Txt_The_list_of_X_users_is_too_large_to_be_displayed =	// Warning: it is very important to include %u in the following sentences
#if   L==0
	"La lista de %u usuarios es demasiado grande para mostrarla.";	// Necessita traduccio
#elif L==1
	"The list of %u users is too large to be displayed.";		// Need bersetzung
#elif L==2
	"The list of %u users is too large to be displayed.";
#elif L==3
	"La lista de %u usuarios es demasiado grande para mostrarla.";
#elif L==4
	"The list of %u users is too large to be displayed.";		// Besoin de traduction
#elif L==5
	"La lista de %u usuarios es demasiado grande para mostrarla.";	// Okoteve traduccin
#elif L==6
	"La lista di %u utenti &egrave; troppo grande per essere mostrata.";
#elif L==7
	"The list of %u users is too large to be displayed.";		// Potrzebujesz tlumaczenie
#elif L==8
	"The list of %u users is too large to be displayed.";		// Necessita de traduo
#endif

const char *Txt_The_lower_limit_of_correct_answers_must_be_less_than_or_equal_to_the_upper_limit =
#if   L==0
	"El l&iacute;mite inferior del intervalo de respuestas correctas"
	" debe ser menor o igual que el l&iacute;mite superior.";	// Necessita traduccio
#elif L==1
	"The lower limit of correct answers"
	" must be less than or equal to the upper limit.";		// Need bersetzung
#elif L==2
	"The lower limit of correct answers"
	" must be less than or equal to the upper limit.";
#elif L==3
	"El l&iacute;mite inferior del intervalo de respuestas correctas"
	" debe ser menor o igual que el l&iacute;mite superior.";
#elif L==4
	"The lower limit of correct answers"
	" must be less than or equal to the upper limit.";		// Besoin de traduction
#elif L==5
	"El l&iacute;mite inferior del intervalo de respuestas correctas"
	" debe ser menor o igual que el l&iacute;mite superior.";	// Okoteve traduccin
#elif L==6
	"Il limite inferiore di risposte corrette"
	" deve essere minore o uguale al limite superiore.";
#elif L==7
	"The lower limit of correct answers"
	" must be less than or equal to the upper limit.";		// Potrzebujesz tlumaczenie
#elif L==8
	"The lower limit of correct answers"
	" must be less than or equal to the upper limit.";		// Necessita de traduo
#endif

const char *Txt_The_mail_domain_X_already_exists =
#if   L==0
	"El dominio de correo ya existe.";		// Necessita traduccio
#elif L==1
	"Die E-Mail-Dom&auml;ne wurde bereits eingegeben.";
#elif L==2
	"The mail domain already exists.";
#elif L==3
	"El dominio de correo ya existe.";
#elif L==4
	"The mail domain already exists.";		// Besoin de traduction
#elif L==5
	"El dominio de correo ya existe.";		// Okoteve traduccin
#elif L==6
	"Il campo mail gi&agrave; esiste.";
#elif L==7
	"The mail domain already exists.";		// Potrzebujesz tlumaczenie
#elif L==8
	"The mail domain already exists.";		// Necessita de traduo
#endif

const char *Txt_The_mail_domain_X_has_been_renamed_as_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==0
	"El dominio de correo <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";	// Necessita traduccio
#elif L==1
	"The mail domain <strong>%s</strong> has been renamed as <strong>%s</strong>.";			// Need bersetzung
#elif L==2
	"The mail domain <strong>%s</strong> has been renamed as <strong>%s</strong>.";
#elif L==3
	"El dominio de correo <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";
#elif L==4
	"The mail domain <strong>%s</strong> has been renamed as <strong>%s</strong>.";			// Besoin de traduction
#elif L==5
	"El dominio de correo <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";	// Okoteve traduccin
#elif L==6
	"Il campo mail <strong>%s</strong> &egrave; stato rinominato come <strong>%s</strong>.";
#elif L==7
	"The mail domain <strong>%s</strong> has been renamed as <strong>%s</strong>.";			// Potrzebujesz tlumaczenie
#elif L==8
	"The mail domain <strong>%s</strong> has been renamed as <strong>%s</strong>.";			// Necessita de traduo
#endif

const char *Txt_The_mail_domain_X_has_not_changed =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"El dominio de correo <strong>%s</strong> no ha cambiado.";	// Necessita traduccio
#elif L==1
	"The mail domain <strong>%s</strong> has not changed.";		// Need bersetzung
#elif L==2
	"The mail domain <strong>%s</strong> has not changed.";
#elif L==3
	"El dominio de correo <strong>%s</strong> no ha cambiado.";
#elif L==4
	"The mail domain <strong>%s</strong> has not changed.";		// Besoin de traduction
#elif L==5
	"El dominio de correo <strong>%s</strong> no ha cambiado.";	// Okoteve traduccin
#elif L==6
	"Il campo mail <strong>%s</strong> non &egrave; cambiato.";
#elif L==7
	"The mail domain <strong>%s</strong> has not changed.";		// Potrzebujesz tlumaczenie
#elif L==8
	"The mail domain <strong>%s</strong> has not changed.";		// Necessita de traduo
#endif

const char *Txt_the_marks_of_a_student_chosen_at_random_ =
#if   L==0
	"se mostrar&aacute;n las calificaciones de un estudiante de la asignatura escogido al azar;<br />"
	" si ve m&aacute;s de un estudiante, significa que los n&uacute;meros de filas de cabecera o de pie no son correctos";	// Necessita traduccio
#elif L==1
	"es wird die Bewertung eines nach dem Zufallsprinzip ausgew&auml;ten Studenten angezeigt;<br />"
	" wenn Sie mehr als einen Student bedeutet dies, dass die Anzahl der Kopf oder Fu von Zeilen ist nicht korrekt";
#elif L==2
	"the marks of a student chosen at random from the course will be shown;<br />"
	" if you see more than one student, it means that the number of header or footer rows is not correct";
#elif L==3
	"se mostrar&aacute;n las calificaciones de un estudiante de la asignatura escogido al azar;<br />"
	" si ve m&aacute;s de un estudiante, significa que los n&uacute;meros de filas de cabecera o de pie no son correctos";
#elif L==4
	"les notes d'un &eacute;tudiant choisi au hasard dans la mati&egrave;re seront pr&eacute;sent&eacute;s;<br />"
	" si vous voyez plus d'un &eacute;tudiant, cela signifie que le nombre de ligned de la t&ecirc;te ou du pied n'est pas correcte";
#elif L==5
	"se mostrar&aacute;n las calificaciones de un estudiante de la asignatura escogido al azar;<br />"
	" si ve m&aacute;s de un estudiante, significa que los n&uacute;meros de filas de cabecera o de pie no son correctos";	// Okoteve traduccin
#elif L==6
	"i risultati di uno studente scelto a caso dal corso verranno mostrati;<br />"
	" se si vede pi&uacute; di uno studente, significa che il numero di righe di intestazione o a pi&egrave; di pagina non &egrave; corretto";
#elif L==7
	"the marks of a student chosen at random from the course will be shown;<br />"
	" if you see more than one student, it means that the number of header or footer rows is not correct";	// Potrzebujesz tlumaczenie
#elif L==8
	"vai mostrar as notas de um estudante da disciplina escolhido ao acaso;<br />"
	" se voc&ecirc; vir mais de um estudante, isso significa que o n&uacute;mero de cabe&ccedil;a ou p&eacute; linhas n&atilde;o est&aacute; correto";
#endif

const char *Txt_The_maximum_number_of_students_in_the_group_X_has_not_changed =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"El n&ordm; m&aacute;ximo de estudiantes"
	" del grupo <strong>%s</strong> no ha cambiado.";	// Necessita traduccio
#elif L==1
	"The maximum number of students"
	" in the group <strong>%s</strong> has not changed.";	// Need bersetzung
#elif L==2
	"The maximum number of students"
	" in the group <strong>%s</strong> has not changed.";
#elif L==3
	"El n&ordm; m&aacute;ximo de estudiantes"
	" del grupo <strong>%s</strong> no ha cambiado.";
#elif L==4
	"The maximum number of students"
	" in the group <strong>%s</strong> has not changed.";	// Besoin de traduction
#elif L==5
	"El n&ordm; m&aacute;ximo de estudiantes"
	" del grupo <strong>%s</strong> no ha cambiado.";	// Okoteve traduccin
#elif L==6
	"Il numero massimo di studenti"
	" nel gruppo <strong>%s</strong> non &egrave; cambiato.";
#elif L==7
	"The maximum number of students"
	" in the group <strong>%s</strong> has not changed.";	// Potrzebujesz tlumaczenie
#elif L==8
	"The maximum number of students"
	" in the group <strong>%s</strong> has not changed.";	// Necessita de traduo
#endif

const char *Txt_The_maximum_number_of_students_in_the_group_X_is_now_Y =	// Warning: it is very important to include %s and %u in the following sentences
#if   L==0
	"El n&ordm; m&aacute;ximo de estudiantes"
	" del grupo <strong>%s</strong> es ahora <strong>%u</strong>.";	// Necessita traduccio
#elif L==1
	"The maximum number of students"
	" in the group <strong>%s</strong> is now <strong>%u</strong>.";	// Need bersetzung
#elif L==2
	"The maximum number of students"
	" in the group <strong>%s</strong> is now <strong>%u</strong>.";
#elif L==3
	"El n&ordm; m&aacute;ximo de estudiantes"
	" del grupo <strong>%s</strong> es ahora <strong>%u</strong>.";
#elif L==4
	"The maximum number of students"
	" in the group <strong>%s</strong> is now <strong>%u</strong>.";	// Besoin de traduction
#elif L==5
	"El n&ordm; m&aacute;ximo de estudiantes"
	" del grupo <strong>%s</strong> es ahora <strong>%u</strong>.";		// Okoteve traduccin
#elif L==6
	"Il numero massimo di studenti"
	" nel gruppo <strong>%s</strong> &egrave; ora <strong>%u</strong>.";
#elif L==7
	"The maximum number of students"
	" in the group <strong>%s</strong> is now <strong>%u</strong>.";	// Potrzebujesz tlumaczenie
#elif L==8
	"The maximum number of students"
	" in the group <strong>%s</strong> is now <strong>%u</strong>.";	// Necessita de traduo
#endif

const char *Txt_The_message_has_not_been_sent_to_any_recipient =
#if   L==0
	"El missatge no ha estat enviat a cap destinatari.";
#elif L==1
	"The message has not been sent to any recipient.";		// Need bersetzung
#elif L==2
	"The message has not been sent to any recipient.";
#elif L==3
	"El mensaje no se ha enviado a ning&uacute;n destinatario.";
#elif L==4
	"The message has not been sent to any recipient.";		// Besoin de traduction
#elif L==5
	"El mensaje no se ha enviado a ning&uacute;n destinatario.";	// Okoteve traduccin
#elif L==6
	"Il messaggio non &egrave; stato inviato a nessun destinatario.";
#elif L==7
	"The message has not been sent to any recipient.";		// Potrzebujesz tlumaczenie
#elif L==8
	"The message has not been sent to any recipient.";		// Necessita de traduo
#endif

const char *Txt_The_message_has_been_sent_to_1_recipient =
#if   L==0
	"El mensaje se ha enviado a 1 destinatario.";	// Necessita traduccio
#elif L==1
	"The message has been sent to 1 recipient.";	// Need bersetzung
#elif L==2
	"The message has been sent to 1 recipient.";
#elif L==3
	"El mensaje se ha enviado a 1 destinatario.";
#elif L==4
	"The message has been sent to 1 recipient.";	// Besoin de traduction
#elif L==5
	"El mensaje se ha enviado a 1 destinatario.";	// Okoteve traduccin
#elif L==6
	"Il messaggio &egrave; stato inviato a 1 destinatario.";
#elif L==7
	"The message has been sent to 1 recipient.";	// Potrzebujesz tlumaczenie
#elif L==8
	"The message has been sent to 1 recipient.";	// Necessita de traduo
#endif

const char *Txt_The_message_has_been_sent_to_X_recipients =	// Warning: it is very important to include %u in the following sentences
#if   L==0
	"El mensaje se ha enviado a %u destinatarios.";	// Necessita traduccio
#elif L==1
	"The message has been sent to %u recipients.";	// Need bersetzung
#elif L==2
	"The message has been sent to %u recipients.";
#elif L==3
	"El mensaje se ha enviado a %u destinatarios.";
#elif L==4
	"The message has been sent to %u recipients.";	// Besoin de traduction
#elif L==5
	"El mensaje se ha enviado a %u destinatarios.";	// Okoteve traduccin
#elif L==6
	"Il messaggio &egrave; stato inviato a %u destinatari.";
#elif L==7
	"The message has been sent to %u recipients.";	// Potrzebujesz tlumaczenie
#elif L==8
	"The message has been sent to %u recipients.";	// Necessita de traduo
#endif

const char *Txt_The_name_of_the_banner_X_has_not_changed =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"El nombre del banner <strong>%s</strong> no ha cambiado.";	// Necessita traduccio
#elif L==1
	"The name of the banner <strong>%s</strong> has not changed.";	// Need bersetzung
#elif L==2
	"The name of the banner <strong>%s</strong> has not changed.";
#elif L==3
	"El nombre del banner <strong>%s</strong> no ha cambiado.";
#elif L==4
	"The name of the banner <strong>%s</strong> has not changed.";	// Besoin de traduction
#elif L==5
	"El nombre del banner <strong>%s</strong> no ha cambiado.";	// Okoteve traduccin
#elif L==6
	"Il nome del banner <strong>%s</strong> non &egrave; cambiato.";
#elif L==7
	"The name of the banner <strong>%s</strong> has not changed.";	// Potrzebujesz tlumaczenie
#elif L==8
	"The name of the banner <strong>%s</strong> has not changed.";	// Necessita de traduo
#endif

const char *Txt_The_name_of_the_centre_X_has_not_changed =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"El nombre del centro <strong>%s</strong> no ha cambiado.";	// Necessita traduccio
#elif L==1
	"The name of the centre <strong>%s</strong> has not changed.";	// Need bersetzung
#elif L==2
	"The name of the centre <strong>%s</strong> has not changed.";
#elif L==3
	"El nombre del centro <strong>%s</strong> no ha cambiado.";
#elif L==4
	"The name of the centre <strong>%s</strong> has not changed.";	// Besoin de traduction
#elif L==5
	"El nombre del centro <strong>%s</strong> no ha cambiado.";	// Okoteve traduccin
#elif L==6
	"Il nome del centro <strong>%s</strong> non &egrave; cambiato.";
#elif L==7
	"The name of the centre <strong>%s</strong> has not changed.";	// Potrzebujesz tlumaczenie
#elif L==8
	"The name of the centre <strong>%s</strong> has not changed.";	// Necessita de traduo
#endif

const char *Txt_The_name_of_the_country_X_has_not_changed =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"El nombre del pa&iacute;s <strong>%s</strong> no ha cambiado.";	// Necessita traduccio
#elif L==1
	"The name of the country <strong>%s</strong> has not changed.";		// Need bersetzung
#elif L==2
	"The name of the country <strong>%s</strong> has not changed.";
#elif L==3
	"El nombre del pa&iacute;s <strong>%s</strong> no ha cambiado.";
#elif L==4
	"The name of the country <strong>%s</strong> has not changed.";		// Besoin de traduction
#elif L==5
	"El nombre del pa&iacute;s <strong>%s</strong> no ha cambiado.";	// Okoteve traduccin
#elif L==6
	"Il nome del paese <strong>%s</strong> non &egrave; cambiato.";
#elif L==7
	"The name of the country <strong>%s</strong> has not changed.";		// Potrzebujesz tlumaczenie
#elif L==8
	"The name of the country <strong>%s</strong> has not changed.";		// Necessita de traduo
#endif

const char *Txt_The_name_of_the_course_X_has_changed_to_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==0
	"El nombre de la asignatura <strong>%s</strong> ha cambiado a <strong>%s</strong>.";	// Necessita traduccio
#elif L==1
	"The name of the course <strong>%s</strong> has changed to <strong>%s</strong> .";	// Need bersetzung
#elif L==2
	"The name of the course <strong>%s</strong> has changed to <strong>%s</strong>.";
#elif L==3
	"El nombre de la asignatura <strong>%s</strong> ha cambiado a <strong>%s</strong>.";
#elif L==4
	"The name of the course <strong>%s</strong> has changed to <strong>%s</strong>.";	// Besoin de traduction
#elif L==5
	"El nombre de la asignatura <strong>%s</strong> ha cambiado a <strong>%s</strong>.";	// Okoteve traduccin
#elif L==6
	"Il nome del corso <strong>%s</strong> &egrave; cambiato a <strong>%s</strong>.";
#elif L==7
	"The name of the course <strong>%s</strong> has changed to <strong>%s</strong>.";	// Potrzebujesz tlumaczenie
#elif L==8
	"The name of the course <strong>%s</strong> has changed to <strong>%s</strong>.";	// Necessita de traduo
#endif

const char *Txt_The_name_of_the_course_X_has_not_changed =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"El nombre de la asignatura <strong>%s</strong> no ha cambiado.";	// Necessita traduccio
#elif L==1
	"The name of the course <strong>%s</strong> has not changed.";		// Need bersetzung
#elif L==2
	"The name of the course <strong>%s</strong> has not changed.";
#elif L==3
	"El nombre de la asignatura <strong>%s</strong> no ha cambiado.";
#elif L==4
	"The name of the course <strong>%s</strong> has not changed.";		// Besoin de traduction
#elif L==5
	"El nombre de la asignatura <strong>%s</strong> no ha cambiado.";	// Okoteve traduccin
#elif L==6
	"Il nome del corso <strong>%s</strong> non &egrave; cambiato.";
#elif L==7
	"The name of the course <strong>%s</strong> has not changed.";		// Potrzebujesz tlumaczenie
#elif L==8
	"The name of the course <strong>%s</strong> has not changed.";		// Necessita de traduo
#endif

const char *Txt_The_name_of_the_degree_X_has_changed_to_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==0
	"El nombre de la titulaci&oacute;n <strong>%s</strong> ha cambiado a <strong>%s</strong>.";	// Necessita traduccio
#elif L==1
	"The name of the degree <strong>%s</strong> has changed to <strong>%s</strong>.";		// Need bersetzung
#elif L==2
	"The name of the degree <strong>%s</strong> has changed to <strong>%s</strong>.";
#elif L==3
	"El nombre de la titulaci&oacute;n <strong>%s</strong> ha cambiado a <strong>%s</strong>.";
#elif L==4
	"The name of the degree <strong>%s</strong> has changed to <strong>%s</strong>.";		// Besoin de traduction
#elif L==5
	"El nombre de la titulaci&oacute;n <strong>%s</strong> ha cambiado a <strong>%s</strong>.";	// Okoteve traduccin
#elif L==6
	"Il nome della laurea <strong>%s</strong> &egrave; cambiato a <strong>%s</strong>.";
#elif L==7
	"The name of the degree <strong>%s</strong> has changed to <strong>%s</strong>.";		// Potrzebujesz tlumaczenie
#elif L==8
	"The name of the degree <strong>%s</strong> has changed to <strong>%s</strong>.";		// Necessita de traduo
#endif

const char *Txt_The_name_of_the_degree_X_has_not_changed =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"El nombre de la titulaci&oacute;n <strong>%s</strong> no ha cambiado.";	// Necessita traduccio
#elif L==1
	"The name of the degree <strong>%s</strong> has not changed.";			// Need bersetzung
#elif L==2
	"The name of the degree <strong>%s</strong> has not changed.";
#elif L==3
	"El nombre de la titulaci&oacute;n <strong>%s</strong> no ha cambiado.";
#elif L==4
	"The name of the degree <strong>%s</strong> has not changed.";			// Besoin de traduction
#elif L==5
	"El nombre de la titulaci&oacute;n <strong>%s</strong> no ha cambiado.";	// Okoteve traduccin
#elif L==6
	"Il nome della laurea <strong>%s</strong> non &egrave; cambiato.";
#elif L==7
	"The name of the degree <strong>%s</strong> has not changed.";			// Potrzebujesz tlumaczenie
#elif L==8
	"The name of the degree <strong>%s</strong> has not changed.";			// Necessita de traduo
#endif

const char *Txt_The_name_of_the_department_X_has_not_changed =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"El nombre del departamento <strong>%s</strong> no ha cambiado.";	// Necessita traduccio
#elif L==1
	"The name of the department <strong>%s</strong> has not changed.";	// Need bersetzung
#elif L==2
	"The name of the department <strong>%s</strong> has not changed.";
#elif L==3
	"El nombre del departamento <strong>%s</strong> no ha cambiado.";
#elif L==4
	"The name of the department <strong>%s</strong> has not changed.";	// Besoin de traduction
#elif L==5
	"El nombre del departamento <strong>%s</strong> no ha cambiado.";	// Okoteve traduccin
#elif L==6
	"Il nome del dipartimento <strong>%s</strong> non &egrave; cambiato.";
#elif L==7
	"The name of the department <strong>%s</strong> has not changed.";	// Potrzebujesz tlumaczenie
#elif L==8
	"The name of the department <strong>%s</strong> has not changed.";	// Necessita de traduo
#endif

const char *Txt_The_name_of_the_field_X_has_not_changed =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"El nombre del campo <strong>%s</strong> no ha cambiado.";	// Necessita traduccio
#elif L==1
	"The name of the field <strong>%s</strong> has not changed.";	// Need bersetzung
#elif L==2
	"The name of the field <strong>%s</strong> has not changed.";
#elif L==3
	"El nombre del campo <strong>%s</strong> no ha cambiado.";
#elif L==4
	"The name of the field <strong>%s</strong> has not changed.";	// Besoin de traduction
#elif L==5
	"El nombre del campo <strong>%s</strong> no ha cambiado.";	// Okoteve traduccin
#elif L==6
	"Il nome del campo <strong>%s</strong> non &egrave; cambiato.";
#elif L==7
	"The name of the field <strong>%s</strong> has not changed.";	// Potrzebujesz tlumaczenie
#elif L==8
	"The name of the field <strong>%s</strong> has not changed.";	// Necessita de traduo
#endif

const char *Txt_The_name_of_the_group_X_has_not_changed =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"El nombre del grupo <strong>%s</strong> no ha cambiado.";	// Necessita traduccio
#elif L==1
	"The name of the group <strong>%s</strong> has not changed.";	// Need bersetzung
#elif L==2
	"The name of the group <strong>%s</strong> has not changed.";
#elif L==3
	"El nombre del grupo <strong>%s</strong> no ha cambiado.";
#elif L==4
	"The name of the group <strong>%s</strong> has not changed.";	// Besoin de traduction
#elif L==5
	"El nombre del grupo <strong>%s</strong> no ha cambiado.";	// Okoteve traduccin
#elif L==6
	"Il nome del gruppo <strong>%s</strong> non &egrave; cambiato.";
#elif L==7
	"The name of the group <strong>%s</strong> has not changed.";	// Potrzebujesz tlumaczenie
#elif L==8
	"The name of the group <strong>%s</strong> has not changed.";	// Necessita de traduo
#endif

const char *Txt_The_name_of_the_holiday_X_has_changed_to_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==0
	"El nombre de la festividad <strong>%s</strong> ha cambiado a <strong>%s</strong>.";	// Necessita traduccio
#elif L==1
	"The name of the holiday <strong>%s</strong> has changed to <strong>%s</strong>.";	// Need bersetzung
#elif L==2
	"The name of the holiday <strong>%s</strong> has changed to <strong>%s</strong>.";
#elif L==3
	"El nombre de la festividad <strong>%s</strong> ha cambiado a <strong>%s</strong>.";
#elif L==4
	"The name of the holiday <strong>%s</strong> has changed to <strong>%s</strong>.";	// Besoin de traduction
#elif L==5
	"El nombre de la festividad <strong>%s</strong> ha cambiado a <strong>%s</strong>.";	// Okoteve traduccin
#elif L==6
	"Il nome della festivit&agrave; <strong>%s</strong> &egrave; cambiato a <strong>%s</strong>.";
#elif L==7
	"The name of the holiday <strong>%s</strong> has changed to <strong>%s</strong>.";	// Potrzebujesz tlumaczenie
#elif L==8
	"The name of the holiday <strong>%s</strong> has changed to <strong>%s</strong>.";	// Necessita de traduo
#endif

const char *Txt_The_name_of_the_holiday_X_has_not_changed =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"El nombre de la festividad <strong>%s</strong> no ha cambiado.";	// Necessita traduccio
#elif L==1
	"The name of the holiday <strong>%s</strong> has not changed.";		// Need bersetzung
#elif L==2
	"The name of the holiday <strong>%s</strong> has not changed.";
#elif L==3
	"El nombre de la festividad <strong>%s</strong> no ha cambiado.";
#elif L==4
	"The name of the holiday <strong>%s</strong> has not changed.";		// Besoin de traduction
#elif L==5
	"El nombre de la festividad <strong>%s</strong> no ha cambiado.";	// Okoteve traduccin
#elif L==6
	"Il nome della festivit&agrave; <strong>%s</strong> non &egrave; cambiato.";
#elif L==7
	"The name of the holiday <strong>%s</strong> has not changed.";		// Potrzebujesz tlumaczenie
#elif L==8
	"The name of the holiday <strong>%s</strong> has not changed.";		// Necessita de traduo
#endif

const char *Txt_The_name_of_the_institution_X_has_changed_to_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==0
	"El nombre de la instituci&oacute;n <strong>%s</strong> ha cambiado a <strong>%s</strong>.";	// Necessita traduccio
#elif L==1
	"The name of the institution <strong>%s</strong> has changed to <strong>%s</strong>.";		// Need bersetzung
#elif L==2
	"The name of the institution <strong>%s</strong> has changed to <strong>%s</strong>.";
#elif L==3
	"El nombre de la instituci&oacute;n <strong>%s</strong> ha cambiado a <strong>%s</strong>.";
#elif L==4
	"The name of the institution <strong>%s</strong> has changed to <strong>%s</strong>.";		// Besoin de traduction
#elif L==5
	"El nombre de la instituci&oacute;n <strong>%s</strong> ha cambiado a <strong>%s</strong>.";	// Okoteve traduccin
#elif L==6
	"Il nome dell'istituzione <strong>%s</strong> &egrave; cambiato a <strong>%s</strong>.";
#elif L==7
	"The name of the institution <strong>%s</strong> has changed to <strong>%s</strong>.";		// Potrzebujesz tlumaczenie
#elif L==8
	"The name of the institution <strong>%s</strong> has changed to <strong>%s</strong>.";		// Necessita de traduo
#endif

const char *Txt_The_name_of_the_institution_X_has_not_changed =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"El nombre de la instituci&oacute;n <strong>%s</strong> no ha cambiado.";	// Necessita traduccio
#elif L==1
	"The name of the institution <strong>%s</strong> has not changed.";		// Need bersetzung
#elif L==2
	"The name of the institution <strong>%s</strong> has not changed.";
#elif L==3
	"El nombre de la instituci&oacute;n <strong>%s</strong> no ha cambiado.";
#elif L==4
	"The name of the institution <strong>%s</strong> has not changed.";		// Besoin de traduction
#elif L==5
	"El nombre de la instituci&oacute;n <strong>%s</strong> no ha cambiado.";	// Okoteve traduccin
#elif L==6
	"Il nome dell'istituzione <strong>%s</strong> non &egrave; cambiato.";
#elif L==7
	"The name of the institution <strong>%s</strong> has not changed.";		// Potrzebujesz tlumaczenie
#elif L==8
	"The name of the institution <strong>%s</strong> has not changed.";		// Necessita de traduo
#endif

const char *Txt_The_name_of_the_link_X_has_not_changed =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"El nombre del enlace <strong>%s</strong> no ha cambiado.";	// Necessita traduccio
#elif L==1
	"The name of the link <strong>%s</strong> has not changed.";	// Need bersetzung
#elif L==2
	"The name of the link <strong>%s</strong> has not changed.";
#elif L==3
	"El nombre del enlace <strong>%s</strong> no ha cambiado.";
#elif L==4
	"The name of the link <strong>%s</strong> has not changed.";	// Besoin de traduction
#elif L==5
	"El nombre del enlace <strong>%s</strong> no ha cambiado.";	// Okoteve traduccin
#elif L==6
	"Il nome del link <strong>%s</strong> non &egrave; cambiato.";
#elif L==7
	"The name of the link <strong>%s</strong> has not changed.";	// Potrzebujesz tlumaczenie
#elif L==8
	"The name of the link <strong>%s</strong> has not changed.";	// Necessita de traduo
#endif

const char *Txt_The_name_of_the_place_X_has_not_changed =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"El nombre del lugar <strong>%s</strong> no ha cambiado.";	// Necessita traduccio
#elif L==1
	"The name of the place <strong>%s</strong> has not changed.";	// Need bersetzung
#elif L==2
	"The name of the place <strong>%s</strong> has not changed.";
#elif L==3
	"El nombre del lugar <strong>%s</strong> no ha cambiado.";
#elif L==4
	"The name of the place <strong>%s</strong> has not changed.";	// Besoin de traduction
#elif L==5
	"El nombre del lugar <strong>%s</strong> no ha cambiado.";	// Okoteve traduccin
#elif L==6
	"Il nome del luogo <strong>%s</strong> non &egrave; cambiato.";
#elif L==7
	"The name of the place <strong>%s</strong> has not changed.";	// Potrzebujesz tlumaczenie
#elif L==8
	"The name of the place <strong>%s</strong> has not changed.";	// Necessita de traduo
#endif

const char *Txt_The_name_of_the_plugin_X_has_not_changed =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"El nombre del complemento <strong>%s</strong> no ha cambiado.";	// Necessita traduccio
#elif L==1
	"The name of the plugin <strong>%s</strong> has not changed.";		// Need bersetzung
#elif L==2
	"The name of the plugin <strong>%s</strong> has not changed.";
#elif L==3
	"El nombre del complemento <strong>%s</strong> no ha cambiado.";
#elif L==4
	"The name of the plugin <strong>%s</strong> has not changed.";		// Besoin de traduction
#elif L==5
	"El nombre del complemento <strong>%s</strong> no ha cambiado.";	// Okoteve traduccin
#elif L==6
	"Il nome del plugin <strong>%s</strong> non &egrave; cambiato.";
#elif L==7
	"The name of the plugin <strong>%s</strong> has not changed.";		// Potrzebujesz tlumaczenie
#elif L==8
	"The name of the plugin <strong>%s</strong> has not changed.";		// Necessita de traduo
#endif

const char *Txt_The_name_of_the_type_of_degree_X_has_not_changed =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"El nombre del tipo de titulaci&oacute;n <strong>%s</strong> no ha cambiado.";	// Necessita traduccio
#elif L==1
	"The name of the type of degree <strong>%s</strong> has not changed.";		// Need bersetzung
#elif L==2
	"The name of the type of degree <strong>%s</strong> has not changed.";
#elif L==3
	"El nombre del tipo de titulaci&oacute;n <strong>%s</strong> no ha cambiado.";
#elif L==4
	"The name of the type of degree <strong>%s</strong> has not changed.";		// Besoin de traduction
#elif L==5
	"El nombre del tipo de titulaci&oacute;n <strong>%s</strong> no ha cambiado.";	// Okoteve traduccin
#elif L==6
	"Il nome del tipo di laurea <strong>%s</strong> non &egrave; cambiato.";
#elif L==7
	"The name of the type of degree <strong>%s</strong> has not changed.";		// Potrzebujesz tlumaczenie
#elif L==8
	"The name of the type of degree <strong>%s</strong> has not changed.";		// Necessita de traduo
#endif

const char *Txt_The_name_of_the_type_of_group_X_has_not_changed =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"El nombre del tipo de grupo <strong>%s</strong> no ha cambiado.";	// Necessita traduccio
#elif L==1
	"The name of the type of group <strong>%s</strong> has not changed.";	// Need bersetzung
#elif L==2
	"The name of the type of group <strong>%s</strong> has not changed.";
#elif L==3
	"El nombre del tipo de grupo <strong>%s</strong> no ha cambiado.";
#elif L==4
	"The name of the type of group <strong>%s</strong> has not changed.";	// Besoin de traduction
#elif L==5
	"El nombre del tipo de grupo <strong>%s</strong> no ha cambiado.";	// Okoteve traduccin
#elif L==6
	"Il nome del tipo di gruppo <strong>%s</strong> non &egrave; cambiato.";
#elif L==7
	"The name of the type of group <strong>%s</strong> has not changed.";	// Potrzebujesz tlumaczenie
#elif L==8
	"The name of the type of group <strong>%s</strong> has not changed.";	// Necessita de traduo
#endif

const char *Txt_The_new_IP_address_is_X =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"La nueva direcci&oacute;n IP es <strong>%s</strong>.";	// Necessita traduccio
#elif L==1
	"The new IP address is <strong>%s</strong>.";		// Need bersetzung
#elif L==2
	"The new IP address is <strong>%s</strong>.";
#elif L==3
	"La nueva direcci&oacute;n IP es <strong>%s</strong>.";
#elif L==4
	"The new IP address is <strong>%s</strong>.";		// Besoin de traduction
#elif L==5
	"La nueva direcci&oacute;n IP es <strong>%s</strong>.";	// Okoteve traduccin
#elif L==6
	"Il nuovo indirizzo IP &egrave; <strong>%s</strong>.";
#elif L==7
	"The new IP address is <strong>%s</strong>.";		// Potrzebujesz tlumaczenie
#elif L==8
	"The new IP address is <strong>%s</strong>.";		// Necessita de traduo
#endif

const char *Txt_The_new_logo_is_X =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"El nuevo logo es <strong>%s</strong>.";	// Necessita traduccio
#elif L==1
	"The new logo is <strong>%s</strong>.";		// Need bersetzung
#elif L==2
	"The new logo is <strong>%s</strong>.";
#elif L==3
	"El nuevo logo es <strong>%s</strong>.";
#elif L==4
	"The new logo is <strong>%s</strong>.";		// Besoin de traduction
#elif L==5
	"El nuevo logo es <strong>%s</strong>.";	// Okoteve traduccin
#elif L==6
	"Il nuovo logo &egrave; <strong>%s</strong>.";
#elif L==7
	"The new logo is <strong>%s</strong>.";		// Potrzebujesz tlumaczenie
#elif L==8
	"The new logo is <strong>%s</strong>.";		// Necessita de traduo
#endif

const char *Txt_The_nickname_entered_X_is_not_valid_ =	// Warning: it is very important to include %s and two %u in the following sentences
#if   L==0
	"El apodo introducido"
	" <strong>%s</strong> no es v&aacute;lido.<br />"
	"El apodo debe comenzar por @"
	" y tener una longitud entre %u y %u caracteres (sin contar la @).<br />"
	"Cada car&aacute;cter puede ser una letra (a-z, A-Z),"
	" un d&iacute;gito (0-9) o &quot;_&quot;.";	// Necessita traduccio
#elif L==1
	"The nickname entered"
	" <strong>%s</strong> is not valid.<br />"
	"Your nickname must start with @"
	" and must have a length between %u and %u characters (not including the @).<br />"
	"Each character can be a letter (a-z, A-Z),"
	" a digit (0-9) or &quot;_&quot;.";		// Need bersetzung
#elif L==2
	"The nickname entered"
	" <strong>%s</strong> is not valid.<br />"
	"Your nickname must start with @"
	" and must have a length between %u and %u characters (not including the @).<br />"
	"Each character can be a letter (a-z, A-Z),"
	" a digit (0-9) or &quot;_&quot;.";
#elif L==3
	"El apodo introducido"
	" <strong>%s</strong> no es v&aacute;lido.<br />"
	"El apodo debe comenzar por @"
	" y tener una longitud entre %u y %u caracteres (sin contar la @).<br />"
	"Cada car&aacute;cter puede ser una letra (a-z, A-Z),"
	" un d&iacute;gito (0-9) o &quot;_&quot;.";
#elif L==4
	"The nickname entered"
	" <strong>%s</strong> is not valid.<br />"
	"Your nickname must start with @"
	" and must have a length between %u and %u characters (not including the @).<br />"
	"Each character can be a letter (a-z, A-Z),"
	" a digit (0-9) or &quot;_&quot;.";		// Besoin de traduction
#elif L==5
	"El apodo introducido"
	" <strong>%s</strong> no es v&aacute;lido.<br />"
	"El apodo debe comenzar por @"
	" y tener una longitud entre %u y %u caracteres (sin contar la @).<br />"
	"Cada car&aacute;cter puede ser una letra (a-z, A-Z),"
	" un d&iacute;gito (0-9) o &quot;_&quot;.";	// Okoteve traduccin
#elif L==6
	"Il nome utente immesso"
	" <strong>%s</strong> non &egrave; valido.<br />"
	"Your nickname must start with @"
	" and must have a length between %u and %u characters (not including the @).<br /> "
	"Each character can be a letter (a-z, A-Z),"
	" a digit (0-9) or &quot;_&quot;.";		// Bisogno di traduzione
#elif L==7
	"The nickname entered"
	" <strong>%s</strong> is not valid.<br />"
	"Your nickname must start with @"
	" and must have a length between %u and %u characters (not including the @).<br />"
	"Each character can be a letter (a-z, A-Z),"
	" a digit (0-9) or &quot;_&quot;.";		// Potrzebujesz tlumaczenie
#elif L==8
	"The nickname entered"
	" <strong>%s</strong> is not valid.<br />"
	"Your nickname must start with @"
	" and must have a length between %u and %u characters (not including the @).<br />"
	"Each character can be a letter (a-z, A-Z),"
	" a digit (0-9) or &quot;_&quot;.";		// Necessita de traduo
#endif

const char *Txt_The_notice_you_enter_here_will_appear_as_a_yellow_note_ =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"El aviso que escriba aqu&iacute; aparecer&aacute; en forma de nota amarilla"
	" a la izquierda de la p&aacute;gina y ser&aacute; <strong>visible por todo el mundo</strong>"
	" (no s&oacute;lo por los usuarios de la asignatura %s).<br />"
	"<strong>El mensaje debe ser breve</strong>."
	" Si desea crear un aviso extenso o de acceso exclusivo para sus estudiantes,"
	" puede usar el foro de la asignatura,"
	" enviar un mensaje a todos a trav&eacute;s del sistema de mensajer&iacute;a interna,"
	" o incluso crear un archivo con el texto y a&ntilde;adirlo a la zona de documentos.";	// Necessita traduccio
#elif L==1
	"The notice you enter here will appear as a yellow note"
	" in the left of the page and will be visible <strong>worldwide</strong>"
	" (not just by users of the course %s).<br />"
	"<strong>The message should be short</strong>."
	" If you want to create a long notice or a notice with exclusive access to your students,"
	" you can use the forum of the course,"
	" or send a message to all through the internal messaging system,"
	" or even create a file with the text and upload it to the documents area.";	// Need bersetzung
#elif L==2
	"The notice you enter here will appear as a yellow note"
	" in the left of the page and will be visible <strong>worldwide</strong>"
	" (not just by users of the course %s).<br />"
	"<strong>The message should be short</strong>."
	" If you want to create a long notice or a notice with exclusive access to your students,"
	" you can use the forum of the course,"
	" or send a message to all through the internal messaging system,"
	" or even create a file with the text and upload it to the documents area.";
#elif L==3
	"El aviso que escriba aqu&iacute; aparecer&aacute; en forma de nota amarilla"
	" a la izquierda de la p&aacute;gina y ser&aacute; <strong>visible por todo el mundo</strong>"
	" (no s&oacute;lo por los usuarios de la asignatura %s).<br />"
	"<strong>El mensaje debe ser breve</strong>."
	" Si desea crear un aviso extenso o de acceso exclusivo para sus estudiantes,"
	" puede usar el foro de la asignatura,"
	" enviar un mensaje a todos a trav&eacute;s del sistema de mensajer&iacute;a interna,"
	" o incluso crear un archivo con el texto y a&ntilde;adirlo a la zona de documentos.";
#elif L==4
	"The notice you enter here will appear as a yellow note"
	" in the left of the page and will be visible <strong>worldwide</strong>"
	" (not just by users of the course %s).<br />"
	"<strong>The message should be short</strong>."
	" If you want to create a long notice or a notice with exclusive access to your students,"
	" you can use the forum of the course,"
	" or send a message to all through the internal messaging system,"
	" or even create a file with the text and upload it to the documents area.";	// Besoin de traduction
#elif L==5
	"El aviso que escriba aqu&iacute; aparecer&aacute; en forma de nota amarilla"
	" a la izquierda de la p&aacute;gina y ser&aacute; <strong>visible por todo el mundo</strong>"
	" (no s&oacute;lo por los usuarios de la asignatura %s).<br />"
	"<strong>El mensaje debe ser breve</strong>."
	" Si desea crear un aviso extenso o de acceso exclusivo para sus estudiantes,"
	" puede usar el foro de la asignatura,"
	" enviar un mensaje a todos a trav&eacute;s del sistema de mensajer&iacute;a interna,"
	" o incluso crear un archivo con el texto y a&ntilde;adirlo a la zona de documentos.";	// Okoteve traduccin
#elif L==6
	"L'avviso che hai inserito qui apparir&agrave; come una nota in giallo"
	" sulla parte sinistra della pagina &egrave; sar&agrave; <strong>visibile a tutti</strong>"
	" (non solo da utenti del corso %s).<br />"
	"<strong>Il messaggio dovr&agrave; essere breve</strong>."
	" Se vuoi creare un avviso lungo o un avviso con accesso esclusivo per i tuoi studenti,"
	" potrai usare il forum del corso,"
	" o inviare un messaggio a tutti attraverso il sistema di messaggistica interno,"
	" o sempre creare un file con il testo e caricarlo nell'area documenti.";
#elif L==7
	"The notice you enter here will appear as a yellow note"
	" in the left of the page and will be visible <strong>worldwide</strong>"
	" (not just by users of the course %s).<br />"
	"<strong>The message should be short</strong>."
	" If you want to create a long notice or a notice with exclusive access to your students,"
	" you can use the forum of the course,"
	" or send a message to all through the internal messaging system,"
	" or even create a file with the text and upload it to the documents area.";	// Potrzebujesz tlumaczenie
#elif L==8
	"The notice you enter here will appear as a yellow note"
	" in the left of the page and will be visible <strong>worldwide</strong>"
	" (not just by users of the course %s).<br />"
	"<strong>The message should be short</strong>."
	" If you want to create a long notice or a notice with exclusive access to your students,"
	" you can use the forum of the course,"
	" or send a message to all through the internal messaging system,"
	" or even create a file with the text and upload it to the documents area.";	// Necessita de traduo
#endif

const char *Txt_The_new_web_address_is_X =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"La nueva direcci&oacute;n web es <strong>%s</strong>.";	// Necessita traduccio
#elif L==1
	"The new web address is <strong>%s</strong>.";			// Need bersetzung
#elif L==2
	"The new web address is <strong>%s</strong>.";
#elif L==3
	"La nueva direcci&oacute;n web es <strong>%s</strong>.";
#elif L==4
	"The new web address is <strong>%s</strong>.";			// Besoin de traduction
#elif L==5
	"La nueva direcci&oacute;n web es <strong>%s</strong>.";	// Okoteve traduccin
#elif L==6
	"Il nuovo indirizzo web &egrave; <strong>%s</strong>.";
#elif L==7
	"The new web address is <strong>%s</strong>.";			// Potrzebujesz tlumaczenie
#elif L==8
	"The new web address is <strong>%s</strong>.";			// Necessita de traduo
#endif

const char *Txt_The_new_description_is_X =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"La nueva descripci&oacute;n web es <strong>%s</strong>.";	// Necessita traduccio
#elif L==1
	"The new description is <strong>%s</strong>.";			// Need bersetzung
#elif L==2
	"The new description is <strong>%s</strong>.";
#elif L==3
	"La nueva descripci&oacute;n web es <strong>%s</strong>.";
#elif L==4
	"The new description is <strong>%s</strong>.";			// Besoin de traduction
#elif L==5
	"La nueva descripci&oacute;n web es <strong>%s</strong>.";	// Okoteve traduccin
#elif L==6
	"La nuova descrizione &egrave; <strong>%s</strong>.";
#elif L==7
	"The new description is <strong>%s</strong>.";			// Potrzebujesz tlumaczenie
#elif L==8
	"The new description is <strong>%s</strong>.";			// Necessita de traduo
#endif

const char *Txt_The_new_image_is_X =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"La nueva imagen es <strong>%s</strong>.";	// Necessita traduccio
#elif L==1
	"The new image is <strong>%s</strong>.";	// Need bersetzung
#elif L==2
	"The new image is <strong>%s</strong>.";
#elif L==3
	"La nueva imagen es <strong>%s</strong>.";
#elif L==4
	"The new image is <strong>%s</strong>.";	// Besoin de traduction
#elif L==5
	"La nueva imagen es <strong>%s</strong>.";	// Okoteve traduccin
#elif L==6
	"La nuova image &egrave; <strong>%s</strong>.";
#elif L==7
	"The new image is <strong>%s</strong>.";	// Potrzebujesz tlumaczenie
#elif L==8
	"The new image is <strong>%s</strong>.";	// Necessita de traduo
#endif

const char *Txt_The_new_password_can_not_consist_only_of_digits =
#if   L==0
	"La contrase&ntilde;a nueva no puede estar formada"
	" s&oacute;lo por d&iacute;gitos.";		// Necessita traduccio
#elif L==1
	"The new password can not consist"
	" only of digits.";				// Need bersetzung
#elif L==2
	"The new password can not consist"
	" only of digits.";
#elif L==3
	"La contrase&ntilde;a nueva no puede estar formada"
	" s&oacute;lo por d&iacute;gitos.";
#elif L==4
	"The new password can not consist"
	" only of digits.";				// Besoin de traduction
#elif L==5
	"La contrase&ntilde;a nueva no puede estar formada"
	" s&oacute;lo por d&iacute;gitos.";		// Okoteve traduccin
#elif L==6
	"La nuova password non pu&ograve; consistere"
	" solo di numeri.";
#elif L==7
	"The new password can not consist"
	" only of digits.";				// Potrzebujesz tlumaczenie
#elif L==8
	"The new password can not consist"
	" only of digits.";				// Necessita de traduo
#endif

const char *Txt_The_new_password_can_not_contain_spaces =
#if   L==0
	"La contrase&ntilde;a nueva no puede tener espacios.";	// Necessita traduccio
#elif L==1
	"The new password can not contain spaces.";		// Need bersetzung
#elif L==2
	"The new password can not contain spaces.";
#elif L==3
	"La contrase&ntilde;a nueva no puede tener espacios.";
#elif L==4
	"The new password can not contain spaces.";		// Besoin de traduction
#elif L==5
	"La contrase&ntilde;a nueva no puede tener espacios.";	// Okoteve traduccin
#elif L==6
	"La nuova password non pu&ograve; contenere spazi.";
#elif L==7
	"The new password can not contain spaces.";		// Potrzebujesz tlumaczenie
#elif L==8
	"The new password can not contain spaces.";		// Necessita de traduo
#endif

const char *Txt_The_new_password_is_too_trivial_ =
#if   L==0
	"La contrase&ntilde;a nueva es demasiado f&aacute;cil."
	" Use otra m&aacute;s dif&iacute;cil de averiguar.";	// Necessita traduccio
#elif L==1
	"The new password is too trivial."
	" Use other harder to guess.";				// Need bersetzung
#elif L==2
	"The new password is too trivial."
	" Use other harder to guess.";
#elif L==3
	"La contrase&ntilde;a nueva es demasiado f&aacute;cil."
	" Use otra m&aacute;s dif&iacute;cil de averiguar.";
#elif L==4
	"The new password is too trivial."
	" Use other harder to guess.";				// Besoin de traduction
#elif L==5
	"La contrase&ntilde;a nueva es demasiado f&aacute;cil."
	" Use otra m&aacute;s dif&iacute;cil de averiguar.";	// Okoteve traduccin
#elif L==6
	"La nuova password &egrave; troppo semplice."
	" Utilizza un'altra pi&ugrave; difficile da individuare.";
#elif L==7
	"The new password is too trivial."
	" Use other harder to guess.";				// Potrzebujesz tlumaczenie
#elif L==8
	"The new password is too trivial."
	" Use other harder to guess.";				// Necessita de traduo
#endif

const char *Txt_The_new_password_must_be_at_least_X_characters =	// Warning: it is very important to include %u in the following sentences
#if   L==0
	"La contrase&ntilde;a nueva ha de tener al menos %u caracteres.";	// Necessita traduccio
#elif L==1
	"The new password must be at least %u characters.";			// Need bersetzung
#elif L==2
	"The new password must be at least %u characters.";
#elif L==3
	"La contrase&ntilde;a nueva ha de tener al menos %u caracteres.";
#elif L==4
	"The new password must be at least %u characters.";			// Besoin de traduction
#elif L==5
	"La contrase&ntilde;a nueva ha de tener al menos %u caracteres.";	// Okoteve traduccin
#elif L==6
	"La nuova password dovrebbe essere di almeno %u caratteri.";
#elif L==7
	"The new password must be at least %u characters.";			// Potrzebujesz tlumaczenie
#elif L==8
	"The new password must be at least %u characters.";			// Necessita de traduo
#endif

const char *Txt_The_new_URL_is_X =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"El nuevo URL es <strong>%s</strong>.";		// Necessita traduccio
#elif L==1
	"The new URL is <strong>%s</strong>.";		// Need bersetzung
#elif L==2
	"The new URL is <strong>%s</strong>.";
#elif L==3
	"El nuevo URL es <strong>%s</strong>.";
#elif L==4
	"The new URL is <strong>%s</strong>.";		// Besoin de traduction
#elif L==5
	"El nuevo URL es <strong>%s</strong>.";		// Okoteve traduccin
#elif L==6
	"La nuova URL &egrave; <strong>%s</strong>.";
#elif L==7
	"The new URL is <strong>%s</strong>.";		// Potrzebujesz tlumaczenie
#elif L==8
	"The new URL is <strong>%s</strong>.";		// Necessita de traduo
#endif

const char *Txt_The_nickname_X_had_been_registered_by_another_user =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"El apodo <strong>@%s</strong>"
	" ya hab&iacute;a sido registrado por otro usuario.";	// Necessita traduccio
#elif L==1
	"The nickname <strong>@%s</strong>"
	" had been registered by another user.";		// Need bersetzung
#elif L==2
	"The nickname <strong>@%s</strong>"
	" had been registered by another user.";
#elif L==3
	"El apodo <strong>@%s</strong>"
	" ya hab&iacute;a sido registrado por otro usuario.";
#elif L==4
	"The nickname <strong>@%s</strong>"
	" had been enrolled by another user.";			// Besoin de traduction
#elif L==5
	"El apodo <strong>@%s</strong>"
	" ya hab&iacute;a sido registrado por otro usuario.";	// Okoteve traduccin
#elif L==6
	"Il nome utente <strong>@%s</strong>"
	" &egrave; stato registrato da un altro utente.";
#elif L==7
	"The nickname <strong>@%s</strong>"
	" had been registered by another user.";		// Potrzebujesz tlumaczenie
#elif L==8
	"The nickname <strong>@%s</strong>"
	" had been registered by another user.";		// Necessita de traduo
#endif

const char *Txt_The_nickname_X_matches_the_one_you_had_previously_registered =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"El apodo <strong>@%s</strong>"
	" coincide con el que ya ten&iacute;a registrado previamente.";	// Necessita traduccio
#elif L==1
	"The nickname <strong>@%s</strong>"
	" matches the one you had previously registered.";		// Need bersetzung
#elif L==2
	"The nickname <strong>@%s</strong>"
	" matches the one you had previously registered.";
#elif L==3
	"El apodo <strong>@%s</strong>"
	" coincide con el que ya ten&iacute;a registrado previamente.";
#elif L==4
	"The nickname <strong>@%s</strong>"
	" matches the one you had previously registered.";		// Besoin de traduction
#elif L==5
	"El apodo <strong>@%s</strong>"
	" coincide con el que ya ten&iacute;a registrado previamente.";	// Okoteve traduccin
#elif L==6
	"Il nome utente <strong>@%s</strong>"
	" coincide con quello che hai precedentemente registrato.";
#elif L==7
	"The nickname <strong>@%s</strong>"
	" matches the one you had previously registered.";		// Potrzebujesz tlumaczenie
#elif L==8
	"The nickname <strong>@%s</strong>"
	" matches the one you had previously registered.";		// Necessita de traduo
#endif

const char *Txt_The_number_of_editing_lines_in_the_record_field_X_has_not_changed =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"El n&ordm; de l&iacute;neas de edici&oacute;n del campo <strong>%s</strong> no ha cambiado.";	// Necessita traduccio
#elif L==1
	"The number of editing lines in the record field <strong>%s</strong> has not changed.";		// Need bersetzung
#elif L==2
	"The number of editing lines in the record field <strong>%s</strong> has not changed.";
#elif L==3
	"El n&ordm; de l&iacute;neas de edici&oacute;n del campo <strong>%s</strong> no ha cambiado.";
#elif L==4
	"The number of editing lines in the record field <strong>%s</strong> has not changed.";		// Besoin de traduction
#elif L==5
	"El n&ordm; de l&iacute;neas de edici&oacute;n del campo <strong>%s</strong> no ha cambiado.";	// Okoteve traduccin
#elif L==6
	"Il numero di linee editate nel campo scheda <strong>%s</strong> non &egrave; cambiato.";
#elif L==7
	"The number of editing lines in the record field <strong>%s</strong> has not changed.";		// Potrzebujesz tlumaczenie
#elif L==8
	"The number of editing lines in the record field <strong>%s</strong> has not changed.";		// Necessita de traduo
#endif

const char *Txt_The_number_of_questions_must_be_in_the_interval_X =	// Warning: it is very important to include two %u in the following sentences
#if   L==0
	"El n&uacute;mero de preguntas debe estar en el intervalo [%u,%u].";	// Necessita traduccio
#elif L==1
	"Die Anzahl der Fragen muss sich im Intervall [%u,%u] befinden.";
#elif L==2
	"The number of questions must be in the interval [%u,%u].";
#elif L==3
	"El n&uacute;mero de preguntas debe estar en el intervalo [%u,%u].";
#elif L==4
	"The number of questions must be in the interval [%u,%u].";		// Besoin de traduction
#elif L==5
	"El n&uacute;mero de preguntas debe estar en el intervalo [%u,%u].";	// Okoteve traduccin
#elif L==6
	"Il numero di domande dovrebbe stare nell'intervallo [%u,%u].";
#elif L==7
	"The number of questions must be in the interval [%u,%u].";		// Potrzebujesz tlumaczenie
#elif L==8
	"The number of questions must be in the interval [%u,%u].";		// Necessita de traduo
#endif

const char *Txt_The_number_of_rows_is_now_X =	// Warning: it is very important to include %u in the following sentences
#if   L==0
	"El n&uacute;mero de filas es ahora <strong>%u</strong>.";	// Necessita traduccio
#elif L==1
	"Die Anzahl der Zeilen ist jetzt <strong>%u</strong>.";
#elif L==2
	"The number of rows is now <strong>%u</strong>.";
#elif L==3
	"El n&uacute;mero de filas es ahora <strong>%u</strong>.";
#elif L==4
	"The number of rows is now <strong>%u</strong>.";		// Besoin de traduction
#elif L==5
	"El n&uacute;mero de filas es ahora <strong>%u</strong>.";	// Okoteve traduccin
#elif L==6
	"Il numero di righe &egrave; ora <strong>%u</strong>.";
#elif L==7
	"The number of rows is now <strong>%u</strong>.";		// Potrzebujesz tlumaczenie
#elif L==8
	"The number of rows is now <strong>%u</strong>.";		// Necessita de traduo
#endif

const char *Txt_The_numerical_code_X_already_exists =	// Warning: it is very important to include %03lu in the following sentences
#if   L==0
	"El c&oacute;digo num&eacute;rico ISO 3166-1 %03ld ya existe.";	// Necessita traduccio
#elif L==1
	"The ISO 3166-1 numerical code %03ld already exists.";		// Need bersetzung
#elif L==2
	"The ISO 3166-1 numerical code %03ld already exists.";
#elif L==3
	"El c&oacute;digo num&eacute;rico ISO 3166-1 %03ld ya existe.";
#elif L==4
	"The ISO 3166-1 numerical code %03ld already exists.";		// Besoin de traduction
#elif L==5
	"El c&oacute;digo num&eacute;rico ISO 3166-1 %03ld ya existe.";	// Okoteve traduccin
#elif L==6
	"Il codice numerico ISO 3166-1 %03ld gi&agrave; esiste.";
#elif L==7
	"The ISO 3166-1 numerical code %03ld already exists.";		// Potrzebujesz tlumaczenie
#elif L==8
	"The ISO 3166-1 numerical code %03ld already exists.";		// Necessita de traduo
#endif

const char *Txt_The_X_password_has_been_changed_successfully =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"La contrase&ntilde;a de %s se ha modificado con &eacute;xito.";	// Necessita traduccio
#elif L==1
	"The %s password has been changed successfully.";			// Need bersetzung
#elif L==2
	"The %s password has been changed successfully.";
#elif L==3
	"La contrase&ntilde;a de %s se ha modificado con &eacute;xito.";
#elif L==4
	"The %s password has been changed successfully.";			// Besoin de traduction
#elif L==5
	"La contrase&ntilde;a de %s se ha modificado con &eacute;xito.";	// Okoteve traduccin
#elif L==6
	"La %s password &egrave; stata cambiata con successo.";
#elif L==7
	"The %s password has been changed successfully.";			// Potrzebujesz tlumaczenie
#elif L==8
	"The %s password has been changed successfully.";			// Necessita de traduo
#endif

const char *Txt_The_place_X_already_exists =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"El lugar <strong>%s</strong> ya existe.";		// Necessita traduccio
#elif L==1
	"Der Standort <strong>%s</strong> wurde bereits eingegeben.";
#elif L==2
	"The place <strong>%s</strong> already exists.";
#elif L==3
	"El lugar <strong>%s</strong> ya existe.";
#elif L==4
	"The place <strong>%s</strong> already exists.";	// Besoin de traduction
#elif L==5
	"El lugar <strong>%s</strong> ya existe.";		// Okoteve traduccin
#elif L==6
	"Il luogo <strong>%s</strong> gi&agrave; esiste.";
#elif L==7
	"The place <strong>%s</strong> already exists.";	// Potrzebujesz tlumaczenie
#elif L==8
	"The place <strong>%s</strong> already exists.";	// Necessita de traduo
#endif

const char *Txt_The_place_X_has_been_renamed_as_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==0
	"El lugar <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";	// Necessita traduccio
#elif L==1
	"The place <strong>%s</strong> has been renamed as <strong>%s</strong>.";	// Need bersetzung
#elif L==2
	"The place <strong>%s</strong> has been renamed as <strong>%s</strong>.";
#elif L==3
	"El lugar <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";
#elif L==4
	"The place <strong>%s</strong> has been renamed as <strong>%s</strong>.";	// Besoin de traduction
#elif L==5
	"El lugar <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";	// Okoteve traduccin
#elif L==6
	"Il luogo <strong>%s</strong> &egrave; stato rinominato come <strong>%s</strong>.";
#elif L==7
	"The place <strong>%s</strong> has been renamed as <strong>%s</strong>.";	// Potrzebujesz tlumaczenie
#elif L==8
	"The place <strong>%s</strong> has been renamed as <strong>%s</strong>.";	// Necessita de traduo
#endif

const char *Txt_The_place_of_the_centre_has_changed =
#if   L==0
	"El lugar del centro ha cambiado.";		// Necessita traduccio
#elif L==1
	"The place of the centre has changed.";		// Need bersetzung
#elif L==2
	"The place of the centre has changed.";
#elif L==3
	"El lugar del centro ha cambiado.";
#elif L==4
	"The place of the centre has changed.";		// Besoin de traduction
#elif L==5
	"El lugar del centro ha cambiado.";		// Okoteve traduccin
#elif L==6
	"Il luogo del centro &egrave; cambiato.";
#elif L==7
	"The place of the centre has changed.";		// Potrzebujesz tlumaczenie
#elif L==8
	"The place of the centre has changed.";		// Necessita de traduo
#endif

const char *Txt_The_place_of_the_holiday_X_has_changed_to_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==0
	"El lugar de la festividad <strong>%s</strong> ha cambiado a <strong>%s</strong>.";	// Necessita traduccio
#elif L==1
	"The place of the holiday <strong>%s</strong> has changed to <strong>%s</strong>.";	// Need bersetzung
#elif L==2
	"The place of the holiday <strong>%s</strong> has changed to <strong>%s</strong>.";
#elif L==3
	"El lugar de la festividad <strong>%s</strong> ha cambiado a <strong>%s</strong>.";
#elif L==4
	"The place of the holiday <strong>%s</strong> has changed to <strong>%s</strong>.";	// Besoin de traduction
#elif L==5
	"El lugar de la festividad <strong>%s</strong> ha cambiado a <strong>%s</strong>.";	// Okoteve traduccin
#elif L==6
	"Il luogo della festivit&agrave; <strong>%s</strong> &egrave; cambiato a <strong>%s</strong>.";
#elif L==7
	"The place of the holiday <strong>%s</strong> has changed to <strong>%s</strong>.";	// Potrzebujesz tlumaczenie
#elif L==8
	"The place of the holiday <strong>%s</strong> has changed to <strong>%s</strong>.";	// Necessita de traduo
#endif

const char *Txt_The_plugin_X_already_exists =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"El complemento <strong>%s</strong> ya existe.";	// Necessita traduccio
#elif L==1
	"Der Plugin <strong>%s</strong> wurde bereits installiert.";
#elif L==2
	"The plugin <strong>%s</strong> already exists.";
#elif L==3
	"El complemento <strong>%s</strong> ya existe.";
#elif L==4
	"The plugin <strong>%s</strong> already exists.";	// Besoin de traduction
#elif L==5
	"El complemento <strong>%s</strong> ya existe.";	// Okoteve traduccin
#elif L==6
	"Il plugin <strong>%s</strong> gi&agrave; esiste.";
#elif L==7
	"The plugin <strong>%s</strong> already exists.";	// Potrzebujesz tlumaczenie
#elif L==8
	"The plugin <strong>%s</strong> already exists.";	// Necessita de traduo
#endif

const char *Txt_The_plugin_X_has_been_renamed_as_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==0
	"El complemento <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";	// Necessita traduccio
#elif L==1
	"The plugin <strong>%s</strong> has been renamed as <strong>%s</strong>.";		// Need bersetzung
#elif L==2
	"The plugin <strong>%s</strong> has been renamed as <strong>%s</strong>.";
#elif L==3
	"El complemento <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";
#elif L==4
	"The plugin <strong>%s</strong> has been renamed as <strong>%s</strong>.";		// Besoin de traduction
#elif L==5
	"El complemento <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";	// Okoteve traduccin
#elif L==6
	"Il plugin <strong>%s</strong> &egrave; stato rinominato come <strong>%s</strong>.";
#elif L==7
	"The plugin <strong>%s</strong> has been renamed as <strong>%s</strong>.";		// Potrzebujesz tlumaczenie
#elif L==8
	"The plugin <strong>%s</strong> has been renamed as <strong>%s</strong>.";		// Necessita de traduo
#endif

const char *Txt_The_properties_of_file_X_have_been_saved =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"Las propiedades del archivo %s se han guardado.";	// Necessita traduccio
#elif L==1
	"The properties of file %s have been saved.";		// Need bersetzung
#elif L==2
	"The properties of file %s have been saved.";
#elif L==3
	"Las propiedades del archivo %s se han guardado.";
#elif L==4
	"The properties of file %s have been saved.";		// Besoin de traduction
#elif L==5
	"Las propiedades del archivo %s se han guardado.";	// Okoteve traduccin
#elif L==6
	"The properties of file %s have been saved.";		// Bisogno di traduzione
#elif L==7
	"The properties of file %s have been saved.";		// Potrzebujesz tlumaczenie
#elif L==8
	"The properties of file %s have been saved.";		// Necessita de traduo
#endif

const char *Txt_The_record_field_X_already_exists =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"El campo de ficha <strong>%s</strong> ya existe.";	// Necessita traduccio
#elif L==1
	"Das Datenfeld <strong>%s</strong> wurde bereits erstellt.";
#elif L==2
	"The record field <strong>%s</strong> already exists.";
#elif L==3
	"El campo de ficha <strong>%s</strong> ya existe.";
#elif L==4
	"The record field <strong>%s</strong> already exists.";	// Besoin de traduction
#elif L==5
	"El campo de ficha <strong>%s</strong> ya existe.";	// Okoteve traduccin
#elif L==6
	"Il campo di scheda <strong>%s</strong> gi&agrave; esiste.";
#elif L==7
	"The record field <strong>%s</strong> already exists.";	// Potrzebujesz tlumaczenie
#elif L==8
	"The record field <strong>%s</strong> already exists.";	// Necessita de traduo
#endif

const char *Txt_The_record_field_X_has_been_renamed_as_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==0
	"El campo de ficha <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";	// Necessita traduccio
#elif L==1
	"The record field <strong>%s</strong> has been renamed as <strong>%s</strong>.";	// Need bersetzung
#elif L==2
	"The record field <strong>%s</strong> has been renamed as <strong>%s</strong>.";
#elif L==3
	"El campo de ficha <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";
#elif L==4
	"The record field <strong>%s</strong> has been renamed as <strong>%s</strong>.";	// Besoin de traduction
#elif L==5
	"El campo de ficha <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";	// Okoteve traduccin
#elif L==6
	"Il campo di scheda <strong>%s</strong> &egrave; stato rinominato come <strong>%s</strong>.";
#elif L==7
	"The record field <strong>%s</strong> has been renamed as <strong>%s</strong>.";	// Potrzebujesz tlumaczenie
#elif L==8
	"The record field <strong>%s</strong> has been renamed as <strong>%s</strong>.";	// Necessita de traduo
#endif

const char *Txt_The_requested_group_changes_were_successful =
#if   L==0
	"Els canvis de grup sollicitats s'han realitzat correctament.";
#elif L==1
	"The requested group changes were successful.";				// Need bersetzung
#elif L==2
	"The requested group changes were successful.";
#elif L==3
	"Los cambios de grupo solicitados se han realizado correctamente.";
#elif L==4
	"Les changements de groupe demand&eacute;es ont r&eacute;ussi.";
#elif L==5
	"Los cambios de grupo solicitados se han realizado correctamente.";	// Okoteve traduccin
#elif L==6
	"I cambiamenti di gruppi richieste hanno avuto successo.";
#elif L==7
	"The requested group changes were successful.";				// Potrzebujesz tlumaczenie
#elif L==8
	"The requested group changes were successful.";				// Necessita de traduo
#endif

const char *Txt_The_rich_text_editor_is_not_yet_available =
#if   L==0
	"L'editor de text enriquit encara no est&agrave; disponible.";
#elif L==1
	"Der Rich Text Editor ist noch nicht verfgbar.";
#elif L==2
	"The rich text editor is not yet available.";
#elif L==3
        "El editor de texto enriquecido a&uacute;n no est&aacute; disponible.";
#elif L==4
        "L'&eacute;diteur de texte riche n'est pas encore disponible.";
#elif L==5
        "El editor de texto enriquecido a&uacute;n no est&aacute; disponible.";	// Okoteve traduccin
#elif L==6
        "Il rich text editor non &egrave; ancora disponibile.";
#elif L==7
        "Edytor tekstu nie jest jeszcze dost&eogon;pna.";
#elif L==8
        "O editor de texto rico ainda n&atilde;o est&aacute; disponvel.";
#endif

const char *Txt_The_role_of_THE_USER_X_in_the_course_Y_has_changed_from_A_to_B =	// Warning: it is very important to include four %s in the following sentences
#if   L==0
	"El rol de <strong>%s</strong>"
	" en la asignatura <strong>%s</strong>"
	" ha cambiado de %s a %s.";		// Necessita traduccio
#elif L==1
	"The role of <strong>%s</strong>"
	" in the course <strong>%s</strong>"
	" has changed from %s to %s.";		// Need bersetzung
#elif L==2
	"The role of <strong>%s</strong>"
	" in the course <strong>%s</strong>"
	" has changed from %s to %s.";
#elif L==3
	"El rol de <strong>%s</strong>"
	" en la asignatura <strong>%s</strong>"
	" ha cambiado de %s a %s.";
#elif L==4
	"The role of <strong>%s</strong>"
	" in the course <strong>%s</strong>"
	" has changed from %s to %s.";		// Besoin de traduction
#elif L==5
	"El rol de <strong>%s</strong>"
	" en la asignatura <strong>%s</strong>"
	" ha cambiado de %s a %s.";		// Okoteve traduccin
#elif L==6
	"Il ruolo de <strong>%s</strong>"
	" nel corso <strong>%s</strong>"
	" &egrave; cambiato da %s a %s.";
#elif L==7
	"The role of <strong>%s</strong>"
	" in the course <strong>%s</strong>"
	" has changed from %s to %s.";		// Potrzebujesz tlumaczenie
#elif L==8
	"The role of <strong>%s</strong>"
	" in the course <strong>%s</strong>"
	" has changed from %s to %s.";		// Necessita de traduo
#endif

const char *Txt_The_X_students_who_belonged_to_the_course_Y_have_been_removed_from_it =	// Warning: it is very important to include %u and %s in the following sentences
#if   L==0
	"Se han eliminado los %u estudiantes que hab&iacute;a en la asignatura <strong>%s</strong>.";	// Necessita traduccio
#elif L==1
	"The %u students who belonged to the course <strong>%s</strong> have been removed from it.";	// Need bersetzung
#elif L==2
	"The %u students who belonged to the course <strong>%s</strong> have been removed from it.";
#elif L==3
	"Se han eliminado los %u estudiantes que hab&iacute;a en la asignatura <strong>%s</strong>.";
#elif L==4
	"The %u students who belonged to the course <strong>%s</strong> have been removed from it.";	// Besoin de traduction
#elif L==5
	"Se han eliminado los %u estudiantes que hab&iacute;a en la asignatura <strong>%s</strong>.";	// Okoteve traduccin
#elif L==6
	"Gli %u studenti che appartengono al corso <strong>%s</strong> sono stati rimossi da esso.";
#elif L==7
	"The %u students who belonged to the course <strong>%s</strong> have been removed from it.";	// Potrzebujesz tlumaczenie
#elif L==8
	"The %u students who belonged to the course <strong>%s</strong> have been removed from it.";	// Necessita de traduo
#endif

const char *Txt_The_search_term_must_be_longer =
#if   L==0
	"La palabra a buscar debe ser m&aacute;s larga.";	// Necessita traduccio
#elif L==1
	"The search term must be longer.";			// Need bersetzung
#elif L==2
	"The search term must be longer.";
#elif L==3
	"La palabra a buscar debe ser m&aacute;s larga.";
#elif L==4
	"The search term must be longer.";			// Besoin de traduction
#elif L==5
	"La palabra a buscar debe ser m&aacute;s larga.";	// Okoteve traduccin
#elif L==6
	"The search term must be longer.";			// Bisogno di traduzione
#elif L==7
	"The search term must be longer.";			// Potrzebujesz tlumaczenie
#elif L==8
	"The search term must be longer.";			// Necessita de traduo
#endif

const char *Txt_The_semester_of_the_course_X_has_changed =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"El cuatrimestre/semestre de la asignatura <strong>%s</strong> ha cambiado.";	// Necessita traduccio
#elif L==1
	"The semester of the couse <strong>%s</strong> has changed.";			// Need bersetzung
#elif L==2
	"The semester of the couse <strong>%s</strong> has changed.";
#elif L==3
	"El cuatrimestre/semestre de la asignatura <strong>%s</strong> ha cambiado.";
#elif L==4
	"The semester of the couse <strong>%s</strong> has changed.";			// Besoin de traduction
#elif L==5
	"El cuatrimestre/semestre de la asignatura <strong>%s</strong> ha cambiado.";	// Okoteve traduccin
#elif L==6
	"Il semestre del corso <strong>%s</strong> &egrave; cambiato.";
#elif L==7
	"The semester of the couse <strong>%s</strong> has changed.";			// Potrzebujesz tlumaczenie
#elif L==8
	"The semester of the couse <strong>%s</strong> has changed.";			// Necessita de traduo
#endif

const char *Txt_The_session_has_been_closed =	// Closed by user
#if   L==0
	"La sessi&oacute; ha finalitzat.";
#elif L==1
	"Die Sitzung wurde beendet.";
#elif L==2
	"The session has been closed.";
#elif L==3
	"La sesi&oacute;n ha finalizado.";
#elif L==4
	"La session a &eacute;t&eacute; ferm&eacute;e.";
#elif L==5
	"La sesi&oacute;n ha finalizado.";	// Okoteve traduccin
#elif L==6
	"La sessione &egrave; stata chiusa.";
#elif L==7
	"Sesja wygas&lstrok;a.";
#elif L==8
	"A sess&atilde;o foi encerrada.";
#endif

const char *Txt_The_session_has_expired_due_to_inactivity =	// Expired due to timeout
#if   L==0
	"La sessi&oacute; ha expirat per inactivitat.";
#elif L==1
	"Die Sitzung wurde wegen Inaktivit&uml;t abgelaufen.";
#elif L==2
	"The session has expired due to inactivity.";
#elif L==3
	"La sesi&oacute;n ha expirado por inactividad.";
#elif L==4
	"La session a expir&eacute; pour cause d'inactivit&eacute;.";
#elif L==5
	"La sesi&oacute;n ha expirado por inactividad.";	// Okoteve traduccin
#elif L==6
	"La sessione &egrave; scaduta per inattivit&agrave;.";
#elif L==7
	"Sesja wygas&lstrok;a z powodu bezczynno&sacute;ci";
#elif L==8
	"A sess&atilde;o expirou devido &agrave; inatividade.";
#endif

const char *Txt_The_status_of_the_centre_X_has_changed =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"El estado del centro <strong>%s</strong> ha cambiado.";	// Necessita traduccio
#elif L==1
	"The status of the centre <strong>%s</strong> has changed.";	// Need bersetzung
#elif L==2
	"The status of the centre <strong>%s</strong> has changed.";
#elif L==3
	"El estado del centro <strong>%s</strong> ha cambiado.";
#elif L==4
	"The status of the centre <strong>%s</strong> has changed.";	// Besoin de traduction
#elif L==5
	"El estado del centro <strong>%s</strong> ha cambiado.";	// Okoteve traduccin
#elif L==6
	"Il stato del centro <strong>%s</strong> &egrave; cambiato.";
#elif L==7
	"The status of the centre <strong>%s</strong> has changed.";	// Potrzebujesz tlumaczenie
#elif L==8
	"The status of the centre <strong>%s</strong> has changed.";	// Necessita de traduo
#endif

const char *Txt_The_status_of_the_course_X_has_changed =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"El estado de la asignatura <strong>%s</strong> ha cambiado.";	// Necessita traduccio
#elif L==1
	"The status of the course <strong>%s</strong> has changed.";	// Need bersetzung
#elif L==2
	"The status of the course <strong>%s</strong> has changed.";
#elif L==3
	"El estado de la asignatura <strong>%s</strong> ha cambiado.";
#elif L==4
	"The status of the course <strong>%s</strong> has changed.";	// Besoin de traduction
#elif L==5
	"El estado de la asignatura <strong>%s</strong> ha cambiado.";	// Okoteve traduccin
#elif L==6
	"Il stato del corso <strong>%s</strong> &egrave; cambiato.";
#elif L==7
	"The status of the course <strong>%s</strong> has changed.";	// Potrzebujesz tlumaczenie
#elif L==8
	"The status of the course <strong>%s</strong> has changed.";	// Necessita de traduo
#endif

const char *Txt_The_status_of_the_degree_X_has_changed =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"El estado de la titulaci&oacute;n <strong>%s</strong> ha cambiado.";	// Necessita traduccio
#elif L==1
	"The status of the degree <strong>%s</strong> has changed.";		// Need bersetzung
#elif L==2
	"The status of the degree <strong>%s</strong> has changed.";
#elif L==3
	"El estado de la titulaci&oacute;n <strong>%s</strong> ha cambiado.";
#elif L==4
	"The status of the degree <strong>%s</strong> has changed.";		// Besoin de traduction
#elif L==5
	"El estado de la titulaci&oacute;n <strong>%s</strong> ha cambiado.";	// Okoteve traduccin
#elif L==6
	"Il stato della laurea <strong>%s</strong> &egrave; cambiato.";
#elif L==7
	"The status of the degree <strong>%s</strong> has changed.";		// Potrzebujesz tlumaczenie
#elif L==8
	"The status of the degree <strong>%s</strong> has changed.";		// Necessita de traduo
#endif

const char *Txt_The_status_of_the_institution_X_has_changed =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"El estado de la instituci&oacute;n <strong>%s</strong> ha cambiado.";	// Necessita traduccio
#elif L==1
	"The status of the institution <strong>%s</strong> has changed.";	// Need bersetzung
#elif L==2
	"The status of the institution <strong>%s</strong> has changed.";
#elif L==3
	"El estado de la instituci&oacute;n <strong>%s</strong> ha cambiado.";
#elif L==4
	"The status of the institution <strong>%s</strong> has changed.";	// Besoin de traduction
#elif L==5
	"El estado de la instituci&oacute;n <strong>%s</strong> ha cambiado.";	// Okoteve traduccin
#elif L==6
	"Il stato della istituzione <strong>%s</strong> &egrave; cambiato.";
#elif L==7
	"The status of the institution <strong>%s</strong> has changed.";	// Potrzebujesz tlumaczenie
#elif L==8
	"The status of the institution <strong>%s</strong> has changed.";	// Necessita de traduo
#endif

const char *Txt_The_survey_has_been_modified =
#if   L==0
	"La encuesta ha sido modificada.";	// Necessita traduccio
#elif L==1
	"Die Umfrage wurde ge&auml;ndert.";
#elif L==2
	"The survey has been modified.";
#elif L==3
	"La encuesta ha sido modificada.";
#elif L==4
	"Le sondage a &eacute;t&eacute; modifi&eacute;.";
#elif L==5
	"La encuesta ha sido modificada.";	// Okoteve traduccin
#elif L==6
	"Il sondaggio &egrave; stato modificato.";
#elif L==7
	"The survey has been modified.";	// Potrzebujesz tlumaczenie
#elif L==8
	"O inqu&eacute;rito foi modificado.";
#endif

const char *Txt_The_syllabus_lectures_of_the_course_X_is_not_available =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"El programa de teor&iacute;a de la asignatura <strong>%s</strong> no est&aacute; disponible.";	// Necessita traduccio
#elif L==1
	"The syllabus (lectures) of the course <strong>%s</strong> is not available.";			// Need bersetzung
#elif L==2
	"The syllabus (lectures) of the course <strong>%s</strong> is not available.";
#elif L==3
	"El programa de teor&iacute;a de la asignatura <strong>%s</strong> no est&aacute; disponible.";
#elif L==4
	"The syllabus (lectures) of the course <strong>%s</strong> is not available.";			// Besoin de traduction
#elif L==5
	"El programa de teor&iacute;a de la asignatura <strong>%s</strong> no est&aacute; disponible.";	// Okoteve traduccin
#elif L==6
	"Il programma di teoria del corso <strong>%s</strong> non &egrave; disponibile.";
#elif L==7
	"The syllabus (lectures) of the course <strong>%s</strong> is not available.";			// Potrzebujesz tlumaczenie
#elif L==8
	"The syllabus (lectures) of the course <strong>%s</strong> is not available.";			// Necessita de traduo
#endif

const char *Txt_The_syllabus_practicals_of_the_course_X_is_not_available =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"El programa de pr&aacute;cticas de la asignatura <strong>%s</strong> no est&aacute; disponible.";	// Necessita traduccio
#elif L==1
	"The syllabus (practicals) of the course <strong>%s</strong> is not available.";			// Need bersetzung
#elif L==2
	"The syllabus (practicals) of the course <strong>%s</strong> is not available.";
#elif L==3
	"El programa de pr&aacute;cticas de la asignatura <strong>%s</strong> no est&aacute; disponible.";
#elif L==4
	"The syllabus (practicals) of the course <strong>%s</strong> is not available.";			// Besoin de traduction
#elif L==5
	"El programa de pr&aacute;cticas de la asignatura <strong>%s</strong> no est&aacute; disponible.";	// Okoteve traduccin
#elif L==6
	"Il programma della pratica <strong>%s</strong> non &egrave; disponibile.";
#elif L==7
	"The syllabus (practicals) of the course <strong>%s</strong> is not available.";			// Potrzebujesz tlumaczenie
#elif L==8
	"The syllabus (practicals) of the course <strong>%s</strong> is not available.";			// Necessita de traduo
#endif

const char *Txt_The_tag_X_has_been_renamed_as_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==0
	"El descriptor <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";	// Necessita traduccio
#elif L==1
	"The tag <strong>%s</strong> has been renamed as <strong>%s</strong>.";			// Need bersetzung
#elif L==2
	"The tag <strong>%s</strong> has been renamed as <strong>%s</strong>.";
#elif L==3
	"El descriptor <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";
#elif L==4
	"The tag <strong>%s</strong> has been renamed as <strong>%s</strong>.";			// Besoin de traduction
#elif L==5
	"El descriptor <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";	// Okoteve traduccin
#elif L==6
	"L'etichetta <strong>%s</strong> &egrave; stata rinominata come <strong>%s</strong>.";
#elif L==7
	"The tag <strong>%s</strong> has been renamed as <strong>%s</strong>.";			// Potrzebujesz tlumaczenie
#elif L==8
	"The tag <strong>%s</strong> has been renamed as <strong>%s</strong>.";			// Necessita de traduo
#endif

const char *Txt_The_tag_X_has_not_changed =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"El descriptor <strong>%s</strong> no ha cambiado.";	// Necessita traduccio
#elif L==1
	"The tag <strong>%s</strong> has not changed.";		// Need bersetzung
#elif L==2
	"The tag <strong>%s</strong> has not changed.";
#elif L==3
	"El descriptor <strong>%s</strong> no ha cambiado.";
#elif L==4
	"The tag <strong>%s</strong> has not changed.";		// Besoin de traduction
#elif L==5
	"El descriptor <strong>%s</strong> no ha cambiado.";	// Okoteve traduccin
#elif L==6
	"L'etichetta <strong>%s</strong> non &egrave; cambiata.";
#elif L==7
	"The tag <strong>%s</strong> has not changed.";		// Potrzebujesz tlumaczenie
#elif L==8
	"The tag <strong>%s</strong> has not changed.";		// Necessita de traduo
#endif

const char *Txt_The_test_X_has_already_been_assessed_previously =	// Warning: it is very important to include %u in the following sentences
#if   L==0
	"El test %u ya ha sido evaluado anteriormente.";	// Necessita traduccio
#elif L==1
	"The test %u has already been assessed previously.";	// Need bersetzung
#elif L==2
	"The test %u has already been assessed previously.";
#elif L==3
	"El test %u ya ha sido evaluado anteriormente.";
#elif L==4
	"The test %u has already been assessed previously.";	// Besoin de traduction
#elif L==5
	"El test %u ya ha sido evaluado anteriormente.";	// Okoteve traduccin
#elif L==6
	"Il test %u &egrave; stato valutato precedentemente.";
#elif L==7
	"The test %u has already been assessed previously.";	// Potrzebujesz tlumaczenie
#elif L==8
	"The test %u has already been assessed previously.";	// Necessita de traduo
#endif

const char *Txt_The_test_configuration_has_been_updated =
#if   L==0
	"La configuraci&oacute;n de tests se ha actualizado.";	// Necessita traduccio
#elif L==1
	"The test configuration has been updated.";		// Need bersetzung
#elif L==2
	"The test configuration has been updated.";
#elif L==3
	"La configuraci&oacute;n de tests se ha actualizado.";
#elif L==4
	"The test configuration has been updated.";		// Besoin de traduction
#elif L==5
	"La configuraci&oacute;n de tests se ha actualizado.";	// Okoteve traduccin
#elif L==6
	"La configurazione di test &egrave; stata aggiornata.";
#elif L==7
	"The test configuration has been updated.";		// Potrzebujesz tlumaczenie
#elif L==8
	"The test configuration has been updated.";		// Necessita de traduo
#endif

const char *Txt_The_thread_is_already_in_this_forum =
#if   L==0
	"La discusi&oacute;n ya est&aacute; en este foro.";	// Necessita traduccio
#elif L==1
	"The thread is already in this forum.";			// Need bersetzung
#elif L==2
	"The thread is already in this forum.";
#elif L==3
	"La discusi&oacute;n ya est&aacute; en este foro.";
#elif L==4
	"The thread is already in this forum.";			// Besoin de traduction
#elif L==5
	"La discusi&oacute;n ya est&aacute; en este foro.";	// Okoteve traduccin
#elif L==6
	"La discussione &egrave; gi&agrave; in questo forum.";
#elif L==7
	"The thread is already in this forum.";			// Potrzebujesz tlumaczenie
#elif L==8
	"The thread is already in this forum.";			// Necessita de traduo
#endif

const char *Txt_The_thread_X_is_already_in_this_forum =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"La discusi&oacute;n <strong>%s</strong> ya est&aacute; en este foro.";	// Necessita traduccio
#elif L==1
	"The thread <strong>%s</strong> is already in this forum.";		// Need bersetzung
#elif L==2
	"The thread <strong>%s</strong> is already in this forum.";
#elif L==3
	"La discusi&oacute;n <strong>%s</strong> ya est&aacute; en este foro.";
#elif L==4
	"The thread <strong>%s</strong> is already in this forum.";		// Besoin de traduction
#elif L==5
	"La discusi&oacute;n <strong>%s</strong> ya est&aacute; en este foro.";	// Okoteve traduccin
#elif L==6
	"La discussione <strong>%s</strong> &egrave; gi&agrave; in questo forum.";
#elif L==7
	"The thread <strong>%s</strong> is already in this forum.";		// Potrzebujesz tlumaczenie
#elif L==8
	"The thread <strong>%s</strong> is already in this forum.";		// Necessita de traduo
#endif

const char *Txt_The_type_of_degree_X_already_exists =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"El tipo de titulaci&oacute;n <strong>%s</strong> ya existe.";	// Necessita traduccio
#elif L==1
	"Die Abschlussart <strong>%s</strong> wurde bereits definiert.";
#elif L==2
	"The type of degree <strong>%s</strong> already exists.";
#elif L==3
	"El tipo de titulaci&oacute;n <strong>%s</strong> ya existe.";
#elif L==4
	"The type of degree <strong>%s</strong> already exists.";	// Besoin de traduction
#elif L==5
	"El tipo de titulaci&oacute;n <strong>%s</strong> ya existe.";	// Okoteve traduccin
#elif L==6
	"Il tipo di laurea <strong>%s</strong> gi&agrave; esiste.";
#elif L==7
	"The type of degree <strong>%s</strong> already exists.";	// Potrzebujesz tlumaczenie
#elif L==8
	"The type of degree <strong>%s</strong> already exists.";	// Necessita de traduo
#endif

const char *Txt_The_type_of_degree_X_has_been_renamed_as_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==0
	"El tipo de titulaci&oacute;n <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";	// Necessita traduccio
#elif L==1
	"The type of degree <strong>%s</strong> has been renamed as <strong>%s</strong>.";			// Need bersetzung
#elif L==2
	"The type of degree <strong>%s</strong> has been renamed as <strong>%s</strong>.";
#elif L==3
	"El tipo de titulaci&oacute;n <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";
#elif L==4
	"The type of degree <strong>%s</strong> has been renamed as <strong>%s</strong>.";			// Besoin de traduction
#elif L==5
	"El tipo de titulaci&oacute;n <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";	// Okoteve traduccin
#elif L==6
	"Il tipo di laurea <strong>%s</strong> &egrave; stato rinominato come <strong>%s</strong>.";
#elif L==7
	"The type of degree <strong>%s</strong> has been renamed as <strong>%s</strong>.";			// Potrzebujesz tlumaczenie
#elif L==8
	"The type of degree <strong>%s</strong> has been renamed as <strong>%s</strong>.";			// Necessita de traduo
#endif

const char *Txt_The_type_of_degree_X_no_longer_allows_direct_authentication =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"El tipus de titulaci&oacute; <strong>%s</strong> ja no permet autenticaci&oacute; directa.";
#elif L==1
	"The type of degree <strong>%s</strong> no longer allows direct authentication.";		// Need bersetzung
#elif L==2
	"The type of degree <strong>%s</strong> no longer allows direct authentication.";
#elif L==3
	"El tipo de titulaci&oacute;n <strong>%s</strong> ya no permite autenticaci&oacute;n directa.";
#elif L==4
	"The type of degree <strong>%s</strong> no longer allows direct authentication.";		// Besoin de traduction
#elif L==5
	"El tipo de titulaci&oacute;n <strong>%s</strong> ya no permite autenticaci&oacute;n directa.";	// Okoteve traduccin
#elif L==6
	"Il tipo di laurea <strong>%s</strong> non permette pi&ugrave; ora l'autenticazione diretta.";
#elif L==7
	"The type of degree <strong>%s</strong> no longer allows direct authentication.";		// Potrzebujesz tlumaczenie
#elif L==8
	"The type of degree <strong>%s</strong> no longer allows direct authentication.";		// Necessita de traduo
#endif

const char *Txt_The_type_of_degree_X_now_allows_direct_authentication =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"El tipus de titulaci&oacute; <strong>%s</strong> ara permet autenticaci&oacute; directa.";
#elif L==1
	"The type of degree <strong>%s</strong> now allows direct authentication.";			// Need bersetzung
#elif L==2
	"The type of degree <strong>%s</strong> now allows direct authentication.";
#elif L==3
	"El tipo de titulaci&oacute;n <strong>%s</strong> ahora permite autenticaci&oacute;n directa.";
#elif L==4
	"The type of degree <strong>%s</strong> now allows direct authentication.";			// Besoin de traduction
#elif L==5
	"El tipo de titulaci&oacute;n <strong>%s</strong> ahora permite autenticaci&oacute;n directa.";	// Okoteve traduccin
#elif L==6
	"Il tipo di laurea <strong>%s</strong> permette ora l'autenticazione diretta.";
#elif L==7
	"The type of degree <strong>%s</strong> now allows direct authentication.";			// Potrzebujesz tlumaczenie
#elif L==8
	"The type of degree <strong>%s</strong> now allows direct authentication.";			// Necessita de traduo
#endif

const char *Txt_The_type_of_degree_of_the_degree_X_has_changed =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"El tipo de titulaci&oacute;n de la titulaci&oacute;n <strong>%s</strong> ha cambiado.";	// Necessita traduccio
#elif L==1
	"The type of degree of the degree <strong>%s</strong> has changed.";				// Need bersetzung
#elif L==2
	"The type of degree of the degree <strong>%s</strong> has changed.";
#elif L==3
	"El tipo de titulaci&oacute;n de la titulaci&oacute;n <strong>%s</strong> ha cambiado.";
#elif L==4
	"The type of degree of the degree <strong>%s</strong> has changed.";				// Besoin de traduction
#elif L==5
	"El tipo de titulaci&oacute;n de la titulaci&oacute;n <strong>%s</strong> ha cambiado.";	// Okoteve traduccin
#elif L==6
	"Il tipo di laurea <strong>%s</strong> &egrave; cambiato.";
#elif L==7
	"The type of degree of the degree <strong>%s</strong> has changed.";				// Potrzebujesz tlumaczenie
#elif L==8
	"The type of degree of the degree <strong>%s</strong> has changed.";				// Necessita de traduo
#endif

const char *Txt_The_type_of_group_X_already_exists =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"El tipus de grup <strong>%s</strong> ja existeix.";
#elif L==1
	"Der Gruppen-Typ <strong>%s</strong> wurde bereits definiert.";
#elif L==2
	"The type of group <strong>%s</strong> already exists.";
#elif L==3
	"El tipo de grupo <strong>%s</strong> ya existe.";
#elif L==4
	"The type of group <strong>%s</strong> already exists.";	// Besoin de traduction
#elif L==5
	"El tipo de grupo <strong>%s</strong> ya existe.";		// Okoteve traduccin
#elif L==6
	"Il tipo di gruppo <strong>%s</strong> gi&agrave; esiste.";
#elif L==7
	"The type of group <strong>%s</strong> already exists.";	// Potrzebujesz tlumaczenie
#elif L==8
	"The type of group <strong>%s</strong> already exists.";	// Necessita de traduo
#endif

const char *Txt_The_type_of_group_X_has_been_renamed_as_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==0
	"El tipo de grupo <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";	// Necessita traduccio
#elif L==1
	"The type of group <strong>%s</strong> has been renamed as <strong>%s</strong>.";	// Need bersetzung
#elif L==2
	"The type of group <strong>%s</strong> has been renamed as <strong>%s</strong>.";
#elif L==3
	"El tipo de grupo <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";
#elif L==4
	"The type of group <strong>%s</strong> has been renamed as <strong>%s</strong>.";	// Besoin de traduction
#elif L==5
	"El tipo de grupo <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";	// Okoteve traduccin
#elif L==6
	"Il tipo di gruppo <strong>%s</strong> &egrave; stato rinominato come <strong>%s</strong>.";
#elif L==7
	"The type of group <strong>%s</strong> has been renamed as <strong>%s</strong>.";	// Potrzebujesz tlumaczenie
#elif L==8
	"The type of group <strong>%s</strong> has been renamed as <strong>%s</strong>.";	// Necessita de traduo
#endif

const char *Txt_The_type_of_group_of_the_group_X_has_changed =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"El tipo de grupo del grupo <strong>%s</strong> ha cambiado.";		// Necessita traduccio
#elif L==1
	"The type of group of the group <strong>%s</strong> has changed.";	// Need bersetzung
#elif L==2
	"The type of group of the group <strong>%s</strong> has changed.";
#elif L==3
	"El tipo de grupo del grupo <strong>%s</strong> ha cambiado.";
#elif L==4
	"The type of group of the group <strong>%s</strong> has changed.";	// Besoin de traduction
#elif L==5
	"El tipo de grupo del grupo <strong>%s</strong> ha cambiado.";		// Okoteve traduccin
#elif L==6
	"Il tipo di gruppo del gruppo <strong>%s</strong> &egrave; cambiato.";
#elif L==7
	"The type of group of the group <strong>%s</strong> has changed.";	// Potrzebujesz tlumaczenie
#elif L==8
	"The type of group of the group <strong>%s</strong> has changed.";	// Necessita de traduo
#endif

const char *Txt_The_type_of_enrollment_of_the_type_of_group_X_has_not_changed =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"El tipo de adscripci&oacute;n del tipo de grupo <strong>%s</strong> no ha cambiado.";	// Necessita traduccio
#elif L==1
	"The type of enrollment of the type of group <strong>%s</strong> has not changed.";	// Need bersetzung
#elif L==2
	"The type of enrollment of the type of group <strong>%s</strong> has not changed.";
#elif L==3
	"El tipo de adscripci&oacute;n del tipo de grupo <strong>%s</strong> no ha cambiado.";
#elif L==4
	"The type of enrollment of the type of group <strong>%s</strong> has not changed.";	// Besoin de traduction
#elif L==5
	"El tipo de adscripci&oacute;n del tipo de grupo <strong>%s</strong> no ha cambiado.";	// Okoteve traduccin
#elif L==6
	"Il tipo di registrazione del tipo di gruppo <strong>%s</strong> non &egrave; cambiato.";
#elif L==7
	"The type of enrollment of the type of group <strong>%s</strong> has not changed.";	// Potrzebujesz tlumaczenie
#elif L==8
	"The type of enrollment of the type of group <strong>%s</strong> has not changed.";	// Necessita de traduo
#endif

const char *Txt_The_type_of_the_holiday_X_has_changed =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"El tipo de la festividad <strong>%s</strong> ha cambiado.";	// Necessita traduccio
#elif L==1
	"The type of the holiday <strong>%s</strong> has changed.";	// Need bersetzung
#elif L==2
	"The type of the holiday <strong>%s</strong> has changed.";
#elif L==3
	"El tipo de la festividad <strong>%s</strong> ha cambiado.";
#elif L==4
	"The type of the holiday <strong>%s</strong> has changed.";	// Besoin de traduction
#elif L==5
	"El tipo de la festividad <strong>%s</strong> ha cambiado.";	// Okoteve traduccin
#elif L==6
	"Il tipo di festivit&agrave; <strong>%s</strong> &egrave; cambiato.";
#elif L==7
	"The type of the holiday <strong>%s</strong> has changed.";	// Potrzebujesz tlumaczenie
#elif L==8
	"The type of the holiday <strong>%s</strong> has changed.";	// Necessita de traduo
#endif

const char *Txt_The_URL_X_has_been_updated =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"El URL <strong>%s</strong> se ha actualizado.";	// Necessita traduccio
#elif L==1
	"The URL <strong>%s</strong> has been updated.";	// Need bersetzung
#elif L==2
	"The URL <strong>%s</strong> has been updated.";
#elif L==3
	"El URL <strong>%s</strong> se ha actualizado.";
#elif L==4
	"The URL <strong>%s</strong> has been updated.";	// Besoin de traduction
#elif L==5
	"El URL <strong>%s</strong> se ha actualizado.";	// Okoteve traduccin
#elif L==6
	"L'URL <strong>%s</strong> &egrave; stata archiviata.";
#elif L==7
	"The URL <strong>%s</strong> has been updated.";	// Potrzebujesz tlumaczenie
#elif L==8
	"The URL <strong>%s</strong> has been updated.";	// Necessita de traduo
#endif

const char *Txt_The_URL_web_address_that_gives_direct_access_to_the_course_X_is =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"El URL (direcci&oacute;n web) que permite acceder directamente"
	" a la asignatura <strong>%s</strong> es:";		// Necessita traduccio
#elif L==1
	"The URL (web address) that gives direct access"
	" to the course <strong>%s</strong> is:";		// Need bersetzung
#elif L==2
	"The URL (web address) that gives direct access"
	" to the course <strong>%s</strong> is:";
#elif L==3
	"El URL (direcci&oacute;n web) que permite acceder directamente"
	" a la asignatura <strong>%s</strong> es:";
#elif L==4
	"The URL (web address) that gives direct access"
	" to the course <strong>%s</strong> is:";		// Besoin de traduction
#elif L==5
	"El URL (direcci&oacute;n web) que permite acceder directamente"
	" a la asignatura <strong>%s</strong> es:";		// Okoteve traduccin
#elif L==6
	"L'URL (indirizzo web) che permette l'accesso diretto"
	" al corso <strong>%s</strong> &egrave;:";
#elif L==7
	"The URL (web address) that gives direct access"
	" to the course <strong>%s</strong> is:";		// Potrzebujesz tlumaczenie
#elif L==8
	"The URL (web address) that gives direct access"
	" to the course <strong>%s</strong> is:";		// Necessita de traduo
#endif

const char *Txt_THE_USER_X_already_exists_in_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==0
	"<strong>%s</strong> ya existe en %s.";		// Necessita traduccio
#elif L==1
	"<strong>%s</strong> already exists in %s.";	// bersetzung notwendig
#elif L==2
	"<strong>%s</strong> already exists in %s.";
#elif L==3
	"<strong>%s</strong> ya existe en %s.";
#elif L==4
	"<strong>%s</strong> already exists in %s.";	// Besoin de traduction
#elif L==5
	"<strong>%s</strong> ya existe en %s.";		// Okoteve traduccin
#elif L==6
	"<strong>%s</strong> esiste gi&agrave; in %s.";
#elif L==7
	"<strong>%s</strong> already exists in %s.";	// Potrzebujesz tlumaczenie
#elif L==8
	"<strong>%s</strong> already exists in %s.";	// Necessita de traduo
#endif

const char *Txt_THE_USER_X_already_exists_in_Y_but_is_not_yet_enrolled_in_the_course_Z =	// Warning: it is very important to include three %s in the following sentences
#if   L==0
	"<strong>%s</strong> ya existe en %s,"
	" pero a&uacute;n no est&aacute; inscrito"
	" en la asignatura <strong>%s</strong>.";	// Necessita traduccio
#elif L==1
	"<strong>%s</strong> already exists in %s,"
	" but is not yet enrolled"
	" in the course <strong>%s</strong>.";		// Need bersetzung
#elif L==2
	"<strong>%s</strong> already exists in %s,"
	" but is not yet enrolled"
	" in the course <strong>%s</strong>.";
#elif L==3
	"<strong>%s</strong> ya existe en %s,"
	" pero a&uacute;n no est&aacute; inscrito"
	" en la asignatura <strong>%s</strong>.";
#elif L==4
	"<strong>%s</strong> already exists in %s,"
	" but is not yet enrolled"
	" in the course <strong>%s</strong>.";		// Besoin de traduction
#elif L==5
	"<strong>%s</strong> ya existe en %s,"
	" pero a&uacute;n no est&aacute; inscrito"
	" en la asignatura <strong>%s</strong>.";	// Okoteve traduccin
#elif L==6
	"<strong>%s</strong> esiste gi&agrave; in %s,"
	" ma non &egrave; ancora registrato"
	" al corso <strong>%s</strong>.";
#elif L==7
	"<strong>%s</strong> already exists in %s,"
	" but is not yet enrolled"
	" in the course <strong>%s</strong>.";		// Potrzebujesz tlumaczenie
#elif L==8
	"<strong>%s</strong> already exists in %s,"
	" but is not yet enrolled"
	" in the course <strong>%s</strong>.";		// Necessita de traduo
#endif

const char *Txt_THE_USER_X_has_been_enrolled_as_administrator_of_the_degree_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==0
	"<strong>%s</strong> ha sido a&ntilde;adido/a como administrador/a"
	" de la titulaci&oacute;n <strong>%s</strong>.";	// Necessita traduccio
#elif L==1
	"<strong>%s</strong> has been enrolled as administrator"
	" of the degree <strong>%s</strong>.";			// bersetzung notwendig
#elif L==2
	"<strong>%s</strong> has been enrolled as administrator"
	" of the degree <strong>%s</strong>.";
#elif L==3
	"<strong>%s</strong> ha sido a&ntilde;adido/a como administrador/a"
	" de la titulaci&oacute;n <strong>%s</strong>.";
#elif L==4
	"<strong>%s</strong> has been enrolled as administrator"
	" of the degree <strong>%s</strong>.";			// Besoin de traduction
#elif L==5
	"<strong>%s</strong> ha sido a&ntilde;adido/a como administrador/a"
	" de la titulaci&oacute;n <strong>%s</strong>.";	// Okoteve traduccin
#elif L==6
	"<strong>%s</strong> &egrave; stato registrato come amministratore"
	" della laurea <strong>%s</strong>.";
#elif L==7
	"<strong>%s</strong> has been enrolled as administrator"
	" of the degree <strong>%s</strong>.";			// Potrzebujesz tlumaczenie
#elif L==8
	"<strong>%s</strong> has been enrolled as administrator"
	" of the degree <strong>%s</strong>.";			// Necessita de traduo
#endif

const char *Txt_THE_USER_X_has_been_enrolled_in_the_course_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==0
	"<strong>%s</strong> ha estat afegit/da"
	" a l'assignatura <strong>%s</strong>.";
#elif L==1
	"<strong>%s</strong> hat sich im"
	" im Kurs <strong>%s</strong> eingeschrieben.";
#elif L==2
	"<strong>%s</strong> has been enrolled"
	" in the course <strong>%s</strong>.";
#elif L==3
	"<strong>%s</strong> ha sido a&ntilde;adido/a"
	" a la asignatura <strong>%s</strong>.";
#elif L==4
	"<strong>%s</strong> a &eacute;t&eacute; inscrit"
	" dans la mati&egrave;re <strong>%s</strong>.";
#elif L==5
	"<strong>%s</strong> ha sido a&ntilde;adido/a"
	" a la asignatura <strong>%s</strong>.";	// Okoteve traduccin
#elif L==6
	"<strong>%s</strong> &egrave; stato registrato/a"
	" nel corso <strong>%s</strong>.";
#elif L==7
	"<strong>%s</strong> zosta&lstrok; zapisany"
	"  do kursu <strong>%s</strong>.";
#elif L==8
	"<strong>%s</strong> foi matriculado/a"
	" na disciplina <strong>%s</strong>.";
#endif

const char *Txt_THE_USER_X_has_been_enrolled_in_the_group_of_type_Y_Z =	// Warning: it is very important to include three %s in the following sentences
#if   L==0
	"<strong>%s</strong> ha sido a&ntilde;adido/a"
	" al grupo de tipo %s <strong>%s</strong>.";		// Necessita traduccio
#elif L==1
	"<strong>%s</strong> has been enrolled"
	" in the group of type %s <strong>%s</strong>.";	// bersetzung notwendig
#elif L==2
	"<strong>%s</strong> has been enrolled"
	" in the group of type %s <strong>%s</strong>.";
#elif L==3
	"<strong>%s</strong> ha sido a&ntilde;adido/a"
	" al grupo de tipo %s <strong>%s</strong>.";
#elif L==4
	"<strong>%s</strong> has been enrolled"
	" in the group of type %s <strong>%s</strong>.";	// Besoin de traduction
#elif L==5
	"<strong>%s</strong> ha sido a&ntilde;adido/a"
	" al grupo de tipo %s <strong>%s</strong>.";		// Okoteve traduccin
#elif L==6
	"<strong>%s</strong> &egrave; stato registrato"
	" nel gruppo di tipo %s <strong>%s</strong>.";
#elif L==7
	"<strong>%s</strong> has been enrolled"
	" in the group of type %s <strong>%s</strong>.";	// Potrzebujesz tlumaczenie
#elif L==8
	"<strong>%s</strong> has been enrolled"
	" in the group of type %s <strong>%s</strong>.";	// Necessita de traduo
#endif

const char *Txt_THE_USER_X_has_been_removed_as_administrator =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"<strong>%s</strong> ha sido eliminado/a como administrador/a.";	// Necessita traduccio
#elif L==1
	"<strong>%s</strong> has been removed as administrator.";	// bersetzung notwendig
#elif L==2
	"<strong>%s</strong> has been removed as administrator.";
#elif L==3
	"<strong>%s</strong> ha sido eliminado/a como administrador/a.";
#elif L==4
	"<strong>%s</strong> has been removed as administrator.";	// Besoin de traduction
#elif L==5
	"<strong>%s</strong> ha sido eliminado/a como administrador/a.";	// Okoteve traduccin
#elif L==6
	"<strong>%s</strong> &egrave; stato rimosso come amministratore.";
#elif L==7
	"<strong>%s</strong> has been removed as administrator.";	// Potrzebujesz tlumaczenie
#elif L==8
	"<strong>%s</strong> has been removed as administrator.";	// Necessita de traduo
#endif

const char *Txt_THE_USER_X_has_been_removed_from_all_groups_in_all_courses =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"<strong>%s</strong> ha sido eliminado/a"
	" de todos los grupos de todas sus asignaturas.";	// Necessita traduccio
#elif L==1
	"<strong>%s</strong> has been removed"
	" from all groups in all courses.";			// bersetzung notwendig
#elif L==2
	"<strong>%s</strong> has been removed"
	" from all groups in all courses.";
#elif L==3
	"<strong>%s</strong> ha sido eliminado/a"
	" de todos los grupos de todas sus asignaturas.";
#elif L==4
	"<strong>%s</strong> has been removed"
	" from all groups in all courses.";			// Besoin de traduction
#elif L==5
	"<strong>%s</strong> ha sido eliminado/a"
	" de todos los grupos de todas sus asignaturas.";	// Okoteve traduccin
#elif L==6
	"<strong>%s</strong> &egrave; stato rimosso"
	" da tutti i gruppi di tutti i corsi.";
#elif L==7
	"<strong>%s</strong> has been removed"
	" from all groups in all courses.";			// Potrzebujesz tlumaczenie
#elif L==8
	"<strong>%s</strong> has been removed"
	" from all groups in all courses.";			// Necessita de traduo
#endif

const char *Txt_THE_USER_X_has_been_removed_from_all_groups_of_the_course_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==0
	"<strong>%s</strong> ha sido eliminado/a"
	" de todos los grupos de la asignatura <strong>%s</strong>.";	// Necessita traduccio
#elif L==1
	"<strong>%s</strong> has been removed"
	" from all groups of the course <strong>%s</strong>.";		// bersetzung notwendig
#elif L==2
	"<strong>%s</strong> has been removed"
	" from all groups of the course <strong>%s</strong>.";
#elif L==3
	"<strong>%s</strong> ha sido eliminado/a"
	" de todos los grupos de la asignatura <strong>%s</strong>.";
#elif L==4
	"<strong>%s</strong> has been removed"
	" from all groups of the course <strong>%s</strong>.";		// Besoin de traduction
#elif L==5
	"<strong>%s</strong> ha sido eliminado/a"
	" de todos los grupos de la asignatura <strong>%s</strong>.";	// Okoteve traduccin
#elif L==6
	"<strong>%s</strong> &egrave; stato rimosso"
	" da tutti i gruppi del corso <strong>%s</strong>.";
#elif L==7
	"<strong>%s</strong> has been removed"
	" from all groups of the course <strong>%s</strong>.";		// Potrzebujesz tlumaczenie
#elif L==8
	"<strong>%s</strong> has been removed"
	" from all groups of the course <strong>%s</strong>.";		// Necessita de traduo
#endif

const char *Txt_THE_USER_X_has_been_removed_from_all_his_her_courses =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"<strong>%s</strong> ha sido eliminado/a de todas sus asignaturas.";	// Necessita traduccio
#elif L==1
	"<strong>%s</strong> has been removed from all her/his courses.";	// bersetzung notwendig
#elif L==2
	"<strong>%s</strong> has been removed from all her/his courses.";
#elif L==3
	"<strong>%s</strong> ha sido eliminado/a de todas sus asignaturas.";
#elif L==4
	"<strong>%s</strong> has been removed from all her/his courses.";	// Besoin de traduction
#elif L==5
	"<strong>%s</strong> ha sido eliminado/a de todas sus asignaturas.";	// Okoteve traduccin
#elif L==6
	"<strong>%s</strong> &egrave; stato rimosso da tutti i suoi corsi.";
#elif L==7
	"<strong>%s</strong> has been removed from all her/his courses.";	// Potrzebujesz tlumaczenie
#elif L==8
	"<strong>%s</strong> has been removed from all her/his courses.";	// Necessita de traduo
#endif

const char *Txt_THE_USER_X_has_been_removed_from_Y_groups =	// Warning: it is very important to include %s and %u in the following sentences
#if   L==0
	"<strong>%s</strong> ha sido eliminado/a de %u grupos.";	// Necessita traduccio
#elif L==1
	"<strong>%s</strong> has been removed from %u groups.";		// bersetzung notwendig
#elif L==2
	"<strong>%s</strong> has been removed from %u groups.";
#elif L==3
	"<strong>%s</strong> ha sido eliminado/a de %u grupos.";
#elif L==4
	"<strong>%s</strong> has been removed from %u groups.";		// Besoin de traduction
#elif L==5
	"<strong>%s</strong> ha sido eliminado/a de %u grupos.";	// Okoteve traduccin
#elif L==6
	"<strong>%s</strong> &egrave; stato rimosso da %u gruppi.";
#elif L==7
	"<strong>%s</strong> has been removed from %u groups.";		// Potrzebujesz tlumaczenie
#elif L==8
	"<strong>%s</strong> has been removed from %u groups.";		// Necessita de traduo
#endif

const char *Txt_THE_USER_X_has_been_removed_from_one_group =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"<strong>%s</strong> ha sido eliminado/a de un grupo.";	// Necessita traduccio
#elif L==1
	"<strong>%s</strong> has been removed from one group.";	// bersetzung notwendig
#elif L==2
	"<strong>%s</strong> has been removed from one group.";
#elif L==3
	"<strong>%s</strong> ha sido eliminado/a de un grupo.";
#elif L==4
	"<strong>%s</strong> has been removed from one group.";	// Besoin de traduction
#elif L==5
	"<strong>%s</strong> ha sido eliminado/a de un grupo.";	// Okoteve traduccin
#elif L==6
	"<strong>%s</strong> &egrave; stato rimosso da un gruppo.";
#elif L==7
	"<strong>%s</strong> has been removed from one group.";	// Potrzebujesz tlumaczenie
#elif L==8
	"<strong>%s</strong> has been removed from one group.";	// Necessita de traduo
#endif

const char *Txt_THE_USER_X_has_been_removed_from_the_course_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==0
	"<strong>%s</strong> ha sido eliminado/a"
	" de la asignatura <strong>%s</strong>.";	// Necessita traduccio
#elif L==1
	"<strong>%s</strong> has been removed"
	" from the course <strong>%s</strong>.";	// bersetzung notwendig
#elif L==2
	"<strong>%s</strong> has been removed"
	" from the course <strong>%s</strong>.";
#elif L==3
	"<strong>%s</strong> ha sido eliminado/a"
	" de la asignatura <strong>%s</strong>.";
#elif L==4
	"<strong>%s</strong> has been removed"
	" from the course <strong>%s</strong>.";	// Besoin de traduction
#elif L==5
	"<strong>%s</strong> ha sido eliminado/a"
	" de la asignatura <strong>%s</strong>.";	// Okoteve traduccin
#elif L==6
	"<strong>%s</strong> &egrave; stato rimosso"
	" dal corso <strong>%s</strong>.";
#elif L==7
	"<strong>%s</strong> has been removed"
	" from the course <strong>%s</strong>.";	// Potrzebujesz tlumaczenie
#elif L==8
	"<strong>%s</strong> has been removed"
	" from the course <strong>%s</strong>.";	// Necessita de traduo
#endif

const char *Txt_THE_USER_X_has_been_removed_as_administrator_of_the_degree_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==0
	"<strong>%s</strong> ha sido eliminado/a como administrador"
	" de la titulaci&oacute;n <strong>%s</strong>.";	// Necessita traduccio
#elif L==1
	"<strong>%s</strong> has been removed as administrator"
	" of the degree <strong>%s</strong>.";			// bersetzung notwendig
#elif L==2
	"<strong>%s</strong> has been removed as administrator"
	" of the degree <strong>%s</strong>.";
#elif L==3
	"<strong>%s</strong> ha sido eliminado/a como administrador"
	" de la titulaci&oacute;n <strong>%s</strong>.";
#elif L==4
	"<strong>%s</strong> has been removed as administrator"
	" of the degree <strong>%s</strong>.";			// Besoin de traduction
#elif L==5
	"<strong>%s</strong> ha sido eliminado/a como administrador"
	" de la titulaci&oacute;n <strong>%s</strong>.";	// Okoteve traduccin
#elif L==6
	"<strong>%s</strong> &egrave; stato rimosso come amministratore"
	" della laurea <strong>%s</strong>.";
#elif L==7
	"<strong>%s</strong> has been removed as administrator"
	" of the degree <strong>%s</strong>.";			// Potrzebujesz tlumaczenie
#elif L==8
	"<strong>%s</strong> has been removed as administrator"
	" of the degree <strong>%s</strong>.";			// Necessita de traduo
#endif

const char *Txt_THE_USER_X_has_been_removed_from_the_group_of_type_Y_to_which_it_belonged =	// Warning: it is very important to include two %s in the following sentences
#if   L==0
	"<strong>%s</strong> ha sido eliminado/a"
	" del grupo de tipo %s al que pertenec&iacute;a.";	// Necessita traduccio
#elif L==1
	"<strong>%s</strong> has been removed"
	" from the group of type %s to which it belonged.";	// bersetzung notwendig
#elif L==2
	"<strong>%s</strong> has been removed"
	" from the group of type %s to which it belonged.";
#elif L==3
	"<strong>%s</strong> ha sido eliminado/a"
	" del grupo de tipo %s al que pertenec&iacute;a.";
#elif L==4
	"<strong>%s</strong> has been removed"
	" from the group of type %s to which it belonged.";	// Besoin de traduction
#elif L==5
	"<strong>%s</strong> ha sido eliminado/a"
	" del grupo de tipo %s al que pertenec&iacute;a.";	// Okoteve traduccin
#elif L==6
	"<strong>%s</strong> &egrave; stato rimosso"
	" dal gruppo di tipo %s al quale apparteneva.";
#elif L==7
	"<strong>%s</strong> has been removed"
	" from the group of type %s to which it belonged.";	// Potrzebujesz tlumaczenie
#elif L==8
	"<strong>%s</strong> has been removed"
	" from the group of type %s to which it belonged.";	// Necessita de traduo
#endif

const char *Txt_THE_USER_X_has_not_been_removed_from_any_group =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"<strong>%s</strong> no ha sido eliminado/a"
	" de ning&uacute;n grupo.";	// Necessita traduccio
#elif L==1
	"<strong>%s</strong> has not been removed"
	" from any group.";		// bersetzung notwendig
#elif L==2
	"<strong>%s</strong> has not been removed"
	" from any group.";
#elif L==3
	"<strong>%s</strong> no ha sido eliminado/a"
	" de ning&uacute;n grupo.";
#elif L==4
	"<strong>%s</strong> has not been removed"
	" from any group.";		// Besoin de traduction
#elif L==5
	"<strong>%s</strong> no ha sido eliminado/a"
	" de ning&uacute;n grupo.";	// Okoteve traduccin
#elif L==6
	"<strong>%s</strong> non &egrave; stato rimosso"
	" da alcun gruppo.";
#elif L==7
	"<strong>%s</strong> has not been removed"
	" from any group.";		// Potrzebujesz tlumaczenie
#elif L==8
	"<strong>%s</strong> has not been removed"
	" from any group.";		// Necessita de traduo
#endif

const char *Txt_THE_USER_X_is_already_an_administrator_of_the_degree_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==0
	"<strong>%s</strong> ya es administrador/a"
	" de la titulaci&oacute;n <strong>%s</strong>.";	// Necessita traduccio
#elif L==1
	"<strong>%s</strong> is already an administrator"
	" of the degree <strong>%s</strong>.";			// bersetzung notwendig
#elif L==2
	"<strong>%s</strong> is already an administrator"
	" of the degree <strong>%s</strong>.";
#elif L==3
	"<strong>%s</strong> ya es administrador/a"
	" de la titulaci&oacute;n <strong>%s</strong>.";
#elif L==4
	"<strong>%s</strong> is already an administrator"
	" of the degree <strong>%s</strong>.";			// Besoin de traduction
#elif L==5
	"<strong>%s</strong> ya es administrador/a"
	" de la titulaci&oacute;n <strong>%s</strong>.";	// Okoteve traduccin
#elif L==6
	"<strong>%s</strong> &egrave; gi&agrave; un amministratore"
	" della laurea <strong>%s</strong>.";
#elif L==7
	"<strong>%s</strong> is already an administrator"
	" of the degree <strong>%s</strong>.";			// Potrzebujesz tlumaczenie
#elif L==8
	"<strong>%s</strong> is already an administrator"
	" of the degree <strong>%s</strong>.";			// Necessita de traduo
#endif

const char *Txt_THE_USER_X_is_already_in_the_course_Y_but_has_not_yet_accepted_the_enrollment =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"<strong>%s</strong> ya est&aacute;"
	" en la asignatura <strong>%s</strong>,"
	" pero a&uacute;n no ha aceptado la inscripci&oacute;n.";	// Necessita traduccio
#elif L==1
	"<strong>%s</strong> is already"
	" in the course <strong>%s</strong>,"
	" but has not yet accepted the enrollment.";			// bersetzung notwendig
#elif L==2
	"<strong>%s</strong> is already"
	" in the course <strong>%s</strong>,"
	" but has not yet accepted the enrollment.";
#elif L==3
	"<strong>%s</strong> ya est&aacute;"
	" en la asignatura <strong>%s</strong>,"
	" pero a&uacute;n no ha aceptado la inscripci&oacute;n.";
#elif L==4
	"<strong>%s</strong> is already"
	" in the course <strong>%s</strong>,"
	" but has not yet accepted the enrollment.";			// Besoin de traduction
#elif L==5
	"<strong>%s</strong> ya est&aacute;"
	" en la asignatura <strong>%s</strong>,"
	" pero a&uacute;n no ha aceptado la inscripci&oacute;n.";	// Okoteve traduccin
#elif L==6
	"<strong>%s</strong> &egrave; gi&agrave;"
	" nel corso <strong>%s</strong>,"
	" ma non ha ancora accettato la registrazione.";
#elif L==7
	"<strong>%s</strong> is already"
	" in the course <strong>%s</strong>,"
	" but has not yet accepted the enrollment.";			// Potrzebujesz tlumaczenie
#elif L==8
	"<strong>%s</strong> is already"
	" in the course <strong>%s</strong>,"
	" but has not yet accepted the enrollment.";			// Necessita de traduo
#endif

const char *Txt_THE_USER_X_is_already_enrolled_in_the_course_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==0
	"<strong>%s</strong> ya est&aacute; inscrito/a"
	" en la asignatura <strong>%s</strong>.";	// Necessita traduccio
#elif L==1
	"<strong>%s</strong> is already enrolled"
	" in the course <strong>%s</strong>.";		// bersetzung notwendig
#elif L==2
	"<strong>%s</strong> is already enrolled"
	" in the course <strong>%s</strong>.";
#elif L==3
	"<strong>%s</strong> ya est&aacute; inscrito/a"
	" en la asignatura <strong>%s</strong>.";
#elif L==4
	"<strong>%s</strong> is already enrolled"
	" in the course <strong>%s</strong>.";		// Besoin de traduction
#elif L==5
	"<strong>%s</strong> ya est&aacute; inscrito/a"
	" en la asignatura <strong>%s</strong>.";	// Okoteve traduccin
#elif L==6
	"<strong>%s</strong> &egrave; gi&agrave; registrato"
	" nel corso <strong>%s</strong>.";
#elif L==7
	"<strong>%s</strong> is already enrolled"
	" in the course <strong>%s</strong>.";		// Potrzebujesz tlumaczenie
#elif L==8
	"<strong>%s</strong> is already enrolled"
	" in the course <strong>%s</strong>.";		// Necessita de traduo
#endif

const char *Txt_The_user_is_new_does_not_exists_yet_in_X =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"El usuario/a es nuevo/a, a&uacute;n no existe en %s.";	// Necessita traduccio
#elif L==1
	"The user is new, does not exist yet in %s.";		// bersetzung notwendig
#elif L==2
	"The user is new, does not exist yet in %s.";
#elif L==3
	"El usuario/a es nuevo/a, a&uacute;n no existe en %s.";
#elif L==4
	"The user is new, does not exist yet in %s.";		// Besoin de traduction
#elif L==5
	"El usuario/a es nuevo/a, a&uacute;n no existe en %s.";	// Okoteve traduccin
#elif L==6
	"L'utente &egrave; nuovo, non esiste ancora in %s.";
#elif L==7
	"The user is new, does not exist yet in %s.";		// Potrzebujesz tlumaczenie
#elif L==8
	"The user is new, does not exist yet in %s.";		// Necessita de traduo
#endif

const char *Txt_THE_USER_X_is_not_an_administrator_of_the_degree_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==0
	"<strong>%s</strong> no es administrador/a"
	" de la titulaci&oacute;n <strong>%s</strong>.";	// Necessita traduccio
#elif L==1
	"<strong>%s</strong> is not an administrator"
	" of the degree <strong>%s</strong>.";			// bersetzung notwendig
#elif L==2
	"<strong>%s</strong> is not an administrator"
	" of the degree <strong>%s</strong>.";
#elif L==3
	"<strong>%s</strong> no es administrador/a"
	" de la titulaci&oacute;n <strong>%s</strong>.";
#elif L==4
	"<strong>%s</strong> is not an administrator"
	" of the degree <strong>%s</strong>.";			// Besoin de traduction
#elif L==5
	"<strong>%s</strong> no es administrador/a"
	" de la titulaci&oacute;n <strong>%s</strong>.";	// Okoteve traduccin
#elif L==6
	"<strong>%s</strong> non &egrave; un amministratore"
	" della laurea <strong>%s</strong>.";
#elif L==7
	"<strong>%s</strong> is not an administrator"
	" of the degree <strong>%s</strong>.";			// Potrzebujesz tlumaczenie
#elif L==8
	"<strong>%s</strong> is not an administrator"
	" of the degree <strong>%s</strong>.";			// Necessita de traduo
#endif

const char *Txt_THE_USER_X_is_not_found_in_the_file_of_marks =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"No se ha encontrado a <strong>%s</strong> en el archivo de calificaciones.";	// Necessita traduccio
#elif L==1
	"<strong>%s</strong> is not found in the file of marks.";			// Need bersetzung
#elif L==2
	"<strong>%s</strong> is not found in the file of marks.";
#elif L==3
	"No se ha encontrado a <strong>%s</strong> en el archivo de calificaciones.";
#elif L==4
	"<strong>%s</strong> is not found in the file of marks.";			// Besoin de traduction
#elif L==5
	"No se ha encontrado a <strong>%s</strong> en el archivo de calificaciones.";	// Okoteve traduccin
#elif L==6
	"<strong>%s</strong> non &egrave; stato trovato nel file dei risultati.";
#elif L==7
	"<strong>%s</strong> is not found in the file of marks.";			// Potrzebujesz tlumaczenie
#elif L==8
	"<strong>%s</strong> is not found in the file of marks.";			// Necessita de traduo
#endif

const char *Txt_The_visibility_of_the_record_field_X_has_not_changed =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"La visibilidad del campo <strong>%s</strong> no ha cambiado.";			// Necessita traduccio
#elif L==1
	"The visibility of the record field <strong>%s</strong> has not changed.";	// Need bersetzung
#elif L==2
	"The visibility of the record field <strong>%s</strong> has not changed.";
#elif L==3
	"La visibilidad del campo <strong>%s</strong> no ha cambiado.";
#elif L==4
	"The visibility of the record field <strong>%s</strong> has not changed.";	// Besoin de traduction
#elif L==5
	"La visibilidad del campo <strong>%s</strong> no ha cambiado.";			// Okoteve traduccin
#elif L==6
	"La visibilit&agrave; del campo di scheda <strong>%s</strong> non &egrave; cambiata.";
#elif L==7
	"The visibility of the record field <strong>%s</strong> has not changed.";	// Potrzebujesz tlumaczenie
#elif L==8
	"The visibility of the record field <strong>%s</strong> has not changed.";	// Necessita de traduo
#endif

const char *Txt_The_whole_course =
#if   L==0
	"Toda la asignatura";		// Necessita traduccio
#elif L==1
	"Die gesamte Kurs";
#elif L==2
	"The whole course";
#elif L==3
	"Toda la asignatura";
#elif L==4
	"L'ensemble de la mati&egrave;re";
#elif L==5
	"Toda la asignatura";		// Okoteve traduccin
#elif L==6
	"L'intero corso";
#elif L==7
	"The whole course";		// Potrzebujesz tlumaczenie
#elif L==8
	"Toda a disciplina";
#endif

const char *Txt_The_works_of_X_have_been_removed_in_a_total_of_Y_of_his_her_Z_courses =	// Warning: it is very important to include all %s and %u, in the correct order, in the following sentences
#if   L==0
	"Se han eliminado todos los trabajos"
	" de <strong>%s</strong>"
	" en un total de %u de sus %u asignaturas.";			// Necessita traduccio
#elif L==1
	"The works of <strong>%s</strong>"
	" have been removed in a total of %u of her/his %u courses.";	// Need bersetzung
#elif L==2
	"The works of <strong>%s</strong>"
	" have been removed in a total of %u of her/his %u courses.";
#elif L==3
	"Se han eliminado todos los trabajos"
	" de <strong>%s</strong>"
	" en un total de %u de sus %u asignaturas.";
#elif L==4
	"The works of <strong>%s</strong>"
	" have been removed in a total of %u of her/his %u courses.";	// Besoin de traduction
#elif L==5
	"Se han eliminado todos los trabajos"
	" de <strong>%s</strong>"
	" en un total de %u de sus %u asignaturas.";			// Okoteve traduccin
#elif L==6
	"I lavori de <strong>%s</strong>"
	" sono stati rimossi in un totale di %u dei suoi corsi.";
#elif L==7
	"The works of <strong>%s</strong>"
	" have been removed in a total of %u of her/his %u courses.";	// Potrzebujesz tlumaczenie
#elif L==8
	"The works of <strong>%s</strong>"
	" have been removed in a total of %u of her/his %u courses.";	// Necessita de traduo
#endif

const char *Txt_The_year_X_is_not_enabled_on_the_degree_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==0
	"El curso <strong>%s</strong> no est&aacute; habilitado"
	" en la titulaci&oacute;n <strong>%s</strong>.";	// Necessita traduccio
#elif L==1
	"The year <strong>%s</strong> is not enabled"
	" on the degree <strong>%s</strong>.";			// Need bersetzung
#elif L==2
	"The year <strong>%s</strong> is not enabled"
	" on the degree <strong>%s</strong>.";
#elif L==3
	"El curso <strong>%s</strong> no est&aacute; habilitado"
	" en la titulaci&oacute;n <strong>%s</strong>.";
#elif L==4
	"The year <strong>%s</strong> is not enabled"
	" on the degree <strong>%s</strong>.";			// Besoin de traduction
#elif L==5
	"El curso <strong>%s</strong> no est&aacute; habilitado"
	" en la titulaci&oacute;n <strong>%s</strong>.";	// Okoteve traduccin
#elif L==6
	"L'anno <strong>%s</strong> non &egrave; abilitato"
	" nella laurea <strong>%s</strong>.";
#elif L==7
	"The year <strong>%s</strong> is not enabled"
	" on the degree <strong>%s</strong>.";			// Potrzebujesz tlumaczenie
#elif L==8
	"The year <strong>%s</strong> is not enabled"
	" on the degree <strong>%s</strong>.";			// Necessita de traduo
#endif

const char *Txt_The_year_of_the_course_X_has_changed =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"El curso de la asignatura <strong>%s</strong> ha cambiado.";	// Necessita traduccio
#elif L==1
	"The year of the couse <strong>%s</strong> has changed.";	// Need bersetzung
#elif L==2
	"The year of the couse <strong>%s</strong> has changed.";
#elif L==3
	"El curso de la asignatura <strong>%s</strong> ha cambiado.";
#elif L==4
	"The year of the couse <strong>%s</strong> has changed.";	// Besoin de traduction
#elif L==5
	"El curso de la asignatura <strong>%s</strong> ha cambiado.";	// Okoteve traduccin
#elif L==6
	"L'anno del corso <strong>%s</strong> &egrave; cambiato.";
#elif L==7
	"The year of the couse <strong>%s</strong> has changed.";	// Potrzebujesz tlumaczenie
#elif L==8
	"The year of the couse <strong>%s</strong> has changed.";	// Necessita de traduo
#endif

const char *Txt_The_years_of_the_degree_have_changed =
#if   L==0
	"Los a&ntilde;os de la titulaci&oacute;n han cambiado.";	// Necessita traduccio
#elif L==1
	"The years of the degree have changed.";			// Need bersetzung
#elif L==2
	"The years of the degree have changed.";
#elif L==3
	"Los a&ntilde;os de la titulaci&oacute;n han cambiado.";
#elif L==4
	"The years of the degree have changed.";			// Besoin de traduction
#elif L==5
	"Los a&ntilde;os de la titulaci&oacute;n han cambiado.";	// Okoteve traduccin
#elif L==6
	"Gli anni della laurea sono cambiati.";
#elif L==7
	"The years of the degree have changed.";			// Potrzebujesz tlumaczenie
#elif L==8
	"The years of the degree have changed.";			// Necessita de traduo
#endif

const char *Txt_The_ZIP_file_has_been_received_successfully =
#if   L==0
	"El archivo <em>ZIP</em> se ha recibido correctamente.";	// Necessita traduccio
#elif L==1
	"The <em>ZIP</em> file has been received successfully.";	// Need bersetzung
#elif L==2
	"The <em>ZIP</em> file has been received successfully.";
#elif L==3
	"El archivo <em>ZIP</em> se ha recibido correctamente.";
#elif L==4
	"The <em>ZIP</em> file has been received successfully.";	// Besoin de traduction
#elif L==5
	"El archivo <em>ZIP</em> se ha recibido correctamente.";	// Okoteve traduccin
#elif L==6
	"Il <em>ZIP</em> file &egrave; stato ricevuto con successo.";
#elif L==7
	"The <em>ZIP</em> file has been received successfully.";	// Potrzebujesz tlumaczenie
#elif L==8
	"The <em>ZIP</em> file has been received successfully.";	// Necessita de traduo
#endif

const char *Txt_The_ZIP_file_has_been_unzipped_successfully =
#if   L==0
	"El archivo <em>ZIP</em> se ha descomprimido correctamente.";	// Necessita traduccio
#elif L==1
	"The <em>ZIP</em> file has been unzipped successfully.";	// Need bersetzung
#elif L==2
	"The <em>ZIP</em> file has been unzipped successfully.";
#elif L==3
	"El archivo <em>ZIP</em> se ha descomprimido correctamente.";
#elif L==4
	"The <em>ZIP</em> file has been unzipped successfully.";	// Besoin de traduction
#elif L==5
	"El archivo <em>ZIP</em> se ha descomprimido correctamente.";	// Okoteve traduccin
#elif L==6
	"Il <em>ZIP</em> file &egrave; stato decompresso con successo.";
#elif L==7
	"The <em>ZIP</em> file has been unzipped successfully.";	// Potrzebujesz tlumaczenie
#elif L==8
	"The <em>ZIP</em> file has been unzipped successfully.";	// Necessita de traduo
#endif

const char *Txt_Theme_SKIN =
#if   L==0
	"Tema";
#elif L==1
	"Skin";
#elif L==2
	"Theme";
#elif L==3
	"Tema";
#elif L==4
	"Th&egrave;me";
#elif L==5
	"Tema";	// Okoteve traduccin
#elif L==6
	"Tema";
#elif L==7
	"Temat";
#elif L==8
	"Skin";
#endif

const char *Txt_There_are_more_than_one_table_in_the_file_of_marks =
#if   L==0
	"Hay m&aacute;s de una tabla en el archivo de calificaciones.";	// Necessita traduccio
#elif L==1
	"There are more than one table in the file of marks.";		// Need bersetzung
#elif L==2
	"There are more than one table in the file of marks.";
#elif L==3
	"Hay m&aacute;s de una tabla en el archivo de calificaciones.";
#elif L==4
	"There are more than one table in the file of marks.";		// Besoin de traduction
#elif L==5
	"Hay m&aacute;s de una tabla en el archivo de calificaciones.";	// Okoteve traduccin
#elif L==6
	"C'&egrave; pi&ugrave; di una tabella nel file dei risultati.";
#elif L==7
	"There are more than one table in the file of marks.";		// Potrzebujesz tlumaczenie
#elif L==8
	"There are more than one table in the file of marks.";		// Necessita de traduo
#endif

const char *Txt_There_are_more_than_one_user_with_the_ID_X_Please_type_a_nick_or_email =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"Hay m&aacute;s de un usuario con el ID <strong>%s</strong>."
	" Por favor, escriba un @apodo o correo.";	// Necessita traduccio
#elif L==1
	"There are more than one user with the ID <strong>%s</strong>."
	" Please type a @nick or e-mail.";		// Need bersetzung
#elif L==2
	"There are more than one user with the ID <strong>%s</strong>."
	" Please type a @nick or e-mail.";
#elif L==3
	"Hay m&aacute;s de un usuario con el ID <strong>%s</strong>."
	" Por favor, escriba un @apodo o correo.";
#elif L==4
	"There are more than one user with the ID <strong>%s</strong>."
	" Please type a @nick or e-mail.";		// Besoin de traduction
#elif L==5
	"Hay m&aacute;s de un usuario con el ID <strong>%s</strong>."
	" Por favor, escriba un @apodo o correo.";	// Okoteve traduccin
#elif L==6
	"Ci sono pi&ugrave; di un utente con l'ID <strong>%s</strong>."
	" Si prega di digitare un @nome-utente o e-mail.";
#elif L==7
	"There are more than one user with the ID <strong>%s</strong>."
	" Please type a @nick or e-mail.";		// Potrzebujesz tlumaczenie
#elif L==8
	"There are more than one user with the ID <strong>%s</strong>."
	" Please type a @nick or e-mail.";		// Necessita de traduo
#endif

const char *Txt_There_are_new_posts =
#if   L==0
	"Hi ha missatges nous";
#elif L==1
	"Es gibt neue Nachrichten";
#elif L==2
	"There are new posts";
#elif L==3
	"Hay mensajes nuevos";
#elif L==4
	"Il y a des nouveaux messages";
#elif L==5
	"Hay mensajes nuevos";	// Okoteve traduccin
#elif L==6
	"Ci sono nuovi post";
#elif L==7
	"Istnieje nowych post&oacute;w";
#elif L==8
	"H&aacute; novas posts";
#endif

const char *Txt_There_are_no_accesses_with_the_selected_search_criteria =
#if   L==0
	"No hay accesos con el criterio de b&uacute;squeda seleccionado.";	// Necessita traduccio
#elif L==1
	"There are no accesses with the selected search criteria.";		// Need bersetzung
#elif L==2
	"There are no accesses with the selected search criteria.";
#elif L==3
	"No hay accesos con el criterio de b&uacute;squeda seleccionado.";
#elif L==4
	"There are no accesses with the selected search criteria.";		// Besoin de traduction
#elif L==5
	"No hay accesos con el criterio de b&uacute;squeda seleccionado.";	// Okoteve traduccin
#elif L==6
	"Non ci sono accessi con il criterio di ricerca selezionato.";
#elif L==7
	"There are no accesses with the selected search criteria.";		// Potrzebujesz tlumaczenie
#elif L==8
	"There are no accesses with the selected search criteria.";		// Necessita de traduo
#endif

const char *Txt_There_are_no_banners =
#if   L==0
	"No existen banners.";		// Necessita traduccio
#elif L==1
	"There are no banners.";	// Need bersetzung
#elif L==2
	"There are no banners.";
#elif L==3
	"No existen banners.";
#elif L==4
	"There are no banners.";	// Besoin de traduction
#elif L==5
	"No existen banners.";		// Okoteve traduccin
#elif L==6
	"Non ci sono banners.";
#elif L==7
	"There are no banners.";	// Potrzebujesz tlumaczenie
#elif L==8
	"There are no banners.";	// Necessita de traduo
#endif

const char *Txt_There_are_no_centres_with_requests_for_degrees_to_be_confirmed =
#if   L==0
	"No hi ha centres amb sollicituds de titulacions pendents de confirmar.";
#elif L==1
	"There are no centres with requests for degrees to be confirmed";		// Need bersetzung
#elif L==2
	"There are no centres with requests for degrees to be confirmed";
#elif L==3
	"No hay centros con solicitudes de titulaciones pendientes de confirmar.";
#elif L==4
	"Il n'y a pas de centres avec demandes de degr&eacute;s &agrave; confirmer.";
#elif L==5
	"No hay centros con solicitudes de titulaciones pendientes de confirmar.";	// Okoteve traduccin
#elif L==6
	"Non ci sono centri con richieste di gradi pendenti de confermare.";
#elif L==7
	"There are no centres with requests for degrees to be confirmed";		// Potrzebujesz tlumaczenie
#elif L==8
	"There are no centres with requests for degrees to be confirmed";		// Necessita de traduo
#endif

const char *Txt_There_are_no_countries_with_requests_for_institutions_to_be_confirmed =
#if   L==0
	"No hi ha pa&iuml;sos amb sollicituds de institucions pendents de confirmar.";
#elif L==1
	"There are no countries with requests for institutions to be confirmed";		// Need bersetzung
#elif L==2
	"There are no countries with requests for institutions to be confirmed";
#elif L==3
	"No hay pa&iacute;ses con solicitudes de instituciones pendientes de confirmar.";
#elif L==4
	"Il n'y a pas de pays avec demandes d'&eacute;tablissements &agrave; confirmer.";
#elif L==5
	"No hay pa&iacute;ses con solicitudes de instituciones pendientes de confirmar.";	// Okoteve traduccin
#elif L==6
	"Non ci sono paesi con richieste di istituzioni pendenti de confermare.";
#elif L==7
	"There are no countries with requests for institutions to be confirmed";		// Potrzebujesz tlumaczenie
#elif L==8
	"There are no countries with requests for institutions to be confirmed";		// Necessita de traduo
#endif

const char *Txt_There_are_no_degrees_with_requests_for_courses_to_be_confirmed =
#if   L==0
	"No hi ha titulacions amb sollicituds d'assignatures pendents de confirmar.";
#elif L==1
	"There are no degrees with requests for courses to be confirmed";		// Need bersetzung
#elif L==2
	"There are no degrees with requests for courses to be confirmed";
#elif L==3
	"No hay titulaciones con solicitudes de asignaturas pendientes de confirmar.";
#elif L==4
	"Il n'y a pas de degr&eacute;s avec demandes de cours &agrave; confirmer.";
#elif L==5
	"No hay titulaciones con solicitudes de asignaturas pendientes de confirmar.";	// Okoteve traduccin
#elif L==6
	"Non ci sono gradi con richieste di corsi pendenti de confermare.";
#elif L==7
	"There are no degrees with requests for courses to be confirmed";		// Potrzebujesz tlumaczenie
#elif L==8
	"There are no degrees with requests for courses to be confirmed";		// Necessita de traduo
#endif

const char *Txt_There_are_no_institutions_with_requests_for_centres_to_be_confirmed =
#if   L==0
	"No hi ha institucions amb sollicituds de centres pendents de confirmar.";
#elif L==1
	"There are no institutions with requests for centres to be confirmed";		// Need bersetzung
#elif L==2
	"There are no institutions with requests for centres to be confirmed";
#elif L==3
	"No hay instituciones con solicitudes de centros pendientes de confirmar.";
#elif L==4
	"Il n'y a pas d'&eacute;tablissements avec demandes de centres &agrave; confirmer.";
#elif L==5
	"No hay instituciones con solicitudes de centros pendientes de confirmar.";	// Okoteve traduccin
#elif L==6
	"Non ci sono istituzioni con richieste di centri pendenti de confermare.";
#elif L==7
	"There are no institutions with requests for centres to be confirmed";		// Potrzebujesz tlumaczenie
#elif L==8
	"There are no institutions with requests for centres to be confirmed";		// Necessita de traduo
#endif

const char *Txt_There_are_no_departments =
#if   L==0
	"No existen departamentos.";	// Necessita traduccio
#elif L==1
	"There are no departments.";	// Need bersetzung
#elif L==2
	"There are no departments.";
#elif L==3
	"No existen departamentos.";
#elif L==4
	"There are no departments.";	// Besoin de traduction
#elif L==5
	"No existen departamentos.";	// Okoteve traduccin
#elif L==6
	"Non ci sono dipartimenti.";
#elif L==7
	"There are no departments.";	// Potrzebujesz tlumaczenie
#elif L==8
	"There are no departments.";	// Necessita de traduo
#endif

const char *Txt_There_are_no_holidays =
#if   L==0
	"No existen d&iacute;as festivos.";	// Necessita traduccio
#elif L==1
	"There are no holidays.";		// Need bersetzung
#elif L==2
	"There are no holidays.";
#elif L==3
	"No existen d&iacute;as festivos.";
#elif L==4
	"There are no holidays.";		// Besoin de traduction
#elif L==5
	"No existen d&iacute;as festivos.";	// Okoteve traduccin
#elif L==6
	"Non ci sono festivit&agrave;.";
#elif L==7
	"There are no holidays.";		// Potrzebujesz tlumaczenie
#elif L==8
	"There are no holidays.";		// Necessita de traduo
#endif

const char *Txt_There_are_no_links =
#if   L==0
	"No existen enlaces.";		// Necessita traduccio
#elif L==1
	"There are no links.";		// Need bersetzung
#elif L==2
	"There are no links.";
#elif L==3
	"No existen enlaces.";
#elif L==4
	"There are no links.";		// Besoin de traduction
#elif L==5
	"No existen enlaces.";		// Okoteve traduccin
#elif L==6
	"Non ci sono links.";
#elif L==7
	"There are no links.";		// Potrzebujesz tlumaczenie
#elif L==8
	"There are no links.";		// Necessita de traduo
#endif

const char *Txt_There_are_no_mail_domains =
#if   L==0
	"No existen dominios de correo para notificaciones.";	// Necessita traduccio
#elif L==1
	"There are no mail domains for notifications.";		// Need bersetzung
#elif L==2
	"There are no mail domains for notifications.";
#elif L==3
	"No existen dominios de correo para notificaciones.";
#elif L==4
	"There are no mail domains for notifications.";		// Besoin de traduction
#elif L==5
	"No existen dominios de correo para notificaciones.";	// Okoteve traduccin
#elif L==6
	"Non ci sono campi mail per le notifiche.";
#elif L==7
	"There are no mail domains for notifications.";		// Potrzebujesz tlumaczenie
#elif L==8
	"There are no mail domains for notifications.";		// Necessita de traduo
#endif

const char *Txt_There_are_no_places =
#if   L==0
	"No existen lugares.";		// Necessita traduccio
#elif L==1
	"There are no places.";		// Need bersetzung
#elif L==2
	"There are no places.";
#elif L==3
	"No existen lugares.";
#elif L==4
	"There are no places.";		// Besoin de traduction
#elif L==5
	"No existen lugares.";		// Okoteve traduccin
#elif L==6
	"Non ci sono luoghi.";
#elif L==7
	"There are no places.";		// Potrzebujesz tlumaczenie
#elif L==8
	"There are no places.";		// Necessita de traduo
#endif

const char *Txt_There_are_no_plugins =
#if   L==0
	"No existen complementos.";	// Necessita traduccio
#elif L==1
	"There are no plugins.";	// Need bersetzung
#elif L==2
	"There are no plugins.";
#elif L==3
	"No existen complementos.";
#elif L==4
	"There are no plugins.";	// Besoin de traduction
#elif L==5
	"No existen complementos.";	// Okoteve traduccin
#elif L==6
	"Non ci sono plugin.";
#elif L==7
	"There are no plugins.";	// Potrzebujesz tlumaczenie
#elif L==8
	"There are no plugins.";	// Necessita de traduo
#endif

const char *Txt_There_are_no_record_fields_in_the_course_X =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"No existen campos de fichas en la asignatura <strong>%s</strong>.";	// Necessita traduccio
#elif L==1
	"There are no record fields in the course <strong>%s</strong>.";	// Need bersetzung
#elif L==2
	"There are no record fields in the course <strong>%s</strong>.";
#elif L==3
	"No existen campos de fichas en la asignatura <strong>%s</strong>.";
#elif L==4
	"There are no record fields in the course <strong>%s</strong>.";	// Besoin de traduction
#elif L==5
	"No existen campos de fichas en la asignatura <strong>%s</strong>.";	// Okoteve traduccin
#elif L==6
	"Non ci sono campi di schede nel corso <strong>%s</strong>.";
#elif L==7
	"There are no record fields in the course <strong>%s</strong>.";	// Potrzebujesz tlumaczenie
#elif L==8
	"There are no record fields in the course <strong>%s</strong>.";	// Necessita de traduo
#endif

const char *Txt_There_are_no_types_of_degree =
#if   L==0
	"No existen tipos de titulaci&oacute;n.";	// Necessita traduccio
#elif L==1
	"There are no types of degree.";		// Need bersetzung
#elif L==2
	"There are no types of degree.";
#elif L==3
	"No existen tipos de titulaci&oacute;n.";
#elif L==4
	"There are no types of degree.";		// Besoin de traduction
#elif L==5
	"No existen tipos de titulaci&oacute;n.";	// Okoteve traduccin
#elif L==6
	"Non ci sono tipi di lauree.";
#elif L==7
	"There are no types of degree.";		// Potrzebujesz tlumaczenie
#elif L==8
	"There are no types of degree.";		// Necessita de traduo
#endif

const char *Txt_There_are_no_types_of_group_in_the_course_X =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"No existen tipos de grupo en la asignatura <strong>%s</strong>.";	// Necessita traduccio
#elif L==1
	"There are no types of group in the course <strong>%s</strong>.";	// Need bersetzung
#elif L==2
	"There are no types of group in the course <strong>%s</strong>.";
#elif L==3
	"No existen tipos de grupo en la asignatura <strong>%s</strong>.";
#elif L==4
	"There are no types of group in the course <strong>%s</strong>.";	// Besoin de traduction
#elif L==5
	"No existen tipos de grupo en la asignatura <strong>%s</strong>.";	// Okoteve traduccin
#elif L==6
	"Non ci sono tipi di gruppi nel corso <strong>%s</strong>.";
#elif L==7
	"There are no types of group in the course <strong>%s</strong>.";	// Potrzebujesz tlumaczenie
#elif L==8
	"There are no types of group in the course <strong>%s</strong>.";	// Necessita de traduo
#endif

const char *Txt_There_are_X_users_with_the_ID_Y =	// Warning: it is very important to include %u and %s in the following sentences
#if   L==0
	"Hi ha %u usuaris amb el ID (DNI/c&eacute;dula) <strong>%s</strong>.";
#elif L==1
	"There are %u users with the ID <strong>%s</strong>.";			// Need bersetzung
#elif L==2
	"There are %u users with the ID <strong>%s</strong>.";
#elif L==3
	"Hay %u usuarios con el ID (DNI/c&eacute;dula) <strong>%s</strong>.";
#elif L==4
	"Il y a %u utilisateurs avec le ID <strong>%s</strong>.";
#elif L==5
	"Hay %u usuarios con el ID (DNI/c&eacute;dula) <strong>%s</strong>.";	// Okoteve traduccin
#elif L==6
	"Ci sono %u utenti con il ID <strong>%s</strong>.";
#elif L==7
	"There are %u users with the ID <strong>%s</strong>.";			// Potrzebujesz tlumaczenie
#elif L==8
	"There are %u users with the ID <strong>%s</strong>.";			// Necessita de traduo
#endif

const char *Txt_There_has_been_no_change_in_groups =
#if   L==0
	"No se ha realizado ning&uacute;n cambio de grupos.";	// Necessita traduccio
#elif L==1
	"There has been no change in groups.";			// bersetzung notwendig
#elif L==2
	"There has been no change in groups.";
#elif L==3
	"No se ha realizado ning&uacute;n cambio de grupos.";
#elif L==4
	"There has been no change in groups.";			// Besoin de traduction
#elif L==5
	"No se ha realizado ning&uacute;n cambio de grupos.";	// Okoteve traduccin
#elif L==6
	"Non ci sono stati cambiamenti in gruppi.";
#elif L==7
	"There has been no change in groups.";			// Potrzebujesz tlumaczenie
#elif L==8
	"There has been no change in groups.";			// Necessita de traduo
#endif

const char *Txt_There_have_been_X_errors_in_sending_the_message =	// Warning: it is very important to include %u in the following sentences
#if   L==0
	"Ha habido %u errores enviando el mensaje.";		// Necessita traduccio
#elif L==1
	"There have been %u errors in sending the message.";	// bersetzung notwendig
#elif L==2
	"There have been %u errors in sending the message.";
#elif L==3
	"Ha habido %u errores enviando el mensaje.";
#elif L==4
	"There have been %u errors in sending the message.";	// Besoin de traduction
#elif L==5
	"Ha habido %u errores enviando el mensaje.";		// Okoteve traduccin
#elif L==6
	"Ci sono stati %u errori inviando il messaggio.";
#elif L==7
	"There have been %u errors in sending the message.";	// Potrzebujesz tlumaczenie
#elif L==8
	"There have been %u errors in sending the message.";	// Necessita de traduo
#endif

const char *Txt_There_is_already_a_file_named_X =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"Ya existe un archivo con el nombre <strong>%s</strong>.";	// Necessita traduccio
#elif L==1
	"There is already a file named <strong>%s</strong>.";		// bersetzung notwendig
#elif L==2
	"There is already a file named <strong>%s</strong>.";
#elif L==3
	"Ya existe un archivo con el nombre <strong>%s</strong>.";
#elif L==4
	"There is already a file named <strong>%s</strong>.";		// Besoin de traduction
#elif L==5
	"Ya existe un archivo con el nombre <strong>%s</strong>.";	// Okoteve traduccin
#elif L==6
	"C'&egrave; gi&agrave; un file chiamato <strong>%s</strong>.";
#elif L==7
	"There is already a file named <strong>%s</strong>.";		// Potrzebujesz tlumaczenie
#elif L==8
	"There is already a file named <strong>%s</strong>.";		// Necessita de traduo
#endif

const char *Txt_There_is_already_a_non_empty_folder_named_X =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"Ya existe una carpeta no vac&iacute;a con el nombre <strong>%s</strong>.";	// Necessita traduccio
#elif L==1
	"There is already a non-empty folder named <strong>%s</strong>.";		// bersetzung notwendig
#elif L==2
	"There is already a non-empty folder named <strong>%s</strong>.";
#elif L==3
	"Ya existe una carpeta no vac&iacute;a con el nombre <strong>%s</strong>.";
#elif L==4
	"There is already a non-empty folder named <strong>%s</strong>.";		// Besoin de traduction
#elif L==5
	"Ya existe una carpeta no vac&iacute;a con el nombre <strong>%s</strong>.";	// Okoteve traduccin
#elif L==6
	"C'&egrave; gi&agrave; una cartella non vuota chiamata <strong>%s</strong>.";
#elif L==7
	"There is already a non-empty folder named <strong>%s</strong>.";		// Potrzebujesz tlumaczenie
#elif L==8
	"There is already a non-empty folder named <strong>%s</strong>.";		// Necessita de traduo
#endif

const char *Txt_There_is_no_external_service_for_authentication_and_official_lists =
#if   L==0
	"No hi ha cap servei extern d'autenticaci&oacute; i llistes oficials.";
#elif L==1
	"Es gibt keinen externen Dienstleister f&uuml;r die Authentifizierung und offiziellen Listen.";
#elif L==2
	"There is no external service for authentication and official lists.";
#elif L==3
	"No existe ning&uacute;n servicio externo de autenticaci&oacute;n y listas oficiales.";
#elif L==4
	"Il n'ya pas de service externe pour authentification et listes officiels.";
#elif L==5
	"No existe ning&uacute;n servicio externo de autenticaci&oacute;n y listas oficiales.";	// Okoteve traduccin
#elif L==6
	"Non c'&egrave; nessun servizio esterno per autenticazione e liste ufficiale.";
#elif L==7
	"Nie ma zewn&eogon;trznych us&lstrok;ug uwierzytelniania.";
#elif L==8
	"N&atilde;o h&aacute; servi&ccedil;o externo de autentica&ccedil;&atilde;o e listas oficiales.";
#endif

const char *Txt_There_is_no_knowing_how_many_users_not_logged_have_accessed =
#if   L==0
	"No se puede saber cu&aacute;ntos usuarios distintos han accedido sin identificarse.";	// Necessita traduccio
#elif L==1
	"There is no knowing how many users not logged have accessed.";				// bersetzung notwendig
#elif L==2
	"There is no knowing how many users not logged have accessed.";
#elif L==3
	"No se puede saber cu&aacute;ntos usuarios distintos han accedido sin identificarse.";
#elif L==4
	"There is no knowing how many users not logged have accessed.";				// Besoin de traduction
#elif L==5
	"No se puede saber cu&aacute;ntos usuarios distintos han accedido sin identificarse.";	// Okoteve traduccin
#elif L==6
	"Non si pu&ograve; sapere quanti utenti distinti hanno avuto accesso senza identificarsi.";
#elif L==7
	"There is no knowing how many users not logged have accessed.";				// Potrzebujesz tlumaczenie
#elif L==8
	"There is no knowing how many users not logged have accessed.";				// Necessita de traduo
#endif

const char *Txt_There_is_no_list_of_centres =
#if   L==0
	"No hay lista de centros.";		// Necessita traduccio
#elif L==1
	"There is no list of centres.";		// bersetzung notwendig
#elif L==2
	"There is no list of centres.";
#elif L==3
	"No hay lista de centros.";
#elif L==4
	"There is no list of centres.";		// Besoin de traduction
#elif L==5
	"No hay lista de centros.";		// Okoteve traduccin
#elif L==6
	"Non c'&egrave; la lista de centri.";
#elif L==7
	"There is no list of centres.";		// Potrzebujesz tlumaczenie
#elif L==8
	"There is no list of centres.";		// Necessita de traduo
#endif

const char *Txt_There_is_no_list_of_types_of_degree =
#if   L==0
	"No hay lista de tipos de titulaci&oacute;n.";	// Necessita traduccio
#elif L==1
	"There is no list of types of degree.";		// bersetzung notwendig
#elif L==2
	"There is no list of types of degree.";
#elif L==3
	"No hay lista de tipos de titulaci&oacute;n.";
#elif L==4
	"There is no list of types of degree.";		// Besoin de traduction
#elif L==5
	"No hay lista de tipos de titulaci&oacute;n.";	// Okoteve traduccin
#elif L==6
	"Non c'&egrave; la lista de tipi di laurea.";
#elif L==7
	"There is no list of types of degree.";		// Potrzebujesz tlumaczenie
#elif L==8
	"There is no list of types of degree.";		// Necessita de traduo
#endif

const char *Txt_There_is_no_list_of_institutions =
#if   L==0
	"No hay lista de instituciones.";		// Necessita traduccio
#elif L==1
	"There is no list of institutions.";		// bersetzung notwendig
#elif L==2
	"There is no list of institutions.";
#elif L==3
	"No hay lista de instituciones.";
#elif L==4
	"There is no list of institutions.";		// Besoin de traduction
#elif L==5
	"No hay lista de instituciones.";		// Okoteve traduccin
#elif L==6
	"Non c'&egrave; la lista delle istituzioni.";
#elif L==7
	"There is no list of institutions.";		// Potrzebujesz tlumaczenie
#elif L==8
	"There is no list of institutions.";		// Necessita de traduo
#endif

const char *Txt_There_is_no_user_with_email_X =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"No existe ning&uacute;n usuario"
	" con correo <strong>%s</strong>.";	// Necessita traduccio
#elif L==1
	"There is no user"
	" with e-mail <strong>%s</strong>.";	// bersetzung notwendig
#elif L==2
	"There is no user"
	" with e-mail <strong>%s</strong>.";
#elif L==3
	"No existe ning&uacute;n usuario"
	" con correo <strong>%s</strong>.";
#elif L==4
	"There is no user"
	" with e-mail <strong>%s</strong>.";	// Besoin de traduction
#elif L==5
	"No existe ning&uacute;n usuario"
	" con correo <strong>%s</strong>.";	// Okoteve traduccin
#elif L==6
	"Non c'&egrave; alcun utente"
	" con e-mail <strong>%s</strong>.";
#elif L==7
	"There is no user"
	" with e-mail <strong>%s</strong>.";	// Potrzebujesz tlumaczenie
#elif L==8
	"There is no user"
	" with e-mail <strong>%s</strong>.";	// Necessita de traduo
#endif

const char *Txt_There_is_no_user_in_X_with_ID_Y_If_you_already_have_an_account_on_Z_ =	// Warning: it is very important to include three %s in the following sentences
#if   L==0
	"No existe ning&uacute;n usuario en %s"
	" con el ID (DNI/c&eacute;dula) <strong>%s</strong>."
	" Si usted ya tiene cuenta en %s,"
	" inicie sesi&oacute;n y a&ntilde;ada este ID a su cuenta."
	" Si no tiene cuenta, puede crear una nueva.";		// Necessita traduccio
#elif L==1
	"There is no user in %s"
	" with ID <strong>%s</strong>."
	" If you already have an account on %s,"
	" login and add this ID to your account."
	" If you have no account, you can create a new one.";	// bersetzung notwendig
#elif L==2
	"There is no user in %s"
	" with ID <strong>%s</strong>."
	" If you already have an account on %s,"
	" login and add this ID to your account."
	" If you have no account, you can create a new one.";
#elif L==3
	"No existe ning&uacute;n usuario en %s"
	" con el ID (DNI/c&eacute;dula) <strong>%s</strong>."
	" Si usted ya tiene cuenta en %s,"
	" inicie sesi&oacute;n y a&ntilde;ada este ID a su cuenta."
	" Si no tiene cuenta, puede crear una nueva.";
#elif L==4
	"Il n'ya pas d'utilisateur dans %s"
	" avec ID <strong>%s</strong>."
	" Si vous avez d&eacute;j&agrave; un compte sur %s,"
	" vous connecter et ajouter cet ID &agrave; votre compte."
	" Si vous n'avez pas de compte, vous pouvez en cr&eacute;er un nouveau.";
#elif L==5
	"No existe ning&uacute;n usuario en %s"
	" con el ID (DNI/c&eacute;dula) <strong>%s</strong>."
	" Si usted ya tiene cuenta en %s,"
	" inicie sesi&oacute;n y a&ntilde;ada este ID a su cuenta."
	" Si no tiene cuenta, puede crear una nueva.";		// Okoteve traduccin
#elif L==6
	"Non c'&egrave; nessun utente in %s"
	" con ID <strong>%s</strong>."
	" Se hai gi&agrave; un account su %s,"
	" login e inserire questo ID al tuo account."
	" Se non avete account, &egrave; possibile crearne uno nuovo.";
#elif L==7
	"There is no user in %s"
	" with ID <strong>%s</strong>."
	" If you already have an account on %s,"
	" login and add this ID to your account."
	" If you have no account, you can create a new one.";	// Potrzebujesz tlumaczenie
#elif L==8
	"There is no user in %s"
	" with ID <strong>%s</strong>."
	" If you already have an account on %s,"
	" login and add this ID to your account."
	" If you have no account, you can create a new one.";	// Necessita de traduo
#endif

const char *Txt_There_is_no_user_with_ID_nick_or_e_mail_X =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"No existe ning&uacute;n usuario"
	" con ID (DNI/c&eacute;dula), apodo o correo <strong>%s</strong>.";	// Necessita traduccio
#elif L==1
	"There is no user"
	" with ID, nick or e-mail <strong>%s</strong>.";			// bersetzung notwendig
#elif L==2
	"There is no user"
	" with ID, nick or e-mail <strong>%s</strong>.";
#elif L==3
	"No existe ning&uacute;n usuario"
	" con ID (DNI/c&eacute;dula), apodo o correo <strong>%s</strong>.";
#elif L==4
	"There is no user"
	" with ID, nick or e-mail <strong>%s</strong>.";			// Besoin de traduction
#elif L==5
	"No existe ning&uacute;n usuario"
	" con ID (DNI/c&eacute;dula), apodo o correo <strong>%s</strong>.";	// Okoteve traduccin
#elif L==6
	"Non c'&egrave; alcun utente"
	" con ID, nome utente o e-mail <strong>%s</strong>.";
#elif L==7
	"There is no user"
	" with ID, nick or e-mail <strong>%s</strong>.";			// Potrzebujesz tlumaczenie
#elif L==8
	"There is no user"
	" with ID, nick or e-mail <strong>%s</strong>.";			// Necessita de traduo
#endif

const char *Txt_There_is_no_user_with_ID_nick_or_e_mail_X_in_the_course_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==0
	"No existe ning&uacute;n usuario"
	" con ID (DNI/c&eacute;dula), apodo o correo <strong>%s</strong>"
	" en la asignatura <strong>%s</strong>.";	// Necessita traduccio
#elif L==1
	"There is no user"
	" with ID, nick or e-mail <strong>%s</strong>"
	" in the course <strong>%s</strong>.";		// bersetzung notwendig
#elif L==2
	"There is no user"
	" with ID, nick or e-mail <strong>%s</strong>"
	" in the course <strong>%s</strong>.";
#elif L==3
	"No existe ning&uacute;n usuario"
	" con ID (DNI/c&eacute;dula), apodo o correo <strong>%s</strong>"
	" en la asignatura <strong>%s</strong>.";
#elif L==4
	"There is no user"
	" with ID, nick or e-mail <strong>%s</strong>"
	" in the course <strong>%s</strong>.";		// Besoin de traduction
#elif L==5
	"No existe ning&uacute;n usuario"
	" con ID (DNI/c&eacute;dula), apodo o correo <strong>%s</strong>"
	" en la asignatura <strong>%s</strong>.";	// Okoteve traduccin
#elif L==6
	"Non c'&egrave; alcun utente"
	" con ID, nome utente o e-mail <strong>%s</strong>"
	" nel corso <strong>%s</strong>.";
#elif L==7
	"There is no user"
	" with ID, nick or e-mail <strong>%s</strong>"
	" in the course <strong>%s</strong>.";		// Potrzebujesz tlumaczenie
#elif L==8
	"There is no user"
	" with ID, nick or e-mail <strong>%s</strong>"
	" in the course <strong>%s</strong>.";		// Necessita de traduo
#endif

const char *Txt_There_is_no_user_with_nickname_X =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"No existe ning&uacute;n usuario"
	" con el apodo <strong>%s</strong>.";		// Necessita traduccio
#elif L==1
	"There is no user"
	" with nickname <strong>%s</strong>.";		// bersetzung notwendig
#elif L==2
	"There is no user"
	" with nickname <strong>%s</strong>.";
#elif L==3
	"No existe ning&uacute;n usuario"
	" con el apodo <strong>%s</strong>.";
#elif L==4
	"There is no user"
	" with nickname <strong>%s</strong>.";		// Besoin de traduction
#elif L==5
	"No existe ning&uacute;n usuario"
	" con el apodo <strong>%s</strong>.";		// Okoteve traduccin
#elif L==6
	"Non c'&egrave; alcun utente"
	" con nome utente <strong>%s</strong>.";
#elif L==7
	"There is no user"
	" with nickname <strong>%s</strong>.";		// Potrzebujesz tlumaczenie
#elif L==8
	"There is no user"
	" with nickname <strong>%s</strong>.";		// Necessita de traduo
#endif

const char *Txt_There_was_a_problem_sending_an_email_automatically =
#if   L==0
	"Ha ocurrido un problema enviando autom&aacute;ticamente un correo electr&oacute;nico.";	// Necessita traduccio
#elif L==1
	"There was a problem sending an email automatically.";						// bersetzung notwendig
#elif L==2
	"There was a problem sending an email automatically.";
#elif L==3
	"Ha ocurrido un problema enviando autom&aacute;ticamente un correo electr&oacute;nico.";
#elif L==4
	"There was a problem sending an email automatically.";						// Besoin de traduction
#elif L==5
	"Ha ocurrido un problema enviando autom&aacute;ticamente un correo electr&oacute;nico.";	// Okoteve traduccin
#elif L==6
	"C'&egrave; stato un problema inviando automaticamente un'e-mail.";
#elif L==7
	"There was a problem sending an email automatically.";						// Potrzebujesz tlumaczenie
#elif L==8
	"There was a problem sending an email automatically.";						// Necessita de traduo
#endif

const char *Txt_There_was_an_error_in_assessing_the_test_X =	// Warning: it is very important to include %u in the following sentences
#if   L==0
	"Se ha producido un error al evaluar el test %u.";	// Necessita traduccio
#elif L==1
	"There was an error in assessing the test %u.";		// Need bersetzung
#elif L==2
	"There was an error in assessing the test %u.";
#elif L==3
	"Se ha producido un error al evaluar el test %u.";
#elif L==4
	"There was an error in assessing the test %u.";		// Besoin de traduction
#elif L==5
	"Se ha producido un error al evaluar el test %u.";	// Okoteve traduccin
#elif L==6
	"C'&egrave; stato un errore nel valutare il test %u.";
#elif L==7
	"There was an error in assessing the test %u.";		// Potrzebujesz tlumaczenie
#elif L==8
	"There was an error in assessing the test %u.";		// Necessita de traduo
#endif

const char *Txt_This_course_fulfills_X_out_of_Y_indicators_ =	// Warning: it is very important to include several %u, %ld, %s in correct order in the following sentences
#if   L==0
	"Esta asignatura cumple %u de %u indicadores."
	" &iexcl;ES <strong>MUY IMPORTANTE</strong> QUE CUMPLA LOS %u INDICADORES!<br />"
	"Por favor, vaya a"
	" <a href=\"%s/?CrsCod=%ld&amp;ActCod=%ld\" target=\"_blank\">"
	"%s &gt; %s &gt; %s</a>,"
	" verifique qu&eacute; indicadores no se satisfacen"
	" (marcados con &quot;<span class=\"DAT_SMALL_RED\">%s</span>&quot;)"
	" y haga todo lo posible por satisfacerlos.";	// Necessita traduccio
#elif L==1
	"This course fulfills %u out of %u indicators."
	" IT IS <strong>VERY IMPORTANT</strong> THAT FULLFILLS %u INDICATORS!<br />"
	"Please go to"
	" <a href=\"%s/?CrsCod=%ld&amp;ActCod=%ld\" target=\"_blank\">"
	"%s &gt; %s &gt; %s</a>,"
	" check what indicators are not fullfilled"
	" (marked with &quot;<span class=\"DAT_SMALL_RED\">%s</span>&quot;)"
	" and do everything possible to satisfy them.";	// Need bersetzung
#elif L==2
	"This course fulfills %u out of %u indicators."
	" IT IS <strong>VERY IMPORTANT</strong> THAT FULLFILLS %u INDICATORS!<br />"
	"Please go to"
	" <a href=\"%s/?CrsCod=%ld&amp;ActCod=%ld\" target=\"_blank\">"
	"%s &gt; %s &gt; %s</a>,"
	" check what indicators are not fullfilled"
	" (marked with &quot;<span class=\"DAT_SMALL_RED\">%s</span>&quot;)"
	" and do everything possible to satisfy them.";
#elif L==3
	"Esta asignatura cumple %u de %u indicadores."
	" &iexcl;ES <strong>MUY IMPORTANTE</strong> QUE CUMPLA LOS %u INDICADORES!<br />"
	"Por favor, vaya a"
	" <a href=\"%s/?CrsCod=%ld&amp;ActCod=%ld\" target=\"_blank\">"
	"%s &gt; %s &gt; %s</a>,"
	" verifique qu&eacute; indicadores no se satisfacen"
	" (marcados con &quot;<span class=\"DAT_SMALL_RED\">%s</span>&quot;)"
	" y haga todo lo posible por satisfacerlos.";
#elif L==4
	"This course fulfills %u out of %u indicators."
	" IT IS <strong>VERY IMPORTANT</strong> THAT FULLFILLS %u INDICATORS!<br />"
	"Please go to"
	" <a href=\"%s/?CrsCod=%ld&amp;ActCod=%ld\" target=\"_blank\">"
	"%s &gt; %s &gt; %s</a>,"
	" check what indicators are not fullfilled"
	" (marked with &quot;<span class=\"DAT_SMALL_RED\">%s</span>&quot;)"
	" and do everything possible to satisfy them.";	// Besoin de traduction
#elif L==5
	"Esta asignatura cumple %u de %u indicadores."
	" &iexcl;ES <strong>MUY IMPORTANTE</strong> QUE CUMPLA LOS %u INDICADORES!<br />"
	"Por favor, vaya a"
	" <a href=\"%s/?CrsCod=%ld&amp;ActCod=%ld\" target=\"_blank\">"
	"%s &gt; %s &gt; %s</a>,"
	" verifique qu&eacute; indicadores no se satisfacen"
	" (marcados con &quot;<span class=\"DAT_SMALL_RED\">%s</span>&quot;)"
	" y haga todo lo posible por satisfacerlos.";	// Okoteve traduccin
#elif L==6
	"This course fulfills %u out of %u indicators."
	" IT IS <strong>VERY IMPORTANT</strong> THAT FULLFILLS %u INDICATORS!<br />"
	"Please go to"
	" <a href=\"%s/?CrsCod=%ld&amp;ActCod=%ld\" target=\"_blank\">"
	"%s &gt; %s &gt; %s</a>,"
	" check what indicators are not fullfilled"
	" (marked with &quot;<span class=\"DAT_SMALL_RED\">%s</span>&quot;)"
	" and do everything possible to satisfy them.";	// Bisogno di traduzione
#elif L==7
	"This course fulfills %u out of %u indicators."
	" IT IS <strong>VERY IMPORTANT</strong> THAT FULLFILLS %u INDICATORS!<br />"
	"Please go to"
	" <a href=\"%s/?CrsCod=%ld&amp;ActCod=%ld\" target=\"_blank\">"
	"%s &gt; %s &gt; %s</a>,"
	" check what indicators are not fullfilled"
	" (marked with &quot;<span class=\"DAT_SMALL_RED\">%s</span>&quot;)"
	" and do everything possible to satisfy them.";	// Potrzebujesz tlumaczenie
#elif L==8
	"This course fulfills %u out of %u indicators."
	" IT IS <strong>VERY IMPORTANT</strong> THAT FULLFILLS %u INDICATORS!<br />"
	"Please go to"
	" <a href=\"%s/?CrsCod=%ld&amp;ActCod=%ld\" target=\"_blank\">"
	"%s &gt; %s &gt; %s</a>,"
	" check what indicators are not fullfilled"
	" (marked with &quot;<span class=\"DAT_SMALL_RED\">%s</span>&quot;)"
	" and do everything possible to satisfy them.";	// Necessita de traduo
#endif

const char *Txt_This_course_requires_log_in_from_X_to_have_full_functionality_ =	// Warning: it is very important to include four %s in the following sentences
#if   L==0
	"Esta asignatura requiere entrar"
	" desde <a href=\"%s\" target=\"_blank\">%s</a>"
	" para disfrutar de la funcionalidad completa."
	" Por favor, entre desde <a href=\"%s\" target=\"_blank\">%s</a>.";	// Necessita traduccio
#elif L==1
	"This course requires log in"
	" from <a href=\"%s\" target=\"_blank\">%s</a>"
	" to have full functionality."
	" Please log in from <a href=\"%s\" target=\"_blank\">%s</a>.";		// Need bersetzung
#elif L==2
	"This course requires log in"
	" from <a href=\"%s\" target=\"_blank\">%s</a>"
	" to have full functionality."
	" Please log in from <a href=\"%s\" target=\"_blank\">%s</a>.";
#elif L==3
	"Esta asignatura requiere entrar"
	" desde <a href=\"%s\" target=\"_blank\">%s</a>"
	" para disfrutar de la funcionalidad completa."
	" Por favor, entre desde <a href=\"%s\" target=\"_blank\">%s</a>.";
#elif L==4
	"This course requires log in"
	" from <a href=\"%s\" target=\"_blank\">%s</a>"
	" to have full functionality."
	" Please log in from <a href=\"%s\" target=\"_blank\">%s</a>.";		// Besoin de traduction
#elif L==5
	"This course requires log in"
	" from <a href=\"%s\" target=\"_blank\">%s</a>"
	" to have full functionality."
	" Please log in from <a href=\"%s\" target=\"_blank\">%s</a>.";		// Okoteve traduccin
#elif L==6
	"This course requires log in"
	" from <a href=\"%s\" target=\"_blank\">%s</a>"
	" to have full functionality."
	" Please log in from <a href=\"%s\" target=\"_blank\">%s</a>.";		// Bisogno di traduzione
#elif L==7
	"This course requires log in"
	" from <a href=\"%s\" target=\"_blank\">%s</a>"
	" to have full functionality."
	" Please log in from <a href=\"%s\" target=\"_blank\">%s</a>.";		// Potrzebujesz tlumaczenie
#elif L==8
	"This course requires log in"
	" from <a href=\"%s\" target=\"_blank\">%s</a>"
	" to have full functionality."
	" Please log in from <a href=\"%s\" target=\"_blank\">%s</a>.";		// Necessita de traduo
#endif

const char *Txt_this_field_is_filled_in_the_record_of_one_student =
#if   L==0
	" (dicho campo est&aacute; relleno en la ficha de un estudiante)";	// Necessita traduccio
#elif L==1
	" (this field is filled in the record of one student)";			// Need bersetzung
#elif L==2
	" (this field is filled in the record of one student)";
#elif L==3
	" (dicho campo est&aacute; relleno en la ficha de un estudiante)";
#elif L==4
	" (this field is filled in the record of one student)";			// Besoin de traduction
#elif L==5
	" (dicho campo est&aacute; relleno en la ficha de un estudiante)";	// Okoteve traduccin
#elif L==6
	" (questo campo &egrave; inserito nella scheda di uno studente)";
#elif L==7
	" (this field is filled in the record of one student)";			// Potrzebujesz tlumaczenie
#elif L==8
	" (this field is filled in the record of one student)";			// Necessita de traduo
#endif

const char *Txt_this_field_is_filled_in_the_records_of_X_students =	// Warning: it is very important to include %u in the following sentences
#if   L==0
	" (dicho campo est&aacute; relleno en las fichas de %u estudiantes)";	// Necessita traduccio
#elif L==1
	" (this field is filled in the records of %u students)";		// Need bersetzung
#elif L==2
	" (this field is filled in the records of %u students)";
#elif L==3
	" (dicho campo est&aacute; relleno en las fichas de %u estudiantes)";
#elif L==4
	" (this field is filled in the records of %u students)";		// Besoin de traduction
#elif L==5
	" (dicho campo est&aacute; relleno en las fichas de %u estudiantes)";	// Okoteve traduccin
#elif L==6
	" (questo campo &egrave; inserito nella scheda di %u studenti)";
#elif L==7
	" (this field is filled in the records of %u students)";		// Potrzebujesz tlumaczenie
#elif L==8
	" (this field is filled in the records of %u students)";		// Necessita de traduo
#endif

const char *Txt_This_option_is_now_in_Documents_Edit =
#if   L==0
	"Aquesta opci&oacute; est&agrave; ara en Documents &gt; Editar";
#elif L==1
	"Diese Option ist jetzt in Documents &gt; Bearbeiten";
#elif L==2
	"This option is now in Documents &gt; Edit";
#elif L==3
	"Esta opci&oacute;n se encuentra ahora en Documentos &gt; Editar";
#elif L==4
	"Cette option est d&eacute;sormais dans Documents &gt; &Eacute;diter";
#elif L==5
	"Esta opci&oacute;n se encuentra ahora en Documentos &gt; Editar";	// Okoteve traduccin
#elif L==6
	"Questa opzione &egrave; disponibile ora in Documenti &gt; Editare";
#elif L==7
	"Ta opcja jest ju&zdot; w Dokumentach &gt; Edytuj";
#elif L==8
	"Esta op&ccedil;&atilde;o est&aacute; agora em Documentos &gt; Editar";
#endif

const char *Txt_This_post_has_been_banned_probably_for_not_satisfy_the_rules_of_the_forums =
#if   L==0
	"Este mensaje ha sido censurado,"
	" probablemente por no cumplir las normas de los foros.";		// Necessita traduccio
#elif L==1
	"This post has been banned,"
	" probably for not satisfy the rules of the forums.";			// Need bersetzung
#elif L==2
	"This post has been banned,"
	" probably for not satisfy the rules of the forums.";
#elif L==3
	"Este mensaje ha sido censurado,"
	" probablemente por no cumplir las normas de los foros.";
#elif L==4
	"This post has been banned,"
	" probably for not satisfy the rules of the forums.";			// Besoin de traduction
#elif L==5
	"Este mensaje ha sido censurado,"
	" probablemente por no cumplir las normas de los foros.";		// Okoteve traduccin
#elif L==6
	"Questo post &egrave; stato censurato,"
	" probabilmente perch&egrave; non soddisfa le regole del forum.";
#elif L==7
	"This post has been banned,"
	" probably for not satisfy the rules of the forums.";			// Potrzebujesz tlumaczenie
#elif L==8
	"This post has been banned,"
	" probably for not satisfy the rules of the forums.";			// Necessita de traduo
#endif

const char *Txt_This_syllabus_has_been_edited_by_teachers_of_the_course_ =
#if   L==0
	"Aquest programa ha estat editat pels professors de l'assignatura"
	" i podria no coincidir amb el programa oficial.";
#elif L==1
	"This syllabus has been edited by teachers of the course"
	" and may not match the official syllabus.";			// Need bersetzung
#elif L==2
	"This syllabus has been edited by teachers of the course"
	" and may not match the official syllabus.";
#elif L==3
	"Este programa ha sido editado por los profesores de la asignatura"
	" y podr&iacute;a no coincidir con el programa oficial.";
#elif L==4
	"This syllabus has been edited by teachers of the course"
	" and may not match the official syllabus.";			// Besoin de traduction
#elif L==5
	"Este programa ha sido editado por los profesores de la asignatura"
	" y podr&iacute;a no coincidir con el programa oficial.";	// Okoteve traduccin
#elif L==6
	"Questo programma &egrave; stato editato per i professori del corso"
	" e potrebbe non corrispondere al programma ufficiale.";
#elif L==7
	"This syllabus has been edited by teachers of the course"
	" and may not match the official syllabus.";			// Potrzebujesz tlumaczenie
#elif L==8
	"This syllabus has been edited by teachers of the course"
	" and may not match the official syllabus.";			// Necessita de traduo
#endif

const char *Txt_This_survey_has_no_questions =
#if   L==0
	"Aquesta enquesta no t&eacute; preguntes";
#elif L==1
	"Diese Umfrage hat keine Fragen";
#elif L==2
	"This survey has no questions";
#elif L==3
	"Esta encuesta no tiene preguntas";
#elif L==4
	"Ce sondage n'a pas de questions";
#elif L==5
	"Esta encuesta no tiene preguntas";	// Okoteve traduccin
#elif L==6
	"Questo sondaggio non ha domande";
#elif L==7
	"Badanie to nie ma pytania";
#elif L==8
	"Este inqu&eacute;rito n&atilde;o tem perguntas";
#endif

const char *Txt_thread =
#if   L==0
	"discussi&oacute;";
#elif L==1
	"Thread";
#elif L==2
	"thread";
#elif L==3
	"discusi&oacute;n";
#elif L==4
	"fil";
#elif L==5
	"discusi&oacute;n";	// Okoteve traduccin
#elif L==6
	"discussione";
#elif L==7
	"w&aogon;tek";
#elif L==8
	"thread";
#endif

const char *Txt_Thread_marked_to_be_moved =
#if   L==0
	"Discusi&oacute;n marcada para moverla.";	// Necessita traduccio
#elif L==1
	"Thread marked to be moved.";			// Need bersetzung
#elif L==2
	"Thread marked to be moved.";
#elif L==3
	"Discusi&oacute;n marcada para moverla.";
#elif L==4
	"Thread marked to be moved.";			// Besoin de traduction
#elif L==5
	"Discusi&oacute;n marcada para moverla.";	// Okoteve traduccin
#elif L==6
	"Discussione segnata per spostarla.";
#elif L==7
	"Thread marked to be moved.";			// Potrzebujesz tlumaczenie
#elif L==8
	"Thread marked to be moved.";			// Necessita de traduo
#endif

const char *Txt_Thread_X_marked_to_be_moved =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"Discusi&oacute;n <strong>%s</strong> marcada para moverla.";	// Necessita traduccio
#elif L==1
	"Thread <strong>%s</strong> marked to be moved.";		// Need bersetzung
#elif L==2
	"Thread <strong>%s</strong> marked to be moved.";
#elif L==3
	"Discusi&oacute;n <strong>%s</strong> marcada para moverla.";
#elif L==4
	"Thread <strong>%s</strong> marked to be moved.";		// Besoin de traduction
#elif L==5
	"Discusi&oacute;n <strong>%s</strong> marcada para moverla.";	// Okoteve traduccin
#elif L==6
	"Discussione <strong>%s</strong> segnata per spostarla.";
#elif L==7
	"Thread <strong>%s</strong> marked to be moved.";		// Potrzebujesz tlumaczenie
#elif L==8
	"Thread <strong>%s</strong> marked to be moved.";		// Necessita de traduo
#endif

const char *Txt_Thread_moved_to_this_forum =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"Discusi&oacute;n movida a este foro.";	// Necessita traduccio
#elif L==1
	"Thread moved to this forum.";		// Need bersetzung
#elif L==2
	"Thread moved to this forum.";
#elif L==3
	"Discusi&oacute;n movida a este foro.";
#elif L==4
	"Thread moved to this forum.";		// Besoin de traduction
#elif L==5
	"Discusi&oacute;n movida a este foro.";	// Okoteve traduccin
#elif L==6
	"Discussione spostata in questo forum.";
#elif L==7
	"Thread moved to this forum.";		// Potrzebujesz tlumaczenie
#elif L==8
	"Thread moved to this forum.";		// Necessita de traduo
#endif

const char *Txt_Thread_X_moved_to_this_forum =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"Discusi&oacute;n <strong>%s</strong> movida a este foro.";	// Necessita traduccio
#elif L==1
	"Thread <strong>%s</strong> moved to this forum.";		// Need bersetzung
#elif L==2
	"Thread <strong>%s</strong> moved to this forum.";
#elif L==3
	"Discusi&oacute;n <strong>%s</strong> movida a este foro.";
#elif L==4
	"Thread <strong>%s</strong> moved to this forum.";		// Besoin de traduction
#elif L==5
	"Discusi&oacute;n <strong>%s</strong> movida a este foro.";	// Okoteve traduccin
#elif L==6
	"Discussione <strong>%s</strong> spostata in questo forum.";
#elif L==7
	"Thread <strong>%s</strong> moved to this forum.";		// Potrzebujesz tlumaczenie
#elif L==8
	"Thread <strong>%s</strong> moved to this forum.";		// Necessita de traduo
#endif

const char *Txt_Thread_removed =
#if   L==0
	"Discusi&oacute;n eliminada.";	// Necessita traduccio
#elif L==1
	"Faden entfernt.";
#elif L==2
	"Thread removed.";
#elif L==3
	"Discusi&oacute;n eliminada.";
#elif L==4
	"Thread removed.";		// Besoin de traduction
#elif L==5
	"Discusi&oacute;n eliminada.";	// Okoteve traduccin
#elif L==6
	"Discussione rimossa.";
#elif L==7
	"Thread removed.";		// Potrzebujesz tlumaczenie
#elif L==8
	"Thread removed.";		// Necessita de traduo
#endif

const char *Txt_Thread_X_removed =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"Discusi&oacute;n <strong>%s</strong> eliminada.";	// Necessita traduccio
#elif L==1
	"Faden <strong>%s</strong> entfernt.";
#elif L==2
	"Thread <strong>%s</strong> removed.";
#elif L==3
	"Discusi&oacute;n <strong>%s</strong> eliminada.";
#elif L==4
	"Thread <strong>%s</strong> removed.";			// Besoin de traduction
#elif L==5
	"Discusi&oacute;n <strong>%s</strong> eliminada.";	// Okoteve traduccin
#elif L==6
	"Discussione <strong>%s</strong> rimossa.";
#elif L==7
	"Thread <strong>%s</strong> removed.";			// Potrzebujesz tlumaczenie
#elif L==8
	"Thread <strong>%s</strong> removed.";			// Necessita de traduo
#endif

const char *Txt_Thread_with_posts_from_you =
#if   L==0
	"Discusi&oacute;n con mensajes de usted.";	// Necessita traduccio
#elif L==1
	"Thread with posts from you.";			// Need bersetzung
#elif L==2
	"Thread with posts from you.";
#elif L==3
	"Discusi&oacute;n con mensajes de usted.";
#elif L==4
	"Thread with posts from you.";			// Besoin de traduction
#elif L==5
	"Discusi&oacute;n con mensajes de usted.";	// Okoteve traduccin
#elif L==6
	"Discussione con post per te.";
#elif L==7
	"Thread with posts from you.";			// Potrzebujesz tlumaczenie
#elif L==8
	"Thread with posts from you.";			// Necessita de traduo
#endif

const char *Txt_Threads =
#if   L==0
	"Discussions";
#elif L==1
	"Threads";
#elif L==2
	"Threads";
#elif L==3
	"Discusiones";
#elif L==4
	"Fils";
#elif L==5
	"Discusiones";	// Okoteve traduccin
#elif L==6
	"Discussione";
#elif L==7
	"W&aogon;tki";
#elif L==8
	"Threads";
#endif

const char *Txt_threads =
#if   L==0
	"discussions";
#elif L==1
	"Threads";
#elif L==2
	"threads";
#elif L==3
	"discusiones";
#elif L==4
	"fils";
#elif L==5
	"discusiones";	// Okoteve traduccin
#elif L==6
	"discussione";
#elif L==7
	"w&aogon;tki";
#elif L==8
	"threads";
#endif

const char *Txt_time =
#if   L==0
	"temps";
#elif L==1
	"Zeit";
#elif L==2
	"time";
#elif L==3
	"tiempo";
#elif L==4
	"temps";
#elif L==5
	"tiempo";	// Okoteve traduccin
#elif L==6
	"tempo";
#elif L==7
	"czas";
#elif L==8
	"tempo";
#endif

const char *Txt_TIMETABLE_TYPES[TT_NUM_TIMETABLE_TYPES] =
	{
   // TT_COURSE_TIMETABLE
#if   L==0
	"Horari de classes de l'assignatura"
#elif L==1
	"Stundenplan"
#elif L==2
	"Class timetable"
#elif L==3
	"Horario de clases de la asignatura"
#elif L==4
	"Horaire"
#elif L==5
	"Horario de clases de la asignatura"	// Okoteve traduccin
#elif L==6
	"Orario del corso"
#elif L==7
	"Harmonogram klasy"
#elif L==8
	"Hor&aacute;rio"
#endif
	 ,
   // TT_MY_TIMETABLE
#if   L==0
	"Horari de les meves assignatures"
#elif L==1
	"Stundenplan"
#elif L==2
	"My timetable"
#elif L==3
	"Horario de mis asignaturas"
#elif L==4
	"Horaire mes mati&egrave;res"
#elif L==5
	"Horario de mis asignaturas"	// Okoteve traduccin
#elif L==6
	"Orario dei miei corsi"
#elif L==7
	"M&oacute;j plan lekcji"
#elif L==8
	"Hor&aacute;rio minha disciplinas"
#endif
	 ,
   // TT_TUTOR_TIMETABLE
#if   L==0
	"Horari de tutories"
#elif L==1
	"Gesch&auml;ftszeiten"
#elif L==2
	"Office hours"
#elif L==3
	"Horario de tutor&iacute;as"
#elif L==4
	"Heures de tuteur"
#elif L==5
	"Horario de tutor&iacute;as"	// Okoteve traduccin
#elif L==6
	"Orario di tutoraggi"
#elif L==7
	"Godziny pracy"
#elif L==8
	"Hor&aacute;rio de tutor"
#endif
	};

const char *Txt_TIMETABLE_CLASS_TYPES[TT_NUM_CLASS_TYPES] =
	{
   // TT_NO_CLASS
#if   L==0
	 "Lliure"
#elif L==1
	 "No class"
#elif L==2
	 "Keine Klasse"
#elif L==3
	 "Libre"
#elif L==4
	 "Pas de classe"
#elif L==5
	 "Libre"			// Okoteve traduccin
#elif L==6
	 "Libero"
#elif L==7
	 "Bez klasy"
#elif L==8
	 "Livre"
#endif
	 ,
	 // TT_THEORY_CLASS
#if   L==0
	 "Teoria"
#elif L==1
	 "Vorlesung"
#elif L==2
	 "Lectures"
#elif L==3
	 "Teor&iacute;a"
#elif L==4
	 "Th&eacute;orie"
#elif L==5
	 "Teor&iacute;a"		// Okoteve traduccin
#elif L==6
	 "Teoria"
#elif L==7
	 "Wyklady"
#elif L==8
	 "Teoria"
#endif
	 ,
	 // TT_PRACT_CLASS
#if   L==0
	 "Pr&agrave;ct."
#elif L==1
	 "&Uuml;bung"
#elif L==2
	 "Practicals"
#elif L==3
	 "Pr&aacute;ct."
#elif L==4
	 "Pratique"
#elif L==5
	 "Pr&aacute;ct."		// Okoteve traduccin
#elif L==6
	 "Pratica"
#elif L==7
	 "praktyki"
#elif L==8
	 "Pr&aacute;ct."
#endif
	 ,
	 // TT_TUTOR_CLASS
#if   L==0
	 "Tutories"
#elif L==1
	 "Anleitungen"
#elif L==2
	 "Tutoring"
#elif L==3
	 "Tutor&iacute;as"
#elif L==4
	 "Tutorat"
#elif L==5
	 "Tutor&iacute;as"	// Okoteve traduccin
#elif L==6
	 "Tutoraggio"
#elif L==7
	 "Korepetycje"
#elif L==8
	 "Tutoria"
#endif
	};

const char *Txt_Title =
#if   L==0
	"T&iacute;tol";
#elif L==1
	"Titel";
#elif L==2
	"Title";
#elif L==3
	"T&iacute;tulo";
#elif L==4
	"Titre";
#elif L==5
	"Moak&atilde;ha";
#elif L==6
	"Titolo";
#elif L==7
	"Tytu&lstrok;";
#elif L==8
	"T&iacute;tulo";
#endif

const char *Txt_To_get_the_official_list_of_your_groups_and_students_ =	// Warning: it is very important to include three %s in the following sentences
#if   L==0
	"Per obtenir la llista oficial dels seus grups i estudiants"
	" ha d'entrar a %s des <a href=\"%s\" target=\"_blank\">%s</a>.";
#elif L==1
	"To get the official list of your groups and students"
	" you must log in to %s from <a href=\"%s\" target=\"_blank\">%s</a>.";	// Need bersetzung
#elif L==2
	"To get the official list of your groups and students"
	" you must log in to %s from <a href=\"%s\" target=\"_blank\">%s</a>.";
#elif L==3
	"Para obtener la lista oficial de sus grupos y estudiantes"
	" debe entrar a %s desde <a href=\"%s\" target=\"_blank\">%s</a>.";
#elif L==4
	"To get the official list of your groups and students"
	" you must log in to %s from <a href=\"%s\" target=\"_blank\">%s</a>.";	// Besoin de traduction
#elif L==5
	"Para obtener la lista oficial de sus grupos y estudiantes"
	" debe entrar a %s desde <a href=\"%s\" target=\"_blank\">%s</a>.";	// Okoteve traduccin
#elif L==6
	"Per ottenere la lista ufficiale dei tuoi gruppi e studenti"
	" devi loggarti a %s da <a href=\"%s\" target=\"_blank\">%s</a>.";
#elif L==7
	"To get the official list of your groups and students"
	" you must log in to %s from <a href=\"%s\" target=\"_blank\">%s</a>.";	// Potrzebujesz tlumaczenie
#elif L==8
	"To get the official list of your groups and students"
	" you must log in to %s from <a href=\"%s\" target=\"_blank\">%s</a>.";	// Necessita de traduo
#endif

const char *Txt_To_remove_a_centre_you_must_first_remove_all_degrees_and_teachers_in_the_centre =
#if   L==0
	"Para eliminar un centro, primero debe eliminar"
	" todas las titulaciones y todos los profesores de ese centro.";	// Necessita traduccio
#elif L==1
	"To remove a centre, you must first remove"
	" all degrees and teachers in the centre.";				// Need bersetzung
#elif L==2
	"To remove a centre, you must first remove"
	" all degrees and teachers in the centre.";
#elif L==3
	"Para eliminar un centro, primero debe eliminar"
	" todas las titulaciones y todos los profesores de ese centro.";
#elif L==4
	"To remove a centre, you must first remove"
	" all degrees and teachers in the centre.";				// Besoin de traduction
#elif L==5
	"Para eliminar un centro, primero debe eliminar"
	" todas las titulaciones y todos los profesores de ese centro.";	// Okoteve traduccin
#elif L==6
	"Per rimuovere un centro, devi prima rimuovere"
	" tutte le lauree e professori di quel centro.";
#elif L==7
	"To remove a centre, you must first remove"
	" all degrees and teachers in the centre.";				// Potrzebujesz tlumaczenie
#elif L==8
	"To remove a centre, you must first remove"
	" all degrees and teachers in the centre.";				// Necessita de traduo
#endif

const char *Txt_To_remove_a_course_you_must_first_remove_all_users_in_the_course =
#if   L==0
	"Para eliminar una asignatura, primero debe eliminar"
	" todos los usuarios de esa asignatura.";		// Necessita traduccio
#elif L==1
	"To remove a course, you must first remove"
	" all users in the course.";				// Need bersetzung
#elif L==2
	"To remove a course, you must first remove"
	" all users in the course.";
#elif L==3
	"Para eliminar una asignatura, primero debe eliminar"
	" todos los usuarios de esa asignatura.";
#elif L==4
	"To remove a course, you must first remove"
	" all users in the course.";				// Besoin de traduction
#elif L==5
	"Para eliminar una asignatura, primero debe eliminar"
	" todos los usuarios de esa asignatura.";		// Okoteve traduccin
#elif L==6
	"Per rimuovere un corso, devi prima rimuovere"
	" tutti gli utenti di quel corso.";
#elif L==7
	"To remove a course, you must first remove"
	" all users in the course.";				// Potrzebujesz tlumaczenie
#elif L==8
	"To remove a course, you must first remove"
	" all users in the course.";				// Necessita de traduo
#endif

const char *Txt_To_remove_a_degree_you_must_first_remove_all_courses_in_the_degree =
#if   L==0
	"Para eliminar una titulaci&oacute;n, primero debe eliminar"
	" todas las asignaturas de esa titulaci&oacute;n.";	// Necessita traduccio
#elif L==1
	"To remove a degree, you must first remove"
	" all courses in the degree.";				// Need bersetzung
#elif L==2
	"To remove a degree, you must first remove"
	" all courses in the degree.";
#elif L==3
	"Para eliminar una titulaci&oacute;n, primero debe eliminar"
	" todas las asignaturas de esa titulaci&oacute;n.";
#elif L==4
	"To remove a degree, you must first remove"
	" all courses in the degree.";				// Besoin de traduction
#elif L==5
	"Para eliminar una titulaci&oacute;n, primero debe eliminar"
	" todas las asignaturas de esa titulaci&oacute;n.";	// Okoteve traduccin
#elif L==6
	"Per rimuovere una laurea, devi prima rimuovere"
	" tutti i corsi della laurea.";
#elif L==7
	"To remove a degree, you must first remove"
	" all courses in the degree.";				// Potrzebujesz tlumaczenie
#elif L==8
	"To remove a degree, you must first remove"
	" all courses in the degree.";				// Necessita de traduo
#endif

const char *Txt_To_remove_a_department_you_must_first_remove_all_teachers_in_the_department =
#if   L==0
	"Para eliminar un departamento, primero debe eliminar"
	" todos los profesores de ese departamento.";	// Necessita traduccio
#elif L==1
	"To remove a department, you must first remove"
	" all teachers in the department.";		// Need bersetzung
#elif L==2
	"To remove a department, you must first remove"
	" all teachers in the department.";
#elif L==3
	"Para eliminar un departamento, primero debe eliminar"
	" todos los profesores de ese departamento.";
#elif L==4
	"To remove a department, you must first remove"
	" all teachers in the department.";		// Besoin de traduction
#elif L==5
	"Para eliminar un departamento, primero debe eliminar"
	" todos los profesores de ese departamento.";	// Okoteve traduccin
#elif L==6
	"Per rimuovere un dipartimento, devi prima rimuovere"
	" tutti i professori di quel dipartimento.";
#elif L==7
	"To remove a department, you must first remove"
	" all teachers in the department.";		// Potrzebujesz tlumaczenie
#elif L==8
	"To remove a department, you must first remove"
	" all teachers in the department.";		// Necessita de traduo
#endif

const char *Txt_To_remove_a_place_you_must_first_remove_all_centres_of_that_place =
#if   L==0
	"Para eliminar un lugar, primero debe eliminar"
	" todos los centros de ese lugar.";		// Necessita traduccio
#elif L==1
	"To remove a place, you must first remove"
	" all centres of that place.";			// Need bersetzung
#elif L==2
	"To remove a place, you must first remove"
	" all centres of that place.";
#elif L==3
	"Para eliminar un lugar, primero debe eliminar"
	" todos los centros de ese lugar.";
#elif L==4
	"To remove a place, you must first remove"
	" all centres of that place.";			// Besoin de traduction
#elif L==5
	"Para eliminar un lugar, primero debe eliminar"
	" todos los centros de ese lugar.";		// Okoteve traduccin
#elif L==6
	"Per rimuovere un luogo, devi prima rimuovere"
	" tutte le centri di quel luogo.";
#elif L==7
	"To remove a place, you must first remove"
	" all centres of that place.";			// Potrzebujesz tlumaczenie
#elif L==8
	"To remove a place, you must first remove"
	" all centres of that place.";			// Necessita de traduo
#endif

const char *Txt_To_remove_a_type_of_degree_you_must_first_remove_all_degrees_of_that_type =
#if   L==0
	"Para eliminar un tipo de titulaci&oacute;n, primero debe eliminar"
	" todas las titulaciones de ese tipo.";		// Necessita traduccio
#elif L==1
	"To remove a type of degree, you must first remove"
	" all degrees of that type.";			// Need bersetzung
#elif L==2
	"To remove a type of degree, you must first remove"
	" all degrees of that type.";
#elif L==3
	"Para eliminar un tipo de titulaci&oacute;n, primero debe eliminar"
	" todas las titulaciones de ese tipo.";
#elif L==4
	"To remove a type of degree, you must first remove"
	" all degrees of that type.";			// Besoin de traduction
#elif L==5
	"Para eliminar un tipo de titulaci&oacute;n, primero debe eliminar"
	" todas las titulaciones de ese tipo.";		// Okoteve traduccin
#elif L==6
	"Per rimuovere un tipo di laurea, devi prima rimuovere"
	" tutte le lauree di quel tipo.";
#elif L==7
	"To remove a type of degree, you must first remove"
	" all degrees of that type.";			// Potrzebujesz tlumaczenie
#elif L==8
	"To remove a type of degree, you must first remove"
	" all degrees of that type.";			// Necessita de traduo
#endif

const char *Txt_To_remove_an_institution_you_must_first_remove_all_centres_and_users_in_the_institution =
#if   L==0
	"Para eliminar una instituci&oacute;n, primero debe eliminar"
	" todos los centros y todos los usuarios de esa instituci&oacute;n.";	// Necessita traduccio
#elif L==1
	"To remove an institution, you must first remove"
	" all centres and users in the institution.";				// Need bersetzung
#elif L==2
	"To remove an institution, you must first remove"
	" all centres and users in the institution.";
#elif L==3
	"Para eliminar una instituci&oacute;n, primero debe eliminar"
	" todos los centros y todos los usuarios de esa instituci&oacute;n.";
#elif L==4
	"To remove an institution, you must first remove"
	" all centres and users in the institution.";				// Besoin de traduction
#elif L==5
	"Para eliminar una instituci&oacute;n, primero debe eliminar"
	" todos centros y todos los usuarios de esa instituci&oacute;n.";	// Okoteve traduccin
#elif L==6
	"Per rimuovere un'istituzione, devi prima rimuovere"
	" tutti gli centri e utenti di quell'istituzione.";
#elif L==7
	"To remove an institution, you must first remove"
	" all centres and users in the institution.";				// Potrzebujesz tlumaczenie
#elif L==8
	"To remove an institution, you must first remove"
	" all centres and users in the institution.";				// Necessita de traduo
#endif

const char *Txt_to_see_photos_of_others_you_have_to_allow_others_to_see_your_photo_you_can_do_it_in_X =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"para ver las fotos de otros,"
	" tiene que permitir que otros vean la suya;"
	" puede hacerlo en %s";		// Necessita traduccio
#elif L==1
	"to see photos of others,"
	" you have to allow others to see your photo;"
	" you can do it in %s";		// Need bersetzung
#elif L==2
	"to see photos of others,"
	" you have to allow others to see your photo;"
	" you can do it in %s";
#elif L==3
	"para ver las fotos de otros,"
	" tiene que permitir que otros vean la suya;"
	" puede hacerlo en %s";
#elif L==4
	"to see photos of others,"
	" you have to allow others to see your photo;"
	" you can do it in %s";		// Besoin de traduction
#elif L==5
	"para ver las fotos de otros,"
	" tiene que permitir que otros vean la suya;"
	" puede hacerlo en %s";		// Okoteve traduccin
#elif L==6
	"per vedere le foto di altri,"
	" devi permettere agli altri di vedere la tua foto;"
	" puoi farlo in %s";
#elif L==7
	"to see photos of others,"
	" you have to allow others to see your photo;"
	" you can do it in %s";		// Potrzebujesz tlumaczenie
#elif L==8
	"to see photos of others,"
	" you have to allow others to see your photo;"
	" you can do it in %s";		// Necessita de traduo
#endif

const char *Txt_to_see_photos_of_others_you_have_to_send_your_photo_you_can_do_it_in_X =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"para ver las fotos de otros, tiene que enviar la suya;"
	" puede hacerlo en %s";		// Necessita traduccio
#elif L==1
	"to see photos of others, you have to send your photo;"
	" you can do it in %s";		// Need bersetzung
#elif L==2
	"to see photos of others, you have to send your photo;"
	" you can do it in %s";
#elif L==3
	"para ver las fotos de otros, tiene que enviar la suya;"
	" puede hacerlo en %s";
#elif L==4
	"to see photos of others, you have to send your photo;"
	" you can do it in %s";		// Besoin de traduction
#elif L==5
	"para ver las fotos de otros, tiene que enviar la suya;"
	" puede hacerlo en %s";		// Okoteve traduccin
#elif L==6
	"per vedere le foto di altri, devi inviare la tua foto;"
	" puoi farlo in %s";
#elif L==7
	"to see photos of others, you have to send your photo;"
	" you can do it in %s";		// Potrzebujesz tlumaczenie
#elif L==8
	"to see photos of others, you have to send your photo;"
	" you can do it in %s";		// Necessita de traduo
#endif

const char *Txt_To_use_chat_you_must_have_installed_the_software_X_and_add_Y_ =	// Warning: it is very important to include three %s in the following sentences
#if   L==0
	"Per utilitzar el xat cal tenir instal lat el programari"
	" <a href=\"%s\" target=\"_blank\"><em>%s</em></a>"
	" i afegir <strong>https://%s</strong>"
	" a la llista de llocs permesos en el panell de control de Java.";
#elif L==1
	"So verwenden Sie chatten, m&uuml;ssen Sie die Software installiert haben"
	" <a href=\"%s\" target=\"_blank\"><em>%s</em></a>"
	" und f&uuml;gen Sie <strong>https://%s</strong>"
	" in die Liste der erlaubten Seiten in der Java-Bedienfeld.";
#elif L==2
	"To use chat you must have installed the software"
	" <a href=\"%s\" target=\"_blank\"><em>%s</em></a>"
	" and add <strong>https://%s</strong>"
	" to the list of allowed sites in the Java control panel.";
#elif L==3
	"Para usar el chat es necesario tener instalado el software"
	" <a href=\"%s\" target=\"_blank\"><em>%s</em></a>"
	" y a&ntilde;adir <strong>https://%s</strong>"
	" a la lista de sitios permitidos en el panel de control de Java.";
#elif L==4
	"Pour utiliser le chat, vous devez avoir install&eacute; le logiciel"
	" <a href=\"%s\" target=\"_blank\"><em>%s</em></a>"
	" et ajouter <strong>https://%s</strong>"
	" &agrave; la liste des sites autoris&eacute;s dans le panneau de contr&ocirc;le Java.";
#elif L==5
	"Para usar el chat es necesario tener instalado el software"
	" <a href=\"%s\" target=\"_blank\"><em>%s</em></a>"
	" y a&ntilde;adir <strong>https://%s</strong>"
	" a la lista de sitios permitidos en el panel de control de Java.";	// Okoteve traduccin
#elif L==6
	"Per utilizzare la chat devi aver installato il software"
	" <a href=\"%s\" target=\"_blank\"><em>%s</em></a>"
	" e aggiungere <strong>https://%s</strong>"
	" alla lista dei siti consentiti nel pannello di controllo Java.";
#elif L==7
	"To use chat you must have installed the software"
	" <a href=\"%s\" target=\"_blank\"><em>%s</em></a>"
	" and add <strong>https://%s</strong>"
	" to the list of allowed sites in the Java control panel.";		// Potrzebujesz tlumaczenie
#elif L==8
	"Para usar o chat &eacute; necess&aacute;rio ter instalado o software"
	" <a href=\"%s\" target=\"_blank\"><em>%s</em></a>"
	" e adicionar <strong>https://%s</strong>"
	" &agrave; lista de sites permitidos no painel de controle Java.";
#endif

const char *Txt_Today =
#if   L==0
	"Avui";
#elif L==1
	"Heute";
#elif L==2
	"Today";
#elif L==3
	"Hoy";
#elif L==4
	"Aujourd'hui";
#elif L==5
	"Ko &aacute;ra";
#elif L==6
	"Oggi";
#elif L==7
	"Today";	// Potrzebujesz tlumaczenie
#elif L==8
	"Hoje";
#endif

const char *Txt_Too_fast =
#if   L==0
	"Massa r&agrave;pid!";
#elif L==1
	"Zu schnell!";
#elif L==2
	"Too fast!";
#elif L==3
	"&iexcl;Demasiado r&aacute;pido!";
#elif L==4
	"Trop vite!";
#elif L==5
	"&iexcl;Demasiado r&aacute;pido!";	// Okoteve traduccin
#elif L==6
	"Troppo veloce!";
#elif L==7
	"Za szybko!";
#elif L==8
	"Muito r&aacute;pido!";
#endif

const char *Txt_Total =
#if   L==0
	"Total";
#elif L==1
	"Gesamtzahl";
#elif L==2
	"Total";
#elif L==3
	"Total";
#elif L==4
	"Total";
#elif L==5
	"Total";	// Okoteve traduccin
#elif L==6
	"Totale";
#elif L==7
	"Ca&lstrok;kowity";
#elif L==8
	"Total";
#endif

const char *Txt_Total_BR_score =
#if   L==0
	"Puntuaci&oacute;<br />total";
#elif L==1
	"Gesamt";
#elif L==2
	"Total<br />score";
#elif L==3
	"Puntuaci&oacute;n<br />total";
#elif L==4
	"Note<br />total";
#elif L==5
	"Puntuaci&oacute;n<br />total ";	// Okoteve traduccin
#elif L==6
	"Punteggio<br />totale";
#elif L==7
	"Ca&lstrok;kowity<br />wynik";
#elif L==8
	"Pontua&ccedil;&atilde;o<br />total";
#endif

const char *Txt_Try_sending_another_image_where_the_face_appears_in_front_position_ =
#if   L==0
	"Pruebe a enviar otra imagen donde el rostro aparezca de frente"
	" y el fondo detr&aacute;s de la cabeza sea blanco o muy claro.";	// Necessita traduccio
#elif L==1
	"Try sending another image where the face appears in front position"
	" and the background behind of the head is white or very light.";	// Need bersetzung
#elif L==2
	"Try sending another image where the face appears in front position"
	" and the background behind of the head is white or very light.";
#elif L==3
	"Pruebe a enviar otra imagen donde el rostro aparezca de frente"
	" y el fondo detr&aacute;s de la cabeza sea blanco o muy claro.";
#elif L==4
	"Try sending another image where the face appears in front position"
	" and the background behind of the head is white or very light.";	// Besoin de traduction
#elif L==5
	"Pruebe a enviar otra imagen donde el rostro aparezca de frente"
	" y el fondo detr&aacute;s de la cabeza sea blanco o muy claro.";	// Okoteve traduccin
#elif L==6
	"Prova a inviare un'altra immagine dove il viso appare in posizione frontale"
	" e lo sfondo dietro della testa &egrave; bianco o molto chiaro.";
#elif L==7
	"Try sending another image where the face appears in front position"
	" and the background behind of the head is white or very light.";	// Potrzebujesz tlumaczenie
#elif L==8
	"Try sending another image where the face appears in front position"
	" and the background behind of the head is white or very light.";	// Necessita de traduo
#endif

const char *Txt_TST_Correct_ANSWER =
#if   L==0
	"Correcta";
#elif L==1
	"Richtige";
#elif L==2
	"Correct";
#elif L==3
	"Correcta";
#elif L==4
	"Bonne";
#elif L==5
	"Correcta";	// Okoteve traduccin
#elif L==6
	"Corretta";
#elif L==7
	"Poprawna";
#elif L==8
	"Correta";
#endif

const char *Txt_TST_PLUGGABLE[Tst_NUM_OPTIONS_PLUGGABLE] =
	 {
	 // Tst_PLUGGABLE_UNKNOWN
	   "",
	   // Tst_PLUGGABLE_NO
#if   L==0
	   "Tests no visibles des de m&ograve;bils"
#elif L==1
	   "Tests nicht zum Download aus Handys"
#elif L==2
	   "Tests not downloadable from mobile phones"
#elif L==3
	   "Tests no visibles desde m&oacute;viles"
#elif L==4
	   "Tests non t&eacute;l&eacute;chargeables &agrave; partir de mobiles"
#elif L==5
	   "Tests no visibles desde m&oacute;viles"	// Okoteve traduccin
#elif L==6
	   "Test non scaricabili da cellulari"
#elif L==7
	   "Nie Testy pobrania z kom&oacute;rkowych"
#elif L==8
	   "Testes n&atilde;o para download de celulares"
#endif
	   ,
	   // Tst_PLUGGABLE_YES
#if   L==0
	   "Tests visibles des de m&ograve;bils (opci&oacute; recomanada)"
#elif L==1
	   "Tests zum Download aus Handys (empfohlene Option)"
#elif L==2
	   "Tests downloadable from mobile phones (recommended option)"
#elif L==3
	   "Tests visibles desde m&oacute;viles (opci&oacute;n recomendada)"
#elif L==4
	   "Tests t&eacute;l&eacute;chargeables &agrave; partir de mobiles (option recommand&eacute;e)"
#elif L==5
	   "Tests visibles desde m&oacute;viles (opci&oacute;n recomendada)"// Okoteve traduccin
#elif L==6
	   "Test scaricabili da cellulari (opzione consigliata)"
#elif L==7
	   "Testy do pobrania z kom&oacute;rkowych (zalecana opcja)"
#elif L==8
	   "Testes para download de celulares (op&ccedil;&atilde;o recomendada)"
#endif
	};

const char *Txt_TST_STR_ANSWER_TYPES[Tst_NUM_ANS_TYPES] =
   {
#if   L==0
	 "Nombre enter"
#elif L==1
	 "Ganze Zahl"
#elif L==2
	 "Integer number"
#elif L==3
	 "N&ordm; entero"
#elif L==4
	 "Nombre entier"
#elif L==5
	 "N&ordm; entero"	// Okoteve traduccin
#elif L==6
	 "Numero intero"
#elif L==7
	 "Integer number"	// Potrzebujesz tlumaczenie
#elif L==8
	 "N&ordm; inteiro"
#endif
	 ,
#if   L==0
	 "Nombre real"
#elif L==1
	 "Reelle Zahl"
#elif L==2
	 "Real number"
#elif L==3
	 "N&ordm; real"
#elif L==4
	 "Nombre r&eacute;el"
#elif L==5
	 "N&ordm; real"	// Okoteve traduccin
#elif L==6
	 "Numero reale"
#elif L==7
	 "Liczba rzeczywista"
#elif L==8
	 "N&ordm; real"
#endif
	 ,
#if   L==0
	 "V/F"
#elif L==1
	 "R/F"
#elif L==2
	 "T/F"
#elif L==3
	 "V/F"
#elif L==4
	 "V/F"
#elif L==5
	 "V/F"	// Okoteve traduccin
#elif L==6
	 "V/F"
#elif L==7
	 "P/F"
#elif L==8
	 "V/F"
#endif
	 ,
#if   L==0
	 "Elecci&oacute; &uacute;nica"
#elif L==1
	 "Unique-Choice"
#elif L==2
	 "Unique choice"
#elif L==3
	 "Elecci&oacute;n &uacute;nica"
#elif L==4
	 "Choix unique"
#elif L==5
	 "Elecci&oacute;n &uacute;nica"		// Okoteve traduccin
#elif L==6
	 "Scelta unica"
#elif L==7
	 "Unikalny wyb&oacute;r"
#elif L==8
	 "&Uacute;nica escolha"
#endif
	 ,
#if   L==0
	 "Elecci&oacute; m&uacute;ltiple"
#elif L==1
	 "Multiple-Choice"
#elif L==2
	 "Multiple choice"
#elif L==3
	 "Elecci&oacute;n m&uacute;ltiple"
#elif L==4
	 "Choix multiple"
#elif L==5
	 "Elecci&oacute;n m&uacute;ltiple"	// Okoteve traduccin
#elif L==6
	 "Scelta multipla"
#elif L==7
	 "Wielokrotnego wyboru"
#elif L==8
	 "M&uacute;ltipla escolha"
#endif
	 ,
#if   L==0
	 "Text"
#elif L==1
	 "Text"
#elif L==2
	 "Text"
#elif L==3
	 "Texto"
#elif L==4
	 "Texte"
#elif L==5
	 "Mba'ehaipyre"
#elif L==6
	 "Testo"
#elif L==7
	 "Tekst"
#elif L==8
	 "Texto"
#endif
	,
	};

const char *Txt_TST_STR_FEEDBACK[Tst_NUM_TYPES_FEEDBACK] =
	{
#if   L==0
	 "Ninguna realimentaci&oacute;n"	// Necessita traduccio
#elif L==1
	 "Keine Feedback"
#elif L==2
	 "No feedback"
#elif L==3
	 "Ninguna realimentaci&oacute;n"
#elif L==4
	 "Pas de feedback"
#elif L==5
	 "Ninguna realimentaci&oacute;n"	// Okoteve traduccin
#elif L==6
	 "Nessun feedback"
#elif L==7
	 "No feedback"				// Potrzebujesz tlumaczenie
#elif L==8
	 "Nenhum feedback"
#endif
	 ,
#if   L==0
	 "Poca: s&oacute;lo puntuaci&oacute;n total"	// Necessita traduccio
#elif L==1
	 "Kleine: nur die Gesamtnote"
#elif L==2
	 "Little: only the total score"
#elif L==3
	 "Poca: s&oacute;lo puntuaci&oacute;n total"
#elif L==4
	 "Petit: seul le note total"
#elif L==5
	 "Poca: s&oacute;lo puntuaci&oacute;n total"	// Okoteve traduccin
#elif L==6
	 "Poco: solo il punteggio totale"
#elif L==7
	 "Little: only the total score"			// Potrzebujesz tlumaczenie
#elif L==8
	 "Pouco: apenas a pontua&ccedil;&atilde;o total"
#endif
	 ,
#if   L==0
	 "Media: puntuaci&oacute;n de cada pregunta"	// Necessita traduccio
#elif L==1
	 "Medium: Ergebnisse der einzelnen Frage"
#elif L==2
	 "Medium: result of each question"
#elif L==3
	 "Media: puntuaci&oacute;n de cada pregunta"
#elif L==4
	 "Medium: le r&eacute;sultat de chaque question"
#elif L==5
	 "Media: puntuaci&oacute;n de cada pregunta"	// Okoteve traduccin
#elif L==6
	 "Medio: risultato di ogni domanda"
#elif L==7
	 "Medium: result of each question"		// Potrzebujesz tlumaczenie
#elif L==8
	 "M&eacute;dio: pontua&ccedil;&atilde;o de cada quest&atilde;o"
#endif
	 ,
#if   L==0
	 "Alta: respuesta correcta en cada pregunta"			// Necessita traduccio
#elif L==1
	 "Maximale Details: die richtige Antwort f&uuml;r jede Frage"
#elif L==2
	 "High: correct answer for each question"
#elif L==3
	 "Alta: respuesta correcta en cada pregunta"
#elif L==4
	 "Haute: bonne r&eacute;ponse pour chaque question"
#elif L==5
	 "Alta: respuesta correcta en cada pregunta"			// Okoteve traduccin
#elif L==6
	 "Alto: la corretta risposta per ogni domanda"
#elif L==7
	 "High: correct answer for each question"			// Potrzebujesz tlumaczenie
#elif L==8
	 "Alto: resposta correta para cada quest&atilde;o"
#endif
	 ,
#if   L==0
	 "M&aacute;xima: mostrar texto de realimentaci&oacute;n"	// Necessita traduccio
#elif L==1
	 "Maximale: zeigen Feedback text"
#elif L==2
	 "Maximum: show feedback text"
#elif L==3
	 "M&aacute;xima: mostrar texto de realimentaci&oacute;n"
#elif L==4
	 "Maximum: afficher le texte de feedback"
#elif L==5
	 "M&aacute;xima: mostrar texto de realimentaci&oacute;n"	// Okoteve traduccin
#elif L==6
	 "Massimo: visualizzare il testo de feedback"
#elif L==7
	 "Maximum: show feedback text"					// Potrzebujesz tlumaczenie
#elif L==8
	 "M&aacute;ximo: mostrar o texto de feedback"
#endif
	};

const char *Txt_TST_STR_ORDER_FULL[Tst_NUM_TYPES_ORDER_QST] =
	 {
#if   L==0	// Tst_ORDER_STEM
	   "Ordenar por enunciado de la pregunta"	// Necessita traduccio
#elif L==1
	   "Sortieren nach Stammzellen in Frage"
#elif L==2
	   "Sort by stem of the question"
#elif L==3
	   "Ordenar por enunciado de la pregunta"
#elif L==4
	   "Trier par question"
#elif L==5
	   "Ordenar por enunciado de la pregunta"	// Okoteve traduccin
#elif L==6
	   "Ordina per domanda"
#elif L==7
	   "Sort by stem of the question"		// Potrzebujesz tlumaczenie
#elif L==8
	   "Classificar por formula&ccedil;&atilde;o da quest&atilde;o"
#endif
	   ,
#if   L==0	// Tst_ORDER_NUM_HITS
	   "Ordenar por n&ordm; de veces que se ha respondido la pregunta"	// Necessita traduccio
#elif L==1
	   "Sortieren nach Anzahl der mal die Frage beantwortet wurde"
#elif L==2
	   "Sort by number of times the question has been answered"
#elif L==3
	   "Ordenar por n&ordm; de veces que se ha respondido la pregunta"
#elif L==4
	   "Trier par nombre de fois qu'on a r&eacute;pondu &agrave; la question"
#elif L==5
	   "Ordenar por n&ordm; de veces que se ha respondido la pregunta"	// Okoteve traduccin
#elif L==6
	   "Ordina per numero di volte che hai risposto alla domanda"
#elif L==7
	   "Sort by number of times the question has been answered"		// Potrzebujesz tlumaczenie
#elif L==8
	   "Classificar por n&uacute;mero de vezes que a quest&atilde;o foi respondida"
#endif
	   ,
#if   L==0	// TST_ORDER_AVERAGE_SCORE
	   "Ordenar por puntuaci&oacute;n media"	// Necessita traduccio
#elif L==1
	   "Sortieren nach Durchschnittsnote"
#elif L==2
	   "Sort by average score"
#elif L==3
	   "Ordenar por puntuaci&oacute;n media"
#elif L==4
	   "Trier par note moyenne"
#elif L==5
	   "Ordenar por puntuaci&oacute;n media"	// Okoteve traduccin
#elif L==6
	   "Ordina per punteggio medio"
#elif L==7
	   "Sort by average score"			// Potrzebujesz tlumaczenie
#elif L==8
	   "Classificar por pontua&ccedil;&atilde;o m&eacute;dia"
#endif
#if   L==0	// Tst_ORDER_NUM_HITS_NOT_BLANK
	   "Ordenar por n&ordm; de veces que se ha respondido"
	   " la pregunta (excluyendo respuestas en blanco)"	// Necessita traduccio
#elif L==1
	   "Sortieren nach Anzahl der mal"
	   " die Frage beantwortet wurde (nicht leer)"
#elif L==2
	   "Sort by number of times the question"
	   " has been answered (excluding blank answers)"
#elif L==3
	   "Ordenar por n&ordm; de veces que se ha respondido"
	   " la pregunta (excluyendo respuestas en blanco)"
#elif L==4
	   "Trier par nombre de fois qu'on a r&eacute;pondu"
	   " &agrave; la question (non vide)"
#elif L==5
	   "Ordenar por n&ordm; de veces que se ha respondido"
	   " la pregunta (excluyendo respuestas en blanco)"	// Okoteve traduccin
#elif L==6
	   "Ordina per numero di volte che hai risposto"
	   " alla domanda (non vuoto)"
#elif L==7
	   "Sort by number of times the question"
	   " has been answered (excluding blank answers)"	// Potrzebujesz tlumaczenie
#elif L==8
		 "Classificar por n&uacute;mero de vezes"
		 " que a quest&atilde;o foi respondida (n&atilde;o em branco)"
#endif
	   ,
#if   L==0	// TST_ORDER_AVERAGE_SCORE_NOT_BLANK
	   "Ordenar por puntuaci&oacute;n media (excluyendo respuestas en blanco)"	// Necessita traduccio
#elif L==1
	   "Sortieren nach Durchschnittsnote (nicht leer)"
#elif L==2
	   "Sort by average score (excluding blank answers)"
#elif L==3
	   "Ordenar por puntuaci&oacute;n media (excluyendo respuestas en blanco)"
#elif L==4
	   "Trier par note moyenne (non vide)"
#elif L==5
	   "Ordenar por puntuaci&oacute;n media (excluyendo respuestas en blanco)"	// Okoteve traduccin
#elif L==6
	   "Ordina per punteggio medio (non vuoto)"
#elif L==7
	   "Sort by average score (excluding blank answers)"				// Potrzebujesz tlumaczenie
#elif L==8
	   "Classificar por pontua&ccedil;&atilde;o m&eacute;dia (n&atilde;o em branco)"
#endif
	};

const char *Txt_TST_STR_ORDER_SHORT[Tst_NUM_TYPES_ORDER_QST] =
	 {
#if   L==0	// Tst_ORDER_STEM
	   "Pregunta"	// Necessita traduccio
#elif L==1
	   "Frage"
#elif L==2
	   "Question"
#elif L==3
	   "Pregunta"
#elif L==4
	   "Question"
#elif L==5
	   "Pregunta"	// Okoteve traduccin
#elif L==6
	   "Domanda"
#elif L==7
	   "Question"	// Potrzebujesz tlumaczenie
#elif L==8
	   "Quest&atilde;o"
#endif
	   ,
#if   L==0	// Tst_ORDER_NUM_HITS
	   "N&ordm;<br />resp."			// Necessita traduccio
#elif L==1
	   "Anzahl<br />Antw."
#elif L==2
	   "No.<br />ans."
#elif L==3
	   "N&ordm;<br />resp."
#elif L==4
	   "Nombre<br />r&eacute;p."
#elif L==5
	   "N&ordm;<br />resp."			// Okoteve traduccin
#elif L==6
	   "Num.<br />risp."
#elif L==7
	   "No.<br />ans."			// Potrzebujesz tlumaczenie
#elif L==8
	   "N&ordm;<br />resp."
#endif
	   ,
#if   L==0	// TST_ORDER_AVERAGE_SCORE
	   "Puntuaci&oacute;n<br />media"	// Necessita traduccio
#elif L==1
	   "Durchschnitt"
#elif L==2
	   "Average<br />score"
#elif L==3
	   "Puntuaci&oacute;n<br />media"
#elif L==4
	   "Note<br />moyenne"
#elif L==5
	   "Puntuaci&oacute;n<br />media"	// Okoteve traduccin
#elif L==6
	   "Punteggio<br />medio"
#elif L==7
	   "Average<br />score"			// Potrzebujesz tlumaczenie
#elif L==8
	   "Pontua&ccedil;&atilde;o<br />m&eacute;dia"
#endif
	   ,
#if   L==0	// Tst_ORDER_NUM_HITS
	   "N&ordm;<br />resp.<br />no en<br />blanco"		// Necessita traduccio
#elif L==1
	   "Anzahl<br />Antw.<br />nicht<br />leer"
#elif L==2
	   "No.<br />ans.<br />not<br />blank"
#elif L==3
	   "N&ordm;<br />resp.<br />no en<br />blanco"
#elif L==4
	   "Nombre<br />r&eacute;p.<br />non<br />vide"
#elif L==5
	   "N&ordm;<br />resp.<br />no en<br />blanco"		// Okoteve traduccin
#elif L==6
	   "Num.<br />risp.<br />non<br />vuoto"
#elif L==7
	   "No.<br />ans.<br />not<br />blank"			// Potrzebujesz tlumaczenie
#elif L==8
	   "N&ordm;<br />resp.<br />n&atilde;o em<br />branco"
#endif
	   ,
#if   L==0	// TST_ORDER_AVERAGE_SCORE
	   "Puntuaci&oacute;n<br />media<br />no en<br />blanco"	// Necessita traduccio
#elif L==1
	   "Durchschnitt<br />nicht<br />leer"
#elif L==2
	   "Average<br />score<br />not<br />blank"
#elif L==3
	   "Puntuaci&oacute;n<br />media<br />no en<br />blanco"
#elif L==4
	   "Note<br />moyenne<br />non<br />vide"
#elif L==5
	   "Puntuaci&oacute;n<br />media<br />no en<br />blanco"	// Okoteve traduccin
#elif L==6
	   "Punteggio<br />medio<br />non<br />vuoto"
#elif L==7
	   "Average<br />score<br />not<br />blank"			// Potrzebujesz tlumaczenie
#elif L==8
	   "Pontua&ccedil;&atilde;o<br />m&eacute;dia<br />n&atilde;o em<br />branco"
#endif
	};

const char *Txt_TF_QST[2] =
   {
#if   L==0	// True
	 "V"
#elif L==1
	 "R"
#elif L==2
	 "T"
#elif L==3
	 "V"
#elif L==4
	 "V"
#elif L==5
	 "V"	// Okoteve traduccin
#elif L==6
	 "V"
#elif L==7
	 "P"
#elif L==8
	 "V"
#endif
	 ,
#if   L==0	// False
	 "F"
#elif L==1
	 "F"
#elif L==2
	 "F"
#elif L==3
	 "F"
#elif L==4
	 "F"
#elif L==5
	 "F"	// Okoteve traduccin
#elif L==6
	 "F"
#elif L==7
	 "F"
#elif L==8
	 "F"
#endif
	};

const char *Txt_Type =
#if   L==0
	"Tipus";
#elif L==1
	"Art";
#elif L==2
	"Type";
#elif L==3
	"Tipo";
#elif L==4
	"Type";
#elif L==5
	"Teko";
#elif L==6
	"Tipo";
#elif L==7
	"Typ";
#elif L==8
	"Tipo";
#endif

const char *Txt_Type_of_BR_answers =		// Answers of a test
#if   L==0
	"Tipus de<br />resposte";
#elif L==1
	"Antworttype";
#elif L==2
	"Type of<br />answers";
#elif L==3
	"Tipo de<br />respuestas";
#elif L==4
	"Type de<br />r&eacute;ponses";
#elif L==5
	"Tipo de<br />respuestas";	// Okoteve traduccin
#elif L==6
	"Tipo di<br />risposte";
#elif L==7
	"Typu<br />odpowiedzi";
#elif L==8
	"Tipo de<br />respostas";
#endif

const char *Txt_Type_of_degree =
#if   L==0
	"Tipus de titulaci&oacute;";
#elif L==1
	"Abschlussart";
#elif L==2
	"Type of degree";
#elif L==3
	"Tipo de titulaci&oacute;n";
#elif L==4
	"Type d'&eacute;tude";
#elif L==5
	"Arandur&atilde; l&aacute;ja";
#elif L==6
	"Tipo di laurea";
#elif L==7
	"Type of degree";		// Potrzebujesz tlumaczenie
#elif L==8
	"Tipo de titula&ccedil;&atilde;o";
#endif

const char *Txt_Type_of_BR_degree =
#if   L==0
	"Tipo de<br />titulaci&oacute;n";	// Necessita traduccio
#elif L==1
	"Abschlussart";
#elif L==2
	"Type of<br />degree";
#elif L==3
	"Tipo de<br />titulaci&oacute;n";
#elif L==4
	"Type<br />d'&eacute;tude";
#elif L==5
	"Tipo de<br />titulaci&oacute;n";	// Okoteve traduccin
#elif L==6
	"Tipo di<br />laurea";
#elif L==7
	"Type of<br />degree";			// Potrzebujesz tlumaczenie
#elif L==8
	"Tipo de<br />titula&ccedil;&atilde;o";
#endif

const char *Txt_Type_of_degree_X_removed =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"Tipo de titulaci&oacute;n <strong>%s</strong> eliminada.";	// Necessita traduccio
#elif L==1
	"Type of degree <strong>%s</strong> removed.";			// bersetzung notwendig
#elif L==2
	"Type of degree <strong>%s</strong> removed.";
#elif L==3
	"Tipo de titulaci&oacute;n <strong>%s</strong> eliminada.";
#elif L==4
	"Type of degree <strong>%s</strong> removed.";			// Besoin de traduction
#elif L==5
	"Tipo de titulaci&oacute;n <strong>%s</strong> eliminada.";	// Okoteve traduccin
#elif L==6
	"Tipo di laurea <strong>%s</strong> rimossa.";
#elif L==7
	"Type of degree <strong>%s</strong> removed.";			// Potrzebujesz tlumaczenie
#elif L==8
	"Type of degree <strong>%s</strong> removed.";			// Necessita de traduo
#endif

const char *Txt_Type_of_group =
#if   L==0
	"Tipo de grupo";	// Necessita traduccio
#elif L==1
	"Art der Gruppe";
#elif L==2
	"Type of group";
#elif L==3
	"Tipo de grupo";
#elif L==4
	"Type de groupe";
#elif L==5
	"Tipo de grupo";	// Okoteve traduccin
#elif L==6
	"Tipo di gruppo";
#elif L==7
	"Type of group";	// Potrzebujesz tlumaczenie
#elif L==8
	"Tipo de grupo";
#endif

const char *Txt_Type_BR_of_group =
#if   L==0
	"Tipo<br />de grupo";	// Necessita traduccio
#elif L==1
	"Art<br />der Gruppe";
#elif L==2
	"Type<br />of group";
#elif L==3
	"Tipo<br />de grupo";
#elif L==4
	"Type<br />de groupe";
#elif L==5
	"Tipo<br />de grupo";	// Okoteve traduccin
#elif L==6
	"Tipo<br />di gruppo";
#elif L==7
	"Type<br />of group";	// Potrzebujesz tlumaczenie
#elif L==8
	"Tipo<br />de grupo";
#endif

const char *Txt_Type_of_group_X_removed =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"Tipo de grupo <strong>%s</strong> eliminado.";	// Necessita traduccio
#elif L==1
	"Gruppen-Typ <strong>%s</strong> wurde entfernt.";
#elif L==2
	"Type of group <strong>%s</strong> removed.";
#elif L==3
	"Tipo de grupo <strong>%s</strong> eliminado.";
#elif L==4
	"Type of group <strong>%s</strong> removed.";	// Besoin de traduction
#elif L==5
	"Tipo de grupo <strong>%s</strong> eliminado.";	// Okoteve traduccin
#elif L==6
	"Tipo di gruppo <strong>%s</strong> rimosso.";
#elif L==7
	"Type of group <strong>%s</strong> removed.";	// Potrzebujesz tlumaczenie
#elif L==8
	"Type of group <strong>%s</strong> removed.";	// Necessita de traduo
#endif

const char *Txt_Types_of_group =
#if L==0
	"Tipus de grup";
#elif L==1
	"Gruppen-Typen";
#elif L==2
	"Types of group";
#elif L==3
	"Tipos de grupo";
#elif L==4
	"Types de groupe";
#elif L==5
	"Tipos de grupo";	// Okoteve traduccin
#elif L==6
	"Tipi di gruppo";
#elif L==7
	"Types of group";	// Potrzebujesz tlumaczenie
#elif L==8
	"Tipos de grupo";
#endif

const char *Txt_Type_or_paste_a_list_of_IDs_nicks_or_emails_ =
#if   L==0
	"Escriba o pegue una lista"
	" con @apodos, correos o ID (DNI/c&eacute;dulas)."
	" Los @apodos, correos o ID (DNI/c&eacute;dulas)"
	" pueden ir separados por cadenas de caracteres arbitrarias"
	" (por ej. nombres, apellidos, espacios, etc.)."
	" S&oacute;lo se tendr&aacute;n en cuenta aquellas palabras"
	" que parezcan @apodos, correos o ID (DNI/c&eacute;dulas);"
	" el resto del texto se ignorar&aacute;.";		// Necessita traduccio
#elif L==1
	"Type or paste a list"
	" of @nicks, e-mails or IDs."
	" The @nicks, e-mails or IDs"
	" can be separated by arbitrary strings"
	" (eg. names, places, etc.)."
	" Only the words that seems @nicks, e-mails or IDs"
	" will be taken into account;"
	" the rest of the text will be ignored.";		// Need bersetzung
#elif L==2
	"Type or paste a list"
	" of @nicks, e-mails or IDs."
	" The @nicks, e-mails or IDs"
	" can be separated by arbitrary strings"
	" (eg. names, places, etc.)."
	" Only the words that seems @nicks, e-mails or IDs"
	" will be taken into account;"
	" the rest of the text will be ignored.";
#elif L==3
	"Escriba o pegue una lista"
	" con @apodos, correos o ID (DNI/c&eacute;dulas)."
	" Los @apodos, correos o ID (DNI/c&eacute;dulas)"
	" pueden ir separados por cadenas de caracteres arbitrarias"
	" (por ej. nombres, apellidos, espacios, etc.)."
	" S&oacute;lo se tendr&aacute;n en cuenta aquellas palabras"
	" que parezcan @apodos, correos o ID (DNI/c&eacute;dulas);"
	" el resto del texto se ignorar&aacute;.";
#elif L==4
	"Type or paste a list"
	" of @nicks, e-mails or IDs."
	" The @nicks, e-mails or IDs"
	" can be separated by arbitrary strings"
	" (eg. names, places, etc.)."
	" Only the words that seems @nicks, e-mails or IDs"
	" will be taken into account;"
	" the rest of the text will be ignored.";		// Besoin de traduction
#elif L==5
	"Escriba o pegue una lista"
	" con @apodos, correos o ID (DNI/c&eacute;dulas)."
	" Los @apodos, correos o ID (DNI/c&eacute;dulas)"
	" pueden ir separados por cadenas de caracteres arbitrarias"
	" (por ej. nombres, apellidos, espacios, etc.)."
	" S&oacute;lo se tendr&aacute;n en cuenta aquellas palabras"
	" que parezcan @apodos, correos o ID (DNI/c&eacute;dulas);"
	" el resto del texto se ignorar&aacute;.";		// Okoteve traduccin
#elif L==6
	"Scrivi o incolla la lista"
	" di @nome-utenti, e-mails o IDs."
	" I @nome-utenti, e-mails o IDs"
	" possono essere separati da caratteri arbitrari"
	" (eg. nomi, luoghi, etc.)."
	" Solo le parole che sembrano"
	" @nome-utenti, e-mails o IDs"
	" saranno presi in considerazione;"
	" il resto del testo sar&agrave; ignorato.";
#elif L==7
	"Type or paste a list"
	" of @nicks, e-mails or IDs."
	" The @nicks, e-mails or IDs"
	" can be separated by arbitrary strings"
	" (eg. names, places, etc.)."
	" Only the words that seems @nicks, e-mails or IDs"
	" will be taken into account;"
	" the rest of the text will be ignored.";		// Potrzebujesz tlumaczenie
#elif L==8
	"Type or paste a list"
	" of @nicks, e-mails or IDs."
	" The @nicks, e-mails or IDs"
	" can be separated by arbitrary strings"
	" (eg. names, places, etc.)."
	" Only the words that seems @nicks, e-mails or IDs"
	" will be taken into account;"
	" the rest of the text will be ignored.";		// Necessita de traduo
#endif

const char *Txt_Types_of_degree =
#if   L==0
	"Tipus de titulaci&oacute;";
#elif L==1
	"Abschlussart";
#elif L==2
	"Types of degree";
#elif L==3
	"Tipos de titulaci&oacute;n";
#elif L==4
	"Types d'&eacute;tude";
#elif L==5
	"Arandur&atilde; l&aacute;ja";
#elif L==6
	"Tipi di laurea";
#elif L==7
	"Types of degree";		// Potrzebujesz tlumaczenie
#elif L==8
	"Tipos de titula&ccedil;&atilde;o";
#endif

const char *Txt_TYPES_OF_DEGREE_With_degrees =
#if   L==0
	"Amb titulacions";
#elif L==1
	"Mit Studiengang";
#elif L==2
	"With degrees";
#elif L==3
	"Con titulaciones";
#elif L==4
	"Avec &eacute;tudes";
#elif L==5
	"Con titulaciones";	// Okoteve traduccin
#elif L==6
	"Con lauree";
#elif L==7
	"Z Stopnie";
#elif L==8
	"Com titula&ccedil;oes";
#endif

const char *Txt_TYPES_OF_DEGREE_Without_degrees =
#if   L==0
	"Sense titulacions";
#elif L==1
	"Ohne Studiengang";
#elif L==2
	"Without degrees";
#elif L==3
	"Sin titulaciones";
#elif L==4
	"Sans &eacute;tudes";
#elif L==5
	"Sin titulaciones";	// Okoteve traduccin
#elif L==6
	"Senza lauree";
#elif L==7
	"Bez Stopnie";
#elif L==8
	"Sem titula&ccedil;oes";
#endif

const char *Txt_Undo =
#if   L==0
	"Desfer";
#elif L==1
	"R&uuml;ckg&auml;ngig";
#elif L==2
	"Undo";
#elif L==3
	"Deshacer";
#elif L==4
	"Annuler";
#elif L==5
	"Deshacer";	// Okoteve traduccin
#elif L==6
	"Annulla";
#elif L==7
	"Anulowa&cacute;";
#elif L==8
	"Desfazer";
#endif

const char *Txt_unknown_course =
#if   L==0
	"assignatura desconeguda";
#elif L==1
	"Kurs unbekannt";
#elif L==2
	"unknown course";
#elif L==3
	"asignatura desconocida";
#elif L==4
	"mati&egrave;re inconnue";
#elif L==5
	"asignatura desconocida";	// Okoteve traduccin
#elif L==6
	"corso sconosciuto";
#elif L==7
	"kurs nieznany";
#elif L==8
	"disciplina desconhecida";
#endif

const char *Txt_unknown_assignment =
#if   L==0
	"activitat desconeguda";
#elif L==1
	"Aufgabe unbekannt";
#elif L==2
	"unknown assignment";
#elif L==3
	"actividad desconocida";
#elif L==4
	"activit&eacute; inconnue";
#elif L==5
	"actividad desconocida";	// Okoteve traduccin
#elif L==6
	"attivit&agrave; sconosciuta";
#elif L==7
	"unknown assignment";		// Potrzebujesz tlumaczenie
#elif L==8
	"atividade desconhecida";
#endif

const char *Txt_unknown_recipient =
#if   L==0
	"destinatario desconocido";	// Necessita traduccio
#elif L==1
	"Empf&auml;nger unbekannt";
#elif L==2
	"unknown recipient";
#elif L==3
	"destinatario desconocido";
#elif L==4
	"destinataire inconnu";
#elif L==5
	"destinatario desconocido";	// Okoteve traduccin
#elif L==6
	"destinatario sconosciuto";
#elif L==7
	"unknown recipient";		// Potrzebujesz tlumaczenie
#elif L==8
	"destinat&aacute;rio desconhecido";
#endif

const char *Txt_unknown_recipients =
#if   L==0
	"destinatarios desconocidos";	// Necessita traduccio
#elif L==1
	"Empf&auml;nger unbekannt";
#elif L==2
	"unknown recipients";
#elif L==3
	"destinatarios desconocidos";
#elif L==4
	"destinataires inconnus";
#elif L==5
	"destinatarios desconocidos";	// Okoteve traduccin
#elif L==6
	"destinatari sconosciuti";
#elif L==7
	"unknown recipients";		// Potrzebujesz tlumaczenie
#elif L==8
	"destinat&aacute;rios desconhecidos";
#endif

const char *Txt_unknown_TIME =
#if   L==0
	"desconocido";	// Necessita traduccio
#elif L==1
	"unbekannt";
#elif L==2
	"unknown";
#elif L==3
	"desconocido";
#elif L==4
	"inconnu";
#elif L==5
	"desconocido";	// Okoteve traduccin
#elif L==6
	"sconosciuto";
#elif L==7
	"unknown";	// Potrzebujesz tlumaczenie
#elif L==8
	"desconhecido";
#endif

const char *Txt_unread_MESSAGE =
#if   L==0
	"no le&iacute;do";	// Necessita traduccio
#elif L==1
	"ungelesene";
#elif L==2
	"unread";
#elif L==3
	"no le&iacute;do";
#elif L==4
	"non lu";
#elif L==5
	"no le&iacute;do";	// Okoteve traduccin
#elif L==6
	"non letto";
#elif L==7
	"unread";		// Potrzebujesz tlumaczenie
#elif L==8
	"n&atilde;o lida";
#endif

const char *Txt_unread_MESSAGES =
#if   L==0
	"no le&iacute;dos";	// Necessita traduccio
#elif L==1
	"ungelesene";
#elif L==2
	"unread";
#elif L==3
	"no le&iacute;dos";
#elif L==4
	"non lus";
#elif L==5
	"no le&iacute;dos";	// Okoteve traduccin
#elif L==6
	"non letti";
#elif L==7
	"unread";		// Potrzebujesz tlumaczenie
#elif L==8
	"n&atilde;o lidas";
#endif

const char *Txt_Unread_BR_msgs =
#if   L==0
	"No<br />le&iacute;dos";	// Necessita traduccio
#elif L==1
	"Ungel.<br />Nachr.";
#elif L==2
	"Unread<br />msgs.";
#elif L==3
	"No<br />le&iacute;dos";
#elif L==4
	"Non<br />lus";
#elif L==5
	"No<br />le&iacute;dos";	// Okoteve traduccin
#elif L==6
	"Non<br />letti";
#elif L==7
	"Unread<br />msgs.";		// Potrzebujesz tlumaczenie
#elif L==8
	"N&atilde;o<br />lidas";
#endif

const char *Txt_unread_messages_received_from =
#if   L==0
	"mensajes no le&iacute;dos recibidos desde";	// Necessita traduccio
#elif L==1
	"ungelesene empfangene Nachrichten aus";
#elif L==2
	"unread messages received from";
#elif L==3
	"mensajes no le&iacute;dos recibidos desde";
#elif L==4
	"messages non lus re&ccedil;us de";
#elif L==5
	"mensajes no le&iacute;dos recibidos desde";	// Okoteve traduccin
#elif L==6
	"messaggi non letti ricevuti da";
#elif L==7
	"unread messages received from";		// Potrzebujesz tlumaczenie
#elif L==8
	"mensagens n&atilde;o lidas recebidas de";
#endif

const char *Txt_Update =
#if   L==0
	"Actualitzar";
#elif L==1
	"Update";
#elif L==2
	"Update";
#elif L==3
	"Actualizar";
#elif L==4
	"Mise &agrave; jour";
#elif L==5
	"Actualizar";	// Okoteve traduccin
#elif L==6
	"Aggiornamento";
#elif L==7
	"Update";	// Potrzebujesz tlumaczenie
#elif L==8
	"Atualizar";
#endif

const char *Txt_Update_attendance =
#if   L==0
	"Actualitzar assist&egrave;ncia";
#elif L==1
	"Update Anwesenheit";
#elif L==2
	"Update attendance";
#elif L==3
	"Actualizar asistencia";
#elif L==4
	"Mise &agrave; jour pr&eacute;sence";
#elif L==5
	"Actualizar asistencia";	// Okoteve traduccin
#elif L==6
	"Aggiornamento presenza";
#elif L==7
	"Aktualizacja bytno&sacute;&cacute;";
#elif L==8
	"Atualizar presen&ccedil;a";
#endif

const char *Txt_Update_attendance_according_to_selected_events =
#if   L==0
	"Actualitzar assist&egrave;ncia segons els esdeveniments seleccionats";
#elif L==1
	"Update Anwesenheit nach ausgew&auml;hlten Ereignisse";
#elif L==2
	"Update attendance according to selected events";
#elif L==3
	"Actualizar asistencia seg&uacute;n los eventos seleccionados";
#elif L==4
	"Mise &agrave; jour pr&eacute;sence en fonction d'&eacute;v&eacute;nements s&eacute;lectionnes";
#elif L==5
	"Actualizar asistencia seg&uacute;n los eventos seleccionados";	// Okoteve traduccin
#elif L==6
	"Aggiornamento presenza in base a eventi selezionati";
#elif L==7
	"Update bytno&sacute;&cacute; according to selected events";	// Potrzebujesz tlumaczenie
#elif L==8
	"Atualizar presen&ccedil;a de acordo com eventos selecionados";
#endif

const char *Txt_Update_connected_users =
#if   L==0
	"Actualitzar usuaris connectats";
#elif L==1
	"Update angemeldet Benutzer";
#elif L==2
	"Update connected users";
#elif L==3
	"Actualizar usuarios conectados";
#elif L==4
	"Mise &agrave; jour des utilisateurs connect&eacute;s";
#elif L==5
	"Actualizar usuarios conectados";	// Okoteve traduccin
#elif L==6
	"Aggiornamento utenti connessi";
#elif L==7
	"Update connected users";		// Potrzebujesz tlumaczenie
#elif L==8
	"Atualizar os utilizadores conectados";
#endif

const char *Txt_Update_indicators =
#if   L==0
	"Actualitzar indicadors";
#elif L==1
	"Update Indikatoren";
#elif L==2
	"Update indicators";
#elif L==3
	"Actualizar indicadores";
#elif L==4
	"Mise &agrave; jour des indicateurs";
#elif L==5
	"Actualizar indicadores";	// Okoteve traduccin
#elif L==6
	"Aggiornamento indicatori";
#elif L==7
	"Aktualizacja wska&zacute;niki";
#elif L==8
	"Atualizar indocadores";
#endif

const char *Txt_Update_students =
#if   L==0
	"Actualitzar estudiants";
#elif L==1
	"Update Studenten";
#elif L==2
	"Update students";
#elif L==3
	"Actualizar estudiantes";
#elif L==4
	"Mise &agrave; jour des &eacute;tudiants";
#elif L==5
	"Actualizar estudiantes";	// Okoteve traduccin
#elif L==6
	"Aggiornamento studenti";
#elif L==7
	"Aktualizacja student&oacute;w";
#elif L==8
	"Atualizar os estudantes";
#endif

const char *Txt_Update_students_according_to_selected_groups =
#if   L==0
	"Actualitzar estudiants segons els grups seleccionats";
#elif L==1
	"Update Studenten nach ausgew&auml;hlten Gruppen";
#elif L==2
	"Update students according to selected groups";
#elif L==3
	"Actualizar estudiantes seg&uacute;n los grupos seleccionados";
#elif L==4
	"Mise &agrave; jour des &eacute;tudiants en fonction de groupes s&eacute;lectionnes";
#elif L==5
	"Actualizar estudiantes seg&uacute;n los grupos seleccionados";	// Okoteve traduccin
#elif L==6
	"Aggiornamento studenti in base a gruppi selezionati";
#elif L==7
	"Update students according to selected groups";			// Potrzebujesz tlumaczenie
#elif L==8
	"Atualizar os estudantes de acordo com grupos selecionados";
#endif

const char *Txt_UPLOAD_FILE_Invalid_name =
#if   L==0
	"Nombre no v&aacute;lido.";	// Necessita traduccio
#elif L==1
	"Invalid name.";	// Need bersetzung
#elif L==2
	"Invalid name.";
#elif L==3
	"Nombre no v&aacute;lido.";
#elif L==4
	"Invalid name.";	// Besoin de traduction
#elif L==5
	"Nombre no v&aacute;lido.";	// Okoteve traduccin
#elif L==6
	"Nome non valido.";
#elif L==7
	"Invalid name.";	// Potrzebujesz tlumaczenie
#elif L==8
	"Invalid name.";	// Necessita de traduo
#endif

const char *Txt_UPLOAD_FILE_X_invalid_name =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"<strong>%s</strong>: nombre no v&aacute;lido.";	// Necessita traduccio
#elif L==1
	"<strong>%s</strong>: invalid name.";			// Need bersetzung
#elif L==2
	"<strong>%s</strong>: invalid name.";
#elif L==3
	"<strong>%s</strong>: nombre no v&aacute;lido.";
#elif L==4
	"<strong>%s</strong>: invalid name.";			// Besoin de traduction
#elif L==5
	"<strong>%s</strong>: nombre no v&aacute;lido.";	// Okoteve traduccin
#elif L==6
	"<strong>%s</strong>: nome non valido.";
#elif L==7
	"<strong>%s</strong>: invalid name.";			// Potrzebujesz tlumaczenie
#elif L==8
	"<strong>%s</strong>: invalid name.";			// Necessita de traduo
#endif

const char *Txt_Upload_file =
#if   L==0
	"Enviar archivo";	// Necessita traduccio
#elif L==1
	"Upload Datei";
#elif L==2
	"Upload file";
#elif L==3
	"Enviar archivo";
#elif L==4
	"T&eacute;l&eacute;charger fichier";
#elif L==5
	"Enviar archivo";	// Okoteve traduccin
#elif L==6
	"Caricare file";
#elif L==7
	"Upload file";		// Potrzebujesz tlumaczenie
#elif L==8
	"Fazer upload de arquivo";
#endif

const char *Txt_Upload_file_or_create_folder_in_FOLDER =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"A&ntilde;adir archivos o crear carpeta en %s";	// Necessita traduccio
#elif L==1
	"Datei uploaden oder Verzeichnis in %s erstellen";
#elif L==2
	"Upload files or create folder in %s";
#elif L==3
	"A&ntilde;adir archivos o crear carpeta en %s";
#elif L==4
	"Charger fichiers ou cr&eacute;er r&eacute;pertoire dans %s";
#elif L==5
	"A&ntilde;adir archivos o crear carpeta en %s";	// Okoteve traduccin
#elif L==6
	"Caricare file o creare cartella in %s";
#elif L==7
	"Upload file or create folder in %s";		// Potrzebujesz tlumaczenie
#elif L==8
	"Fazer upload de arquivos ou criar diret&oacute;rio em %s";
#endif

const char *Txt_Upload_files =
#if   L==0
	"Enviar fitxers";
#elif L==1
	"Upload Dateien";
#elif L==2
	"Upload files";
#elif L==3
	"Subir archivos";
#elif L==4
	"T&eacute;l&eacute;charger fichiers";
#elif L==5
	"Subir archivos";	// Okoteve traduccin
#elif L==6
	"Caricare file";
#elif L==7
	"Upload files";		// Potrzebujesz tlumaczenie
#elif L==8
	"Fazer upload de arquivos";
#endif

const char *Txt_Upload_files_QUESTION =
#if   L==0
	"Enviar<br />fitxers?";
#elif L==1
	"Upload<br />Dateien?";
#elif L==2
	"Upload<br />files?";
#elif L==3
	"&iquest;Enviar<br />archivos?";
#elif L==4
	"T&eacute;l&eacute;charger<br />fichiers?";
#elif L==5
	"&iquest;Enviar<br />archivos?";	// Okoteve traduccin
#elif L==6
	"Caricare<br />file?";
#elif L==7
	"Upload<br />files?";			// Potrzebujesz tlumaczenie
#elif L==8
	"Fazer upload<br />de arquivos?";
#endif

const char *Txt_Upload_photo =
#if   L==0
	"Enviar foto";
#elif L==1
	"Upload Foto";
#elif L==2
	"Upload photo";
#elif L==3
	"Subir foto";
#elif L==4
	"T&eacute;l&eacute;charger photo";
#elif L==5
	"Subir foto";		// Okoteve traduccin
#elif L==6
	"Caricare foto";
#elif L==7
	"Upload photo";		// Potrzebujesz tlumaczenie
#elif L==8
	"Fazer upload de foto";
#endif

const char *Txt_Uploaded_by =
#if   L==0
	"Penjat per";
#elif L==1
	"Hochgeladen von";
#elif L==2
	"Uploaded by";
#elif L==3
	"Subido por";
#elif L==4
	"Post&eacute; par";
#elif L==5
	"Subido por";	// Okoteve traduccin
#elif L==6
	"Caricato da";
#elif L==7
	"Wyslane przez";
#elif L==8
	"Enviado por";
#endif

const char *Txt_URL =		// Uniform Resource Locator
#if   L==0
	"URL";
#elif L==1
	"URL";
#elif L==2
	"URL";
#elif L==3
	"URL";
#elif L==4
	"URL";
#elif L==5
	"URL";	// Okoteve traduccin
#elif L==6
	"URL";
#elif L==7
	"URL";
#elif L==8
	"URL";
#endif

const char *Txt_Use_this_email =
#if   L==0
	"Utilitzar aquest correu";
#elif L==1
	"Verwenden Sie diese E-Mail";
#elif L==2
	"Use this e-mail";
#elif L==3
	"Usar este correo";
#elif L==4
	"Utiliser ce courrier";
#elif L==5
	"Usar este correo";	// Okoteve traduccin
#elif L==6
	"Utilizzare questa e-mail";
#elif L==7
	"U&zdot;yj tej e-mail";
#elif L==8
	"Utilizar este e-mail";
#endif

const char *Txt_Use_this_nickname =
#if   L==0
	"Utilitzar aquest sobrenom";
#elif L==1
	"Verwenden Sie diese Benutzername";
#elif L==2
	"Use this nickname";
#elif L==3
	"Usar este apodo";
#elif L==4
	"Utiliser ce surnom";
#elif L==5
	"Usar este apodo";	// Okoteve traduccin
#elif L==6
	"Utilizzare questo nome utente";
#elif L==7
	"U&zdot;yj tej nick";
#elif L==8
	"Utilizar esta alcunha";
#endif

const char *Txt_User =
#if   L==0
	"Usuari";
#elif L==1
	"Benutzer";
#elif L==2
	"User";
#elif L==3
	"Usuario";
#elif L==4
	"Utilisateur";
#elif L==5
	"Puruh&aacute;ra";
#elif L==6
	"Utente";
#elif L==7
	"User";		// Potrzebujesz tlumaczenie
#elif L==8
	"Utilizador";
#endif

const char *Txt_user[Usr_NUM_SEXS] =
	{	// Usr_SEX_UNKNOWN
#if   L==0
	"usuari/a"
#elif L==1
	"Benutzer"
#elif L==2
	"user"
#elif L==3
	"usuario/a"
#elif L==4
	"utilisateur/se"
#elif L==5
	"puruh&atilde;ra"
#elif L==6
	"utente"
#elif L==7
	"u&zdot;ytkownika"
#elif L==8
	"utilizador/a"
#endif
	,	// Usr_SEX_FEMALE
#if   L==0
	"usuaria"
#elif L==1
	"Benutzer"
#elif L==2
	"user"
#elif L==3
	"usuaria"
#elif L==4
	"utilisateuse"
#elif L==5
	"usuaria"	// Okoteve traduccin
#elif L==6
	"utente"
#elif L==7
	"u&zdot;ytkownika"
#elif L==8
	"utilizadora"
#endif
	,	// Usr_SEX_MALE
#if   L==0
	"usuari"
#elif L==1
	"Benutzer"
#elif L==2
	"user"
#elif L==3
	"usuario"
#elif L==4
	"utilisateur"
#elif L==5
	"puruh&atilde;ra"
#elif L==6
	"utente"
#elif L==7
	"u&zdot;ytkownika"
#elif L==8
	"utilizador"
#endif
	,	// Usr_SEX_ALL
#if   L==0
	"usuari/a"
#elif L==1
	"Benutzer"
#elif L==2
	"user"
#elif L==3
	"usuario/a"
#elif L==4
	"utilisateur/se"
#elif L==5
	"puruh&atilde;ra"
#elif L==6
	"utente"
#elif L==7
	"u&zdot;ytkownika"
#elif L==8
	"utilizador/a"
#endif
	};

const char *Txt_User_account =
#if   L==0
	"Compte d'usuari";
#elif L==1
	"Benutzerkonto";
#elif L==2
	"User account";
#elif L==3
	"Cuenta de usuario";
#elif L==4
	"Compte d'utilisateur";
#elif L==5
	"Cuenta de usuario";	// Okoteve traduccin
#elif L==6
	"Account utente";
#elif L==7
	"Konto u&zdot;ytkownika";
#elif L==8
	"Conta de usu&aacute;rio";
#endif

const char *Txt_USER_comments =
#if   L==0
	"Observaciones";	// Necessita traduccio
#elif L==1
	"Kommentare";
#elif L==2
	"Comments";
#elif L==3
	"Observaciones";
#elif L==4
	"Commentaires";
#elif L==5
	"Observaciones";	// Okoteve traduccin
#elif L==6
	"Commenti";
#elif L==7
	"Comments";		// Potrzebujesz tlumaczenie
#elif L==8
	"Coment&aacute;rios";
#endif

const char *Txt_User_ID =
#if   L==0
	"ID (DNI/c&egrave;dula/passaport) de l'usuari";
#elif L==1
	"Ausweis-Nr.";
#elif L==2
	"User ID";
#elif L==3
	"ID (DNI/c&eacute;dula) del usuario";
#elif L==4
	"Num&eacute;ro d'identit&eacute; d'utilisateur";
#elif L==5
	"ID (DNI/c&eacute;dula) del usuario";		// Okoteve traduccin
#elif L==6
	"Numero carta d'identit&agrave; utente";
#elif L==7
	"User ID";					// Potrzebujesz tlumaczenie
#elif L==8
	"N&ordm; de identif. do utilizador";
#endif

const char *Txt_User_not_found_or_you_do_not_have_permission_ =
#if   L==0
	"Usuari no trobat"
	" o b&eacute; vost&egrave; no t&eacute; perm&iacute;s"
	" per realitzar aquesta acci&oacute;.";
#elif L==1
	"Benutzer nicht gefunden"
	" oder Sie haben keine Berechtigung,"
	" um diese Handlung durchzuf&uuml;hren.";
#elif L==2
	"User not found"
	" or you do not have permission"
	" to perform this action.";
#elif L==3
	"Usuario/a no encontrado/a"
	" o bien usted no tiene permiso"
	" para realizar esta acci&oacute;n.";
#elif L==4
	"Utilisateur introuvable"
	" ou vous n'avez pas l'autorisation"
	" d'effectuer cette action.";
#elif L==5
	"Usuario/a no encontrado/a"
	" o bien usted no tiene permiso"
	" para realizar esta acci&oacute;n.";	// Okoteve traduccin
#elif L==6
	"IUtente non trovato"
	" o non si dispone dell'autorizzazione"
	" per eseguire questa azione.";
#elif L==7
	"Utilisateur introuvable"
	" or you do not have permission"
	" to perform this action.";		// Potrzebujesz tlumaczenie
#elif L==8
	"Utilizador n&atilde;o encontrado"
	" ou voc&ecirc; n&atilde;o tem permiss&atilde;o"
	" para realizar esta a&ccedil;&atilde;o.";
#endif

const char *Txt_User_record_card_in_the_course_X_has_been_removed =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"La ficha del usuario en la asignatura <strong>%s</strong> ha sido eliminada.";	// Necessita traduccio
#elif L==1
	"User record card in the course <strong>%s</strong> has been removed.";		// Need bersetzung
#elif L==2
	"User record card in the course <strong>%s</strong> has been removed.";
#elif L==3
	"La ficha del usuario en la asignatura <strong>%s</strong> ha sido eliminada.";
#elif L==4
	"User record card in the course <strong>%s</strong> has been removed.";		// Besoin de traduction
#elif L==5
	"La ficha del usuario en la asignatura <strong>%s</strong> ha sido eliminada.";	// Okoteve traduccin
#elif L==6
	"La scheda dell'utente nel corso <strong>%s</strong> &egrave; stata rimossa.";
#elif L==7
	"User record card in the course <strong>%s</strong> has been removed.";		// Potrzebujesz tlumaczenie
#elif L==8
	"User record card in the course <strong>%s</strong> has been removed.";		// Necessita de traduo
#endif

const char *Txt_User_record_cards_in_all_courses_have_been_removed =
#if   L==0
	"Se han eliminado las fichas del usuario en todas las asignaturas.";	// Necessita traduccio
#elif L==1
	"User record cards in all courses have been removed.";			// Need bersetzung
#elif L==2
	"User record cards in all courses have been removed.";
#elif L==3
	"Se han eliminado las fichas del usuario en todas las asignaturas.";
#elif L==4
	"User record cards in all courses have been removed.";			// Besoin de traduction
#elif L==5
	"Se han eliminado las fichas del usuario en todas las asignaturas.";	// Okoteve traduccin
#elif L==6
	"Le schede dell'utente in tutti i corsi sono state rimosse.";
#elif L==7
	"User record cards in all courses have been removed.";			// Potrzebujesz tlumaczenie
#elif L==8
	"User record cards in all courses have been removed.";			// Necessita de traduo
#endif

const char *Txt_Users =
#if   L==0
	"Usuaris";
#elif L==1
	"Benutzer";
#elif L==2
	"Users";
#elif L==3
	"Usuarios";
#elif L==4
	"Utilisateurs";
#elif L==5
	"Puruh&aacute;ra";
#elif L==6
	"Utenti";
#elif L==7
	"U&zdot;ytkownicy";
#elif L==8
	"Utilizadores";
#endif

const char *Txt_users =
#if   L==0
	"usuaris";
#elif L==1
	"Benutzer";
#elif L==2
	"users";
#elif L==3
	"usuarios";
#elif L==4
	"utilisateurs";
#elif L==5
	"puruh&aacute;ra";
#elif L==6
	"utenti";
#elif L==7
	"u&zdot;ytkownicy";
#elif L==8
	"utilizadores";
#endif

const char *Txt_X_users_have_been_eliminated =	// Warning: it is very important to include %u in the following sentences
#if   L==0
	"Se ha(n) eliminado %u usuario(s).";		// Necessita traduccio
#elif L==1
	"%u user(s) have been eliminated.";		// Need bersetzung
#elif L==2
	"%u user(s) have been eliminated.";
#elif L==3
	"Se ha(n) eliminado %u usuario(s).";
#elif L==4
	"%u users have been eliminated.";		// Besoin de traduction
#elif L==5
	"Se ha(n) eliminado %u usuario(s).";		// Okoteve traduccin
#elif L==6
	"%u utenti sono stati rimossi.";
#elif L==7
	"%u user(s) have been eliminated.";		// Potrzebujesz tlumaczenie
#elif L==8
	"%u user(s) have been eliminated.";		// Necessita de traduo
#endif

const char *Txt_X_users_have_been_enrolled_including_possible_repetitions =	// Warning: it is very important to include %u in the following sentences
#if   L==0
	"Se han inscrito %u usuarios (incluyendo posibles repeticiones).";	// Necessita traduccio
#elif L==1
	"%u users have been enrolled (including possible repetitions).";	// Need bersetzung
#elif L==2
	"%u users have been enrolled (including possible repetitions).";
#elif L==3
	"Se han inscrito %u usuarios (incluyendo posibles repeticiones).";
#elif L==4
	"%u users have been enrolled (including possible repetitions).";	// Besoin de traduction
#elif L==5
	"Se han inscrito %u usuarios (incluyendo posibles repeticiones).";	// Okoteve traduccin
#elif L==6
	"%u utenti sono stati registrati (comprese le eventuali ripetizioni).";
#elif L==7
	"%u users have been enrolled (including possible repetitions).";	// Potrzebujesz tlumaczenie
#elif L==8
	"%u users have been enrolled (including possible repetitions).";	// Necessita de traduo
#endif

const char *Txt_X_users_have_been_removed =	// Warning: it is very important to include %u in the following sentences
#if   L==0
	"Se han eliminado %u usuarios.";	// Necessita traduccio
#elif L==1
	"%u users have been removed.";		// Need bersetzung
#elif L==2
	"%u users have been removed.";
#elif L==3
	"Se han eliminado %u usuarios.";
#elif L==4
	"%u users have been removed.";		// Besoin de traduction
#elif L==5
	"Se han eliminado %u usuarios.";	// Okoteve traduccin
#elif L==6
	"%u utenti sono stati rimossi.";
#elif L==7
	"%u users have been removed.";		// Potrzebujesz tlumaczenie
#elif L==8
	"%u users have been removed.";		// Necessita de traduo
#endif

const char *Txt_usr =		// Abbreviation of user (three characters + dot)
#if   L==0
	"usr.";	// Necessita traduccio
#elif L==1
	"Ben.";
#elif L==2
	"usr.";
#elif L==3
	"usr.";
#elif L==4
	"uti.";
#elif L==5
	"usr.";	// Okoteve traduccin
#elif L==6
	"ute.";
#elif L==7
	"usr.";	// Potrzebujesz tlumaczenie
#elif L==8
	"uti.";
#endif

const char *Txt_USR_LIST_TYPES[Usr_NUM_USR_LIST_TYPES] =
   {
#if   L==0
	 "Orla"
#elif L==1
	 "Klass Foto"
#elif L==2
	 "Class photo"
#elif L==3
	 "Orla"
#elif L==4
	 "Photo de classe"
#elif L==5
	 "Orla"		// Okoteve traduccin
#elif L==6
	 "Foto di classe"
#elif L==7
	 "Class photo"	// Potrzebujesz tlumaczenie
#elif L==8
	 "Foto de formatura"
#endif
	 ,
#if   L==0
	 "Llista"
#elif L==1
	 "Liste"
#elif L==2
	 "List"
#elif L==3
	 "Lista"
#elif L==4
	 "Liste"
#elif L==5
	 "Lista"	// Okoteve traduccin
#elif L==6
	 "Lista"
#elif L==7
	 "List"		// Potrzebujesz tlumaczenie
#elif L==8
	 "Lista"
#endif
	};

const char *Txt_Vacants =
#if   L==0
	"Vacants";
#elif L==1
	"Frei";
#elif L==2
	"Vacants";
#elif L==3
	"Vacantes";
#elif L==4
	"Vacants";
#elif L==5
	"Vacantes";	// Okoteve traduccin
#elif L==6
	"Liberi";
#elif L==7
	"Vacants";	// Potrzebujesz tlumaczenie
#elif L==8
	"Desocupados";
#endif

const char *Txt_View_all_recipients =
#if   L==0
	"Ver todos los destinatarios";	// Necessita traduccio
#elif L==1
	"Anzeigen alle Empf&auml;nger";
#elif L==2
	"View all recipients";
#elif L==3
	"Ver todos los destinatarios";
#elif L==4
	"Voir tous les destinataires";
#elif L==5
	"Ver todos los destinatarios";	// Okoteve traduccin
#elif L==6
	"Vedi tutti i destinatari";
#elif L==7
	"View all recipients";		// Potrzebujesz tlumaczenie
#elif L==8
	"Ver todos os estinat&aacute;rios";
#endif

const char *Txt_View_as_a_student =
#if   L==0
	"Veure com estudiant";
#elif L==1
	"Anzeigen als Student";
#elif L==2
	"View as a student";
#elif L==3
	"Ver como estudiante";
#elif L==4
	"Voir comme &eacute;tudiant/e";
#elif L==5
	"Ver como estudiante";	// Okoteve traduccin
#elif L==6
	"Vedi come studente";
#elif L==7
	"Wy&sacute;wietl jako student";
#elif L==8
	"Ver como estudante";
#endif

const char *Txt_View_data_of_FILE_OR_LINK_X =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"Veure dades de %s";
#elif L==1
	"Siehe Daten von %s";
#elif L==2
	"View data of %s";
#elif L==3
	"Ver datos de %s";
#elif L==4
	"Voir les donn&eacute;es de %s";
#elif L==5
	"Ver datos de %s";	// Okoteve traduccin
#elif L==6
	"Visualizza i dati de %s";
#elif L==7
	"Zobacz dane z %s";
#elif L==8
	"Veja os dados de %s";
#endif

const char *Txt_View_event =
#if   L==0
	"Veure esdeveniment";
#elif L==1
	"Anzeigen Ereignis";
#elif L==2
	"View event";
#elif L==3
	"Ver evento";
#elif L==4
	"Voir &eacute;v&eacute;nement";
#elif L==5
	"Ver evento";			// Okoteve traduccin
#elif L==6
	"Vedi evento";
#elif L==7
	"View event";			// Potrzebujesz tlumaczenie
#elif L==8
	"Ver evento";
#endif

const char *Txt_View_in_a_new_window =
#if   L==0
	"Mostrar en una finestra nova";
#elif L==1
	"Ansicht in einem neuen Fenster";
#elif L==2
	"View in a new window";
#elif L==3
	"Mostrar en una ventana nueva";
#elif L==4
	"Voir dans une nouvelle fen&ecirc;tre";
#elif L==5
	"Mostrar en una ventana nueva";	// Okoteve traduccin
#elif L==6
	"Visualizza in una nuova finestra";
#elif L==7
	"View in a new window";		// Potrzebujesz tlumaczenie
#elif L==8
	"Ver em uma nova janela";
#endif

const char *Txt_View_messages =
#if   L==0
	"Veure missatges";
#elif L==1
	"Anzeigen Nachrichten";
#elif L==2
	"View messages";
#elif L==3
	"Ver mensajes";
#elif L==4
	"Voir messages";
#elif L==5
	"Ver mensajes";		// Okoteve traduccin
#elif L==6
	"Vedi messaggi";
#elif L==7
	"View messages";	// Potrzebujesz tlumaczenie
#elif L==8
	"Ver mensagens";
#endif

const char *Txt_View_my_courses =
#if   L==0
	"Veure els meus assignatures";
#elif L==1
	"Anzeigen meine Kursen";
#elif L==2
	"View my courses";
#elif L==3
	"Ver mis asignaturas";
#elif L==4
	"Voir mes mati&egrave;res";
#elif L==5
	"Ver mis asignaturas";	// Okoteve traduccin
#elif L==6
	"Vedi i miei corsi";
#elif L==7
	"View my courses";	// Potrzebujesz tlumaczenie
#elif L==8
	"Ver as minha disciplinas";
#endif

const char *Txt_View_my_record_for_this_course =
#if   L==0
	"Veure la meva fitxa en aquesta assignatura";
#elif L==1
	"Anzeigen profil bei Kurs";
#elif L==2
	"View my record for this course";
#elif L==3
	"Ver mi ficha en esta asignatura";
#elif L==4
	"Voir ma carte dans mati&egrave;re";
#elif L==5
	"Ver mi ficha en esta asignatura";	// Okoteve traduccin
#elif L==6
	"Vedi la mia scheda per questo corso";
#elif L==7
	"Zobacz m&oacute;j rekord na ten kurs";
#elif L==8
	"Meu cart&atilde;o em disciplina";
#endif

const char *Txt_View_survey =
#if   L==0
	"Veure enquesta";
#elif L==1
	"Anzeigen Umfrage";
#elif L==2
	"View survey";
#elif L==3
	"Ver encuesta";
#elif L==4
	"Voir sondage";
#elif L==5
	"Ver encuesta";			// Okoteve traduccin
#elif L==6
	"Vedi sondaggio";
#elif L==7
	"View survey";			// Potrzebujesz tlumaczenie
#elif L==8
	"Ver inqu&eacute;rito";
#endif

const char *Txt_View_survey_results =
#if   L==0
	"Ver resultados";	// Necessita traduccio
#elif L==1
	"Anzeigen Ergebnisse";
#elif L==2
	"View survey results";
#elif L==3
	"Ver resultados";
#elif L==4
	"Voir r&eacute;sultats";
#elif L==5
	"Ver resultados";	// Okoteve traduccin
#elif L==6
	"Vedi risultati";
#elif L==7
	"View survey results";	// Potrzebujesz tlumaczenie
#elif L==8
	"Ver resultados";
#endif

const char *Txt_View_works =
#if   L==0
	"Veure treballs";
#elif L==1
	"Anzeigen Hausarbeiten";
#elif L==2
	"View homework";
#elif L==3
	"Ver trabajos";
#elif L==4
	"Voir travaux";
#elif L==5
	"Ver trabajos";		// Okoteve traduccin
#elif L==6
	"Vedi lavori";
#elif L==7
	"View homework";	// Potrzebujesz tlumaczenie
#elif L==8
	"Ver trabalhos";
#endif

const char *Txt_Virtual_pendrive_of_THE_USER_X_has_been_removed =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"Se ha eliminado el malet&iacute;n de <strong>%s</strong>.";	// Necessita traduccio
#elif L==1
	"Virtual pendrive of <strong>%s</strong> has been removed.";	// Need bersetzung
#elif L==2
	"Virtual pendrive of <strong>%s</strong> has been removed.";
#elif L==3
	"Se ha eliminado el malet&iacute;n de <strong>%s</strong>.";
#elif L==4
	"Virtual pendrive of <strong>%s</strong> has been removed.";	// Besoin de traduction
#elif L==5
	"Se ha eliminado el malet&iacute;n de <strong>%s</strong>.";	// Okoteve traduccin
#elif L==6
	"La penna USB de <strong>%s</strong> &egrave; stata rimossa.";
#elif L==7
	"Virtual pendrive of <strong>%s</strong> has been removed.";	// Potrzebujesz tlumaczenie
#elif L==8
	"Virtual pendrive of <strong>%s</strong> has been removed.";	// Necessita de traduo
#endif

const char *Txt_Virtual_pendrives =
#if   L==0
	"Maletins";
#elif L==1
	"Aktentasche";
#elif L==2
	"Virtual pendrives";
#elif L==3
	"Maletines";
#elif L==4
	"Porte-documents";
#elif L==5
	"Maletines";		// Okoteve traduccin
#elif L==6
	"Penna USB";
#elif L==7
	"Virtual pendrives";	// Potrzebujesz tlumaczenie
#elif L==8
	"Pasta";
#endif

const char *Txt_Visible_by_BR_the_student =
#if   L==0
	"&iquest;Visible por<br />el estudiante?";	// Necessita traduccio
#elif L==1
	"Sichtbare durch<br />die Student?";
#elif L==2
	"Visible by<br />the student?";
#elif L==3
	"&iquest;Visible por<br />el estudiante?";
#elif L==4
	"Visible par<br />l'&eacute;tudiant/e?";
#elif L==5
	"&iquest;Visible por<br />el estudiante?";	// Okoteve traduccin
#elif L==6
	"Visibili dallo<br />studente?";
#elif L==7
	"Visible by<br />the student?";			// Potrzebujesz tlumaczenie
#elif L==8
	"Vis&iacute;vel pelo<br />estudante?";
#endif

const char *Txt_Visible_MALE_PLURAL =
#if   L==0
	"Visibles";
#elif L==1
	"Sichtbar";
#elif L==2
	"Visible";
#elif L==3
	"Visibles";
#elif L==4
	"Visibles";
#elif L==5
	"Visibles";	// Okoteve traduccin
#elif L==6
	"Visibili";
#elif L==7
	"Widoczne";
#elif L==8
	"Vis&iacute;veis";
#endif

const char *Txt_Visible_exams =
#if   L==0
	"Ex&agrave;mens visibles";
#elif L==1
	"Visible Pr%uuml;fungen";
#elif L==2
	"Visible exams";
#elif L==3
	"Ex&aacute;menes visibles";
#elif L==4
	"Examens visibles";
#elif L==5
	"Ex&aacute;menes visibles";	// Okoteve traduccin
#elif L==6
	"Esami visibili";
#elif L==7
	"Widoczne egzaminy";
#elif L==8
	"Exames vis&iacute;veis";
#endif

const char *Txt_Visible_survey =
#if   L==0
	"Encuesta visible";	// Necessita traduccio
#elif L==1
	"Sichtbare Umfrage";
#elif L==2
	"Visible survey";
#elif L==3
	"Encuesta visible";
#elif L==4
	"Sondage visible";
#elif L==5
	"Encuesta visible";	// Okoteve traduccin
#elif L==6
	"Sondaggio visibile";
#elif L==7
	"Visible survey";	// Potrzebujesz tlumaczenie
#elif L==8
	"Inqu&eacute;rito vis&iacute;vel";
#endif

const char *Txt_Visits_to_course =
#if   L==0
	"Accessos assignatura";
#elif L==1
	"Anmeldungen bei Kurs";
#elif L==2
	"Visits to course";
#elif L==3
	"Accesos asignatura";
#elif L==4
	"Visites mati&egrave;re";
#elif L==5
	"Accesos asignatura";	// Okoteve traduccin
#elif L==6
	"Accessi al corso";
#elif L==7
	"Wizyty w trakcie";
#elif L==8
	"Visitas disciplina";
#endif

const char *Txt_Web_page =
#if   L==0
	"P&agrave;gina web";
#elif L==1
	"Webseite";
#elif L==2
	"Web page";
#elif L==3
	"P&aacute;gina web";
#elif L==4
	"Page Web";
#elif L==5
	"P&aacute;gina web";	// Okoteve traduccin
#elif L==6
	"Pagina Web";
#elif L==7
	"Strona internetowa";
#elif L==8
	"P&aacute;gina da Web";
#endif

const char *Txt_Web_page_of_the_course =
#if   L==0
	"P&aacute;gina web de la asignatura";	// Necessita traduccio
#elif L==1
	"Webseite der Kurs";
#elif L==2
	"Web page of the course";
#elif L==3
	"P&aacute;gina web de la asignatura";
#elif L==4
	"Page Web de la mati&egrave;re";
#elif L==5
	"P&aacute;gina web de la asignatura";	// Okoteve traduccin
#elif L==6
	"Pagina Web del corso";
#elif L==7
	"Web page of the course";		// Potrzebujesz tlumaczenie
#elif L==8
	"P&aacute;gina da Web da disciplina";
#endif

const char *Txt_Web_social_network =
#if   L==0
	"Web / xarxa social";
#elif L==1
	"Web / Soziales Netzwerk";
#elif L==2
	"Web / social network";
#elif L==3
	"Web / red social";
#elif L==4
	"Web / r&eacute;seau social";
#elif L==5
	"Web / red social";	// Okoteve traduccin
#elif L==6
	"Web / social network";
#elif L==7
	"Web / sieci spo&lstrok;eczne";
#elif L==8
	"Web / rede social";
#endif

const char *Txt_Webs_social_networks =
#if   L==0
	"Webs / xarxes socials";
#elif L==1
	"Webs / Soziale Netzwerke";
#elif L==2
	"Webs / social networks";
#elif L==3
	"Webs / redes sociales";
#elif L==4
	"Webs / r&eacute;seaux sociaux";
#elif L==5
	"Webs / redes sociales";	// Okoteve traduccin
#elif L==6
	"Webs / reti sociali";
#elif L==7
	"Webs / sieci spo&lstrok;eczne";
#elif L==8
	"Webs / redes sociais";
#endif

const char *Txt_Week =
#if   L==0
	"Semana";	// Necessita traduccio
#elif L==1
	"Woche";
#elif L==2
	"Week";
#elif L==3
	"Semana";
#elif L==4
	"Semaine";
#elif L==5
	"Arapok&otilde;indy";
#elif L==6
	"Settimana";
#elif L==7
	"Week";		// Potrzebujesz tlumaczenie
#elif L==8
	"Semana";
#endif

const char *Txt_Welcome[Usr_NUM_SEXS] =
   {
#if   L==0	// Usr_SEX_UNKNOWN
	 "Benvingut/da."
#elif L==1
	 "Willkommen."
#elif L==2
	 "Welcome."
#elif L==3
	 "Bienvenido/a."
#elif L==4
	 "Bienvenu(e)."
#elif L==5
	 "Bienvenido/a."	// Okoteve traduccin
#elif L==6
	 "Benvenuto/a."
#elif L==7
	 "Witamy."
#elif L==8
	 "Bem-vindo/a."
#endif
	 ,
#if   L==0	// Usr_SEX_FEMALE
	 "Benvinguda."
#elif L==1
	 "Willkommen."
#elif L==2
	 "Welcome."
#elif L==3
	 "Bienvenida."
#elif L==4
	 "Bienvenue."
#elif L==5
	 "Bienvenida."	// Okoteve traduccin
#elif L==6
	 "Benvenuta."
#elif L==7
	 "Witamy."
#elif L==8
	 "Bem-vinda."
#endif
	 ,
#if   L==0	// Usr_SEX_MALE
	 "Benvingut."
#elif L==1
	 "Willkommen."
#elif L==2
	 "Welcome."
#elif L==3
	 "Bienvenido."
#elif L==4
	 "Bienvenu."
#elif L==5
	 "Bienvenido."	// Okoteve traduccin
#elif L==6
	 "Benvenuto."
#elif L==7
	 "Witamy."
#elif L==8
	 "Bem-vindo."
#endif
	 ,
#if   L==0	// Usr_SEX_ALL
	 "Benvingut/da."
#elif L==1
	 "Willkommen."
#elif L==2
	 "Welcome."
#elif L==3
	 "Bienvenido/a."
#elif L==4
	 "Bienvenu(e)."
#elif L==5
	 "Bienvenido/a."	// Okoteve traduccin
#elif L==6
	 "Benvenuto/a."
#elif L==7
	 "Witamy."
#elif L==8
	 "Bem-vindo/a."
#endif
	};

const char *Txt_Welcome_X[Usr_NUM_SEXS] =	// Warning: it is very important to include %s in the following sentences
	 {
#if   L==0	// Usr_SEX_UNKNOWN
	   "Benvingut/da, %s."
#elif L==1
	   "Willkommen, %s."
#elif L==2
	   "Welcome, %s."
#elif L==3
	   "Bienvenido/a, %s."
#elif L==4
	   "Bienvenu(e), %s."
#elif L==5
	   "Bienvenido/a, %s."	// Okoteve traduccin
#elif L==6
	   "Benvenuto/a, %s."
#elif L==7
	   "Witamy, %s."
#elif L==8
	   "Bem-vindo/a, %s."
#endif
	   ,
#if   L==0	// Usr_SEX_FEMALE
	   "Benvinguda, %s."
#elif L==1
	   "Willkommen, %s."
#elif L==2
	   "Welcome, %s."
#elif L==3
	   "Bienvenida, %s."
#elif L==4
	   "Bienvenue, %s."
#elif L==5
	   "Bienvenida, %s."	// Okoteve traduccin
#elif L==6
	   "Benvenuta, %s."
#elif L==7
	   "Witamy, %s."
#elif L==8
	   "Bem-vinda, %s."
#endif
	   ,
#if   L==0	// Usr_SEX_MALE
	   "Benvingut, %s."
#elif L==1
	   "Willkommen, %s."
#elif L==2
	   "Welcome, %s."
#elif L==3
	   "Bienvenido, %s."
#elif L==4
	   "Bienvenu, %s."
#elif L==5
	   "Bienvenido, %s."	// Okoteve traduccin
#elif L==6
	   "Benvenuto, %s."
#elif L==7
	   "Witamy, %s."
#elif L==8
	   "Bem-vindo, %s."
#endif
	   ,
#if   L==0	// Usr_SEX_ALL
	   "Benvingut/da, %s."
#elif L==1
	   "Willkommen, %s."
#elif L==2
	   "Welcome, %s."
#elif L==3
	   "Bienvenido/a, %s."
#elif L==4
	   "Bienvenu(e), %s."
#elif L==5
	   "Bienvenido/a, %s."	// Okoteve traduccin
#elif L==6
	   "Benvenuto/a, %s."
#elif L==7
	   "Witamy, %s."
#elif L==8
	   "Bem-vindo/a, %s."
#endif
	};

const char *Txt_Welcome_X_and_happy_birthday[Usr_NUM_SEXS] =	// Warning: it is very important to include %s in the following sentences
	 {
#if   L==0	// Usr_SEX_UNKNOWN
	   "Benvingut/da, %s, i feli&ccedil; aniversari!"
#elif L==1
	   "Willkommen, %s, und alles Gute zum Geburtstag!"
#elif L==2
	   "Welcome, %s, and happy birthday!"
#elif L==3
	   "Bienvenido/a, %s, &iexcl;y feliz cumplea&ntilde;os!"
#elif L==4
	   "Bienvenu(e), %s, et joyeux anniversaire!"
#elif L==5
	   "Bienvenido/a, %s, &iexcl;y feliz cumplea&ntilde;os!"// Okoteve traduccin
#elif L==6
	   "Benvenuto/a, %s, e buon compleanno!"
#elif L==7
	   "Witamy, %s, i wszystkiego najlepszego!"
#elif L==8
	   "Bem-vindo/a, %s, e feliz anivers&aacute;rio!"
#endif
	   ,
#if   L==0	// Usr_SEX_FEMALE
	   "Benvinguda, %s, i feli&ccedil; aniversari!"
#elif L==1
	   "Willkommen, %s, und alles Gute zum Geburtstag!"
#elif L==2
	   "Welcome, %s, and happy birthday!"
#elif L==3
	   "Bienvenida, %s, &iexcl;y feliz cumplea&ntilde;os!"
#elif L==4
	   "Bienvenue, %s, et joyeux anniversaire!"
#elif L==5
	   "Bienvenida, %s, &iexcl;y feliz cumplea&ntilde;os!"// Okoteve traduccin
#elif L==6
	   "Benvenuta, %s, e buon compleanno!"
#elif L==7
	   "Witamy, %s, i wszystkiego najlepszego!"
#elif L==8
	   "Bem-vinda, %s, e feliz anivers&aacute;rio!"
#endif
	   ,
#if   L==0	// Usr_SEX_MALE
	   "Benvingut, %s, i feli&ccedil; aniversari!"
#elif L==1
	   "Willkommen, %s, und alles Gute zum Geburtstag!"
#elif L==2
	   "Welcome, %s, and happy birthday!"
#elif L==3
	   "Bienvenido, %s, &iexcl;y feliz cumplea&ntilde;os!"
#elif L==4
	   "Bienvenu, %s, et joyeux anniversaire!"
#elif L==5
	   "Bienvenido, %s, &iexcl;y feliz cumplea&ntilde;os!"// Okoteve traduccin
#elif L==6
	   "Benvenuto, %s, e buon compleanno!"
#elif L==7
	   "Witamy, %s, i wszystkiego najlepszego!"
#elif L==8
	   "Bem-vindo, %s, e feliz anivers&aacute;rio!"
#endif
	   ,
#if   L==0	// Usr_SEX_ALL
	   "Benvingut/da, %s, i feli&ccedil; aniversari!"
#elif L==1
	   "Willkommen, %s, und alles Gute zum Geburtstag!"
#elif L==2
	   "Welcome, %s, and happy birthday!"
#elif L==3
	   "Bienvenido/a, %s, &iexcl;y feliz cumplea&ntilde;os!"
#elif L==4
	   "Bienvenu(e), %s, et joyeux anniversaire!"
#elif L==5
	   "Bienvenido/a, %s, &iexcl;y feliz cumplea&ntilde;os!"// Okoteve traduccin
#elif L==6
	   "Benvenuto/a, %s, e buon compleanno!"
#elif L==7
	   "Witamy, %s, i wszystkiego najlepszego!"
#elif L==8
	   "Bem-vindo/a, %s, e feliz anivers&aacute;rio!"
#endif
	};

const char *Txt_With_centres =
#if   L==0
	"Amb centres";
#elif L==1
	"Mit Lehrinstitute";
#elif L==2
	"With centres";
#elif L==3
	"Con centros";
#elif L==4
	"Avec centres";
#elif L==5
	"Con centros";	// Okoteve traduccin
#elif L==6
	"Con centri";
#elif L==7
	"Z centra";
#elif L==8
	"Com centros";
#endif

const char *Txt_With_courses =
#if   L==0
	"Amb assignatures";
#elif L==1
	"Mit Kursen";
#elif L==2
	"With courses";
#elif L==3
	"Con asignaturas";
#elif L==4
	"Avec mati&egrave;res";
#elif L==5
	"Con asignaturas";	// Okoteve traduccin
#elif L==6
	"Con corsi";
#elif L==7
	"Z kurs&oacute;w";
#elif L==8
	"Com disciplinas";
#endif

const char *Txt_With_degrees =
#if   L==0
	"Amb titulacions";
#elif L==1
	"Mit Studiengang";
#elif L==2
	"With degrees";
#elif L==3
	"Con titulaciones";
#elif L==4
	"Avec &eacute;tudes";
#elif L==5
	"Con titulaciones";	// Okoteve traduccin
#elif L==6
	"Con lauree";
#elif L==7
	"Z stopnie";
#elif L==8
	"Com titula&ccedil;oes";
#endif

const char *Txt_With_institutions =
#if   L==0
	"Amb institucions";
#elif L==1
	"Mit Hochschulen";
#elif L==2
	"With institutions";
#elif L==3
	"Con instituciones";
#elif L==4
	"Avec &eacute;tablissements";
#elif L==5
	"Con instituciones";	// Okoteve traduccin
#elif L==6
	"Con istituzioni";
#elif L==7
	"Z instytucje";
#elif L==8
	"Com institu&ccedil;&otilde;es";
#endif

const char *Txt_with_new_posts =
#if   L==0
	"con mensajes nuevos";	// Necessita traduccio
#elif L==1
	"mit neuen Beitr&auml;ge";
#elif L==2
	"with new posts";
#elif L==3
	"con mensajes nuevos";
#elif L==4
	"avec de nouveaux messages";
#elif L==5
	"con mensajes nuevos";	// Okoteve traduccin
#elif L==6
	"con nuovi post";
#elif L==7
	"with new posts";	// Potrzebujesz tlumaczenie
#elif L==8
	"com novas posts";
#endif

const char *Txt_With_students =
#if   L==0
	"Amb estudiants";
#elif L==1
	"Mit Teilnehmern";
#elif L==2
	"With students";
#elif L==3
	"Con estudiantes";
#elif L==4
	"Avec &eacute;tudiants";
#elif L==5
	"Con estudiantes";	// Okoteve traduccin
#elif L==6
	"Con studenti";
#elif L==7
	"Z student&oacute;w";
#elif L==8
	"Com estudantes";
#endif

const char *Txt_With_teachers =
#if   L==0
	"Amb professors";
#elif L==1
	"Mit Lehrkr&auml;ften";
#elif L==2
	"With teachers";
#elif L==3
	"Con profesores";
#elif L==4
	"Avec enseignants";
#elif L==5
	"Con profesores";	// Okoteve traduccin
#elif L==6
	"Con professori";
#elif L==7
	"Z nauczycielami";
#elif L==8
	"Com professores";
#endif

const char *Txt_works_ZIP_FILE_NAME =	// Name (without extension) of the ZIP file for works. Must be a valid filename in small letters!
#if   L==0
	"treballs";
#elif L==1
	"Hausarbeiten";
#elif L==2
	"homework";
#elif L==3
	"trabajos";
#elif L==4
	"travaux";
#elif L==5
	"tembiapo";
#elif L==6
	"lavori";
#elif L==7
	"homework";	// Potrzebujesz tlumaczenie
#elif L==8
	"trabalhos";
#endif

const char *Txt_Works_of_X_in_Y_removed =	// Warning: it is very important to include two %s in the following sentences
#if   L==0
	"Trabajos de <strong>%s</strong>"
	" en <strong>%s</strong> eliminados.";		// Necessita traduccio
#elif L==1
	"Die Hausarbeiten von <strong>%s</strong>"
	" in <strong>%s</strong> wurden entfernt.";
#elif L==2
	"The homework of <strong>%s</strong>"
	" in <strong>%s</strong> has been removed.";
#elif L==3
	"Trabajos de <strong>%s</strong>"
	" en <strong>%s</strong> eliminados.";
#elif L==4
	"Homework of <strong>%s</strong>"
	" in <strong>%s</strong> removed.";		// Besoin de traduction
#elif L==5
	"Trabajos de <strong>%s</strong>"
	" en <strong>%s</strong> eliminados.";		// Okoteve traduccin
#elif L==6
	"Lavori de <strong>%s</strong>"
	" in <strong>%s</strong> rimossi.";
#elif L==7
	"The homework of <strong>%s</strong>"
	" in <strong>%s</strong> has been removed.";	// Potrzebujesz tlumaczenie
#elif L==8
	"Homework of <strong>%s</strong>"
	" in <strong>%s</strong> removed.";		// Necessita de traduo
#endif

const char *Txt_Works_zone =
#if   L==0
	"Zona d'altres treballs";
#elif L==1
	"Hausarbeiten-Bereich";
#elif L==2
	"Homework area";
#elif L==3
	"Zona de otros trabajos";
#elif L==4
	"Zone de travaux";
#elif L==5
	"Zona de otros trabajos";	// Okoteve traduccin
#elif L==6
	"Area lavori";
#elif L==7
	"Homework area";		// Potrzebujesz tlumaczenie
#elif L==8
	"Zona de trabalhos";
#endif

const char *Txt_works_zone =
#if   L==0
	"zona de trabajos";	// Necessita traduccio
#elif L==1
	"Hausarbeiten-Bereich";
#elif L==2
	"homework area";
#elif L==3
	"zona de trabajos";
#elif L==4
	"zone de travaux";
#elif L==5
	"zona de trabajos";	// Okoteve traduccin
#elif L==6
	"area lavori";
#elif L==7
	"homework area";	// Potrzebujesz tlumaczenie
#elif L==8
	"zona de trabalhos";
#endif

const char *Txt_WriBRters =
#if   L==0
	"Escri-<br />tores";	// Necessita traduccio
#elif L==1
	"Schrift-<br />steller";
#elif L==2
	"Wri-<br />ters";
#elif L==3
	"Escri-<br />tores";
#elif L==4
	"&Eacute;cri-<br />vains";
#elif L==5
	"Escri-<br />tores";	// Okoteve traduccin
#elif L==6
	"Scri-<br />ttori";
#elif L==7
	"Wri-<br />ters";	// Potrzebujesz tlumaczenie
#elif L==8
	"Escri-<br />tores";
#endif

const char *Txt_Write_a_message_to_X =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"Escribir un mensaje a %s";		// Necessita traduccio
#elif L==1
	"Schreiben Sie %s eine Nachricht";
#elif L==2
	"Write a message to %s";
#elif L==3
	"Escribir un mensaje a %s";
#elif L==4
	"Write a message to %s";		// Besoin de traduction
#elif L==5
	"Escribir un mensaje a %s";		// Okoteve traduccin
#elif L==6
	"Scrivi un messaggio a %s";
#elif L==7
	"Write a message to %s";		// Potrzebujesz tlumaczenie
#elif L==8
	"Write a message to %s";		// Necessita de traduo
#endif

const char *Txt_WWW =		// World Wide Web
#if   L==0
	"WWW";
#elif L==1
	"WWW";
#elif L==2
	"WWW";
#elif L==3
	"WWW";
#elif L==4
	"WWW";
#elif L==5
	"WWW";
#elif L==6
	"WWW";
#elif L==7
	"WWW";
#elif L==8
	"WWW";
#endif

const char *Txt_XML_file =
#if   L==0
	"Fitxer XML";
#elif L==1
	"XML-Datei";
#elif L==2
	"XML file";
#elif L==3
	"Archivo XML";
#elif L==4
	"Fichier XML";
#elif L==5
	"Archivo XML";	// Okoteve traduccin
#elif L==6
	"File XML";
#elif L==7
	"Plik XML";
#elif L==8
	"Arquivo XML";
#endif

const char *Txt_XML_file_content =
#if   L==0
	"Contingut del fitxer XML";
#elif L==1
	"XML-Datei-Inhalt";
#elif L==2
	"XML file content";
#elif L==3
	"Contenido del archivo XML";
#elif L==4
	"Contenu du fichier XML";
#elif L==5
	"Contenido del archivo XML";	// Okoteve traduccin
#elif L==6
	"Contenuto di file XML";
#elif L==7
	"Zawarto&sacute;&cacute; pliku XML";
#elif L==8
	"Conte&uacute;do do arquivo XML";
#endif

const char *Txt_Year_OF_A_DEGREE =		// 1, 2, 3..., not 1984, 2011...
#if   L==0
	"Curs";
#elif L==1
	"Jahr";
#elif L==2
	"Year";
#elif L==3
	"Curso";
#elif L==4
	"Ann&eacute;e";
#elif L==5
	"Curso";	// Okoteve traduccin
#elif L==6
	"Anno";
#elif L==7
	"Year";		// Potrzebujesz tlumaczenie
#elif L==8
	"Ano";
#endif

const char *Txt_YEAR_OF_DEGREE[1 + Deg_MAX_YEARS_PER_DEGREE] =
   {
#if   L==0
	 "Opt./Libre conf."		// Necessita traduccio
#elif L==1
	 "Opt./Free choice"		// Need bersetzung
#elif L==2
	 "Opt./Free choice"
#elif L==3
	 "Opt./Libre conf."
#elif L==4
	 "Opt./Free choice"		// Besoin de traduction
#elif L==5
	 "Opt./Libre conf."		// Okoteve traduccin
#elif L==6
	 "Opz./Scelta libera"
#elif L==7
	 "Opt./Free choice"		// Potrzebujesz tlumaczenie
#elif L==8
	 "Opt./Free choice"		// Necessita de traduo
#endif
	 ,
#if   L==0
	 "1&ordm;"	// Necessita traduccio
#elif L==1
	 "1st"		// Need bersetzung
#elif L==2
	 "1st"
#elif L==3
	 "1&ordm;"
#elif L==4
	 "1st"		// Besoin de traduction
#elif L==5
	 "1&ordm;"	// Okoteve traduccin
#elif L==6
	 "Primo"
#elif L==7
	 "1st"		// Potrzebujesz tlumaczenie
#elif L==8
	 "1st"		// Necessita de traduo
#endif
	 ,
#if   L==0
	 "2&ordm;"	// Necessita traduccio
#elif L==1
	 "2nd"		// Need bersetzung
#elif L==2
	 "2nd"
#elif L==3
	 "2&ordm;"
#elif L==4
	 "2nd"		// Besoin de traduction
#elif L==5
	 "2&ordm;"	// Okoteve traduccin
#elif L==6
	 "Secondo"
#elif L==7
	 "2nd"		// Potrzebujesz tlumaczenie
#elif L==8
	 "2nd"		// Necessita de traduo
#endif
	 ,
#if   L==0
	 "3&ordm;"	// Necessita traduccio
#elif L==1
	 "3rd"
#elif L==2
	 "3rd"
#elif L==3
	 "3&ordm;"
#elif L==4
	 "3rd"		// Besoin de traduction
#elif L==5
	 "3&ordm;"	// Okoteve traduccin
#elif L==6
	 "Terzo"
#elif L==7
	 "3rd"		// Potrzebujesz tlumaczenie
#elif L==8
	 "3rd"		// Necessita de traduo
#endif
	 ,
#if   L==0
	 "4&ordm;"	// Necessita traduccio
#elif L==1
	 "4th"		// Need bersetzung
#elif L==2
	 "4th"
#elif L==3
	 "4&ordm;"
#elif L==4
	 "Quarto"
#elif L==5
	 "4&ordm;"	// Okoteve traduccin
#elif L==6
	 "4th"		// Bisogno di traduzione
#elif L==7
	 "4th"		// Potrzebujesz tlumaczenie
#elif L==8
	 "4th"		// Necessita de traduo
#endif
	 ,
#if   L==0
	 "5&ordm;"	// Necessita traduccio
#elif L==1
	 "5th"		// Need bersetzung
#elif L==2
	 "5th"
#elif L==3
	 "5&ordm;"
#elif L==4
	 "5th"		// Besoin de traduction
#elif L==5
	 "5&ordm;"	// Okoteve traduccin
#elif L==6
	 "Quinto"
#elif L==7
	 "5th"		// Potrzebujesz tlumaczenie
#elif L==8
	 "5th"		// Necessita de traduo
#endif
	 ,
#if   L==0
	 "6&ordm;"	// Necessita traduccio
#elif L==1
	 "6th"		// Need bersetzung
#elif L==2
	 "6th"
#elif L==3
	 "6&ordm;"
#elif L==4
	 "6th"		// Besoin de traduction
#elif L==5
	 "6&ordm;"	// Okoteve traduccin
#elif L==6
	 "Sesto"
#elif L==7
	 "6th"		// Potrzebujesz tlumaczenie
#elif L==8
	 "6th"		// Necessita de traduo
#endif
	 ,
#if   L==0
	 "7&ordm;"	// Necessita traduccio
#elif L==1
	 "7th"		// Need bersetzung
#elif L==2
	 "7th"
#elif L==3
	 "7&ordm;"
#elif L==4
	 "7th"		// Besoin de traduction
#elif L==5
	 "7&ordm;"	// Okoteve traduccin
#elif L==6
	 "Settimo"
#elif L==7
	 "7th"		// Potrzebujesz tlumaczenie
#elif L==8
	 "7th"		// Necessita de traduo
#endif
	 ,
#if   L==0
	 "8&ordm;"	// Necessita traduccio
#elif L==1
	 "8th"		// Need bersetzung
#elif L==2
	 "8th"
#elif L==3
	 "8&ordm;"
#elif L==4
	 "8th"		// Besoin de traduction
#elif L==5
	 "8&ordm;"	// Okoteve traduccin
#elif L==6
	 "Ottavo"
#elif L==7
	 "8th"		// Potrzebujesz tlumaczenie
#elif L==8
	 "8th"		// Necessita de traduo
#endif
	 ,
#if   L==0
	 "9&ordm;"	// Necessita traduccio
#elif L==1
	 "9th"		// Need bersetzung
#elif L==2
	 "9th"
#elif L==3
	 "9&ordm;"
#elif L==4
	 "9th"		// Besoin de traduction
#elif L==5
	 "9&ordm;"	// Okoteve traduccin
#elif L==6
	 "Nono"
#elif L==7
	 "9th"		// Potrzebujesz tlumaczenie
#elif L==8
	 "9th"		// Necessita de traduo
#endif
	 ,
#if   L==0
	 "10&ordm;"	// Necessita traduccio
#elif L==1
	 "10th"		// Need bersetzung
#elif L==2
	 "10th"
#elif L==3
	 "10&ordm;"
#elif L==4
	 "10th"		// Besoin de traduction
#elif L==5
	 "10&ordm;"	// Okoteve traduccin
#elif L==6
	 "Decimo"
#elif L==7
	 "10th"		// Potrzebujesz tlumaczenie
#elif L==8
	 "10th"		// Necessita de traduo
#endif
	 ,
#if   L==0
	 "11&ordm;"	// Necessita traduccio
#elif L==1
	 "11th"		// Need bersetzung
#elif L==2
	 "11th"
#elif L==3
	 "11&ordm;"
#elif L==4
	 "11th"		// Besoin de traduction
#elif L==5
	 "11&ordm;"	// Okoteve traduccin
#elif L==6
	 "Undicesimo"
#elif L==7
	 "11th"		// Potrzebujesz tlumaczenie
#elif L==8
	 "11th"		// Necessita de traduo
#endif
	 ,
#if   L==0
	 "12&ordm;"	// Necessita traduccio
#elif L==1
	 "12th"		// Need bersetzung
#elif L==2
	 "12th"
#elif L==3
	 "12&ordm;"
#elif L==4
	 "12th"		// Besoin de traduction
#elif L==5
	 "12&ordm;"	// Okoteve traduccin
#elif L==6
	 "Sesto"
#elif L==7
	 "12th"		// Potrzebujesz tlumaczenie
#elif L==8
	 "12th"		// Necessita de traduo
#endif
	};

const char *Txt_YES =
#if   L==0
	"S&Iacute;";
#elif L==1
	"JA";
#elif L==2
	"YES";
#elif L==3
	"S&Iacute;";
#elif L==4
	"OUI";
#elif L==5
	"HE&#7868;";
#elif L==6
	"SI";
#elif L==7
	"YES";		// Potrzebujesz tlumaczenie
#elif L==8
	"SIM";
#endif

const char *Txt_Yes =
#if   L==0
	"S&iacute;";
#elif L==1
	"Ja";
#elif L==2
	"Yes";
#elif L==3
	"S&iacute;";
#elif L==4
	"Oui";
#elif L==5
	"He&#7869;";
#elif L==6
	"S&igrave;";
#elif L==7
	"Yes";		// Potrzebujesz tlumaczenie
#elif L==8
	"Sim";
#endif

const char *Txt_Yesterday =
#if   L==0
	"Ahir";
#elif L==1
	"Gestern";
#elif L==2
	"Yesterday";
#elif L==3
	"Ayer";
#elif L==4
	"Hier";
#elif L==5
	"Kuehe";
#elif L==6
	"Ieri";
#elif L==7
	"Yesterday";	// Potrzebujesz tlumaczenie
#elif L==8
	"Ontem";
#endif

const char *Txt_You_already_answered_this_survey_before =
#if   L==0
	"Usted ya respondi&oacute; esta encuesta anteriormente.";	// Necessita traduccio
#elif L==1
	"Sie haben bereits beantwortet dieser Umfrage vor.";
#elif L==2
	"You already answered this survey before.";
#elif L==3
	"Usted ya respondi&oacute; esta encuesta anteriormente.";
#elif L==4
	"Vous avez d&eacute;j&agrave; r&eacute;pondu &agrave; ce sondage avant.";
#elif L==5
	"Usted ya respondi&oacute; esta encuesta anteriormente.";	// Okoteve traduccin
#elif L==6
	"Hai gi&agrave; risposto a questo sondaggio prima.";
#elif L==7
	"You already answered this survey before.";			// Potrzebujesz tlumaczenie
#elif L==8
	"Voc&ecirc; j&aacute; respondeu a esso inqu&eacute;rito antes.";
#endif

const char *Txt_You_are_LOGGED_as_X =	// Warning: it is very important to include two %s in the following sentences
#if   L==0
	"Ets identificat com a <strong>%s</strong>.";
#elif L==1
	"Sie sind %s als <strong>%s</strong>.";			// German translator doesn't understand this
#elif L==2
	"You are %s as <strong>%s</strong>.";
#elif L==3
	"Usted est&aacute; %s como <strong>%s</strong>.";
#elif L==4
	"Vous &ecirc;tes %s comme <strong>%s</strong>.";
#elif L==5
	"Usted est&aacute; %s como <strong>%s</strong>.";	// Okoteve traduccin
#elif L==6
	"Sei %s come <strong>%s</strong>.";
#elif L==7
	"You are %s as <strong>%s</strong>.";			// Potrzebujesz tlumaczenie
#elif L==8
	"Voc&ecirc; est&aacute; %s como <strong>%s</strong>.";
#endif

const char *Txt_You_are_not_enrolled_in_any_course[Usr_NUM_SEXS] =
	 {
#if   L==0	// Usr_SEX_UNKNOWN
	   "Usted no est&aacute; inscrito/a en ninguna asignatura."	// Necessita traduccio
#elif L==1
	   "Sie sind zu keiner Kurs eingeschrieben."
#elif L==2
	   "You are not enrolled in any course."
#elif L==3
	   "Usted no est&aacute; inscrito/a en ninguna asignatura."
#elif L==4
	   "Vous n'&ecirc;tes pas inscrit &agrave; une mati&egrave;re."
#elif L==5
	   "Usted no est&aacute; inscrito/a en ninguna asignatura."	// Okoteve traduccin
#elif L==6
	   "Non ti sei iscritto/a ad alcun corso."
#elif L==7
	   "You are not enrolled in any course."			// Potrzebujesz tlumaczenie
#elif L==8
	   "Voc&ecirc; n&atilde;o est&aacute; matriculado/a em nenhuma disciplina."
#endif
	   ,
#if   L==0	// Usr_SEX_FEMALE
	   "Usted no est&aacute; inscrita en ninguna asignatura."	// Necessita traduccio
#elif L==1
	   "Sie sind zu keiner Kurs eingeschrieben."
#elif L==2
	   "You are not enrolled in any course."
#elif L==3
	   "Usted no est&aacute; inscrita en ninguna asignatura."
#elif L==4
	   "Vous n'&ecirc;tes pas inscrit &agrave; une mati&egrave;re."
#elif L==5
	   "Usted no est&aacute; inscrita en ninguna asignatura."	// Okoteve traduccin
#elif L==6
	   "Non ti sei iscritta ad alcun corso."
#elif L==7
	   "You are not enrolled in any course."			// Potrzebujesz tlumaczenie
#elif L==8
	   "Voc&ecirc; n&atilde;o est&aacute; matriculada em nenhuma disciplina."
#endif
	   ,
#if   L==0	// Usr_SEX_MALE
	   "Usted no est&aacute; inscrito en ninguna asignatura."	// Necessita traduccio
#elif L==1
	   "Sie sind zu keiner Kurs eingeschrieben."
#elif L==2
	   "You are not enrolled in any course."
#elif L==3
	   "Usted no est&aacute; inscrito en ninguna asignatura."
#elif L==4
	   "Vous n'&ecirc;tes pas inscrit &agrave; une mati&egrave;re."
#elif L==5
	   "Usted no est&aacute; inscrito en ninguna asignatura."	// Okoteve traduccin
#elif L==6
	   "Non ti sei iscritto ad alcun corso."
#elif L==7
	   "You are not enrolled in any course."			// Potrzebujesz tlumaczenie
#elif L==8
	   "Voc&ecirc; n&atilde;o est&aacute; matriculado em nenhuma disciplina."
#endif
	   ,
#if   L==0	// Usr_SEX_ALL
	   "Usted no est&aacute; inscrito/a en ninguna asignatura."	// Necessita traduccio
#elif L==1
	   "Sie sind zu keiner Kurs eingeschrieben."
#elif L==2
	   "You are not enrolled in any course."
#elif L==3
	   "Usted no est&aacute; inscrito/a en ninguna asignatura."
#elif L==4
	   "Vous n'&ecirc;tes pas inscrit &agrave; une mati&egrave;re."
#elif L==5
	   "Usted no est&aacute; inscrito/a en ninguna asignatura."	// Okoteve traduccin
#elif L==6
	   "Non ti sei iscritto/a ad alcun corso."
#elif L==7
	   "You are not enrolled in any course."			// Potrzebujesz tlumaczenie
#elif L==8
	   "Voc&ecirc; n&atilde;o est&aacute; matriculado/a em nenhuma disciplina."
#endif
	};

const char *Txt_You_can_create_a_new_folder_inside_the_folder_X =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"Podeu crear una nova carpeta"
	" dins de la carpeta <strong>%s</strong>:";
#elif L==1
	"Sie k&ouml;nnen ein neues Verzeichnis"
	" im Verzeichnis <strong>%s</strong> anlegen:";
#elif L==2
	"You can create a new folder"
	" inside the folder <strong>%s</strong>:";
#elif L==3
	"Puede crear una nueva carpeta"
	" dentro de la carpeta <strong>%s</strong>:";
#elif L==4
	"Vous pouvez cr&eacute;er un nouveau r&eacute;pertoire"
	" dans le r&eacute;pertoire <strong>%s</strong>:";
#elif L==5
	"Puede crear una nueva carpeta"
	" dentro de la carpeta <strong>%s</strong>:";	// Okoteve traduccin
#elif L==6
	"&Egrave; possibile creare una nuova cartella"
	" all'interno della cartella <strong>%s</strong>:";
#elif L==7
	"Mo&zdot;esz utworzy&cacute; nowy folder"
	" wewn&aogon;trz folderu <strong>%s</strong>:";
#elif L==8
	"Voc&ecirc; pode criar um novo diret&oacute;rio"
	" dentro do diret&oacute;rio <strong>%s</strong>:";
#endif

const char *Txt_You_can_not_create_folders_files_or_links_here =
#if   L==0
	"No puede crear carpetas, archivos o enlaces aqu&iacute;.";	// Necessita traduccio
#elif L==1
	"You can not create folders, files or links here.";		// Need bersetzung
#elif L==2
	"You can not create folders, files or links here.";
#elif L==3
	"No puede crear carpetas, archivos o enlaces aqu&iacute;.";
#elif L==4
	"You can not create folders, files or links here.";		// Besoin de traduction
#elif L==5
	"No puede crear carpetas, archivos o enlaces aqu&iacute;.";	// Okoteve traduccin
#elif L==6
	"Non puoi creare cartelle, file o linki qui.";
#elif L==7
	"You can not create folders, files or links here.";		// Potrzebujesz tlumaczenie
#elif L==8
	"You can not create folders, files or links here.";		// Necessita de traduo
#endif

const char *Txt_You_can_not_create_folders_here =
#if   L==0
	"No puede crear carpetas aqu&iacute;.";	// Necessita traduccio
#elif L==1
	"You can not create folders here.";	// Need bersetzung
#elif L==2
	"You can not create folders here.";
#elif L==3
	"No puede crear carpetas aqu&iacute;.";
#elif L==4
	"You can not create folders here.";	// Besoin de traduction
#elif L==5
	"No puede crear carpetas aqu&iacute;.";	// Okoteve traduccin
#elif L==6
	"Non puoi creare cartelle qui.";
#elif L==7
	"You can not create folders here.";	// Potrzebujesz tlumaczenie
#elif L==8
	"You can not create folders here.";	// Necessita de traduo
#endif

const char *Txt_You_can_not_create_links_here =
#if   L==0
	"No puede crear enlaces aqu&iacute;.";	// Necessita traduccio
#elif L==1
	"You can not create links here.";	// Need bersetzung
#elif L==2
	"You can not create links here.";
#elif L==3
	"No puede crear enlaces aqu&iacute;.";
#elif L==4
	"You can not create links here.";	// Besoin de traduction
#elif L==5
	"No puede crear enlaces aqu&iacute;.";	// Okoteve traduccin
#elif L==6
	"Non puoi creare linki qui.";
#elif L==7
	"You can not create links here.";	// Potrzebujesz tlumaczenie
#elif L==8
	"You can not create links here.";	// Necessita de traduo
#endif

const char *Txt_You_can_not_delete_your_current_email =
#if   L==0
	"No puede eliminar su correo electr&oacute;nico actual.";	// Necessita traduccio
#elif L==1
	"You can not delete your current e-mail.";			// Need bersetzung
#elif L==2
	"You can not delete your current e-mail.";
#elif L==3
	"No puede eliminar su correo electr&oacute;nico actual.";
#elif L==4
	"You can not delete your current e-mail.";			// Besoin de traduction
#elif L==5
	"No puede eliminar su correo electr&oacute;nico actual.";	// Okoteve traduccin
#elif L==6
	"Non puoi cancellare il tuo e-mail attuale.";
#elif L==7
	"You can not delete your current e-mail.";			// Potrzebujesz tlumaczenie
#elif L==8
	"You can not delete your current e-mail.";			// Necessita de traduo
#endif

const char *Txt_You_can_not_delete_your_current_nickname =
#if   L==0
	"No puede eliminar su apodo actual.";		// Necessita traduccio
#elif L==1
	"You can not delete your current nickname.";	// Need bersetzung
#elif L==2
	"You can not delete your current nickname.";
#elif L==3
	"No puede eliminar su apodo actual.";
#elif L==4
	"You can not delete your current nickname.";	// Besoin de traduction
#elif L==5
	"No puede eliminar su apodo actual.";		// Okoteve traduccin
#elif L==6
	"Non puoi cancellare il tuo nome utente attuale.";
#elif L==7
	"You can not delete your current nickname.";	// Potrzebujesz tlumaczenie
#elif L==8
	"You can not delete your current nickname.";	// Necessita de traduo
#endif

const char *Txt_You_can_not_delete_this_ID =
#if   L==0
	"No puede eliminar este ID (DNI/c&eacute;dula).";	// Necessita traduccio
#elif L==1
	"You can not delete this ID.";				// Need bersetzung
#elif L==2
	"You can not delete this ID.";
#elif L==3
	"No puede eliminar este ID (DNI/c&eacute;dula).";
#elif L==4
	"You can not delete this ID.";				// Besoin de traduction
#elif L==5
	"No puede eliminar este ID (DNI/c&eacute;dula).";	// Okoteve traduccin
#elif L==6
	"Non puoi cancellare questo ID.";
#elif L==7
	"You can not delete this ID.";				// Potrzebujesz tlumaczenie
#elif L==8
	"You can not delete this ID.";				// Necessita de traduo
#endif

const char *Txt_You_can_not_disable_file_uploading_once_folders_have_been_created =
#if   L==0
	"No puede desactivar la subida de archivos"
	" una vez que se han creado carpetas.";	// Necessita traduccio
#elif L==1
	"Sie k&ouml;nnen den Upload nicht deaktivieren,"
	" wenn bereits Verzeichnisse angelegt worden sind.";
#elif L==2
	"You can not disable file uploading"
	" once folders have been created.";
#elif L==3
	"No puede desactivar la subida de archivos"
	" una vez que se han creado carpetas.";
#elif L==4
	"Vous ne pouvez pas d&eacute;sactiver le t&eacute;l&eacute;chargement de fichiers,"
	" une fois des r&eacute;pertoires ont &eacute;t&eacute; cr&eacute;&eacute;s.";
#elif L==5
	"No puede desactivar la subida de archivos"
	" una vez que se han creado carpetas.";	// Okoteve traduccin
#elif L==6
	"Non &egrave; possibile disabilitare il caricamento dei file"
	" una volta che le cartelle sono state create.";
#elif L==7
	"You can not disable file uploading"
	" once folders have been created.";	// Potrzebujesz tlumaczenie
#elif L==8
	"Voc&ecirc; n&atilde;o pode desativar o upload de arquivos,"
	" uma vez diret&oacute;rios foram criados.";
#endif

const char *Txt_You_can_not_leave_empty_intermediate_answers =
#if   L==0
	"No puede dejar respuestas intermedias sin rellenar.";	// Necessita traduccio
#elif L==1
	"You can not leave empty intermediate answers.";	// Need bersetzung
#elif L==2
	"You can not leave empty intermediate answers.";
#elif L==3
	"No puede dejar respuestas intermedias sin rellenar.";
#elif L==4
	"You can not leave empty intermediate answers.";	// Besoin de traduction
#elif L==5
	"No puede dejar respuestas intermedias sin rellenar.";	// Okoteve traduccin
#elif L==6
	"Non puoi lasciare vuote domande intermedie.";
#elif L==7
	"You can not leave empty intermediate answers.";	// Potrzebujesz tlumaczenie
#elif L==8
	"You can not leave empty intermediate answers.";	// Necessita de traduo
#endif

const char *Txt_You_can_not_leave_the_description_empty =
#if   L==0
	"No puede dejar la descripci&oacute;n vac&iacute;a.";	// Necessita traduccio
#elif L==1
	"You can not leave the description empty.";		// Need bersetzung
#elif L==2
	"You can not leave the description empty.";
#elif L==3
	"No puede dejar la descripci&oacute;n vac&iacute;a.";
#elif L==4
	"You can not leave the description empty.";		// Besoin de traduction
#elif L==5
	"No puede dejar la descripci&oacute;n vac&iacute;a.";	// Okoteve traduccin
#elif L==6
	"Non puoi lasciare vuota la descrizione.";
#elif L==7
	"You can not leave the description empty.";		// Potrzebujesz tlumaczenie
#elif L==8
	"You can not leave the description empty.";		// Necessita de traduo
#endif

const char *Txt_You_can_not_leave_the_image_empty =
#if   L==0
	"No puede dejar la imagen vac&iacute;a.";	// Necessita traduccio
#elif L==1
	"You can not leave the image empty.";		// Need bersetzung
#elif L==2
	"You can not leave the image empty.";
#elif L==3
	"No puede dejar la imagen vac&iacute;a.";
#elif L==4
	"You can not leave the image empty.";		// Besoin de traduction
#elif L==5
	"No puede dejar la imagen vac&iacute;a.";	// Okoteve traduccin
#elif L==6
	"Non puoi lasciare l'image vuota.";
#elif L==7
	"You can not leave the image empty.";		// Potrzebujesz tlumaczenie
#elif L==8
	"You can not leave the image empty.";		// Necessita de traduo
#endif

const char *Txt_You_can_not_leave_the_IP_address_empty =
#if   L==0
	"No puede dejar la direcci&oacute;n IP vac&iacute;a.";	// Necessita traduccio
#elif L==1
	"You can not leave the IP address empty.";		// Need bersetzung
#elif L==2
	"You can not leave the IP address empty.";
#elif L==3
	"No puede dejar la direcci&oacute;n IP vac&iacute;a.";
#elif L==4
	"You can not leave the IP address empty.";		// Besoin de traduction
#elif L==5
	"No puede dejar la direcci&oacute;n IP vac&iacute;a.";	// Okoteve traduccin
#elif L==6
	"Non puoi lasciare vuoto l'indirizzo IP.";
#elif L==7
	"You can not leave the IP address empty.";		// Potrzebujesz tlumaczenie
#elif L==8
	"You can not leave the IP address empty.";		// Necessita de traduo
#endif

const char *Txt_You_can_not_leave_the_logo_empty =
#if   L==0
	"No puede dejar el logo vac&iacute;o.";		// Necessita traduccio
#elif L==1
	"You can not leave the logo empty.";		// Need bersetzung
#elif L==2
	"You can not leave the logo empty.";
#elif L==3
	"No puede dejar el logo vac&iacute;o.";
#elif L==4
	"You can not leave the logo empty.";		// Besoin de traduction
#elif L==5
	"No puede dejar el logo vac&iacute;o.";		// Okoteve traduccin
#elif L==6
	"Non puoi lasciare vuoto il logo.";
#elif L==7
	"You can not leave the logo empty.";		// Potrzebujesz tlumaczenie
#elif L==8
	"You can not leave the logo empty.";		// Necessita de traduo
#endif

const char *Txt_You_can_not_leave_the_name_of_the_banner_X_empty =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"No puede dejar el nombre"
	" del banner <strong>%s</strong> vac&iacute;o.";	// Necessita traduccio
#elif L==1
	"You can not leave the name"
	" of the banner <strong>%s</strong> empty.";		// Need bersetzung
#elif L==2
	"You can not leave the name"
	" of the banner <strong>%s</strong> empty.";
#elif L==3
	"No puede dejar el nombre"
	" del banner <strong>%s</strong> vac&iacute;o.";
#elif L==4
	"You can not leave the name"
	" of the banner <strong>%s</strong> empty.";		// Besoin de traduction
#elif L==5
	"No puede dejar el nombre"
	" del banner <strong>%s</strong> vac&iacute;o.";	// Okoteve traduccin
#elif L==6
	"Non puoi lasciare il nome"
	" del banner <strong>%s</strong> vuoto.";
#elif L==7
	"You can not leave the name"
	" of the banner <strong>%s</strong> empty.";		// Potrzebujesz tlumaczenie
#elif L==8
	"You can not leave the name"
	" of the banner <strong>%s</strong> empty.";		// Necessita de traduo
#endif

const char *Txt_You_can_not_leave_the_name_of_the_centre_X_empty =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"No puede dejar el nombre"
	" del centro <strong>%s</strong> vac&iacute;o.";	// Necessita traduccio
#elif L==1
	"You can not leave the name"
	" of the centre <strong>%s</strong> empty.";		// Need bersetzung
#elif L==2
	"You can not leave the name"
	" of the centre <strong>%s</strong> empty.";
#elif L==3
	"No puede dejar el nombre"
	" del centro <strong>%s</strong> vac&iacute;o.";
#elif L==4
	"You can not leave the name"
	" of the centre <strong>%s</strong> empty.";		// Besoin de traduction
#elif L==5
	"No puede dejar el nombre"
	" del centro <strong>%s</strong> vac&iacute;o.";	// Okoteve traduccin
#elif L==6
	"Non puoi lasciare il nome"
	" del centro <strong>%s</strong> vuoto.";
#elif L==7
	"You can not leave the name"
	" of the centre <strong>%s</strong> empty.";		// Potrzebujesz tlumaczenie
#elif L==8
	"You can not leave the name"
	" of the centre <strong>%s</strong> empty.";		// Necessita de traduo
#endif

const char *Txt_You_can_not_leave_the_name_of_the_course_X_empty =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"No puede dejar el nombre"
	" de la asignatura <strong>%s</strong> vac&iacute;o.";	// Necessita traduccio
#elif L==1
	"You can not leave the name"
	" of the course <strong>%s</strong> empty.";		// Need bersetzung
#elif L==2
	"You can not leave the name"
	" of the course <strong>%s</strong> empty.";
#elif L==3
	"No puede dejar el nombre"
	" de la asignatura <strong>%s</strong> vac&iacute;o.";
#elif L==4
	"You can not leave the name"
	" of the course <strong>%s</strong> empty.";		// Besoin de traduction
#elif L==5
	"No puede dejar el nombre"
	" de la asignatura <strong>%s</strong> vac&iacute;o.";	// Okoteve traduccin
#elif L==6
	"Non puoi lasciare il nome"
	" del corso <strong>%s</strong> vuoto.";
#elif L==7
	"You can not leave the name"
	" of the course <strong>%s</strong> empty.";		// Potrzebujesz tlumaczenie
#elif L==8
	"You can not leave the name"
	" of the course <strong>%s</strong> empty.";		// Necessita de traduo
#endif

const char *Txt_You_can_not_leave_the_name_of_the_country_X_empty =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"No puede dejar el nombre"
	" del pa%iacute;s <strong>%s</strong> vac&iacute;o.";	// Necessita traduccio
#elif L==1
	"You can not leave the name"
	" of the country <strong>%s</strong> empty.";		// Need bersetzung
#elif L==2
	"You can not leave the name"
	" of the country <strong>%s</strong> empty.";
#elif L==3
	"No puede dejar el nombre"
	" del pa%iacute;s <strong>%s</strong> vac&iacute;o.";
#elif L==4
	"You can not leave the name"
	" of the country <strong>%s</strong> empty.";		// Besoin de traduction
#elif L==5
	"No puede dejar el nombre"
	" del pa%iacute;s <strong>%s</strong> vac&iacute;o.";	// Okoteve traduccin
#elif L==6
	"Non puoi lasciare il nome"
	" del paese <strong>%s</strong> vuoto.";
#elif L==7
	"You can not leave the name"
	" of the country <strong>%s</strong> empty.";		// Potrzebujesz tlumaczenie
#elif L==8
	"You can not leave the name"
	" of the country <strong>%s</strong> empty.";		// Necessita de traduo
#endif

const char *Txt_You_can_not_leave_the_name_of_the_degree_X_empty =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"No puede dejar el nombre"
	" de la titulaci&oacute;n <strong>%s</strong> vac&iacute;o.";	// Necessita traduccio
#elif L==1
	"You can not leave the name"
	" of the degree <strong>%s</strong> empty.";			// Need bersetzung
#elif L==2
	"You can not leave the name"
	" of the degree <strong>%s</strong> empty.";
#elif L==3
	"No puede dejar el nombre"
	" de la titulaci&oacute;n <strong>%s</strong> vac&iacute;o.";
#elif L==4
	"You can not leave the name"
	" of the degree <strong>%s</strong> empty.";			// Besoin de traduction
#elif L==5
	"No puede dejar el nombre"
	" de la titulaci&oacute;n <strong>%s</strong> vac&iacute;o.";	// Okoteve traduccin
#elif L==6
	"Non puoi lasciare il nome"
	" della laurea <strong>%s</strong> vuoto.";
#elif L==7
	"You can not leave the name"
	" of the degree <strong>%s</strong> empty.";			// Potrzebujesz tlumaczenie
#elif L==8
	"You can not leave the name"
	" of the degree <strong>%s</strong> empty.";			// Necessita de traduo
#endif

const char *Txt_You_can_not_leave_the_name_of_the_department_X_empty =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"No puede dejar el nombre"
	" del departamento <strong>%s</strong> vac&iacute;o.";	// Necessita traduccio
#elif L==1
	"You can not leave the name"
	" of the department <strong>%s</strong> empty.";	// Need bersetzung
#elif L==2
	"You can not leave the name"
	" of the department <strong>%s</strong> empty.";
#elif L==3
	"No puede dejar el nombre"
	" del departamento <strong>%s</strong> vac&iacute;o.";
#elif L==4
	"You can not leave the name"
	" of the department <strong>%s</strong> empty.";	// Besoin de traduction
#elif L==5
	"No puede dejar el nombre"
	" del departamento <strong>%s</strong> vac&iacute;o.";	// Okoteve traduccin
#elif L==6
	"Non puoi lasciare il nome"
	" del dipartimento <strong>%s</strong> vuoto.";
#elif L==7
	"You can not leave the name"
	" of the department <strong>%s</strong> empty.";	// Potrzebujesz tlumaczenie
#elif L==8
	"You can not leave the name"
	" of the department <strong>%s</strong> empty.";	// Necessita de traduo
#endif

const char *Txt_You_can_not_leave_the_name_of_the_field_X_empty =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"No puede dejar el nombre"
	" del campo <strong>%s</strong> vac&iacute;o.";	// Necessita traduccio
#elif L==1
	"You can not leave the name"
	" of the field <strong>%s</strong> empty.";	// Need bersetzung
#elif L==2
	"You can not leave the name"
	" of the field <strong>%s</strong> empty.";
#elif L==3
	"No puede dejar el nombre"
	" del campo <strong>%s</strong> vac&iacute;o.";
#elif L==4
	"You can not leave the name"
	" of the field <strong>%s</strong> empty.";	// Besoin de traduction
#elif L==5
	"No puede dejar el nombre"
	" del campo <strong>%s</strong> vac&iacute;o.";	// Okoteve traduccin
#elif L==6
	"Non puoi lasciare il nome"
	" del campo <strong>%s</strong> vuoto.";
#elif L==7
	"You can not leave the name"
	" of the field <strong>%s</strong> empty.";	// Potrzebujesz tlumaczenie
#elif L==8
	"You can not leave the name"
	" of the field <strong>%s</strong> empty.";	// Necessita de traduo
#endif

const char *Txt_You_can_not_leave_the_name_of_the_group_X_empty =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"No puede dejar el nombre"
	" del grupo <strong>%s</strong> vac&iacute;o.";	// Necessita traduccio
#elif L==1
	"You can not leave the name"
	" of the group <strong>%s</strong> empty.";	// Need bersetzung
#elif L==2
	"You can not leave the name"
	" of the group <strong>%s</strong> empty.";
#elif L==3
	"No puede dejar el nombre"
	" del grupo <strong>%s</strong> vac&iacute;o.";
#elif L==4
	"You can not leave the name"
	" of the group <strong>%s</strong> empty.";	// Besoin de traduction
#elif L==5
	"No puede dejar el nombre"
	" del grupo <strong>%s</strong> vac&iacute;o.";	// Okoteve traduccin
#elif L==6
	"Non puoi lasciare il nome"
	" del gruppo <strong>%s</strong> vuoto.";
#elif L==7
	"You can not leave the name"
	" of the group <strong>%s</strong> empty.";	// Potrzebujesz tlumaczenie
#elif L==8
	"You can not leave the name"
	" of the group <strong>%s</strong> empty.";	// Necessita de traduo
#endif

const char *Txt_You_can_not_leave_the_name_of_the_holiday_X_empty =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"No puede dejar el nombre"
	" de la festividad <strong>%s</strong> vac&iacute;o.";	// Necessita traduccio
#elif L==1
	"You can not leave the name"
	" of the holiday <strong>%s</strong> empty.";		// Need bersetzung
#elif L==2
	"You can not leave the name"
	" of the holiday <strong>%s</strong> empty.";
#elif L==3
	"No puede dejar el nombre"
	" de la festividad <strong>%s</strong> vac&iacute;o.";
#elif L==4
	"You can not leave the name"
	" of the holiday <strong>%s</strong> empty.";		// Besoin de traduction
#elif L==5
	"No puede dejar el nombre"
	" de la festividad <strong>%s</strong> vac&iacute;o.";	// Okoteve traduccin
#elif L==6
	"Non puoi lasciare il nome"
	" della festivit&agrave; <strong>%s</strong> vuoto.";
#elif L==7
	"You can not leave the name"
	" of the holiday <strong>%s</strong> empty.";		// Potrzebujesz tlumaczenie
#elif L==8
	"You can not leave the name"
	" of the holiday <strong>%s</strong> empty.";		// Necessita de traduo
#endif

const char *Txt_You_can_not_leave_the_name_of_the_institution_X_empty =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"No puede dejar el nombre"
	" de la instituci&oacute;n <strong>%s</strong> vac&iacute;o.";	// Necessita traduccio
#elif L==1
	"You can not leave the name"
	" of the institution <strong>%s</strong> empty.";		// Need bersetzung
#elif L==2
	"You can not leave the name"
	" of the institution <strong>%s</strong> empty.";
#elif L==3
	"No puede dejar el nombre"
	" de la instituci&oacute;n <strong>%s</strong> vac&iacute;o.";
#elif L==4
	"You can not leave the name"
	" of the institution <strong>%s</strong> empty.";		// Besoin de traduction
#elif L==5
	"No puede dejar el nombre"
	" de la instituci&oacute;n <strong>%s</strong> vac&iacute;o.";	// Okoteve traduccin
#elif L==6
	"Non puoi lasciare il nome"
	" dell'istituzione <strong>%s</strong> vuoto.";
#elif L==7
	"You can not leave the name"
	" of the institution <strong>%s</strong> empty.";		// Potrzebujesz tlumaczenie
#elif L==8
	"You can not leave the name"
	" of the institution <strong>%s</strong> empty.";		// Necessita de traduo
#endif

const char *Txt_You_can_not_leave_the_name_of_the_link_X_empty =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"No puede dejar el nombre"
	" del enlace <strong>%s</strong> vac&iacute;o.";	// Necessita traduccio
#elif L==1
	"You can not leave the name"
	" of the link <strong>%s</strong> empty.";		// Need bersetzung
#elif L==2
	"You can not leave the name"
	" of the link <strong>%s</strong> empty.";
#elif L==3
	"No puede dejar el nombre"
	" del enlace <strong>%s</strong> vac&iacute;o.";
#elif L==4
	"You can not leave the name"
	" of the link <strong>%s</strong> empty.";		// Besoin de traduction
#elif L==5
	"No puede dejar el nombre"
	" del enlace <strong>%s</strong> vac&iacute;o.";	// Okoteve traduccin
#elif L==6
	"Non puoi lasciare il nome"
	" del link <strong>%s</strong> vuoto.";
#elif L==7
	"You can not leave the name"
	" of the link <strong>%s</strong> empty.";		// Potrzebujesz tlumaczenie
#elif L==8
	"You can not leave the name"
	" of the link <strong>%s</strong> empty.";		// Necessita de traduo
#endif

const char *Txt_You_can_not_leave_the_name_of_the_mail_domain_X_empty =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"No puede dejar el nombre "
	"del dominio de correo <strong>%s</strong> vac&iacute;o.";	// Necessita traduccio
#elif L==1
	"You can not leave the name"
	" of the mail domain <strong>%s</strong> empty.";		// Need bersetzung
#elif L==2
	"You can not leave the name"
	" of the mail domain <strong>%s</strong> empty.";
#elif L==3
	"No puede dejar el nombre"
	" del dominio de correo <strong>%s</strong> vac&iacute;o.";
#elif L==4
	"You can not leave the name"
	" of the mail domain <strong>%s</strong> empty.";		// Besoin de traduction
#elif L==5
	"No puede dejar el nombre"
	" del dominio de correo <strong>%s</strong> vac&iacute;o.";	// Okoteve traduccin
#elif L==6
	"Non puoi lasciare il nome"
	" del campo mail <strong>%s</strong> vuoto.";
#elif L==7
	"You can not leave the name"
	" of the mail domain <strong>%s</strong> empty.";		// Potrzebujesz tlumaczenie
#elif L==8
	"You can not leave the name"
	" of the mail domain <strong>%s</strong> empty.";		// Necessita de traduo
#endif

const char *Txt_You_can_not_leave_the_name_of_the_place_X_empty =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"No puede dejar el nombre"
	" del lugar <strong>%s</strong> vac&iacute;o.";	// Necessita traduccio
#elif L==1
	"You can not leave the name"
	" of the place <strong>%s</strong> empty.";	// Need bersetzung
#elif L==2
	"You can not leave the name"
	" of the place <strong>%s</strong> empty.";
#elif L==3
	"No puede dejar el nombre"
	" del lugar <strong>%s</strong> vac&iacute;o.";
#elif L==4
	"You can not leave the name"
	" of the place <strong>%s</strong> empty.";	// Besoin de traduction
#elif L==5
	"No puede dejar el nombre"
	" del lugar <strong>%s</strong> vac&iacute;o.";	// Okoteve traduccin
#elif L==6
	"Non puoi lasciare il nome"
	" del luogo <strong>%s</strong> vuoto.";
#elif L==7
	"You can not leave the name"
	" of the place <strong>%s</strong> empty.";	// Potrzebujesz tlumaczenie
#elif L==8
	"You can not leave the name"
	" of the place <strong>%s</strong> empty.";	// Necessita de traduo
#endif

const char *Txt_You_can_not_leave_the_name_of_the_plugin_X_empty =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"No puede dejar el nombre"
	" del complemento <strong>%s</strong> vac&iacute;o.";	// Necessita traduccio
#elif L==1
	"You can not leave the name"
	" of the plugin <strong>%s</strong> empty.";		// Need bersetzung
#elif L==2
	"You can not leave the name"
	" of the plugin <strong>%s</strong> empty.";
#elif L==3
	"No puede dejar el nombre"
	" del complemento <strong>%s</strong> vac&iacute;o.";
#elif L==4
	"You can not leave the name"
	" of the plugin <strong>%s</strong> empty.";		// Besoin de traduction
#elif L==5
	"No puede dejar el nombre"
	" del complemento <strong>%s</strong> vac&iacute;o.";	// Okoteve traduccin
#elif L==6
	"Non puoi lasciare il nome"
	" del plugin <strong>%s</strong> vuoto.";
#elif L==7
	"You can not leave the name"
	" of the plugin <strong>%s</strong> empty.";		// Potrzebujesz tlumaczenie
#elif L==8
	"You can not leave the name"
	" of the plugin <strong>%s</strong> empty.";		// Necessita de traduo
#endif

const char *Txt_You_can_not_leave_the_name_of_the_tag_X_empty =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"No puede dejar el nombre"
	" del descriptor <strong>%s</strong> vac&iacute;o.";		// Necessita traduccio
#elif L==1
	"You can not leave the name"
	" of the tag <strong>%s</strong> empty.";			// Need bersetzung
#elif L==2
	"You can not leave the name"
	" of the tag <strong>%s</strong> empty.";
#elif L==3
	"No puede dejar el nombre"
	" del descriptor <strong>%s</strong> vac&iacute;o.";
#elif L==4
	"You can not leave the name"
	" of the tag <strong>%s</strong> empty.";			// Besoin de traduction
#elif L==5
	"No puede dejar el nombre"
	" del descriptor <strong>%s</strong> vac&iacute;o.";		// Okoteve traduccin
#elif L==6
	"Non puoi lasciare il nome"
	" dell'etichetta <strong>%s</strong> vuoto.";
#elif L==7
	"You can not leave the name"
	" of the tag <strong>%s</strong> empty.";			// Potrzebujesz tlumaczenie
#elif L==8
	"You can not leave the name"
	" of the tag <strong>%s</strong> empty.";			// Necessita de traduo
#endif

const char *Txt_You_can_not_leave_the_name_of_the_type_of_degree_X_empty =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"No puede dejar el nombre"
	" del tipo de titulaci&oacute;n <strong>%s</strong> vac&iacute;o.";	// Necessita traduccio
#elif L==1
	"You can not leave the name"
	" of the type of degree <strong>%s</strong> empty.";			// Need bersetzung
#elif L==2
	"You can not leave the name"
	" of the type of degree <strong>%s</strong> empty.";
#elif L==3
	"No puede dejar el nombre"
	" del tipo de titulaci&oacute;n <strong>%s</strong> vac&iacute;o.";
#elif L==4
	"You can not leave the name"
	" of the type of degree <strong>%s</strong> empty.";			// Besoin de traduction
#elif L==5
	"No puede dejar el nombre"
	" del tipo de titulaci&oacute;n <strong>%s</strong> vac&iacute;o.";	// Okoteve traduccin
#elif L==6
	"Non puoi lasciare il nome"
	" del tipo di laurea <strong>%s</strong> vuoto.";
#elif L==7
	"You can not leave the name"
	" of the type of degree <strong>%s</strong> empty.";			// Potrzebujesz tlumaczenie
#elif L==8
	"You can not leave the name"
	" of the type of degree <strong>%s</strong> empty.";			// Necessita de traduo
#endif

const char *Txt_You_can_not_leave_the_name_of_the_type_of_group_X_empty =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"No puede dejar el nombre"
	" del tipo de grupo <strong>%s</strong> vac&iacute;o.";	// Necessita traduccio
#elif L==1
	"You can not leave the name"
	" of the type of group <strong>%s</strong> empty.";	// Need bersetzung
#elif L==2
	"You can not leave the name"
	" of the type of group <strong>%s</strong> empty.";
#elif L==3
	"No puede dejar el nombre"
	" del tipo de grupo <strong>%s</strong> vac&iacute;o.";
#elif L==4
	"You can not leave the name"
	" of the type of group <strong>%s</strong> empty.";	// Besoin de traduction
#elif L==5
	"No puede dejar el nombre"
	" del tipo de grupo <strong>%s</strong> vac&iacute;o.";	// Okoteve traduccin
#elif L==6
	"Non puoi lasciare il nome"
	" del tipo di gruppo <strong>%s</strong> vuoto.";
#elif L==7
	"You can not leave the name"
	" of the type of group <strong>%s</strong> empty.";	// Potrzebujesz tlumaczenie
#elif L==8
	"You can not leave the name"
	" of the type of group <strong>%s</strong> empty.";	// Necessita de traduo
#endif

const char *Txt_You_can_not_leave_the_URL_empty =
#if   L==0
	"No puede dejar el URL vac&iacute;o.";		// Necessita traduccio
#elif L==1
	"You can not leave the URL empty.";		// Need bersetzung
#elif L==2
	"You can not leave the URL empty.";
#elif L==3
	"No puede dejar el URL vac&iacute;o.";
#elif L==4
	"You can not leave the URL empty.";		// Besoin de traduction
#elif L==5
	"No puede dejar el URL vac&iacute;o.";		// Okoteve traduccin
#elif L==6
	"Non puoi lasciare l'URL vuoto.";
#elif L==7
	"You can not leave the URL empty.";		// Potrzebujesz tlumaczenie
#elif L==8
	"You can not leave the URL empty.";		// Necessita de traduo
#endif

const char *Txt_You_can_not_leave_the_web_address_empty =
#if   L==0
	"No puede dejar la direcci&oacute;n web vac&iacute;a.";	// Necessita traduccio
#elif L==1
	"You can not leave the web address empty.";		// Need bersetzung
#elif L==2
	"You can not leave the web address empty.";
#elif L==3
	"No puede dejar la direcci&oacute;n web vac&iacute;a.";
#elif L==4
	"You can not leave the web address empty.";		// Besoin de traduction
#elif L==5
	"No puede dejar la direcci&oacute;n web vac&iacute;a.";	// Okoteve traduccin
#elif L==6
	"Non puoi lasciare l'indirizzo web vuoto.";
#elif L==7
	"You can not leave the web address empty.";		// Potrzebujesz tlumaczenie
#elif L==8
	"You can not leave the web address empty.";		// Necessita de traduo
#endif

const char *Txt_You_can_not_make_a_new_test_in_the_course_X_until_TIME_Y =	// Warning: it is very important to include %s and three %02u in the following sentences
#if   L==0
	"No puede hacer un nuevo test en la asignatura <strong>%s</strong>"
	" hasta las %02uh%02u'%02u&quot;";		// Necessita traduccio
#elif L==1
	"You can not make a new test in the course <strong>%s</strong>"
	" until %02uh%02u'%02u&quot;";			// Need bersetzung
#elif L==2
	"You can not make a new test in the course <strong>%s</strong>"
	" until %02uh%02u'%02u&quot;";
#elif L==3
	"No puede hacer un nuevo test en la asignatura <strong>%s</strong>"
	" hasta las %02uh%02u'%02u&quot;";
#elif L==4
	"You can not make a new test in the course <strong>%s</strong>"
	" until %02uh%02u'%02u&quot;";			// Besoin de traduction
#elif L==5
	"No puede hacer un nuevo test en la asignatura <strong>%s</strong>"
	" hasta las %02uh%02u'%02u&quot;";		// Okoteve traduccin
#elif L==6
	"Non puoi fare un nuovo test nel corso <strong>%s</strong>"
	" fino al %02uh%02u'%02u&quot;";
#elif L==7
	"You can not make a new test in the course <strong>%s</strong>"
	" until %02uh%02u'%02u&quot;";			// Potrzebujesz tlumaczenie
#elif L==8
	"You can not make a new test in the course <strong>%s</strong>"
	" until %02uh%02u'%02u&quot;";			// Necessita de traduo
#endif

const char *Txt_You_can_not_make_a_new_test_in_the_course_X_until_TIME_Y_on_DATE_Z =	// Warning: it is very important to include %s, five %02u and %04u in the following sentences
#if   L==0
	"No puede hacer un nuevo test en la asignatura <strong>%s</strong>"
	" hasta las %02uh%02u'%02u&quot; del %02u/%02u/%04u";		// Necessita traduccio
#elif L==1
	"You can not make a new test in the course <strong>%s</strong>"
	" until %02uh%02u'%02u&quot; on %02u/%02u/%04u";		// Need bersetzung
#elif L==2
	"You can not make a new test in the course <strong>%s</strong>"
	" until %02uh%02u'%02u&quot; on %02u/%02u/%04u";
#elif L==3
	"No puede hacer un nuevo test en la asignatura <strong>%s</strong>"
	" hasta las %02uh%02u'%02u&quot; del %02u/%02u/%04u";
#elif L==4
	"You can not make a new test in the course <strong>%s</strong>"
	" until %02uh%02u'%02u&quot; on %02u/%02u/%04u";		// Besoin de traduction
#elif L==5
	"No puede hacer un nuevo test en la asignatura <strong>%s</strong>"
	" hasta las %02uh%02u'%02u&quot; del %02u/%02u/%04u";		// Okoteve traduccin
#elif L==6
	"Non puoi fare un nuovo test nel corso <strong>%s</strong>"
	" fino al %02uh%02u'%02u&quot; del %02u/%02u/%04u";
#elif L==7
	"You can not make a new test in the course <strong>%s</strong>"
	" until %02uh%02u'%02u&quot; on %02u/%02u/%04u";		// Potrzebujesz tlumaczenie
#elif L==8
	"You can not make a new test in the course <strong>%s</strong>"
	" until %02uh%02u'%02u&quot; on %02u/%02u/%04u";		// Necessita de traduo
#endif

const char *Txt_You_can_not_paste_file_or_folder_here =
#if   L==0
	"No puede pegar un archivo o carpeta aqu&iacute;.";		// Necessita traduccio
#elif L==1
	"You can not paste file or folder here.";			// Need bersetzung
#elif L==2
	"You can not paste file or folder here.";
#elif L==3
	"No puede pegar un archivo o carpeta aqu&iacute;.";
#elif L==4
	"You can not paste file or folder here.";			// Besoin de traduction
#elif L==5
	"No puede pegar un archivo o carpeta aqu&iacute;.";		// Okoteve traduccin
#elif L==6
	"Non puoi copiare file o cartella qui.";
#elif L==7
	"You can not paste file or folder here.";			// Potrzebujesz tlumaczenie
#elif L==8
	"You can not paste file or folder here.";			// Necessita de traduo
#endif

const char *Txt_You_can_not_remove_this_file_or_link =
#if   L==0
	"No puede eliminar este archivo o enlace.";		// Necessita traduccio
#elif L==1
	"You can not remove this file or link.";		// Need bersetzung
#elif L==2
	"You can not remove this file or link.";
#elif L==3
	"No puede eliminar este archivo o enlace.";
#elif L==4
	"You can not remove this file or link.";		// Besoin de traduction
#elif L==5
	"No puede eliminar este archivo o enlace.";		// Okoteve traduccin
#elif L==6
	"Non puoi rimuovere questo file o link.";
#elif L==7
	"You can not remove this file or link.";		// Potrzebujesz tlumaczenie
#elif L==8
	"You can not remove this file or link.";		// Necessita de traduo
#endif

const char *Txt_You_can_not_remove_this_folder =
#if   L==0
	"No puede eliminar esta carpeta.";		// Necessita traduccio
#elif L==1
	"You can not remove this folder.";		// Need bersetzung
#elif L==2
	"You can not remove this folder.";
#elif L==3
	"No puede eliminar esta carpeta.";
#elif L==4
	"You can not remove this folder.";		// Besoin de traduction
#elif L==5
	"No puede eliminar esta carpeta.";		// Okoteve traduccin
#elif L==6
	"Non puoi rimuovere questa cartella.";
#elif L==7
	"You can not remove this folder.";		// Potrzebujesz tlumaczenie
#elif L==8
	"You can not remove this folder.";		// Necessita de traduo
#endif

const char *Txt_You_can_not_remove_a_country_with_institutions_or_users =
#if   L==0
	"No puede eliminar un pa&iacute;s con instituciones o usuarios.";	// Necessita traduccio
#elif L==1
	"You can not remove a country with institutions or users.";		// Need bersetzung
#elif L==2
	"You can not remove a country with institutions or users.";
#elif L==3
	"No puede eliminar un pa&iacute;s con instituciones o usuarios.";
#elif L==4
	"You can not remove a country with instituciones or users.";		// Besoin de traduction
#elif L==5
	"No puede eliminar un pa&iacute;s con instituciones o usuarios.";	// Okoteve traduccin
#elif L==6
	"Non puoi rimuovere un paese con istituzioni o utenti.";
#elif L==7
	"You can not remove a country with instituciones or users.";		// Potrzebujesz tlumaczenie
#elif L==8
	"You can not remove a country with instituciones or users.";		// Necessita de traduo
#endif

const char *Txt_You_can_not_rename_this_folder =
#if   L==0
	"No puede cambiar el nombre de esta carpeta.";	// Necessita traduccio
#elif L==1
	"You can not rename this folder.";		// Need bersetzung
#elif L==2
	"You can not rename this folder.";
#elif L==3
	"No puede cambiar el nombre de esta carpeta.";
#elif L==4
	"You can not rename this folder.";		// Besoin de traduction
#elif L==5
	"No puede cambiar el nombre de esta carpeta.";	// Okoteve traduccin
#elif L==6
	"Non puoi rinominare questa cartella.";
#elif L==7
	"You can not rename this folder.";		// Potrzebujesz tlumaczenie
#elif L==8
	"You can not rename this folder.";		// Necessita de traduo
#endif

const char *Txt_You_can_not_send_a_message_to_so_many_recipients_ =
#if   L==0
	"No puede enviar un mensaje a tantsimos destinatarios."
	" Es mejor usar los foros.";		// Necessita traduccio
#elif L==1
	"You can not send a message to so many recipients."
	" Better use the forums.";		// Need bersetzung
#elif L==2
	"You can not send a message to so many recipients."
	" Better use the forums.";
#elif L==3
	"No puede enviar un mensaje a tantsimos destinatarios."
	" Es mejor usar los foros.";
#elif L==4
	"You can not send a message to so many recipients."
	" Better use the forums.";		// Besoin de traduction
#elif L==5
	"No puede enviar un mensaje a tantsimos destinatarios."
	" Es mejor usar los foros.";		// Okoteve traduccin
#elif L==6
	"Non puoi inviare un messaggio a cos tanti destinatari."
	" Meglio usare il forum.";
#elif L==7
	"You can not send a message to so many recipients."
	" Better use the forums.";		// Potrzebujesz tlumaczenie
#elif L==8
	"You can not send a message to so many recipients."
	" Better use the forums.";		// Necessita de traduo
#endif

const char *Txt_You_can_only_perform_X_further_actions_ =	// Warning: it is very important to include %u in the following sentences
#if   L==0
	"S&oacute;lo podr&aacute; realizar %u acciones m&aacute;s"
	" si no env&iacute;a su fotograf&iacute;a.";		// Necessita traduccio
#elif L==1
	"You can only perform %u further actions"
	" if you don't send your photo.";			// Need bersetzung
#elif L==2
	"You can only perform %u further actions"
	" if you don't send your photo.";
#elif L==3
	"S&oacute;lo podr&aacute; realizar %u acciones m&aacute;s"
	" si no env&iacute;a su fotograf&iacute;a.";
#elif L==4
	"You can only perform %u further actions"
	" if you don't send your photo.";			// Besoin de traduction
#elif L==5
	"S&oacute;lo podr&aacute; realizar %u acciones m&aacute;s"
	" si no env&iacute;a su fotograf&iacute;a.";		// Okoteve traduccin
#elif L==6
	"Puoi solo realizzare %u massime azioni"
	" se non hai inviato la tua foto.";
#elif L==7
	"You can only perform %u further actions"
	" if you don't send your photo.";			// Potrzebujesz tlumaczenie
#elif L==8
	"You can only perform %u further actions"
	" if you don't send your photo.";			// Necessita de traduo
#endif

const char *Txt_You_can_only_receive_email_notifications_if_ =
#if   L==0
	"S&oacute;lo podr&aacute; recibir notificaciones por correo electr&oacute;nico"
	" si su direcci&oacute;n de correo termina en uno de los dominios"
	" listados en Titulaci&oacute;n &gt; Dominios correo.";	// Necessita traduccio
#elif L==1
	"You can only receive email notifications"
	" if your email ends in one of the domains"
	" listed in Degree &gt; Mail domains.";			// Need bersetzung
#elif L==2
	"You can only receive email notifications"
	" if your email ends in one of the domains"
	" listed in Degree &gt; Mail domains.";
#elif L==3
	"S&oacute;lo podr&aacute; recibir notificaciones por correo electr&oacute;nico"
	" si su direcci&oacute;n de correo termina en uno de los dominios"
	" listados en Titulaci&oacute;n &gt; Dominios correo.";
#elif L==4
	"You can only receive email notifications"
	" if your email ends in one of the domains"
	" listed in Degree &gt; Mail domains.";			// Besoin de traduction
#elif L==5
	"S&oacute;lo podr&aacute; recibir notificaciones por correo electr&oacute;nico"
	" si su direcci&oacute;n de correo termina en uno de los dominios"
	" listados en Titulaci&oacute;n &gt; Dominios correo.";	// Okoteve traduccin
#elif L==6
	"Puoi solo ricevere e-mail di notifica"
	" se la tua e-mail termina in uno dei campi"
	" listati in Lauree &gt; Campo mail.";
#elif L==7
	"You can only receive email notifications"
	" if your email ends in one of the domains"
	" listed in Degree &gt; Mail domains.";			// Potrzebujesz tlumaczenie
#elif L==8
	"You can only receive email notifications"
	" if your email ends in one of the domains"
	" listed in Degree &gt; Mail domains.";			// Necessita de traduo
#endif

const char *Txt_You_can_register_voluntarily_in_one_group_of_type_X =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"Puede apuntarse voluntariamente a un grupo de tipo %s.";	// Necessita traduccio
#elif L==1
	"You can register voluntarily in one group of type %s.";	// Need bersetzung
#elif L==2
	"You can register voluntarily in one group of type %s.";
#elif L==3
	"Puede apuntarse voluntariamente a un grupo de tipo %s.";
#elif L==4
	"You can register voluntarily in one group of type %s.";	// Besoin de traduction
#elif L==5
	"Puede apuntarse voluntariamente a un grupo de tipo %s.";	// Okoteve traduccin
#elif L==6
	"Ti puoi registrare volontariamente in uno dei gruppi di tipo %s.";
#elif L==7
	"You can register voluntarily in one group of type %s.";	// Potrzebujesz tlumaczenie
#elif L==8
	"You can register voluntarily in one group of type %s.";	// Necessita de traduo
#endif

const char *Txt_You_can_register_voluntarily_in_one_or_more_groups_of_type_X =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"Puede apuntarse voluntariamente"
	" a uno o a varios grupos de tipo %s.";		// Necessita traduccio
#elif L==1
	"You can register voluntarily"
	" in one or more groups of type %s.";		// Need bersetzung
#elif L==2
	"You can register voluntarily"
	" in one or more groups of type %s.";
#elif L==3
	"Puede apuntarse voluntariamente"
	" a uno o a varios grupos de tipo %s.";
#elif L==4
	"You can register voluntarily"
	" in one or more groups of type %s.";		// Besoin de traduction
#elif L==5
	"Puede apuntarse voluntariamente"
	" a uno o a varios grupos de tipo %s.";		// Okoteve traduccin
#elif L==6
	"Ti puoi registrare volontariamente"
	" in uno o pi&ugrave; gruppi di tipo %s.";
#elif L==7
	"You can register voluntarily"
	" in one or more groups of type %s.";		// Potrzebujesz tlumaczenie
#elif L==8
	"You can register voluntarily"
	" in one or more groups of type %s.";		// Necessita de traduo
#endif

const char *Txt_You_can_send_a_file_with_an_image_in_jpg_format_ =
#if   L==0
	"Puede enviar un archivo con una imagen en formato <em>jpg</em>.<br />"
	"La imagen debe contener al menos un rostro en posici&oacute;n frontal,"
	" y con el fondo blanco o muy claro.";				// Necessita traduccio
#elif L==1
	"You can send a file with an image in <em>jpg</em> format.<br />"
	"The image must contain at least one face in front position,"
	" and the background must be white or very light.";		// Need bersetzung
#elif L==2
	"You can send a file with an image in <em>jpg</em> format.<br />"
	"The image must contain at least one face in front position,"
	" and the background must be white or very light.";
#elif L==3
	"Puede enviar un archivo con una imagen en formato <em>jpg</em>.<br />"
	"La imagen debe contener al menos un rostro en posici&oacute;n frontal,"
	" y con el fondo blanco o muy claro.";
#elif L==4
	"You can send a file with an image in <em>jpg</em> format.<br />"
	"The image must contain at least one face in front position,"
	" and the background must be white or very light.";		// Besoin de traduction
#elif L==5
	"Puede enviar un archivo con una imagen en formato <em>jpg</em>.<br />"
	"La imagen debe contener al menos un rostro en posici&oacute;n frontal,"
	" y con el fondo blanco o muy claro.";				// Okoteve traduccin
#elif L==6
	"Puoi inviare un file con un'immagine in formato <em>jpg</em>.<br />"
	"L'immagine deve contenere almeno un viso in posizione frontale,"
	" e lo sfondo deve essere bianco o molto chiaro.";
#elif L==7
	"You can send a file with an image in <em>jpg</em> format.<br />"
	"The image must contain at least one face in front position,"
	" and the background must be white or very light.";		// Potrzebujesz tlumaczenie
#elif L==8
	"You can send a file with an image in <em>jpg</em> format.<br />"
	"The image must contain at least one face in front position,"
	" and the background must be white or very light.";		// Necessita de traduo
#endif

const char *Txt_You_can_send_a_file_with_an_image_in_jpg_format_and_size_X_Y =	// Warning: it is very important to include two %u in the following sentences
#if   L==0
	"Puede enviar un archivo con una imagen en formato <em>jpg</em>"
	" y tama&ntilde;o %u&times;%u p&iacute;xeles.";		// Necessita traduccio
#elif L==1
	"You can send a file with an image in <em>jpg</em> format"
	" and size %u&times;%u pixels.";			// Need bersetzung
#elif L==2
	"You can send a file with an image in <em>jpg</em> format"
	" and size %u&times;%u pixels.";
#elif L==3
	"Puede enviar un archivo con una imagen en formato <em>jpg</em>"
	" y tama&ntilde;o %u&times;%u p&iacute;xeles.";
#elif L==4
	"You can send a file with an image in <em>jpg</em> format"
	" and size %u&times;%u pixels.";			// Besoin de traduction
#elif L==5
	"Puede enviar un archivo con una imagen en formato <em>jpg</em>"
	" y tama&ntilde;o %u&times;%u p&iacute;xeles.";		// Okoteve traduccin
#elif L==6
	"Puoi inviare un file con un'immagine in formato <em>jpg</em>"
	" e di dimensione %u&times;%u pixel.";
#elif L==7
	"You can send a file with an image in <em>jpg</em> format"
	" and size %u&times;%u pixels.";			// Potrzebujesz tlumaczenie
#elif L==8
	"You can send a file with an image in <em>jpg</em> format"
	" and size %u&times;%u pixels.";			// Necessita de traduo
#endif

const char *Txt_You_dont_have_permission_to_access_to_this_forum =
#if   L==0
	"No t&eacute; perm&iacute;s per accedir a aquest f&ograve;rum.";
#elif L==1
	"Sie haben keine Berechtigung, dieses Forum benutzen.";
#elif L==2
	"You don't have permission to access to this forum.";
#elif L==3
	"Usted no tiene permiso para acceder a este foro.";
#elif L==4
	"Vous n'avez pas la permission d'acc&eacute;der &agrave; ce forum.";
#elif L==5
	"Usted no tiene permiso para acceder a este foro.";	// Okoteve traduccin
#elif L==6
	"Non si dispone dell'autorizzazione per accedere a questo forum.";
#elif L==7
	"Nie masz dost&eogon;pu do tej forum.";
#elif L==8
	"Voc&ecirc; n&atilde;o tem permiss&atilde;o para acessar este f&oacute;rum.";
#endif

const char *Txt_You_dont_have_permission_to_change_the_properties_of_file_X =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"Usted no tiene permiso para cambiar las propiedades del archivo %s.";		// Necessita traduccio
#elif L==1
	"You don't have permission to change the properties of file %s.";		// Need bersetzung
#elif L==2
	"You don't have permission to change the properties of file %s.";
#elif L==3
	"Usted no tiene permiso para cambiar las propiedades del archivo %s.";
#elif L==4
	"You don't have permission to change the properties of file %s.";		// Besoin de traduction
#elif L==5
	"Usted no tiene permiso para cambiar las propiedades del archivo %s.";		// Okoteve traduccin
#elif L==6
	"You don't have permission to change the properties of file %s.";		// Bisogno di traduzione
#elif L==7
	"You don't have permission to change the properties of file %s.";		// Potrzebujesz tlumaczenie
#elif L==8
	"You don't have permission to change the properties of file %s.";		// Necessita de traduo
#endif

const char *Txt_You_dont_have_permission_to_edit_this_course =
#if   L==0
	"Usted no tiene permiso para editar esta asignatura.";		// Necessita traduccio
#elif L==1
	"You don't have permission to edit this course.";		// Need bersetzung
#elif L==2
	"You don't have permission to edit this course.";
#elif L==3
	"Usted no tiene permiso para editar esta asignatura.";
#elif L==4
	"You don't have permission to edit this course.";		// Besoin de traduction
#elif L==5
	"Usted no tiene permiso para editar esta asignatura.";		// Okoteve traduccin
#elif L==6
	"Non hai l'autorizzazione di editare questo corso.";
#elif L==7
	"You don't have permission to edit this course.";		// Potrzebujesz tlumaczenie
#elif L==8
	"You don't have permission to edit this course.";		// Necessita de traduo
#endif

const char *Txt_You_dont_have_permission_to_move_courses_to_the_degree_X =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"Usted no tiene permiso para mover asignaturas"
	" a la titulaci&oacute;n <strong>%s</strong>.";	// Necessita traduccio
#elif L==1
	"You don't have permission to move courses"
	" to the degree <strong>%s</strong>.";		// Need bersetzung
#elif L==2
	"You don't have permission to move courses"
	" to the degree <strong>%s</strong>.";
#elif L==3
	"Usted no tiene permiso para mover asignaturas"
	" a la titulaci&oacute;n <strong>%s</strong>.";
#elif L==4
	"You don't have permission to move courses"
	" to the degree <strong>%s</strong>.";		// Besoin de traduction
#elif L==5
	"Usted no tiene permiso para mover asignaturas"
	" a la titulaci&oacute;n <strong>%s</strong>.";	// Okoteve traduccin
#elif L==6
	"Non hai l'autorizzazione di spostare i corsi"
	" nella laurea <strong>%s</strong>.";
#elif L==7
	"You don't have permission to move courses"
	" to the degree <strong>%s</strong>.";		// Potrzebujesz tlumaczenie
#elif L==8
	"You don't have permission to move courses"
	" to the degree <strong>%s</strong>.";		// Necessita de traduo
#endif

const char *Txt_You_dont_have_permission_to_perform_this_action =
#if   L==0
	"Vost&egrave; no t&eacute; perm&iacute;s"
	" per realitzar aquesta acci&oacute;.";
#elif L==1
	"Sie haben keine Berechtigung,"
	" um diese Handlung durchzuf&uuml;hren.";
#elif L==2
	"You don't have permission"
	" to perform this action.";
#elif L==3
	"Usted no tiene permiso"
	" para realizar esta acci&oacute;n.";
#elif L==4
	"Vous n'avez pas la permission"
	" d'effectuer cette action.";
#elif L==5
	"Usted no tiene permiso"
	" para realizar esta acci&oacute;n.";		// Okoteve traduccin
#elif L==6
	"Non si dispone dell'autorizzazione"
	" per eseguire questa azione.";
#elif L==7
	"You don't have permission"
	" to perform this action.";			// Potrzebujesz tlumaczenie
#elif L==8
	"Voc&ecirc; n&atilde;o tem permiss&atilde;o"
	" para realizar esta a&ccedil;&atilde;o.";
#endif

const char *Txt_You_dont_have_photo =
#if   L==0
	"Usted no tiene fotograf&iacute;a.";	// Necessita traduccio
#elif L==1
	"Sie haben keine Foto.";
#elif L==2
	"You don't have photo.";
#elif L==3
	"Usted no tiene fotograf&iacute;a.";
#elif L==4
	"Vous n'avez pas de photo.";
#elif L==5
	"Usted no tiene fotograf&iacute;a.";	// Okoteve traduccin
#elif L==6
	"Non hai foto.";
#elif L==7
	"You don't have photo.";		// Potrzebujesz tlumaczenie
#elif L==8
	"Voc&ecirc; n&atilde;o tem foto.";
#endif

const char *Txt_You_have_confirmed_that_you_have_read_this_information =
#if   L==0
	"Usted ha confirmado que ha leido esta informaci&oacute;n.";	// Necessita traduccio
#elif L==1
	"You have confirmed that you have read this information.";	// Need bersetzung
#elif L==2
	"You have confirmed that you have read this information.";
#elif L==3
	"Usted ha confirmado que ha leido esta informaci&oacute;n.";
#elif L==4
	"You have confirmed that you have read this information.";	// Besoin de traduction
#elif L==5
	"Usted ha confirmado que ha leido esta informaci&oacute;n.";	// Okoteve traduccin
#elif L==6
	"Hai confermato che hai letto questa informazione.";
#elif L==7
	"You have confirmed that you have read this information.";	// Potrzebujesz tlumaczenie
#elif L==8
	"You have confirmed that you have read this information.";	// Necessita de traduo
#endif

const char *Txt_You_have_confirmed_your_enrollment_in_the_course_X =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"Usted ha confirmado su inscripci&oacute;n"
	" en la asignatura <strong>%s</strong>.";	// Necessita traduccio
#elif L==1
	"You have confirmed your enrollment"
	" in the course <strong>%s</strong>.";		// Need bersetzung
#elif L==2
	"You have confirmed your enrollment"
	" in the course <strong>%s</strong>.";
#elif L==3
	"Usted ha confirmado su inscripci&oacute;n"
	" en la asignatura <strong>%s</strong>.";
#elif L==4
	"You have confirmed your enrollment"
	" in the course <strong>%s</strong>.";		// Besoin de traduction
#elif L==5
	"Usted ha confirmado su inscripci&oacute;n"
	" en la asignatura <strong>%s</strong>.";	// Okoteve traduccin
#elif L==6
	"Hai confermato la tua registrazione"
	" in questo corso <strong>%s</strong>.";
#elif L==7
	"You have confirmed your enrollment"
	" in the course <strong>%s</strong>.";		// Potrzebujesz tlumaczenie
#elif L==8
	"You have confirmed your enrollment"
	" in the course <strong>%s</strong>.";		// Necessita de traduo
#endif

const char *Txt_You_have_eliminated_the_confirmation_that_you_have_read_this_information =
#if   L==0
	"Usted ha eliminado la confirmaci&oacute;n"
	" de que ha leido esta informaci&oacute;n.";	// Necessita traduccio
#elif L==1
	"You have eliminated the confirmation"
	" that you have read this information.";	// Need bersetzung
#elif L==2
	"You have eliminated the confirmation"
	" that you have read this information.";
#elif L==3
	"Usted ha eliminado la confirmaci&oacute;n"
	" de que ha leido esta informaci&oacute;n.";
#elif L==4
	"You have eliminated the confirmation"
	" that you have read this information.";	// Besoin de traduction
#elif L==5
	"Usted ha eliminado la confirmaci&oacute;n"
	" de que ha leido esta informaci&oacute;n.";	// Okoteve traduccin
#elif L==6
	"Hai rimosso la conferma"
	" che hai letto questa informazione.";
#elif L==7
	"You have eliminated the confirmation"
	" that you have read this information.";	// Potrzebujesz tlumaczenie
#elif L==8
	"You have eliminated the confirmation"
	" that you have read this information.";	// Necessita de traduo
#endif

const char *Txt_You_have_not_banned_any_sender =
#if   L==0
	"Usted no ha bloqueado ning&uacute;n remitente.";	// Necessita traduccio
#elif L==1
	"You have not banned any sender.";			// Need bersetzung
#elif L==2
	"You have not banned any sender.";
#elif L==3
	"Usted no ha bloqueado ning&uacute;n remitente.";
#elif L==4
	"You have not banned any sender.";			// Besoin de traduction
#elif L==5
	"Usted no ha bloqueado ning&uacute;n remitente.";	// Okoteve traduccin
#elif L==6
	"You have not banned any sender.";			// Bisogno di traduzione
#elif L==7
	"You have not banned any sender.";			// Potrzebujesz tlumaczenie
#elif L==8
	"You have not banned any sender.";			// Necessita de traduo
#endif

const char *Txt_You_have_not_confirmed_the_action =
#if   L==0
	"No ha confirmat l'acci&oacute;.";
#elif L==1
	"Sie haben nicht die Aktion best&auml;tigt.";
#elif L==2
	"You have not confirmed the action.";
#elif L==3
	"No ha confirmado la acci&oacute;n.";
#elif L==4
	"Vous n'avez pas confirm&eacute; l'action.";
#elif L==5
	"No ha confirmado la acci&oacute;n.";		// Okoteve traduccin
#elif L==6
	"Non hai inserito correttamente la tua password.";
#elif L==7
	"Nie potwierdzi&lstrok;y dzia&lstrok;anie.";
#elif L==8
	"Voc&ecirc; ainda n&atilde;o confirmou a a&ccedil;&atilde;o.";
#endif

const char *Txt_You_have_not_entered_your_password_correctly =
#if   L==0
	"No ha introdu&iuml;t correctament la seva contrasenya.";
#elif L==1
	"Sie haben Ihr Passwort nicht korrekt eingegeben wurde.";
#elif L==2
	"You have not entered your password correctly.";
#elif L==3
	"No ha introducido correctamente su contrase&ntilde;a.";
#elif L==4
	"Vous n'avez pas entr&eacute; votre mot de passe correctement.";
#elif L==5
	"No ha introducido correctamente su contrase&ntilde;a.";	// Okoteve traduccin
#elif L==6
	"Non hai inserito correttamente la tua password.";
#elif L==7
	"Nie wpisa&lstrok;e&sacute; poprawnie has&lstrok;o.";
#elif L==8
	"Voc&ecirc; n&atilde;o digitou sua senha corretamente.";
#endif

const char *Txt_You_have_not_written_twice_the_same_new_password =
#if   L==0
	"Usted no ha escrito dos veces la misma contrase&ntilde;a nueva.";	// Necessita traduccio
#elif L==1
	"You have not written twice the same new password.";			// Need bersetzung
#elif L==2
	"You have not written twice the same new password.";
#elif L==3
	"Usted no ha escrito dos veces la misma contrase&ntilde;a nueva.";
#elif L==4
	"You have not written twice the same new password.";			// Besoin de traduction
#elif L==5
	"Usted no ha escrito dos veces la misma contrase&ntilde;a nueva.";	// Okoteve traduccin
#elif L==6
	"Non hai scritto due volte la stessa password nuova.";
#elif L==7
	"You have not written twice the same new password.";			// Potrzebujesz tlumaczenie
#elif L==8
	"You have not written twice the same new password.";			// Necessita de traduo
#endif

const char *Txt_You_have_to_register_compulsorily_at_least_in_one_group_of_type_X =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"Tiene que apuntarse obligatoriamente al menos a un grupo de tipo %s.";	// Necessita traduccio
#elif L==1
	"You have to register compulsorily at least in one group of type %s.";	// Need bersetzung
#elif L==2
	"You have to register compulsorily at least in one group of type %s.";
#elif L==3
	"Tiene que apuntarse obligatoriamente al menos a un grupo de tipo %s.";
#elif L==4
	"You have to register compulsorily at least in one group of type %s.";	// Besoin de traduction
#elif L==5
	"Tiene que apuntarse obligatoriamente al menos a un grupo de tipo %s.";	// Okoteve traduccin
#elif L==6
	"Devi registrarti obbligatoriamente almeno in uno dei gruppi del tipo %s.";
#elif L==7
	"You have to register compulsorily at least in one group of type %s.";	// Potrzebujesz tlumaczenie
#elif L==8
	"You have to register compulsorily at least in one group of type %s.";	// Necessita de traduo
#endif

const char *Txt_You_have_to_register_compulsorily_in_one_group_of_type_X =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"Tiene que apuntarse obligatoriamente a un grupo de tipo %s.";	// Necessita traduccio
#elif L==1
	"You have to register compulsorily in one group of type %s.";	// Need bersetzung
#elif L==2
	"You have to register compulsorily in one group of type %s.";
#elif L==3
	"Tiene que apuntarse obligatoriamente a un grupo de tipo %s.";
#elif L==4
	"You have to register compulsorily in one group of type %s.";	// Besoin de traduction
#elif L==5
	"Tiene que apuntarse obligatoriamente a un grupo de tipo %s.";	// Okoteve traduccin
#elif L==6
	"Devi registrarti obbligatoriamente in uno dei gruppi del tipo %s.";
#elif L==7
	"You have to register compulsorily in one group of type %s.";	// Potrzebujesz tlumaczenie
#elif L==8
	"You have to register compulsorily in one group of type %s.";	// Necessita de traduo
#endif

const char *Txt_You_have_written_1_post_in_this_thread =
#if   L==0
	"Usted ha escrito 1 mensaje en esta discusi&oacute;n";		// Necessita traduccio
#elif L==1
	"Sie haben 1 Nachricht in diesem Thread";
#elif L==2
	"You have written 1 post in this thread";
#elif L==3
	"Usted ha escrito 1 mensaje en esta discusi&oacute;n";
#elif L==4
	"Vous avez &eacute;crit 1 message dans ce fil";
#elif L==5
	"Usted ha escrito 1 mensaje en esta discusi&oacute;n";		// Okoteve traduccin
#elif L==6
	"Hai scritto 1 post in questa discussione";
#elif L==7
	"You have written 1 post in this thread";			// Potrzebujesz tlumaczenie
#elif L==8
	"Voc&ecirc; escreveu 1 post neste thread";
#endif

const char *Txt_You_have_written_X_posts_in_this_thread =		// Warning: it is very important to include %u in the following sentences
#if   L==0
	"Usted ha escrito %u mensajes en esta discusi&oacute;n";	// Necessita traduccio
#elif L==1
	"Sie haben %u Nachrichten in diesem Thread";
#elif L==2
	"You have written %u posts in this thread";
#elif L==3
	"Usted ha escrito %u mensajes en esta discusi&oacute;n";
#elif L==4
	"Vous avez &eacute;crit %u messages dans ce fil";
#elif L==5
	"Usted ha escrito %u mensajes en esta discusi&oacute;n";	// Okoteve traduccin
#elif L==6
	"Hai scritto %u post in questa discussione";
#elif L==7
	"You have written %u posts in this thread";			// Potrzebujesz tlumaczenie
#elif L==8
	"Voc&ecirc; escreveu %u posts neste thread";
#endif

const char *Txt_You_must_create_at_least_one_centre_before_creating_degrees =
#if   L==0
	"Debe crear al menos un centro antes de crear titulaciones.";	// Necessita traduccio
#elif L==1
	"You must create at least one centre before creating degrees.";	// bersetzung notwendig
#elif L==2
	"You must create at least one centre before creating degrees.";
#elif L==3
	"Debe crear al menos un centro antes de crear titulaciones.";
#elif L==4
	"You must create at least one centre before creating degrees.";	// Besoin de traduction
#elif L==5
	"Debe crear al menos un centro antes de crear titulaciones.";	// Okoteve traduccin
#elif L==6
	"Devi creare almeno un centro prima di creare lauree.";
#elif L==7
	"You must create at least one centro before creating degrees.";	// Potrzebujesz tlumaczenie
#elif L==8
	"You must create at least one centro before creating degrees.";	// Necessita de traduo
#endif

const char *Txt_You_must_create_at_least_one_institution_before_creating_centres =
#if   L==0
	"Debe crear al menos una instituci&oacute;n"
	" antes de crear centros.";		// Necessita traduccio
#elif L==1
	"You must create at least one institution"
	" before creating centres.";		// bersetzung notwendig
#elif L==2
	"You must create at least one institution"
	" before creating centres.";
#elif L==3
	"Debe crear al menos una instituci&oacute;n"
	" antes de crear centros.";
#elif L==4
	"You must create at least one institution"
	" before creating centres.";		// Besoin de traduction
#elif L==5
	"Debe crear al menos una instituci&oacute;n"
	" antes de crear centros.";		// Okoteve traduccin
#elif L==6
	"Devi creare almeno un'istituzione"
	" prima di creare centri.";
#elif L==7
	"You must create at least one institution"
	" before creating centres.";		// Potrzebujesz tlumaczenie
#elif L==8
	"You must create at least one institution"
	" before creating centres.";		// Necessita de traduo
#endif

const char *Txt_You_must_create_at_least_one_type_of_degree_before_creating_degrees =
#if   L==0
	"Debe crear al menos un tipo de titulaci&oacute;n"
	" antes de crear titulaciones.";		// Necessita traduccio
#elif L==1
	"You must create at least one type of degree"
	" before creating degrees.";			// bersetzung notwendig
#elif L==2
	"You must create at least one type of degree"
	" before creating degrees.";
#elif L==3
	"Debe crear al menos un tipo de titulaci&oacute;n"
	" antes de crear titulaciones.";
#elif L==4
	"You must create at least one type of degree"
	" before creating degrees.";			// Besoin de traduction
#elif L==5
	"Debe crear al menos un tipo de titulaci&oacute;n"
	" antes de crear titulaciones.";		// Okoteve traduccin
#elif L==6
	"Devi creare almeno un tipo di laurea"
	" prima di creare lauree.";
#elif L==7
	"You must create at least one type of degree"
	" before creating degrees.";			// Potrzebujesz tlumaczenie
#elif L==8
	"You must create at least one type of degree"
	" before creating degrees.";			// Necessita de traduo
#endif

const char *Txt_You_must_enter_an_integer_value_as_the_correct_answer =
#if   L==0
	"Debe escribir un valor entero como respuesta correcta.";	// Necessita traduccio
#elif L==1
	"You must enter an integer value as the correct answer.";	// bersetzung notwendig
#elif L==2
	"You must enter an integer value as the correct answer.";
#elif L==3
	"Debe escribir un valor entero como respuesta correcta.";
#elif L==4
	"You must enter an integer value as the correct answer.";	// Besoin de traduction
#elif L==5
	"Debe escribir un valor entero como respuesta correcta.";	// Okoteve traduccin
#elif L==6
	"Devi inserire un valore intero come risposta corretta.";
#elif L==7
	"You must enter an integer value as the correct answer.";	// Potrzebujesz tlumaczenie
#elif L==8
	"You must enter an integer value as the correct answer.";	// Necessita de traduo
#endif

const char *Txt_You_must_enter_the_range_of_floating_point_values_allowed_as_answer =
#if   L==0
	"Debe escribir el intervalo de valores reales permitidos como respuesta.";	// Necessita traduccio
#elif L==1
	"You must enter the range of floating point values allowed as answer.";		// bersetzung notwendig
#elif L==2
	"You must enter the range of floating point values allowed as answer.";
#elif L==3
	"Debe escribir el intervalo de valores reales permitidos como respuesta.";
#elif L==4
	"You must enter the range of floating point values allowed as answer.";		// Besoin de traduction
#elif L==5
	"Debe escribir el intervalo de valores reales permitidos como respuesta.";	// Okoteve traduccin
#elif L==6
	"Devi inserire il range di valori reali permessi come risposta.";
#elif L==7
	"You must enter the range of floating point values allowed as answer.";		// Potrzebujesz tlumaczenie
#elif L==8
	"You must enter the range of floating point values allowed as answer.";		// Necessita de traduo
#endif

const char *Txt_You_must_enter_your_ID_or_your_nickname =
#if   L==0
	"Debe escribir su ID (DNI/c&eacute;dula) o su apodo.";	// Necessita traduccio
#elif L==1
	"Bitte geben Sie Ihre Ausweis-Nr. oder Ihren Benutzernamen ein.";
#elif L==2
	"You must enter your ID or your nickname.";
#elif L==3
	"Debe escribir su ID (DNI/c&eacute;dula) o su apodo.";
#elif L==4
	"Vous devez entrer votre num&eacute;ro d'identit&eacute; ou votre surnom.";
#elif L==5
	"Debe escribir su ID (DNI/c&eacute;dula) o su apodo.";	// Okoteve traduccin
#elif L==6
	"Devi inserire il tuo numero di carta d'identit&agrave; o il tuo nome utente.";
#elif L==7
	"You must enter your ID or your nickname.";		// Potrzebujesz tlumaczenie
#elif L==8
	"Voc&ecirc; deve digitar seu n&ordm; identif. ou seu alcunha.";
#endif

const char *Txt_You_must_mark_an_answer_as_correct =
#if   L==0
	"Debe marcar alguna respuesta como correcta.";	// Necessita traduccio
#elif L==1
	"You must mark an answer as correct.";		// Need bersetzung
#elif L==2
	"You must mark an answer as correct.";
#elif L==3
	"Debe marcar alguna respuesta como correcta.";
#elif L==4
	"You must mark an answer as correct.";		// Besoin de traduction
#elif L==5
	"Debe marcar alguna respuesta como correcta.";	// Okoteve traduccin
#elif L==6
	"Devi segnare una risposta come corretta.";
#elif L==7
	"You must mark an answer as correct.";		// Potrzebujesz tlumaczenie
#elif L==8
	"You must mark an answer as correct.";		// Necessita de traduo
#endif

const char *Txt_You_must_select_a_T_F_answer =
#if   L==0
	"Debe seleccionar una respuesta V o F.";	// Necessita traduccio
#elif L==1
	"You must select a T/F answer.";		// Need bersetzung
#elif L==2
	"You must select a T/F answer.";
#elif L==3
	"Debe seleccionar una respuesta V o F.";
#elif L==4
	"You must select a T/F answer.";		// Besoin de traduction
#elif L==5
	"Debe seleccionar una respuesta V o F.";	// Okoteve traduccin
#elif L==6
	"Devi selezionare una risposta V/F.";
#elif L==7
	"You must select a T/F answer.";		// Potrzebujesz tlumaczenie
#elif L==8
	"You must select a T/F answer.";		// Necessita de traduo
#endif

const char *Txt_You_must_select_one_ore_more_recipients =
#if   L==0
	"Debe seleccionar uno o m&aacute;s destinatarios.";	// Necessita traduccio
#elif L==1
	"Es muss mindestens ein Empf&auml;nger angegeben werden.";
#elif L==2
	"You must select one or more recipients.";
#elif L==3
	"Debe seleccionar uno o m&aacute;s destinatarios.";
#elif L==4
	"You must select one or more recipients.";		// Besoin de traduction
#elif L==5
	"Debe seleccionar uno o m&aacute;s destinatarios.";	// Okoteve traduccin
#elif L==6
	"Devi selezionare uno o pi&ugrave; destinatari.";
#elif L==7
	"You must select one or more recipients.";		// Potrzebujesz tlumaczenie
#elif L==8
	"You must select one or more recipients.";		// Necessita de traduo
#endif

const char *Txt_You_must_select_one_ore_more_students =
#if   L==0
	"Debe seleccionar uno o m&aacute;s estudiantes.";	// Necessita traduccio
#elif L==1
	"Es muss mindestens ein Student ausgew&auml;hlt werden.";
#elif L==2
	"You must select one or more students.";
#elif L==3
	"Debe seleccionar uno o m&aacute;s estudiantes.";
#elif L==4
	"You must select one or more students.";		// Besoin de traduction
#elif L==5
	"Debe seleccionar uno o m&aacute;s estudiantes.";	// Okoteve traduccin
#elif L==6
	"Devi selezionare uno o pi&ugrave; studenti.";
#elif L==7
	"You must select one or more students.";		// Potrzebujesz tlumaczenie
#elif L==8
	"You must select one or more students.";		// Necessita de traduo
#endif

const char *Txt_You_must_select_one_ore_more_tags =
#if   L==0
	"Debe seleccionar uno o m&aacute;s descriptores.";	// Necessita traduccio
#elif L==1
	"Es muss mindestens ein Tag ausgew&auml;hlt werden.";
#elif L==2
	"You must select one or more tags.";
#elif L==3
	"Debe seleccionar uno o m&aacute;s descriptores.";
#elif L==4
	"You must select one or more tags.";			// Besoin de traduction
#elif L==5
	"Debe seleccionar uno o m&aacute;s descriptores.";	// Okoteve traduccin
#elif L==6
	"Devi selezionare una o pi&ugrave; etichette.";
#elif L==7
	"You must select one or more tags.";			// Potrzebujesz tlumaczenie
#elif L==8
	"You must select one or more tags.";			// Necessita de traduo
#endif

const char *Txt_You_must_select_one_ore_more_teachers =
#if   L==0
	"Debe seleccionar uno o m&aacute;s profesores.";		// Necessita traduccio
#elif L==1
	"Es muss mindestens ein Lehrer ausgew&auml;hlt werden.";
#elif L==2
	"You must select one or more teachers.";
#elif L==3
	"Debe seleccionar uno o m&aacute;s profesores.";
#elif L==4
	"You must select one or more teachers.";			// Besoin de traduction
#elif L==5
	"Debe seleccionar uno o m&aacute;s profesores.";		// Okoteve traduccin
#elif L==6
	"Devi selezionare uno o pi&ugrave; professori.";
#elif L==7
	"You must select one or more teachers.";			// Potrzebujesz tlumaczenie
#elif L==8
	"You must select one or more teachers.";			// Necessita de traduo
#endif

const char *Txt_You_must_select_one_ore_more_types_of_answer =
#if   L==0
	"Debe seleccionar uno o m&aacute;s tipos de respuesta.";	// Necessita traduccio
#elif L==1
	"Es muss mindestens ein Antworttyp ausgew&auml;hlt werden.";
#elif L==2
	"You must select one or more types of answer.";
#elif L==3
	"Debe seleccionar uno o m&aacute;s tipos de respuesta.";
#elif L==4
	"You must select one or more types of answer.";			// Besoin de traduction
#elif L==5
	"Debe seleccionar uno o m&aacute;s tipos de respuesta.";	// Okoteve traduccin
#elif L==6
	"Devi selezionare uno o pi&ugrave; tipi di risposta.";
#elif L==7
	"You must select one or more types of answer.";			// Potrzebujesz tlumaczenie
#elif L==8
	"You must select one or more types of answer.";			// Necessita de traduo
#endif

const char *Txt_You_must_select_one_ore_more_users =
#if   L==0
	"Debe seleccionar uno o m&aacute;s usuarios.";	// Necessita traduccio
#elif L==1
	"Es muss mindestens ein Benutzer ausgew&auml;hlt werden.";
#elif L==2
	"You must select one or more users.";
#elif L==3
	"Debe seleccionar uno o m&aacute;s usuarios.";
#elif L==4
	"You must select one or more users.";		// Besoin de traduction
#elif L==5
	"Debe seleccionar uno o m&aacute;s usuarios.";	// Okoteve traduccin
#elif L==6
	"Devi selezionare uno o pi&ugrave; utenti.";
#elif L==7
	"You must select one or more users.";		// Potrzebujesz tlumaczenie
#elif L==8
	"You must select one or more users.";		// Necessita de traduo
#endif

const char *Txt_You_must_send_your_photo_because_ =
#if   L==0
	"Tiene que enviar su fotograf&iacute;a,"
	" ya que ha agotado el n&uacute;mero de accesos"
	" permitido sin fotograf&iacute;a.";	// Necessita traduccio
#elif L==1
	"You must send your photo,"
	" because you have exhausted the number of clicks"
	" allowed without photo.";		// Need bersetzung
#elif L==2
	"You must send your photo,"
	" because you have exhausted the number of clicks"
	" allowed without photo.";
#elif L==3
	"Tiene que enviar su fotograf&iacute;a,"
	" ya que ha agotado el n&uacute;mero de accesos"
	" permitido sin fotograf&iacute;a.";
#elif L==4
	"You must send your photo,"
	" because you have exhausted the number of clicks"
	" allowed without photo.";		// Besoin de traduction
#elif L==5
	"Tiene que enviar su fotograf&iacute;a,"
	" ya que ha agotado el n&uacute;mero de accesos"
	" permitido sin fotograf&iacute;a.";	// Okoteve traduccin
#elif L==6
	"Devi inviare la tua foto,"
	" perch&egrave; hai esaurito il numero di accessi"
	" permessi senza foto.";
#elif L==7
	"You must send your photo,"
	" because you have exhausted the number of clicks"
	" allowed without photo.";		// Potrzebujesz tlumaczenie
#elif L==8
	"You must send your photo,"
	" because you have exhausted the number of clicks"
	" allowed without photo.";		// Necessita de traduo
#endif

const char *Txt_You_must_specify_in_step_3_the_action_to_perform =
#if   L==0
	"S'ha d'especificar en el pas 3 l'acci&oacute; a realitzar.";
#elif L==1
	"Sie m&uuml;ssen in Schritt 3 die auszuf&uuml;hrende Aktion festzulegen.";
#elif L==2
	"You must specify in step 3 the action to perform.";
#elif L==3
	"Debe especificar en el paso 3 la acci&oacute;n a realizar.";
#elif L==4
	"Vous devez sp&eacute;cifier &agrave; l'&eacute;tape 3 l'action &agrave; effectuer.";
#elif L==5
	"Debe especificar en el paso 3 la acci&oacute;n a realizar.";	// Okoteve traduccin
#elif L==6
	"&Egrave; necessario specificare al punto 3 l'azione da eseguire.";
#elif L==7
	"You must specify in step 3 the action to perform.";		// Potrzebujesz tlumaczenie
#elif L==8
	"Voc&ecirc; deve especificar no passo 3 a a&ccedil;&atilde;o a ser executada.";
#endif

const char *Txt_You_must_specify_the_image_of_the_new_banner =
#if   L==0
	"Debe especificar la imagen"
	" del nuevo banner.";		// Necessita traduccio
#elif L==1
	"You must specify the image"
	" of the new banner.";		// Need bersetzung
#elif L==2
	"You must specify the image"
	" of the new banner.";
#elif L==3
	"Debe especificar la imagen"
	" del nuevo banner.";
#elif L==4
	"You must specify the image"
	" of the new banner.";		// Besoin de traduction
#elif L==5
	"Debe especificar la imagen"
	" del nuevo banner.";		// Okoteve traduccin
#elif L==6
	"Devi specificare l'image"
	" del nuovo banner.";
#elif L==7
	"You must specify the image"
	" of the new banner.";		// Potrzebujesz tlumaczenie
#elif L==8
	"You must specify the image"
	" of the new banner.";		// Necessita de traduo
#endif

const char *Txt_You_must_specify_the_logo_the_application_key_the_URL_and_the_IP_address_of_the_new_plugin =
#if   L==0
	"Debe especificar el logo, la clave de aplicaci&oacute;n,"
	" el URL y la IP del nuevo complemento.";		// Necessita traduccio
#elif L==1
	"You must specify the logo, the application key,"
	" the URL and the IP address of the new plugin.";	// Need bersetzung
#elif L==2
	"You must specify the logo, the application key,"
	" the URL and the IP address of the new plugin.";
#elif L==3
	"Debe especificar el logo, la clave de aplicaci&oacute;n,"
	" el URL y la IP del nuevo complemento.";
#elif L==4
	"You must specify the logo, the application key,"
	" the URL and the IP address of the new plugin.";	// Besoin de traduction
#elif L==5
	"Debe especificar el logo, la clave de aplicaci&oacute;n,"
	" el URL y la IP del nuevo complemento.";		// Okoteve traduccin
#elif L==6
	"Devi specificare il logo, il codice di applicazione,"
	" l'URL e l'indirizzo IP del nuovo plugin.";
#elif L==7
	"You must specify the logo, the application key,"
	" the URL and the IP address of the new plugin.";	// Potrzebujesz tlumaczenie
#elif L==8
	"You must specify the logo, the application key,"
	" the URL and the IP address of the new plugin.";	// Necessita de traduo
#endif

const char *Txt_You_must_specify_the_numerical_code_of_the_new_country =
#if   L==0
	"Debe especificar el c&oacute;digo num&eacute;rico ISO 3166-1 del nuevo pa&iacute;s.";	// Necessita traduccio
#elif L==1
	"You must specify the ISO 3166-1 numerical code of the new country.";			// Need bersetzung
#elif L==2
	"You must specify the ISO 3166-1 numerical code of the new country.";
#elif L==3
	"Debe especificar el c&oacute;digo num&eacute;rico ISO 3166-1 del nuevo pa&iacute;s.";
#elif L==4
	"You must specify the ISO 3166-1 numerical code of the new country.";			// Besoin de traduction
#elif L==5
	"Debe especificar el c&oacute;digo num&eacute;rico ISO 3166-1 del nuevo pa&iacute;s.";	// Okoteve traduccin
#elif L==6
	"Devi specificare il codice numerico ISO 3166-1 del nuovo paese.";
#elif L==7
	"You must specify the ISO 3166-1 numerical code of the new country.";			// Potrzebujesz tlumaczenie
#elif L==8
	"You must specify the ISO 3166-1 numerical code of the new country.";			// Necessita de traduo
#endif

const char *Txt_You_must_specify_the_name_and_the_description_of_the_new_plugin =
#if   L==0
	"Debe especificar el nombre y la descripci&oacute;n del nuevo complemento.";	// Necessita traduccio
#elif L==1
	"You must specify the name and the description of the new plugin.";		// Need bersetzung
#elif L==2
	"You must specify the name and the description of the new plugin.";
#elif L==3
	"Debe especificar el nombre y la descripci&oacute;n del nuevo complemento.";
#elif L==4
	"You must specify the name and the description of the new plugin.";		// Besoin de traduction
#elif L==5
	"Debe especificar el nombre y la descripci&oacute;n del nuevo complemento.";	// Okoteve traduccin
#elif L==6
	"Devi specificare il nome e la descrizione del nuovo plugin.";
#elif L==7
	"You must specify the name and the description of the new plugin.";		// Potrzebujesz tlumaczenie
#elif L==8
	"You must specify the name and the description of the new plugin.";		// Necessita de traduo
#endif

const char *Txt_You_must_specify_the_name_of_the_new_country_in_all_languages =
#if   L==0
	"Debe especificar el nombre del nuevo pa&iacute;s en todos los idiomas.";	// Necessita traduccio
#elif L==1
	"You must specify the name of the new country in all languages.";		// Need bersetzung
#elif L==2
	"You must specify the name of the new country in all languages.";
#elif L==3
	"Debe especificar el nombre del nuevo pa&iacute;s en todos los idiomas.";
#elif L==4
	"You must specify the name of the new country in all languages.";		// Besoin de traduction
#elif L==5
	"Debe especificar el nombre del nuevo pa&iacute;s en todos los idiomas.";	// Okoteve traduccin
#elif L==6
	"Devi specificare il nome del nuovo paese in tutte le lingue.";
#elif L==7
	"You must specify the name of the new country in all languages.";		// Potrzebujesz tlumaczenie
#elif L==8
	"You must specify the name of the new country in all languages.";		// Necessita de traduo
#endif

const char *Txt_You_must_specify_the_name_of_the_new_group =
#if   L==0
	"Debe especificar el nombre del nuevo grupo.";	// Necessita traduccio
#elif L==1
	"You must specify the name of the new group.";	// Need bersetzung
#elif L==2
	"You must specify the name of the new group.";
#elif L==3
	"Debe especificar el nombre del nuevo grupo.";
#elif L==4
	"You must specify the name of the new group.";	// Besoin de traduction
#elif L==5
	"Debe especificar el nombre del nuevo grupo.";	// Okoteve traduccin
#elif L==6
	"Devi specificare il nome del nuovo gruppo.";
#elif L==7
	"You must specify the name of the new group.";	// Potrzebujesz tlumaczenie
#elif L==8
	"You must specify the name of the new group.";	// Necessita de traduo
#endif

const char *Txt_You_must_specify_the_name_of_the_new_holiday =
#if   L==0
	"Debe especificar el nombre de la nueva festividad.";	// Necessita traduccio
#elif L==1
	"You must specify the name of the new holiday.";	// Need bersetzung
#elif L==2
	"You must specify the name of the new holiday.";
#elif L==3
	"Debe especificar el nombre de la nueva festividad.";
#elif L==4
	"You must specify the name of the new holiday.";	// Besoin de traduction
#elif L==5
	"Debe especificar el nombre de la nueva festividad.";	// Okoteve traduccin
#elif L==6
	"Devi specificare il nome della nuova festivit&agrave;.";
#elif L==7
	"You must specify the name of the new holiday.";	// Potrzebujesz tlumaczenie
#elif L==8
	"You must specify the name of the new holiday.";	// Necessita de traduo
#endif

const char *Txt_You_must_specify_the_name_of_the_new_record_field =
#if   L==0
	"Debe especificar el nombre del nuevo campo de ficha.";	// Necessita traduccio
#elif L==1
	"You must specify the name of the new record field.";	// Need bersetzung
#elif L==2
	"You must specify the name of the new record field.";
#elif L==3
	"Debe especificar el nombre del nuevo campo de ficha.";
#elif L==4
	"You must specify the name of the new record field.";	// Besoin de traduction
#elif L==5
	"Debe especificar el nombre del nuevo campo de ficha.";	// Okoteve traduccin
#elif L==6
	"Devi specificare il nome del nuovo campo di scheda.";
#elif L==7
	"You must specify the name of the new record field.";	// Potrzebujesz tlumaczenie
#elif L==8
	"You must specify the name of the new record field.";	// Necessita de traduo
#endif

const char *Txt_You_must_specify_the_name_of_the_new_type_of_degree =
#if   L==0
	"Debe especificar el nombre del nuevo tipo de titulaci&oacute;n.";	// Necessita traduccio
#elif L==1
	"You must specify the name of the new type of degree.";			// Need bersetzung
#elif L==2
	"You must specify the name of the new type of degree.";
#elif L==3
	"Debe especificar el nombre del nuevo tipo de titulaci&oacute;n.";
#elif L==4
	"You must specify the name of the new type of degree.";			// Besoin de traduction
#elif L==5
	"Debe especificar el nombre del nuevo tipo de titulaci&oacute;n.";	// Okoteve traduccin
#elif L==6
	"Devi specificare il nome del nuovo tipo di laurea.";
#elif L==7
	"You must specify the name of the new type of degree.";			// Potrzebujesz tlumaczenie
#elif L==8
	"You must specify the name of the new type of degree.";			// Necessita de traduo
#endif

const char *Txt_You_must_specify_the_name_of_the_new_type_of_group =
#if   L==0
	"Debe especificar el nombre del nuevo tipo de grupo.";	// Necessita traduccio
#elif L==1
	"You must specify the name of the new type of group.";	// Need bersetzung
#elif L==2
	"You must specify the name of the new type of group.";
#elif L==3
	"Debe especificar el nombre del nuevo tipo de grupo.";
#elif L==4
	"You must specify the name of the new type of group.";	// Besoin de traduction
#elif L==5
	"Debe especificar el nombre del nuevo tipo de grupo.";	// Okoteve traduccin
#elif L==6
	"Devi specificare il nome del nuovo tipo di gruppo.";
#elif L==7
	"You must specify the name of the new type of group.";	// Potrzebujesz tlumaczenie
#elif L==8
	"You must specify the name of the new type of group.";	// Necessita de traduo
#endif

const char *Txt_You_must_specify_the_short_name_and_the_full_name_of_the_new_banner =
#if   L==0
	"Debe especificar el nombre breve y el nombre completo"
	" del nuevo banner.";		// Necessita traduccio
#elif L==1
	"You must specify the short name and the full name"
	" of the new banner.";		// Need bersetzung
#elif L==2
	"You must specify the short name and the full name"
	" of the new banner.";
#elif L==3
	"Debe especificar el nombre breve y el nombre completo"
	" del nuevo banner.";
#elif L==4
	"You must specify the short name and the full name"
	" of the new banner.";		// Besoin de traduction
#elif L==5
	"Debe especificar el nombre breve y el nombre completo"
	" del nuevo banner.";		// Okoteve traduccin
#elif L==6
	"Devi specificare il nome breve e completo"
	" del nuovo banner.";
#elif L==7
	"You must specify the short name and the full name"
	" of the new banner.";		// Potrzebujesz tlumaczenie
#elif L==8
	"You must specify the short name and the full name"
	" of the new banner.";		// Necessita de traduo
#endif

const char *Txt_You_must_specify_the_short_name_and_the_full_name_of_the_new_centre =
#if   L==0
	"Debe especificar el nombre breve y el nombre completo"
	" del nuevo centro.";		// Necessita traduccio
#elif L==1
	"You must specify the short name and the full name"
	" of the new centre.";		// Need bersetzung
#elif L==2
	"You must specify the short name and the full name"
	" of the new centre.";
#elif L==3
	"Debe especificar el nombre breve y el nombre completo"
	" del nuevo centro.";
#elif L==4
	"You must specify the short name and the full name"
	" of the new centre.";		// Besoin de traduction
#elif L==5
	"Debe especificar el nombre breve y el nombre completo"
	" del nuevo centro.";		// Okoteve traduccin
#elif L==6
	"Devi specificare il nome breve e completo"
	" del nuovo centro.";
#elif L==7
	"You must specify the short name and the full name"
	" of the new centre.";		// Potrzebujesz tlumaczenie
#elif L==8
	"You must specify the short name and the full name"
	" of the new centre.";		// Necessita de traduo
#endif

const char *Txt_You_must_specify_the_short_name_and_the_full_name_of_the_new_course =
#if   L==0
	"Debe especificar el nombre breve y el nombre completo"
	" de la nueva asignatura.";		// Necessita traduccio
#elif L==1
	"You must specify the short name and the full name"
	" of the new course.";			// Need bersetzung
#elif L==2
	"You must specify the short name and the full name"
	" of the new course.";
#elif L==3
	"Debe especificar el nombre breve y el nombre completo"
	" de la nueva asignatura.";
#elif L==4
	"You must specify the short name and the full name"
	" of the new course.";			// Besoin de traduction
#elif L==5
	"Debe especificar el nombre breve y el nombre completo"
	" de la nueva asignatura.";		// Okoteve traduccin
#elif L==6
	"Devi specificare il nome breve e completo"
	" del nuovo corso.";
#elif L==7
	"You must specify the short name and the full name"
	" of the new course.";			// Potrzebujesz tlumaczenie
#elif L==8
	"You must specify the short name and the full name"
	" of the new course.";			// Necessita de traduo
#endif

const char *Txt_You_must_specify_the_short_name_and_the_full_name_of_the_new_degree =
#if   L==0
	"Debe especificar el nombre breve y el nombre completo"
	" de la nueva titulaci&oacute;n.";	// Necessita traduccio
#elif L==1
	"You must specify the short name and the full name"
	" of the new degree.";			// Need bersetzung
#elif L==2
	"You must specify the short name and the full name"
	" of the new degree.";
#elif L==3
	"Debe especificar el nombre breve y el nombre completo"
	" de la nueva titulaci&oacute;n.";
#elif L==4
	"You must specify the short name and the full name"
	" of the new degree.";			// Besoin de traduction
#elif L==5
	"Debe especificar el nombre breve y el nombre completo"
	" de la nueva titulaci&oacute;n.";	// Okoteve traduccin
#elif L==6
	"Devi specificare il nome breve e completo"
	" della nuova laurea.";
#elif L==7
	"You must specify the short name and the full name"
	" of the new degree.";			// Potrzebujesz tlumaczenie
#elif L==8
	"You must specify the short name and the full name"
	" of the new degree.";			// Necessita de traduo
#endif

const char *Txt_You_must_specify_the_short_name_and_the_full_name_of_the_new_department =
#if   L==0
	"Debe especificar el nombre breve y el nombre completo"
	" del nuevo departamento.";		// Necessita traduccio
#elif L==1
	"You must specify the short name and the full name"
	" of the new department.";		// Need bersetzung
#elif L==2
	"You must specify the short name and the full name"
	" of the new department.";
#elif L==3
	"Debe especificar el nombre breve y el nombre completo"
	" del nuevo departamento.";
#elif L==4
	"You must specify the short name and the full name"
	" of the new department.";		// Besoin de traduction
#elif L==5
	"Debe especificar el nombre breve y el nombre completo"
	" del nuevo departamento.";		// Okoteve traduccin
#elif L==6
	"Devi specificare il nome breve e completo"
	" del nuovo dipartimento.";
#elif L==7
	"You must specify the short name and the full name"
	" of the new department.";		// Potrzebujesz tlumaczenie
#elif L==8
	"You must specify the short name and the full name"
	" of the new department.";		// Necessita de traduo
#endif

const char *Txt_You_must_specify_the_short_name_and_the_full_name_of_the_new_institution =
#if   L==0
	"Debe especificar el nombre breve y el nombre completo"
	" de la nueva instituci&oacute;n.";		// Necessita traduccio
#elif L==1
	"You must specify the short name and the full name"
	" of the new institution.";			// Need bersetzung
#elif L==2
	"You must specify the short name and the full name"
	" of the new institution.";
#elif L==3
	"Debe especificar el nombre breve y el nombre completo"
	" de la nueva instituci&oacute;n.";
#elif L==4
	"You must specify the short name and the full name"
	" of the new institution.";			// Besoin de traduction
#elif L==5
	"Debe especificar el nombre breve y el nombre completo"
	" de la nueva instituci&oacute;n.";		// Okoteve traduccin
#elif L==6
	"Devi specificare il nome breve e completo"
	" della nuova instituzione.";
#elif L==7
	"You must specify the short name and the full name"
	" of the new institution.";			// Potrzebujesz tlumaczenie
#elif L==8
	"You must specify the short name and the full name"
	" of the new institution.";			// Necessita de traduo
#endif

const char *Txt_You_must_specify_the_short_name_and_the_full_name_of_the_new_link =
#if   L==0
	"Debe especificar el nombre breve y el nombre completo"
	" del nuevo enlace.";		// Necessita traduccio
#elif L==1
	"You must specify the short name and the full name"
	" of the new link.";		// Need bersetzung
#elif L==2
	"You must specify the short name and the full name"
	" of the new link.";
#elif L==3
	"Debe especificar el nombre breve y el nombre completo"
	" del nuevo enlace.";
#elif L==4
	"You must specify the short name and the full name"
	" of the new link.";		// Besoin de traduction
#elif L==5
	"Debe especificar el nombre breve y el nombre completo"
	" del nuevo enlace.";		// Okoteve traduccin
#elif L==6
	"Devi specificare il nome breve e completo"
	" del nuovo link.";
#elif L==7
	"You must specify the short name and the full name"
	" of the new link.";		// Potrzebujesz tlumaczenie
#elif L==8
	"You must specify the short name and the full name"
	" of the new link.";		// Necessita de traduo
#endif

const char *Txt_You_must_specify_the_short_name_and_the_full_name_of_the_new_mail_domain =
#if   L==0
	"Debe especificar el nombre breve y el nombre completo"
	" del nuevo dominio de correo.";	// Necessita traduccio
#elif L==1
	"You must specify the short name and the full name"
	" of the new mail domain.";		// Need bersetzung
#elif L==2
	"You must specify the short name and the full name"
	" of the new mail domain.";
#elif L==3
	"Debe especificar el nombre breve y el nombre completo"
	" del nuevo dominio de correo.";
#elif L==4
	"You must specify the short name and the full name"
	" of the new mail domain.";		// Besoin de traduction
#elif L==5
	"Debe especificar el nombre breve y el nombre completo"
	" del nuevo dominio de correo.";	// Okoteve traduccin
#elif L==6
	"Devi specificare il nome breve e completo"
	" del nuovo campo mail.";
#elif L==7
	"You must specify the short name and the full name"
	" of the new mail domain.";		// Potrzebujesz tlumaczenie
#elif L==8
	"You must specify the short name and the full name"
	" of the new mail domain.";		// Necessita de traduo
#endif

const char *Txt_You_must_specify_the_short_name_and_the_full_name_of_the_new_place =
#if   L==0
	"Debe especificar el nombre breve y el nombre completo"
	" del nuevo lugar.";		// Necessita traduccio
#elif L==1
	"You must specify the short name and the full name"
	" of the new place.";		// Need bersetzung
#elif L==2
	"You must specify the short name and the full name"
	" of the new place.";
#elif L==3
	"Debe especificar el nombre breve y el nombre completo"
	" del nuevo lugar.";
#elif L==4
	"You must specify the short name and the full name"
	" of the new place.";		// Besoin de traduction
#elif L==5
	"Debe especificar el nombre breve y el nombre completo"
	" del nuevo lugar.";		// Okoteve traduccin
#elif L==6
	"Devi specificare il nome breve e completo"
	" del nuovo luogo.";
#elif L==7
	"You must specify the short name and the full name"
	" of the new place.";		// Potrzebujesz tlumaczenie
#elif L==8
	"You must specify the short name and the full name"
	" of the new place.";		// Necessita de traduo
#endif

const char *Txt_You_must_specify_the_title_of_the_assignment =
#if   L==0
	"Debe especificar el t&iacute;tulo de la actividad.";	// Necessita traduccio
#elif L==1
	"Die Aufgabe muss angegeben werden.";
#elif L==2
	"You must specify the title of the assignment.";
#elif L==3
	"Debe especificar el t&iacute;tulo de la actividad.";
#elif L==4
	"Vous devez sp&eacute;cifier le titre de l'activit&eacute;.";
#elif L==5
	"Debe especificar el t&iacute;tulo de la actividad.";	// Okoteve traduccin
#elif L==6
	"&Egrave; necessario specificare il titolo dell'attivit&agrave;.";
#elif L==7
	"You must specify the title of the assignment.";	// Potrzebujesz tlumaczenie
#elif L==8
	"Voc&ecirc; deve especificar o t&iacute;tulo da atividade.";
#endif

const char *Txt_You_must_specify_the_title_of_the_event =
#if   L==0
	"Debe especificar el t&iacute;tulo del evento.";	// Necessita traduccio
#elif L==1
	"Der Ereignis muss angegeben werden.";
#elif L==2
	"You must specify the title of the event.";
#elif L==3
	"Debe especificar el t&iacute;tulo del evento.";
#elif L==4
	"Vous devez sp&eacute;cifier le titre du &eacute;v&eacute;nement.";
#elif L==5
	"Debe especificar el t&iacute;tulo del evento.";	// Okoteve traduccin
#elif L==6
	"&Egrave; necessario specificare il titolo del evento.";
#elif L==7
	"You must specify the title of the event.";		// Potrzebujesz tlumaczenie
#elif L==8
	"Voc&ecirc; deve especificar o t&iacute;tulo do evento.";
#endif

const char *Txt_You_must_specify_the_title_of_the_survey =
#if   L==0
	"Debe especificar el t&iacute;tulo de la encuesta.";	// Necessita traduccio
#elif L==1
	"Die Umfrage muss angegeben werden.";
#elif L==2
	"You must specify the title of the survey.";
#elif L==3
	"Debe especificar el t&iacute;tulo de la encuesta.";
#elif L==4
	"Vous devez sp&eacute;cifier le titre du sondage.";
#elif L==5
	"Debe especificar el t&iacute;tulo de la encuesta.";	// Okoteve traduccin
#elif L==6
	"&Egrave; necessario specificare il titolo del sondaggio.";
#elif L==7
	"You must specify the title of the survey.";		// Potrzebujesz tlumaczenie
#elif L==8
	"Voc&ecirc; deve especificar o t&iacute;tulo do inqu&eacute;rito.";
#endif

const char *Txt_You_must_specify_the_URL_of_the_new_banner =
#if   L==0
	"Debe especificar el URL del nuevo banner.";	// Necessita traduccio
#elif L==1
	"You must specify the URL of the new banner.";	// Need bersetzung
#elif L==2
	"You must specify the URL of the new banner.";
#elif L==3
	"Debe especificar el URL del nuevo banner.";
#elif L==4
	"You must specify the URL of the new banner.";	// Besoin de traduction
#elif L==5
	"Debe especificar el URL del nuevo banner.";	// Okoteve traduccin
#elif L==6
	"Devi specificare l'URL del nuovo banner.";
#elif L==7
	"You must specify the URL of the new banner.";	// Potrzebujesz tlumaczenie
#elif L==8
	"You must specify the URL of the new banner.";	// Necessita de traduo
#endif

const char *Txt_You_must_specify_the_URL_of_the_new_link =
#if   L==0
	"Debe especificar el URL del nuevo enlace.";	// Necessita traduccio
#elif L==1
	"You must specify the URL of the new link.";	// Need bersetzung
#elif L==2
	"You must specify the URL of the new link.";
#elif L==3
	"Debe especificar el URL del nuevo enlace.";
#elif L==4
	"You must specify the URL of the new link.";	// Besoin de traduction
#elif L==5
	"Debe especificar el URL del nuevo enlace.";	// Okoteve traduccin
#elif L==6
	"Devi specificare l'URL del nuovo link.";
#elif L==7
	"You must specify the URL of the new link.";	// Potrzebujesz tlumaczenie
#elif L==8
	"You must specify the URL of the new link.";	// Necessita de traduo
#endif

const char *Txt_You_must_specify_the_web_address_of_the_new_centre =
#if   L==0
	"Debe especificar la direcci&oacute;n web"
	" del nuevo centro.";				// Necessita traduccio
#elif L==1
	"You must specify the web address"
	" of the new centre.";				// Need bersetzung
#elif L==2
	"You must specify the web address"
	" of the new centre.";
#elif L==3
	"Debe especificar la direcci&oacute;n web"
	" del nuevo centro.";
#elif L==4
	"You must specify the web address"
	" of the new centre.";				// Besoin de traduction
#elif L==5
	"Debe especificar la direcci&oacute;n web"
	" del nuevo centro.";				// Okoteve traduccin
#elif L==6
	"Devi specificare l'indirizzo web"
	" del nuovo centro.";
#elif L==7
	"You must specify the web address"
	" of the new centre.";				// Potrzebujesz tlumaczenie
#elif L==8
	"You must specify the web address"
	" of the new centre.";				// Necessita de traduo
#endif


const char *Txt_You_must_specify_the_web_address_of_the_new_degree =
#if   L==0
	"Debe especificar la direcci&oacute;n web"
	" de la nueva titulaci&oacute;n.";		// Necessita traduccio
#elif L==1
	"You must specify the web address"
	" of the new degree.";				// Need bersetzung
#elif L==2
	"You must specify the web address"
	" of the new degree.";
#elif L==3
	"Debe especificar la direcci&oacute;n web"
	" de la nueva titulaci&oacute;n.";
#elif L==4
	"You must specify the web address"
	" of the new degree.";				// Besoin de traduction
#elif L==5
	"Debe especificar la direcci&oacute;n web"
	" de la nueva titulaci&oacute;n.";		// Okoteve traduccin
#elif L==6
	"Devi specificare l'indirizzo web"
	" della nuova laurea.";
#elif L==7
	"You must specify the web address"
	" of the new degree.";				// Potrzebujesz tlumaczenie
#elif L==8
	"You must specify the web address"
	" of the new degree.";				// Necessita de traduo
#endif

const char *Txt_You_must_specify_the_web_address_of_the_new_department =
#if   L==0
	"Debe especificar la direcci&oacute;n web"
	" del nuevo departamento.";			// Necessita traduccio
#elif L==1
	"You must specify the web address"
	" of the new department.";			// Need bersetzung
#elif L==2
	"You must specify the web address"
	" of the new department.";
#elif L==3
	"Debe especificar la direcci&oacute;n web"
	" del nuevo departamento.";
#elif L==4
	"You must specify the web address"
	" of the new department.";			// Besoin de traduction
#elif L==5
	"Debe especificar la direcci&oacute;n web"
	" del nuevo departamento.";			// Okoteve traduccin
#elif L==6
	"Devi specificare l'indirizzo web"
	" del nuovo dipartimento.";
#elif L==7
	"You must specify the web address"
	" of the new department.";			// Potrzebujesz tlumaczenie
#elif L==8
	"You must specify the web address"
	" of the new department.";			// Necessita de traduo
#endif

const char *Txt_You_must_specify_the_web_address_of_the_new_institution =
#if   L==0
	"Debe especificar la direcci&oacute;n web"
	" de la nueva instituci&oacute;n.";		// Necessita traduccio
#elif L==1
	"You must specify the web address"
	" of the new institution.";			// Need bersetzung
#elif L==2
	"You must specify the web address"
	" of the new institution.";
#elif L==3
	"Debe especificar la direcci&oacute;n web"
	" de la nueva instituci&oacute;n.";
#elif L==4
	"You must specify the web address"
	" of the new institution.";			// Besoin de traduction
#elif L==5
	"Debe especificar la direcci&oacute;n web"
	" de la nueva instituci&oacute;n.";		// Okoteve traduccin
#elif L==6
	"Devi specificare l'indirizzo web"
	" della nuova istituzione.";
#elif L==7
	"You must specify the web address"
	" of the new institution.";			// Potrzebujesz tlumaczenie
#elif L==8
	"You must specify the web address"
	" of the new institution.";			// Necessita de traduo
#endif

const char *Txt_You_must_type_at_least_one_tag_for_the_question =
#if   L==0
	"Debe escribir al menos un descriptor para la pregunta.";	// Necessita traduccio
#elif L==1
	"You must type at least one tag for the question.";		// Need bersetzung
#elif L==2
	"You must type at least one tag for the question.";
#elif L==3
	"Debe escribir al menos un descriptor para la pregunta.";
#elif L==4
	"You must type at least one tag for the question.";		// Besoin de traduction
#elif L==5
	"Debe escribir al menos un descriptor para la pregunta.";	// Okoteve traduccin
#elif L==6
	"Devi scrivere almeno un'etichetta per la domanda.";
#elif L==7
	"You must type at least one tag for the question.";		// Potrzebujesz tlumaczenie
#elif L==8
	"You must type at least one tag for the question.";		// Necessita de traduo
#endif

const char *Txt_You_must_type_at_least_the_first_answer =
#if   L==0
	"Debe escribir al menos la primera respuesta.";	// Necessita traduccio
#elif L==1
	"You must type at least the first answer.";	// Need bersetzung
#elif L==2
	"You must type at least the first answer.";
#elif L==3
	"Debe escribir al menos la primera respuesta.";
#elif L==4
	"You must type at least the first answer.";	// Besoin de traduction
#elif L==5
	"Debe escribir al menos la primera respuesta.";	// Okoteve traduccin
#elif L==6
	"Devi scrivere almeno la prima risposta.";
#elif L==7
	"You must type at least the first answer.";	// Potrzebujesz tlumaczenie
#elif L==8
	"You must type at least the first answer.";	// Necessita de traduo
#endif

const char *Txt_You_must_type_at_least_the_first_two_answers =
#if   L==0
	"Debe escribir al menos las dos primeras respuestas.";	// Necessita traduccio
#elif L==1
	"You must type at least the first two answers.";	// Need bersetzung
#elif L==2
	"You must type at least the first two answers.";
#elif L==3
	"Debe escribir al menos las dos primeras respuestas.";
#elif L==4
	"You must type at least the first two answers.";	// Besoin de traduction
#elif L==5
	"Debe escribir al menos las dos primeras respuestas.";	// Okoteve traduccin
#elif L==6
	"Devi scrivere almeno le prime due risposte.";
#elif L==7
	"You must type at least the first two answers.";	// Potrzebujesz tlumaczenie
#elif L==8
	"You must type at least the first two answers.";	// Necessita de traduo
#endif

const char *Txt_You_must_type_the_stem_of_the_question =
#if   L==0
	"Debe escribir el enunciado de la pregunta.";	// Necessita traduccio
#elif L==1
	"You must type the stem of the question.";	// Need bersetzung
#elif L==2
	"You must type the stem of the question.";
#elif L==3
	"Debe escribir el enunciado de la pregunta.";
#elif L==4
	"You must type the stem of the question.";	// Besoin de traduction
#elif L==5
	"Debe escribir el enunciado de la pregunta.";	// Okoteve traduccin
#elif L==6
	"Devi scrivere l'enunciato della domanda.";
#elif L==7
	"You must type the stem of the question.";	// Potrzebujesz tlumaczenie
#elif L==8
	"You must type the stem of the question.";	// Necessita de traduo
#endif

const char *Txt_You_need_an_XML_file_containing_a_list_of_questions_to_import =
#if   L==0
	"Es necesario un archivo XML"
	" que contenga una lista de preguntas a importar.";	// Necessita traduccio
#elif L==1
	"You need an XML file"
	" containing a list of questions to import.";		// Need bersetzung
#elif L==2
	"You need an XML file"
	" containing a list of questions to import.";
#elif L==3
	"Es necesario un archivo XML"
	" que contenga una lista de preguntas a importar.";
#elif L==4
	"You need an XML file"
	" containing a list of questions to import.";		// Besoin de traduction
#elif L==5
	"Es necesario un archivo XML"
	" que contenga una lista de preguntas a importar.";	// Okoteve traduccin
#elif L==6
	"You need an XML file"
	" containing a list of questions to import.";		// Bisogno di traduzione
#elif L==7
	"You need an XML file"
	" containing a list of questions to import.";		// Potrzebujesz tlumaczenie
#elif L==8
	"You need an XML file"
	" containing a list of questions to import.";		// Necessita de traduo
#endif

const char *Txt_You_should_read_the_following_information_on_the_course_X =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"Usted debe leer la siguiente informaci&oacute;n"
	" sobre la asignatura <strong>%s</strong>:";	// Necessita traduccio
#elif L==1
	"You should read the following information"
	" on the course <strong>%s</strong>:";		// Need bersetzung
#elif L==2
	"You should read the following information"
	" on the course <strong>%s</strong>:";
#elif L==3
	"Usted debe leer la siguiente informaci&oacute;n"
	" sobre la asignatura <strong>%s</strong>:";
#elif L==4
	"You should read the following information"
	" on the course <strong>%s</strong>:";		// Besoin de traduction
#elif L==5
	"Usted debe leer la siguiente informaci&oacute;n"
	" sobre la asignatura <strong>%s</strong>:";	// Okoteve traduccin
#elif L==6
	"Dovresti leggere le seguenti informazioni"
	" sul corso <strong>%s</strong>:";
#elif L==7
	"You should read the following information"
	" on the course <strong>%s</strong>:";		// Potrzebujesz tlumaczenie
#elif L==8
	"You should read the following information"
	" on the course <strong>%s</strong>:";		// Necessita de traduo
#endif

const char *Txt_You_were_already_enrolled_as_X_in_the_course_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==0
	"Usted ya estaba inscrito/a como <strong>%s</strong>"
	" en la asignatura <strong>%s</strong>.";	// Necessita traduccio
#elif L==1
	"You were already enrolled as <strong>%s</strong>"
	" in the course <strong>%s</strong>.";		// bersetzung notwendig
#elif L==2
	"You were already enrolled as <strong>%s</strong>"
	" in the course <strong>%s</strong>.";
#elif L==3
	"Usted ya estaba inscrito/a como <strong>%s</strong>"
	" en la asignatura <strong>%s</strong>.";
#elif L==4
	"You were already enrolled as <strong>%s</strong>"
	" in the course <strong>%s</strong>.";		// Besoin de traduction
#elif L==5
	"Usted ya estaba inscrito/a como <strong>%s</strong>"
	" en la asignatura <strong>%s</strong>.";	// Okoteve traduccin
#elif L==6
	"You were already enrolled as <strong>%s</strong>"
	" in the course <strong>%s</strong>.";		// Bisogno di traduzione
#elif L==7
	"You were already enrolled as <strong>%s</strong>"
	" in the course <strong>%s</strong>.";		// Potrzebujesz tlumaczenie
#elif L==8
	"You were already enrolled as <strong>%s</strong>"
	" in the course <strong>%s</strong>.";		// Necessita de traduo
#endif

const char *Txt_Your_comment_has_been_updated =
#if   L==0
	"El seu comentari ha estat actualitzat.";
#elif L==1
	"Ihr Kommentar wurde aktualisiert.";
#elif L==2
	"Your comment has been updated.";
#elif L==3
	"Su comentario ha sido actualizado.";
#elif L==4
	"Votre commentaire a &eacute;t&eacute; mis &agrave; jour.";
#elif L==5
	"Su comentario ha sido actualizado.";		// Okoteve traduccin
#elif L==6
	"Il tuo commento &egrave; stato aggiornato.";
#elif L==7
	"Komentarz zosta&lstrok; zaktualizowany.";
#elif L==8
	"O seu coment&aacute;rio foi atualizado.";
#endif

const char *Txt_Your_email_address_X_has_been_registered_successfully =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"Su correo electr&oacute;nico <strong>%s</strong>"
	" se ha registrado correctamente.";		// Necessita traduccio
#elif L==1
	"Your e-mail address <strong>%s</strong>"
	" has been registered successfully.";		// Need bersetzung
#elif L==2
	"Your e-mail address <strong>%s</strong>"
	" has been registered successfully.";
#elif L==3
	"Su correo electr&oacute;nico <strong>%s</strong>"
	" se ha registrado correctamente.";
#elif L==4
	"Your e-mail address <strong>%s</strong>"
	" has been registered successfully.";		// Besoin de traduction
#elif L==5
	"Su correo electr&oacute;nico <strong>%s</strong>"
	" se ha registrado correctamente.";		// Okoteve traduccin
#elif L==6
	"Il tuo e-mail <strong>%s</strong>"
	" &egrave; stato registrato con successo.";
#elif L==7
	"Your e-mail address <strong>%s</strong>"
	" has been registered successfully.";		// Potrzebujesz tlumaczenie
#elif L==8
	"Your e-mail address <strong>%s</strong>"
	" has been registered successfully.";		// Necessita de traduo
#endif

const char *Txt_Your_language_has_changed_to_LANGUAGE =
#if   L==0
	"El seu idioma ha canviat a catal&agrave;.";
#elif L==1
	"Ihre Sprache ist nun Deutsch.";
#elif L==2
	"Your language has changed to English.";
#elif L==3
	"Su idioma ha cambiado a espa&ntilde;ol.";
#elif L==4
	"Votre langue a chang&eacute; au fran&ccedil;ais.";
#elif L==5
	"Pe &ntilde;e'&#7869; ha moambuepapyre pe ava&ntilde;e'&#7869;.";
#elif L==6
	"La tua lingua &egrave; cambiata a italiano.";
#elif L==7
	"J&eogon;zyk zosta&lstrok; zmieniony na polski.";
#elif L==8
	"Suo idioma mudou para o portugu&ecirc;s.";
#endif

const char *Txt_Your_nickname_X_has_been_registered_successfully =	// Warning: it is very important to include %s in the following sentences
#if   L==0
	"Su apodo <strong>@%s</strong>"
	" se ha registrado correctamente.";		// Necessita traduccio
#elif L==1
	"Your nickname <strong>@%s</strong>"
	" has been registered successfully.";		// Need bersetzung
#elif L==2
	"Your nickname <strong>@%s</strong>"
	" has been registered successfully.";
#elif L==3
	"Su apodo <strong>@%s</strong>"
	" se ha registrado correctamente.";
#elif L==4
	"Your nickname <strong>@%s</strong>"
	" has been registered successfully.";		// Besoin de traduction
#elif L==5
	"Su apodo <strong>@%s</strong>"
	" se ha registrado correctamente.";		// Okoteve traduccin
#elif L==6
	"Il tuo nome utente <strong>@%s</strong>"
	" &egrave; stato registrato con successo.";
#elif L==7
	"Your nickname <strong>@%s</strong>"
	" has been registered successfully.";		// Potrzebujesz tlumaczenie
#elif L==8
	"Your nickname <strong>@%s</strong>"
	" has been registered successfully.";		// Necessita de traduo
#endif

const char *Txt_Your_password_has_been_changed_successfully =
#if   L==0
	"Su contrase&ntilde;a se ha modificado con &eacute;xito.";	// Necessita traduccio
#elif L==1
	"Your password has been changed successfully.";			// Need bersetzung
#elif L==2
	"Your password has been changed successfully.";
#elif L==3
	"Su contrase&ntilde;a se ha modificado con &eacute;xito.";
#elif L==4
	"Your password has been changed successfully.";			// Besoin de traduction
#elif L==5
	"Su contrase&ntilde;a se ha modificado con &eacute;xito.";	// Okoteve traduccin
#elif L==6
	"La tua password &egrave; stata cambiata con successo.";
#elif L==7
	"Your password has been changed successfully.";			// Potrzebujesz tlumaczenie
#elif L==8
	"Your password has been changed successfully.";			// Necessita de traduo
#endif

const char *Txt_Your_password_is_not_secure_enough =
#if   L==0
	"Su contrase&ntilde;a no es suficientemente segura.";	// Necessita traduccio
#elif L==1
	"Your password is not secure enough.";			// Need bersetzung
#elif L==2
	"Your password is not secure enough.";
#elif L==3
	"Su contrase&ntilde;a no es suficientemente segura.";
#elif L==4
	"Your password is not secure enough.";			// Besoin de traduction
#elif L==5
	"Su contrase&ntilde;a no es suficientemente segura.";	// Okoteve traduccin
#elif L==6
	"La tua password non &egrave; sicura abbastanza.";
#elif L==7
	"Your password is not secure enough.";			// Potrzebujesz tlumaczenie
#elif L==8
	"Your password is not secure enough.";			// Necessita de traduo
#endif

const char *Txt_Your_password_must_be_at_least_X_characters_and_can_not_contain_spaces_ =	// Warning: it is very important to include %u in the following sentences
#if   L==0
	"La contrase&ntilde;a debe tener al menos %u caracteres"
	" y no puede contener espacios."
	" Es muy importante que no use s&oacute;lo"
	" d&iacute;gitos num&eacute;ricos, ni un nombre o apellido, ni una palabra conocida.";	// Necessita traduccio
#elif L==1
	"Your password must be at least %u characters"
	" and can not contain spaces."
	"It's very important not to use only"
	" numeric digits, or a name or surname, or a known word.";				// Need bersetzung
#elif L==2
	"Your password must be at least %u characters"
	" and can not contain spaces."
	"It's very important not to use only"
	" numeric digits, or a name or surname, or a known word.";
#elif L==3
	"La contrase&ntilde;a debe tener al menos %u caracteres"
	" y no puede contener espacios."
	" Es muy importante que no use s&oacute;lo"
	" d&iacute;gitos num&eacute;ricos, ni un nombre o apellido, ni una palabra conocida.";
#elif L==4
	"Your password must be at least %u characters"
	" and can not contain spaces."
	"It's very important not to use only"
	" numeric digits, or a name or surname, or a known word.";				// Besoin de traduction
#elif L==5
	"La contrase&ntilde;a debe tener al menos %u caracteres"
	" y no puede contener espacios."
	" Es muy importante que no use s&oacute;lo"
	" d&iacute;gitos num&eacute;ricos, ni un nombre o apellido, ni una palabra conocida.";	// Okoteve traduccin
#elif L==6
	"La tua password deve essere almeno di %u caratteri"
	" e non pu&ograve; contenere spazi."
	"E' molto importante non usare solo"
	" tasti numerici, o un nome o cognome, o una parola conosciuta.";
#elif L==7
	"Your password must be at least %u characters"
	" and can not contain spaces."
	"It's very important not to use only"
	" numeric digits, or a name or surname, or a known word.";				// Potrzebujesz tlumaczenie
#elif L==8
	"Your password must be at least %u characters"
	" and can not contain spaces."
	"It's very important not to use only"
	" numeric digits, or a name or surname, or a known word.";				// Necessita de traduo
#endif

const char *Txt_Your_personal_data_have_been_updated =
#if   L==0
	"Sus datos personales se han actualizado.";	// Necessita traduccio
#elif L==1
	"Your personal data have been updated.";	// Need bersetzung
#elif L==2
	"Your personal data have been updated.";
#elif L==3
	"Sus datos personales se han actualizado.";
#elif L==4
	"Your personal data have been updated.";	// Besoin de traduction
#elif L==5
	"Sus datos personales se han actualizado.";	// Okoteve traduccin
#elif L==6
	"I tuoi dati personali sono stati aggiornati.";
#elif L==7
	"Your personal data have been updated.";	// Potrzebujesz tlumaczenie
#elif L==8
	"Your personal data have been updated.";	// Necessita de traduo
#endif

const char *Txt_Your_preferences_about_notifications_have_changed =
#if   L==0
	"Les seves prefer&egrave;ncies sobre les notificacions han canviat.";
#elif L==1
	"Your preferences about notifications have changed.";		// Need bersetzung
#elif L==2
	"Your preferences about notifications have changed.";
#elif L==3
	"Sus preferencias sobre las notificaciones han cambiado.";
#elif L==4
	"Your preferences about notifications have changed.";		// Besoin de traduction
#elif L==5
	"Sus preferencias sobre las notificaciones han cambiado.";	// Okoteve traduccin
#elif L==6
	"Le tue preferenze sulle notifiche sono cambiate.";
#elif L==7
	"Your preferences about notifications have changed.";		// Potrzebujesz tlumaczenie
#elif L==8
	"Your preferences about notifications have changed.";		// Necessita de traduo
#endif

const char *Txt_Your_record_card_in_this_course_has_been_updated =
#if   L==0
	"Su ficha en la asignatura se ha actualizado:";		// Necessita traduccio
#elif L==1
	"Your record card in this course has been updated:";	// Need bersetzung
#elif L==2
	"Your record card in this course has been updated:";
#elif L==3
	"Su ficha en la asignatura se ha actualizado:";
#elif L==4
	"Your record card in this course has been updated:";	// Besoin de traduction
#elif L==5
	"Su ficha en la asignatura se ha actualizado:";		// Okoteve traduccin
#elif L==6
	"La tua scheda in questo corso &egrave; stata aggiornata:";
#elif L==7
	"Your record card in this course has been updated:";	// Potrzebujesz tlumaczenie
#elif L==8
	"Your record card in this course has been updated:";	// Necessita de traduo
#endif

const char *Txt_Your_request_for_enrollment_as_X_in_the_course_Y_has_been_accepted_for_processing =	// Warning: it is very important to include two %s in the following sentences
#if   L==0
	"Su petici&oacute;n de inscripci&oacute;n como <strong>%s</strong>"
	" en la asignatura <strong>%s</strong>"
	" ha sido realizada.";				// Necessita traduccio
#elif L==1
	"Your request for enrollment as <strong>%s</strong>"
	" in the course <strong>%s</strong>"
	" has been accepted for processing.";		// Need bersetzung
#elif L==2
	"Your request for enrollment as <strong>%s</strong>"
	" in the course <strong>%s</strong>"
	" has been accepted for processing.";
#elif L==3
	"Su petici&oacute;n de inscripci&oacute;n como <strong>%s</strong>"
	" en la asignatura <strong>%s</strong>"
	" ha sido realizada.";
#elif L==4
	"Your request for enrollment as <strong>%s</strong>"
	" in the course <strong>%s</strong>"
	" has been accepted for processing.";		// Besoin de traduction
#elif L==5
	"Su petici&oacute;n de inscripci&oacute;n como <strong>%s</strong>"
	" en la asignatura <strong>%s</strong>"
	" ha sido realizada.";				// Okoteve traduccin
#elif L==6
	"Your request for enrollment as <strong>%s</strong>"
	" in the course <strong>%s</strong>"
	" has been accepted for processing.";		// Bisogno di traduzione
#elif L==7
	"Your request for enrollment as <strong>%s</strong>"
	" in the course <strong>%s</strong>"
	" has been accepted for processing.";		// Potrzebujesz tlumaczenie
#elif L==8
	"Your request for enrollment as <strong>%s</strong>"
	" in the course <strong>%s</strong>"
	" has been accepted for processing.";		// Necessita de traduo
#endif

const char *Txt_ZIP_file =
#if   L==0
	"Fitxer ZIP";
#elif L==1
	"ZIP-Datei";
#elif L==2
	"ZIP file";
#elif L==3
	"Archivo ZIP";
#elif L==4
	"Fichier ZIP";
#elif L==5
	"Archivo ZIP";	// Okoteve traduccin
#elif L==6
	"File ZIP";
#elif L==7
	"Plik ZIP";
#elif L==8
	"Arquivo ZIP";
#endif
